package de.uniwue.mk.kall.athen.annotationbrowserview.ui;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.AnnotationImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TreeItem;
import org.osgi.service.event.Event;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.FeatureJoniDisplayDrawingStrategy;
import de.uniwue.mk.kall.athen.annotationbrowserwidget.AnnotationBrowserWidget;
import de.uniwue.mk.kall.athen.annotationbrowserwidget.FeatureValueWrapper;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.JoniSegmentDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.TokenDrawingStrategy;

public class AnnotationBrowserView extends AEditorSubordinateViewPart {

	private static final Class<?> CLASS_THIS = AnnotationBrowserView.class;

	private AnnotationBrowserWidget widget;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.4";

	@Inject
	private static EPartService partService;

	@Inject
	private static IEventBroker broker;

	@Inject
	public AnnotationBrowserView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Construct Annotation Browser");
		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

	}

	@PreDestroy
	public void preDestroy() {
		// nothing to disposie i guess
	}

	@Focus
	public void onFocus() {

		super.onFocus();
		renewPage();
	}

	@Persist
	public void save() {
		// nothing to save
	}

	@Override
	protected void initLayout(Composite composite) {

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;
		super.overlay.setLayout(layout);

		widget = new AnnotationBrowserWidget(composite, editor);

		widget.getViewer().getTree().addMouseListener(new MouseAdapter() {

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				IStructuredSelection sel = (IStructuredSelection) widget.getViewer().getSelection();

				Object firstElement = sel.getFirstElement();

				if (firstElement instanceof FeatureValueWrapper) {

					FeatureValueWrapper wrapper = (FeatureValueWrapper) firstElement;

					if (wrapper.feature.getShortName().matches("begin|end"))
						return;

					if (wrapper.feature.getRange().toString().equals("uima.cas.String")) {

						String val;

						if (wrapper.value == null)
							val = "";

						else {
							val = wrapper.value.toString();
						}

						InputDialog dialog = new InputDialog(new Shell(), "Set Feature Value",
								wrapper.feature.getShortName() + ":", val, null);

						int open = dialog.open();

						if (open == 0) {
							String featureValue = dialog.getValue();

							TreeItem parentItem = widget.getViewer().getTree().getSelection()[0].getParentItem();

							if (parentItem.getData() instanceof AnnotationFS) {
								AnnotationFS anno = (AnnotationFS) parentItem.getData();

								anno.setFeatureValueFromString(wrapper.feature, featureValue);

								editor.changeAnnotation(anno);
							}

						}

					}

				}

			}
		});

		widget.getViewer().getTree().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) widget.getViewer().getSelection();

				Object firstElement = sel.getFirstElement();

				// selection of the checkbox
				if (e.item instanceof TreeItem && e.detail == 32) {

					TreeItem item = (TreeItem) e.item;

					if (item.getData() instanceof AnnotationFS) {

						Type t = null;

						if (item.getData() instanceof AnnotationFS) {
							t = ((AnnotationFS) item.getData()).getType();
						}

						if (item.getChecked()) {
							editor.changeVisibility(t, true);

							// setze alle haken
							TreeItem parentItem = item.getParentItem();
							parentItem.setChecked(true);

							for (TreeItem it : parentItem.getItems()) {
								it.setChecked(true);
							}
						}

						else {
							editor.changeVisibility(t, false);

							// setze alle haken
							TreeItem parentItem = item.getParentItem();
							parentItem.setChecked(false);

							for (TreeItem it : parentItem.getItems()) {
								it.setChecked(false);
							}
						}
					}

					else if (item.getData() instanceof TypeStyleDataStructure) {

						Type t = ((TypeStyleDataStructure) item.getData()).getAnnotationType();

						if (item.getChecked()) {
							editor.changeVisibility(t, true);

							for (TreeItem it : item.getItems()) {
								it.setChecked(true);
							}
						}

						else {
							editor.changeVisibility(t, false);

							for (TreeItem it : item.getItems()) {
								it.setChecked(false);
							}
						}
					}

				}

				if (firstElement instanceof AnnotationImpl) {

					AnnotationImpl anno = (AnnotationImpl) firstElement;
					editor.highlightRange(anno.getBegin(), anno.getEnd() - anno.getBegin(), true);
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});
		// create menumanager
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(widget.getViewer().getControl());

		// attach that menu manager to the treeviewer
		widget.getViewer().getControl().setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {

				Object firstElement = ((IStructuredSelection) widget.getViewer().getSelection()).getFirstElement();

				if (firstElement instanceof TypeStyleDataStructure) {
					Action actionSetToModus = new Action() {

						@Override
						public void run() {
							TypeStyleDataStructure tsds = (TypeStyleDataStructure) firstElement;
							editor.setActiveMode(tsds.getAnnotationType());
							renewPage();
						}
					};
					actionSetToModus.setText("Set to Modus");
					menuMgr.add(actionSetToModus);
				}
			}
		});

		menuMgr.setRemoveAllWhenShown(true);

		overlay.layout();
		overlay.getParent().layout();

	}

	public TreeViewer getViewer() {
		return widget.getViewer();
	}

	public void setViewer(TreeViewer viewer) {
		widget.setViewer(viewer);
	}

	public void renewPage() {
		if (widget == null) {
			return;
		}
		widget.renewPage();
	}

	@Override
	public void onEditorClosed(Event arg0) {
		super.onEditorClosed(arg0);
	}

	@Override
	public void onAnnotationAdded(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationRemoved(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationChanged(Event arg0) {
		renewPage();

	}

	@Override
	public void onEditorStyleChanged(Event arg0) {
		renewPage();

	}

	@Override
	public void initModel() {
		// no model in this view required

	}

	@Override
	public boolean initTypes() {

		// try to set default editor stylings to given annotations
		Type typeDWHeadline = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.Headline");
		Type typeDWHeadlineCandidate = editor.getCas().getTypeSystem()
				.getType("de.uniwue.dw.segmentation.HeadlineCandidate");
		Type typeParagraph = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.Paragraph");
		Type typeDWSection = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.Section");
		Type typeDWSectionBody = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.SectionBody");
		Type typeDWSectionG = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.SectionG");
		Type typeDWSegment = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.Segment");
		Type typeDWSegmentG = editor.getCas().getTypeSystem().getType("de.uniwue.dw.segmentation.SegmentG");
		Type typePar = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Paragraph");

		// joni kasten
		activateJoniTypes(typeDWHeadline, typeDWHeadline, typeParagraph, typeDWSection, typeDWSectionBody,
				typeDWSectionG, typeDWHeadlineCandidate, typePar);

		// joni alternierend mit farbe
		activateJoniBackground( typeDWSegmentG);

		return false;
	}

	private void activateJoniBackground(Type... types) {
		for (Type t : types) {
			if (t == null)
				continue;

			StyleRange sr = new StyleRange();
			sr.background = editor.getStyleStruct().getDSFromType(t).getStyle().getStyleRange().background;

			editor.changeStyle(t, new AnnoStyle(new JoniSegmentDrawingStrategy(sr), sr));
		}

	}

	private void activateJoniTypes(Type... types) {

		for (Type t : types) {
			if (t == null)
				continue;

			StyleRange sr = new StyleRange();
			sr.background = editor.getStyleStruct().getDSFromType(t).getStyle().getStyleRange().background;

			editor.changeStyle(t, new AnnoStyle(new FeatureJoniDisplayDrawingStrategy(sr, "category"), sr));
		}

	}
}
