package de.uniwue.mk.kall.athen.annotationbrowserwidget;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.impl.ArrayFSImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class AnnotationContentProvider implements ITreeContentProvider {

	private CAS cas;

	public AnnotationContentProvider(CAS cas) {

		this.cas = cas;
	}

	@Override
	public void dispose() {
		// TODO Auto-generated method stub

	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

	@Override
	public Object[] getElements(Object inputElement) {

		if (inputElement instanceof AnnotationEditorWidget) {

			AnnotationEditorWidget editor = (AnnotationEditorWidget) inputElement;

			List<TypeStyleDataStructure> styles = editor.getStyleStruct().getStylesFromLowestLayerToHighest();

			return styles.toArray(new TypeStyleDataStructure[0]);
		}

		return null;
	}

	@Override
	public Object[] getChildren(Object parentElement) {

		if (parentElement instanceof TypeStyleDataStructure) {

			TypeStyleDataStructure style = (TypeStyleDataStructure) parentElement;

			AnnotationIndex<AnnotationFS> annotationIndex = this.cas.getAnnotationIndex(style.getAnnotationType());

			AnnotationFS[] annoArr = new AnnotationFS[annotationIndex.size()];

			int index = 0;
			for (AnnotationFS a : annotationIndex) {

				annoArr[index] = a;

				index++;
			}

			return annoArr;

		}

		else if (parentElement instanceof AnnotationFS) {

			AnnotationFS anno = (AnnotationFS) parentElement;

			List<Feature> features = anno.getType().getFeatures();

			// TODO add a wrapper type for the features so that you can access
			// their values !!

			List<FeatureValueWrapper> wrapperObjects = new ArrayList<FeatureValueWrapper>();
			for (Feature f : features) {

				if (f.getRange().isPrimitive()) {
					wrapperObjects.add(new FeatureValueWrapper(f, anno.getFeatureValueAsString(f)));
				}

				else {

					if (!f.getName().contains(":sofa")) {
						wrapperObjects.add(new FeatureValueWrapper(f, anno.getFeatureValue(f)));
					}
				}
			}
			return wrapperObjects.toArray(new FeatureValueWrapper[0]);

		}

		else if (parentElement instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) parentElement;

			if (wrapper.feature.getRange().isPrimitive())
				return null;

			else {

				if (wrapper.value != null) {
					if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation")) {
						return new Object[] { (AnnotationFS) wrapper.value };
					}

					// TODO here need all the types to be added
					else if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation[]")) {

						List<AnnotationFS> annos = new ArrayList<AnnotationFS>();

						ArrayFSImpl arr = (ArrayFSImpl) wrapper.value;
						for (int i = 0; i < arr.size(); i++) {
							annos.add((AnnotationFS) arr.get(i));
						}
						return annos.toArray(new AnnotationFS[0]);
					}
				}
			}
		}

		return null;
	}

	@Override
	public Object getParent(Object element) {

		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public boolean hasChildren(Object element) {

		if (element instanceof TypeStyleDataStructure) {

			TypeStyleDataStructure style = (TypeStyleDataStructure) element;

			AnnotationIndex<AnnotationFS> annotationIndex = this.cas.getAnnotationIndex(style.getAnnotationType());

			return annotationIndex.size() > 0;
		}

		else if (element instanceof AnnotationEditorWidget) {

			AnnotationEditorWidget editor = (AnnotationEditorWidget) element;

			List<TypeStyleDataStructure> styles = editor.getStyleStruct().getStylesFromLowestLayerToHighest();

			return styles.size() > 0;

		}

		else if (element instanceof AnnotationFS) {
			return true;
		}

		else if (element instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) element;
			if (wrapper.feature.getRange().isPrimitive())
				return false;

			else {

				if (wrapper.value != null) {
					if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation")) {
						return true;
					}

					// TODO here need all the types to be added
					else if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation[]")) {
						return true;
					}
				}
			}
		}
		return false;
	}
}
