package de.uniwue.mk.kall.athen.annotationbrowserwidget;

import java.util.HashMap;

import javax.annotation.PreDestroy;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class AnnotationLabelContentProvider extends StyledCellLabelProvider {

	HashMap<String, String> featureMap = new HashMap<String, String>();
	HashMap<Type, Image> typesHash = new HashMap<Type, Image>();

	private AnnotationBrowserWidget view;

	private AnnotationEditorWidget editor;

	private GC gc;
	private Color colorIsMode;

	public AnnotationLabelContentProvider(AnnotationEditorWidget editor, 
			AnnotationBrowserWidget view) {
		this.editor = editor;
		this.view = view;
		this.gc = new GC(view.getDisplay());
		colorIsMode = new Color(Display.getCurrent(), new RGB(0, 0, 0));
	}

	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof TypeStyleDataStructure) {

			TypeStyleDataStructure tsds = (TypeStyleDataStructure) element;

			text.append(tsds.getAnnotationType().getShortName() + " ["
					+ editor.getCas().getAnnotationIndex(tsds.getAnnotationType()).size() + "]");

			if (editor.getActiveMode() != null && editor.getActiveMode().equals(tsds.getAnnotationType())) {
				cell.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));
			} else {
				cell.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
			}

			// create the image
			setImage(cell, tsds);

		}

		else if (element instanceof AnnotationFS) {

			AnnotationFS anno = (AnnotationFS) element;

			text.append(anno.getCoveredText());

			// set iomage

			setImage(cell, getTsdsFromAnno(anno));
		}

		else if (element instanceof Feature) {

			Feature f = (Feature) element;

			IStructuredSelection selection = (IStructuredSelection) this.view.getViewer().getSelection();
			AnnotationFS selectedAnno = null;
			if (selection.getFirstElement() instanceof TypeStyleDataStructure) {
				// should not happen :)
				text.append("Retry clicking !!");

			} else {
				selectedAnno = (AnnotationFS) selection.getFirstElement();

				// TODO dirty

				if (f.getRange().isPrimitive()) {

					text.append(f.getShortName() + ":" + selectedAnno.getFeatureValueAsString(f));

				}

				else {

					// TODO

					text.append("Unsupported");

				}
			}

		}

		else if (element instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) element;

			if (wrapper.feature.getRange().isPrimitive()) {

				String value = (String) wrapper.value;

				text.append(wrapper.feature.getShortName() + ":" + value);
			}

			else {

				if (wrapper.value != null) {

					text.append(wrapper.feature.getShortName());

				}

			}

		}

		// cell.setImage(image);

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	private TypeStyleDataStructure getTsdsFromAnno(AnnotationFS anno) {

		TypeStyleDataStructure dsFromType = editor.getDSFromType(anno.getType());
		return dsFromType;
	}

	private void setImage(ViewerCell cell, TypeStyleDataStructure tsds) {

		Image im = null;

		if (typesHash.containsKey(tsds.getAnnotationType())) {

			im = typesHash.get(tsds.getAnnotationType());
		}

		else {

			im = new Image(editor.getParent().getDisplay(), 16, 16);

			typesHash.put(tsds.getAnnotationType(), im);

		}

		gc = new GC(im);

		gc.setBackground(tsds.getStyle().getStyle().background);

		String substring = tsds.getStyle().getDrawingStrategy().getClass().toString();

		String[] split = substring.split("\\.");

		String name = split[split.length - 1];

		if (name.length() > 2) {
			name = name.substring(0, 2);
		}

		name = name.toUpperCase();

		gc.fillRectangle(0, 0, 16, 16);

		gc.drawString(name, 0, 0);
		cell.setImage(im);

		gc.dispose();

	}

	@PreDestroy
	public void dispose() {

		for (Image i : typesHash.values()) {

			i.dispose();
		}

		gc.dispose();
		colorIsMode.dispose();
	}
}
