package de.uniwue.mk.kall.athen.appDelegation.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.advanced.MPerspective;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.mk.kall.athen.appDelegation.structs.ClassEventPair;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;

public class ApplicationManager {

	/*
	 * A Class that registers all other parts/classes to the eventbroker im
	 * doing this in this central class to assure no class registers twice
	 */

	private IEventBroker broker;
	private EModelService modelService;
	private MApplication app;
	private EPartService partService;

	private HashMap<MPart, List<MPerspective>> partMapping;

	public ApplicationManager(IEventBroker broker, EModelService service, MApplication app, EPartService partService) {

		this.broker = broker;
		subscribingSet = new HashSet<ClassEventPair>();
		this.partMapping = new HashMap<MPart, List<MPerspective>>();
		this.modelService = service;
		this.app = app;
		this.partService = partService;

		updatePartMapping();

		// register to the perspective Switch event

	}

	// need to call updatePartMapping
	public ApplicationManager(IEventBroker eventBroker) {
		this.broker = eventBroker;
		subscribingSet = new HashSet<ClassEventPair>();
		this.partMapping = new HashMap<MPart, List<MPerspective>>();
	}

	public void updatePartMapping() {

		// get all parts
		List<MPart> parts = modelService.findElements(app, null, MPart.class, null);

		// get all perspectives
		List<MPerspective> perspectives = modelService.findElements(app, null, MPerspective.class, null);

		// every part gets a new list
		for (MPart part : parts) {

			partMapping.put(part, new ArrayList<MPerspective>());

		}

		for (MPerspective pers : perspectives) {

			for (MPart part : parts) {

				// find this part within that perspective
				List<MPart> findElements = modelService.findElements(pers, null, MPart.class, null);

				boolean contained = false;
				for (MPart persParts : findElements) {
					if (persParts.equals(part)) {
						contained = true;
						break;
					}
				}

				if (contained) {
					List<MPerspective> list = this.partMapping.get(part);

					list.add(pers);
				}
			}
		}

	}

	// mapping between the eventhandler and an event
	private static HashSet<ClassEventPair> subscribingSet;

	public static HashSet<ClassEventPair> getSubscribingSet() {
		return subscribingSet;
	}

	public void registerForEvent(Class<?> subscribingClass, String event, EventHandler handler) {

		if (!subscribingSet.contains(new ClassEventPair(subscribingClass, event))) {

			ApplicationManager.this.broker.subscribe(event, handler);
			subscribingSet.add(new ClassEventPair(subscribingClass, event));
		}

	}

	public void unsubscribeFromEvent(Class<?> subscribingClass, String event, EventHandler handler) {

		subscribingSet.remove(new ClassEventPair(subscribingClass, event));

		this.broker.unsubscribe(handler);
	}


	public IEventBroker getBroker() {
		return broker;
	}

	public void setBroker(IEventBroker broker) {
		this.broker = broker;
	}

	public EModelService getModelService() {
		return modelService;
	}

	public void setModelService(EModelService modelService) {
		this.modelService = modelService;
	}

	public MApplication getApp() {
		return app;
	}

	public void setApp(MApplication app) {
		this.app = app;
	}

	public EPartService getPartService() {
		return partService;
	}

	public void setPartService(EPartService partService) {
		this.partService = partService;
	}

	public HashMap<MPart, List<MPerspective>> getPartMapping() {
		return partMapping;
	}

	public void setPartMapping(HashMap<MPart, List<MPerspective>> partMapping) {
		this.partMapping = partMapping;
	}

	public static void setSubscribingSet(HashSet<ClassEventPair> subscribingSet) {
		ApplicationManager.subscribingSet = subscribingSet;
	}

}
