package de.uniwue.mk.kall.athen.attributeAcceptingView.widgets;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.swt.SWT;

import de.uniwue.mk.kall.athen.attributeAcceptingView.ui.AttributeAcceptingView;

public class ColViewerComparatorAttributeView extends ViewerComparator {
	private int propertyIndex;
	private static final int DESCENDING = 1;
	private int direction = DESCENDING;
	private Feature priority;
	private Feature id;
	private AttributeAcceptingView page;

	public ColViewerComparatorAttributeView(Feature id, Feature priority, AttributeAcceptingView page) {
		this.propertyIndex = 0;
		this.priority = priority;
		this.id = id;
		this.page = page;
		direction = DESCENDING;
	}

	public int getDirection() {
		return direction == 1 ? SWT.DOWN : SWT.UP;
	}

	public void setColumn(int column) {
		if (column == this.propertyIndex) {
			// Same column as last sort; toggle the direction
			direction = 1 - direction;
		} else {
			// New column; do an ascending sort
			this.propertyIndex = column;
			direction = DESCENDING;
		}
	}

	@Override
	public int compare(Viewer viewer, Object e1, Object e2) {
		AnnotationFS p1 = (AnnotationFS) e1;
		AnnotationFS p2 = (AnnotationFS) e2;
		int rc = 0;

		switch (propertyIndex) {
		case 0:
			// name
			rc = p1.getFeatureValueAsString(priority).compareTo(p2.getFeatureValueAsString(priority));
			break;
		case 1:
			// freq

			int freq1 = 0;
			int freq2 = 0;
			if (page.freqMapAttributes.containsKey(p1.getFeatureValueAsString(id))) {
				freq1 = page.freqMapAttributes.get(p1.getFeatureValueAsString(id));
			}

			if (page.freqMapAttributes.containsKey(p2.getFeatureValueAsString(id))) {

				freq2 = page.freqMapAttributes.get(p2.getFeatureValueAsString(id));
			}
			rc = freq1 - freq2;
			break;
		case 2:
			// id
			int id1 = Integer.parseInt(p1.getFeatureValueAsString(id));
			int id2 = Integer.parseInt(p2.getFeatureValueAsString(id));

			rc = id1 - id2;
			break;
		case 3:
			// synonyme
			rc = p1.getCoveredText().compareTo(p2.getCoveredText());
			break;
		default:
			rc = p1.getFeatureValueAsString(priority).compareTo(p2.getFeatureValueAsString(priority));
		}
		// If descending order, flip the direction
		if (direction == DESCENDING) {
			rc = -rc;
		}
		return rc;
	}

}
