package de.uniwue.mk.kall.athen.corefExplanationview.ui;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.ruta.rule.AnnotationComparator;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.text.JFaceTextUtil;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.jfree.ui.RefineryUtilities;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.ItalicDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.OvalDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.corefExplanationview.widgets.CorefExplainInteractionDialog;
import de.uniwue.mk.kall.athen.corefExplanationview.widgets.CorefExplainPreferencesDialog;
import de.uniwue.mk.kall.athen.corefExplanationview.widgets.CorefExplainTooltip;
import de.uniwue.mk.kall.athen.corefExplanationview.widgets.CorefPersonMetaanalysisDialog;
import de.uniwue.mk.kall.athen.corefExplanationview.widgets.ErrorCategoryDialog;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.IDFilter;
import de.uniwue.mk.kall.athen.part.editor.NameFilter;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.CorefCorrectionDrawingStrategy;

public class CorefExplanationView implements IPerspectiveDecoupledPart {

	@Inject
	EPartService partService;

	@Inject
	MApplication app;

	String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.corefexplain";

	// the editor this view communicates with
	private AnnotationEditorWidget editor;

	// temp types
	private String tempTypeString = "de.uniwue.mk.kallimachos.CoreferenceDebugtypeTemp";

	private String temptypeAgreeString = "de.uniwue.kalimachos.coref.type.temp1";
	private String temptypeDisAgreeString = "de.uniwue.kalimachos.coref.type.temp2";
	private String speaker = "de.uniwue.kalimachos.coref.type.temp3";
	private String spokenTo = "de.uniwue.kalimachos.coref.type.temp6";
	// used for ds
	private String temptypeDSBegEnd = "de.uniwue.kalimachos.coref.type.temp4";
	private String temptypeDSBegEnd2 = "de.uniwue.kalimachos.coref.type.temp5";

	private Type tempType;

	private Type agreeType;
	private Type disagreeType;

	// images for the table
	private HashMap<String, Image> imageMapping;

	private boolean subscribed = false;

	private Button buttonOpenInteractionDiagram;
	private Button buttonOpenSentenceEntityDiagram;

	// WIDGETS

	private Composite overlay;
	private ComboViewer comboViewer;
	private static HashSet<String> problemSet = new HashSet<String>();

	static {

		problemSet.add("falsches Geschlecht");
		problemSet.add("falscher Numerus");
		problemSet.add("falsche Person");
		problemSet.add("Sprecher falsch erkannt");
		problemSet.add("Programmierfehler");
		problemSet.add("Sonstiges");
	}

	// DATA CONTAINER
	private List<AnnotationFS> annoList;
	public HashMap<String, Integer> freqMap;

	// E4 DI OBJECTS
	@Inject
	IEventBroker broker;

	@Inject
	EModelService modelService;

	private Type debugTyp;
	private Feature idFeat;

	private Feature numerusFeat;

	private Feature personFeat;

	private CorefExplainTooltip tooltip;
	private int amountOfLastAnnos = 5;

	private Feature genderFeat;

	private Feature isNEFeat;

	private Feature isResolvedCorrectlyFeat;

	private Feature resolvedTo;

	private Feature isSubject;

	private long lastTimeCheckForTooltip;

	private Button showErrorStatistics;
	private Button showErrorStatisticsHuman;

	private Text textLinkAccuracy;

	// compounds

	// private ColViewerComparator colViewerComp;

	private MPart part;

	private Button buttonNewErrorCategory;
	private Button buttonJumpToNext;

	// Eventhandler

	EventHandler editorSelectioNChangedHandler;
	EventHandler editorCaretChangedHandler;
	EventHandler editorMouseMovedListener;

	private EventHandler onViewStackChangedHandler;

	private EventHandler onEditorInputChangedHandler;

	private EventHandler onEditorClosedHandler;

	private Button idButton;

	private TableViewer viewer;

	private IDFilter idFilter;

	private NameFilter filter;

	private Color black;

	private Color red;

	private Color green;

	private Color white;
	private Color dsOne;
	private Color dsTwo;

	private Color blue;

	private Type dSBegEndType;

	private Type dsType;

	private Type dSBegEndType2;

	private Type speakerType;
	private Type spokenToType;

	@Inject
	public CorefExplanationView() {
		// TODO Your code here
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		this.overlay = parent;
		this.overlay.setLayout(new FillLayout());

		this.part = partService.findPart(viewId);

		imageMapping = new HashMap<String, Image>();

		// ==========SUBSCRIBE TO EVENTS=============

		onEditorClosedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event arg0) {
				onEditorClosed();

			}
		};

		onEditorInputChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event e) {

				CorefExplanationView.this.onEditorInputChanged((AnnotationEditorWidget) e
						.getProperty(IEventBroker.DATA));

			}
		};

		onViewStackChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event e) {

				MPart findPart = partService.findPart(viewId);

				// we are in a diffrent perspective and the eventhandler should
				// be
				// unsubscribed already
				if (findPart == null) {
					unsubscribeFromEvents();
					return;
				}
				MElementContainer<MUIElement> parent = findPart.getParent();
				MUIElement selectedElement = parent.getSelectedElement();

				if (!selectedElement.getElementId().equals(viewId)) {
					// this part is not visible => dont render it but
					// unsubscribe the
					// listener if they are
					// remove all listeners
					unsubscribeFromEvents();
				}

			}
		};

		// subscribeToEvents();

	}

	private void unsubscribeFromEvents() {

		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_SELECTION_CHANGED,
				editorSelectioNChangedHandler);
		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);
		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_MOUSE_MOVED,
				editorMouseMovedListener);
		subscribed = false;
	}

	private boolean checkForAgreement(String genderAct, String genderPrev, String numerusAct, String numerusPrev) {

		boolean genderOk = false;

		if (genderAct.toLowerCase().contains("unknown") || genderPrev.toLowerCase().contains("unknown")) {
			genderOk = true;
		}

		else if (genderAct.equals(genderPrev)) {
			genderOk = true;
		}

		boolean numberOK = false;

		if (numerusAct.equals(numerusPrev)) {
			numberOK = true;
		}
		return (genderOk && numberOK);
	}

	private void subscribeToEvents() {

		subscribed = true;

		editorMouseMovedListener = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {

				if (!subscribed) {
					return;
				}

				if (System.currentTimeMillis() - lastTimeCheckForTooltip < 80) {
					return;
				}

				lastTimeCheckForTooltip = System.currentTimeMillis();

				Point loc = (Point) arg0.getProperty(IEventBroker.DATA);

				// for each Annotation calculate its bounds
				int bottomIndex = JFaceTextUtil.getPartialBottomIndex(editor.getWidget());
				int topIndex = JFaceTextUtil.getPartialTopIndex(editor.getWidget());

				boolean found = false;
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(debugTyp)) {
					int actualIndex = editor.getWidget().getLineAtOffset(a.getEnd());

					if (actualIndex >= topIndex && actualIndex <= bottomIndex) {
						Rectangle textBounds = editor.getWidget().getTextBounds(a.getBegin(), a.getEnd());

						if (textBounds.contains(loc)) {
							found = true;
							// open tooltip

							// add Features to the Tooltip
							String tooltipText = "";
							List<StyleRange> ranges = new ArrayList<StyleRange>();
							int lastIndex = 0;

							// "table" header

							String header = "Token     Geschlecht   Numerus         Person         IsSubject?   \n";

							tooltipText += header;
							StyleRange headerSr = new StyleRange(0, tooltipText.length(), null, null);
							headerSr.fontStyle = SWT.BOLD;

							ranges.add(headerSr);

							lastIndex = tooltipText.length();

							String actualtext = "";

							// text

							String textAct = getTextIn10Letters(a);
							actualtext += textAct;

							// gender
							String genderAct = getGenderIn10Letters(a);
							actualtext += genderAct;

							// numerus
							String numerusAct = getNumerusIn10Letters(a);
							actualtext += numerusAct;

							// person
							String personAct = getPersonIn10Letters(a);
							actualtext += personAct;

							String isSubjectTextAct = getIsSubjectIn10Letters(a);
							actualtext += isSubjectTextAct;

							tooltipText += actualtext + "\n";
							StyleRange actualSr = new StyleRange(lastIndex, actualtext.length(), blue, null);
							ranges.add(actualSr);

							FSIterator<AnnotationFS> iterator = editor.getCas().getAnnotationIndex(debugTyp).iterator();
							iterator.moveTo(a);
							lastIndex = tooltipText.length();
							for (int i = 0; i < amountOfLastAnnos; i++) {

								iterator.moveToPrevious();

								if (iterator.isValid()) {

									AnnotationFS previous = iterator.get();

									String textNE = "";

									// text

									String textPrev = getTextIn10Letters(previous);
									textNE += textPrev;

									// gender
									String genderPrev = getGenderIn10Letters(previous);
									textNE += genderPrev;

									// numerus
									String numerusPrev = getNumerusIn10Letters(previous);
									textNE += numerusPrev;

									// person
									String personPrev = getPersonIn10Letters(previous);
									textNE += personPrev;

									// subject
									String isSubPrev = getIsSubjectIn10Letters(previous);
									textNE += isSubPrev + "\n";

									tooltipText += textNE;
									// also add a rnage

									boolean match = checkForAgreement(genderAct, genderPrev, numerusAct, numerusPrev);
									if (match) {
										StyleRange range = new StyleRange(lastIndex, textNE.length(), green, null);
										ranges.add(range);
									}

									else {
										StyleRange range = new StyleRange(lastIndex, textNE.length(), red, null);
										ranges.add(range);
									}
									lastIndex = tooltipText.length();

								}
							}

							tooltip.setText(tooltipText, ranges.toArray(new StyleRange[0]));
							tooltip.activate();
							break;
						}

					}
				}

				if (!found) {
					tooltip.deactivate();
				}

			}

			private String getIsSubjectIn10Letters(AnnotationFS a) {
				String text = a.getFeatureValueAsString(isSubject);

				if (text == null) {
					text = "NULL";
				}

				while (text.length() < 15) {
					text += " ";
				}
				return text;
			}

			private String getGenderIn10Letters(AnnotationFS a) {

				String text = a.getFeatureValueAsString(genderFeat);

				if (text == null) {
					text = "NULL";
				}

				while (text.length() < 15) {
					text += " ";
				}
				return text;

			}

			private String getNumerusIn10Letters(AnnotationFS a) {

				String text = a.getFeatureValueAsString(numerusFeat);
				if (text == null) {
					text = "NULL";
				}

				while (text.length() < 15) {
					text += " ";
				}
				return text;

			}

			private String getPersonIn10Letters(AnnotationFS a) {

				String text = a.getFeatureValueAsString(personFeat);
				if (text == null) {
					text = "NULL";
				}

				while (text.length() < 15) {
					text += " ";
				}
				return text;

			}

			private String getTextIn10Letters(AnnotationFS a) {

				if (a.getCoveredText().length() <= 10) {
					String text = a.getCoveredText();

					while (text.length() < 15) {
						text += " ";
					}
					return text;
				}

				return a.getCoveredText().substring(0, 10);
			}
		};

		editorCaretChangedHandler = new EventHandler() {

			private Point actualSelection;

			public void handleEvent(org.osgi.service.event.Event arg0) {

				// check if corefdebugtyp is covered by caret
				boolean contained = false;

				Integer p = (Integer) arg0.getProperty(IEventBroker.DATA);

				// delete all temporarys

				for (AnnotationFS debug : editor.getCas().getAnnotationIndex(debugTyp)) {

					if (debug.getBegin() > p) {
						break;
					}

					if (p >= debug.getBegin() && p <= debug.getEnd()) {
						contained = true;
						break;

					}
				}

				if (!contained) {
					List<AnnotationFS> temps = new ArrayList<AnnotationFS>();

					for (AnnotationFS temp : editor.getCas().getAnnotationIndex(tempType)) {
						temps.add(temp);
					}
					for (AnnotationFS temp : editor.getCas().getAnnotationIndex(agreeType)) {
						temps.add(temp);
					}
					for (AnnotationFS temp : editor.getCas().getAnnotationIndex(disagreeType)) {
						temps.add(temp);
					}

					for (AnnotationFS t : temps) {
						editor.getCas().removeFsFromIndexes(t);
					}

				}

				// step1 delete speaker and spokenTo
				ArrayList<AnnotationFS> speakerAnnos = new ArrayList<AnnotationFS>();
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(speakerType)) {
					speakerAnnos.add(a);
				}
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(spokenToType)) {
					speakerAnnos.add(a);
				}

				for (AnnotationFS b : speakerAnnos) {
					editor.removeAnnotation(b);
				}
				// check if a DS is covered

				AnnotationFS coveringDS = null;

				for (AnnotationFS ds : editor.getCas().getAnnotationIndex(dsType)) {

					if (ds.getBegin() <= p && ds.getEnd() >= p) {
						coveringDS = ds;
						break;
					}

				}

				if (coveringDS != null) {

					// TODO dirty
					Feature speaker = dsType.getFeatureByBaseName("Speaker");
					// get the speaker of that anno
					AnnotationFS speakerNE = (AnnotationFS) coveringDS.getFeatureValue(speaker);

					if (speakerNE == null) {
						return;
					}

					AnnotationFS createAnnotation = editor.getCas().createAnnotation(speakerType, speakerNE.getBegin(),
							speakerNE.getEnd());

					editor.addAnnotation(createAnnotation);

					// also add spokenToAnnotation if possible
					Feature spokenToF = dsType.getFeatureByBaseName("SpokenTo");
					AnnotationFS spokenTo = (AnnotationFS) coveringDS.getFeatureValue(spokenToF);

					if (spokenTo == null) {
						return;
					}

					AnnotationFS createAnnotationFS = editor.getCas().createAnnotation(spokenToType,
							spokenTo.getBegin(), spokenTo.getEnd());

					editor.addAnnotation(createAnnotationFS);

					System.out.println("Added " + createAnnotationFS.getCoveredText());

				}

			}
		};

		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);

		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_MOUSE_MOVED,
				editorMouseMovedListener);

		editorSelectioNChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event arg0) {

				// delete all temporarys
				AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(tempType);

				List<AnnotationFS> aList = new ArrayList<AnnotationFS>();

				// temptypes
				for (AnnotationFS a : annotationIndex) {
					aList.add(a);
				}
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(agreeType)) {
					aList.add(a);
				}

				for (AnnotationFS a : editor.getCas().getAnnotationIndex(disagreeType)) {
					aList.add(a);
				}

				for (AnnotationFS a : aList) {
					editor.getCas().removeFsFromIndexes(a);
				}

				// get the according anno

				List<AnnotationFS> selection = (List<AnnotationFS>) arg0.getProperty(IEventBroker.DATA);

				if (selection.size() != 1) {
					return;
				}

				AnnotationFS anno = selection.get(0);
				AnnotationFS fss = editor.getCas().createAnnotation(tempType, anno.getBegin(), anno.getEnd());
				editor.addAnnotation(fss);

				// get the previous 5

				FSIterator<AnnotationFS> iterator = editor.getCas().getAnnotationIndex(debugTyp).iterator();

				iterator.moveTo(anno);

				List<AnnotationFS> previousList = new ArrayList<AnnotationFS>();

				for (int i = 0; i < amountOfLastAnnos; i++) {
					iterator.moveToPrevious();
					if (iterator.isValid())
						previousList.add(iterator.get());
				}

				// annotate more temporarys

				for (AnnotationFS last : previousList) {

					boolean agree = checkForAgreement(anno.getFeatureValueAsString(genderFeat),
							last.getFeatureValueAsString(genderFeat), anno.getFeatureValueAsString(numerusFeat),
							last.getFeatureValueAsString(numerusFeat));

					AnnotationFS fs = null;
					if (agree) {
						fs = editor.getCas().createAnnotation(agreeType, last.getBegin(), last.getEnd());
					}

					else {

						fs = editor.getCas().createAnnotation(disagreeType, last.getBegin(), last.getEnd());
					}
					editor.addAnnotation(fs);

				}

				// redraw
				overlay.layout();

			}
		};
		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_SELECTION_CHANGED,
				editorSelectioNChangedHandler);
	}

	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		this.editor = editor;

		black = new Color(overlay.getDisplay(), new RGB(0, 0, 0));
		red = new Color(overlay.getDisplay(), new RGB(255, 0, 0));
		green = new Color(overlay.getDisplay(), new RGB(39, 176, 87));
		white = new Color(overlay.getDisplay(), new RGB(255, 255, 255));
		blue = new Color(overlay.getDisplay(), new RGB(17, 103, 194));
		dsOne = new Color(overlay.getDisplay(), new RGB(209, 205, 212));
		dsTwo = new Color(overlay.getDisplay(), new RGB(178, 174, 181));

		tooltip = new CorefExplainTooltip(editor.getWidget());
		tooltip.setShift(new Point(5, 5));

		// only initialize if the part is visible

		MPart findPart = partService.findPart(viewId);
		MUIElement selectedElement = null;
		if (findPart != null) {
			MElementContainer<MUIElement> parent = findPart.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but unsubscribe
			// the
			// listener if they are
			// remove all listeners
			unsubscribeFromEvents();
			subscribed = false;
			return;
		}

		boolean typesValid = initTypes();

		while (!typesValid) {

			// open preference dialogue and ask the user to enter valid types
			CorefExplainPreferencesDialog dialogue = new CorefExplainPreferencesDialog(this.overlay.getShell());

			int open = dialogue.open();

			if (open == 0) {
				// success

				HashMap<String, String> preferences = dialogue.getPreferences();

				// save all the preferences to the store
				for (String l : preferences.keySet()) {

					ApplicationUtil.saveToPreferences(l, preferences.get(l));
				}

			}

			else {
				break;
			}

			typesValid = initTypes();

		}

		// if the part is active add the listener

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// init AnnoList

		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(debugTyp)) {
			annoList.add(a);
		}

		// ====FREQUENCIES=======
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getCoveredText())) {
				int freq = freqMap.get(anno.getCoveredText());
				freqMap.put(anno.getCoveredText(), freq + 1);
			}

			else {
				freqMap.put(anno.getCoveredText(), 1);
			}
		}

		initLayout(overlay);

		if (!subscribed) {
			subscribeToEvents();
		}

		// ====reload=====

		overlay.layout();
		overlay.getParent().layout();
	}

	public void onEditorClosed() {

		// remove all components

		System.out.println("close corefview");

		for (Control c : this.overlay.getChildren()) {
			c.dispose();
		}

		// unsubscribe from editor notifications

		unsubscribeFromEvents();

		subscribed = false;
		this.editor = null;
	}

	private boolean initTypes() {

		// get all Strings from preferences
		String sIsNe = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_ISNE, "null");
		String sDebugType = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_DEBUGTYPE, "null");
		String sgender = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_GENDER, "null");
		String sNumerus = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_NUMERUS, "null");
		String sPerson = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_PERSON, "null");
		String sResolvedTo = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RESOLVED_TO,
				"null");
		String sId = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_ID_FEATURE, "null");
		String sIsCorrect = ApplicationUtil.getFromPreferences(
				ApplicationPreferences.EXPLAIN_COREF_ISRESOLVEDCORRECTLY, "null");
		String sIsSubject = ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_ISSUBJECT, "null");
		boolean fullyInitalized;
		fullyInitalized = (sIsNe.equals("null") || sDebugType.equals("null") || sgender.equals("null")
				|| sNumerus.equals("null") || sPerson.equals("null") || sResolvedTo.equals("null")
				|| sId.equals("null") || sIsCorrect.equals("null") || sIsSubject.equals("null")) ? false : true;

		if (!fullyInitalized)
			return false;

		this.debugTyp = editor.getCas().getTypeSystem().getType(sDebugType);

		// also get the temptypes
		this.tempType = editor.getCas().getTypeSystem().getType(tempTypeString);
		this.agreeType = editor.getCas().getTypeSystem().getType(temptypeAgreeString);
		this.disagreeType = editor.getCas().getTypeSystem().getType(temptypeDisAgreeString);
		this.dSBegEndType = editor.getCas().getTypeSystem().getType(temptypeDSBegEnd);
		this.dSBegEndType2 = editor.getCas().getTypeSystem().getType(temptypeDSBegEnd2);
		this.speakerType = editor.getCas().getTypeSystem().getType(speaker);
		this.spokenToType = editor.getCas().getTypeSystem().getType(spokenTo);

		// change the font of the editor widget

		// set everything unnneded invisible
		this.editor.allStylesInvisible();

		this.idFeat = this.debugTyp.getFeatureByBaseName(sId);
		this.isResolvedCorrectlyFeat = this.debugTyp.getFeatureByBaseName(sIsCorrect);
		this.isNEFeat = this.debugTyp.getFeatureByBaseName(sIsNe);
		this.personFeat = this.debugTyp.getFeatureByBaseName(sPerson);
		this.numerusFeat = this.debugTyp.getFeatureByBaseName(sNumerus);
		this.genderFeat = this.debugTyp.getFeatureByBaseName(sgender);
		this.resolvedTo = this.debugTyp.getFeatureByBaseName(sResolvedTo);
		this.isSubject = this.debugTyp.getFeatureByBaseName(sIsSubject);

		// if NES are set we can use them

		Type netype = editor.getCas().getTypeSystem()
				.getType(ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_TYPE, "null"));
		if (netype != null) {

			editor.changeVisibility(netype, true);
			editor.changeLayer(netype, 1);
			StyleRange rangeTyp2 = new StyleRange();
			rangeTyp2.background = new Color(overlay.getDisplay(), 224, 139, 27);
			AnnoStyle style2 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp2), rangeTyp2);
			editor.changeStyle(netype, style2);
		}

		// Directspeechtype

		// TODO dirty
		dsType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.DirectSpeech");
		annotateDstempType();
		// use another Temptype to display begin and end of the direct speech
		// annotations

		// Visibility

		editor.changeVisibility(debugTyp, true);
		editor.changeVisibility(tempType, true);
		editor.changeVisibility(agreeType, true);
		editor.changeVisibility(disagreeType, true);
		// editor.changeVisibility(dsType, true);
		editor.changeVisibility(dSBegEndType, true);
		editor.changeVisibility(dSBegEndType2, true);
		editor.changeVisibility(speakerType, true);
		editor.changeVisibility(spokenToType, true);

		// layer

		editor.changeLayer(debugTyp, 1);
		editor.changeLayer(dsType, 0);
		editor.changeLayer(tempType, 2);
		editor.changeLayer(agreeType, 2);
		editor.changeLayer(disagreeType, 2);

		// style - red
		StyleRange rangeTyp = new StyleRange();
		rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle style = new AnnoStyle(new CorefCorrectionDrawingStrategy(rangeTyp), rangeTyp);

		// tempostyles
		StyleRange rangeTyp2 = new StyleRange();
		rangeTyp2.background = blue;

		AnnoStyle style2 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp2), rangeTyp2);

		StyleRange rangeTyp3 = new StyleRange();
		rangeTyp3.background = red;

		AnnoStyle style3 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp3), rangeTyp3);

		StyleRange rangeTyp4 = new StyleRange();
		rangeTyp4.background = green;

		AnnoStyle style4 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp4), rangeTyp4);

		// ds temp

		StyleRange rangeTyp5 = new StyleRange();
		rangeTyp5.background = dsOne;

		AnnoStyle style5 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp5), rangeTyp5);

		StyleRange rangeTyp6 = new StyleRange();
		rangeTyp6.background = dsTwo;

		AnnoStyle style6 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTyp6), rangeTyp6);

		// ds style
		StyleRange rangeTypDS = new StyleRange();

		AnnoStyle styleDS = new AnnoStyle(new ItalicDrawingStrategy(rangeTypDS), rangeTypDS);

		// speaker
		StyleRange rangeTypSpeaker = new StyleRange();
		rangeTypSpeaker.background = red;

		AnnoStyle styleSpeaker = new AnnoStyle(new OvalDrawingStrategy(rangeTypSpeaker), rangeTypSpeaker);

		// spokenTo
		StyleRange rangeTypSpokenTo = new StyleRange();
		rangeTypSpokenTo.background = blue;

		AnnoStyle styleSpokenTo = new AnnoStyle(new OvalDrawingStrategy(rangeTypSpokenTo), rangeTypSpokenTo);

		// apply
		editor.changeStyle(debugTyp, style);
		editor.changeStyle(tempType, style2);
		editor.changeStyle(disagreeType, style3);
		editor.changeStyle(agreeType, style4);
		editor.changeStyle(dsType, styleDS);
		editor.changeStyle(dSBegEndType, style5);
		editor.changeStyle(dSBegEndType2, style6);
		editor.changeStyle(speakerType, styleSpeaker);
		editor.changeStyle(spokenToType, styleSpokenTo);
		// also change the modus
		editor.setActiveMode(debugTyp);

		return true;

	}

	private void annotateDstempType() {

		if (dSBegEndType == null) {
			return;
		}

		// step 1 delete all old ones

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS a : editor.getCas().getAnnotationIndex(dSBegEndType)) {
			toDelete.add(a);
		}

		for (AnnotationFS a : editor.getCas().getAnnotationIndex(dSBegEndType2)) {
			toDelete.add(a);
		}

		for (AnnotationFS a : toDelete) {
			editor.getCas().removeFsFromIndexes(a);
		}

		// step 2 create new ones with the temptypes

		int index = 0;
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(dsType)) {

			if (index % 2 == 0) {
				AnnotationFS createAnnotation = editor.getCas()
						.createAnnotation(dSBegEndType, a.getBegin(), a.getEnd());

				editor.getCas().addFsToIndexes(createAnnotation);
			}

			else {

				AnnotationFS createAnnotation = editor.getCas().createAnnotation(dSBegEndType2, a.getBegin(),
						a.getEnd());

				editor.getCas().addFsToIndexes(createAnnotation);
			}

			index++;
		}

	}

	private void initLayout(Composite parent) {
		// this.overlay = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 5;
		layout.numColumns = 2;
		layout.verticalSpacing = 5;
		layout.marginWidth = 5;
		layout.marginHeight = 5;
		overlay.setLayout(layout);

		// / ==== Buttons + Fields

		Composite comp2 = new Composite(overlay, SWT.BORDER);
		comp2.setVisible(true);
		GridLayout gdd = new GridLayout();
		GridData gridData2 = new GridData();
		gridData2.horizontalSpan = 2;
		gridData2.grabExcessHorizontalSpace = true;
		// gridData2.grabExcessVerticalSpace = true;
		gridData2.horizontalAlignment = GridData.FILL;
		comp2.setLayout(gdd);
		comp2.setLayoutData(gridData2);

		// comboviewer

		this.comboViewer = new ComboViewer(comp2, SWT.NONE);
		initComboViewer();

		buttonNewErrorCategory = new Button(comp2, SWT.PUSH);
		buttonNewErrorCategory.setText("New Category");
		buttonNewErrorCategory.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// open a dialog => save the new error category =>reload
				ErrorCategoryDialog dialog = new ErrorCategoryDialog(overlay.getShell());

				int open = dialog.open();

				// success
				if (open == 0) {

					String text = dialog.getErrorCategory();

					// create string for preferences

					String prefText = "";
					for (String errorCode : problemSet) {
						prefText += errorCode + ";";
					}

					prefText += text;

					ApplicationUtil.saveToPreferences(ApplicationPreferences.COREFERENCE_ERROR_CATEGORIES, prefText);

					// reload
					try {
						renewPage();
					} catch (CASException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		showErrorStatistics = new Button(comp2, SWT.NONE);
		showErrorStatistics.setText("Error Statistic in words");
		showErrorStatisticsHuman = new Button(comp2, SWT.NONE);
		showErrorStatisticsHuman.setText("Error Statistics per Category");

		showErrorStatistics.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				HashMap<String, Integer> map = getErrorStatistics();
				final CorefErrorStatistics demo = new CorefErrorStatistics("Error Statistik pro Wort", map);
				demo.pack();
				RefineryUtilities.centerFrameOnScreen(demo);
				demo.setVisible(true);

			}

			private HashMap<String, Integer> getErrorStatistics() {
				HashMap<String, Integer> errorMapping = new HashMap<String, Integer>();

				for (AnnotationFS fs : editor.getCas().getAnnotationIndex(debugTyp)) {

					if (!fs.getFeatureValueAsString(isResolvedCorrectlyFeat).equals("true")) {

						if (errorMapping.containsKey(fs.getCoveredText())) {

							Integer integer = errorMapping.get(fs.getCoveredText());
							integer++;
							errorMapping.put(fs.getCoveredText(), integer);
						}

						else {
							errorMapping.put(fs.getCoveredText(), 1);

						}
					}
				}
				return errorMapping;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}

		});

		showErrorStatisticsHuman.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				HashMap<String, Integer> map = getErrorStatistics();
				final CorefErrorStatistics demo = new CorefErrorStatistics("Error Statistik pro Wort", map);
				demo.pack();
				RefineryUtilities.centerFrameOnScreen(demo);
				demo.setVisible(true);

			}

			private HashMap<String, Integer> getErrorStatistics() {
				HashMap<String, Integer> errorMapping = new HashMap<String, Integer>();

				Feature errorCat = debugTyp.getFeatureByBaseName("ErrorCategory");

				for (AnnotationFS fs : editor.getCas().getAnnotationIndex(debugTyp)) {

					if (fs.getFeatureValueAsString(errorCat) != null) {

						if (errorMapping.containsKey(fs.getFeatureValueAsString(errorCat))) {

							Integer integer = errorMapping.get(fs.getFeatureValueAsString(errorCat));
							integer++;
							errorMapping.put(fs.getFeatureValueAsString(errorCat), integer);
						}

						else {
							errorMapping.put(fs.getFeatureValueAsString(errorCat), 1);

						}
					}
				}
				return errorMapping;
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		buttonJumpToNext = new Button(comp2, SWT.PUSH);

		buttonJumpToNext.setText("Jump To Next Error ");

		buttonJumpToNext.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS toHIghlight = getNextToHighLight(editor.getWidget().getCaretOffset());

				if (toHIghlight != null) {
					editor.highlightRange(toHIghlight.getBegin(), toHIghlight.getEnd() - toHIghlight.getBegin(), true);

					editor.setSelection(toHIghlight);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		textLinkAccuracy = new Text(comp2, SWT.None);
		textLinkAccuracy.setEditable(false);
		String linkAccuracy = calculateLinkAccuracy();

		if (linkAccuracy.length() > 4) {
			linkAccuracy = linkAccuracy.substring(0, 4) + "%";

			textLinkAccuracy.setText("Link Accuracy: " + linkAccuracy);
		}

		buttonOpenInteractionDiagram = new Button(comp2, SWT.PUSH);
		buttonOpenSentenceEntityDiagram = new Button(comp2, SWT.PUSH);

		buttonOpenInteractionDiagram.setText("Open Interaction Diagram");
		buttonOpenSentenceEntityDiagram.setText("Open Entity Appearance Overview");

		buttonOpenSentenceEntityDiagram.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// open a nonmodal dialog

				CorefPersonMetaanalysisDialog dialog = new CorefPersonMetaanalysisDialog(
						CorefExplanationView.this.overlay.getShell(), editor);

				dialog.open();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		buttonOpenInteractionDiagram.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				CorefExplainInteractionDialog dialog = new CorefExplainInteractionDialog(
						CorefExplanationView.this.overlay.getShell(), editor);

				dialog.open();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// Label actions = new Label(comp2, SWT.NULL);
		// actions.setText("Actions");
		// Composite comp = new Composite(overlay, SWT.BORDER);
		// comp.setVisible(true);

		GridLayout gridBelow = new GridLayout(2, false);
		comp2.setLayout(gridBelow);
		// listener

		// Radiobuttons
		Composite composite = new Composite(overlay, SWT.NULL);
		gdd = new GridLayout();
		gdd.numColumns = 3;
		gdd.marginBottom = gdd.marginTop = gdd.marginLeft = gdd.marginRight = 0;
		composite.setLayout(gdd);

		// // ==== Table
		comp2.setSize(overlay.getSize());

		Composite compp = new Composite(overlay, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		Label idFilterLabel = new Label(compp, SWT.NONE);
		idFilterLabel.setText("ID Filter active: ");
		idButton = new Button(compp, SWT.CHECK);
		idButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		idButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					renewPage();
				} catch (CASException e1) {
					e1.printStackTrace();
				}
			}
		});

		// searchLabel
		Label searchLabel = new Label(compp, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compp, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		viewer = new TableViewer(compp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		// add Menu

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);
		// New to support the search

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				// filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		// filter
		filter = new NameFilter();
		viewer.addFilter(filter);

		// id filter

		idFilter = new IDFilter();
		viewer.addFilter(idFilter);

		createTableViewer();
		// comparator

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

	}

	private String calculateLinkAccuracy() {

		double correct = 0;
		for (AnnotationFS a : this.annoList) {

			if (a.getFeatureValueAsString(isResolvedCorrectlyFeat).contains("t")) {
				correct++;

			}
		}

		double value = correct / this.annoList.size();

		value = value * 100;
		return new Double(value).toString();
	}

	private void initComboViewer() {

		// read all the preferences that have ever been set
		comboViewer.setContentProvider(ArrayContentProvider.getInstance());

		String fromPreferences = ApplicationUtil.getFromPreferences(
				ApplicationPreferences.COREFERENCE_ERROR_CATEGORIES, "null");

		if (!fromPreferences.equals("null")) {

			problemSet.addAll(Arrays.asList(fromPreferences.split(";")));
		}

		// set the input

		comboViewer.setInput(problemSet.toArray(new String[0]));

		comboViewer.getCombo().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// set the error category
				// TODO dirty
				Feature errorFeature = debugTyp.getFeatureByBaseName("ErrorCategory");

				// set this feature
				String sel = (String) ((IStructuredSelection) comboViewer.getSelection()).getFirstElement();

				// check selection of editor

				if (editor.getSelectionOfVisibleAnnotations().size() == 1) {

					AnnotationFS annotationFS = editor.getSelectionOfVisibleAnnotations().get(0);

					annotationFS.setFeatureValueFromString(errorFeature, sel);
					editor.changeAnnotation(annotationFS);

					// jump to next error

					AnnotationFS toHIghlight = getNextToHighLight(annotationFS);

					if (toHIghlight != null) {
						getEditor().highlightRange(toHIghlight.getBegin(),
								toHIghlight.getEnd() - toHIghlight.getBegin(), true);

						getEditor().setSelection(toHIghlight);
					}
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

	}

	private AnnotationFS getNextToHighLight(AnnotationFS actual) {

		Collections.sort(this.annoList, new AnnotationComparator());

		for (int i = 0; i < annoList.size(); i++) {
			if (actual.getBegin() < annoList.get(i).getBegin()
					&& !annoList.get(i).getFeatureValueAsString(isResolvedCorrectlyFeat).equals("true")) {
				return annoList.get(i);
			}
		}
		return null;

	}

	private AnnotationFS getNextToHighLight(int caretOffset) {

		Collections.sort(this.annoList, new AnnotationComparator());

		for (int i = 0; i < annoList.size(); i++) {
			if (caretOffset < annoList.get(i).getBegin()
					&& !annoList.get(i).getFeatureValueAsString(isResolvedCorrectlyFeat).equals("true")) {
				return annoList.get(i);
			}
		}
		return null;

	}

	public void renewPage() throws CASException {

		annoList = new ArrayList<AnnotationFS>();

		for (AnnotationFS a : editor.getCas().getAnnotationIndex(debugTyp)) {
			annoList.add(a);
		}

		// update frequencyMap
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getFeatureValueAsString(idFeat))) {
				int freq = freqMap.get(anno.getFeatureValueAsString(idFeat));
				freq += 1;
				freqMap.put(anno.getFeatureValueAsString(idFeat), freq);
			}

			else {
				freqMap.put(anno.getFeatureValueAsString(idFeat), 1);
			}
		}

		//

		Runnable r = new Runnable() {

			@Override
			public void run() {
				onEditorInputChanged(editor);

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void createTableViewer() {
		Integer maxId = 0;
		for (AnnotationFS annot : this.annoList) {
			if (annot.getFeatureValueAsString(idFeat) != null && annot.getFeatureValueAsString(idFeat) != "") {
				Integer ids = Integer.parseInt(annot.getFeatureValueAsString(idFeat));
				if (ids > maxId) {
					maxId = ids;
				}
			}
		}

		for (AnnotationFS anno : this.annoList) {

			String id = anno.getFeatureValueAsString(idFeat);
			if (id == "" || id == null) {
				maxId++;
				// unassigned ids to -1
				anno.setFeatureValueFromString(idFeat, maxId.toString());

			}
		}

		// set input of Tableviewer
		viewer.setInput(annoList);

		viewer.refresh();
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Text", 100, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getCoveredText();

			}

		});

		TableViewerColumn colFreq = createTableViewerColumn("Freq.", 30, 1);
		colFreq.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String id = p.getFeatureValueAsString(idFeat);
				if (freqMap.containsKey(id)) {
					return freqMap.get(id).toString();

				}

				else {
					return "1";
				}
			}
		});

		TableViewerColumn colID = createTableViewerColumn("ID", 30, 2);
		colID.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(idFeat);
			}
		});

		TableViewerColumn numerus = createTableViewerColumn("Numerus", 30, 3);
		numerus.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return null;
			}

			@Override
			public Image getImage(Object element) {
				AnnotationFS p = (AnnotationFS) element;

				Image numerus = loadImage(p.getFeatureValueAsString(numerusFeat));

				return numerus;

			}

		});

		TableViewerColumn colGender = createTableViewerColumn("Gender", 30, 4);
		colGender.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return null;
			}

			@Override
			public Image getImage(Object element) {
				AnnotationFS p = (AnnotationFS) element;

				Image numerus = loadImage(p.getFeatureValueAsString(genderFeat));

				return numerus;

			}
		});

		TableViewerColumn colPerson = createTableViewerColumn("Person", 30, 5);
		colPerson.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String featureValue = p.getFeatureValueAsString(personFeat);

				return featureValue;
			}
		});

		TableViewerColumn colIsNE = createTableViewerColumn("IsNe", 30, 6);
		colIsNE.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;

				return p.getFeatureValueAsString(isNEFeat);
			}
		});

		TableViewerColumn colCorrect = createTableViewerColumn("Correct", 30, 7);
		colCorrect.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(isResolvedCorrectlyFeat);
			}
		});

		TableViewerColumn colResolvedTo = createTableViewerColumn("ResolvedTo", 30, 8);
		colResolvedTo.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				if (p == null || resolvedTo == null) {
					return "NULL";
				}

				String coveredText = ((AnnotationFS) p.getFeatureValue(resolvedTo)).getCoveredText();
				if (coveredText == null) {
					return "NULL";
				}
				return coveredText;
			}
		});

	}

	public Image loadImage(String feature) {

		if (feature == null) {
			feature = "empty";
		}

		if (this.imageMapping.containsKey(feature)) {

			return this.imageMapping.get(feature);
		}

		else {

			if (feature.toLowerCase().equals("singular")) {

				Image createImage = ApplicationUtil.createImage(ApplicationUtil.WSD_CAT_MENSCH);
				imageMapping.put(feature, createImage);
				return createImage;
			}

			else if (feature.toLowerCase().equals("plural")) {

				Image createImage = ApplicationUtil.createImage(ApplicationUtil.WSD_CAT_MENGE);
				imageMapping.put(feature, createImage);
				return createImage;
			}

			else if (feature.toLowerCase().equals("male")) {

				Image createImage = ApplicationUtil.createImage(ApplicationUtil.GENDER_MALE);
				imageMapping.put(feature, createImage);
				return createImage;
			}

			else if (feature.toLowerCase().equals("female")) {

				Image createImage = ApplicationUtil.createImage(ApplicationUtil.GENDER_FEMALE);
				imageMapping.put(feature, createImage);
				return createImage;
			}

			else if (feature.toLowerCase().equals("unknown")) {

				Image createImage = ApplicationUtil.createImage(ApplicationUtil.GENDER_UNKNOWN);
				imageMapping.put(feature, createImage);
				return createImage;
			}

			else {

				Image im = new Image(editor.getParent().getDisplay(), 16, 16);

				GC gc = new GC(im);

				gc.setBackground(new Color(editor.getParent().getDisplay(), new RGB(255, 0, 0)));

				gc.fillRectangle(0, 0, 16, 16);

				gc.dispose();

				imageMapping.put(feature, im);
				return im;
			}

		}

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	// boolean moveCursor
	private void changeHighlighting(int beg, int end, boolean b) {
		editor.highlightRange(beg, end - beg, true);

	}

	public List<AnnotationFS> getAnnoList() {
		return annoList;
	}

	public void setAnnoList(List<AnnotationFS> annoList) {
		this.annoList = annoList;
	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public TableViewer getViewer() {
		return viewer;
	}

	public void setViewer(TableViewer viewer) {
		this.viewer = viewer;
	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {

		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		if (!subscribed && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

	}

	@Persist
	public void save() {
		// TODO Your code here

		black.dispose();
		red.dispose();
		green.dispose();
		white.dispose();
		blue.dispose();
		dsOne.dispose();
		dsTwo.dispose();
	}

	@Override
	public void coupleToPerspective() {
		System.out.println("Couple CorefCorrectionView");

		partService.bringToTop(part);

		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedHandler);

		MasterPart.subscribeForEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputChangedHandler);

		subscribeToEvents();

	}

	@Override
	public void decoupleFromPerspective() {
		System.out.println("Decouple CorefCorrectionView");

		// free the expensive objects TODO better if this is in a separate
		// method
		unsubscribeFromEvents();
		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedHandler);

		MasterPart.unsubscribeFromEvent(CorefExplanationView.class, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputChangedHandler);

	}

}