package de.uniwue.mk.kall.athen.corefExplanationview.widgets;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.corefExplanationview.ui.InteractionPersonFilter;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class CorefExplainInteractionDialog extends Dialog {

	private HashMap<String, Color> idColorMapping;
	private AnnotationEditorWidget editor;

	private Composite parent;
	private int r = 141;
	private int g = 223;
	private int b = 187;

	private Point lastClick;

	private TableViewer viewer;
	private InteractionPersonFilter personFilter;
	private InteractionFilter interactionFilter;

	public CorefExplainInteractionDialog(Shell parentShell, AnnotationEditorWidget editor) {
		super(parentShell);
		setShellStyle(getShellStyle() | SWT.RESIZE);

		this.editor = editor;

		idColorMapping = new HashMap<String, Color>();
	}

	protected Control createDialogArea(Composite parent) {

		this.parent = parent;

		// parent.getShell().setSize(new Point(500, 800));

		parent.getShell().setText("The Interaction Graph of the underlying Text");

		GridLayout layout = new GridLayout();
		// layout.marginBottom = 10;
		// layout.marginTop = 10;
		// layout.marginLeft = 15;
		// layout.marginRight = 15;
		parent.setLayout(layout);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);

		// set input

		List<AnnotationFS> annoList = new ArrayList<AnnotationFS>();

		Type intActType = editor.getCas().getTypeSystem()
				.getType(ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_CONVTYPE, "null"));

		if (intActType != null) {
			for (AnnotationFS a : editor.getCas().getAnnotationIndex(intActType)) {

				annoList.add(a);
			}

		}

		viewer.setInput(annoList.toArray(new AnnotationFS[0]));

		// listener and menu

		viewer.getTable().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				lastClick = new Point(e.x, e.y);

			}

			@Override
			public void mouseDown(MouseEvent e) {
				lastClick = new Point(e.x, e.y);

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				lastClick = new Point(e.x, e.y);

			}
		});

		personFilter = new InteractionPersonFilter();
		viewer.addFilter(personFilter);

		interactionFilter = new InteractionFilter();
		viewer.addFilter(interactionFilter);

		// Menu
		final Menu menu = new Menu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// jump to
		MenuItem del = new MenuItem(menu, SWT.None);

		del.setText("Jump To Selection Interaction");

		del.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				editor.highlightRange(firstElement.getBegin(), 5, true);

			}

		});

		// personfilter - aktivate
		MenuItem personFilterMenu = new MenuItem(menu, SWT.None);

		personFilterMenu.setText("Activate Personfilter");

		personFilterMenu.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				ViewerCell cell = viewer.getCell(lastClick);

				int columnIndex = cell.getColumnIndex();

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS p = (AnnotationFS) sel.getFirstElement();

				// if first index => Agent

				if (columnIndex == 1) {
					FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_AGENT, "null")));

					AnnotationFS anno = (AnnotationFS) featureValue;

					String id = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

					personFilter.setId(id);
					viewer.refresh();

				}

				// if 2nd => recipient

				if (columnIndex == 2) {
					FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RECIPIENT, "null")));

					AnnotationFS anno = (AnnotationFS) featureValue;

					String id = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));
					personFilter.setId(id);
					viewer.refresh();
				}
			}

		});

		// personfilter - deaktivate
		MenuItem personFilterDeakMenu = new MenuItem(menu, SWT.None);

		personFilterDeakMenu.setText("Deactivate Personfilter");

		personFilterDeakMenu.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent event) {
				personFilter.setId(null);
				interactionFilter.setId(null);
				viewer.refresh();
			}
		});

		// interactionfilter - aktivate
		MenuItem intFilterMenu = new MenuItem(menu, SWT.None);

		intFilterMenu.setText("Activate Interaction");

		intFilterMenu.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS p = (AnnotationFS) sel.getFirstElement();

				// if first index => Agent

				FeatureStructure featureValueA = p.getFeatureValue(p.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_AGENT, "null")));

				AnnotationFS anno = (AnnotationFS) featureValueA;

				String idAg = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

				// if 2nd => recipient

				FeatureStructure featureValueR = p.getFeatureValue(p.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RECIPIENT, "null")));

				AnnotationFS annoR = (AnnotationFS) featureValueR;

				String idRec = annoR.getFeatureValueAsString(annoR.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));
				interactionFilter.setId(idAg + idRec);
				viewer.refresh();

			}

		});

		return parent;

	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Interaktion", 50, 0);

		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getType().getShortName();

			}

			@Override
			public Color getBackground(final Object element) {
				if (element instanceof AnnotationFS) {

					AnnotationFS p = (AnnotationFS) element;

					FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_AGENT, "null")));

					AnnotationFS annoAgent = (AnnotationFS) featureValue;

					String idAgent = annoAgent.getFeatureValueAsString(annoAgent.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

					// rec
					FeatureStructure featureValueR = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RECIPIENT, "null")));

					AnnotationFS annoR = (AnnotationFS) featureValueR;

					String idRec = annoR.getFeatureValueAsString(annoR.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

					String id = idAgent + idRec;

					if (idColorMapping.containsKey(id)) {
						return idColorMapping.get(id);
					}

					else {
						Color c = generateNextColor(id);
						return c;
					}

				}

				return super.getBackground(element);
			}

		});

		TableViewerColumn colFreq = createTableViewerColumn("Agent", 100, 1);
		colFreq.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_AGENT, "null")));

				AnnotationFS anno = (AnnotationFS) featureValue;

				return anno.getCoveredText();

			}

			@Override
			public Color getBackground(final Object element) {
				if (element instanceof AnnotationFS) {

					AnnotationFS p = (AnnotationFS) element;

					FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_AGENT, "null")));

					AnnotationFS anno = (AnnotationFS) featureValue;

					String id = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

					if (idColorMapping.containsKey(id)) {
						return idColorMapping.get(id);
					}

					else {
						Color c = generateNextColor(id);
						return c;
					}

				}

				return super.getBackground(element);
			}

		});

		TableViewerColumn colID = createTableViewerColumn("Recipient", 100, 2);
		colID.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
						ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RECIPIENT, "null")));

				AnnotationFS anno = (AnnotationFS) featureValue;

				return anno.getCoveredText();

			}

			@Override
			public Color getBackground(final Object element) {
				if (element instanceof AnnotationFS) {

					AnnotationFS p = (AnnotationFS) element;

					FeatureStructure featureValue = p.getFeatureValue(p.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.EXPLAIN_COREF_RECIPIENT, "null")));

					AnnotationFS anno = (AnnotationFS) featureValue;

					String id = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(
							ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null")));

					if (idColorMapping.containsKey(id)) {
						return idColorMapping.get(id);
					}

					else {
						Color c = generateNextColor(id);
						return c;
					}
				}

				return super.getBackground(element);
			}
		});

	}

	private Color generateNextColor(String id) {

		// update rgb paradigm
		r = r + 21;
		if (r > 254)
			r = 132;

		g = g - 17;
		if (g < 130)
			g = 242;

		b = b + 33;
		if (b > 254)
			b = 151;

		Color c = new Color(parent.getDisplay(), new RGB(r, g, b));

		this.idColorMapping.put(id, c);
		return c;
	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

}
