package de.uniwue.mk.kall.athen.corefExplanationview.widgets;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;


import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.corefExplanationview.ui.AnnotationContainerModel;
import de.uniwue.mk.kall.athen.corefExplanationview.ui.ModelObjectsComparator;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class CorefPersonMetaanalysisDialog extends Dialog {

	private Composite parent;

	private AnnotationEditorWidget editor;

	// types

	private Type neType;
	private Type sentenceType;
	private Feature neId;
	private Type posType;
	private Feature posTagFeature;

	// widgets

	private TableViewer viewer;
	private Button buttonaddGroup;
	private Button buttonExportToExcel;

	// helper

	HashMap<AnnotationFS, List<AnnotationFS>> sentNEMapping;
	HashMap<AnnotationFS, String> sentIndexMapping;
	HashMap<String, String> idNameMapping;
	HashMap<String, Set<String>> idSynonymMap;

	List<AnnotationContainerModel> modelObjects;

	private List<Pair<String, Integer>> neMapping;

	private int groupindex = 0;

	private Point lastClick;

	public CorefPersonMetaanalysisDialog(Shell parentShell, AnnotationEditorWidget editor) {
		super(parentShell);

		this.editor = editor;

		this.neType = editor.getCas().getTypeSystem()
				.getType(ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_TYPE, "null"));

		this.neId = neType.getFeatureByBaseName(ApplicationUtil.getFromPreferences(
				ApplicationPreferences.COREFVIEW_NE_ID, "null"));

		// TODO hardcoded types
		this.sentenceType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Sentence");
		this.posType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.POS");
		this.posTagFeature = posType.getFeatureByBaseName("POSTag");
	}

	protected Control createDialogArea(Composite parent) {

		this.parent = parent;

		// parent.getShell().setSize(new Point(500, 800));

		parent.getShell().setText("Meta-analysis of the underlying document");

		GridLayout layout = new GridLayout();
		// layout.marginBottom = 10;
		// layout.marginTop = 10;
		// layout.marginLeft = 15;
		// layout.marginRight = 15;
		parent.setLayout(layout);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		fillMappings();
		AnnotationContainerModel[] modelObejects = createModelObjects();
		createColumns(viewer);

		// make lines and header visible
		final Table table = viewer.getTable();

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);

		// input we use a datastructure that maps a sentence to all NEs in it

		// build mapping

		viewer.setInput(modelObejects);
		viewer.getTable().getColumn(0).setWidth((int) ((parent.getShell().getBounds().width + 0.0) / 2));
		viewer.getTable().pack();

		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// Menu
		final Menu menu = new Menu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// jump to
		MenuItem del = new MenuItem(menu, SWT.None);

		del.setText("Jump to Sentence");

		del.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationContainerModel firstElement = (AnnotationContainerModel) sel.getFirstElement();

				AnnotationFS anno = firstElement.getAnno();

				if (anno == null)
					return;

				editor.highlightRange(anno.getBegin(), anno.getEnd() - anno.getBegin(), true);

			}

		});

		viewer.getControl().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				lastClick = new Point(e.x, e.y);

			}

			@Override
			public void mouseDown(MouseEvent e) {

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				// get the according element
				ISelection selection = viewer.getSelection();

				AnnotationContainerModel element = (AnnotationContainerModel) ((IStructuredSelection) selection)
						.getFirstElement();

				// update the data structure - dont allow any editing for
				// sentences
				if (element.getId().startsWith("Sent")) {
					return;
				}

				ViewerCell cell = viewer.getCell(lastClick);

				int columnIndex = cell.getColumnIndex();

				TableColumn column = viewer.getTable().getColumn(columnIndex);

				String colId = (String) column.getData("coloumnID");

				// update the model

				element.addAdditionalId(colId);

				// relaod

				viewer.refresh();
				CorefPersonMetaanalysisDialog.this.parent.layout();

			}
		});

		// button

		buttonaddGroup = new Button(parent, SWT.PUSH);
		buttonaddGroup.setText("Add Group");

		buttonaddGroup.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// opens another dialog with just a window to enter a new name
				// for the group
				GroupCreationDialog dialog = new GroupCreationDialog(getShell());

				int open = dialog.open();

				// success
				if (open == 0) {

					String groupName = dialog.getGroupName();

					// now its getting difficult !! we need to update our table

					AnnotationContainerModel[] model = (AnnotationContainerModel[]) viewer.getInput();

					// update the model

					AnnotationContainerModel[] modelUpdate = new AnnotationContainerModel[model.length + 1];
					int index = 0;
					boolean inserted = false;
					for (AnnotationContainerModel m : model) {

						if (m.getId().startsWith("Sent") && !inserted) {
							modelUpdate[index] = new AnnotationContainerModel(groupName, "Group" + groupindex, "Group"
									+ groupindex);
							groupindex++;
							index++;
							inserted = true;

						}

						modelUpdate[index] = m;
						index++;
					}

					viewer.setInput(modelUpdate);
					viewer.refresh();
					CorefPersonMetaanalysisDialog.this.parent.layout();

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		buttonExportToExcel = new Button(parent, SWT.PUSH);

		buttonExportToExcel.setText("Export as TSV");

		buttonExportToExcel.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				FileDialog fd = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN);
				fd.setText("Open");
				String selected = fd.open();

				if (selected == null)
					return;

				try {
					File file = new File(selected);

					FileWriter fw = new FileWriter(file);

					// add header

					fw.append("ID\t");
					for (int i = 1; i < viewer.getTable().getColumns().length; i++) {

						TableColumn col = viewer.getTable().getColumn(i);
						fw.append(idNameMapping.get(col.getData("coloumnID"))).append("\t");
					}
					fw.append("\n");

					// add rest

					for (AnnotationContainerModel model : (AnnotationContainerModel[]) viewer.getInput()) {

						fw.append(model.getTextToDisplay().replaceAll("\\t", " "));

						if (!model.getId().startsWith("Sent") && !model.getId().startsWith("Group")) {

							fw.append("; ");

							for (String s : idSynonymMap.get(model.getId())) {
								fw.append(s + "; ");
							}
						}

						fw.append("\t");

						for (int i = 1; i < viewer.getTable().getColumns().length; i++) {

							String data = (String) viewer.getTable().getColumn(i).getData("coloumnID");

							if (model.hasIdWith(data)) {
								fw.append("X\t");
							}

							else {
								fw.append(" \t");
							}

						}
						fw.append("\n");
					}

					fw.flush();
					fw.close();

					// rename

				} catch (IOException e1) {
					e1.printStackTrace();
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

		return parent;

	}

	private AnnotationContainerModel[] createModelObjects() {

		// first all entities in a sorted manner , then all sentences

		neMapping = new ArrayList<Pair<String, Integer>>();

		HashMap<String, Integer> neMap = new HashMap<String, Integer>();

		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {

			String id = ne.getFeatureValueAsString(neId);
			if (neMap.containsKey(id)) {

				Integer integer = neMap.get(id);

				integer++;

				neMap.put(id, integer);

			}

			else {

				neMap.put(id, 1);
			}
		}

		// create pairs

		for (String id : neMap.keySet()) {

			Pair<String, Integer> p = new Pair<String, Integer>(id, neMap.get(id));

			neMapping.add(p);
		}

		// sort them

		Collections.sort(neMapping, new ModelObjectsComparator());

		List<AnnotationContainerModel> models = new ArrayList<AnnotationContainerModel>();

		// af first add the header

		AnnotationContainerModel modelHeader = new AnnotationContainerModel("null", "Header");
		// models.add(modelHeader);

		// add all sorted nes

		for (Pair<String, Integer> p : neMapping) {

			AnnotationContainerModel model = new AnnotationContainerModel(idNameMapping.get(p.getKey()), p.getKey(),
					p.getKey());

			models.add(model);

		}

		// next all sentences

		int index = 1;
		for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentenceType)) {

			List<AnnotationFS> list = sentNEMapping.get(sent);

			List<String> list2 = new ArrayList<String>();

			for (AnnotationFS a : list) {

				list2.add(a.getFeatureValueAsString(neId));
			}
			AnnotationContainerModel model = new AnnotationContainerModel(sent.getCoveredText(), "Sent" + index,
					list2.toArray(new String[0]));
			model.setAnno(sent);

			models.add(model);
		}

		return models.toArray(new AnnotationContainerModel[0]);
	}

	private void fillMappings() {
		sentNEMapping = new HashMap<AnnotationFS, List<AnnotationFS>>();
		sentIndexMapping = new HashMap<AnnotationFS, String>();
		idNameMapping = new HashMap<String, String>();

		Integer index = 1;
		for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentenceType)) {

			// get all covered NEs

			List<AnnotationFS> covered = getCovered(sent, neType);

			sentNEMapping.put(sent, covered);
			sentIndexMapping.put(sent, index.toString());
			index++;

		}

		idSynonymMap = new HashMap<String, Set<String>>();
		HashMap<String, HashSet<String>> tempMapping = new HashMap<String, HashSet<String>>();

		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {

			String id = ne.getFeatureValueAsString(neId);

			// tempmapping
			if (tempMapping.containsKey(id)) {

				HashSet<String> hashSet = tempMapping.get(id);

				hashSet.add(ne.getCoveredText());
			}

			else {

				HashSet<String> name = new HashSet<String>();
				name.add(ne.getCoveredText());
				tempMapping.put(id, name);
			}

			// idsyn mapping

			if (idSynonymMap.containsKey(id)) {
				Set<String> set = idSynonymMap.get(id);
				set.add(ne.getCoveredText());
			}

			else {

				Set<String> set = new HashSet<String>();
				set.add(ne.getCoveredText());

				idSynonymMap.put(id, set);

			}

		}

		determineNamesForID(tempMapping);

	}

	private void determineNamesForID(HashMap<String, HashSet<String>> tempMapping) {

		for (String id : tempMapping.keySet()) {

			HashSet<String> hashSet = tempMapping.get(id);

			String longest = "";

			for (String s : hashSet) {

				if (s.length() > longest.length() && Character.isUpperCase(s.charAt(0))) {
					longest = s;
				}
			}

			// check if not null

			if (longest.isEmpty()) {

				for (String s : hashSet) {

					if (s.toLowerCase().equals("ich")) {
						longest = "ich";
						break;
					}
					if (s.toLowerCase().equals("wir")) {
						longest = "wir";
						break;
					}
				}
			}

			// if still empty just use the longest

			if (longest.isEmpty()) {
				for (String s : hashSet) {

					if (s.length() > longest.length()) {
						longest = s;
					}
				}

			}

			idNameMapping.put(id, longest);

		}
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colSent = createTableViewerColumn("ID", 100, 0);

		colSent.setLabelProvider(new StyledCellLabelProvider() {
			public void update(ViewerCell cell) {

				AnnotationContainerModel element = (AnnotationContainerModel) cell.getElement();

				cell.setText(element.getTextToDisplay());

				if (element.getAnno() != null && element.getAnno().getType().equals(sentenceType)) {

					List<AnnotationFS> list = sentNEMapping.get(element.getAnno());
					StyleRange[] range = new StyleRange[list.size()];
					int index = 0;
					for (AnnotationFS a : list) {
						StyleRange myStyledRange = new StyleRange(a.getBegin() - element.getAnno().getBegin(), a
								.getEnd() - a.getBegin(), null, Display.getCurrent().getSystemColor(SWT.COLOR_YELLOW));

						range[index] = myStyledRange;
						index++;
					}

					cell.setStyleRanges(range);
				}

				super.update(cell);
			}

			@Override
			public String getToolTipText(Object element) {
				AnnotationContainerModel model = (AnnotationContainerModel) element;

				String id = model.getId();

				if (id.startsWith("Sent") || id.startsWith("Group")) {

					return model.getTextToDisplay();
				}

				else {

					Set<String> set = idSynonymMap.get(id);

					String syn = "Synonyme ";

					for (String ss : set) {
						syn += ss + " ; ";
					}

					return syn;
				}

			}

			@Override
			public Point getToolTipShift(Object object) {
				return new Point(5, 5);
			}

			@Override
			public int getToolTipDisplayDelayTime(Object object) {
				return 100; // msec
			}

			@Override
			public int getToolTipTimeDisplayed(Object object) {
				return 5000; // msec
			}
		});

		int nr = 1;
		for (Pair<String, Integer> pair : neMapping) {

			String s = pair.getKey();

			final TableViewerColumn colName = createTableViewerColumn(idNameMapping.get(s).substring(0, 1), 30, nr);
			nr++;
			colName.getColumn().setData("coloumnID", s);

			colName.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {

					String id = (String) colName.getColumn().getData("coloumnID");
					AnnotationContainerModel p = (AnnotationContainerModel) element;

					if (p.getId().equals("Header")) {

						return null;
					}

					if (p.getIdsWithX().contains(id)) {
						return "X";
					}

					return "";

				}

				@Override
				public String getToolTipText(Object element) {

					String id = (String) colName.getColumn().getData("coloumnID");

					return idNameMapping.get(id);

				}

				@Override
				public Point getToolTipShift(Object object) {
					return new Point(5, 5);
				}

				@Override
				public int getToolTipDisplayDelayTime(Object object) {
					return 100; // msec
				}

				@Override
				public int getToolTipTimeDisplayed(Object object) {
					return 5000; // msec
				}

				// @Override
				// public Image getImage(Object element) {
				//
				// String id = (String)
				// colName.getColumn().getData("coloumnID");
				// AnnotationContainerModel p = (AnnotationContainerModel)
				// element;
				//
				// // just for the header
				// // if (!p.getId().equals("Header")) {
				// //
				// // return null;
				// // }
				//
				// return Display.getCurrent().getSystemImage(3);
				// }

			});
		}
	}

	private List<String> getAmountOfDistinctPersons() {

		HashSet<String> uniques = new HashSet<String>();
		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {

			uniques.add(ne.getFeatureValueAsString(neId));
		}

		List<String> list = new ArrayList<String>();
		list.addAll(uniques);
		return list;
	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();

		int headerHeight = viewer.getTable().getHeaderHeight();

		// column.setImage(parent.getDisplay().getSystemImage(SWT.ICON_WARNING));
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		column.pack();
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

	public List<AnnotationFS> getCovered(AnnotationFS cover, Type t) {

		List<AnnotationFS> covered = new ArrayList<AnnotationFS>();

		int beg = (cover.getBegin() - 1) > 0 ? cover.getBegin() : cover.getBegin();

		int end = (cover.getEnd() + 1 > editor.getCas().getDocumentText().length()) ? cover.getEnd() : cover.getEnd();

		AnnotationFS dummyAnno = editor.getCas().createAnnotation(t, beg, end);

		FSIterator<AnnotationFS> subiterator = editor.getCas().getAnnotationIndex(t).subiterator(dummyAnno);

		while (subiterator.hasNext()) {
			covered.add(subiterator.next());
		}

		return covered;

	}
}
