package de.uniwue.mk.kall.athen.corefGlobalview.structs;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;

public class CorefGlobalStruct {

	/**
	 * Standard POJO class which is used in the coreference global view each of
	 * this objects represents everything we know about a person
	 */

	// personrelated
	private String trueName;
	private Integer nrTrueName;
	private Integer nrAppelatives;
	private Integer nrPron;
	private String systemName;
	private Set<String> synonyms;
	private List<AnnotationFS> annos;
	private List<AnnotationFS> cores;
	private List<AnnotationFS> appellatives;
	private List<AnnotationFS> pronouns;
	private String gender;
	private String numerus;
	private String age;

	// relations
	private List<CorefGlobalRelation> relations;

	public CorefGlobalStruct(String trueName, Integer nrTrueName, Integer nrAppelatives, Integer nrPron,
			String systemName, Set<String> synonyms, List<AnnotationFS> annos, List<AnnotationFS> cores,
			List<AnnotationFS> appelatives, List<AnnotationFS> pron, String gender, String numerus, String age,
			List<CorefGlobalRelation> relations) {
		super();
		this.trueName = trueName;
		this.nrTrueName = nrTrueName;
		this.nrAppelatives = nrAppelatives;
		this.nrPron = nrPron;
		this.systemName = systemName;
		this.synonyms = synonyms;
		this.annos = annos;
		this.gender = gender;
		this.age = age;
		this.relations = relations;
		this.numerus = numerus;
		this.pronouns = pron;
		this.cores = cores;
		this.appellatives = appelatives;
	}

	public String getNumerus() {
		return numerus;
	}

	public void setNumerus(String numerus) {
		this.numerus = numerus;
	}

	public String getTrueName() {
		return trueName;
	}

	public void setTrueName(String trueName) {
		this.trueName = trueName;
	}

	public Integer getNrTrueName() {
		return nrTrueName;
	}

	public void setNrTrueName(Integer nrTrueName) {
		this.nrTrueName = nrTrueName;
	}

	public Integer getNrAppelatives() {
		return nrAppelatives;
	}

	public void setNrAppelatives(Integer nrAppelatives) {
		this.nrAppelatives = nrAppelatives;
	}

	public Integer getNrPron() {
		return nrPron;
	}

	public void setNrPron(Integer nrPron) {
		this.nrPron = nrPron;
	}

	public String getSystemName() {
		return systemName;
	}

	public void setSystemName(String systemName) {
		this.systemName = systemName;
	}

	public Set<String> getSynonyms() {
		return synonyms;
	}

	public void setSynonyms(Set<String> synonyms) {
		this.synonyms = synonyms;
	}

	public List<AnnotationFS> getAnnos() {
		return annos;
	}

	public void setAnnos(List<AnnotationFS> annos) {
		this.annos = annos;
	}

	public String getGender() {
		return gender;
	}

	public void setGender(String gender) {
		this.gender = gender;
	}

	public String getAge() {
		return age;
	}

	public void setAge(String age) {
		this.age = age;
	}

	public List<CorefGlobalRelation> getRelations() {
		return relations;
	}

	public void setRelations(List<CorefGlobalRelation> relations) {
		this.relations = relations;
	}

	public String getSynonyme() {

		StringBuilder sb = new StringBuilder();
		for (String s : this.synonyms) {
			sb.append(s).append(",");
		}
		return sb.toString();
	}

	public String getAppelative() {
		// TODO Auto-generated method stub
		return "Appelative";
	}

	public List<AnnotationFS> getCores() {
		return cores;
	}

	public void setCores(List<AnnotationFS> cores) {
		this.cores = cores;
	}

	public List<AnnotationFS> getAppellatives() {
		return appellatives;
	}

	public void setAppellatives(List<AnnotationFS> appellatives) {
		this.appellatives = appellatives;
	}

	public List<AnnotationFS> getPronouns() {
		return pronouns;
	}

	public void setPronouns(List<AnnotationFS> pronouns) {
		this.pronouns = pronouns;
	}

	public List<AnnotationFS> getRelationsToPerson(String idFrom, String idTo, Feature idFeature) {
		List<AnnotationFS> rels = new ArrayList<AnnotationFS>();

		// we add all relations that have AgensId = idFrom and Agens2Id = idTo

		for (CorefGlobalRelation rel : this.relations) {

			String agensId = rel.getAgens().getFeatureValueAsString(idFeature);
			String agens2Id = rel.getRecipient().getFeatureValueAsString(idFeature);

			if (idFrom.equals(agensId) && idTo.equals(agens2Id)) {
				rels.add(rel.getRelation());
			}
		}

		return rels;
	}

}
