package de.uniwue.mk.kall.athen.corefGlobalview.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.corefGlobalview.structs.CorefGlobalRelation;
import de.uniwue.mk.kall.athen.corefGlobalview.structs.CorefGlobalStruct;
import de.uniwue.mk.kall.athen.corefGlobalview.structs.ERelationCertainty;
import de.uniwue.mk.kall.athen.corefGlobalview.widgets.CorefGlobalPersonShell;
import de.uniwue.mk.kall.athen.corefGlobalview.widgets.CorefGlobalRelationShell;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.NamedEntityDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.RelationDanielDrawingStrategy;

public class CorefGlobalView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<CorefGlobalView> CLASS_THIS = CorefGlobalView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.18";

	// DATA CONTAINER - the model, since its usually very simple we have it here
	private List<CorefGlobalStruct> modelStructs;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	// ====UIMA TYPINGS=====
	// if you want to display if a user has already annotated use this separate
	// annotype
	private Type typOverride;

	// TEMP
	private Type typeTemp1;

	// all Features (UIMA) this view needs to know about
	// NEType
	private Type typeNE;
	private Feature featNumerus;
	private Feature featGender;
	private Feature featNeType;
	private Feature featName;
	private Feature featId;

	// Relation
	private Type typeRelation;
	private Feature featAgens;
	private Feature featAgens2;
	private Feature featRelType;

	// compounds - UI

	@Inject
	public CorefGlobalView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

	}

	@Override
	public void initModel() {

		modelStructs = new ArrayList<CorefGlobalStruct>();

		HashMap<String, List<AnnotationFS>> personMap = initPersonMap(editor);
		HashMap<String, List<AnnotationFS>> personRelationsMap = initPersonRelationsMap(editor);

		// for each person we generate 1 struct

		for (String id : personMap.keySet()) {

			List<AnnotationFS> appearances = personMap.get(id);

			List<AnnotationFS> coreNames = findCoreNames(appearances);
			List<AnnotationFS> appNames = findAppelativesNames(appearances);
			List<AnnotationFS> pronouns = findPronouns(appearances);
			String systemName = appearances.get(0).getFeatureValueAsString(this.featName);
			String gender = appearances.get(0).getFeatureValueAsString(this.featGender);
			String numerus = appearances.get(0).getFeatureValueAsString(this.featNumerus);

			String age = "E";

			// synonyms
			Set<String> synonyms = new HashSet<String>();

			for (AnnotationFS app : appNames) {
				synonyms.add(app.getCoveredText());
			}

			for (AnnotationFS name : coreNames) {
				synonyms.add(name.getCoveredText());
			}

			// relations - here we need to create the wrapper objects

			List<CorefGlobalRelation> relations = new ArrayList<CorefGlobalRelation>();
			List<AnnotationFS> list = personRelationsMap.get(id);

			if (list != null) {
				for (AnnotationFS rel : list) {

					AnnotationFS agens = (AnnotationFS) rel.getFeatureValue(featAgens);
					AnnotationFS agens2 = (AnnotationFS) rel.getFeatureValue(featAgens2);

					String type = rel.getFeatureValueAsString(this.featRelType);

					boolean isAttribute = false;
					if (agens.getBegin() == agens2.getBegin())
						isAttribute = true;

					CorefGlobalRelation relStruct = new CorefGlobalRelation(rel, agens, agens2, type, 0, 10,
							ERelationCertainty.PROBABLY, isAttribute, false);

					relations.add(relStruct);
				}
			}

			CorefGlobalStruct struct = new CorefGlobalStruct("", coreNames.size(), appNames.size(), pronouns.size(),
					systemName, synonyms, appearances, coreNames, appNames, pronouns, gender, numerus, age, relations);

			modelStructs.add(struct);
		}

	}

	// maps personId to all relationannos in which this person appears
	private HashMap<String, List<AnnotationFS>> initPersonRelationsMap(AnnotationEditorWidget editor) {

		HashMap<String, List<AnnotationFS>> personRelationMap = new HashMap<String, List<AnnotationFS>>();

		for (AnnotationFS rel : editor.getCas().getAnnotationIndex(this.typeRelation)) {

			AnnotationFS agens = (AnnotationFS) rel.getFeatureValue(featAgens);
			AnnotationFS agens2 = (AnnotationFS) rel.getFeatureValue(featAgens2);

			String idAgens = agens.getFeatureValueAsString(this.featId);
			String idAgens2 = agens2.getFeatureValueAsString(this.featId);

			// add to map

			addToPersonRelMap(personRelationMap, rel, idAgens2);
			addToPersonRelMap(personRelationMap, rel, idAgens);
		}

		return personRelationMap;
	}

	private void addToPersonRelMap(HashMap<String, List<AnnotationFS>> personRelationMap, AnnotationFS rel,
			String idAgens2) {
		if (personRelationMap.containsKey(idAgens2)) {

			personRelationMap.get(idAgens2).add(rel);
		}

		else {
			List<AnnotationFS> relList = new ArrayList<AnnotationFS>();
			relList.add(rel);

			personRelationMap.put(idAgens2, relList);

		}
	}

	private List<AnnotationFS> findPronouns(List<AnnotationFS> appearances) {

		List<AnnotationFS> annos = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : appearances) {

			String type = a.getFeatureValueAsString(this.featNeType);

			if (type.toLowerCase().equals("pron")) {
				annos.add(a);
			}
		}
		return annos;
	}

	private List<AnnotationFS> findAppelativesNames(List<AnnotationFS> appearances) {
		List<AnnotationFS> annos = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : appearances) {

			String type = a.getFeatureValueAsString(this.featNeType);

			if (type.toLowerCase().startsWith("app")) {
				annos.add(a);
			}
		}
		return annos;
	}

	private List<AnnotationFS> findCoreNames(List<AnnotationFS> appearances) {
		List<AnnotationFS> annos = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : appearances) {

			String type = a.getFeatureValueAsString(this.featNeType);

			if (type.toLowerCase().equals("core")) {
				annos.add(a);
			}
		}
		return annos;
	}

	// return a map with personid- list of annotations
	private HashMap<String, List<AnnotationFS>> initPersonMap(AnnotationEditorWidget editor) {

		HashMap<String, List<AnnotationFS>> personMap = new HashMap<String, List<AnnotationFS>>();

		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(this.typeNE)) {

			String id = ne.getFeatureValueAsString(featId);

			// add to map
			if (personMap.containsKey(id)) {

				personMap.get(id).add(ne);
			}

			else {
				List<AnnotationFS> neList = new ArrayList<AnnotationFS>();
				neList.add(ne);

				personMap.put(id, neList);

			}

		}

		return personMap;
	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		// init types

		// NE Type + Features
		this.typeNE = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.NamedEntity");
		this.featGender = this.typeNE.getFeatureByBaseName("Gender");
		this.featName = this.typeNE.getFeatureByBaseName("Name");
		this.featNeType = this.typeNE.getFeatureByBaseName("NEType");
		this.featNumerus = this.typeNE.getFeatureByBaseName("Numerus");
		this.featId = this.typeNE.getFeatureByBaseName("ID");

		// Relationtype + features
		this.typeRelation = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Relation");
		this.featAgens = this.typeRelation.getFeatureByBaseName("Agens");
		this.featAgens2 = this.typeRelation.getFeatureByBaseName("Agens2");
		this.featRelType = this.typeRelation.getFeatureByBaseName("Type");

		// Override Type
		this.typOverride = (editor.getCas().getTypeSystem()
				.getType("de.uniwue.kallimachos.coref.type.CorefGlobalOverride"));

		// Temp
		this.typeTemp1 = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp1");

		// Visibility
		editor.changeVisibility(typeNE, true);
		editor.changeVisibility(typeRelation, true);
		editor.changeVisibility(typOverride, true);
		editor.changeVisibility(typeTemp1, true);

		// layer
		editor.changeLayer(typOverride, 1);
		editor.changeLayer(typeTemp1, 1);

		// override styling
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle styleOverride = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		editor.changeStyle(typOverride, styleOverride);

		// TEMP
		StyleRange rangeTemp1 = new StyleRange();
		rangeTemp1.background = new Color(overlay.getDisplay(), 247, 165, 12);
		AnnoStyle styleTemp1 = new AnnoStyle(new BackgroundDrawingStrategy(rangeTemp1), rangeTemp1);

		editor.changeStyle(typeTemp1, styleTemp1);

		// style - NE type
		StyleRange rangeTyp = new StyleRange();
		rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle styleNE = new AnnoStyle(new NamedEntityDrawingStrategy(rangeTyp, featId.getShortName(),
				featNeType.getShortName(), null, featNumerus.getShortName(), featGender.getShortName(), null), rangeTyp);

		editor.changeStyle(typeNE, styleNE);

		// Style Relation
		StyleRange rangeTypRelation = new StyleRange();
		rangeTypRelation.background = new Color(overlay.getDisplay(), 0, 0, 255);
		AnnoStyle styleRelation = new AnnoStyle(new RelationDanielDrawingStrategy(rangeTypRelation,
				this.featAgens.getShortName(), this.featAgens2.getShortName()), rangeTypRelation);

		editor.changeStyle(typeRelation, styleRelation);

		// also change the modus
		editor.setActiveMode(typeNE);

		return true;

	}

	// init your Layout in here
	protected void initLayout(Composite parent) {

		Composite compUpper = new Composite(parent, SWT.BORDER);
		compUpper.setLayout(new GridLayout(2, false));

		final Button buttonShowCoreferences = new Button(compUpper, SWT.PUSH);
		buttonShowCoreferences.setText("Show Coreferences");
		buttonShowCoreferences.setData("VIEW", this);
		buttonShowCoreferences.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// open a new shell with that table
				CorefGlobalPersonShell shell = new CorefGlobalPersonShell(broker,
						(CorefGlobalView) buttonShowCoreferences.getData("VIEW"));
			}

		});

		final Button buttonShowRelations = new Button(compUpper, SWT.PUSH);
		buttonShowRelations.setText("Show Relations");
		buttonShowRelations.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// open a new shell with that table
				CorefGlobalRelationShell shell = new CorefGlobalRelationShell(broker,
						(CorefGlobalView) buttonShowCoreferences.getData("VIEW"));
			}

		});

		final Button buttonJumpToNext = new Button(compUpper, SWT.PUSH);

		buttonJumpToNext.setText("Next");
		buttonJumpToNext.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// jump to the next temptype starting the search at the current
				// cursor location
				AnnotationFS next = getNextToHighLight();

				if (next != null) {
					editor.highlightRange(next.getBegin(), next.getEnd() - next.getBegin(), true);
				}
			}

		});

		final Button buttonJumpToPrevious = new Button(compUpper, SWT.PUSH);

		buttonJumpToPrevious.setText("Previous");
		buttonJumpToPrevious.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// jump to the next temptype starting the search at the current
				// cursor location
				AnnotationFS previous = getPreviousToHighLight();

				if (previous != null) {
					editor.highlightRange(previous.getBegin(), previous.getEnd() - previous.getBegin(), true);
				}
			}

		});

	}

	private AnnotationFS getPreviousToHighLight() {
		int caretOffset = editor.getWidget().getCaretOffset();
		AnnotationFS old = null;
		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTemp1)) {

			if (temp.getEnd() < caretOffset) {
				old = temp;
			}

			if (temp.getBegin() >= caretOffset && old != null) {

				return old;
			}

		}
		return null;
	}

	private AnnotationFS getNextToHighLight() {

		int caretOffset = editor.getWidget().getCaretOffset();
		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTemp1)) {

			if (temp.getBegin() > caretOffset) {

				return temp;
			}

		}
		return null;
	}

	// use this method to update the view
	public void renewPage() throws CASException {
		updateModel();

		// redraw in a new Thread, executed by the Display
		Runnable r = new Runnable() {

			@Override
			public void run() {
				refresh();

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void updateModel() {

		// TODO for now its just init model
		initModel();
	}

	private void refresh() {
		// TODO
	}

	@PreDestroy
	public void preDestroy() {
		// unused
	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

		if (this.editor == null) {

			// request the editor if possible
			broker.send(IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR, this);
			super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr
	}

	// Perspective listener
	@Override
	public void coupleToPerspective() {

		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		super.coupleToPerspective();
	}

	@Override
	public void decoupleFromPerspective() {
		super.standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);
		super.decoupleFromPerspective();
	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications - yep this is important
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {

		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	// what to do when the editor has an added annotation?
	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an removed annotation?
	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation?
	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		AnnotationFS changed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation selection ( this is
	// an
	// arrayList<AnnotationFS> )
	@Override
	public void onAnnotationSelectionChanged(org.osgi.service.event.Event arg0) {
		// unused hjh

	}

	// what to do when the editor was double clicked
	@Override
	public void onEditorDoubleClicked(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was scrolled
	@Override
	public void onEditorWasScrolled(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor caret was changed
	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was clicked
	@Override
	public void onEditorMouseUp(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor is clicked
	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {
		// unused for this view

	}

	// what to do when the editor has a changed textselection
	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

	}

	// on editor mouse moved - use this if you want some fancy effects
	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {
		// TODO Auto-generated method stub

	}

	public AnnotationEditorWidget getEditor() {
		return this.editor;
	}

	public CorefGlobalStruct[] getModel() {

		updateModel();
		return this.modelStructs.toArray(new CorefGlobalStruct[0]);
	}

	public Feature getGenderFeature() {
		return this.featGender;
	}

	public Feature getNumerusFeature() {
		return this.featNumerus;
	}

	public void showPersons(List<AnnotationFS> annos) {

		if (annos.size() == 0)
			return;
		deleteTemp(typeTemp1);

		// create new ones

		addTemp(typeTemp1, annos);

	}

	private void addTemp(Type typeTemp, List<AnnotationFS> annos) {

		List<AnnotationFS> temps = new ArrayList<AnnotationFS>();
		for (AnnotationFS temp : annos) {
			AnnotationFS fsTemp = editor.getCas().createAnnotation(typeTemp, temp.getBegin(), temp.getEnd());
			temps.add(fsTemp);
			editor.getCas().addFsToIndexes(fsTemp);
		}

		// only invoke redraw event once
		editor.addAnnotation(temps.get(0));
		editor.highlightRange(annos.get(0).getBegin(), annos.get(0).getEnd() - annos.get(0).getBegin(), true);

	}

	private void deleteTemp(Type typeTemp) {

		List<AnnotationFS> tempToDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTemp)) {
			tempToDelete.add(temp);
		}

		for (AnnotationFS temp : tempToDelete) {
			editor.getCas().removeFsFromIndexes(temp);
		}

		// only invoke redraw event once
		if (tempToDelete.size() > 0) {
			editor.removeAnnotation(tempToDelete.get(tempToDelete.size() - 1));
		}

	}

	public Feature getNameFeature() {
		return this.featName;
	}

	public Feature getIDFeature() {
		return this.featId;
	}

	public Feature getRelationTypeFeature() {
		return this.featRelType;
	}

}