package de.uniwue.mk.kall.athen.corefGlobalview.widgets;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.corefGlobalview.structs.CorefGlobalStruct;
import de.uniwue.mk.kall.athen.corefGlobalview.ui.CorefGlobalView;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class CorefGlobalPersonShell {

	private static final String SELECTED_ID = "SELECTED_ID";
	// UI
	private TableViewer viewer;
	private TableViewer viewerPerson;

	// sorter for the table
	private ColViewerComparatorCorefGlobalShell colViewerComp;

	IEventBroker broker;

	CorefGlobalView view;
	AnnotationEditorWidget editor;

	private Shell shell;

	private ToolItem itemMerge;

	private ToolItem itemInspect;

	private ToolItem itemShow;
	private ToolItem itemShowCore;
	private ToolItem itemShowApp;
	private ToolItem itemShowPron;

	public CorefGlobalPersonShell(IEventBroker broker, CorefGlobalView view) {

		this.broker = broker;
		this.view = view;
		this.editor = view.getEditor();

		initUI();
	}

	private void initUI() {

		shell = new Shell(Display.getCurrent());

		shell.setText("Personeditingshell");
		shell.setLayout(new GridLayout(1, true));

		shell.setSize(900, 600);

		initToolbar();
		initTableViewer();
		// shell.pack();
		shell.open();

	}

	private void initToolbar() {
		ToolBar toolBar = new ToolBar(shell, SWT.FLAT | SWT.WRAP | SWT.RIGHT);

		itemMerge = new ToolItem(toolBar, SWT.PUSH);
		itemMerge.setText("Zusammenfhren");
		itemMerge.setEnabled(false);
		Image iconMerge = ApplicationUtil.createImage(ApplicationUtil.COREF_CLUSTER_MERGE);
		Image iconFind = ApplicationUtil.createImage(ApplicationUtil.COREF_PERSON_FIND);
		Image iconInspect = ApplicationUtil.createImage(ApplicationUtil.COREF_PERSON_INSPECT);

		itemMerge.setImage(iconMerge);

		itemMerge.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// get all items that are checked
				TableItem[] items = viewer.getTable().getItems();

				List<CorefGlobalStruct> selectedStructs = new ArrayList<CorefGlobalStruct>();
				for (TableItem item : items) {

					if (item.getChecked()) {
						selectedStructs.add((CorefGlobalStruct) item.getData());
					}

				}

				// add all structs to the id of the first and reload

				String idNew = selectedStructs.get(0).getAnnos().get(0).getFeatureValueAsString(view.getIDFeature());

				for (int i = 1; i < selectedStructs.size(); i++) {

					for (AnnotationFS anno : selectedStructs.get(i).getAnnos()) {

						anno.setFeatureValueFromString(view.getIDFeature(), idNew);
					}
				}

				editor.changeAnnotation(selectedStructs.get(0).getAnnos().get(0));
				refreshViewer();
				shell.layout();

			}
		});

		itemInspect = new ToolItem(toolBar, SWT.PUSH);
		itemInspect.setText("Cluster bearbeiten");
		itemInspect.setImage(iconInspect);
		itemInspect.setEnabled(false);

		itemInspect.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (itemInspect.getText().equals("Cluster bearbeiten") && viewer.getSelection() != null) {

					initViewerPerson();
					itemInspect.setText("Cluster fertig bearbeitet");
				}

				else {
					initTableViewer();
					itemInspect.setText("Cluster bearbeiten");

				}
				shell.layout();

			}

		});

		itemShow = new ToolItem(toolBar, SWT.PUSH);
		itemShow.setText("Zeige alle Personen");
		itemShow.setImage(iconFind);
		itemShow.setEnabled(false);
		itemShow.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) sel.getFirstElement();
				if (struct == null)
					return;
				view.showPersons(struct.getAnnos());

			}

		});

		itemShowCore = new ToolItem(toolBar, SWT.PUSH);
		itemShowCore.setText("Zeige Core Namen");
		itemShowCore.setImage(iconFind);
		itemShowCore.setEnabled(false);
		itemShowCore.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) sel.getFirstElement();
				if (struct == null)
					return;
				view.showPersons(struct.getCores());

			}

		});

		itemShowApp = new ToolItem(toolBar, SWT.PUSH);
		itemShowApp.setText("Zeige Appellative");
		itemShowApp.setImage(iconFind);
		itemShowApp.setEnabled(false);
		itemShowApp.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) sel.getFirstElement();
				if (struct == null)
					return;
				view.showPersons(struct.getAppellatives());

			}

		});

		itemShowPron = new ToolItem(toolBar, SWT.PUSH);
		itemShowPron.setText("Zeige Pronomen");
		itemShowPron.setImage(iconFind);
		itemShowPron.setEnabled(false);
		itemShowPron.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) sel.getFirstElement();
				if (struct == null)
					return;
				view.showPersons(struct.getPronouns());

			}

		});

	}

	private void initTableViewer() {

		for (Control c : shell.getChildren()) {

			if (c instanceof Table)
				c.dispose();
		}
		viewer = new TableViewer(shell, SWT.CHECK | SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION
				| SWT.BORDER | SWT.FILL);

		createColumns(viewer);

		viewer.getTable().setLayoutData(
				new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());

		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				updateToolbarButtons();

			}

		});

		colViewerComp = new ColViewerComparatorCorefGlobalShell(this.view);
		viewer.setComparator(colViewerComp);
		viewer.getTable().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));

				int columnIndex = cell.getColumnIndex();
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) selection.getFirstElement();

				// Gender
				if (columnIndex == 7) {

					String gender = struct.getGender();

					if (gender == null)
						gender = "m";
					String genderNew = "";
					if (gender.equals("m")) {
						genderNew = "f";
					} else if (gender.equals("f")) {
						genderNew.equals("n");
					}

					else {
						genderNew = "m";
					}

					// update that feature in all NEs in that cluster
					List<AnnotationFS> toChange = new ArrayList<AnnotationFS>();
					for (AnnotationFS ne : struct.getAnnos()) {
						ne.setFeatureValueFromString(view.getGenderFeature(), genderNew);
						toChange.add(ne);
					}
					editor.changeAnnotations(toChange);
				}

				// Numerus
				else if (columnIndex == 8) {

					String numerus = struct.getNumerus();
					if (numerus == null)
						numerus = "pl";
					String numerusNew = "";

					if (numerus.equals("pl")) {
						numerusNew = "si";
					}

					else {
						numerusNew = "pl";
					}

					List<AnnotationFS> toChange = new ArrayList<AnnotationFS>();
					for (AnnotationFS ne : struct.getAnnos()) {
						ne.setFeatureValueFromString(view.getNumerusFeature(), numerusNew);
						toChange.add(ne);
					}
					editor.changeAnnotations(toChange);
				}

				// Alter TODO

				refreshViewer();
				shell.layout();
			}

		});

		viewer.setInput(view.getModel());

	}

	private void initViewerPerson() {

		IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
		CorefGlobalStruct struct = (CorefGlobalStruct) sel.getFirstElement();

		for (Control c : shell.getChildren()) {

			if (c instanceof Table)
				c.dispose();
		}

		// Composite with second table
		viewerPerson = new TableViewer(shell, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		viewerPerson.getTable().setLayoutData(
				new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		// make lines and header visible
		final Table tablePerson = viewerPerson.getTable();
		tablePerson.setHeaderVisible(true);
		tablePerson.setLinesVisible(true);

		viewerPerson.setContentProvider(new ArrayContentProvider());

		// columns
		createColumnsPersonTable(viewerPerson);

		// listener

		viewerPerson.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				// highlight in the editor
				IStructuredSelection sel = (IStructuredSelection) viewerPerson.getSelection();
				AnnotationFS anno = (AnnotationFS) sel.getFirstElement();
				if (anno != null) {
					view.getEditor().highlightRange(anno.getBegin(), anno.getEnd() - anno.getBegin(), true);
				}

			}
		});

		viewerPerson.getTable().addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewerPerson.getSelection();
				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String id = firstElement.getFeatureValueAsString(view.getIDFeature());
				if (e.keyCode == SWT.ARROW_LEFT) {

					if (id.startsWith("x")) {
						id = id.substring(1);
						firstElement.setFeatureValueFromString(view.getIDFeature(), id);
						view.getEditor().changeAnnotation(firstElement);

						// update the shell
						viewerPerson.refresh();

					}
				}

				else if (e.keyCode == SWT.ARROW_RIGHT) {
					if (id.startsWith("x") && id.equals(viewerPerson.getData(SELECTED_ID)) || !id.startsWith("x")) {
						id = "x" + id;
						firstElement.setFeatureValueFromString(view.getIDFeature(), id);
						view.getEditor().changeAnnotation(firstElement);

						// update the shell
						viewerPerson.refresh();

					}
				}

			}

			@Override
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub

			}
		});

		viewerPerson.setData(SELECTED_ID, struct.getAnnos().get(0).getFeatureValueAsString(view.getIDFeature()));
		viewerPerson.setInput(struct.getAnnos().toArray(new AnnotationFS[0]));
		viewerPerson.refresh();

		itemShow.setEnabled(false);
		itemShowApp.setEnabled(false);
		itemShowCore.setEnabled(false);
		itemShowPron.setEnabled(false);
	}

	private void createColumnsPersonTable(final TableViewer viewerPerson) {

		TableViewerColumn colName = createTableViewerColumn("Text", 100, 0, viewerPerson);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS struct = (AnnotationFS) element;
				return struct.getCoveredText();

			}

		});

		TableViewerColumn colInCluster = createTableViewerColumn("Im Cluster?", 100, 0, viewerPerson);
		colInCluster.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS struct = (AnnotationFS) element;

				String id = struct.getFeatureValueAsString(view.getIDFeature());

				if (id.equals(viewerPerson.getData(SELECTED_ID))) {
					return "X";
				}

				else {
					return "";
				}

			}

		});

		TableViewerColumn colOutCluster = createTableViewerColumn("Nicht im Cluster?", 100, 0, viewerPerson);
		colOutCluster.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS struct = (AnnotationFS) element;

				String id = struct.getFeatureValueAsString(view.getIDFeature());

				if (!id.equals(viewerPerson.getData(SELECTED_ID))) {
					return "X";
				}

				else {
					return "";
				}

			}

		});

	}

	private void refreshViewer() {
		CorefGlobalStruct[] model = view.getModel();

		viewer.setInput(model);
		viewer.refresh();

	}

	private void updateToolbarButtons() {

		TableItem[] items = viewer.getTable().getItems();
		ISelection selection = viewer.getSelection();

		int amountCheckedItems = 0;
		for (TableItem item : items) {
			if (item.getChecked()) {
				amountCheckedItems++;
			}
		}

		if (amountCheckedItems > 1) {
			this.itemMerge.setEnabled(true);
		}

		else {
			this.itemMerge.setEnabled(false);
		}

		if (selection != null) {
			this.itemInspect.setEnabled(true);
			this.itemShow.setEnabled(true);
			this.itemShowApp.setEnabled(true);
			this.itemShowCore.setEnabled(true);
			this.itemShowPron.setEnabled(true);
		}

		else {
			this.itemInspect.setEnabled(false);
			this.itemShow.setEnabled(false);
			this.itemShowApp.setEnabled(false);
			this.itemShowCore.setEnabled(false);
			this.itemShowPron.setEnabled(false);

		}

	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn(" Wahrer Name", 100, 0, viewer);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return struct.getAnnos().get(0).getFeatureValueAsString(view.getNameFeature());

			}

		});

		colName.setEditingSupport(new EditingSupport(this.viewer) {

			@Override
			protected void setValue(Object element, Object value) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;

				for (AnnotationFS a : struct.getAnnos()) {
					a.setFeatureValueFromString(view.getNameFeature(), String.valueOf(value));
				}

				refreshViewer();
				shell.layout();

			}

			@Override
			protected Object getValue(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return struct.getAnnos().get(0).getFeatureValueAsString(view.getNameFeature());
			}

			@Override
			protected CellEditor getCellEditor(Object element) {
				TableViewer viewer = (TableViewer) getViewer();
				return new TextCellEditor(viewer.getTable());
			}

			@Override
			protected boolean canEdit(Object element) {
				return true;
			}
		});

		TableViewerColumn colAmountName = createTableViewerColumn("#Name", 80, 1, viewer);
		colAmountName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrTrueName());

			}

		});

		colAmountName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrTrueName());
			}

		});

		TableViewerColumn colAmountAppel = createTableViewerColumn("#Appelative", 80, 2, viewer);
		colAmountAppel.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrAppelatives());

			}

		});

		colAmountAppel.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrAppelatives());
			}

		});

		TableViewerColumn colAmountPron = createTableViewerColumn("#Pronomen", 80, 3, viewer);
		colAmountPron.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrPron());

			}

		});

		colAmountPron.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNrPron());
			}

		});

		TableViewerColumn colSystemname = createTableViewerColumn("Systemname", 80, 4, viewer);
		colSystemname.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getSystemName());

			}

		});

		colSystemname.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getSystemName());
			}

		});

		TableViewerColumn colSynonyme = createTableViewerColumn("Synonyme", 80, 5, viewer);
		colSynonyme.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getSynonyme());

			}

		});

		colSynonyme.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getSynonyme());
			}

		});

		// TODO add another column with appleativen??
		TableViewerColumn colAppelative = createTableViewerColumn("Appleative", 80, 6, viewer);
		colAppelative.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getAppelative());

			}

		});

		colAppelative.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public void update(final ViewerCell cell) {
				Object element = cell.getElement();
				cell.setText(getText(element));
				cell.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
			}

			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getAppelative());
			}

		});

		TableViewerColumn colgender = createTableViewerColumn("Geschlecht", 80, 7, viewer);
		colgender.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getGender());

			}

		});

		TableViewerColumn colNumerus = createTableViewerColumn("Numerus", 80, 8, viewer);
		colNumerus.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getNumerus());

			}

		});

		TableViewerColumn colAlter = createTableViewerColumn("Alter", 80, 8, viewer);
		colAlter.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getAge());

			}

		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber, TableViewer viewer) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber, viewer));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index, final TableViewer viewer) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				colViewerComp.setColumn(index);
				int dir = colViewerComp.getDirection();
				viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
