package de.uniwue.mk.kall.athen.corefGlobalview.widgets;

import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.corefGlobalview.structs.CorefGlobalStruct;
import de.uniwue.mk.kall.athen.corefGlobalview.ui.CorefGlobalView;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class CorefGlobalRelationShell {

	private static final String PERSON = "PERSON";
	private static final String SELECTED_ID = "SELECTED_ID";
	// UI
	private TableViewer viewer;

	// sorter for the table

	IEventBroker broker;

	CorefGlobalView view;
	AnnotationEditorWidget editor;

	private Shell shell;

	public CorefGlobalRelationShell(IEventBroker broker, CorefGlobalView view) {

		this.broker = broker;
		this.view = view;
		this.editor = view.getEditor();

		initUI();
	}

	private void initUI() {

		shell = new Shell(Display.getCurrent());

		shell.setText("Relationeditingshell");
		shell.setLayout(new GridLayout(1, true));

		shell.setSize(900, 600);

		initTableViewer();
		// shell.pack();
		shell.open();

	}

	private void initTableViewer() {

		for (Control c : shell.getChildren()) {

			if (c instanceof Table)
				c.dispose();
		}

		viewer = new TableViewer(shell, SWT.MULTI | SWT.VIRTUAL | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION
				| SWT.BORDER | SWT.FILL);

		createColumns(viewer);

		viewer.getTable().setLayoutData(
				new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new LazyRelationContentProvider(viewer));
		viewer.setUseHashlookup(true);

		// colViewerComp = new ColViewerComparatorCorefGlobalShell(this.view);
		// viewer.setComparator(colViewerComp);
		viewer.getTable().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));

				int columnIndex = cell.getColumnIndex();
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				CorefGlobalStruct struct = (CorefGlobalStruct) selection.getFirstElement();

				// Gender
				if (columnIndex == 7) {

				}

				refreshViewer();
				shell.layout();
			}

		});

		viewer.setInput(view.getModel());
		viewer.setItemCount(view.getModel().length);

	}

	private void refreshViewer() {
		CorefGlobalStruct[] model = view.getModel();

		viewer.setInput(model);
		viewer.refresh();

	}

	private void createColumns(TableViewer viewer) {

		// col 1 is the name
		TableViewerColumn colName = createTableViewerColumn("Name von links", 100, 0, viewer);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return struct.getAnnos().get(0).getFeatureValueAsString(view.getNameFeature());

			}

		});

		// col 2 is the amount of appearances
		TableViewerColumn colAmount = createTableViewerColumn("Anzahl", 50, 0, viewer);
		colAmount.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return String.valueOf(struct.getAnnos().size());

			}

		});

		// col 3 is the gender
		TableViewerColumn colGender = createTableViewerColumn("Geschlecht", 60, 0, viewer);
		colGender.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return struct.getAnnos().get(0).getFeatureValueAsString(view.getGenderFeature());

			}

		});

		// col 4 is the age
		TableViewerColumn colAge = createTableViewerColumn("Alter", 50, 0, viewer);
		colAge.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return "E";

			}

		});

		// col 5 are attributes
		TableViewerColumn colAtts = createTableViewerColumn("Eigenschaften", 100, 0, viewer);
		colAtts.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CorefGlobalStruct struct = (CorefGlobalStruct) element;
				return "Eigenschaft";

			}

		});

		// res of the columns are dynamic
		// for each person
		for (CorefGlobalStruct person : view.getModel()) {
			final TableViewerColumn colPerson = createTableViewerColumn(person.getSystemName(), 100, 0, viewer);
			colPerson.getColumn().setData(PERSON, person);
			colPerson.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					// row
					CorefGlobalStruct struct = (CorefGlobalStruct) element;

					// col
					CorefGlobalStruct colPers = (CorefGlobalStruct) colPerson.getColumn().getData(PERSON);

					// if they are the same we print X
					if (struct.getAnnos().get(0).getBegin() == colPers.getAnnos().get(0).getBegin())
						return "X";

					// if struct has a relation to colPers we print those

					String idTo = colPers.getAnnos().get(0).getFeatureValueAsString(view.getIDFeature());
					String idFrom = struct.getAnnos().get(0).getFeatureValueAsString(view.getIDFeature());
					List<AnnotationFS> relations = struct.getRelationsToPerson(idFrom, idTo, view.getIDFeature());

					if (relations.size() > 0) {
						String rels = "";
						for (AnnotationFS rel : relations) {
							rels += rel.getFeatureValueAsString(view.getRelationTypeFeature()) + ";";
						}
						return rels;
					}

					return "";

				}

			});

		}

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber, TableViewer viewer) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber, viewer));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index, final TableViewer viewer) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
