package de.uniwue.mk.kall.athen.depparse.analyzer;

import java.io.File;
import java.util.List;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.osgi.service.event.Event;

import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.athen.widget.editor.CASEditorInput;
import de.uniwue.mk.kall.athen.widget.editor.EEditorEvent;
import de.uniwue.mk.kall.athen.widget.editor.IAnnotationEditorListener;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public abstract class AEditorSubordinateAnalyzer extends AEditorSubordinateViewPart implements
		IAnnotationEditorListener {

	// this widget is not the same as super.editor !!!
	protected AnnotationEditorWidget widget;
	protected Util_impl utilNew;
	protected Util_impl utilOld;
	protected CAS casOld;
	protected CAS casNew;
	protected Type sentenceType;
	protected AnnotationFS editorSelectedSentence;

	protected int editorSentBegin;

	public abstract void copyFromOldCASToNewCAS(CAS casOld, CAS casNew, AnnotationFS sentence);

	public abstract void initNewCasTypes();

	public abstract void synchronize();

	@Override
	public void onEditorCaretChanged(Event arg0) {

		// the new CAS
		CAS cas = Util_impl.createCas();

		// but before we need to copy the annotations we need form the old cas
		List<AnnotationFS> sentenceSelection = super.editor.getCurrentSelectionOfType(sentenceType);

		if (sentenceSelection.size() != 1)
			return;

		cas.setDocumentText(sentenceSelection.get(0).getCoveredText());
		utilNew = new Util_impl(cas);

		// util for new cas

		initNewCasTypes();
		// add the gu tokens

		// covered in old cas

		// copy them
		this.editorSentBegin = sentenceSelection.get(0).getBegin();
		this.editorSelectedSentence = sentenceSelection.get(0);

		int sentIndex = utilNew.getIndexOfTypeInDocument(sentenceSelection.get(0));
		// set the new input into the widget

		File openedFile = editor.getOpenedFile();
		String openedFileName = openedFile.getAbsolutePath().replaceAll(".xml", "").replaceAll(".xmi", "")
				.replaceAll(".txt", "");

		File newFile = new File(openedFileName + "sentNr[" + sentIndex + "].xmi");
		// set the new cas as input
		this.widget.setInput(new CASEditorInput(cas, newFile));

		// copy the old annos
		copyFromOldCASToNewCAS(editor.getCas(), cas, editorSelectedSentence);

		this.widget.getWidget().setTopMargin(
				this.overlay.getBounds().height - 2 * this.widget.getWidget().getLineHeight());
		this.widget.getWidget().setWordWrap(false);

		initAnalyzerTypings();

		renewPage();

	}

	public abstract void initAnalyzerTypings();

	@Override
	protected void initLayout(Composite parent) {

		// composite gets three coloumns

		parent.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		this.widget = new AnnotationEditorWidget(parent, SWT.NONE);
		this.widget.setNotifyDocumentChange(false);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_REMOVED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_ADDED, this);
		this.widget.addListener(EEditorEvent.EDITOR_WIDGET_MOUSEDOWN, this);
		this.widget.addListener(EEditorEvent.EDITOR_WIDGET_DOUBLECLICKED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ALT_ENTER_PRESSED, this);
		this.widget.addListener(EEditorEvent.EDITOR_KEY_PRESSED, this);
		this.widget.addListener(EEditorEvent.EDITOR_CTRL_PRESSED, this);
		// this.widget.addListener(EEditorEvent., listener);

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	@Override
	public void onAnnotationAdded(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationRemoved(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationChanged(Event arg0) {
		renewPage();

	}

	public void renewPage() {

		super.overlay.layout();

	}

}
