package de.uniwue.mk.kall.athen.depparse.analyzer;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.ArrayFS;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.ArrayFSImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.mk.kall.athen.part.editor.dialog.EditorDoubleClickDialog;
import de.uniwue.mk.kall.athen.widget.editor.EEditorEvent;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.GrammarRealDrawingStrategy;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class ConstituencyParseAnalyzer extends AEditorSubordinateAnalyzer {

	// CONSTANTS

	private static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.constituencyparseanalyzer";

	// FIELDS======
	private List<AnnotationFS> annoList;

	private static Class<?> CLASS_THIS = ConstituencyParseAnalyzer.class;

	@Inject
	private static IEventBroker broker;
	@Inject
	static EPartService partService;

	@Inject
	EModelService modelService;

	// types - i need to define those twice - once for the old cas and once for
	// the new cas!!
	private Type grammaticTypeOldCAS;

	private Feature featPhraseTypeOldCAS;

	private Feature featSemanticRoleOldCAS;

	private Feature featSyntacticRoleOldCAS;

	private Feature featIsSelectedOldCAS;
	private Feature featSyntacticHeadOldCAS;
	private Feature featLayerOldCAS;
	private Feature featParseChildrenOldCAS;
	private Feature featParseFatherOldCAS;

	// new features
	private Type grammaticTypeNewCAS;

	private Feature featPhraseTypeNewCAS;

	private Feature featSemanticRoleNewCAS;

	private Feature featSyntacticRoleNewCAS;

	private Feature featIsSelectedNewCAS;
	private Feature featSyntacticHeadNewCAS;
	private Feature featLayerNewCAS;
	private Feature featParseChildrenNewCAS;
	private Feature featParseFatherNewCAS;

	@Inject
	public ConstituencyParseAnalyzer() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);
	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {
		super.onFocus();
	}

	@Override
	public void initNewCasTypes() {

		this.grammaticTypeNewCAS = super.utilNew.getCas().getTypeSystem()
				.getType("de.uniwue.kallimachos.coref.type.GrammaticUnit");
		this.featLayerNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("Layer");
		this.featPhraseTypeNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("Phrasetype");
		this.featSemanticRoleNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("SemanticRole");
		this.featSyntacticRoleNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("SyntacticRole");
		this.featSyntacticHeadNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("SyntacticHead");
		this.featIsSelectedNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("IsSelected");

		this.featParseChildrenNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("Parsechildren");
		this.featParseFatherNewCAS = this.grammaticTypeNewCAS.getFeatureByBaseName("Parsefather");

	}

	private AnnotationFS findAccordingAnno(AnnotationFS anno, AnnotationIndex<AnnotationFS> annotationIndex,
			boolean oldToNew) {

		// text und layer sind nicht über das gesamte dokument eindeutig!!! TODO
		int begIndex = anno.getBegin();

		if (oldToNew) {
			begIndex = anno.getBegin() - editorSelectedSentence.getBegin();
		}

		else {
			begIndex = anno.getBegin() + editorSelectedSentence.getBegin();
		}
		for (AnnotationFS a : annotationIndex) {
			if (anno.getCoveredText().equals(a.getCoveredText()) && begIndex == a.getBegin()) {

				if (oldToNew) {

					if (anno.getFeatureValueAsString(featLayerOldCAS) == null
							&& a.getFeatureValueAsString(featLayerNewCAS) == null) {
						return a;

					}
					if (anno.getFeatureValueAsString(featLayerOldCAS)
							.equals(a.getFeatureValueAsString(featLayerNewCAS)))
						return a;
				}

				else {
					if (anno.getFeatureValueAsString(featLayerNewCAS) == null
							&& a.getFeatureValueAsString(featLayerOldCAS) == null) {
						return a;

					}
					if (anno.getFeatureValueAsString(featLayerNewCAS)
							.equals(a.getFeatureValueAsString(featLayerOldCAS)))
						return a;
				}
			}
		}
		System.out.println("Should nto happen");
		return null;
	}

	// called when the editor cas changes
	@Override
	public void initModel() {
		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(this.grammaticTypeOldCAS)) {
			annoList.add(anno);
		}

	}

	@Override
	public boolean initTypes() {

		utilOld = new Util_impl(super.editor.getCas());
		this.sentenceType = utilOld.getSentenceType();

		this.grammaticTypeOldCAS = utilOld.getCas().getTypeSystem()
				.getType("de.uniwue.kallimachos.coref.type.GrammaticUnit");
		this.featPhraseTypeOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("Phrasetype");
		this.featSemanticRoleOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("SemanticRole");
		this.featSyntacticRoleOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("SyntacticRole");
		this.featSyntacticHeadOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("SyntacticHead");
		this.featLayerOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("Layer");
		this.featIsSelectedOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("IsSelected");
		// Visibility
		this.featParseChildrenOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("Parsechildren");
		this.featParseFatherOldCAS = this.grammaticTypeOldCAS.getFeatureByBaseName("Parsefather");

		return true;

	}

	@Override
	public void performEditorChange(EEditorEvent eventType, Object changedObject) {

		if (eventType == EEditorEvent.EDITOR_ANNOTATION_ADDED) {

			onAnnotationAdded((AnnotationFS) changedObject, false);
		}

		if (eventType == EEditorEvent.EDITOR_ANNOTATION_REMOVED) {

			onAnnotationRemoved((AnnotationFS) changedObject);
		}
		if (eventType == EEditorEvent.EDITOR_WIDGET_MOUSEDOWN) {
			onWidgetClicked((Point) changedObject);
		}
		if (eventType == EEditorEvent.EDITOR_WIDGET_DOUBLECLICKED) {
			onWidgetDoubleClicked((Point) changedObject);
		}

		if (eventType == EEditorEvent.EDITOR_ALT_ENTER_PRESSED) {

			// create the annotation
			Point selection = widget.getWidget().getSelection();

			if (selection.y - selection.x != 0) {
				AnnotationFS fs = widget.getCas().createAnnotation(this.grammaticTypeNewCAS, selection.x, selection.y);
				widget.getCas().addFsToIndexes(fs);
				onAnnotationAdded(fs, true);
			}
		}

		if (eventType == EEditorEvent.EDITOR_KEY_PRESSED) {
			onEditorKeyPressed((Character) changedObject);
		}

		if (eventType == EEditorEvent.EDITOR_CTRL_PRESSED) {
			onEditorCTRLPressed(changedObject);
		}

		// synchronize with the real editor
		synchronize();

	}

	private void onEditorCTRLPressed(Object changedObject) {
		KeyEvent event = (KeyEvent) changedObject;
		System.out.println(event.keyCode);

		if (event.keyCode == 121) {
			// STRG + y => NP
			AnnotationFS gUNew = widget.getCas().createAnnotation(grammaticTypeNewCAS,
					widget.getWidget().getSelection().x, widget.getWidget().getSelection().y);
			gUNew.setFeatureValueFromString(featPhraseTypeNewCAS, "NP");
			widget.addAnnotation(gUNew);
		}

		else if (event.keyCode == 120) {
			// STRG +x => PP
			AnnotationFS gUNew = widget.getCas().createAnnotation(grammaticTypeNewCAS,
					widget.getWidget().getSelection().x, widget.getWidget().getSelection().y);
			gUNew.setFeatureValueFromString(featPhraseTypeNewCAS, "PP");
			widget.addAnnotation(gUNew);
		}

		else if (event.keyCode == 99) {
			// STRG +c => KONJ
			AnnotationFS gUNew = widget.getCas().createAnnotation(grammaticTypeNewCAS,
					widget.getWidget().getSelection().x, widget.getWidget().getSelection().y);
			gUNew.setFeatureValueFromString(featPhraseTypeNewCAS, "KONJ");
			widget.addAnnotation(gUNew);
		}
	}

	private void onEditorKeyPressed(Character changedObject) {

		int key = (int) changedObject;

		// DEL
		if (key == 127) {

			for (AnnotationFS guNew : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

				String isSel = guNew.getFeatureValueAsString(featIsSelectedNewCAS);

				if (isSel != null && isSel.matches("true")) {
					widget.removeAnnotation(guNew);
					break;
				}
			}
		}

		if (changedObject == 'a') {
			// previous
			highlightPreviousSelection();

		}

		if (changedObject == 'd') {
			// next
			highlightNextSelection();
		}

	}

	private void highlightNextSelection() {
		Util_impl util = new Util_impl(widget.getCas());
		AnnotationFS selected = getCurrentUserSelection();
		if (selected == null) {
			// we use the first one
			List<AnnotationFS> covered = util.getCovered(0, widget.getCas().getDocumentText().length(),
					grammaticTypeNewCAS);
			if (covered.size() == 0)
				return;
			updateUserSelection(covered.get(0));
		} else {
			AnnotationFS nextAnnotation = util.getNextAnnotation(selected);
			if (nextAnnotation != null) {
				updateUserSelection(nextAnnotation);
			}
		}
	}

	private void highlightPreviousSelection() {
		Util_impl util = new Util_impl(widget.getCas());
		AnnotationFS selected = getCurrentUserSelection();
		if (selected == null) {
			// we use the first one
			List<AnnotationFS> covered = util.getCovered(0, widget.getCas().getDocumentText().length(),
					grammaticTypeNewCAS);
			if (covered.size() == 0)
				return;
			updateUserSelection(covered.get(0));
		} else {
			AnnotationFS previousAnnotation = util.getPreviousAnnotation(selected);
			if (previousAnnotation != null) {
				updateUserSelection(previousAnnotation);
			}
		}

	}

	private void updateUserSelection(AnnotationFS anno) {

		if (anno == null)
			return;
		// deselect all others
		for (AnnotationFS a : this.annoList) {
			a.setFeatureValueFromString(featIsSelectedNewCAS, "");
		}

		// update the selected anno !!!
		anno.setFeatureValueFromString(featIsSelectedNewCAS, "true");
		widget.changeAnnotation(anno);
		widget.highlightRange(anno.getEnd(), 0, true);

	}

	private AnnotationFS getCurrentUserSelection() {

		for (AnnotationFS guNew : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			String selectedFeat = guNew.getFeatureValueAsString(featIsSelectedNewCAS);

			if (selectedFeat.startsWith("tr")) {
				return guNew;
			}
		}
		return null;
	}

	// this method synchronizes with the main editor
	@Override
	public void synchronize() {

		// delete all annotations in that sentence

		Util_impl util_old = new Util_impl(editor.getCas());
		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS gUOld : util_old.getCovered(editorSelectedSentence, grammaticTypeOldCAS)) {
			toDelete.add(gUOld);
		}

		for (AnnotationFS del : toDelete) {
			editor.getCas().removeFsFromIndexes(del);
		}

		if (toDelete.size() > 0) {
			editor.removeAnnotation(toDelete.get(0));
		}

		List<AnnotationFS> toAdd = new ArrayList<AnnotationFS>();
		for (AnnotationFS gU : this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			AnnotationFS fs = editor.getCas().createAnnotation(grammaticTypeOldCAS, gU.getBegin() + editorSentBegin,
					gU.getEnd() + editorSentBegin);

			// assign layer
			fs.setFeatureValueFromString(featLayerOldCAS, gU.getFeatureValueAsString(featLayerNewCAS));

			// phrase type
			fs.setFeatureValueFromString(featPhraseTypeOldCAS, gU.getFeatureValueAsString(featPhraseTypeNewCAS));

			// semantic role
			fs.setFeatureValueFromString(featSemanticRoleOldCAS, gU.getFeatureValueAsString(featSemanticRoleNewCAS));

			// grammtical role
			fs.setFeatureValueFromString(featSyntacticRoleOldCAS, gU.getFeatureValueAsString(featSyntacticRoleNewCAS));

			toAdd.add(fs);

		}

		// delete and add
		for (AnnotationFS a : toAdd) {
			editor.getCas().addFsToIndexes(a);
		}

		// assign parent features
		for (AnnotationFS newGU : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			AnnotationFS father = (AnnotationFS) newGU.getFeatureValue(featParseFatherNewCAS);

			if (father != null) {

				AnnotationFS oldGu = findAccordingAnno(newGU, editor.getCas().getAnnotationIndex(grammaticTypeOldCAS),
						false);

				AnnotationFS fatherOld = findAccordingAnno(father,
						editor.getCas().getAnnotationIndex(grammaticTypeOldCAS), false);

				if (oldGu != null && fatherOld != null) {
					oldGu.setFeatureValue(featParseFatherOldCAS, fatherOld);
				}

			}

		}

		// force a reload
		if (toAdd.size() > 0) {
			editor.changeAnnotation(toAdd.get(0));
		}
	}

	private void onWidgetDoubleClicked(Point changedObject) {

		AnnotationFS sel = determineMouseSelectedAnno(changedObject);

		if (sel == null)
			return;

		// open the dialog to edit the properties
		EditorDoubleClickDialog dialog = new EditorDoubleClickDialog(widget.getShell(), sel, widget);

		if (dialog.open() == SWT.OK) {
			// refresh the widget
			widget.invokeReload();
		}

		// perform a simple mouslick to change the widget selection
		widget.getWidget().notifyListeners(SWT.MouseUp, new org.eclipse.swt.widgets.Event());

	}

	private void onWidgetClicked(Point p) {

		AnnotationFS sel = determineMouseSelectedAnno(p);

		if (sel != null) {
			// deselect all others
			for (AnnotationFS a : this.annoList) {
				a.setFeatureValueFromString(featIsSelectedNewCAS, "");
			}

			// update the selected anno !!!
			sel.setFeatureValueFromString(featIsSelectedNewCAS, "true");
			widget.changeAnnotation(sel);

		}

	}

	private AnnotationFS determineMouseSelectedAnno(Point p) {

		int base = widget.getWidget().getTextBounds(0, 1).y;

		int layerHeight = 30;

		int layerNr = (base - p.y) / layerHeight + 1;

		if (layerNr < 1)
			return null;

		// search for an annotation that spans the caret and has layer = layerNR

		int offset = widget.getWidget().getCaretOffset();
		AnnotationFS sel = null;
		for (AnnotationFS a : this.annoList) {
			if (a.getBegin() <= offset && a.getEnd() >= offset) {
				String layer = a.getFeatureValueAsString(featLayerNewCAS);

				if (layer != null && layer.equals(String.valueOf(layerNr))) {

					sel = a;
					break;
				}
			}
		}
		return sel;
	}

	private void onAnnotationRemoved(AnnotationFS changedObject) {
		// update the parent
		ArrayFSImpl array = (ArrayFSImpl) changedObject.getFeatureValue(featParseChildrenNewCAS);
		AnnotationFS father = (AnnotationFS) changedObject.getFeatureValue(featParseFatherNewCAS);

		int size = array == null ? 0 : array.size();
		// if there are children we need to update their parent feature, set to
		// null if there is no parent
		if (array != null) {

			for (int i = 0; i < size; i++) {
				AnnotationFS child = (AnnotationFS) array.get(i);

				setFather(father, child);

			}
		}

		// also assign the children to the parent

		// if (father != null) {
		//
		// // remove self
		// removeChild(father, changedObject);
		//
		// // add children of self to father
		// if (array != null) {
		// for (int i = 0; i < array.size(); i++) {
		// addChild(father, (AnnotationFS) array.get(i));
		// }
		// }
		//
		// }

		// update all children
		for (AnnotationFS newGu : this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			updateChildrenFor(newGu);
		}

		updateModel();
		widget.invokeReload();

	}

	private void updateModel() {
		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {
			annoList.add(anno);

		}

		// perform some checks TODO
		performConfidenceChecks();

		// remove null values in childarrays!!
		// removeChildArrayNullValues();

		// update layerfeature to enforce correct drawing
		for (AnnotationFS anno : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			int layer = determineLayer(anno);

			anno.setFeatureValueFromString(featLayerNewCAS, String.valueOf(layer));
		}

	}

	private void performConfidenceChecks() {

		// assert that every annotation has a father or there is no bigger anno

		for (AnnotationFS gu : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			FeatureStructure featureValue = gu.getFeatureValue(featParseFatherNewCAS);

			if (featureValue == null) {

				List<AnnotationFS> candidates = determineSmallestContainingAnno(gu);

				if (candidates.size() > 0) {
					System.out.println("There should be a parent for: " + gu.getCoveredText());
				}

			}
		}

	}

	private void removeChildArrayNullValues() {
		for (AnnotationFS anno : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			ArrayFSImpl array = (ArrayFSImpl) anno.getFeatureValue(featParseChildrenNewCAS);

			if (array == null)
				continue;

			int amountNull = 0;
			for (int i = 0; i < array.size(); i++) {

				AnnotationFS a = (AnnotationFS) array.get(i);

				if (a == null) {
					amountNull++;
				}
			}

			// we need to removet hose null values!!
			if (amountNull > 0) {
				ArrayFS createArrayFS = widget.getCas().createArrayFS(array.size() - amountNull);

				int arrayIndex = 0;
				for (int i = 0; i < array.size(); i++) {

					AnnotationFS a = (AnnotationFS) array.get(i);

					if (a != null) {
						createArrayFS.set(arrayIndex, a);
						arrayIndex++;
					}
				}

				anno.setFeatureValue(featParseChildrenNewCAS, createArrayFS);
			}

		}
	}

	private void updateChildrenFor(AnnotationFS anno) {
		// update child features
		List<AnnotationFS> children = determineChildrenFor(anno);

		// set into the cas
		ArrayFS arrayFS = widget.getCas().createArrayFS(children.size());

		for (int i = 0; i < children.size(); i++) {
			arrayFS.set(i, children.get(i));
		}

		anno.setFeatureValue(featParseChildrenNewCAS, arrayFS);
	}

	private void rescaleWhiteSpaces() {

		List<AnnotationFS> toRescale = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			if (!anno.getCoveredText().equals(anno.getCoveredText().trim())) {
				toRescale.add(anno);

			}
		}

		for (AnnotationFS anno : toRescale) {
			String trimmedText = anno.getCoveredText().trim();

			int firstIndex = anno.getCoveredText().indexOf(trimmedText);
			int begNew = anno.getBegin() + firstIndex;
			int endNew = anno.getBegin() + firstIndex + trimmedText.length();

			Feature begFeature = anno.getType().getFeatureByBaseName("begin");
			anno.setIntValue(begFeature, begNew);

			Feature endFeature = anno.getType().getFeatureByBaseName("end");
			anno.setIntValue(endFeature, endNew);
		}
	}

	private void rescaleAnnotations() {
		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		List<AnnotationFS> toAdd = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			if (!anno.getCoveredText().equals(anno.getCoveredText().trim())) {

				System.out.println("Rescale");
				toDelete.add(anno);
				String trimmedText = anno.getCoveredText().trim();

				int firstIndex = anno.getCoveredText().indexOf(trimmedText);
				AnnotationFS fs = widget.getCas().createAnnotation(grammaticTypeNewCAS, anno.getBegin() + firstIndex,
						anno.getBegin() + firstIndex + trimmedText.length());

				// copy the values

				// assign layer
				fs.setFeatureValueFromString(featLayerNewCAS, anno.getFeatureValueAsString(featLayerNewCAS));

				// phrase type
				fs.setFeatureValueFromString(featPhraseTypeNewCAS, anno.getFeatureValueAsString(featPhraseTypeNewCAS));

				// semantic role
				fs.setFeatureValueFromString(featSemanticRoleNewCAS,
						anno.getFeatureValueAsString(featSemanticRoleNewCAS));

				// grammtical role
				fs.setFeatureValueFromString(featSyntacticRoleNewCAS,
						anno.getFeatureValueAsString(featSyntacticRoleNewCAS));

				// syntactic head
				fs.setFeatureValue(featSyntacticHeadNewCAS, anno.getFeatureValue(featSyntacticHeadNewCAS));

				// parent
				fs.setFeatureValue(featParseFatherNewCAS, anno.getFeatureValue(featParseFatherNewCAS));

				// children
				fs.setFeatureValue(featParseChildrenNewCAS, anno.getFeatureValue(featParseChildrenNewCAS));

				ArrayFSImpl childArray = (ArrayFSImpl) anno.getFeatureValue(featParseChildrenNewCAS);

				if (childArray != null) {
					for (int i = 0; i < childArray.size(); i++) {

						AnnotationFS child = (AnnotationFS) childArray.get(i);

						if (child != null) {
							setFather(fs, child);
						}
					}
				}
				// add the anno
				toAdd.add(fs);
			}
		}

		for (AnnotationFS add : toAdd) {
			widget.getCas().addFsToIndexes(add);
		}

		for (AnnotationFS del : toDelete) {
			widget.getCas().removeFsFromIndexes(del);
			;
		}
	}

	// assumes children are set -recursive determination
	private int determineLayer(AnnotationFS anno) {

		// get the children
		ArrayFSImpl array = (ArrayFSImpl) anno.getFeatureValue(featParseChildrenNewCAS);

		if (array == null || array.size() == 0)
			return 1;

		else {

			int max = -1;
			for (int i = 0; i < array.size(); i++) {
				AnnotationFS child = (AnnotationFS) array.get(i);
				if (!child.equals(anno)) {
					int layChild = determineLayer(child);
					if (layChild > max)
						max = layChild;
				}
			}
			return max + 1;
		}

	}

	// assumes parent features are set
	private List<AnnotationFS> determineChildrenFor(AnnotationFS anno) {

		List<AnnotationFS> children = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			AnnotationFS father = (AnnotationFS) a.getFeatureValue(featParseFatherNewCAS);
			if (father != null && father.equals(anno)) {
				children.add(a);
			}
		}
		return children;
	}

	private void updateParentFeature(AnnotationFS anno) {

		AnnotationFS oldFather = (AnnotationFS) anno.getFeatureValue(featParseFatherNewCAS);
		List<AnnotationFS> biggerAnno = determineSmallestContainingAnno(anno);

		if (oldFather != null) {
			setFather(oldFather, anno);
			// job is done
			return;
		}

		// check if the oldfather was already set
		if (biggerAnno.size() == 1) {

			// check if the sizes of anno and father are the same size otherwise
			// we might get a loop!!

			if (biggerAnno.get(0).getBegin() == anno.getBegin() && biggerAnno.get(0).getEnd() == anno.getEnd()) {

				// check if a real biggerone exists

				// anno.setFeatureValueFromString(this.featLayerNewCAS, "2");

			}

			else {

				// we should be safe here
				anno.setFeatureValue(featParseFatherNewCAS, biggerAnno.get(0));
			}

		}

		if (biggerAnno.size() > 1) {

			// ask the user - List Dialog

			// askUserForFather(anno, biggerAnno);
			// this should actually not happen in a real scenario!!
			// anno.setFeatureValueFromString(this.featLayerNewCAS,
			// String.valueOf((biggerAnno.size() + 1)));
		}

	}

	private void askUserForFather(AnnotationFS anno, List<AnnotationFS> biggerAnno) {
		AnnoSelectionDialog dialog = new AnnoSelectionDialog(Display.getCurrent().getActiveShell(), biggerAnno,
				this.featPhraseTypeNewCAS, anno);
		int open = dialog.open();

		if (open == 0) {
			AnnotationFS sel = dialog.getSelectedAnno();

			if (sel != null) {
				setFather(sel, anno);
			}
		}
	}

	private List<AnnotationFS> determineSmallestContainingAnno(AnnotationFS anno) {

		// can be a liste of not uqniue!!
		List<AnnotationFS> smallestContaining = new ArrayList<AnnotationFS>();

		AnnotationFS smallest = null;
		for (AnnotationFS a : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			if (a.getBegin() <= anno.getBegin() && a.getEnd() >= anno.getEnd()) {

				if (smallest == null) {
					smallest = a;
				} else {

					if (a.getBegin() >= smallest.getBegin() && a.getEnd() <= smallest.getEnd() && !a.equals(anno)) {
						FeatureStructure fatherAnno = anno.getFeatureValue(featParseFatherNewCAS);
						FeatureStructure fatherA = a.getFeatureValue(featParseFatherNewCAS);

						if ((fatherAnno != null && fatherAnno.equals(a)) || (fatherA != null && fatherA.equals(anno)))
							continue;

						smallest = a;
					}
				}
			}
		}

		// now find everything with the same span as smallest and add to the
		// list
		for (AnnotationFS a : widget.getCas().getAnnotationIndex(this.grammaticTypeNewCAS)) {

			if (a.getBegin() == smallest.getBegin() && a.getEnd() == smallest.getEnd()) {
				if (!anno.equals(a)) {
					smallestContaining.add(a);
				}
			}
		}

		return smallestContaining;
	}

	private void onAnnotationAdded(AnnotationFS anno, boolean rescale) {

		// this is a little bit tricky since we must find the position of the
		// new annotations, and adapt all children/father relations of the annos
		// around

		int oldBeg = anno.getBegin();
		int oldEnd = anno.getEnd();
		Feature begFeature = anno.getType().getFeatureByBaseName("begin");

		Feature endFeature = anno.getType().getFeatureByBaseName("end");
		if (rescale) {

			// we enlarge the annotation
			anno.setIntValue(begFeature, oldBeg - 1);
		}

		for (AnnotationFS a : widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			if (anno.getBegin() <= a.getBegin() && anno.getEnd() >= a.getEnd()) {

				// TODO what do we do with new ones that have the same size as
				// an already created anno?
				if (anno.getBegin() == a.getBegin() && anno.getEnd() == a.getEnd()) {
					continue;
				}

				// here we have an anno a that is smaller than the new added
				// anno, therefore we are some ancestor of it...

				// now climb the tree until we found our position (aka once the
				// parent is null or bigger)
				AnnotationFS parent = (AnnotationFS) a.getFeatureValue(featParseFatherNewCAS);

				// if the parent is null right away we know we are the father
				if (parent == null) {

					setFather(anno, a);
				}
				// assign the correct ones in the hierachy
				while (parent != null) {

					if (parent.getBegin() <= anno.getBegin() && parent.getEnd() >= anno.getEnd()) {

						if (!parent.equals(anno)) {
							setFather(parent, anno);
						}

						// also assign anno as parent to all children of parent
						// if possible
						ArrayFSImpl array = (ArrayFSImpl) parent.getFeatureValue(featParseChildrenNewCAS);

						if (array != null) {
							for (int i = 0; i < array.size(); i++) {

								AnnotationFS c = (AnnotationFS) array.get(i);

								// everything that is smaller gets a new paren
								if (c.getBegin() >= anno.getBegin() && c.getEnd() <= anno.getEnd()) {
									if (!c.equals(anno)) {

										setFather(anno, c);
									}
								}
							}
						}
						break;
					}

					parent = (AnnotationFS) parent.getFeatureValue(featParseFatherNewCAS);
				}

				// continue iteration!!
			}

			// if there are no children we have to assume that this annogets to
			// the lowest layer, however we still need to adjust a possible
			// parent in that case

			AnnotationFS fa = (AnnotationFS) anno.getFeatureValue(featParseFatherNewCAS);

			if (fa == null) {
				List<AnnotationFS> possibleFather = determineSmallestContainingAnno(anno);

				if (possibleFather != null && !possibleFather.isEmpty()) {

					AnnotationFS father = possibleFather.get(0);
					setFather(father, anno);

				}
			}
		}

		// we need to rescale the annotations since we abuse the whitespace
		// feature!!

		// update all children
		for (AnnotationFS newGu : this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			updateChildrenFor(newGu);
		}

		if (rescale) {

			// set old begin and end
			anno.setIntValue(begFeature, oldBeg);
		}
		rescaleWhiteSpaces();
		updateModel();
		widget.invokeReload();

	}

	private void setFather(AnnotationFS anno, AnnotationFS c) {

		if (c.equals(anno)) {
			System.out.println("hjkhhkhjkh");
			List<AnnotationFS> annos = determineSmallestContainingAnno(c);

			if (annos.size() > 0) {
				c.setFeatureValue(featParseFatherNewCAS, annos.get(0));
			}
		} else {

			c.setFeatureValue(featParseFatherNewCAS, anno);
		}

	}

	private void removeChild(AnnotationFS father, AnnotationFS child) {

		ArrayFSImpl children = (ArrayFSImpl) father.getFeatureValue(featParseChildrenNewCAS);

		if (children == null)
			return;

		Set<AnnotationFS> uniqueChildren = new HashSet<AnnotationFS>();

		for (int i = 0; i < children.size(); i++) {
			AnnotationFS c = (AnnotationFS) children.get(i);

			if (c != null && !c.equals(child)) {
				uniqueChildren.add(c);
			}
		}

		ArrayFS arrayNew = createArrayFrom(uniqueChildren);
		father.setFeatureValue(featParseChildrenNewCAS, arrayNew);

	}

	private void addChild(AnnotationFS father, AnnotationFS child) {

		ArrayFSImpl children = (ArrayFSImpl) father.getFeatureValue(featParseChildrenNewCAS);

		Set<AnnotationFS> uniqueAnnos = new HashSet<AnnotationFS>();

		// we add all former children
		if (children != null) {
			for (int i = 0; i < children.size(); i++) {
				AnnotationFS c = (AnnotationFS) children.get(i);

				if (c != null && !c.equals(father)) {
					uniqueAnnos.add(c);
				}

			}
		}
		// and us
		uniqueAnnos.add(child);

		ArrayFS arrayNew = createArrayFrom(uniqueAnnos);
		father.setFeatureValue(featParseChildrenNewCAS, arrayNew);

	}

	private ArrayFS createArrayFrom(Set<AnnotationFS> uniqueAnnos) {

		ArrayFS array = widget.getCas().createArrayFS(uniqueAnnos.size());

		int index = 0;
		for (AnnotationFS a : uniqueAnnos) {
			array.set(index, a);
			index++;
		}
		return array;
	}

	@Override
	public void copyFromOldCASToNewCAS(CAS casOld, CAS casNew, AnnotationFS sentence) {
		List<AnnotationFS> covered = utilOld.getCovered(sentence, grammaticTypeOldCAS);
		for (AnnotationFS c : covered) {
			AnnotationFS gUCopy = casNew.createAnnotation(this.grammaticTypeNewCAS, c.getBegin() - editorSentBegin,
					c.getEnd() - editorSentBegin);

			// set features
			gUCopy.setFeatureValueFromString(featPhraseTypeNewCAS, c.getFeatureValueAsString(featPhraseTypeOldCAS));
			gUCopy.setFeatureValueFromString(featLayerNewCAS, c.getFeatureValueAsString(featLayerOldCAS));
			gUCopy.setFeatureValueFromString(featSyntacticRoleNewCAS,
					c.getFeatureValueAsString(featSyntacticRoleOldCAS));
			gUCopy.setFeatureValueFromString(featSemanticRoleNewCAS, c.getFeatureValueAsString(featSemanticRoleOldCAS));

			// syntactic head
			// gUCopy.setFeatureValue(featSyntacticHeadNewCAS,
			// c.getFeatureValue(featSyntacticHeadOldCAS));
			//

			this.widget.getCas().addFsToIndexes(gUCopy);
		}

		// also update the parent feature
		for (AnnotationFS oldGU : covered) {
			AnnotationFS father = (AnnotationFS) oldGU.getFeatureValue(featParseFatherOldCAS);

			if (father != null) {
				AnnotationFS newGu = findAccordingAnno(oldGU,
						this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS), true);

				AnnotationFS fatherNew = findAccordingAnno(father,
						this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS), true);

				if (newGu != null && fatherNew != null) {
					setFather(fatherNew, newGu);
				}
			}
		}

		// assign children
		for (AnnotationFS newGu : this.widget.getCas().getAnnotationIndex(grammaticTypeNewCAS)) {

			updateChildrenFor(newGu);
		}

		updateModel();
	}

	@Override
	public void initAnalyzerTypings() {

		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle style2 = new AnnoStyle(new GrammarRealDrawingStrategy(), rangeOv);
		// apply
		this.widget.changeStyle(this.grammaticTypeNewCAS, style2);

		// also change the modus
		// this.widget.setActiveMode(this.grammaticTypeNewCAS);

		this.widget.changeVisibility(this.grammaticTypeNewCAS, true);
		this.widget.changeLayer(this.grammaticTypeNewCAS, 0);
	}

}
