package de.uniwue.mk.kall.athen.dualEditor.ui;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.cas.impl.XmiCasDeserializer;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.fit.factory.TypeSystemDescriptionFactory;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.metadata.TypeSystemDescription;
import org.apache.uima.util.CasCreationUtils;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.custom.TableEditor;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.xml.sax.SAXException;

import de.uniwue.kalimachos.coref.paintingStrategies.DualEditorDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.FeaturePosition;
import de.uniwue.kalimachos.coref.paintingStrategies.FeaturePositionPair;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.CASEditorInput;

public class AgreementSelectionView implements IPerspectiveDecoupledPart {

	// CONSTANTS

	private String tpString = "de.uniwue.kalimachos.coref.type.TP";
	private String fnString = "de.uniwue.kalimachos.coref.type.FN";
	private String fpString = "de.uniwue.kalimachos.coref.type.FP";

	private boolean isInitialized = false;

	// Hashmap that maps all evaluation types to the rule annotations

	private HashMap<AnnotationFS, AnnotationFS> evalMapping;

	private HashMap<Feature, FeaturePositionPair> visibilityMap;

	String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.9";

	// FIELDS======

	private Composite composite;
	private LocalResourceManager resManager;
	@Inject
	private IEventBroker broker;
	@Inject
	EPartService partService;

	@Inject
	EModelService modelService;

	// the gold type
	private Type goldType;

	private TypeSystem typesystem;

	private CAS goldCAS;
	private File goldFile;

	private CAS compareCAS;
	private File compareFile;

	private Type tpType;
	private Type fnType;
	private Type fpType;

	// Widgets

	private Text textCompare;
	private Label labelCompare;
	private Button buttonCompare;

	private Button buttonCompareType;
	private Label labelCompareType;
	private Text textCompareType;

	private Text textTo;
	private Label labelTo;
	private Button buttonTo;

	private Button buttonDeserialize;

	private MPart part;

	// widgets for this view
	// table that contains the features
	private Table table;
	private boolean unCreated = true;

	public HashMap<AnnotationFS, AnnotationFS> getEvalMapping() {
		return evalMapping;
	}

	public void setEvalMapping(HashMap<AnnotationFS, AnnotationFS> evalMapping) {
		this.evalMapping = evalMapping;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Construct AgreementSelectionView");

		this.visibilityMap = new HashMap<Feature, FeaturePositionPair>();

		this.composite = parent;
		resManager = new LocalResourceManager(JFaceResources.getResources(), composite);

		// editor events

		this.composite.setLayout(new FillLayout());

		// ==========SUBSCRIBE TO EVENTS=============
		this.part = partService.findPart(viewId);

	}

	private void subscribeToEvents() {

	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {
		// for (Control c : composite.getChildren()) {
		// c.dispose();
		// }
		if (unCreated) {
			initLayout();
		}

	}

	public void onEditorInputChanged(AnnotatorAgreementAnnotationEditor editor) {

		// only initialize if the part is visible

		MPart findPart = partService.findPart(viewId);

		// if the part is active add the listener

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : composite.getChildren()) {
			c.dispose();
		}

		// init AnnoList

		initLayout();

		// annotate types if they are already set

		// ====reload=====

		composite.layout();
		composite.getParent().layout();
	}

	private void initLayout() {
		System.out.println("initLayout");

		unCreated = false;

		for (Control c : composite.getChildren()) {
			c.dispose();
		}

		// composite gets three coloumns

		GridLayout layout = new GridLayout();

		layout.numColumns = 3;

		this.composite.setLayout(layout);
		this.composite.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL
				| GridData.GRAB_VERTICAL));

		// add a Label/textfield for the types that are about to be compared
		this.labelTo = new Label(composite, SWT.NULL);
		this.labelTo.setText("To (GoldFile [Left]): ");
		this.textTo = new Text(composite, SWT.BORDER);
		this.buttonTo = new Button(composite, SWT.PUSH);
		this.buttonTo.setText("Choose Type");

		this.labelCompare = new Label(composite, SWT.NULL);
		this.labelCompare.setText("Compare File[Right]: ");
		this.textCompare = new Text(composite, SWT.BORDER);
		this.buttonCompare = new Button(composite, SWT.PUSH);
		this.buttonCompare.setText("Choose Type");

		// add selection listener

		this.buttonTo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				FileDialog dialog = new FileDialog(composite.getShell());
				dialog.setFilterExtensions(new String[] { "*.xmi" });
				dialog.setText("Choose a Goldfile");
				String selection = dialog.open();

				if (selection != null) {
					goldFile = new File(selection);

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// add selection listener

		this.buttonCompareType = new Button(composite, SWT.NULL);
		buttonCompareType.setText("Choose: ");
		this.labelCompareType = new Label(composite, SWT.NULL);
		labelCompareType.setText("Select type to Compare");
		this.textCompareType = new Text(composite, SWT.NULL);
		this.textCompareType.setText("Type: ");

		this.buttonCompareType.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				TypeSystem ts = null;

				if (goldCAS != null) {
					ts = goldCAS.getTypeSystem();
				}

				else if (compareCAS != null) {
					ts = compareCAS.getTypeSystem();
				}

				if (ts == null) {
					return;
				}

				AgreementViewTypeChoosingDialog dialog = new AgreementViewTypeChoosingDialog(composite.getShell(), ts);

				int exitCode = dialog.open();

				if (exitCode == 0) {

					goldType = dialog.getSelectedType();
					createTable();

					// broker.send(AnnotationEditorEvents.DUAL_EDITOR_VISIBILITY_CHANGED,
					// goldType);

				}

				else {
					//
					System.out.println("Aborted");
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		this.buttonCompare.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				FileDialog dialog = new FileDialog(composite.getShell());
				dialog.setText("Choose a File you want to compare");
				dialog.setFilterExtensions(new String[] { "*.xmi" });
				String selection = dialog.open();

				if (selection != null) {
					compareFile = new File(selection);
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		this.buttonDeserialize = new Button(composite, SWT.NULL);

		buttonDeserialize.setText("Compare ! ");

		buttonDeserialize.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (goldFile != null && compareFile != null) {

					try {
						try {
							deserialize();
						} catch (SAXException | IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}
					} catch (ResourceInitializationException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// ===BEGIN TABLE=====

		// create coloumns

		// ==END TABLE====

		// feature button

		composite.layout();
		composite.getParent().layout();
	}

	private void createTable() {

		System.out.println("create Table");

		if (this.goldType == null) {
			return;
		}

		System.out.println("Create table");
		table = new Table(composite, SWT.NONE);
		table.setHeaderVisible(true);

		GridData tableGrid = new GridData();
		tableGrid.horizontalSpan = 3;
		table.setLayoutData(tableGrid);

		TableColumn tableColumnVisible = new TableColumn(table, SWT.NONE);
		tableColumnVisible.setWidth(100);
		tableColumnVisible.setText("Visible ?");

		TableColumn tableColumnFeatureName = new TableColumn(table, SWT.NONE);
		tableColumnFeatureName.setWidth(100);
		tableColumnFeatureName.setText("Feature Name");

		TableColumn tableColumnFeatureStrategy = new TableColumn(table, SWT.NONE);
		tableColumnFeatureStrategy.setWidth(100);
		tableColumnFeatureStrategy.setText("Drawing Strategy");

		// get the features of that type

		for (Feature f : this.goldType.getFeatures()) {

			if (f.getShortName().equals("begin") || f.getShortName().equals("end")) {
				continue;
			}

			if (f.getRange().isPrimitive()) {
				TableItem item = new TableItem(table, SWT.NONE);
				TableEditor editor = new TableEditor(table);
				CCombo combo = new CCombo(table, SWT.NONE);
				combo.setText("NONE");

				for (FeaturePosition s : FeaturePosition.values()) {

					combo.add(s.name());
				}

				combo.addSelectionListener(new SelectionListener() {

					@Override
					public void widgetSelected(SelectionEvent e) {

						CCombo b = (CCombo) e.getSource();

						Feature data = (Feature) b.getData("FEATURE");
						Button button = (Button) b.getData("VISIBILITY");

						updateStyles(data, button.getSelection(), b.getItem(b.getSelectionIndex()));

					}

					@Override
					public void widgetDefaultSelected(SelectionEvent e) {
						widgetSelected(e);

					}
				});
				editor.grabHorizontal = true;
				editor.setEditor(combo, item, 2);
				editor = new TableEditor(table);

				Text text = new Text(table, SWT.NONE);
				text.setText(f.getShortName());
				editor.grabHorizontal = true;
				editor.setEditor(text, item, 1);

				editor = new TableEditor(table);
				Button button = new Button(table, SWT.CHECK);
				button.setData("FEATURE", f);
				button.setData("STYLE", combo);
				button.pack();
				editor.minimumWidth = button.getSize().x;
				editor.horizontalAlignment = SWT.LEFT;
				editor.setEditor(button, item, 0);

				combo.setData("FEATURE", f);
				combo.setData("VISIBILITY", button);

				button.addSelectionListener(new SelectionListener() {

					@Override
					public void widgetSelected(SelectionEvent e) {
						Button b = (Button) e.getSource();

						Feature data = (Feature) b.getData("FEATURE");
						CCombo combo = (CCombo) b.getData("STYLE");

						int selectionIndex = combo.getSelectionIndex();
						String item2 = "";
						if (selectionIndex == -1) {
							item2 = "NONE";
						} else {

							item2 = combo.getItem(selectionIndex);
						}

						updateStyles(data, b.getSelection(), item2);

					}

					@Override
					public void widgetDefaultSelected(SelectionEvent e) {
						widgetSelected(e);
					}
				});

			}
		}

		this.composite.layout();

	}

	private void updateStyles(Feature f, boolean selection, String style) {

		if (this.visibilityMap.containsKey(f)) {

			// update the feature styles
			FeaturePositionPair featurePositionPair = visibilityMap.get(f);

			featurePositionPair.setVisible(selection);
			featurePositionPair.setPosition(FeaturePosition.valueOf(style));

		}

		else {
			FeaturePositionPair featurePositionPair = new FeaturePositionPair(f, FeaturePosition.valueOf(style),
					selection);
			this.visibilityMap.put(f, featurePositionPair);

		}
		invokeEditorReload();

	}

	private void invokeEditorReload() {

		System.out.println("Invoke Editor reload");

		DualEditorDrawingStrategy strat = new DualEditorDrawingStrategy(this.visibilityMap.values(),
				goldCAS.getAnnotationIndex(goldType), compareCAS.getAnnotationIndex(goldType), false);

		// apply
		broker.send(IAnnotationEditorEvents.DUAL_EDITOR_STYLES_CHANGED, strat);
		// editor.changeStyle(temporary, style3);

		// TODO
	}

	public Type getTpType() {
		return tpType;
	}

	public void setTpType(Type tpType) {
		this.tpType = tpType;
	}

	public Type getFnType() {
		return fnType;
	}

	public void setFnType(Type fnType) {
		this.fnType = fnType;
	}

	public Type getFpType() {
		return fpType;
	}

	public void setFpType(Type fpType) {
		this.fpType = fpType;
	}

	// private void annotateEvaluationTypes() {
	//
	// System.out.println("Annotate");
	//
	// if (editor == null)
	// return;
	//
	// this.tpType = editor.getCas().getTypeSystem().getType(tpString);
	// this.fpType = editor.getCas().getTypeSystem().getType(fpString);
	// this.fnType = editor.getCas().getTypeSystem().getType(fnString);
	//
	// deleteEvaluationTypes();
	// evalMapping = new HashMap<AnnotationFS, AnnotationFS>();
	//
	// // add the new ones
	//
	// // check if the typesystem has POS types defined and if they are
	// // available
	//
	// boolean tokenBased = false;
	// String posTypeS = "de.uniwue.kalimachos.coref.type.POS";
	// Type type = editor.getCas().getTypeSystem().getType(posTypeS);
	//
	// if (type != null) {
	//
	// int size = editor.getCas().getAnnotationIndex(type).size();
	//
	// if (size > 0) {
	// // use POS Tags for a token based evaluation
	// tokenBased = true;
	// }
	// }
	//
	// // TODO
	// tokenBased = false;
	// AnnotationIndex<AnnotationFS> goldIndex =
	// editor.getCas().getAnnotationIndex(goldType);
	//
	// AnnotationIndex<AnnotationFS> compareIndex =
	// editor.getCas().getAnnotationIndex(compareType);
	//
	// ArrayList<AnnotationFS> goldList = new ArrayList<AnnotationFS>();
	//
	// for (AnnotationFS a : goldIndex) {
	//
	// if (tokenBased) {
	//
	// for (AnnotationFS token : editor.getCas().getAnnotationIndex(type)) {
	// if (token.getBegin() >= a.getBegin() && token.getEnd() <= a.getEnd()) {
	// goldList.add(token);
	// }
	// }
	// }
	//
	// else {
	// goldList.add(a);
	// }
	// }
	//
	// // a set to detect all fns
	// HashSet<AnnotationFS> annoSet = new HashSet<AnnotationFS>();
	//
	// for (AnnotationFS comp : compareIndex) {
	//
	// boolean found = false;
	//
	// for (AnnotationFS gold : goldList) {
	//
	// if (gold.getBegin() == comp.getBegin() && gold.getEnd() == comp.getEnd())
	// {
	// found = true;
	// annoSet.add(gold);
	// break;
	// }
	//
	// }
	//
	// if (found == true) {
	// AnnotationFS tp = editor.getCas().createAnnotation(tpType,
	// comp.getBegin(), comp.getEnd());
	// editor.getCas().addFsToIndexes(tp);
	//
	// evalMapping.put(tp, comp);
	// }
	//
	// else {
	//
	// AnnotationFS fp = editor.getCas().createAnnotation(fpType,
	// comp.getBegin(), comp.getEnd());
	// editor.getCas().addFsToIndexes(fp);
	// evalMapping.put(fp, comp);
	//
	// }
	// }
	//
	// // last but not least the Fns
	//
	// for (AnnotationFS gold : goldList) {
	//
	// if (!annoSet.contains(gold)) {
	// AnnotationFS fn = editor.getCas().createAnnotation(fnType,
	// gold.getBegin(), gold.getEnd());
	//
	// editor.getCas().addFsToIndexes(fn);
	// evalMapping.put(fn, gold);
	// }
	// }
	//
	// // change visibility
	//
	// // Visibility
	//
	// editor.changeVisibility(tpType, true);
	// editor.changeVisibility(fnType, true);
	// editor.changeVisibility(fpType, true);
	//
	// // layer
	//
	// editor.changeLayer(tpType, 2);
	// editor.changeLayer(fpType, 3);
	// editor.changeLayer(fnType, 3);
	//
	// // tp - green
	// StyleRange rangeOv = new StyleRange();
	//
	// rangeOv.background = resManager.createColor(new RGB(0, 255, 0));
	// AnnoStyle styleTP = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv),
	// rangeOv);
	//
	// // fp blue
	// StyleRange rangeFP = new StyleRange();
	//
	// rangeFP.background = resManager.createColor(new RGB(35, 35, 255));
	// AnnoStyle styleFP = new AnnoStyle(new BackgroundDrawingStrategy(rangeFP),
	// rangeFP);
	//
	// // fn red
	// StyleRange rangeFN = new StyleRange();
	//
	// rangeFN.background = resManager.createColor(new RGB(255, 0, 0));
	// AnnoStyle styleFN = new AnnoStyle(new BackgroundDrawingStrategy(rangeFN),
	// rangeFN);
	//
	// editor.changeStyle(tpType, styleTP);
	// editor.changeStyle(fpType, styleFP);
	// editor.changeStyle(fnType, styleFN);
	//
	// try {
	// renewPage();
	// } catch (CASException e) {
	// // TODO Auto-generated catch block
	// e.printStackTrace();
	// }
	// }

	private void deleteEvaluationTypes() {

		// if (this.tpType == null || this.fnType == null || this.fpType == null
		// || this.editor == null) {
		// return;
		// }
		//
		// ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		//
		// for (AnnotationFS tp : editor.getCas().getAnnotationIndex(tpType)) {
		// toDelete.add(tp);
		// }
		//
		// for (AnnotationFS fp : editor.getCas().getAnnotationIndex(fpType)) {
		// toDelete.add(fp);
		// }
		//
		// for (AnnotationFS fn : editor.getCas().getAnnotationIndex(fnType)) {
		// toDelete.add(fn);
		// }
		//
		// for (AnnotationFS anno : toDelete) {
		// editor.getCas().removeFsFromIndexes(anno);
		// }

		System.out.println("Old types deleted");
	}

	private void updateDrawingSytle() {

		// if (goldFeature == null || compFeature == null) {
		// return;
		// }
		//
		// System.out.println("Update Comparision drawingstyle");
		//
		// // Visibility - we use this one as a fake comparison type
		// editor.changeVisibility(this.goldType, true);
		//
		// // layer
		//
		// editor.changeLayer(this.goldType, 0);
		//
		// // yellow
		// StyleRange rangeOv = new StyleRange();
		// rangeOv.background = new Color(composite.getDisplay(), 255, 255,
		// 255);
		// AnnoStyle style = new AnnoStyle(new
		// FeatureComparisionDrawingStrategy(goldType, compareType, goldFeature,
		// compFeature, editor.getCas().getAnnotationIndex(goldType),
		// editor.getCas().getAnnotationIndex(
		// compareType)), rangeOv);
		//
		// // apply
		// editor.changeStyle(goldType, style);

	}

	public void renewPage() throws CASException {

	}

	private void fileSelected() {

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	private void unsubscribeFromEvents() {

	}

	private void deserialize() throws ResourceInitializationException, FileNotFoundException, SAXException, IOException {

		// check that we can deserialize was already made !

		// get the typesystem from preferences
		String tsdPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.EDITOR_TSD_PATH, "null");

		File tsdFile = new File(tsdPref);
		TypeSystemDescription tsd = null;

		if (tsdFile.exists()) {

			// only if the Typesystem exists
			tsd = TypeSystemDescriptionFactory.createTypeSystemDescriptionFromPath(tsdFile.toURI().toString());

			goldCAS = CasCreationUtils.createCas(tsd, null, null);
			compareCAS = CasCreationUtils.createCas(tsd, null, null);

			// deserialize both files into the cas

			XmiCasDeserializer.deserialize(new FileInputStream(goldFile), goldCAS, true);
			XmiCasDeserializer.deserialize(new FileInputStream(compareFile), compareCAS, true);

			// notify the editor !!!

			broker.send(IAnnotationEditorEvents.DUALEDITOR_INPUT_CHANGED, new DualCASEditorInput(new CASEditorInput(
					goldCAS, goldFile), new CASEditorInput(compareCAS, compareFile)));

		}

		else {
			// TODO open a dialog to select a typesystem !!
		}

	}

	@Override
	public void coupleToPerspective() {

		subscribeToEvents();

	}

	@Override
	public void decoupleFromPerspective() {

		unsubscribeFromEvents();

	}

}
