package de.uniwue.mk.kall.athen.dualEditor.ui;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeFilter;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class AgreementViewTypeChoosingDialog extends Dialog {

	private TableViewer viewer;

	private TypeFilter filter;

	private Composite parent;

	private Label labelSelectedType;

	private Text textSelectedType;

	private Type selectedType;

	private TypeSystem tsd;

	private boolean useFilter;

	public AgreementViewTypeChoosingDialog(Shell parent, TypeSystem tsd) {
		super(parent);

		this.tsd = tsd;

	}

	@Override
	protected Control createDialogArea(Composite parent) {

		Composite composite = (Composite) super.createDialogArea(parent);
		this.parent = parent;

		composite.getShell().setText("Type choosing dialog");

		composite.setLayout(new GridLayout(1, true));

		draw(composite); // Contents of Dialog

		return composite;
	}

	private void draw(Composite parent) {

		// at first a Tableviewer with the list of Types

		Composite compp = new Composite(parent, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		Label searchLabel = new Label(compp, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compp, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		viewer = new TableViewer(compp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER);

		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.heightHint = 600;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		createColumns(viewer);
		// content
		viewer.setContentProvider(new ArrayContentProvider());

		// TODO
		Type topAnno = tsd.getType("uima.tcas.Annotation");

		List<TypeStyleDataStructure> tsdsList = new ArrayList<TypeStyleDataStructure>();

		for (Type next : tsd.getProperlySubsumedTypes(topAnno)) {

			TypeStyleDataStructure tsds = new TypeStyleDataStructure(next, null, 0);
			tsdsList.add(tsds);

		}

		viewer.setInput(tsdsList.toArray(new TypeStyleDataStructure[0]));
		// filter
		filter = new TypeFilter();
		viewer.addFilter(filter);

		labelSelectedType = new Label(compp, SWT.NONE);
		labelSelectedType.setText("Actual selected Type:");

		textSelectedType = new Text(compp, SWT.NONE);
		textSelectedType.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL
				| GridData.GRAB_VERTICAL));

	}

	private void createColumns(final TableViewer viewer) {

		// Typename
		TableViewerColumn colName = createTableViewerColumn("Type", 300, 1);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				TypeStyleDataStructure t = (TypeStyleDataStructure) element;
				return t.getAnnotationType().toString();

			}

		});

		colName.getViewer().getControl().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {

			}

			@Override
			public void mouseDown(MouseEvent e) {

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				TypeStyleDataStructure firstElement = (TypeStyleDataStructure) sel.getFirstElement();

				String shortName = firstElement.getAnnotationType().getShortName();

				textSelectedType.setText(shortName);
				selectedType = firstElement.getAnnotationType();
			}
		});

	}

	public Type getSelectedType() {
		return selectedType;
	}

	public void setSelectedType(Type selectedType) {
		this.selectedType = selectedType;
	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
