package de.uniwue.mk.kall.athen.dualEditor.ui;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.text.JFaceTextUtil;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.editor.drawingstructure.StyleDataStructure;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.DualEditorDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.dialog.SearchDialog;

public class AnnotatorAgreementAnnotationEditor implements IPerspectiveDecoupledPart {

	private String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.dualEditor";

	// the model
	private CAS casGold;
	private File openedFileGold;

	private CAS casCompare;
	private File openedFileCompare;

	// the viewer;

	private Composite composite;

	@Inject
	MDirtyable dirty;

	@Inject
	EModelService modelService;

	// CAS

	// Text Widgets
	StyledText widgetGold;
	StyledText widgetCompare;

	// parent Composite
	Composite parent;

	// Default COLOR - Yellow
	Color DEFAULT_COLOR;

	// actual selections
	List<AnnotationFS> actualSelectionGold;

	// actual Textselection
	Point textSelectionGold;
	Point textSelectionCompare;

	// Visible Annotations - both editors will have the same visibilities

	private HashMap<AnnotationFS, TypeStyleDataStructure> visibleMapGold;

	private HashMap<AnnotationFS, TypeStyleDataStructure> visibleMapComp;

	private AnnotatorAgreementAnnotationEditor editor;

	private long timeOfLastUpdate = 0;

	private static final int COLORSTEP = 30;

	// HashMap for Styles

	private StyleDataStructure styleStructGold;

	// Listener Helper

	// TODO
	private DualCaretAdapter caretListener;

	private TypeSystem typeSystem;

	@Inject
	IEventBroker broker;
	private EventHandler dualInputChangedHandler;
	private EventHandler visibilityChangedHandler;
	private EventHandler strategyChangedHandler;

	@Inject
	private EPartService partService;
	private MPart part;

	@PostConstruct
	public void postConstruct(Composite parent) {
		System.out.println("Construct DualEditor");

		// Save the composite
		this.composite = parent;

		// add key listener to save the editor
		this.part = partService.findPart(viewId);

		dualInputChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event e) {

				DualCASEditorInput dualInput = (DualCASEditorInput) e.getProperty(IEventBroker.DATA);

				onDualCASChanged(dualInput);

			}
		};

		strategyChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event e) {

				DualEditorDrawingStrategy strat = (DualEditorDrawingStrategy) e.getProperty(IEventBroker.DATA);
				TypeStyleDataStructure dsFromType = getDSFromType(strat.getActualType());

				dsFromType.getStyle().setDrawingStrategy(strat);

				changeVisibility(strat.getActualType(), true);
				styleChanged(dsFromType);
			}

		};

		visibilityChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {

				Type t = (Type) arg0.getProperty(IEventBroker.DATA);

				TypeStyleDataStructure dsFromType = editor.getDSFromType(t);

				Boolean boolean1 = styleStructGold.getStyleMap().get(dsFromType);

				changeVisibility(t, !boolean1);
			}
		};
	}

	protected TypeStyleDataStructure getDSFromType(Type t) {
		// TODO Auto-generated method stub
		return this.styleStructGold.getDSFromType(t);
	}

	/*
	 * this method is called whenever a new document is opened
	 */
	public void onDualCASChanged(DualCASEditorInput input) {

		this.casGold = input.getGold().getCas();
		this.openedFileGold = input.getGold().getEditorInputFile();

		this.casCompare = input.getCompare().getCas();
		this.openedFileCompare = input.getCompare().getEditorInputFile();

		// clear compositei

		deleteAttachedWidgets();

		FillLayout layout = new FillLayout();
		composite.setLayout(layout);

		// initialize COlor and Mode
		DEFAULT_COLOR = new Color(composite.getDisplay(), 255, 255, 0);
		this.editor = this;

		// selection provider

		// initialize the Editor Input
		widgetGold = new StyledText(composite, SWT.V_SCROLL | SWT.H_SCROLL | SWT.WRAP);
		widgetGold.setText(this.casGold.getDocumentText());
		widgetGold.setEditable(false);

		widgetGold.setData("NAME", "Gold");

		widgetCompare = new StyledText(composite, SWT.V_SCROLL | SWT.H_SCROLL | SWT.WRAP);
		widgetCompare.setText(this.casCompare.getDocumentText());
		widgetCompare.setEditable(false);

		widgetCompare.setData("NAME", "Compare");

		// addKeyListener(widgetGold);

		widgetCompare.setAlwaysShowScrollBars(true);

		widgetCompare.getVerticalBar().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				widgetGold.setTopIndex(widgetCompare.getTopIndex());
				widgetCompare.setTopIndex(widgetGold.getTopIndex());

				// TODO force a redraw on the other editor ...but how ???

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		widgetGold.getVerticalBar().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				widgetCompare.setTopIndex(widgetGold.getTopIndex());
				widgetGold.setTopIndex(widgetCompare.getTopIndex());

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);
			}
		});

		this.typeSystem = casGold.getTypeSystem();

		// init Listener
		// caretListener = new CaretAdapter(this, broker);
		// selectionListener = new SelectionEditorAdapter(this);

		// vis List
		visibleMapGold = new HashMap<AnnotationFS, TypeStyleDataStructure>();
		visibleMapComp = new HashMap<AnnotationFS, TypeStyleDataStructure>();

		// setStyleOfText
		initStyle();

		// addListener
		// widgetGold.addCaretListener(caretListener);
		widgetGold.addPaintListener(new PaintListener() {

			@Override
			public void paintControl(PaintEvent e) {

				Runnable r = new Runnable() {

					@Override
					public void run() {

						for (AnnotationFS anno : visibleMapGold.keySet()) {
							TypeStyleDataStructure ds = visibleMapGold.get(anno);

							IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

							// only draw if the annotation is visible

							int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widgetGold);
							// The index of the first (possibly only partially)
							// visible line of the widget
							int topIndex = JFaceTextUtil.getPartialTopIndex(widgetGold);

							int actualIndex = widgetGold.getLineAtOffset(anno.getEnd());
							if (actualIndex > topIndex && actualIndex < bottomIndex) {

								GC gc = new GC(widgetGold);
								drawingStrategy.drawAnnotation(ds, anno, widgetGold, gc);
								gc.dispose();
							}
						}
					}
				};

				if (System.currentTimeMillis() - timeOfLastUpdate > 500) {
					timeOfLastUpdate = System.currentTimeMillis();
					Display.getCurrent().asyncExec(r);

				}

			}
		});

		widgetCompare.addPaintListener(new PaintListener() {

			@Override
			public void paintControl(PaintEvent e) {

				Runnable r = new Runnable() {

					@Override
					public void run() {

						for (AnnotationFS anno : visibleMapComp.keySet()) {
							TypeStyleDataStructure ds = visibleMapComp.get(anno);

							IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

							if (drawingStrategy instanceof DualEditorDrawingStrategy) {

								int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widgetCompare);
								// The index of the first (possibly only
								// partially)
								// visible line of the widget
								int topIndex = JFaceTextUtil.getPartialTopIndex(widgetCompare);

								int actualIndex = widgetCompare.getLineAtOffset(anno.getEnd());
								if (actualIndex > topIndex && actualIndex < bottomIndex) {
									GC gc = new GC(widgetCompare);
									drawingStrategy.drawAnnotation(ds, anno, widgetCompare, gc);

									gc.dispose();
								}
							}
						}
					}
				};

				if (System.currentTimeMillis() - timeOfLastUpdate > 500) {
					timeOfLastUpdate = System.currentTimeMillis();
					Display.getCurrent().asyncExec(r);

				}

			}
		});

		// styleMap
		initStyleMap(parent);

		// rightclickMenu
		// initializeMenu();

		// ====RELOAD PART=====
		composite.layout();
		composite.getParent().layout();

	}

	private void addKeyListener(StyledText widgetGold) {
		widgetGold.addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {

				// ctrl + s
				if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 115)) {
					save();
				}

			}

			@Override
			public void keyPressed(KeyEvent e) {

				if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'f')) {
					// open search dialog
					SearchDialog sd = new SearchDialog(editor.getParent().getShell(), "");
					// sd.addFindListener(new
					// EditorFindHandler(editor.getWidget(), sd, editor));

					sd.open();
				}

			}
		});
		widgetGold.addSelectionListener(new SelectionListener() {

			// get every change of the textwidget
			@Override
			public void widgetSelected(SelectionEvent e) {
				// set the actual selection
				// System.out.println(e.getSource());
				if (e.getSource().toString().split(" ")[0].equals("StyledText")) {
					// change both Selections
					textSelectionGold = AnnotatorAgreementAnnotationEditor.this.widgetGold.getSelection();

					// also update the selected Annotations

					Point selection = AnnotatorAgreementAnnotationEditor.this.widgetGold.getSelection();
					// clear the actual selection
					actualSelectionGold.clear();

					if (selection.x - selection.y == 0) {
						// no "real" selection this is more tricky

					}

					else {

						// a real selection this one is the easier
						// implementation
						AnnotationIndex<AnnotationFS> annotationIndex = casGold.getAnnotationIndex();

						// create a dummy annotation and select all Annotations
						// in the span of this anno
						Type topAnno = typeSystem.getType("uima.tcas.Annotation");
						AnnotationFS dummyAnno = casGold.createAnnotation(topAnno, selection.x, selection.y);

						FSIterator<AnnotationFS> subiterator = annotationIndex.subiterator(dummyAnno);

						while (subiterator.hasNext()) {
							AnnotationFS next = subiterator.next();
							actualSelectionGold.add(next);
						}

					}

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

			}
		});
	}

	private void deleteAttachedWidgets() {
		for (Control c : composite.getChildren()) {
			c.dispose();
		}
	}

	public void fsSelectionChanged() {

		// for (IFSSelectionListener lis : this.fsSelectionListener) {
		// lis.fsSelectionChanged(actualSelection);
		// }

		// notify listener

		broker.send(IAnnotationEditorEvents.EDITOR_SELECTION_CHANGED, actualSelectionGold);

		invokeReload(widgetGold.getCaretOffset(), widgetCompare.getCaretOffset(), 2000);
		// Dirty Diana!!!
		dirty.setDirty(true);
	}

	public void invokeReload(int caretOffset, int careOffSetComp, int reloadDistance) {
		System.out.println("invokeReload");

		// this method gets All Annotations that are visible atm
		visibleMapGold.clear();

		// TODO major speedup possible
		widgetGold.setStyleRanges(0, widgetGold.getText().length(), null, null);

		// create a dummy annotation
		Type topAnno = typeSystem.getType("uima.tcas.Annotation");
		// AnnotationFS dummyAnno = casGold.createAnnotation(topAnno, endGold,
		// from);
		// TODO major speedup possible
		AnnotationFS dummyAnno = casGold.createAnnotation(topAnno, 0, widgetGold.getText().length());

		// for each visible AnnotationType ...
		for (TypeStyleDataStructure ds : styleStructGold.getStylesFromLowestLayerToHighest()) {
			if (styleStructGold.getStyleMap().get(ds) == false) {
				continue;
			}

			// Test=====
			AnnotationIndex<AnnotationFS> annotationIndex = casGold.getAnnotationIndex(ds.getAnnotationType());
			FSIterator<AnnotationFS> subiterator = annotationIndex.subiterator(dummyAnno);
			List<StyleRange> ranges = new ArrayList<StyleRange>();
			while (subiterator.hasNext()) {
				AnnotationFS anno = subiterator.next();

				// delegate to the Styledrawing object in the AnnoStyles which
				// are part of a TSDS
				IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

				if (drawingStrategy instanceof BackgroundDrawingStrategy) {
					// draw this here cause it doesnt need to be updated
					drawingStrategy.drawAnnotation(ds, anno, this.widgetGold, new GC(this.widgetGold));
				} else if (drawingStrategy instanceof DualEditorDrawingStrategy) {

					int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widgetGold);
					// The index of the first (possibly only partially)
					// visible line of the widget
					int topIndex = JFaceTextUtil.getPartialTopIndex(widgetGold);

					int actualIndex = widgetGold.getLineAtOffset(anno.getEnd());
					if (actualIndex > topIndex || actualIndex < bottomIndex) {

						DualEditorDrawingStrategy strat = (DualEditorDrawingStrategy) drawingStrategy;

						Color color = strat.determineColor(anno, widgetGold);

						if (color != null) {
							// it should be drawn
							StyleRange styleRange = new StyleRange();
							styleRange.start = anno.getBegin();
							styleRange.length = anno.getEnd() - anno.getBegin();
							styleRange.background = color;
							ranges.add(styleRange);

						}

						// draw the text updates here
						visibleMapGold.put(anno, ds);
					}

				}

				else {
					visibleMapGold.put(anno, ds);
				}

			}

			widgetGold.setStyleRanges(ranges.toArray(new StyleRange[0]));

		}

		invokeReloadCompare(careOffSetComp, reloadDistance);

	}

	private void invokeReloadCompare(int caretOffset, int reloadDistance) {

		// this method gets All Annotations that are visible atm
		visibleMapComp.clear();

		widgetCompare.setStyleRanges(0, widgetCompare.getText().length(), null, null);

		// create a dummy annotation
		Type topAnno = typeSystem.getType("uima.tcas.Annotation");
		// AnnotationFS dummyAnno = casGold.createAnnotation(topAnno, endGold,
		// from);
		AnnotationFS dummyAnno = casGold.createAnnotation(topAnno, 0, widgetCompare.getText().length());

		// for each visible AnnotationType ...
		for (TypeStyleDataStructure ds : styleStructGold.getStylesFromLowestLayerToHighest()) {
			if (styleStructGold.getStyleMap().get(ds) == false) {
				continue;
			}

			// Test=====
			AnnotationIndex<AnnotationFS> annotationIndex = casCompare.getAnnotationIndex(ds.getAnnotationType());
			FSIterator<AnnotationFS> subiterator = annotationIndex.subiterator(dummyAnno);

			List<StyleRange> ranges = new ArrayList<StyleRange>();
			while (subiterator.hasNext()) {
				AnnotationFS anno = subiterator.next();

				// delegate to the Styledrawing object in the AnnoStyles which
				// are part of a TSDS
				IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

				if (drawingStrategy instanceof BackgroundDrawingStrategy) {
					// draw this here cause it doesnt need to be updated
					drawingStrategy.drawAnnotation(ds, anno, this.widgetCompare, new GC(this.widgetCompare));
				}

				else if (drawingStrategy instanceof DualEditorDrawingStrategy) {

					int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widgetCompare);
					// The index of the first (possibly only partially)
					// visible line of the widget
					int topIndex = JFaceTextUtil.getPartialTopIndex(widgetCompare);

					int actualIndex = widgetCompare.getLineAtOffset(anno.getEnd());
					if (actualIndex > topIndex || actualIndex < bottomIndex) {

						DualEditorDrawingStrategy strat = (DualEditorDrawingStrategy) drawingStrategy;

						Color color = strat.determineColor(anno, widgetCompare);

						if (color != null) {
							// it should be drawn

							StyleRange styleRange = new StyleRange();
							styleRange.start = anno.getBegin();
							styleRange.length = anno.getEnd() - anno.getBegin();
							styleRange.background = color;

							ranges.add(styleRange);

						}
						// draw the text updates here
						visibleMapComp.put(anno, ds);

					}
				}

				else {
					visibleMapComp.put(anno, ds);
				}

			}

			widgetCompare.setStyleRanges(ranges.toArray(new StyleRange[0]));

		}

	}

	public void highlightRange(int beg, int len, boolean visible) {
		widgetGold.setSelectionRange(beg, len);

		// update
		if (textSelectionGold != null) {
			this.textSelectionGold.x = beg;
			this.textSelectionGold.y = beg + len;
		}

		if (visible) {
			widgetGold.showSelection();
		}

	}

	public void changeLayer(Type t, int layer) {
		TypeStyleDataStructure ds = getDSFromType(t);

		ds.setLayer(layer);

		// layer changed
		layerChanged(ds);

	}

	// visibility change

	public void changeVisibility(Type t, boolean isVisible) {
		this.styleStructGold.changeVisibilityOfType(t, isVisible);

		visibilityChanged(getDSFromType(t));
	}

	// change style

	public void changeStyle(Type t, AnnoStyle style) {

		TypeStyleDataStructure ds = getDSFromType(t);
		ds.setStyles(style);

		styleChanged(ds);

	}

	public void visibilityChanged(TypeStyleDataStructure tsds) {

		broker.send(IAnnotationEditorEvents.EDITOR_VISIBILITY_CHANGED, tsds);
		invokeReload(widgetGold.getCaretOffset(), widgetCompare.getCaretOffset(), 2000);
		// Dirty Diana!!!
		dirty.setDirty(true);
	}

	public void allStylesInvisible() {

		// iterate over all types and set them to not visible

		for (TypeStyleDataStructure tsds : this.styleStructGold.getStyleMap().keySet()) {
			this.styleStructGold.getStyleMap().put(tsds, false);
		}
	}

	public StyleDataStructure getStyleStruct() {
		return styleStructGold;
	}

	public void setStyleStruct(StyleDataStructure styleStruct) {
		this.styleStructGold = styleStruct;
	}

	public void styleChanged(TypeStyleDataStructure tsds) {

		invokeReload(widgetGold.getCaretOffset(), widgetCompare.getCaretOffset(), 2000);
	}

	public void layerChanged(TypeStyleDataStructure tsds) {

		broker.send(IAnnotationEditorEvents.EDITOR_LAYER_CHANGED, tsds);

		invokeReload(widgetGold.getCaretOffset(), widgetCompare.getCaretOffset(), 2000);
		// Dirty Diana!!!
		dirty.setDirty(true);
	}

	// ============================helper

	public HashSet<Type> getVisibleTypes() {
		return this.styleStructGold.getVisibleTypes();
	}

	public int getlayerOfType(Type t) {
		return this.styleStructGold.getLayerNumber(t);
	}

	public Composite getParent() {
		return this.composite;
	}

	public void setParent(Composite parent) {
		this.parent = parent;
	}

	public List<AnnotationFS> getActualSelection() {
		return this.actualSelectionGold;

	}

	public Point getActualSelectedText() {
		return this.textSelectionGold;
	}

	private void initStyleMap(Composite parent) {
		// init StyleMap
		HashMap<TypeStyleDataStructure, Boolean> map = new HashMap<TypeStyleDataStructure, Boolean>();

		Type topAnno = typeSystem.getType("uima.tcas.Annotation");

		Color actual = this.DEFAULT_COLOR;
		for (Type next : typeSystem.getProperlySubsumedTypes(topAnno)) {

			// add to the color

			StyleRange styleRange = new StyleRange();
			styleRange.background = actual;

			// update Color - - - per default the Colors are calculated and only
			// the background is
			// higlighted
			int r = (actual.getRed() + COLORSTEP) % 256;
			int g = (actual.getGreen() - COLORSTEP) % 256;
			int b = (actual.getBlue() - COLORSTEP / 2) % 256;
			if (g < 0) {
				g += 256;
			}
			if (b < 0) {
				b += 256;
			}
			actual = new Color(composite.getDisplay(), r, g, b);
			// default is backgroundDrawingStrategy

			AnnoStyle style = new AnnoStyle(new BackgroundDrawingStrategy(styleRange), styleRange);
			// AnnoStyle style = new AnnoStyle(strat, styleRange);
			TypeStyleDataStructure asds = new TypeStyleDataStructure(next, style, 0);

			map.put(asds, false);

		}
		// create StyleDataStructure
		this.styleStructGold = new StyleDataStructure(map);

	}

	private void initStyle() {

		StyleRange styleRange = new StyleRange();
		styleRange.start = 0;
		styleRange.length = widgetGold.getText().length();
		widgetGold.setStyleRange(styleRange);

		Font initialFont = widgetGold.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(14);
		}
		Font newFont = new Font(composite.getDisplay(), fontData);
		widgetGold.setFont(newFont);
		widgetCompare.setFont(newFont);
	}

	public CAS getCas() {
		return casGold;
	}

	private void initializeMenu() {

		// =====MENU======
		final Menu menu = new Menu(widgetGold);
		widgetGold.setMenu(menu);

		// Visible Annotations
		MenuItem visibleAnnos = new MenuItem(menu, SWT.CASCADE);
		visibleAnnos.setText("Visible Annotations");

		// List All Annotations in this Menu
		final Menu visibleMenu = new Menu(menu);
		visibleAnnos.setMenu(visibleMenu);

		for (TypeStyleDataStructure struct : this.styleStructGold.getStylesFromLowestLayerToHighest()) {
			Type t = struct.getAnnotationType();
			MenuItem item = new MenuItem(visibleMenu, SWT.CHECK);
			item.setText(t.getName());
			item.addSelectionListener(new SelectionListener() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					String code = e.getSource().toString();

					code = code.split(" ")[1].substring(1, code.split(" ")[1].length() - 1);
					TypeStyleDataStructure ds = getDSFromType(typeSystem.getType(code));

					if (ds != null) {

						// change visibility
						Boolean struct = styleStructGold.getStyleMap().get(ds);
						styleStructGold.addTypeSytle(ds, !struct);

						// let listener know
						visibilityChanged(ds);
					}

				}

				@Override
				public void widgetDefaultSelected(SelectionEvent e) {

				}
			});
		}

		// Modus

		MenuItem modusMenu = new MenuItem(menu, SWT.CASCADE);
		modusMenu.setText("Actual Modus");

		// List All Annotations in this Menu
		final Menu modusSubMenu = new Menu(menu);
		modusMenu.setMenu(modusSubMenu);

		for (TypeStyleDataStructure struct : this.styleStructGold.getStylesFromLowestLayerToHighest()) {
			Type t = struct.getAnnotationType();
			MenuItem item = new MenuItem(modusSubMenu, SWT.CHECK);
			item.setText(t.getName());
			item.addSelectionListener(new SelectionListener() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					String code = e.getSource().toString();

					// Deactivate the current mode
					for (MenuItem item : modusSubMenu.getItems()) {
						if (!item.toString().equals(e.getSource().toString())) {
							// dont display the modus before
							if (item.getSelection() == true) {
								// deactivate
								String code2 = item.toString().split(" ")[1].substring(1,
										item.toString().split(" ")[1].length() - 1);
								TypeStyleDataStructure ds = getDSFromType(typeSystem.getType(code2));

								if (styleStructGold.getStyleMap().get(ds) == true) {
									styleStructGold.changeVisibilityOfType(ds);
								}

							}
							item.setSelection(false);
						}
					}

					// activate the new Mode
					code = code.split(" ")[1].substring(1, code.split(" ")[1].length() - 1);
					TypeStyleDataStructure ds = getDSFromType(typeSystem.getType(code));

				}

				@Override
				public void widgetDefaultSelected(SelectionEvent e) {

				}
			});
		}

		// StyleConfiguration
		MenuItem styleMenu = new MenuItem(menu, SWT.NONE);
		styleMenu.setText("Configurate Styles");

		// per adapter
		// styleMenu.addSelectionListener(selectionListener);

		// Invoke Reload

		MenuItem invokeReload = new MenuItem(menu, SWT.NONE);
		invokeReload.setText("Invoke Reload");
		invokeReload.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				invokeReload(widgetGold.getCaretOffset(), widgetCompare.getCaretOffset(), 2000);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

			}
		});

		MenuItem fontChangingEntry = new MenuItem(menu, SWT.None);
		fontChangingEntry.setText("Change Font");

		// fontChangingEntry.addSelectionListener(selectionListener);

		// search dialog

		// StyleConfiguration
		MenuItem searchMenu = new MenuItem(menu, SWT.NONE);
		searchMenu.setText("TextSearch...");

		// per adapter
		// searchMenu.addSelectionListener(selectionListener);

	}

	public StyledText getWidget() {
		return widgetGold;
	}

	@PreDestroy
	public void preDestroy() {

		// TODO Your code here

		broker.send(IAnnotationEditorEvents.DUALEDITOR_CLOSED_EVENT, "");
	}

	@Focus
	public void onFocus() {
		// TODO Your code here

	}

	@Persist
	public void save() {

		System.out.println("No Save in InterAnnotatorAgreement Perspective");
	}

	@Override
	public void coupleToPerspective() {

		System.out.println("Couple DualEditor");
		// subscribe to events in the beginning this editor has to listen to the
		// project explorer
		// MasterPart.registerForEvent(AnnotatorAgreementAnnotationEditor.class,
		// AnnotationEditorEvents.OPEN_XMI_REQUESTED, new
		// DeserializeXMIHandler(this, broker));

		MasterPart.subscribeForEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUALEDITOR_INPUT_CHANGED, dualInputChangedHandler);

		// if a visibility has changed
		MasterPart.subscribeForEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUAL_EDITOR_VISIBILITY_CHANGED, visibilityChangedHandler);
		MasterPart.subscribeForEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUAL_EDITOR_STYLES_CHANGED, strategyChangedHandler);

	}

	@Override
	public void decoupleFromPerspective() {
		System.out.println("Decouple DualEditor");

		// MasterPart.UnsubscribeFromEvent(AnnotatorAgreementAnnotationEditor.class,
		// AnnotationEditorEvents.OPEN_XMI_REQUESTED, new
		// DeserializeXMIHandler(this, broker));

		MasterPart.unsubscribeFromEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUALEDITOR_INPUT_CHANGED, dualInputChangedHandler);

		MasterPart.unsubscribeFromEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUAL_EDITOR_VISIBILITY_CHANGED, visibilityChangedHandler);

		MasterPart.unsubscribeFromEvent(AnnotatorAgreementAnnotationEditor.class,
				IAnnotationEditorEvents.DUAL_EDITOR_STYLES_CHANGED, strategyChangedHandler);

		// reset the expensive objects;
		// casGold = null;
		// openedFileGold = null;

	}
}