package de.uniwue.mk.kall.athen.luceneView;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;

import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.queryparser.classic.ParseException;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.CASRuntimeException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.luceneView.widget.IndexTypeConfigView;
import de.uniwue.mk.kall.athen.luceneView.wizard.IndexCreationWizard;
import de.uniwue.mk.kall.athen.part.editor.DeserializeXMIHandler;
import de.uniwue.mk.kall.lucene.AthenLuceneIndex;
import de.uniwue.mk.kall.lucene.AthenLuceneResult;

public class LuceneView extends Composite{
	private Composite parent;
	private CAS cas;

	private AthenLuceneIndex activeIndex;

	private TableViewer viewer;
	private Label lbIdxPath;
	private Button btAdd;
	private Button btInfo;
	private Text txtQuery;


	public LuceneView(Composite parent, int style) {
		super(parent, style);
		this.parent = parent;
		cas = DeserializeXMIHandler.createCas();
		createControls();
	}

	public void createControls() {
		parent.setLayout(new FillLayout());

		GridLayout mainLayout = new GridLayout(6, false);
		mainLayout.horizontalSpacing = 5;
		mainLayout.verticalSpacing = 5;
		setLayout(mainLayout);

		/*
		 * Buttons
		 */
		Label lbIdxDescr = new Label(this, SWT.SINGLE);
		lbIdxDescr.setText("Index:");

		lbIdxPath = new Label(this, SWT.SINGLE);
		lbIdxPath.setText(" - ");
		{
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			data.grabExcessHorizontalSpace = true;
			lbIdxPath.setLayoutData(data);
		}

		Button btCreate = new Button(this, SWT.PUSH);
		btCreate.setText("Create Index");

		Button btLoad = new Button(this, SWT.PUSH);
		btLoad.setText("Load Index");

		btAdd = new Button(this, SWT.PUSH);
		btAdd.setText("Add Documents");
		btAdd.setEnabled(false);

		btInfo = new Button(this, SWT.PUSH);
		btInfo.setText("Info");
		btInfo.setEnabled(false);

		/*
		 * Query bar
		 */
		txtQuery = new Text(this, SWT.SEARCH);
		txtQuery.setMessage("Enter query");
		{
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			data.horizontalSpan = 6;
			data.grabExcessHorizontalSpace = true;
			txtQuery.setLayoutData(data);
		}

		/*
		 * Result area
		 */
		viewer = new TableViewer(this, SWT.SINGLE);
		viewer.getTable().setHeaderVisible(true);
		{
			GridData data = new GridData(GridData.FILL_BOTH);
			data.horizontalSpan = 6;
			data.grabExcessHorizontalSpace = true;
			data.grabExcessVerticalSpace = true;
			data.widthHint = 1100;
			data.heightHint = 400;
			viewer.getTable().setLayoutData(data);
		}

		TableViewerColumn colFile = new TableViewerColumn(viewer, SWT.LEFT);
		colFile.getColumn().setText("File");
		colFile.getColumn().setWidth(400);
		colFile.setLabelProvider(new ColumnLabelProvider() {

			@Override
			public String getText(Object element) {
				if (element instanceof AthenLuceneResult) {
					return ((AthenLuceneResult) element).getPath();
				}
				return "";
			}

		});

		TableViewerColumn colText = new TableViewerColumn(viewer, SWT.LEFT);
		colText.getColumn().setText("Text");
		colText.getColumn().setWidth(700);
		colText.setLabelProvider(new ColumnLabelProvider() {

			@Override
			public String getText(Object element) {
				if (element instanceof AthenLuceneResult) {
					return ((AthenLuceneResult) element).getText();
				}
				return "";
			}

		});

		viewer.setContentProvider(new ArrayContentProvider());

		ArrayList<AthenLuceneResult> list = new ArrayList<>();
		viewer.setInput(list);


		/*
		 * ActionListeners
		 */
		btCreate.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				createNewIndex();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		btLoad.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				loadIndex();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		btAdd.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				addDocuments();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		btInfo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				showTypeInfo();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}
		});

		txtQuery.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				runQuery();
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				runQuery();
			}
		});
	}

	public void createNewIndex() {
		IndexCreationWizard wizard = new IndexCreationWizard(cas.getTypeSystem());
		WizardDialog dia = new WizardDialog(getShell(), wizard);
		int open = dia.open();
		if (open == Window.OK) {
			System.out.println("index created: " + wizard.getIndex().getDirectory().getAbsolutePath());
			setIndex(wizard.getIndex());
			addDocuments(wizard.getDocumentsFolder());
		}
	}

	public void loadIndex() {
		DirectoryDialog dialog = new DirectoryDialog(getShell());
		String open = dialog.open();
		if (open != null) {
			try {
				AthenLuceneIndex index = AthenLuceneIndex.loadIndex(new File(open), 
						cas.getTypeSystem());
				if (index != null) {
					setIndex(index);
				}
			} catch (CASRuntimeException | IOException e) {
				e.printStackTrace();
			}
		}
	}

	public void addDocuments() {
		if (activeIndex == null) {
			MessageDialog.openError(getShell(), "No active Index", 
					"You have to create or load an index before adding documents to it!");
			return;
		}
		DirectoryDialog dialog = new DirectoryDialog(getShell());
		String directory = dialog.open();
		if (directory == null) {
			return;
		}
		File dir = new File(directory);
		addDocuments(dir);
	}

	private void addDocuments(File directory) {
		File[] files = directory.listFiles(file -> file.isFile() && file.getName().endsWith("xmi"));

		IRunnableWithProgress runnable = new IRunnableWithProgress() {

			@Override
			public void run(IProgressMonitor monitor) 
					throws InvocationTargetException, InterruptedException {
				try {
					monitor.beginTask("Adding documents to the index", files.length + 2);
					monitor.subTask("Opening index writer...");
					IndexWriter writer = activeIndex.openWriter();
					monitor.worked(1);
					for (int i = 0; i < files.length; i++) {
						monitor.subTask("Adding document " + (i + 1) + " of " + files.length + 
								" (might take a while)...");
						activeIndex.addDocument(writer, files[i]);
						monitor.worked(1);
					}
					monitor.subTask("Closing index writer...");
					activeIndex.closeWriter(writer);
					monitor.worked(1);
					monitor.done();
				} catch (IOException e) {
					e.printStackTrace();
				}

			}

		};

		ProgressMonitorDialog pmDia = new ProgressMonitorDialog(getShell());
		try {
			pmDia.run(true, false, runnable);
		} catch (InvocationTargetException | InterruptedException e) {
			MessageDialog.openError(getShell(), "Processing aborted", 
					"An exception occured during processing:\\n" + e.getMessage());
			e.printStackTrace();
		}
	}

	public void showTypeInfo() {
		if (activeIndex == null) {
			MessageDialog.openError(getShell(), "No active Index", 
					"You have to create or load an index first!");
			return;
		}
		Shell shell = new Shell(SWT.CLOSE | SWT.TITLE | SWT.RESIZE | SWT.MAX | 
				SWT.MIN | SWT.MODELESS);
		shell.setText("Index Information");
		shell.setLayout(new GridLayout());
		new IndexTypeConfigView(shell, SWT.NONE, cas.getTypeSystem(), 
				activeIndex.getConfiguration());
		shell.pack();
		shell.open();
	}

	private void runQuery() {
		if (activeIndex == null) {
			MessageDialog.openError(getShell(), "Query not possible", 
					"You have to create or open an index before running a query!");
			return;
		}
		try {
			ArrayList<AthenLuceneResult> results = activeIndex.query(txtQuery.getText());
			viewer.setInput(results);
			viewer.refresh();
		} catch (IOException | ParseException e) {
			e.printStackTrace();
		}
	}

	private void setIndex(AthenLuceneIndex index) {
		activeIndex = index;
		lbIdxPath.setText(activeIndex != null ? activeIndex.getDirectory().getAbsolutePath() : " - ");
		btAdd.setEnabled(activeIndex != null);
		btInfo.setEnabled(activeIndex != null);
	}
}
