package de.uniwue.mk.kall.athen.luceneView.widget;

import java.util.List;
import java.util.Map;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.uniwue.mk.kall.lucene.IndexTypeConfiguration;

public class IndexTypeConfigView extends Composite {

	public IndexTypeConfigView(Composite parent, int style, TypeSystem ts, 
			IndexTypeConfiguration config) {
		super(parent, style);
		parent.setLayout(new GridLayout());
		createControls(ts, config);
	}


	public void createControls(TypeSystem ts, IndexTypeConfiguration config) {
		GridLayout layout = new GridLayout(2, false);
		layout.horizontalSpacing = 10;
		layout.verticalSpacing = 7;
		setLayout(layout);

		Label lbToken = new Label(this, SWT.SINGLE);
		lbToken.setText("Token Type:");
		{
			GridData data = new GridData();
			data.horizontalIndent = 5;
			lbToken.setLayoutData(data);
		}

		Label lbTokenName = new Label(this, SWT.SINGLE);
		lbTokenName.setText(config.getTokenType().getShortName());

		Label lbDocpart = new Label(this, SWT.SINGLE);
		lbDocpart.setText("DocumentPart Type:");
		{
			GridData data = new GridData();
			data.horizontalIndent = 5;
			lbDocpart.setLayoutData(data);
		}

		Label lbDocpartName = new Label(this, SWT.SINGLE);
		lbDocpartName.setText(config.getDocumentPartType().getShortName());

		TypeBrowserWidget widget = new TypeBrowserWidget(this, false);
		widget.setTypeSystem(ts);
		{
			GridData data = new GridData(GridData.FILL_BOTH);
			data.horizontalSpan = 2;
			widget.setLayoutData(data);
		}

		widget.getViewer().expandAll();
		widget.getViewer().addFilter(new ViewerFilter() {

			@Override
			public boolean select(Viewer viewer, Object parentElement, Object element) {
				Map<Type, List<Feature>> map = config.getTypesAndFeatures();
				if (element instanceof Type) {
					return map.keySet().contains(element);
				}
				return map.keySet().contains(parentElement) && 
						map.get(parentElement).contains(element);
			}
		});

		Label lbNote = new Label(this, SWT.WRAP);
		lbNote.setText("Note: Use the types' short name for queries (the part after the last '.').");
		{
			GridData data = new GridData(GridData.FILL_HORIZONTAL);
			data.horizontalSpan = 2;
			lbNote.setLayoutData(data);
		}
	}
}
