package de.uniwue.mk.kall.athen.luceneView.wizard;

import java.util.List;
import java.util.Map;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.TreeItem;

import de.uniwue.mk.kall.athen.luceneView.widget.TypeBrowserWidget;

public class IndexCreationPageOne extends WizardPage {
	private Map<Type, List<Feature>> types;
	private TypeBrowserWidget widget;
	private TypeSystem typeSystem;


	protected IndexCreationPageOne(TypeSystem typeSystem, Map<Type, List<Feature>> types) {
		super("Types and Features");
		this.typeSystem = typeSystem;
		this.types = types;
	}


	@Override
	public void createControl(Composite parent) {
		widget = new TypeBrowserWidget(parent, true);
		widget.setTypeSystem(typeSystem);
		setControl(widget);
		setPageComplete(false);

		// page is complete if at least one Type is selected
		widget.getViewer().getTree().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				if (e.item instanceof TreeItem && e.detail == SWT.CHECK) {
					/*
					 * update map of types and features
					 * ("slow" but prevents inconsistencies with the tree's selection model)
					 */
					types.clear();
					types.putAll(widget.getSelectedElements());
					if (e.item.getData() instanceof Type && ((TreeItem) e.item).getChecked()) {
						setPageComplete(true);
					} else {
						setPageComplete(checkPageCompletion());
					}
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});
	}

	private boolean checkPageCompletion() {
		return widget.getSelectedElements().keySet().size() > 0;
	}

	public Map<Type, List<Feature>> getSelectedTypesAndFeatures() {
		return widget.getSelectedElements();
	}
}
