package de.uniwue.mk.kall.athen.luceneView.wizard;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

public class IndexCreationPageTwo extends WizardPage {
	private Map<Type, List<Feature>> map;
	private Map<String, Type> types;

	private Combo cbToken;
	private Combo cbDocpart;
	private Text tfDocs;
	private Text tfIndex;


	protected IndexCreationPageTwo(Map<Type, List<Feature>> map) {
		super("Token-Type and Folders");
		this.map = map;
		types = new HashMap<>();
		map.keySet().forEach(item -> types.put(item.getName(), item));
	}

	@Override
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout(3, false);
		layout.horizontalSpacing = 20;
		layout.verticalSpacing = 10;
		composite.setLayout(layout);

		// Token-Type
		String msgToken = "Select a Type as Token-Type, e.g. POS-Tag";
		Label lbToken = new Label(composite, SWT.NONE);
		lbToken.setText("Select Token-Type:");
		lbToken.setToolTipText(msgToken);

		cbToken = new Combo(composite, SWT.READ_ONLY);
		cbToken.setItems(types.keySet().toArray(new String[0]));
		cbToken.setToolTipText(msgToken);

		GridData gdCombo = new GridData(GridData.FILL_HORIZONTAL);
		gdCombo.horizontalSpan = 2;
		cbToken.setLayoutData(gdCombo);

		// DocumentPart-Type
		String msgDocpart = "Select a Type that partitions the document, e.g. Sentence";
		Label lbDocpart = new Label(composite, SWT.NONE);
		lbDocpart.setText("Select DocumentPart-Type:");
		lbDocpart.setToolTipText(msgDocpart);

		cbDocpart = new Combo(composite, SWT.READ_ONLY);
		cbDocpart.setItems(types.keySet().toArray(new String[0]));
		cbDocpart.setToolTipText(msgDocpart);

		GridData gdDocpart = new GridData(GridData.FILL_HORIZONTAL);
		gdDocpart.horizontalSpan = 2;
		cbDocpart.setLayoutData(gdDocpart);

		// Document Folder
		String msgDocFolder = "Choose the folder that contains the documents "
				+ "which should be added to the index";
		Label lbDocs = new Label(composite, SWT.NONE);
		lbDocs.setText("Choose Document Folder:");
		lbDocs.setToolTipText(msgDocFolder);

		tfDocs = new Text(composite, SWT.BORDER);
		tfDocs.setEditable(false);
		tfDocs.setToolTipText(msgDocFolder);

		tfDocs.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button btDocs = new Button(composite, SWT.PUSH);
		btDocs.setText("Browse...");

		// Index Folder
		String msgIndexFolder = "Choose the folder where the index should be stored";
		Label lbIndex = new Label(composite, SWT.NONE);
		lbIndex.setText("Choose Index Folder:");
		lbIndex.setToolTipText(msgIndexFolder);

		tfIndex = new Text(composite, SWT.BORDER);
		tfIndex.setEditable(false);
		tfIndex.setToolTipText(msgIndexFolder);

		tfIndex.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button btIndex = new Button(composite, SWT.PUSH);
		btIndex.setText("Browse...");

		DirectoryDialog dialogInput = new DirectoryDialog(composite.getShell());
		dialogInput.setMessage(msgDocFolder);

		DirectoryDialog dialogIndex = new DirectoryDialog(composite.getShell());
		dialogIndex.setMessage(msgIndexFolder);

		cbToken.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				setPageComplete(checkPageCompletion());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		cbDocpart.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				setPageComplete(checkPageCompletion());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		btDocs.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				//				if (!tfDocs.getText().isEmpty()) {
				//					dialogInput.setFilterPath(tfDocs.getText());
				//				}
				String open = dialogInput.open();
				if (open != null) {
					tfDocs.setText(open);
				}
				setPageComplete(checkPageCompletion());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		btIndex.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				//				if (!tfIndex.getText().isEmpty()) {
				//					dialogIndex.setFilterPath(tfDocs.getText());
				//				}
				String open = dialogIndex.open();
				if (open != null) {
					tfIndex.setText(open);
				}
				setPageComplete(checkPageCompletion());
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		setControl(composite);
		setPageComplete(false);
	}

	private boolean checkPageCompletion() {
		// check whether a token-type and a documentpart-type have been selected
		if (cbToken.getSelectionIndex() < 0 || cbDocpart.getSelectionIndex() < 0) {
			return false;
		}
		// check whether document and index folders are set
		if (tfDocs.getText().isEmpty() || tfIndex.getText().isEmpty()) {
			return false;
		}
		try {
			File fDocs = new File(tfDocs.getText());
			File fIndex = new File(tfIndex.getText());
			return fDocs.isDirectory() && fIndex.isDirectory();
		} catch (Exception e) {
			return false;
		}
	}

	@Override
	public void setVisible(boolean visible) {
		if (visible) {
			/*
			 *  types and features from page one might have changed => update
			 *  try to keep selection
			 */
			String tokenSelection = "";
			if (cbToken.getSelectionIndex() >= 0) {
				tokenSelection = cbToken.getItem(cbToken.getSelectionIndex());
			}
			String docpartSelection = "";
			if (cbDocpart.getSelectionIndex() >= 0) {
				docpartSelection = cbDocpart.getItem(cbDocpart.getSelectionIndex());
			}
			types.clear();
			map.keySet().forEach(item -> types.put(item.getName(), item));
			ArrayList<String> list = new ArrayList<>(types.keySet());
			Collections.sort(list);
			cbToken.setItems(list.toArray(new String[0]));
			cbToken.select(cbToken.indexOf(tokenSelection));
			cbDocpart.setItems(list.toArray(new String[0]));
			cbDocpart.select(cbDocpart.indexOf(docpartSelection));
		}
		super.setVisible(visible);
	}

	public Type getTokenType() {
		if (cbToken.getSelectionIndex() < 0) {
			return null;
		}
		String tokenName = cbToken.getItem(cbToken.getSelectionIndex());
		return types.get(tokenName);
	}

	public Type getDocumentPartType() {
		if (cbDocpart.getSelectionIndex() < 0) {
			return null;
		}
		String docpartName = cbDocpart.getItem(cbDocpart.getSelectionIndex());
		return types.get(docpartName);
	}

	public String getDocumentsFolder() {
		return tfDocs.getText();
	}

	public String getIndexFolder() {
		return tfIndex.getText();
	}
}
