package de.uniwue.mk.kall.athen.luceneView.wizard;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.eclipse.jface.wizard.Wizard;

import de.uniwue.mk.kall.lucene.AthenLuceneIndex;
import de.uniwue.mk.kall.lucene.IndexTypeConfiguration;

public class IndexCreationWizard extends Wizard {
	private IndexCreationPageOne pageOne;
	private IndexCreationPageTwo pageTwo;

	private AthenLuceneIndex index;
	private File documentsFolder;	


	public IndexCreationWizard(TypeSystem typeSystem) {
		super();
		Map<Type, List<Feature>> types = new HashMap<>();
		pageOne = new IndexCreationPageOne(typeSystem, types);
		pageTwo = new IndexCreationPageTwo(types);
	}


	@Override
	public void addPages() {
		addPage(pageOne);
		addPage(pageTwo);
	}

	@Override
	public boolean performFinish() {
		Map<Type, List<Feature>> typesAndFeatures = pageOne.getSelectedTypesAndFeatures();
		String indexFolder = pageTwo.getIndexFolder();
		documentsFolder = new File(pageTwo.getDocumentsFolder());
		Type tokenType = pageTwo.getTokenType();
		Type docpartType = pageTwo.getDocumentPartType();
		IndexTypeConfiguration config = new IndexTypeConfiguration(typesAndFeatures, 
				tokenType, docpartType);
		try {
			index = AthenLuceneIndex.createIndex(new File(indexFolder), config);
		} catch (IOException e) {
			e.printStackTrace();
			return false;
		}
		System.out.println("wizard success");
		return true;
	}

	public AthenLuceneIndex getIndex() {
		return index;
	}

	public File getDocumentsFolder() {
		return documentsFolder;
	}

}
