package de.uniwue.mk.kall.athen.part.editor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.impl.XmiCasDeserializer;
import org.apache.uima.fit.factory.TypeSystemDescriptionFactory;
import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.metadata.TypeSystemDescription;
import org.apache.uima.util.CasCreationUtils;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;
import org.xml.sax.SAXException;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.dialog.EditorPreferencesDialog;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorPart;
import de.uniwue.mk.kall.athen.widget.editor.CASEditorInput;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class DeserializeXMIHandler implements EventHandler {

	private AnnotationEditorPart editor;

	private static IEventBroker broker;

	private File currentlyOpenedFile;

	public DeserializeXMIHandler(AnnotationEditorPart annotationEditorPart, IEventBroker broker,
			EPartService partService) {

		this.editor = annotationEditorPart;
		this.broker = broker;

		broker.subscribe(IAnnotationEditorEvents.FILE_MODIFIED, this);
	}

	@Override
	public void handleEvent(Event e) {

		if (e.getTopic().equals(IAnnotationEditorEvents.FILE_MODIFIED)) {
			Object property = e.getProperty(IEventBroker.DATA);
			File f = (File) property;
			// this is only important if old and new file are the same - nothing
			// to do if they arent the same
			if (this.currentlyOpenedFile != null && !this.currentlyOpenedFile.equals(f)) {
				return;
			}

			if (this.currentlyOpenedFile == null) {
				return;
			}

			// do not reload a saved document
			return;

		}

		Object property = e.getProperty(IEventBroker.DATA);

		if (property instanceof File) {
			File f = (File) property;

			this.currentlyOpenedFile = f;

			// TODO load tsd from preferences

			CAS cas = createCas();

			try {
				// delay to assure an xmi is serilaizied before
				// deserializing again
				Thread.sleep(500);

			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			// TODO save the lenient loading in preferences

			try {

				if (editor.getParent().isDisposed()) {
					// open the editor if disposed TODO

				}

				FileInputStream fileInputStream = new FileInputStream(f);
				new ProgressMonitorDialog(Display.getCurrent().getActiveShell()).run(true, true,
						new XMIDeserializationRunnables(fileInputStream, cas, true));

				fileInputStream.close();
			} catch (InvocationTargetException | FileNotFoundException | InterruptedException e1) {
				e1.printStackTrace();
			} catch (IOException e1) {
				e1.printStackTrace();
			}
			editor.onCASChanged(new CASEditorInput(cas, f));

		}
	}

	public static CAS createCas() {
		String tsdPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.EDITOR_TSD_PATH, "null");

		if (tsdPref.equals("null")) {

			// open the filechooser
			EditorPreferencesDialog dlg = new EditorPreferencesDialog(Display.getCurrent().getActiveShell(), broker);

			int selectedTypeSystem = dlg.open();

			// save to preferences

			if (selectedTypeSystem == 0) {
				// success

				HashMap<String, String> preferences = dlg.getPreferences();

				for (String s : preferences.keySet()) {
					ApplicationUtil.saveToPreferences(s, preferences.get(s));
				}

			}
			// ApplicationUtil.saveToPreferences(ApplicationPreferences.EDITOR_TSD_PATH,
			// selectedTypeSystem);
		}

		File tsdFile = new File(tsdPref);
		TypeSystemDescription tsd = null;
		if (tsdFile.exists()) {
			tsd = TypeSystemDescriptionFactory.createTypeSystemDescriptionFromPath(tsdFile.toURI().toString());

		}

		else {

			// TODO no longer needed!
			// try {
			// tsd = XmiToTypeSystemGenerator.generateTypeSystemFromXmi(f);
			// } catch (ResourceInitializationException |
			// ParserConfigurationException | SAXException | IOException e1) {
			// // TODO Auto-generated catch block
			// e1.printStackTrace();
			// }

		}

		CAS cas = null;

		// also add the default typesystem specified in the util project
		TypeSystemDescription tsdNew = Util_impl.mergeToDefaultTypesystems(tsd);
		try {
			cas = CasCreationUtils.createCas(tsdNew, null, null);
		} catch (ResourceInitializationException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}
		return cas;
	}

	class XMIDeserializationRunnables implements IRunnableWithProgress {

		private boolean lenient = false;
		private CAS cas;
		private FileInputStream is;

		/**
		 * LongRunningOperation constructor
		 * 
		 * @param cas
		 * @param fileInputStream
		 * 
		 * @param indeterminate
		 *            whether the animation is unknown
		 */
		public XMIDeserializationRunnables(FileInputStream fileInputStream, CAS cas, boolean lenient) {

			this.is = fileInputStream;
			this.cas = cas;
			this.lenient = lenient;
		}

		@Override
		public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
			monitor.beginTask("Deserializing XMI-Document this might take a while", 1000);
			try {
				XmiCasDeserializer.deserialize(this.is, this.cas, this.lenient);
			} catch (SAXException | IOException e) {
				e.printStackTrace();
			}
			monitor.done();
			if (monitor.isCanceled())
				throw new InterruptedException("XMI Deserializing was cancelled");

		}
	}

}
