package de.uniwue.mk.kall.athen.part.editor.dialog;

import java.util.HashSet;

import org.apache.uima.cas.Type;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeFilter;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class AnnotationStyleEditingDialog extends Dialog {

	private AnnotationEditorWidget editor;

	private TableViewer viewer;

	private TypeFilter filter;

	private Composite parent;

	public AnnotationStyleEditingDialog(Shell parent, AnnotationEditorWidget editor) {
		super(parent);

		this.editor = editor;
	}

	@Override
	protected Control createDialogArea(Composite parent) {

		Composite composite = (Composite) super.createDialogArea(parent);
		this.parent = parent;

		composite.getShell().setText("Annotation Style Editing Dialog");

		composite.setLayout(new GridLayout(1, true));

		draw(composite); // Contents of Dialog

		return composite;
	}

	private void draw(Composite parent) {

		// at first a Tableviewer with the list of Types

		Composite compp = new Composite(parent, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		Label searchLabel = new Label(compp, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compp, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		viewer = new TableViewer(compp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER);

		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.heightHint = 600;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		createColumns(viewer);
		// content
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setInput(editor.getStyleStruct().getStyleMap().keySet());
		// filter
		filter = new TypeFilter();
		viewer.addFilter(filter);

	}

	private void createColumns(final TableViewer viewer) {

		// Typename
		TableViewerColumn colName = createTableViewerColumn("Type", 300, 1);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				TypeStyleDataStructure t = (TypeStyleDataStructure) element;
				return t.getAnnotationType().toString();

			}

		});

		// Layer
		TableViewerColumn colLayer = createTableViewerColumn("Layer", 40, 0);
		colLayer.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				TypeStyleDataStructure t = (TypeStyleDataStructure) element;
				return String.valueOf(t.getLayer());

			}

		});

		colLayer.setEditingSupport(new LayerEditingSupport(viewer, editor));

		// Style
		TableViewerColumn colStyle = createTableViewerColumn("Style", 40, 2);
		colStyle.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				TypeStyleDataStructure t = (TypeStyleDataStructure) element;
				return t.getStyle().getStyle().toString();

			}

		});

		// Visible
		TableViewerColumn colVisible = createTableViewerColumn("Visible", 60, 3);
		colVisible.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				TypeStyleDataStructure tsds = (TypeStyleDataStructure) element;
				Type t = tsds.getAnnotationType();
				HashSet<Type> visibleTypes = editor.getVisibleTypes();
				if (visibleTypes.contains(t)) {
					return "true";
				}

				return "false";

			}

		});

		colVisible.getViewer().getControl().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {

			}

			@Override
			public void mouseDown(MouseEvent e) {

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				// get the coloumn of that click
				int x = e.x;
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

				int beg = 0;
				int colNr = 0;
				for (TableColumn col : viewer.getTable().getColumns()) {
					if (x >= beg && x <= beg + col.getWidth()) {
						break;
					}
					colNr++;
					beg += col.getWidth();
				}

				if (colNr == 3) {

					// change visibility
					TypeStyleDataStructure tsds = (TypeStyleDataStructure) selection.getFirstElement();
					Boolean boolean1 = editor.getStyleStruct().getStyleMap().get(tsds);

					editor.getStyleStruct().getStyleMap().put(tsds, !boolean1);

					// Visibility changed
					editor.visibilityChanged(tsds,!boolean1);

					viewer.refresh();
				}

				else if (colNr == 2 || colNr == 0) {
					// style
					// TODO open a window for the styleChanges

					TypeStyleDataStructure tsds = (TypeStyleDataStructure) selection.getFirstElement();
					StyleChangingDialog dialog = new StyleChangingDialog(parent.getShell(), tsds, editor);

					if (dialog.open() == Window.OK) {
						System.out.println("Yissss");
					}

				}
			}
		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
