package de.uniwue.mk.kall.athen.part.editor.dialog;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.BoxDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.BracketDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.EPaintingStyles;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.UnderlineDrawingStrategy;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class StyleChangingDialog extends Dialog {

	private TypeStyleDataStructure tsds;

	private AnnotationEditorWidget editor;

	public StyleChangingDialog(Shell parent, TypeStyleDataStructure tsds, AnnotationEditorWidget editor) {
		super(parent);
		this.editor = editor;
		this.tsds = tsds;
	}

	@Override
	protected Control createDialogArea(Composite parent) {

		Composite composite = (Composite) super.createDialogArea(parent);

		composite.getShell().setText("Annotation Style Editing Dialog");

		composite.setLayout(new GridLayout(1, true));

		draw(composite); // Contents of Dialog

		return composite;
	}

	private void draw(final Composite parent) {

		// at first a Tableviewer with the list of Types

		Composite comp1 = new Composite(parent, SWT.BORDER);
		comp1.setVisible(true);

		GridLayout layout = new GridLayout();
		layout.numColumns = 1;

		comp1.setLayout(layout);
		// this compositie only gets the Type
		Label typeLabel = new Label(comp1, SWT.NONE);
		typeLabel.setText(tsds.getAnnotationType().toString());

		// new Composite which displays the actual color
		Composite comp2 = new Composite(parent, SWT.BORDER);
		comp2.setVisible(true);

		layout = new GridLayout();
		layout.numColumns = 2;
		comp2.setLayout(layout);
		// first Element is a Label which displays the Color second is a Button
		// to open the dialog
		final Label colorLabel = new Label(comp2, SWT.NONE);
		colorLabel.setText("                              ");
		colorLabel.setBackground(tsds.getStyle().getStyleRange().background);

		Button button = new Button(comp2, SWT.PUSH);
		button.setText("Change Color");

		button.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// open a colorchangedialog
				// Create the color-change dialog
				ColorDialog dlg = new ColorDialog(parent.getShell());

				// Set the selected color in the dialog from
				// user's selected color
				dlg.setRGB(colorLabel.getBackground().getRGB());

				// Change the title bar text
				dlg.setText("Choose a Color");

				// Open the dialog and retrieve the selected color
				RGB rgb = dlg.open();
				if (rgb != null) {
					// Dispose the old color, create the
					// new one, and set into the label

					Color color = new Color(parent.getDisplay(), rgb);
					colorLabel.setBackground(color);

					// update TSDS
					tsds.getStyle().getStyle().background = color;

					// inform listener
					editor.styleChanged(tsds);
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

			}
		});

		// third Composite contains a dropdown with a style
		Composite comp3 = new Composite(parent, SWT.BORDER);
		comp3.setVisible(true);

		comp3.setLayout(layout);
		comp3.setData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		Label styleLabel = new Label(comp3, SWT.NONE);
		styleLabel.setText("Style");

		Combo combo = new Combo(comp3, SWT.NONE);
		EPaintingStyles[] values = EPaintingStyles.values();

		String[] styles = new String[values.length];
		for (int i = 0; i < values.length; i++) {
			styles[i] = String.valueOf(values[i]);

		}
		combo.setItems(styles);

		combo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// BackgroundStyle
				if (e.getSource().toString().contains(String.valueOf(EPaintingStyles.BACKGROUND))) {
					IAnnotationDrawingStrategy strat = new BackgroundDrawingStrategy(tsds.getStyle().getStyle());
					tsds.getStyle().setDrawingStrategy(strat);

				}

				else if (e.getSource().toString().contains(String.valueOf(EPaintingStyles.UNDERLINE))) {
					IAnnotationDrawingStrategy strat = new UnderlineDrawingStrategy(tsds.getStyle().getStyle());
					tsds.getStyle().setDrawingStrategy(strat);
				}

				else if (e.getSource().toString().contains(String.valueOf(EPaintingStyles.BRACKET))) {
					IAnnotationDrawingStrategy strat = new BracketDrawingStrategy(tsds.getStyle().getStyle());
					tsds.getStyle().setDrawingStrategy(strat);
				}

				else if (e.getSource().toString().contains(String.valueOf(EPaintingStyles.BOX))) {
					IAnnotationDrawingStrategy strat = new BoxDrawingStrategy(tsds.getStyle().getStyle());
					tsds.getStyle().setDrawingStrategy(strat);
				}

				// TODO integrate other Styles to be selectable

				// Inform listener
				editor.styleChanged(tsds);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

			}
		});
	}

}
