package de.uniwue.mk.kall.athen.part.editor.dialog;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class UIMAFeatureChangeComposite extends Composite {

	private static final String DATA_FEATURE = "DATA_FEATURE";
	private static final String FEATURE_ANNO_END = "uima.tcas.Annotation:end";
	private static final String FEATURE_ANNO_BEGIN = "uima.tcas.Annotation:begin";
	private static final String FEATURE_ANNO_SOFA = "uima.cas.AnnotationBase:sofa";

	private static final String FEATURE_STRING = "uima.cas.String";
	private AnnotationFS anno;
	private AnnotationEditorWidget editor;

	public UIMAFeatureChangeComposite(Composite parent, int style, AnnotationEditorWidget editor) {
		super(parent, style);

		this.editor = editor;
		initLayout();
	}

	public void setAnnotation(AnnotationFS anno) {

		this.anno = anno;

		initLayout();

	}

	private void initLayout() {

		if (anno == null)
			return;

		this.setLayout(new GridLayout(1, false));
		for (Feature f : anno.getType().getFeatures()) {

			if (f.getName().matches(FEATURE_ANNO_BEGIN + "|" + FEATURE_ANNO_END + "|" + FEATURE_ANNO_SOFA))
				continue;

			if (f.getRange().toString().equals(FEATURE_STRING)) {

				addStringFeatureElements(f);

			}

		}

	}

	private void addStringFeatureElements(Feature f) {
		GridData gdGrabHorizontal = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		Composite stringFeatureComposite = new Composite(this, SWT.NONE);
		stringFeatureComposite.setData(DATA_FEATURE, f);
		stringFeatureComposite.setLayoutData(gdGrabHorizontal);

		stringFeatureComposite.setLayout(new GridLayout(3, false));

		Label labelFeatureName = new Label(stringFeatureComposite, SWT.NONE);
		labelFeatureName.setText(f.getShortName() + ":");

		final Text textNewFeatureValue = new Text(stringFeatureComposite, SWT.BORDER);

		textNewFeatureValue.setLayoutData(gdGrabHorizontal);

		textNewFeatureValue.setText(anno.getFeatureValueAsString(f) == null ? "" : anno.getFeatureValueAsString(f));

		textNewFeatureValue.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				// perform a live update - reload only when the dialog gets
				// closed
				Feature assignedFeature = (Feature) textNewFeatureValue.getParent().getData(DATA_FEATURE);

				anno.setFeatureValueFromString(assignedFeature, textNewFeatureValue.getText());
				editor.changeAnnotation(anno);

			}

		});
	}

}
