package de.uniwue.mk.kall.athen.part.editor.subordinate;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.EditorWarningDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IAnnotationEditorDialog;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public abstract class AEditorSubordinateViewPart implements IPerspectiveDecoupledPart {

	// Handler
	private EventHandler editorAnnotationAddedHandler;
	private EventHandler editorAnnotationRemovedHandler;
	private EventHandler editorAnnotationChangedHandler;
	private EventHandler editorSelectioNChangedHandler;
	private EventHandler editorTextSelectionChangedHandler;
	private EventHandler editorDoubleClickedHandler;
	private EventHandler editorWasScrolledHandler;
	private EventHandler editorCaretChangedHandler;
	private EventHandler editorAnnotationHoveredHandler;
	private EventHandler editorRightClickMenuRequestedHandler;

	private EventHandler editorMouseUpHandler;
	private EventHandler editorMouseDownHandler;

	protected EventHandler onViewStackChangedHandler;

	private EventHandler onEditorInputChangedHandler;

	private EventHandler onEditorClosedHandler;

	private EventHandler editorMouseMovedHandler;

	private EventHandler editorStyleChangedHandler;
	private EventHandler editorKeyPressedHandler;
	// common fields

	protected String viewId;

	protected AnnotationEditorWidget editor;
	public Composite overlay;
	public MPart part;
	protected boolean isSubscribedToEditorEvents = false;
	protected Class<?> class_this;
	protected EPartService partService;
	private IEventBroker broker;

	protected Type editorErrorType;
	protected Feature editorErrorMessageFeature;
	protected Feature editorErrorTypeFeature;

	// for the preferences dialog
	protected Dialog preferencesDialog = null;

	// struct====
	public AEditorSubordinateViewPart() {
		// broker can only be set in the postcontextcreate method!!

		initEditorEventHandler();
	}

	public void setBroker(IEventBroker broker) {
		this.broker = broker;
	}

	private void initEditorEventHandler() {
		editorAnnotationAddedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {

				onAnnotationAdded(arg0);
			}
		};
		editorAnnotationRemovedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onAnnotationRemoved(arg0);
			}
		};

		editorRightClickMenuRequestedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorRightClickMenuRequested(arg0);

			}

		};

		editorKeyPressedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorKeyPressed(arg0);

			}

		};
		editorAnnotationChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onAnnotationChanged(arg0);

			}
		};
		editorSelectioNChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onAnnotationSelectionChanged(arg0);

			}
		};
		editorTextSelectionChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorTextSelectionChanged(arg0);

			}

		};
		editorDoubleClickedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorDoubleClicked(arg0);

			}
		};

		editorAnnotationHoveredHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onAnnotationHovered(arg0);

			}

		};
		editorWasScrolledHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorWasScrolled(arg0);

			}
		};
		editorCaretChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorCaretChanged(arg0);

			}
		};

		editorMouseUpHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorMouseUp(arg0);

			}
		};
		editorMouseDownHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorMouseDown(arg0);

			}
		};

		editorMouseMovedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorMouseMoved(arg0);

			}
		};

		editorStyleChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onEditorStyleChanged(arg0);

			}

		};

		onViewStackChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {
				onViewStackChanged(arg0);

			}
		};

		onEditorInputChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event e) {
				AEditorSubordinateViewPart.this.editor = (AnnotationEditorWidget) e.getProperty(IEventBroker.DATA);
				onEditorInputChanged(editor);

			}
		};

		onEditorClosedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event e) {
				onEditorClosed(e);

			}
		};

	}

	public void onEditorClosed(Event arg0) {

		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications
		standardUnsubscribeFromEditorEvents(class_this);

		this.editor = null;
	}

	public void onEditorInputChanged(AnnotationEditorWidget widget) {

		this.editor = widget;

		MPart findPart = partService.findPart(viewId);

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but unsubscribe the
			// listener if they are
			// remove all listeners

			this.standardUnsubscribeFromEditorEvents(class_this);
			return;
		}

		// subscribe - masterpart guarantees we cannot subscribe twice
		this.standardSubscribeToEditorEvents(class_this);

		// init the required typings and set the preferences
		boolean typesValid = initTypes();
		initStandardTypes();

		while (!typesValid) {

			// if this view doesnt specify a preferences dialog we skip this
			// loop
			if (this.preferencesDialog == null) {
				break;
			}

			// open preference dialogue and ask the user to enter valid types
			int open = this.preferencesDialog.open();

			if (open == 0) {
				// success

				IAnnotationEditorDialog dialog = (IAnnotationEditorDialog) this.preferencesDialog;
				HashMap<String, String> preferences = dialog.getPreferences();

				// save all the preferences to the store
				for (String l : preferences.keySet()) {

					ApplicationUtil.saveToPreferences(l, preferences.get(l));
				}

			}

			else {
				break;
			}

			typesValid = initTypes();
			// TODO dialog code handling
		}

		// update the model of this view
		initModel();

		// clear the view
		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		initLayout(overlay);
		// ====reload=====

		overlay.layout();
		overlay.getParent().layout();
	}

	protected void initStandardTypes() {
		this.editorErrorType = this.editor.getCas().getTypeSystem()
				.getType("de.uniwue.kalimachos.coref.type.athenWarningType");
		this.editorErrorMessageFeature = this.editorErrorType.getFeatureByBaseName("ErrorMessage");
		this.editorErrorTypeFeature = this.editorErrorType.getFeatureByBaseName("ErrorType");

		editor.changeVisibility(editorErrorType, true);

		StyleRange sr = new StyleRange();
		// TODO leak!!
		sr.background = new Color(Display.getCurrent(), new RGB(255, 255, 255));
		AnnoStyle style = new AnnoStyle(new EditorWarningDrawingStrategy(), sr);

		editor.changeStyle(editorErrorType, style);

	}

	public abstract void initModel();

	public abstract boolean initTypes();

	protected abstract void initLayout(Composite parent);

	// defaut implementation, removes all unnecessary listener if not selected
	public void onViewStackChanged(Event arg0) {
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			System.out.println(class_this);
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			standardUnsubscribeFromEditorEvents(class_this);
			this.editor = null;
		}

	}

	public void onEditorStyleChanged(Event arg0) {
		// TODO Auto-generated method stub

	}

	// called when a new annotation was added
	public void onAnnotationAdded(Event arg0) {

	}

	// called when an annotationw as removed
	public void onAnnotationRemoved(Event arg0) {

	}

	public void onAnnotationChanged(Event arg0) {

	}

	public void onAnnotationSelectionChanged(Event arg0) {

	}

	public void onEditorDoubleClicked(Event arg0) {

	}

	public void onEditorWasScrolled(Event arg0) {

	}

	public void onEditorRightClickMenuRequested(Event arg0) {

	}

	public void onEditorCaretChanged(Event arg0) {

	}

	public void onEditorMouseUp(Event arg0) {

	}

	public void onEditorMouseDown(Event arg0) {

	}

	public void onAnnotationHovered(Event arg0) {

	}

	public void onEditorTextSelectionChanged(Event arg0) {

	}

	public void onEditorMouseMoved(Event e) {

	}

	public void onEditorKeyPressed(Event arg0) {

	}

	// eventhandler

	public void standardUnsubscribeFromEditorEvents(Class<?> classToUnsubscribe) {
		isSubscribedToEditorEvents = false;
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_ADDED,
				editorAnnotationAddedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_CHANGED,
				editorAnnotationChangedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_REMOVED,
				editorAnnotationRemovedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_SELECTION_CHANGED,
				editorSelectioNChangedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_TEXTSELECTION_CHANGED,
				editorTextSelectionChangedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_DOUBLECLICKED,
				editorDoubleClickedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_WAS_SCROLLED,
				editorWasScrolledHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEUP,
				editorMouseUpHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEDOWN,
				editorMouseDownHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_MOUSE_MOVED,
				editorMouseMovedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_STYLE_CHANGED,
				editorStyleChangedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_HOVERED,
				editorAnnotationHoveredHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_KEY_PRESSED,
				editorKeyPressedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.RIGHTCLICK_MENU_REQUESTED,
				editorRightClickMenuRequestedHandler);

	}

	// Same as below but with editor input changed and editor closed
	public void standardSubscribeToEditorDocumentChangedEvents(Class<?> classToSubscribe) {

		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputChangedHandler);

		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		if (this.editor != null) {
			standardSubscribeToEditorEvents(classToSubscribe);
		}

	}

	public void standardUnsubscribeFromEditorDocumentChangedEvents(Class<?> classToUnsubscribe) {

		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputChangedHandler);

		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedHandler);
		MasterPart.unsubscribeFromEvent(classToUnsubscribe, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		if (this.editor != null) {
			standardUnsubscribeFromEditorEvents(classToUnsubscribe);
		}

	}

	public void standardSubscribeToEditorEvents(Class<?> classToSubscribe) {

		isSubscribedToEditorEvents = true;
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_ADDED,
				editorAnnotationAddedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_CHANGED,
				editorAnnotationChangedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_REMOVED,
				editorAnnotationRemovedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_SELECTION_CHANGED,
				editorSelectioNChangedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_TEXTSELECTION_CHANGED,
				editorTextSelectionChangedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_DOUBLECLICKED,
				editorDoubleClickedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_WAS_SCROLLED,
				editorWasScrolledHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEUP,
				editorMouseUpHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEDOWN,
				editorMouseDownHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_MOUSE_MOVED,
				editorMouseMovedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_STYLE_CHANGED,
				editorStyleChangedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_ANNOTATION_HOVERED,
				editorAnnotationHoveredHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.EDITOR_KEY_PRESSED,
				editorKeyPressedHandler);
		MasterPart.subscribeForEvent(classToSubscribe, IAnnotationEditorEvents.RIGHTCLICK_MENU_REQUESTED,
				editorRightClickMenuRequestedHandler);

	}

	@Override
	public void coupleToPerspective() {

		// refresh the view
		if (this.editor != null) {
			onEditorInputChanged(editor);
		}
		this.standardSubscribeToEditorEvents(class_this);
		this.standardSubscribeToEditorDocumentChangedEvents(class_this);
	}

	@Override
	public void decoupleFromPerspective() {
		this.standardUnsubscribeFromEditorEvents(class_this);
		this.standardUnsubscribeFromEditorDocumentChangedEvents(class_this);
	}

	public void onFocus() {

		// we are not even built!!
		if (broker == null)
			return;

		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

		if (this.editor == null) {

			System.out.println("On editor input changed " + this.getClass());
			// request the editor if possible
			broker.send(IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR, this);
			standardSubscribeToEditorDocumentChangedEvents(class_this);
		}

	}

}
