package de.uniwue.mk.kall.athen.part.editor.subordinate;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.CASException;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.ruta.rule.AnnotationComparator;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.osgi.service.event.Event;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.NameFilter;
import de.uniwue.mk.kall.athen.part.editor.UnderlyingAnnotationFilter;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

/**
 * 
 * @author mkrug
 * 
 *         this class can be use if you want to inherit the most basic funtions
 *         of the views includifn the table
 *
 */

public abstract class ARightViewTablePart extends AEditorSubordinateViewPart {

	// UI
	private Button buttonRemoveAnnotation;
	private Button buttonAddAnnotation;
	private Button buttonAccept;
	private TableViewer viewer;
	private Text neuAnno;
	protected List<TableColumnStruct> columns;

	protected static final String FEAT_COVERED_TEXT = "FEAT_COVERED_TEXT";

	private Type typeHandled;
	protected String typeHandledS;
	private Type typeModus;
	protected String typeModusS;
	protected AnnoStyle styleModus;

	NameFilter filter;

	// model
	private List<AnnotationFS> annoList;

	// identifier
	protected static Class<?> CLASS_THIS;
	protected String viewId;

	// eclipse
	// copy and modify this class if you want to create a new view

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;
	private Button buttonOnlyThatAreHandled;
	private Button jumpButton;
	private UnderlyingAnnotationFilter ovFilter;

	public ARightViewTablePart(String id) {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;

	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);
	}

	@Override
	public void coupleToPerspective() {
		standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		super.coupleToPerspective();
	}

	@Override
	public void decoupleFromPerspective() {
		editor = null;
		standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);
		super.decoupleFromPerspective();
	}

	@Override
	public void onEditorClosed(Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications - yep this is important
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onEditorInputChanged(AnnotationEditorWidget editor) {
		super.editor = editor;

		// only initialize if the part is visible
		MUIElement selectedElement = null;
		if (part != null) {
			MElementContainer<MUIElement> parent = part.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render but unsibscribe
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
			return;
		}

		// init all the types we need in this view, open a dialog to specify
		// them if not already done so
		boolean typesValid = initTypes();

		while (!typesValid) {

			// open preference dialogue and ask the user to enter valid types

			typesValid = initTypes();

		}

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// init the Model
		initModel();

		// init the layout
		initLayout(overlay);

		// if we managed to get here subscribe to all editor events if not
		// already done so
		if (!isSubscribedToEditorEvents) {
			// use the standard method to get all events
			super.standardSubscribeToEditorEvents(CLASS_THIS);
		}

		// ====reload=====
		viewer.getTable().pack();

		try {
			renewPage();
		} catch (CASException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		overlay.layout();
		overlay.getParent().layout();

	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		this.typeHandled = editor.getCas().getTypeSystem().getType(typeHandledS);
		this.typeModus = editor.getCas().getTypeSystem().getType(typeModusS);
		// set everything unnneded invisible
		this.editor.allStylesInvisible();

		// visibility
		editor.changeVisibility(typeModus, true);
		editor.changeVisibility(typeHandled, true);

		// layer
		editor.changeLayer(typeHandled, 1);

		// yellow
		StyleRange rangeHandled = new StyleRange();
		rangeHandled.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle styleHandled = new AnnoStyle(new BackgroundDrawingStrategy(rangeHandled), rangeHandled);

		// apply
		editor.changeStyle(typeHandled, styleHandled);
		editor.changeStyle(typeModus, styleModus);

		// also change the modus
		editor.setActiveMode(typeModus);

		return true;

	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

		// this.overlay = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 5;
		layout.numColumns = 2;
		layout.verticalSpacing = 5;
		layout.marginWidth = 5;
		layout.marginHeight = 5;
		overlay.setLayout(layout);

		// / ==== Buttons + Fields

		Composite comp2 = new Composite(overlay, SWT.BORDER);
		comp2.setVisible(true);
		GridLayout gdd = new GridLayout();
		GridData gridData2 = new GridData();
		gridData2.horizontalSpan = 2;
		gridData2.grabExcessHorizontalSpace = true;
		// gridData2.grabExcessVerticalSpace = true;
		gridData2.horizontalAlignment = GridData.FILL;
		comp2.setLayout(gdd);
		comp2.setLayoutData(gridData2);

		// Label actions = new Label(comp2, SWT.NULL);
		// actions.setText("Actions");
		// Composite comp = new Composite(overlay, SWT.BORDER);
		// comp.setVisible(true);

		GridLayout gridBelow = new GridLayout(2, false);
		comp2.setLayout(gridBelow);
		buttonAddAnnotation = new Button(comp2, SWT.NONE);
		buttonAddAnnotation.setText("New Annotation:");
		buttonAddAnnotation.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typeModus)) {
							selected = anno;

						}
					}
				}

				// remove vorschlag
				CAS cas = editor.getCas();
				Type t = typeModus;
				// get the selection

				if (selected == null) {

					Point selection = editor.getActualSelectedText();
					if (selection == null || selection.x == selection.y) {
						return;
					}
					AnnotationFS anno = cas.createAnnotation(t, selection.x, selection.y);

					addAnnotation(anno);

					if (jumpButton.getSelection()) {

						AnnotationFS nextToHighLight = getNextToHighLight(anno);

						if (nextToHighLight != null) {
							editor.highlightRange(nextToHighLight.getBegin(), nextToHighLight.getEnd()
									- nextToHighLight.getBegin(), true);
						}
					}

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

				widgetSelected(e);

			}
		});

		// listener

		this.neuAnno = new Text(comp2, SWT.BORDER);
		neuAnno.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		buttonAccept = new Button(comp2, SWT.PUSH);
		buttonAccept.setText("Accept Annotation");

		buttonAccept.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typeModus)) {
							selected = anno;

						}
					}
				}

				if (selected != null) {
					// delete all overrides in that spot and add a new one

					List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
					for (AnnotationFS handled : editor.getCas().getAnnotationIndex(typeHandled)) {

						if (handled.getBegin() == selected.getBegin() && handled.getEnd() == selected.getEnd()) {
							toDelete.add(handled);
						}
					}

					// delete quietly
					for (AnnotationFS handled : toDelete)
						editor.getCas().removeFsFromIndexes(handled);

					// add a new override and add with reload
					AnnotationFS handledNew = editor.getCas().createAnnotation(typeHandled, selected.getBegin(),
							selected.getEnd());

					editor.addAnnotation(handledNew);

					if (jumpButton.getSelection()) {

						AnnotationFS nextToHighLight = getNextToHighLight(selected);

						if (nextToHighLight != null) {
							editor.highlightRange(nextToHighLight.getBegin(), nextToHighLight.getEnd()
									- nextToHighLight.getBegin(), true);
						}
					}

				}

			}

		});

		buttonRemoveAnnotation = new Button(comp2, SWT.NONE);
		buttonRemoveAnnotation.setText("Remove Annotation");
		buttonRemoveAnnotation.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typeModus)) {
							selected = anno;

						}
					}
				}

				// we delete only the one
				if (selected != null) {

					deleteAnnotation(selected);
					if (jumpButton.getSelection()) {

						AnnotationFS nextToHighLight = getNextToHighLight(selected);

						if (nextToHighLight != null) {
							editor.highlightRange(nextToHighLight.getBegin(), nextToHighLight.getEnd()
									- nextToHighLight.getBegin(), true);
						}
					}

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

				widgetSelected(e);

			}
		});

		// // ==== Table
		comp2.setSize(overlay.getSize());

		Composite compp = new Composite(overlay, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		buttonOnlyThatAreHandled = new Button(compp, SWT.CHECK);
		buttonOnlyThatAreHandled.setText("Only handled");

		buttonOnlyThatAreHandled.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				try {
					renewPage();
				} catch (CASException e1) {
					e1.printStackTrace();
				}
			}
		});

		// no jump option !
		jumpButton = new Button(compp, SWT.CHECK);
		jumpButton.setText("Jump to next?:");
		jumpButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		jumpButton.setSelection(true);

		// searchLabel
		Label searchLabel = new Label(compp, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compp, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		viewer = new TableViewer(compp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		// add Menu
		// MenuListener
		final Menu menu = new Menu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// Delete
		MenuItem del = new MenuItem(menu, SWT.None);

		del.setText("Delete Selected Annotation");

		del.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				deleteAnnotation(firstElement);

				try {
					renewPage();
				} catch (CASException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}

		});

		// GoTo

		MenuItem goTo = new MenuItem(menu, SWT.None);

		goTo.setText("GoTo Selected Annotation");

		goTo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				int beg = firstElement.getBegin();

				changeHighlighting(beg, firstElement.getEnd(), true);

			}

		});

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);
		// New to support the search

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		// filter
		filter = new NameFilter();
		viewer.addFilter(filter);

		// id filter

		ovFilter = new UnderlyingAnnotationFilter();
		viewer.addFilter(ovFilter);

		createTableViewer();

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

	}

	private void createColumns(TableViewer viewer) {

		int colIndex = 0;

		for (final TableColumnStruct struct : this.columns) {
			final TableViewerColumn colName = createTableViewerColumn(struct.getColName(), struct.getColSize(),
					colIndex);
			colName.getColumn().setData("FEATURE", struct.getFeat());
			colName.setLabelProvider(new ColumnLabelProvider() {
				@Override
				public String getText(Object element) {
					AnnotationFS p = (AnnotationFS) element;

					String feat = struct.getFeat();

					if (feat.equals(FEAT_COVERED_TEXT)) {
						return p.getCoveredText();
					}

					Feature featureByBaseName = typeModus.getFeatureByBaseName(feat);
					return p.getFeatureValueAsString(featureByBaseName);

				}

			});

			String colFeat = (String) colName.getColumn().getData("FEATURE");

			if (!colFeat.equals(FEAT_COVERED_TEXT)) {
				// let all columsn be edited ( except fort he text obviously)
				colName.setEditingSupport(new RightViewTableEditingSupport(viewer, typeModus
						.getFeatureByBaseName(colFeat), editor));
			}
			colIndex++;
		}
	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	@Override
	public void onViewStackChanged(Event arg0) {
		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	@PreDestroy
	public void preDestroy() {
		// TODO unused
	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

		if (this.editor == null) {

			// request the editor if possible
			broker.send(IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR, this);
			super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr
	}

	@Override
	public void onAnnotationAdded(Event e) {
		updateModel();

	}

	@Override
	public void onAnnotationRemoved(Event arg0) {
		updateModel();
	}

	@Override
	public void onAnnotationChanged(Event arg0) {

		updateModel();

	}

	@Override
	public void onAnnotationSelectionChanged(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorDoubleClicked(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorWasScrolled(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorCaretChanged(Event arg0) {

		AnnotationFS selected = null;

		// do not update while the editor is in drag mode
		if (editor.getMouseDrag())
			return;

		int caretOffset = editor.getWidget().getCaretOffset();
		for (AnnotationFS anno : this.annoList) {

			if (anno.getBegin() <= caretOffset && anno.getEnd() >= caretOffset) {
				selected = anno;
				break;
			}
		}

		if (selected != null) {

			viewer.setSelection(new StructuredSelection(selected), true);

		}

	}

	@Override
	public void onEditorMouseUp(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorMouseDown(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorTextSelectionChanged(Event arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public void onEditorMouseMoved(Event e) {
		// TODO Auto-generated method stub

	}

	// use this method to update the view
	public void renewPage() throws CASException {

		// redraw in a new Thread, executed by the Display
		Runnable r = new Runnable() {

			@Override
			public void run() {
				createTableViewer();

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void createTableViewer() {

		initModel();

		viewer.setInput(annoList);

		List<AnnotationFS> handled = new ArrayList<AnnotationFS>();

		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typeHandled)) {
			handled.add(anno);
		}
		ovFilter.setAnnotations(handled, buttonOnlyThatAreHandled.getSelection());

		viewer.refresh();
	}

	@Override
	public void initModel() {
		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typeModus)) {
			annoList.add(anno);
		}

	}

	private void updateModel() {
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typeModus);

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}
		try {
			renewPage();
		} catch (CASException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void deleteAnnotation(AnnotationFS firstElement) {
		// delete all overrides in that stage

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typeHandled);
		ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : annotationIndex) {
			if (anno.getBegin() == firstElement.getBegin() && anno.getEnd() == firstElement.getEnd()) {
				toDelete.add(anno);
				// editor.removeAnnotation(anno);
			}
		}

		for (AnnotationFS a : toDelete) {
			editor.removeAnnotation(a);
		}

		editor.removeAnnotation(firstElement);

	}

	private void addAnnotation(AnnotationFS anno) {

		// add Annotation - means we add an Annotation of type NER also an
		// Override Type
		// to display that this annotation was already handled

		this.editor.addAnnotation(anno);

		// add the override Annotation
		CAS cas = this.editor.getCas();

		AnnotationFS fs = cas.createAnnotation(typeHandled, anno.getBegin(), anno.getEnd());

		this.editor.addAnnotation(fs);

	}

	private void changeHighlighting(int beg, int end, boolean b) {

		editor.highlightRange(beg, end - beg, b);

	}

	private AnnotationFS getNextToHighLight(AnnotationFS actual) {

		updateModel();
		Collections.sort(this.annoList, new AnnotationComparator());

		for (int i = 0; i < annoList.size(); i++) {
			if (actual.getBegin() < annoList.get(i).getBegin()) {
				return annoList.get(i);
			}
		}
		return null;

	}

}
