package de.uniwue.mk.kall.athen.part.editor.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.uima.resource.ResourceInitializationException;
import org.apache.uima.resource.metadata.TypeDescription;
import org.apache.uima.resource.metadata.TypeSystemDescription;
import org.apache.uima.resource.metadata.impl.TypeSystemDescription_impl;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * 
 * @author pdbeck
 *
 */

public class XmiToTypeSystemGenerator {

	private static Map<String, String> namespaces = new HashMap<String, String>();

	private static Document document;

	private static TypeSystemDescription tsd;

	private static HashMap<String, Set<String>> types;

	// public static void main(String[] args) throws
	// ParserConfigurationException, SAXException, IOException,
	// ResourceInitializationException {
	// File srcFile = new File(PATH_TO_XMI_FILE);
	// File dstFile = new File(PATH_TO_DST_TSD_FILE);
	// generateTypeSystemFromXmi(srcFile, dstFile);
	// }

	public static TypeSystemDescription generateTypeSystemFromXmi(File srcFile) throws ParserConfigurationException,
			SAXException, IOException, ResourceInitializationException {
		init(srcFile);
		resolveNamespaces();
		createTypes();

		return writeTypesToTsd();
	}

	private static TypeSystemDescription writeTypesToTsd() throws FileNotFoundException, SAXException, IOException {
		for (String type : types.keySet()) {
			Set<String> featureSet = types.get(type);
			String typeName = getNamespace(type);
			TypeDescription addType = tsd.addType(typeName, "", "uima.tcas.Annotation");
			for (String feature : featureSet) {
				if (feature.startsWith("simple:")) {
					addType.addFeature(feature.split(":")[1], "", "uima.cas.String");
				} else {
					addType.addFeature(feature.split(":")[1], "", "uima.cas.StringList");
				}
			}

		}
		return tsd;
	}

	private static String getNamespace(String type) {
		String[] split = type.split("\\:");
		return namespaces.get(split[0]) + "." + split[1];
	}

	private static void init(File srcFile) throws ParserConfigurationException, SAXException, IOException,
			ResourceInitializationException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();
		document = builder.parse(srcFile);
		types = new HashMap<String, Set<String>>();
		tsd = new TypeSystemDescription_impl();
		// XmiToTypeSystemGenerator.dstFile = dstFile;
	}

	private static void createTypes() {
		NodeList childNodes = document.getFirstChild().getChildNodes();
		for (int i = 0; i < childNodes.getLength(); i++) {
			Node item = childNodes.item(i);
			String nodeName = item.getNodeName();
			if (!nodeName.startsWith("cas:")) {
				Set<String> newFeatureSet = getFeatureSet(item);
				Set<String> oldFeatureSet = new HashSet<String>();
				if (types.containsKey(nodeName)) {
					oldFeatureSet.addAll(types.get(nodeName));
				}
				newFeatureSet.addAll(oldFeatureSet);
				types.put(nodeName, newFeatureSet);
			}
		}
	}

	private static Set<String> getFeatureSet(Node item) {
		Set<String> featureSet = new HashSet<String>();
		NodeList childNodes = item.getChildNodes();
		for (int i = 0; i < childNodes.getLength(); i++) {
			Node child = childNodes.item(i);
			featureSet.add("complex:" + child.getNodeName());
		}
		NamedNodeMap attributes = item.getAttributes();
		for (int i = 0; i < attributes.getLength(); i++) {
			Node child = attributes.item(i);
			String nodeName = child.getNodeName();
			if (!nodeName.equals("begin") && !nodeName.equals("end") && !nodeName.equals("sofa")
					&& !nodeName.equals("xmi:id")) {
				featureSet.add("simple:" + nodeName);
			}
		}
		return featureSet;
	}

	private static void resolveNamespaces() {
		NamedNodeMap attributes = document.getFirstChild().getAttributes();
		for (int i = 0; i < attributes.getLength(); i++) {
			Node attributeItem = attributes.item(i);
			String nodeName = attributeItem.getNodeName();
			if (nodeName.startsWith("xmlns")) {
				String[] split = nodeName.split(":");
				String nodeValue = attributeItem.getNodeValue();
				String pre = nodeValue.replaceAll("http:///", "");
				pre = pre.replaceAll(".ecore", "");
				pre = pre.replaceAll("/", ".");
				String namespace = split[1];
				namespaces.put(namespace, pre);
			}
		}
	}

}
