package de.uniwue.mk.kall.athen.widget.editor;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MDirtyable;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.model.application.ui.basic.MWindow;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.DeserializeXMIHandler;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;

public class AnnotationEditorPart implements IAnnotationEditorListener, IPerspectiveDecoupledPart {

	/**
	 * this part merely delegates all the events of the underlying Widget to the
	 * rest of the application, therefore performs listener subscription
	 */

	// this does the most work
	private AnnotationEditorWidget widget;

	// id of this part
	private String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.2";

	// e4 injected objects
	@Inject
	MDirtyable dirty;

	@Inject
	EModelService modelService;

	@Inject
	IEventBroker broker;

	@Inject
	private EPartService partService;

	private MPart editorPart;

	// HANDLER

	private EventHandler requestEditorPartHandler;
	private EventHandler openXmiHandler;

	private Composite parent;

	@PostConstruct
	public void postConstruct(Composite parent, MPart part, MWindow window) {

		// Save the composite
		this.parent = parent;
		// part.setCloseable(false);
		// this.labelLeftSaeule = new Label(parent, SWT.NONE);

		// initialize the widget
		this.widget = new AnnotationEditorWidget(parent, SWT.NONE);
		// and subscribe to all events
		subscribeToWidgetEvents();

		this.editorPart = part;

		requestEditorPartHandler = new EventHandler() {

			@Override
			public void handleEvent(Event e) {

				System.out.println("Request");
				if (widget.getCas() != null) {
					AEditorSubordinateViewPart part = (AEditorSubordinateViewPart) e.getProperty(IEventBroker.DATA);

					part.onEditorInputChanged(widget);
				}

			}
		};

		// subscribe to events in the beginning this editor has to listen to the
		// project explorer
		openXmiHandler = new DeserializeXMIHandler(this, broker, partService);
	}

	private void subscribeToWidgetEvents() {

		this.widget.addListener(EEditorEvent.EDITOR_MOUSE_MOVED, this);
		this.widget.addListener(EEditorEvent.EDITOR_FEATURE_SELECTION_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_TEXTSELECTION_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_FILE_SAVED, this);
		this.widget.addListener(EEditorEvent.EDITOR_DIRTY_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_LAYER_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_STYLE_CHANEGD, this);
		this.widget.addListener(EEditorEvent.EDITOR_VISIBILITY_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_REMOVED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_ADDED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_INPUT_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_WIDGET_DOUBLECLICKED, this);
		this.widget.addListener(EEditorEvent.EDITOR_WIDGET_MOUSEDOWN, this);
		this.widget.addListener(EEditorEvent.EDITOR_WIDGET_MOUSEUP, this);
		this.widget.addListener(EEditorEvent.EDITOR_WAS_SCROLLED_HBAR, this);
		this.widget.addListener(EEditorEvent.EDITOR_WAS_SCROLLED_VBAR, this);
		this.widget.addListener(EEditorEvent.EDITOR_CARET_CHANGED, this);
		this.widget.addListener(EEditorEvent.EDITOR_ANNOTATION_HOVERED, this);
		this.widget.addListener(EEditorEvent.EDITOR_KEY_PRESSED, this);
		this.widget.addListener(EEditorEvent.RIGHTCLICK_MENU_REQUESTED, this);
	}

	@Persist
	public void save() {
		this.widget.save();
	}

	// DOCUMENT CHANGES
	public void onCASChanged(CASEditorInput input) {

		// set the new input into the widget
		this.widget.setInput(input);

		// also change the label
		this.editorPart.setLabel(input.getEditorInputFile().getName());

	}

	// PERSPECTIVE EVENTS
	public void coupleToPerspective() {

		System.out.println("Couple Editor");
		// subscribe to events in the beginning this editor has to listen to the
		// project explorer
		MasterPart.subscribeForEvent(AnnotationEditorPart.class, IAnnotationEditorEvents.OPEN_XMI_REQUESTED,
				openXmiHandler);
		MasterPart.subscribeForEvent(AnnotationEditorPart.class, IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR,
				requestEditorPartHandler);
	}

	public void decoupleFromPerspective() {
		System.out.println("Decouple Editor");

		MasterPart.unsubscribeFromEvent(AnnotationEditorPart.class, IAnnotationEditorEvents.OPEN_XMI_REQUESTED,
				openXmiHandler);
		MasterPart.unsubscribeFromEvent(AnnotationEditorPart.class, IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR,
				requestEditorPartHandler);
	}

	public Composite getParent() {

		return this.parent;
	}

	public MPart getEditorPart() {
		return editorPart;
	}

	@Override
	public void performEditorChange(EEditorEvent eventType, Object changedObject) {

		switch (eventType) {
		case EDITOR_ANNOTATION_ADDED:
			broker.send(IAnnotationEditorEvents.EDITOR_ANNOTATION_ADDED, changedObject);
			break;
		case EDITOR_ANNOTATION_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_ANNOTATION_CHANGED, changedObject);
			break;
		case EDITOR_ANNOTATION_REMOVED:
			broker.send(IAnnotationEditorEvents.EDITOR_ANNOTATION_REMOVED, changedObject);
			break;
		case EDITOR_CARET_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_CARET_CHANGED, changedObject);
			break;
		case EDITOR_DIRTY_CHANGED:
			dirty.setDirty((boolean) changedObject);
			break;
		case EDITOR_FEATURE_SELECTION_CHANGED:
			// TODO is there no event or am i blind?
			break;
		case RIGHTCLICK_MENU_REQUESTED:
			broker.send(IAnnotationEditorEvents.RIGHTCLICK_MENU_REQUESTED, changedObject);
			break;
		case EDITOR_FILE_SAVED:
			broker.send(IAnnotationEditorEvents.RELOAD_PROJECTS, changedObject);
			break;
		case EDITOR_INPUT_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_INPUT_CHANGED, changedObject);
			break;
		case EDITOR_LAYER_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_LAYER_CHANGED, changedObject);
			break;
		case EDITOR_MOUSE_MOVED:
			broker.send(IAnnotationEditorEvents.EDITOR_MOUSE_MOVED, changedObject);
			break;
		case EDITOR_STYLE_CHANEGD:
			broker.send(IAnnotationEditorEvents.EDITOR_STYLE_CHANGED, changedObject);
			break;
		case EDITOR_TEXTSELECTION_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_TEXTSELECTION_CHANGED, changedObject);
			break;
		case EDITOR_VISIBILITY_CHANGED:
			broker.send(IAnnotationEditorEvents.EDITOR_VISIBILITY_CHANGED, changedObject);
			break;
		case EDITOR_WAS_SCROLLED_HBAR:
			broker.send(IAnnotationEditorEvents.EDITOR_WAS_SCROLLED, changedObject);
			break;
		case EDITOR_WAS_SCROLLED_VBAR:
			broker.send(IAnnotationEditorEvents.EDITOR_WAS_SCROLLED, changedObject);
			break;
		case EDITOR_WIDGET_DOUBLECLICKED:
			broker.send(IAnnotationEditorEvents.EDITOR_WIDGET_DOUBLECLICKED, changedObject);
			break;
		case EDITOR_WIDGET_MOUSEDOWN:
			broker.send(IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEDOWN, changedObject);
			break;
		case EDITOR_WIDGET_MOUSEUP:
			broker.send(IAnnotationEditorEvents.EDITOR_WIDGET_MOUSEUP, changedObject);
			break;
		case EDITOR_ANNOTATION_HOVERED: {
			broker.send(IAnnotationEditorEvents.EDITOR_ANNOTATION_HOVERED, changedObject);
			break;
		}
		case EDITOR_KEY_PRESSED: {
			broker.send(IAnnotationEditorEvents.EDITOR_KEY_PRESSED, changedObject);
			break;
		}
		default:
			break;

		}

	}

	public boolean isDirty() {
		return dirty.isDirty();
	}
}
