package de.uniwue.mk.kall.athen.widget.editor;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.TypeSystem;
import org.apache.uima.cas.impl.XmiCasSerializer;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.text.JFaceTextUtil;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CaretEvent;
import org.eclipse.swt.custom.CaretListener;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.MouseMoveListener;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.xml.sax.SAXException;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.editor.drawingstructure.StyleDataStructure;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IBorderDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IExpensiveDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.INeedBothAnnotationDrawingStrategy;
import de.uniwue.mk.kall.athen.part.editor.dialog.AnnotationStyleEditingDialog;
import de.uniwue.mk.kall.athen.part.editor.dialog.EditorDoubleClickDialog;
import de.uniwue.mk.kall.athen.part.editor.dialog.EditorFindHandler;
import de.uniwue.mk.kall.athen.part.editor.dialog.SearchDialog;

public class AnnotationEditorWidget extends Composite {

	private static final int DEFAULT_LINESPACING = 19;
	// the model
	private CAS cas;
	private File openedFile;

	// the viewer;
	private List<AnnotationFS> surpressedAnnos;

	private AnnotationEditorWidget synchronizedWidget;

	// parent Composite
	Composite parent;
	// Text Widget
	StyledText widget;
	// Default COLOR - Yellow
	Color DEFAULT_COLOR;

	// save the last configuration
	HashSet<Type> visibleTypesOld;
	int oldCaretPosition;
	int oldTopIndex;

	// Visible Annotations
	private HashMap<AnnotationFS, TypeStyleDataStructure> visibleMap;

	private long timeOfLastUpdate = 0;

	// needed for the function that assigns initial colors to the types
	private static final int COLORSTEP = 30;

	// HashMap for Styles
	private StyleDataStructure styleStruct;

	private TypeSystem typeSystem;

	protected boolean mousedrag;
	private boolean isDirty;
	private boolean notifyDocumentChange = true;

	// listener

	private Map<EEditorEvent, List<IAnnotationEditorListener>> editorListener;
	protected int oldOffSet;
	private int lastVisibleFrom;
	private int lastVisibleTo;
	private boolean preventPaintListenerPaint;
	private Type activeModeType;

	public AnnotationEditorWidget(Composite parent, int style) {
		super(parent, style);
		// Save the composite
		this.parent = parent;

		surpressedAnnos = new ArrayList<AnnotationFS>();

		editorListener = new HashMap<EEditorEvent, List<IAnnotationEditorListener>>();

	}

	/*
	 * this method is called whenever a new document is opened
	 */

	public void handleSaveAndReload(CASEditorInput input) {
		if (this.cas != null && this.openedFile != null && !this.openedFile.equals(input.getEditorInputFile())
				&& notifyDocumentChange) {

			// Save the last one first if the editor was opened first
			boolean answer = MessageDialog
					.openQuestion(Display.getCurrent().getActiveShell(), "Document Change detected",
							"Do you want to save your document " + this.openedFile.getName() + " ?");
			if (answer) {
				save();
			}
		}

	}

	public void surpressAnnotation(AnnotationFS anno, boolean surpressed) {

		if (surpressed) {
			this.surpressedAnnos.add(anno);
		}

		else {
			this.surpressedAnnos.remove(anno);
		}
	}

	public void setInput(CASEditorInput input) {

		// if the user changed the document without saving we do this now, also
		// prevent a reload cause by a save
		handleSaveAndReload(input);

		// clear all surpressed annotation
		surpressedAnnos.clear();
		// if the old and the new files are the same, remember EVERYTHING

		if (this.openedFile != null && this.openedFile.equals(input.getEditorInputFile())) {
			visibleTypesOld = this.styleStruct.getVisibleTypes();
			oldTopIndex = widget.getTopIndex();
			oldCaretPosition = widget.getCaretOffset();

		}

		else {
			if (visibleTypesOld != null) {
				visibleTypesOld.clear();
			}
		}

		this.cas = input.getCas();
		this.typeSystem = cas.getTypeSystem();
		this.lastVisibleFrom = -1;
		this.lastVisibleTo = -1;
		this.openedFile = input.getEditorInputFile();
		visibleMap = new HashMap<AnnotationFS, TypeStyleDataStructure>();

		// clear compositei
		deleteAttachedWidgets();

		this.setLayout(new FillLayout());

		// initialize COlor and Mode
		DEFAULT_COLOR = new Color(parent.getDisplay(), 255, 255, 0);

		// init the widget
		initWidget();
		initWidgetStyling();

		// init the listener
		initKeyListener();
		initWidgetSelectionListener();
		addCaretListener();
		addPaintListener();
		addMouseMovedListener();
		addMouseListener();
		addScrollbarListener();

		// styleMap -the default
		initStyleMap(parent);

		// rightclickMenu
		initializeMenu();

		// ====RELOAD PART=====
		this.layout();
		parent.layout();

		// notify listening views
		fireEvent(EEditorEvent.EDITOR_INPUT_CHANGED, this);

		// set vertical scrolbar to old position if necessary this is to reapply
		// an old state
		if (this.visibleTypesOld != null && !this.visibleTypesOld.isEmpty()) {

			widget.setTopIndex(oldTopIndex);
			invokeReload();
			widget.setCaretOffset(oldCaretPosition);
			parent.layout();
		}

	}

	private void addMouseListener() {
		widget.addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {

				fireEvent(EEditorEvent.EDITOR_WIDGET_MOUSEUP, new Point(e.x, e.y));
				AnnotationEditorWidget.this.mousedrag = false;
			}

			@Override
			public void mouseDown(MouseEvent e) {
				AnnotationEditorWidget.this.mousedrag = true;
				fireEvent(EEditorEvent.EDITOR_WIDGET_MOUSEDOWN, new Point(e.x, e.y));

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				// send the point of the click
				fireEvent(EEditorEvent.EDITOR_WIDGET_DOUBLECLICKED, new Point(e.x, e.y));

			}
		});
	}

	private void addScrollbarListener() {
		widget.getVerticalBar().addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widget);

				// The index of the first (possibly only partially)
				// visible line of the widget
				int topIndex = JFaceTextUtil.getPartialTopIndex(widget);

				// send a point with from -to

				fireEvent(EEditorEvent.EDITOR_WAS_SCROLLED_VBAR, new Point(bottomIndex, topIndex));
				// also reload the editor if needed

				invokeReload();

				determineVisibleRange();
			}

		});

		widget.getHorizontalBar().addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widget);

				// The index of the first (possibly only partially)
				// visible line of the widget
				int topIndex = JFaceTextUtil.getPartialTopIndex(widget);

				// send a point with from -to

				fireEvent(EEditorEvent.EDITOR_WAS_SCROLLED_HBAR, new Point(bottomIndex, topIndex));

				// also reload the editor if needed
				// also adjust the synchronized widget!
				if (AnnotationEditorWidget.this.synchronizedWidget != null) {
					int thisSelection = widget.getHorizontalBar().getSelection();
					int selection = AnnotationEditorWidget.this.synchronizedWidget.getWidget().getHorizontalBar()
							.getSelection();

					if (selection != thisSelection) {
						AnnotationEditorWidget.this.synchronizedWidget.getWidget().getHorizontalBar()
								.setSelection(thisSelection);
					}
				}

				invokeReload();

			}

		});
	}

	private void addMouseMovedListener() {
		// views can use this to calculate tooltips
		widget.addMouseMoveListener(new MouseMoveListener() {

			private List<AnnotationFS> lastHoveredAnnos;

			@Override
			public void mouseMove(MouseEvent e) {
				fireEvent(EEditorEvent.EDITOR_MOUSE_MOVED, new Point(e.x, e.y));

				if (this.lastHoveredAnnos == null) {
					this.lastHoveredAnnos = new ArrayList<AnnotationFS>();
				}

				// also check if an annotation is hovered

				List<AnnotationFS> hoveredAnnos = new ArrayList<AnnotationFS>();
				for (AnnotationFS a : visibleMap.keySet()) {

					if (isHovered(new Point(e.x, e.y), a)) {
						hoveredAnnos.add(a);
					}
				}

				for (AnnotationFS a : hoveredAnnos) {
					if (!lastHoveredAnnos.contains(a)) {
						fireEvent(EEditorEvent.EDITOR_ANNOTATION_HOVERED, a);
					}
				}

				lastHoveredAnnos = hoveredAnnos;

			}

			private boolean isHovered(Point point, AnnotationFS anno) {

				// TODO speed up for left and right border
				if (anno.getBegin() < 0 || anno.getEnd() > widget.getText().length()
						|| anno.getCoveredText().matches("\\s+"))
					return false;

				if (isLeftBorder(point.x)) {

					return false;
				}

				else if (isRightBorder(point.x)) {
					return false;
				}

				else {
					// border drawingstrategies can never
					if (visibleMap.get(anno) != null
							&& visibleMap.get(anno).getStyle().getDrawingStrategy() instanceof IBorderDrawingStrategy) {
						return false;
					}
					Rectangle textBounds;
					try {
						textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd());
					}

					// TODO investigate why this is required in some
					// documents...
					catch (IllegalArgumentException e) {
						return false;
					}

					return textBounds.contains(point);
				}

			}

			private boolean isRightBorder(int x) {

				return x > widget.getBounds().width - widget.getRightMargin();
			}

			private boolean isLeftBorder(int x) {
				return x < widget.getLeftMargin();
			}
		});
	}

	private void addPaintListener() {
		widget.addPaintListener(new PaintListener() {

			@Override
			public void paintControl(PaintEvent e) {

				// see if the lock is set
				if (preventPaintListenerPaint)
					return;

				Runnable r = new Runnable() {

					@Override
					public void run() {

						for (AnnotationFS anno : visibleMap.keySet()) {

							TypeStyleDataStructure ds = visibleMap.get(anno);

							if (ds == null) {
								continue;
							}
							IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

							if (!(drawingStrategy instanceof IExpensiveDrawingStrategy)) {
								drawingStrategy.drawAnnotation(ds, anno, widget, new GC(widget));
							}

						}

					}
				};

				// Save some performance only reload every 20 ms, and only
				// paint
				// if there are changes
				// if (visibleMap.keySet().equals(oldKeyset)) {
				if (System.currentTimeMillis() - timeOfLastUpdate > 1) {
					// widget.redraw();
					timeOfLastUpdate = System.currentTimeMillis();
					Display.getCurrent().asyncExec(r);

				}
				// }

			}
		});
	}

	private void addCaretListener() {
		widget.addCaretListener(new CaretListener() {

			@Override
			public void caretMoved(CaretEvent event) {
				fireEvent(EEditorEvent.EDITOR_CARET_CHANGED, event.caretOffset);

			}
		});
	}

	private void initWidgetSelectionListener() {
		widget.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				textSelectionChanged();

			}

		});
	}

	private void initWidget() {
		widget = new StyledText(this, SWT.V_SCROLL | SWT.H_SCROLL);
		// widget = sourceViewer.getTextWidget();
		widget.setText(this.cas.getDocumentText());
		widget.setEditable(false);
		widget.setMargins(25, 10, 25, 0);
		widget.setAlwaysShowScrollBars(true);
		widget.setWordWrap(true);
		widget.setLineSpacing(DEFAULT_LINESPACING);
	}

	private void initKeyListener() {
		widget.addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {

				// ctrl + s
				if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 115)) {
					save();
				}

			}

			@Override
			public void keyPressed(KeyEvent e) {

				if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'f')) {
					// open search dialog
					SearchDialog sd = new SearchDialog(Display.getCurrent().getActiveShell(), widget.getSelectionText());
					sd.addFindListener(new EditorFindHandler(getWidget(), sd, AnnotationEditorWidget.this));

					sd.open();
				}

				if (((e.stateMask & SWT.CTRL) == SWT.CTRL)) {
					// fire event that the ctrl even is fired
					fireEvent(EEditorEvent.EDITOR_CTRL_PRESSED, e);
				}

				if ((e.keyCode == SWT.DEL)) {
					// check how many annotations that are visible are in this
					// position

					List<AnnotationFS> toDeleteCands = new ArrayList<AnnotationFS>();
					for (AnnotationFS a : visibleMap.keySet()) {
						if (a.getType().getName().contains(".temp"))
							continue;
						if (a.getBegin() <= widget.getCaretOffset() && a.getEnd() >= widget.getCaretOffset()) {
							toDeleteCands.add(a);
						}
					}

					if (toDeleteCands.size() == 1) {
						removeAnnotation(toDeleteCands.get(0));
					} else {

						if (activeModeType != null) {
							for (AnnotationFS a : toDeleteCands) {
								if (a.getType().equals(activeModeType)) {
									removeAnnotation(a);
								}
							}
						} else {

							// TODO not supported yet to delete multiple annos
						}
					}
				}

				if ((e.keyCode == SWT.CR)) {

					if ((e.stateMask & SWT.ALT) == SWT.ALT) {

						System.out.println("ALT Enter");
						fireEvent(EEditorEvent.EDITOR_ALT_ENTER_PRESSED, null);
					} else {
						HashSet<Type> visibleTypes = styleStruct.getVisibleTypes();

						List<Type> toRemove = new ArrayList<Type>();
						for (Type t : visibleTypes) {

							if (t.getShortName().startsWith("athen")) {
								toRemove.add(t);
							}
						}
						visibleTypes.removeAll(toRemove);
						if (widget.getSelection().y - widget.getSelection().x == 0) {

							List<AnnotationFS> toChangeCands = new ArrayList<AnnotationFS>();
							for (AnnotationFS a : visibleMap.keySet()) {
								if (a.getType().getName().contains(".temp"))
									continue;
								if (a.getBegin() <= widget.getCaretOffset() && a.getEnd() >= widget.getCaretOffset()) {
									toChangeCands.add(a);
								}
							}

							if (toChangeCands.size() == 1) {

								EditorDoubleClickDialog dialog = new EditorDoubleClickDialog(getParent().getShell(),
										toChangeCands.get(0), AnnotationEditorWidget.this);

								if (dialog.open() == SWT.OK) {
									// refresh the widget
									invokeReload();
								}
							}

						}

						else if (visibleTypes.size() == 1) {

							for (Type t : visibleTypes) {
								AnnotationFS anno = cas.createAnnotation(t, widget.getSelection().x,
										widget.getSelection().y);
								addAnnotation(anno);
								break;
							}
						} else {

							if (activeModeType != null && visibleTypes.size() > 1
									&& visibleTypes.contains(activeModeType)) {

								AnnotationFS anno = cas.createAnnotation(activeModeType, widget.getSelection().x,
										widget.getSelection().y);
								addAnnotation(anno);
							}
							// TODO unsupported!! multiple annotations
						}
					}
				}

				fireEvent(EEditorEvent.EDITOR_KEY_PRESSED, e.character);
			}
		});
	}

	// =======LISTENER CHANGES
	protected void fireEvent(EEditorEvent eventType, Object toFire) {

		List<IAnnotationEditorListener> list = this.editorListener.get(eventType);
		// nothign to fire
		if (list == null)
			return;

		for (IAnnotationEditorListener listener : list) {
			listener.performEditorChange(eventType, toFire);
		}

	}

	public void addListener(EEditorEvent eventType, IAnnotationEditorListener listener) {

		if (this.editorListener.containsKey(eventType)) {

			this.editorListener.get(eventType).add(listener);
		}

		else {
			List<IAnnotationEditorListener> listenerList = new ArrayList<IAnnotationEditorListener>();
			listenerList.add(listener);
			this.editorListener.put(eventType, listenerList);
		}
	}

	// =====END LISTENER

	public void setWidgetFont(Font f) {
		this.widget.setFont(f);
	}

	private void deleteAttachedWidgets() {
		for (Control c : this.getChildren()) {
			c.dispose();
		}
	}

	@Deprecated
	public void fsSelectionChanged() {

		// notify listener
		// fireEvent(EEditorEvent.EDITOR_FEATURE_SELECTION_CHANGED,
		// actualSelection);
	}

	// this is only tolerated but the offset and stuff is ignored
	public void invokeReload(boolean forceReload) {

		if (forceReload) {

			widget.setStyleRanges(lastVisibleFrom, lastVisibleTo - lastVisibleFrom, null, null);
		} else {
			invokeReload();
		}
	}

	public void synchronizeTo(AnnotationEditorWidget other) {
		if (other == null || other.getCas() == null)
			return;

		if (this.synchronizedWidget != null) {
			MessageDialog.openInformation(Display.getCurrent().getActiveShell(),
					"Warning: Another document is already attached", "Please disconnect the other document first");
			return;
		}

		if (!other.getCas().getDocumentText().equals(this.cas.getDocumentText())) {
			ErrorDialog.openError(Display.getCurrent().getActiveShell(), "Unsupported Action",
					"You can not compare two documents with different text!", null);
		}

		this.synchronizedWidget = other;
		other.synchronizeTo(this);
	}

	public void removeSynchronization() {

		if (this.synchronizedWidget != null) {
			this.synchronizedWidget.dispose();
			this.synchronizedWidget = null;
		}
		layout();
		getParent().layout();
	}

	public void invokeReload() {

		// refresh the widget

		// determine the required spacing by the visibletypes
		// for each visible AnnotationType ...
		widget.redraw();
		widget.update();
		widget.layout();

		// this method gets All Annotations that are visible atm
		visibleMap.clear();
		int maximumLineHeight = 0;

		/*
		 * Dispose Controls that have been created for the TerminologyView and
		 * belong to a StyleRange. Disposing them via styleRange.data does not
		 * work reliably.
		 */
		for (Control ctrl : widget.getChildren()) {
			if (ctrl.getData("origin").equals("ttv")) {
				ctrl.dispose();
			}
		}

		// reset old styleranges??? (i mean they arent even
		// visible...)
		// widget.setStyleRanges(lastVisibleFrom, lastVisibleTo -
		// lastVisibleFrom, null, null);
		Point visibelRange = determineVisibleRange();
		clearOldRanges(visibelRange);

		// for each visible AnnotationType ...
		for (TypeStyleDataStructure ds : styleStruct.getStylesFromLowestLayerToHighest()) {
			if (styleStruct.getStyleMap().get(ds) == false) {
				continue;
			}

			// always draw border annotations
			if (ds.getStyle().getDrawingStrategy() instanceof IBorderDrawingStrategy) {

				// add all oft hose in the cas
				for (AnnotationFS anno : cas.getAnnotationIndex(ds.getAnnotationType())) {
					visibleMap.put(anno, ds);
				}
			}

			// Test=====
			List<AnnotationFS> expensiveAnnos = new ArrayList<AnnotationFS>();
			for (AnnotationFS anno : cas.getAnnotationIndex(ds.getAnnotationType())) {
				if (isSurpressed(anno))
					continue;

				if (anno.getEnd() < visibelRange.x || anno.getBegin() > visibelRange.y) {
					continue;
				}
				// delegate to the Styledrawing object in the AnnoStyles which
				// are part of a TSDS
				IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

				// TODO fix the default method!!
				// maximumLineHeight = DEFAULT_LINESPACING;
				maximumLineHeight = Math.max(maximumLineHeight, drawingStrategy.getLineHeight(anno));
				if (drawingStrategy instanceof IExpensiveDrawingStrategy
						|| drawingStrategy instanceof INeedBothAnnotationDrawingStrategy) {
					// draw this here cause it doesnt need to be updated
					// manually
					expensiveAnnos.add(anno);
					visibleMap.put(anno, ds);
				}

				else {
					visibleMap.put(anno, ds);
				}

			}

			// prevent redraw of all other annotations
			preventPaintListenerPaint = true;
			widget.setRedraw(false);
			for (AnnotationFS a : expensiveAnnos) {
				IAnnotationDrawingStrategy drawingStrategy = ds.getStyle().getDrawingStrategy();

				// if (a.getEnd() <= lastVisibleFrom || a.getBegin() >
				// lastVisibleTo) {
				drawingStrategy.drawAnnotation(ds, a, widget, new GC(widget));
			}
			preventPaintListenerPaint = false;
			widget.setRedraw(true);

			// update old offsets
			this.lastVisibleFrom = visibelRange.x;
			this.lastVisibleTo = visibelRange.y;

		}
		widget.setLineSpacing(maximumLineHeight);
	}

	private void clearOldRanges(Point visibelRange) {

		// check if the old ranges are valid or if they are chanegd by a
		// document change
		if (lastVisibleFrom == -1 || lastVisibleTo == -1)
			return;

		// same spans so we clear everything
		if (this.lastVisibleFrom == visibelRange.x && this.lastVisibleTo == visibelRange.y) {

			widget.setStyleRanges(this.lastVisibleFrom, lastVisibleTo - lastVisibleFrom, null, null);
			return;
		}

		// if old and new ranges do not overlap

		if (this.lastVisibleTo < visibelRange.x || this.lastVisibleFrom > visibelRange.y) {

			// reset the whole range
			widget.setStyleRanges(this.lastVisibleFrom, lastVisibleTo - lastVisibleFrom, null, null);
			return;
		}

		// we scrolled upwards
		if (visibelRange.x < lastVisibleFrom && visibelRange.y <= lastVisibleTo) {

			// reset the lower part
			int min = Math.min(visibelRange.y + 1, widget.getText().length());
			widget.setStyleRanges(min, Math.max(0, lastVisibleTo - min), null, null);
			return;
		}

		// we scrolled downwards
		if (visibelRange.x > lastVisibleFrom && visibelRange.y >= lastVisibleTo) {

			// reset the lower part
			widget.setStyleRanges(lastVisibleFrom, visibelRange.x - lastVisibleFrom, null, null);
			return;
		}

	}

	public void textSelectionChanged() {

		fireEvent(EEditorEvent.EDITOR_TEXTSELECTION_CHANGED, this.widget.getSelection());

	}

	// =====manipulate the CAS===================

	// add Annotation
	public void addAnnotation(AnnotationFS anno) {
		this.cas.addFsToIndexes(anno);
		annotationAdded(anno);

	}

	// remove Annotation
	public void removeAnnotation(AnnotationFS anno) {
		this.cas.removeFsFromIndexes(anno);
		annotationRemoved(anno);
	}

	// change Annotation
	public void changeAnnotation(AnnotationFS anno) {
		// get the Annotation; remove it and the add it again
		// TODO

		fireEvent(EEditorEvent.EDITOR_ANNOTATION_CHANGED, anno);
		// TODO dirty
		invokeReload();
		// Dirty Diana!!!
		fireEvent(EEditorEvent.EDITOR_DIRTY_CHANGED, true);
		this.isDirty = true;

	}

	// Batch update - only notify once to save performance
	public void changeAnnotations(List<AnnotationFS> toChange) {

		if (!toChange.isEmpty()) {
			fireEvent(EEditorEvent.EDITOR_ANNOTATION_CHANGED, toChange.get(0));
		}
		// TODO reload distance
		invokeReload();
		// Dirty Diana!!!
		fireEvent(EEditorEvent.EDITOR_DIRTY_CHANGED, true);
		this.isDirty = true;

	}

	private Point determineVisibleRange() {

		int bottomIndex = JFaceTextUtil.getPartialBottomIndex(widget);

		// The index of the first (possibly only partially)
		// visible line of the widget
		int topIndex = JFaceTextUtil.getPartialTopIndex(widget);
		if (bottomIndex < widget.getLineCount() - 1) {
			bottomIndex++;
		}

		if (topIndex < 0)
			topIndex = 0;

		int topAtLine = widget.getOffsetAtLine(topIndex);
		int bottomAtLine = widget.getOffsetAtLine(bottomIndex);

		// full text if we are at the bottom
		if (bottomIndex >= widget.getLineCount() - 1) {
			bottomAtLine = widget.getText().length();
		}

		// we can show the whole document
		if (widget.getLineCount() < 3) {
			return new Point(0, widget.getText().length());
		}

		return new Point(topAtLine, bottomAtLine);

	}

	public void highlightRange(int beg, int len, boolean visible) {
		widget.setSelectionRange(beg, len);

		if (visible) {
			widget.showSelection();
		}

		// notice Listener
		textSelectionChanged();

		// set the caret
		// widget.setCaretOffset(beg + len / 2);

	}

	public void setSelection(AnnotationFS anno) {

		// notice Listener
		fsSelectionChanged();
	}

	// layer change

	public void changeLayer(Type t, int layer) {
		TypeStyleDataStructure ds = getDSFromType(t);

		ds.setLayer(layer);

		// layer changed
		layerChanged(ds);

	}

	// visibility change

	public void changeVisibility(Type t, boolean isVisible) {
		this.styleStruct.changeVisibilityOfType(t, isVisible);

		visibilityChanged(getDSFromType(t), isVisible);
	}

	// change style

	public void changeStyle(Type t, AnnoStyle style) {

		TypeStyleDataStructure ds = getDSFromType(t);
		ds.setStyles(style);

		styleChanged(ds);

	}

	// AnnotationListener
	public void annotationAdded(AnnotationFS added) {

		fireEvent(EEditorEvent.EDITOR_ANNOTATION_ADDED, added);
		invokeReload();
		// Dirty Diana!!!
		fireEvent(EEditorEvent.EDITOR_DIRTY_CHANGED, true);
		this.isDirty = true;

	}

	public void annotationRemoved(AnnotationFS rem) {

		fireEvent(EEditorEvent.EDITOR_ANNOTATION_REMOVED, rem);
		invokeReload();
		// Dirty Diana!!!
		fireEvent(EEditorEvent.EDITOR_DIRTY_CHANGED, true);
		this.isDirty = true;
	}

	public void visibilityChanged(TypeStyleDataStructure tsds, boolean isVisible) {

		fireEvent(EEditorEvent.EDITOR_VISIBILITY_CHANGED, tsds);
		if (!isVisible) {
			invokeReload(true);
		}

		else {
			invokeReload();
		}
	}

	public void allStylesInvisible() {

		// iterate over all types and set them to not visible
		for (TypeStyleDataStructure tsds : this.styleStruct.getStyleMap().keySet()) {
			this.styleStruct.getStyleMap().put(tsds, false);
		}
	}

	public StyleDataStructure getStyleStruct() {
		return styleStruct;
	}

	public void setStyleStruct(StyleDataStructure styleStruct) {
		this.styleStruct = styleStruct;
	}

	public void styleChanged(TypeStyleDataStructure tsds) {

		fireEvent(EEditorEvent.EDITOR_STYLE_CHANEGD, tsds);
		invokeReload();
	}

	public void layerChanged(TypeStyleDataStructure tsds) {

		fireEvent(EEditorEvent.EDITOR_LAYER_CHANGED, tsds);

		invokeReload();
	}

	private void initStyleMap(Composite parent) {
		// init StyleMap
		HashMap<TypeStyleDataStructure, Boolean> map = new HashMap<TypeStyleDataStructure, Boolean>();

		Type topAnno = typeSystem.getType("uima.tcas.Annotation");

		Color actual = this.DEFAULT_COLOR;
		for (Type next : typeSystem.getProperlySubsumedTypes(topAnno)) {

			// add to the color

			StyleRange styleRange = new StyleRange();
			styleRange.background = actual;

			// update Color - - - per default the Colors are calculated and only
			// the background is
			// higlighted
			int r = (actual.getRed() + COLORSTEP) % 256;
			int g = (actual.getGreen() - COLORSTEP) % 256;
			int b = (actual.getBlue() - COLORSTEP / 2) % 256;
			if (g < 0) {
				g += 256;
			}
			if (b < 0) {
				b += 256;
			}
			actual = new Color(parent.getDisplay(), r, g, b);
			// default is backgroundDrawingStrategy

			AnnoStyle style = new AnnoStyle(new BackgroundDrawingStrategy(styleRange), styleRange);
			// AnnoStyle style = new AnnoStyle(strat, styleRange);
			TypeStyleDataStructure asds = new TypeStyleDataStructure(next, style, 0);

			boolean alreadyInserted = false;
			if (visibleTypesOld != null) {

				for (Type t : visibleTypesOld) {

					if (t.getName().equals(next.getName())) {
						map.put(asds, true);
						alreadyInserted = true;
						break;
					}
				}

			}
			if (!alreadyInserted) {
				map.put(asds, false);
			}

		}

		// create StyleDataStructure
		this.styleStruct = new StyleDataStructure(map);

	}

	private void initWidgetStyling() {

		StyleRange styleRange = new StyleRange();
		styleRange.start = 0;
		styleRange.length = widget.getText().length();
		widget.setStyleRange(styleRange);

		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(14);
		}
		Font newFont = new Font(parent.getDisplay(), fontData);
		widget.setFont(newFont);
	}

	private void initializeMenu() {

		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(widget);
		menuMgr.setRemoveAllWhenShown(true);
		// attach that menu manager to the treeviewer
		widget.setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {

				// change style
				Action changeStyleAction = new Action() {

					@Override
					public void run() {

						Shell activeShell = Display.getCurrent().getActiveShell();

						AnnotationStyleEditingDialog id = new AnnotationStyleEditingDialog(activeShell,
								AnnotationEditorWidget.this);
						int open = id.open();

						if (open == Window.OK) {
							// read everything from the Dailog !!
						}
					}
				};
				changeStyleAction.setText("Configurate Styles");
				manager.add(changeStyleAction);

				// Invoke Reload

				Action invokeReloadAction = new Action() {
					@Override
					public void run() {
						invokeReload();
					}

				};
				invokeReloadAction.setText("Invoke Reload");
				manager.add(invokeReloadAction);

				// change font
				Action changeFontAction = new Action() {
					@Override
					public void run() {
						FontDialog dlg = new FontDialog(getParent().getShell());

						FontData open = dlg.open();

						if (open != null) {
							Font font = getWidget().getFont();

							font = new Font(getParent().getDisplay(), dlg.getFontList());

							getWidget().setFont(font);

						}

					}

				};
				changeFontAction.setText("Change Font");
				manager.add(changeFontAction);

				// textsearch

				Action textSearchAction = new Action() {
					@Override
					public void run() {
						System.out.println("search");

						SearchDialog sd = new SearchDialog(getParent().getShell(), getWidget().getSelectionText());
						sd.addFindListener(new EditorFindHandler(getWidget(), sd, AnnotationEditorWidget.this));

						sd.open();
					}

				};
				textSearchAction.setText("Textsearch...");
				manager.add(textSearchAction);

				manager.add(new Separator());

				fireEvent(EEditorEvent.RIGHTCLICK_MENU_REQUESTED, manager);
			}
		});
	}

	private void initializeMenu2() {

		// =====MENU======
		final Menu menu = new Menu(widget);
		widget.setMenu(menu);

		// Visible Annotations
		MenuItem visibleAnnos = new MenuItem(menu, SWT.CASCADE);
		visibleAnnos.setText("Visible Annotations");

		// List All Annotations in this Menu
		final Menu visibleMenu = new Menu(menu);
		visibleAnnos.setMenu(visibleMenu);

		for (TypeStyleDataStructure struct : this.styleStruct.getStylesFromLowestLayerToHighest()) {
			Type t = struct.getAnnotationType();
			Boolean isVisible = AnnotationEditorWidget.this.styleStruct.getStyleMap().get(struct);

			MenuItem item = new MenuItem(visibleMenu, SWT.CHECK);
			item.setText(t.getName());
			if (isVisible != null && isVisible) {
				item.setSelection(true);
			}
			item.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					String code = e.getSource().toString();

					code = code.split(" ")[1].substring(1, code.split(" ")[1].length() - 1);
					TypeStyleDataStructure ds = getDSFromType(typeSystem.getType(code));

					if (ds != null) {

						// change visibility
						Boolean struct = styleStruct.getStyleMap().get(ds);
						styleStruct.addTypeSytle(ds, !struct);

						// let listener know
						visibilityChanged(ds, !struct);
					}

				}

			});
		}

		// save to image TODO does not print GC drawn objects
		// MenuItem saveAsImage = new MenuItem(menu, SWT.NONE);
		// saveAsImage.setText("Save as Image");
		// saveAsImage.addSelectionListener(new SelectionAdapter() {
		//
		// @Override
		// public void widgetSelected(SelectionEvent e) {
		// widget.layout();
		// ApplicationUtil.saveControlToPath(widget.getParent(), new File(
		// "C:\\Users\\HP\\Pictures\\appImages\\img.png").toPath().toString());
		// }
		//
		// });

		// search dialog

		// StyleConfiguration

		// add a spearator
		new MenuItem(menu, SWT.SEPARATOR);

		System.out.println("????");
		// ask the suscribing views if there need to be more entries

	}

	public List<AnnotationFS> getCurrentSelectionOfType(Type typ) {

		List<AnnotationFS> selection = new ArrayList<AnnotationFS>();

		for (AnnotationFS anno : cas.getAnnotationIndex(typ)) {

			if (anno.getBegin() <= widget.getCaretOffset() && anno.getEnd() >= widget.getCaretOffset()) {
				selection.add(anno);
			}
		}

		return selection;
	}

	public void save() {

		try {
			FileOutputStream fileOutputStream = new FileOutputStream(openedFile);
			XmiCasSerializer.serialize(cas, fileOutputStream);

			fileOutputStream.flush();
			fileOutputStream.close();

		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (SAXException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		fireEvent(EEditorEvent.EDITOR_FILE_SAVED, null);
		fireEvent(EEditorEvent.EDITOR_DIRTY_CHANGED, false);
		this.isDirty = false;

	}

	// =======GETTER AND SETTER============

	public boolean isSurpressed(AnnotationFS a) {

		return this.surpressedAnnos.contains(a);
	}

	public File getOpenedFile() {
		return this.openedFile;
	}

	public boolean isDirty() {
		return this.isDirty;
	}

	public void setNotifyDocumentChange(boolean docChange) {
		this.notifyDocumentChange = docChange;
	}

	public TypeStyleDataStructure getDSFromType(Type type) {
		return this.styleStruct.getDSFromType(type);
	}

	public StyledText getWidget() {
		return widget;
	}

	public boolean getMouseDrag() {
		return this.mousedrag;
	}

	public CAS getCas() {
		return cas;
	}

	public HashSet<Type> getVisibleTypes() {
		return this.styleStruct.getVisibleTypes();
	}

	public int getlayerOfType(Type t) {
		return this.styleStruct.getLayerNumber(t);
	}

	public List<AnnotationFS> getSelectionOfVisibleAnnotations() {
		List<AnnotationFS> selection = new ArrayList<AnnotationFS>();

		for (AnnotationFS anno : visibleMap.keySet()) {

			if (anno.getBegin() <= widget.getSelection().x && anno.getEnd() >= widget.getSelection().y) {
				selection.add(anno);
			}
		}
		return selection;

	}

	public Point getActualSelectedText() {
		return this.widget.getSelection();
	}

	public int getLineAtOffset(int offset) {
		return widget.getLineAtOffset(offset);
	}

	public void setActiveMode(Type typ) {

		this.activeModeType = typ;

	}

	public Type getActiveMode() {
		return this.activeModeType;
	}

	public List<AnnotationFS> getCoveringAnnotationOfType(int caretOffset, Type relationType) {

		List<AnnotationFS> coveringAnnos = new ArrayList<AnnotationFS>();

		for (AnnotationFS a : cas.getAnnotationIndex(relationType)) {

			if (a.getBegin() <= caretOffset && a.getEnd() >= caretOffset)
				coveringAnnos.add(a);

			if (a.getBegin() > caretOffset)
				break;
		}

		return coveringAnnos;

	}

}
