package de.uniwue.mk.kall.athen.projectExplorer.actions;

import java.io.File;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.ADocument;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.DocumentFormatUtil;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.EFormat;

public class OpenRemainingAction extends Action {

	private IEventBroker broker;
	private File f;

	// it is expected that we can convert anything and return a cas to our
	// editor
	public OpenRemainingAction(File f, IEventBroker broker) {

		this.broker = broker;
		this.f = f;
	}

	@Override
	public void run() {

		EFormat source = DocumentFormatUtil.determineFormat(f);

		ADocument convertDocument = DocumentFormatUtil.convertDocument(f, source, EFormat.APPLICATION_XMI);

		if (convertDocument == null || convertDocument.getFormat() == EFormat.UNSUPPORTED_FORMAT) {
			MessageDialog
					.openInformation(Display.getCurrent().getActiveShell(), "Unsupported Format",
							"I am sorry to inform you, but your current document cannot be opened since its format is unsupported");
		}

		else {
			// save the current document and send a notification that this
			// document needs to be serialized
			File toFile = new File(f.getAbsolutePath() + ".xmi");
			MessageDialog
					.openInformation(
							Display.getCurrent().getActiveShell(),
							"Information",
							"This document will be converted as .xmi, the file will be saved in the same folder and instantly opened in the editor!");
			convertDocument.serialize(toFile);

			// notify the request
			broker.send(IAnnotationEditorEvents.OPEN_XMI_REQUESTED, toFile);
		}

	}

}
