package de.uniwue.mk.kall.athen.projectExplorer.conversion;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

public class DocumentFormatUtil {

	private static List<AFormatConverter> converter;
	static {

		converter = new ArrayList<AFormatConverter>();
		converter.add(new TxtToApplicationXMIFormatConverter());
	}

	public static ADocument convertDocument(File f, EFormat source, EFormat target) {
		if (source == null || target == null)
			return null;

		for (AFormatConverter conv : converter) {

			if (conv.canConvert(source, target)) {
				return conv.convertFormat(f);
			}
		}

		// unsupported if no converted can convert it content is null
		return new UnsupportedDocument(null);
	}

	public static EFormat determineFormat(File toDetermine) {

		if (toDetermine.getAbsolutePath().endsWith(".txt"))
			return EFormat.TXT;
		return null;
	}

	public static Collection<EFormat> getPossibleTargetFormats(EFormat source) {
		HashSet<EFormat> set = new HashSet<EFormat>();
		for (AFormatConverter conv : converter) {
			if (conv.getSource().equals(source)) {
				set.add(conv.getTarget());
			}
		}
		return set;
	}

	public static File formatFileEnding(File file, ADocument document) {
		if (file.getName().endsWith(document.getFileEnding())) {
			return file;
		}
		File parent = file.getParentFile();
		String fileName = file.getName().substring(0, file.getName().indexOf('.')) 
				+ document.getFileEnding();
		return new File(parent, fileName);
	}
}
