package de.uniwue.mk.kall.athen.projectExplorer.projectDescription;

import java.io.File;
import java.io.IOException;
import java.nio.file.FileSystems;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.StandardWatchEventKinds;
import java.nio.file.WatchEvent;
import java.nio.file.WatchKey;
import java.nio.file.WatchService;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.e4.core.services.events.IEventBroker;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;

public class AnnotationWorkspace {

	private File wsFile;

	private List<IAnnotationProject> projects;

	private WatchService watcher;

	private WatchKey key;

	private IEventBroker broker;

	public AnnotationWorkspace(File wsFile, IEventBroker broker) {
		this.wsFile = wsFile;
		this.broker = broker;

		// attach a FileWatcher to the workspace

		Runnable r = new Runnable() {

			@Override
			public void run() {
				try {
					watcher = FileSystems.getDefault().newWatchService();

					Path dir = AnnotationWorkspace.this.wsFile.toPath();

					registerAll(dir);
					// key = dir.register(watcher,
					// StandardWatchEventKinds.ENTRY_CREATE,
					// StandardWatchEventKinds.ENTRY_DELETE,
					// StandardWatchEventKinds.ENTRY_MODIFY);

					registerFileWatcher();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		};

		// execute the watch service

		Thread t = new Thread(r);
		t.start();

		this.projects = new ArrayList<IAnnotationProject>();

		for (File f : wsFile.listFiles()) {

			projects.add(new AnnotationProject(f));

		}
	}

	private void registerAll(final Path start) throws IOException {
		// register directory and sub-directories
		Files.walkFileTree(start, new SimpleFileVisitor<Path>() {
			@Override
			public FileVisitResult preVisitDirectory(Path dir, BasicFileAttributes attrs) throws IOException {
				register(dir);
				return FileVisitResult.CONTINUE;
			}
		});
	}

	private void register(Path dir) throws IOException {
		WatchKey key = dir.register(watcher, StandardWatchEventKinds.ENTRY_CREATE,
				StandardWatchEventKinds.ENTRY_DELETE, StandardWatchEventKinds.ENTRY_MODIFY);

		// if (trace) {
		// Path prev = keys.get(key);
		// if (prev == null) {
		// System.out.format("register: %s\n", dir);
		// } else {
		// if (!dir.equals(prev)) {
		// System.out.format("update: %s -> %s\n", prev, dir);
		// }
		// }
		// }
		// keys.put(key, dir);
	}

	private void registerFileWatcher() throws IOException {

		for (;;) {

			// wait for key to be signaled
			WatchKey key;
			try {
				key = watcher.take();
			} catch (InterruptedException x) {
				return;
			}

			for (WatchEvent<?> event : key.pollEvents()) {
				WatchEvent.Kind<?> kind = event.kind();

				// This key is registered only
				// for ENTRY_CREATE events,
				// but an OVERFLOW event can
				// occur regardless if events
				// are lost or discarded.
				if (kind == StandardWatchEventKinds.OVERFLOW) {
					continue;
				}
				// get the full path of the changed file
				Path dir = (Path) key.watchable();
				Path fullPath = dir.resolve((Path) event.context());

				if (event.kind() == StandardWatchEventKinds.ENTRY_CREATE) {
					System.out.println("Created: " + fullPath);
					broker.send(IAnnotationEditorEvents.RELOAD_PROJECTS, fullPath);
				}
				if (event.kind() == StandardWatchEventKinds.ENTRY_DELETE) {
					System.out.println("Delete: " + fullPath);
					broker.send(IAnnotationEditorEvents.FILE_DELETED_ACTION_EVENT, fullPath);
				}
				if (event.kind() == StandardWatchEventKinds.ENTRY_MODIFY) {
					System.out.println("Modify: " + fullPath);
					broker.send(IAnnotationEditorEvents.RELOAD_PROJECTS, fullPath);

					// notify the editor, if the underlying file is modified the
					// editor should reload
					broker.send(IAnnotationEditorEvents.FILE_MODIFIED, fullPath.toFile());
					System.out.println("Modified:" + fullPath);
				}

			}

			// Reset the key -- this step is critical if you want to
			// receive further watch events. If the key is no longer valid,
			// the directory is inaccessible so exit the loop.
			boolean valid = key.reset();
			if (!valid) {
				break;
			}
		}

	}

	public File getWsFile() {
		return wsFile;
	}

	public void setWsFile(File wsFile) {
		this.wsFile = wsFile;
	}

	public List<IAnnotationProject> getProjects() {
		return projects;
	}

	public void setProjects(List<IAnnotationProject> projects) {
		this.projects = projects;
	}

	public void addProject(IAnnotationProject project) {
		this.projects.add(project);
	}
}
