package de.uniwue.mk.kall.athen.projectExplorer.ui;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.TreeItem;
import org.osgi.service.event.EventHandler;

import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.projectExplorer.actions.DeleteResourceAction;
import de.uniwue.mk.kall.athen.projectExplorer.actions.ImportDataAction;
import de.uniwue.mk.kall.athen.projectExplorer.actions.OpenRemainingAction;
import de.uniwue.mk.kall.athen.projectExplorer.actions.OpenXMIResourceAction;
import de.uniwue.mk.kall.athen.projectExplorer.projectDescription.AnnotationWorkspace;
import de.uniwue.mk.kall.athen.projectExplorer.projectDescription.ProjectManager;

public class ProjectExplorer {

	private TreeViewer tree;

	private Composite composite;

	private AnnotationWorkspace workspace = null;

	private ProjectManager manager;

	// handler
	private WorkspaceChangedEventHandler wsChangedHandler;

	@Inject
	private EPartService partService;

	@Inject
	private EModelService modelService;

	@Inject
	MApplication app;

	@Inject
	IEventBroker broker;

	private boolean isAboutToClose = false;

	@Inject
	public ProjectExplorer() {
		// TODO Your code here
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Constrcut Explorer");

		// // // this was used for debugging
		// Preferences preferences =
		// InstanceScope.INSTANCE.getNode(ApplicationUtil.APPLICATION_ID);
		// try {
		// preferences.clear();
		// } catch (BackingStoreException e1) {
		// // TODO Auto-generated catch block
		// e1.printStackTrace();
		// }

		this.composite = parent;

		tree = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		tree.setContentProvider(new ProjectExplorerContentProvider());
		tree.setLabelProvider(new ProjectExplorerStyledLabelProvider());

		initHandler();

		MasterPart.subscribeForEvent(ProjectExplorer.class, IAnnotationEditorEvents.WORKSPACE_CHANGED_EVENT,
				wsChangedHandler);

		// a workspace was set already ????

		String wsPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.WORKSPACE, "nope");

		if (!wsPref.equals("nope")) {
			this.workspace = new AnnotationWorkspace(new File(wsPref), broker);
			this.tree.setInput(workspace);
		}

		this.manager = new ProjectManager(workspace, broker);

		// if (workspace != null) {
		// tree.setInput(workspace);
		// }

		// add listener

		tree.getControl().addListener(SWT.MouseDoubleClick, new Listener() {

			@Override
			public void handleEvent(Event event) {

				// get selection
				IStructuredSelection selection = (IStructuredSelection) ProjectExplorer.this.tree.getSelection();

				Object firstElement = selection.getFirstElement();

				if (firstElement instanceof File) {

					File f = (File) firstElement;

					if (f.getAbsolutePath().endsWith(".xmi")) {

						OpenXMIResourceAction openAction = new OpenXMIResourceAction(f, broker, partService);
						openAction.run();

					} else {

						OpenRemainingAction openRemainingAction = new OpenRemainingAction(f, broker);
						openRemainingAction.run();
					}

				}

			}
		});

		tree.getControl().addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void keyPressed(KeyEvent e) {

				if (e.keyCode == SWT.CR) {

					// get selection
					IStructuredSelection selection = (IStructuredSelection) ProjectExplorer.this.tree.getSelection();

					Object firstElement = selection.getFirstElement();

					if (firstElement instanceof File) {

						File f = (File) firstElement;

						if (f.getAbsolutePath().endsWith(".xmi")) {

							OpenXMIResourceAction openAction = new OpenXMIResourceAction(f, broker, partService);
							openAction.run();

							// broker.send(AnnotationEditorEvents.OPEN_XMI_REQUESTED,
							// f);

						}
					}

				}

				// allow delete TODO maybe add a dialog?
				else if (e.keyCode == SWT.DEL) {
					TreeItem[] multiSelection2 = tree.getTree().getSelection();
					DeleteResourceAction delAction = new DeleteResourceAction(multiSelection2, broker);
					delAction.run();
				}

				// copy!!
				else if ((e.stateMask & SWT.CTRL) == SWT.CTRL && e.keyCode == 'c') {
					Clipboard clipboard = new Clipboard(Display.getCurrent());
					String[] selectedFiles = determineSelectedFiles();
					clipboard.setContents(new Object[] { selectedFiles },
							new Transfer[] { FileTransfer.getInstance() });
					clipboard.dispose();
				}

				// paste!
				else if ((e.stateMask & SWT.CTRL) == SWT.CTRL && e.keyCode == 'v') {
					Clipboard clipboard = new Clipboard(Display.getCurrent());

					Object contents = clipboard.getContents(FileTransfer.getInstance());

					File parentFolder = determineParentFolderOfPaste();
					if (parentFolder == null) {
						return;
					}
					if (contents instanceof String[]) {
						String[] selectedFileNames = (String[]) contents;
						for (String s : selectedFileNames) {
							try {
								FileOutputStream fos = new FileOutputStream(new File(
										parentFolder.getAbsolutePath() + File.separator + new File(s).getName()));
								Files.copy(new File(s).toPath(), fos);
								
								fos.close();
							} catch (IOException e1) {
								e1.printStackTrace();
							}
						}
					}
					clipboard.dispose();
				}
			}

			private File determineParentFolderOfPaste() {
				IStructuredSelection sel = (IStructuredSelection) tree.getSelection();
				Object firstElement = sel.getFirstElement();

				if (firstElement instanceof File) {
					File selFile = (File) firstElement;

					if (selFile.isDirectory())
						return selFile;
					else if (selFile.getParentFile().isDirectory()) {
						return selFile.getParentFile();
					}
				}
				return null;
			}

		});

		// dragn drop for the tree

		int ops = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transfers = new Transfer[] { LocalSelectionTransfer.getTransfer(),FileTransfer.getInstance() };
		tree.addDragSupport(ops, transfers, new ProjectExplorerDragSourceListener(tree));
		tree.addDropSupport(ops, transfers, new ProjectExplorerDropAdapter(tree));

		// menu

		// create menumanager
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(tree.getControl());

		// attach that menu manager to the treeviewer
		tree.getControl().setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {
				if (tree.getSelection().isEmpty()) {
					return;
				}

				if (tree.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection) tree.getSelection();
					Object object = selection.getFirstElement();

					// this one is used to be able to delete all selected at
					// once
					TreeItem[] multiSelection2 = tree.getTree().getSelection();

					// correctionFiles can be deleted
					if (object instanceof File) {
						File file = (File) object;

						if (!file.isDirectory()) {
							// we add "delete resource" action
							DeleteResourceAction delAction = new DeleteResourceAction(multiSelection2, broker);
							manager.add(delAction);

							// add the open action

							OpenXMIResourceAction openAction = new OpenXMIResourceAction(file, broker, partService);
							manager.add(openAction);

						}

						else if (file.isDirectory() && file.getName().contains("input")) {

							// add an import action
							ImportDataAction importDataAction = new ImportDataAction(broker, composite.getShell(),
									file);

							manager.add(importDataAction);
						}

					}
				}
			}
		});

		menuMgr.setRemoveAllWhenShown(true);

		// =====register for events

		// boolean notifies that the tree should be reload as its fully expaned
		// state or not
		MasterPart.subscribeForEvent(ProjectExplorer.class, IAnnotationEditorEvents.RELOAD_PROJECTS,
				new ReloadProjectHandler(this, false));

		MasterPart.subscribeForEvent(ProjectExplorer.class, IAnnotationEditorEvents.FILE_DELETED_ACTION_EVENT,
				new ReloadProjectHandler(this, false));

	}

	private void initHandler() {

		this.wsChangedHandler = new WorkspaceChangedEventHandler();

	}

	/*
	 * called when the workspace is changed
	 */
	public void onWorkSpaceChanged(AnnotationWorkspace ws) {
		this.workspace = ws;

		reload(true);

	}

	public AnnotationWorkspace getWorkspace() {
		return workspace;
	}

	public void setWorkspace(AnnotationWorkspace workspace) {
		this.workspace = workspace;
	}

	public ProjectManager getManager() {
		return manager;
	}

	@PreDestroy
	public void preDestroy() {

		this.isAboutToClose = true;
	}

	@Focus
	public void onFocus() {
		// TODO Your code here
	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	public void reload(boolean fullyExpand) {

		Runnable r = new Runnable() {

			@Override
			public void run() {

				if (!tree.getControl().isDisposed()) {
					Object[] expandedElements = ProjectExplorer.this.tree.getExpandedElements();
					ProjectExplorer.this.tree.setInput(workspace);
					ProjectExplorer.this.tree.refresh();

					ProjectExplorer.this.tree.setExpandedElements(expandedElements);
					composite.layout();

				}
			}
		};

		if (!this.isAboutToClose) {
			composite.getDisplay().asyncExec(r);
		}

	}

	//get all non directory files that are in current selection
	private String[] determineSelectedFiles() {
		
	IStructuredSelection sel = 	(IStructuredSelection) tree.getSelection();
	
	Iterator iterator = sel.iterator();
	List<String> fileList = new LinkedList<>();
	while(iterator.hasNext()){
		Object next = iterator.next();
		
		if(next instanceof File){
			fileList.add(((File)next).getAbsolutePath());
		}
	}
		return fileList.toArray(new String[0]);
	}

	// =====INNER CLASSES

	private class WorkspaceChangedEventHandler implements EventHandler {

		@Override
		public void handleEvent(org.osgi.service.event.Event arg0) {

			AnnotationWorkspace ws = new AnnotationWorkspace(new File((String) arg0.getProperty(IEventBroker.DATA)),
					broker);

			onWorkSpaceChanged(ws);
		}

	}

}