package de.uniwue.mk.kall.athen.projectExplorer.ui;

import java.io.File;
import java.net.URL;
import java.util.HashMap;

import javax.annotation.PreDestroy;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Image;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.projectExplorer.projectDescription.IAnnotationProject;

public class ProjectExplorerStyledLabelProvider extends StyledCellLabelProvider {

	private TreeViewer viewer;

	private HashMap<String, Image> imageMapping = new HashMap<String, Image>();

	@Override
	public void update(ViewerCell cell) {

		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof IAnnotationProject) {
			IAnnotationProject proj = (IAnnotationProject) element;

			if (proj.getProjectFolder().isDirectory()) {
				text.append(getFileName(proj.getProjectFolder()));
				cell.setImage(createImage(ApplicationUtil.PROJECT_IMAGE));
				String[] files = proj.getProjectFolder().list();
				if (files != null) {
					text.append(" (" + files.length + ") ", StyledString.COUNTER_STYLER);
				}
			} else {
				text.append(getFileName(proj.getProjectFolder()));
			}
		}

		else {
			File file = (File) element;
			if (file.isDirectory()) {
				text.append(getFileName(file));
				cell.setImage(createImage(ApplicationUtil.FOLDER_IMAGE));
				String[] files = file.list();
				if (files != null) {
					text.append(" (" + files.length + ") ", StyledString.COUNTER_STYLER);
				}
			} else {

				// its a real file

				if (file.getAbsolutePath().endsWith(".xmi")) {
					// UIMA FILE
					cell.setImage(createImage(ApplicationUtil.XMI_IMAGE));

				}

				else if (file.getAbsolutePath().endsWith(".xml")) {
					// descriptor
					cell.setImage(createImage(ApplicationUtil.XML_IMAGE));

				}
				text.append(getFileName(file));
			}
		}

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	private String getFileName(File file) {
		String name = file.getName();
		return name.isEmpty() ? file.getPath() : name;
	}

	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	@PreDestroy
	public void dispose() {

		for (Image i : imageMapping.values()) {
			i.dispose();
		}
	}

	private Image createImage(String image) {

		if (imageMapping.containsKey(image)) {
			return imageMapping.get(image);

		}

		Bundle bundle = FrameworkUtil.getBundle(ProjectExplorerStyledLabelProvider.class);
		URL url = FileLocator.find(bundle, new Path(image), null);
		ImageDescriptor imageDcr = ImageDescriptor.createFromURL(url);
		Image createImage = imageDcr.createImage();

		imageMapping.put(image, createImage);

		return imageDcr.createImage();
	}
}
