package de.uniwue.mk.kall.athen.relationview.ui;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class RelationMetaAnnotationDialog extends Dialog {

	private RelationAnnotationView view;

	private StyledText widget;

	private Text textFromRelation;
	private ComboViewer viewer;
	private Text textToRelation;

	private Text textSearchfiled;
	private Button buttonPreviousHit;
	private Button buttonNextHit;

	private static String KEY_PRESS = "Ctrl+Space";

	// matcher related
	private String lastQuery = "";
	private int lastindex;
	private Point actualmatch;

	private Shell shell;

	public RelationMetaAnnotationDialog(Shell parentShell, RelationAnnotationView view) {
		super(parentShell);

		this.shell = parentShell;

		this.shell.setSize(800, 800);
		this.view = view;

		setShellStyle(getShellStyle() | SWT.RESIZE);
		setShellStyle(getShellStyle() | SWT.SHELL_TRIM);

	}

	protected Control createDialogArea(final Composite parent) {

		parent.getShell().setText("Add a new Meta Annotation");
		// parent.getShell().setSize(800, 800);

		GridLayout layout = new GridLayout();
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		layout.numColumns = 1;
		parent.setLayout(layout);

		// at first there is a row with 2 Textfield and a comboviewer
		Composite c1 = new Composite(parent, SWT.NONE);

		c1.setLayout(new GridLayout(3, false));

		textFromRelation = new Text(c1, SWT.BORDER);
		textFromRelation.setSize(100, textFromRelation.getSize().y);

		textFromRelation.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				// Method for autocompletion
				setAutoCompletion(textFromRelation, textFromRelation.getText());
			}
		});

		viewer = new ComboViewer(c1);

		viewer.setContentProvider(ArrayContentProvider.getInstance());

		viewer.setInput(view.getAllRelationTypes());

		textToRelation = new Text(c1, SWT.BORDER);

		textToRelation.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				// Method for autocompletion
				setAutoCompletion(textToRelation, textToRelation.getText());
			}
		});

		// then there is a textsearchfield with 2 buttons Previous and Next

		Composite c2 = new Composite(parent, SWT.BORDER);

		c2.setLayout(new GridLayout(3, false));
		textSearchfiled = new Text(c2, SWT.BORDER);
		textSearchfiled.setText("Enter Searchstring");

		buttonNextHit = new Button(c2, SWT.PUSH);

		buttonNextHit.setText("next match");

		buttonNextHit.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String text = textSearchfiled.getText();

				Matcher m = Pattern.compile(Pattern.quote(text)).matcher(view.getEditor().getCas().getDocumentText());

				int startIndex = 0;

				if (lastQuery.equals(text)) {
					startIndex = lastindex;
				}

				else {
					lastindex = 0;
				}

				Point range = new Point();
				while (m.find(startIndex)) {

					lastindex = m.end();

					lastQuery = m.group();

					range.x = m.start();
					range.y = m.end();
					break;
				}

				actualmatch = range;
				actualmatch.x = actualmatch.x - 150;

				// highlight this range

				if (range.x - 150 > 0 && range.y + 150 < view.getEditor().getCas().getDocumentText().length()) {

					widget.setText(view.getEditor().getCas().getDocumentText().substring(range.x - 150, range.y + 150));
				}

				widget.pack();
				parent.layout();
				parent.getShell().pack();
				parent.getShell().layout();

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});
		buttonPreviousHit = new Button(c2, SWT.PUSH);

		buttonPreviousHit.setText("previous match");

		buttonPreviousHit.setVisible(false);
		buttonPreviousHit.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String text = textSearchfiled.getText();

				Matcher m = Pattern.compile(Pattern.quote(text)).matcher(
						view.getEditor().getCas().getDocumentText().substring(0, lastindex));

				int startIndex = 0;

				if (lastQuery.equals(text)) {
					startIndex = lastindex;
				}

				else {
					lastindex = 0;
				}

				Point range = new Point();
				while (m.find()) {

					lastindex = m.end();

					lastQuery = m.group();

					range.x = m.start();
					range.y = m.end();
				}

				// highlight this range

				if (range.x - 150 > 0 && range.y + 150 < view.getEditor().getCas().getDocumentText().length()) {

					widget.setText(view.getEditor().getCas().getDocumentText().substring(range.x - 150, range.y + 150));
				}

				widget.pack();
				parent.layout();
				parent.getShell().pack();
				parent.getShell().layout();

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// last but not least a widget that contains all the text in the cas

		// Create a child composite to hold the controls
		Composite child = new Composite(parent, SWT.NONE);
		child.setLayout(new FillLayout());

		// c3.setLayout(new FillLayout());
		widget = new StyledText(child, SWT.V_SCROLL | SWT.H_SCROLL | SWT.WRAP);

		// widget.setAlwaysShowScrollBars(true);

		widget.pack();
		widget.setEditable(false);

		// GridData layoutData = new GridData();
		//
		// layoutData.horizontalSpan = 3;
		// widget.setLayoutData(layoutData);

		String text = "                                                                                           \n";
		text += "                                                                                           \n";
		text += "                                                                                           \n";
		widget.setText(text);

		widget.layout();

		parent.redraw();

		return parent;
	}

	private void setAutoCompletion(Text text, String value) {
		try {
			ContentProposalAdapter adapter = null;
			String[] defaultProposals = getAllProposals(value);
			SimpleContentProposalProvider scp = new SimpleContentProposalProvider(defaultProposals);
			scp.setProposals(defaultProposals);
			KeyStroke ks = KeyStroke.getInstance(KEY_PRESS);
			adapter = new ContentProposalAdapter(text, new TextContentAdapter(), scp, ks, null);
			adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private String[] getAllProposals(String text) {
		String[] proposals = new String[5];
		if (text == null || text.length() == 0)
			proposals = new String[] {};
		else {
			List<String> NeStrings = view.getAllNEsAsString();

			List<String> candidates = new ArrayList<String>();

			for (String ne : NeStrings) {

				if (ne.startsWith(text)) {
					candidates.add(ne);
				}
			}
			proposals = candidates.toArray(new String[0]);
		}
		return proposals;
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

	@Override
	protected boolean isResizable() {
		return true;
	}

	@Override
	protected void okPressed() {

		// add an annotation
		AnnotationEditorWidget editor = view.getEditor();

		CAS cas = view.getEditor().getCas();

		int beg = 0, end = 1;

		if (widget.getSelection() != null && widget.getSelection().y - widget.getSelection().x != 0) {

			beg = actualmatch.x - 150 + widget.getSelection().x;
			end = actualmatch.x - 150 + widget.getSelection().y;
		}
		AnnotationFS fs = cas.createAnnotation(view.getRelationType(), beg, end);

		String selectedType = (String) ((IStructuredSelection) viewer.getSelection()).getFirstElement();

		fs.setFeatureValueFromString(view.getRelationTypeFeature(), selectedType);
		fs.setFeatureValueFromString(view.getRelationDescription(), textFromRelation.getText() + " " + selectedType
				+ " " + textToRelation.getText());

		// also add snapped features
		List<AnnotationFS> coveringNes = view.getCoveringNes(fs);

		if (coveringNes.size() > 0) {

			fs.setFeatureValue(view.getAgensFeature(), coveringNes.get(0));
			fs.setFeatureValue(view.getAgens2Feature(), coveringNes.get(coveringNes.size() - 1));

		}

		editor.addAnnotation(fs);

		// also add overrride

		if (coveringNes.size() > 0) {
			AnnotationFS ovverr = cas.createAnnotation(view.getOverwriteType(), coveringNes.get(0).getBegin(),
					coveringNes.get(0).getEnd());

			editor.addAnnotation(ovverr);

			AnnotationFS ovverr2 = cas.createAnnotation(view.getOverwriteType(), coveringNes
					.get(coveringNes.size() - 1).getBegin(), coveringNes.get(coveringNes.size() - 1).getEnd());

			editor.addAnnotation(ovverr2);
		}
		setReturnCode(OK);
		close();
	}
}
