package de.uniwue.mk.kall.athen.relationview.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class RelationSelectTypeInputDialog extends Dialog {

	private Text text;

	private static String KEY_PRESS = "Ctrl+Space";
	private RelationAnnotationView view;

	private String selectedType;

	public RelationSelectTypeInputDialog(Shell parentShell, RelationAnnotationView view) {
		super(parentShell);

		this.view = view;
	}

	protected Control createDialogArea(Composite parent) {

		parent.getShell().setText("Select a Relationtype");

		GridLayout layout = new GridLayout();
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		layout.numColumns = 1;
		parent.setLayout(layout);

		text = new Text(parent, SWT.BORDER);
		text.setSize(200, text.getSize().y);

		text.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				// Method for autocompletion
				setAutoCompletion(text, text.getText());
			}
		});

		return parent;
	}

	private String[] getAllProposals(String text) {
		String[] proposals = new String[5];
		if (text == null || text.length() == 0)
			proposals = new String[] {};
		else {
			String[] types = view.getAllRelationTypes();

			List<String> typesMatching = new ArrayList<String>();

			for (String s : types) {
				if (s.startsWith(text)) {
					typesMatching.add(s);
				}
			}

			proposals = typesMatching.toArray(new String[0]);
		}
		return proposals;
	}

	private void setAutoCompletion(Text text, String value) {
		try {
			ContentProposalAdapter adapter = null;
			String[] defaultProposals = getAllProposals(value);
			SimpleContentProposalProvider scp = new SimpleContentProposalProvider(defaultProposals);
			scp.setProposals(defaultProposals);
			KeyStroke ks = KeyStroke.getInstance(KEY_PRESS);
			adapter = new ContentProposalAdapter(text, new TextContentAdapter(), scp, ks, null);
			adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public String getSelectedType() {
		return this.selectedType;
	}

	@Override
	protected void okPressed() {

		this.selectedType = text.getText();
		setReturnCode(OK);
		close();
	}

}
