package de.uniwue.mk.kall.athen.sceneannotationview.ui;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.text.JFaceTextUtil;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.BorderSceneDrawingStrategy;

public class SceneAnnotationView extends AEditorSubordinateViewPart {

	private static final String STUMP = "STUMP";

	// copy and modify this class if you want to create a new view
	private static final Class<SceneAnnotationView> CLASS_THIS = SceneAnnotationView.class;

	@Inject
	static EPartService partService;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.17";

	// DATA CONTAINER - the model, since its usually very simple we have it here
	private List<AnnotationFS> annoList;
	private List<SceneStump> stumpsList;

	// the main type this view is about
	private Type sceneType;

	// the Stringrepresentation of this maintype - used to get the type of the
	// preferencestore
	private String stringSceneType;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	// all Features (UIMA) this view needs to know about as well as other types
	private Feature featSceneMarker;
	private Feature featSceneType;
	private Feature featSceneLinkToScene;
	private Feature featSceneName;

	private Type typeSceneMarker;
	private Feature featScenemarkerType;

	// compounds - UI

	private Button buttonSceneBegin;
	private Text textSceneName;
	private Button buttonRemoveScene;
	private Button buttonAddSceneMarker;
	private Button buttonRemoveSceneMarker;
	private Button buttonAddSceneUebergang;
	private Button buttonAddNoScene;

	private TableViewer viewer;

	private Composite middleComp;

	// UI Helper

	// to change the sorting direction in the table
	// private ColViewerComparator colViewerComp;

	@Inject
	public SceneAnnotationView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		// layout this should stay as it is
		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);
	}

	public void initModel() {
		annoList = new ArrayList<AnnotationFS>();
		stumpsList = new ArrayList<SceneStump>();
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(sceneType)) {
			annoList.add(anno);
		}

		renewPage();
	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		// TODO parse this from preferences

		// refresh the editor
		editor.allStylesInvisible();
		this.typeSceneMarker = editor.getCas().getTypeSystem().getType("de.uniwue.kallimachos.coref.type.SceneMarker");
		this.sceneType = editor.getCas().getTypeSystem().getType("de.uniwue.kallimachos.coref.type.Scene");
		this.featSceneLinkToScene = this.sceneType.getFeatureByBaseName("Link");
		this.featSceneMarker = this.sceneType.getFeatureByBaseName("SceneMarker");
		this.featSceneType = this.sceneType.getFeatureByBaseName("Type");
		this.featScenemarkerType = this.typeSceneMarker.getFeatureByBaseName("Type");
		this.featSceneName = this.sceneType.getFeatureByBaseName("Name");

		// Visibility
		editor.changeVisibility(sceneType, true);
		editor.changeVisibility(typeSceneMarker, true);

		// Styles - yellow is a marker
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(Display.getCurrent(), 233, 247, 74);
		AnnoStyle styleSceneMarker = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		// Boxes at the border for a scene
		StyleRange sRScene = new StyleRange();
		sRScene.background = new Color(Display.getCurrent(), 233, 247, 74);
		AnnoStyle styleScene = new AnnoStyle(new BorderSceneDrawingStrategy(), sRScene);

		// apply
		editor.changeStyle(typeSceneMarker, styleSceneMarker);
		editor.changeStyle(sceneType, styleScene);

		// also change the modus
		editor.setActiveMode(sceneType);

		return true;

	}

	private void updateSceneCoverage() {

		for (AnnotationFS scene : editor.getCas().getAnnotationIndex(sceneType)) {

			boolean covered = false;

			for (AnnotationFS sceneDummy : editor.getCas().getAnnotationIndex(sceneType)) {

				if (sceneDummy.getBegin() < scene.getBegin() && sceneDummy.getEnd() >= scene.getEnd()
						|| sceneDummy.getBegin() <= scene.getBegin() && sceneDummy.getEnd() > scene.getEnd()) {

					covered = true;
					break;

				}
			}

			if (covered) {

				// set the feature value
				Feature featCovered = sceneType.getFeatureByBaseName("Covered");
				scene.setFeatureValueFromString(featCovered, "true");
			}

			else {
				// set the feature value
				Feature featCovered = sceneType.getFeatureByBaseName("Covered");
				scene.setFeatureValueFromString(featCovered, "false");
			}
		}

	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

		parent.setLayout(new GridLayout(1, false));
		Composite topComp = new Composite(parent, SWT.NONE);

		topComp.setLayout(new GridLayout(3, false));
		topComp.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));

		// add the buttons
		buttonSceneBegin = new Button(topComp, SWT.PUSH);
		buttonSceneBegin.setText("Begin: Scene");

		buttonSceneBegin.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// we use the current caretposition as start for the annotation
				String defaultName = calculateDefaultName("Szene");
				stumpsList.add(new SceneStump(JFaceTextUtil.getLineIndex(editor.getWidget(), editor.getWidget()
						.getCaretOffset()), editor.getWidget().getCaretOffset(),
						textSceneName.getText().equals("") ? defaultName : textSceneName.getText(), "Szene"));

				updateMiddleComposite();

			}

		});

		buttonAddNoScene = new Button(topComp, SWT.PUSH);
		buttonAddNoScene.setText(" Begin: No Scene");
		buttonAddNoScene.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// we use the current caretposition as start for the annotation
				String defaultName = calculateDefaultName("keine Szene");
				stumpsList.add(new SceneStump(JFaceTextUtil.getLineIndex(editor.getWidget(), editor.getWidget()
						.getCaretOffset()), editor.getWidget().getCaretOffset(),
						textSceneName.getText().equals("") ? defaultName : textSceneName.getText(), "keine Szene"));

				updateMiddleComposite();

			}

		});

		buttonAddSceneUebergang = new Button(topComp, SWT.PUSH);
		buttonAddSceneUebergang.setText("Begin: Scenetransition");
		buttonAddSceneUebergang.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// we use the current caretposition as start for the annotation
				String defaultName = calculateDefaultName("Szenenbergang");
				stumpsList.add(new SceneStump(JFaceTextUtil.getLineIndex(editor.getWidget(), editor.getWidget()
						.getCaretOffset()), editor.getWidget().getCaretOffset(),
						textSceneName.getText().equals("") ? defaultName : textSceneName.getText(), "Szenenbergang"));

				updateMiddleComposite();

			}

		});

		Label labelSceneName = new Label(topComp, SWT.NONE);
		labelSceneName.setText("Name:");

		textSceneName = new Text(topComp, SWT.BORDER);
		textSceneName.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));

		buttonRemoveScene = new Button(topComp, SWT.PUSH);
		buttonRemoveScene.setText(" Remove  Scene   ");
		buttonRemoveScene.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				List<AnnotationFS> actualSelection = editor.getSelectionOfVisibleAnnotations();

				List<AnnotationFS> scenesinSelection = new ArrayList<AnnotationFS>();

				for (AnnotationFS anno : actualSelection) {

					if (anno.getType().equals(sceneType)) {
						scenesinSelection.add(anno);
					}
				}

				if (scenesinSelection.size() == 0)
					return;

				if (scenesinSelection.size() == 1) {
					editor.removeAnnotation(scenesinSelection.get(0));
				}

				else {
					// delete the one that is subsumed
					int maxBeg = -1;
					AnnotationFS subsumedScene = null;
					for (AnnotationFS scene : scenesinSelection) {

						if (scene.getBegin() > maxBeg) {
							maxBeg = scene.getBegin();
							subsumedScene = scene;
						}
					}

					if (subsumedScene != null) {
						editor.removeAnnotation(subsumedScene);
					}

					// if no such is found the shortest
					if (subsumedScene == null) {
						editor.removeAnnotation(scenesinSelection.get(0));
					}

				}

			}

		});

		// Button buttonDummy1 = new Button(topComp, SWT.PUSH);
		// buttonDummy1.setVisible(false);
		// Button buttonDummy2 = new Button(topComp, SWT.PUSH);
		// buttonDummy2.setVisible(false);

		buttonAddSceneMarker = new Button(topComp, SWT.PUSH);
		buttonAddSceneMarker.setText(" Add  Scenemarker ");

		buttonAddSceneMarker.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				Point selection = editor.getWidget().getSelection();

				if (selection != null && selection.y - selection.x > 0) {

					AnnotationFS sceneMarker = editor.getCas().createAnnotation(typeSceneMarker, selection.x,
							selection.y);

					editor.addAnnotation(sceneMarker);
				}

			}

		});

		Button buttonDummy3 = new Button(topComp, SWT.PUSH);
		buttonDummy3.setVisible(false);

		buttonRemoveSceneMarker = new Button(topComp, SWT.PUSH);
		buttonRemoveSceneMarker.setText("Remove Scenemarker");
		buttonRemoveSceneMarker.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				int caretOffset = editor.getWidget().getCaretOffset();
				for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typeSceneMarker)) {

					if (anno.getBegin() <= caretOffset && anno.getEnd() >= caretOffset) {
						editor.removeAnnotation(anno);
					}
				}

			}

		});
		// Label l = new Label(parent, SWT.NONE);
		// l.setText("Wo bin ich");

		// middle comp that represents all open selections
		middleComp = new Composite(parent, SWT.NONE);

		middleComp.setLayout(new GridLayout(3, false));
		middleComp.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));

		middleComp.setVisible(false);

		// the table to check what youve done so far

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		viewer.setContentProvider(new ArrayContentProvider());
		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		table.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_BOTH | GridData.GRAB_VERTICAL));
		createColumns(viewer);

		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				if (firstElement != null) {

					editor.highlightRange(firstElement.getBegin(), firstElement.getEnd() - firstElement.getBegin(),
							true);
				}
			}
		});

		// this.colViewerComp = new ColViewerComparator(null, null, null, this);
		// viewer.setComparator(colViewerComp);

	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Name", 100, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(featSceneName);

			}

		});

		TableViewerColumn colColor = createTableViewerColumn("Type", 50, 0);
		colColor.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;

				return p.getFeatureValueAsString(featSceneType);

			}

		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	// use this method to update the view
	public void renewPage() {
		updateModel();

		// redraw in a new Thread, executed by the Display
		Runnable r = new Runnable() {

			@Override
			public void run() {
				createTableViewer();

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void updateModel() {
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(sceneType);

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

	}

	// Composite of the scenestumps
	private void updateMiddleComposite() {

		// clear middle comp
		for (Control c : middleComp.getChildren()) {
			c.dispose();
		}
		middleComp.setVisible(true);

		// init layout based on stumps

		for (SceneStump stump : this.stumpsList) {

			Label labelName = new Label(middleComp, SWT.BORDER);
			labelName.setText(stump.getName());

			final Text textType = new Text(middleComp, SWT.BORDER);
			textType.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));
			textType.setText(stump.getType());

			final Button buttonEndScene = new Button(middleComp, SWT.PUSH);
			buttonEndScene.setText("End Scene");
			buttonEndScene.setData(STUMP, stump);

			buttonEndScene.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {

					SceneStump stump = (SceneStump) buttonEndScene.getData(STUMP);

					// no annotation without text
					if (stump.getAnnoStart() == editor.getWidget().getCaretOffset()
							|| editor.getWidget().getCaretOffset() < stump.getAnnoStart())
						return;

					AnnotationFS fs = editor.getCas().createAnnotation(sceneType, stump.getAnnoStart(),
							editor.getWidget().getCaretOffset());

					fs.setFeatureValueFromString(featSceneName, stump.getName());
					fs.setFeatureValueFromString(featSceneType, textType.getText());

					// TODO set features
					editor.addAnnotation(fs);

					// remove from modellist
					stumpsList.remove(stump);

					updateMiddleComposite();
				}

			});
		}

		middleComp.layout();
		super.overlay.layout();
	}

	private void createTableViewer() {

		updateModel();
		viewer.setInput(annoList);

		viewer.refresh();
	}

	@PreDestroy
	public void preDestroy() {
		// TODO unused
	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr
	}

	// Perspective listener
	@Override
	public void coupleToPerspective() {

		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);

	}

	private String calculateDefaultName(String type) {

		int sameSceneType = 0;
		for (AnnotationFS scene : editor.getCas().getAnnotationIndex(sceneType)) {
			String sceneType = scene.getFeatureValueAsString(featSceneType);

			System.out.println(sceneType + " " + type);
			if (sceneType.equals(type)) {
				sameSceneType++;
			}
		}

		System.out.println(sameSceneType);
		String defaultName = type + String.valueOf(sameSceneType + 1);
		return defaultName;
	}

	@Override
	public void decoupleFromPerspective() {
		System.out.println("Decouple CorefView");

		// free the expensive objects TODO better if this is in a separate
		// method
		MasterPart.unsubscribeFromEvent(CLASS_THIS, 
				IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, 
				onViewStackChangedHandler);
		super.standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);

	}

	// ====== Abstract Methods from Superclass

	// what to do when the editor has an added annotation?
	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {

		updateSceneCoverage();
		renewPage();
	}

	// what to do when the editor has an removed annotation?
	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {

		// update the iscovered which is required for the drawing
		updateSceneCoverage();
		renewPage();

	}

	// what to do when the editor has an changed annotation?
	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		AnnotationFS changed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);
		renewPage();

	}

	// what to do when the editor is clicked
	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

		if (point.x < editor.getWidget().getLeftMargin()) {

			AnnotationFS smallest = null;
			for (AnnotationFS scene : editor.getCas().getAnnotationIndex(sceneType)) {

				boolean doesCover = coveresRectangle(point, scene);
				if (doesCover) {

					if (smallest == null)
						smallest = scene;

					if (scene.getBegin() >= smallest.getBegin() && scene.getEnd() <= smallest.getEnd()) {
						smallest = scene;
					}

				}
			}
			// jump to the beginning
			if (smallest != null)
				editor.highlightRange(smallest.getBegin(), smallest.getEnd() - smallest.getBegin(), true);

		}

	}

	// on editor mouse moved - use this if you want some fancy effects
	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {
		Point point = (Point) e.getProperty(IEventBroker.DATA);

		if (point.x < editor.getWidget().getLeftMargin()) {

			// check if we are hovered over a rectangle

			boolean found = false;
			for (AnnotationFS scene : editor.getCas().getAnnotationIndex(sceneType)) {

				boolean doesCover = coveresRectangle(point, scene);
				if (doesCover) {
					editor.getWidget().setCursor(new Cursor(Display.getCurrent(), SWT.CURSOR_HAND));
					found = true;
					break;
				}
			}

			if (found) {

				return;
			}
		}
		editor.getWidget().setCursor(new Cursor(Display.getCurrent(), SWT.CURSOR_IBEAM));

	}

	private boolean coveresRectangle(Point point, AnnotationFS anno) {
		Rectangle textBounds = editor.getWidget().getTextBounds(0, 1);
		int lineTop = Math.abs(editor.getWidget().getTextBounds(anno.getBegin(), anno.getBegin() + 1).y - textBounds.y);
		int lineBottom = Math.abs(editor.getWidget().getTextBounds(anno.getEnd() - 1, anno.getEnd()).y - textBounds.y);

		Rectangle textBounds2 = editor.getWidget().getTextBounds(editor.getWidget().getText().length() - 2,
				editor.getWidget().getText().length() - 1);
		int lineCnt = Math.abs(textBounds.y) + Math.abs(textBounds2.y);

		int pixelTop = BorderSceneDrawingStrategy.lineToPixel(editor.getWidget().getBounds().height, lineTop, lineCnt);
		int pixelBottom = BorderSceneDrawingStrategy.lineToPixel(editor.getWidget().getBounds().height, lineBottom,
				lineCnt);

		if (point.y >= pixelTop && point.y <= pixelBottom) {
			return true;
		}

		return false;
	}

}