package de.uniwue.mk.kall.athen.selection.analyzer.ui;

import java.util.ArrayList;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.AnnotationImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.osgi.service.event.Event;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class SelectionAnalyzer extends AEditorSubordinateViewPart {

	// CONSTANTS

	private static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.6";

	// FIELDS======

	private static Class<?> CLASS_THIS = SelectionAnalyzer.class;

	// maybe use this manager in the whole app??
	private LocalResourceManager resManager;

	@Inject
	private static IEventBroker broker;
	@Inject
	static EPartService partService;

	@Inject
	EModelService modelService;

	// Widget
	private TreeViewer selectionTree;

	@Inject
	public SelectionAnalyzer() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);
	}

	@PreDestroy
	public void preDestroy() {

	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	public void initLayout(Composite parent) {

		// composite gets three coloumns

		parent.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		// add a Label/textfield for the types that are about to be compared

		// add selection listener

		// add selection listener

		// ====Start tree creration
		GridData gridData = new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);

		// /selection tree

		selectionTree = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		selectionTree.getControl().setLayoutData(gridData);

		selectionTree.setContentProvider(new SelectionAnalyzerTreeContentProvider(this));
		selectionTree.setLabelProvider(
				new SelectionAnalyzerLabelProvider(parent.getDisplay(), editor.getCas(), editor, this));

		selectionTree.getTree().setLayoutData(gridData);

		selectionTree.getTree().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) selectionTree.getSelection();

				Object firstElement = sel.getFirstElement();

				if (firstElement instanceof AnnotationImpl) {

					AnnotationImpl anno = (AnnotationImpl) firstElement;

					editor.highlightRange(anno.getBegin(), anno.getEnd() - anno.getBegin(), true);
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		selectionTree.getTree().addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {

				if (e.character == SWT.DEL) {
					IStructuredSelection sel = (IStructuredSelection) selectionTree.getSelection();

					// remove the elements
					Object firstElement = sel.getFirstElement();
					if (firstElement instanceof AnnotationFS) {
						AnnotationFS anno = (AnnotationFS) firstElement;
						editor.removeAnnotation(anno);
					}
				}
			}

		});

		createTreeViewer();

	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void setEditor(AnnotationEditorWidget editor) {
		this.editor = editor;
	}

	private void createTreeViewer() {

		if (selectionTree == null)
			return;
		selectionTree.refresh();
		selectionTree.expandToLevel(1);

		super.overlay.layout();

	}

	public void renewPage() {

		createTreeViewer();

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	public TreeViewer getViewer() {
		return this.selectionTree;
	}

	@Override
	public void onAnnotationAdded(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationRemoved(Event arg0) {
		renewPage();

	}

	@Override
	public void onAnnotationChanged(Event arg0) {
		renewPage();

	}

	@Override
	public void onEditorCaretChanged(Event arg0) {
		if (editor == null) {
			return;
		}
		int caretPosition = (int) arg0.getProperty(IEventBroker.DATA);

		// get the 3 Annotation indexes

		ArrayList<AnnotationFS> covered = new ArrayList<AnnotationFS>();

		for (TypeStyleDataStructure ts : editor.getStyleStruct().getStylesFromLowestLayerToHighest()) {

			Type t = ts.getAnnotationType();
			for (AnnotationFS tp : editor.getCas().getAnnotationIndex(t)) {
				if (tp.getBegin() <= caretPosition && tp.getEnd() >= caretPosition) {
					covered.add(tp);
				}
				if (tp.getBegin() > caretPosition)
					break;
			}
		}

		//
		if (covered != null && selectionTree != null) {
			selectionTree.setInput(covered);
		}
		renewPage();

	}

	@Override
	public void initModel() {
		// no model in this view

	}

	@Override
	public boolean initTypes() {
		// we do not need any types
		return true;
	}

}
