package de.uniwue.mk.kall.athen.selection.analyzer.ui;

import java.util.HashMap;

import javax.annotation.PreDestroy;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.StyleDataStructure;
import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class SelectionAnalyzerLabelProvider extends StyledCellLabelProvider {

	HashMap<String, String> featureMap = new HashMap<String, String>();
	HashMap<Type, Image> typesHash = new HashMap<Type, Image>();
	private AnnotationFS selectedAnno;

	private AnnotationEditorWidget editor;

	private GC gc;

	private Display display;

	private SelectionAnalyzer view;

	public SelectionAnalyzerLabelProvider(Display display, CAS cas, AnnotationEditorWidget editor2,
			SelectionAnalyzer view) {
		this.gc = new GC(display);
		this.display = display;
		this.editor = editor2;
		this.view = view;
	}

	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof Type) {

			Type tsds = (Type) element;

			text.append(tsds.getShortName());

			// create the image

			setImage(cell, editor.getDSFromType(tsds));

		}

		else if (element instanceof AnnotationFS) {

			AnnotationFS anno = (AnnotationFS) element;
			selectedAnno = anno;

			text.append(anno.getCoveredText());

			// set iomage

			setImage(cell, getTsdsFromAnno(anno));
		}

		else if (element instanceof Feature) {

			Feature f = (Feature) element;

			IStructuredSelection selection = (IStructuredSelection) this.view.getViewer().getSelection();

			AnnotationFS selectedAnno = (AnnotationFS) selection.getFirstElement();

			// TODO dirty

			if (f.getRange().isPrimitive()) {

				text.append(f.getShortName() + ":" + selectedAnno.getFeatureValueAsString(f));

			}

			else {

				// TODO

				text.append("Unsupported");

			}

			// if (f.getRange().getName().contains("String")) {
			//
			// }
			//
			// else if
			// (f.getRange().getName().toLowerCase().contains("integer")) {
			//
			// text.append(f.getShortName() + ":" +
			// selectedAnno.getIntValue(f));
			//
			// }
			//
			// else {
			// text.append(selectedAnno.getFeatureValueAsString(f));
			// }
			//
			// // set image
			//
			// setImage(cell, getTsdsFromAnno(selectedAnno));

		}

		else if (element instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) element;

			if (wrapper.feature.getRange().isPrimitive()) {

				String value = (String) wrapper.value;

				text.append(wrapper.feature.getShortName() + ":" + value);
			}

			else {

				text.append(wrapper.feature.getShortName());

			}

		}

		// cell.setImage(image);

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	private TypeStyleDataStructure getTsdsFromAnno(AnnotationFS anno) {

		TypeStyleDataStructure dsFromType =editor.getDSFromType(anno.getType());
		return dsFromType;
	}

	private void setImage(ViewerCell cell, TypeStyleDataStructure tsds) {

		Image im = null;

		if (typesHash.containsKey(tsds.getAnnotationType())) {

			im = typesHash.get(tsds.getAnnotationType());
		}

		else {

			im = new Image(display, 16, 16);

			typesHash.put(tsds.getAnnotationType(), im);

		}

		gc = new GC(im);

		gc.setBackground(tsds.getStyle().getStyle().background);

		String substring = tsds.getStyle().getDrawingStrategy().getClass().toString();

		String[] split = substring.split("\\.");

		String name = split[split.length - 1];

		if (name.length() > 2) {
			name = name.substring(0, 2);
		}

		name = name.toUpperCase();

		gc.fillRectangle(0, 0, 16, 16);
		gc.drawString(name, 0, 0);
		cell.setImage(im);

		gc.dispose();

	}

	@PreDestroy
	public void dispose() {

		for (Image i : typesHash.values()) {

			i.dispose();
		}

		gc.dispose();
	}
}
