package de.uniwue.mk.kall.athen.selection.analyzer.ui;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.ArrayFSImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

public class SelectionAnalyzerTreeContentProvider implements ITreeContentProvider {

	private SelectionAnalyzer analyzer;

	private List<AnnotationFS> inpList;

	public SelectionAnalyzerTreeContentProvider(SelectionAnalyzer selectionAnalyzer) {

		this.analyzer = selectionAnalyzer;
	}

	@Override
	public void dispose() {
	}

	@Override
	public Object[] getElements(Object inputElement) {

		if (inputElement instanceof List<?>) {
			// should return 3 elements tp/fp/fn

			inpList = (List<AnnotationFS>) inputElement;

			HashSet<Type> typeSet = new HashSet<Type>();

			for (AnnotationFS a : inpList) {
				typeSet.add(a.getType());
			}

			return typeSet.toArray(new Type[0]);

		}

		return null;
	}

	@Override
	public Object[] getChildren(Object parentElement) {

		if (parentElement instanceof Type) {

			List<AnnotationFS> sameTypeCoveredAnnos = new ArrayList<AnnotationFS>();

			for (AnnotationFS a : this.inpList) {
				if (parentElement.equals(a.getType())) {
					sameTypeCoveredAnnos.add(a);
				}
			}

			return sameTypeCoveredAnnos.toArray(new AnnotationFS[0]);
		}

		else if (parentElement instanceof AnnotationFS) {

			AnnotationFS anno = (AnnotationFS) parentElement;

			List<Feature> features = anno.getType().getFeatures();

			// TODO add a wrapper type for the features so that you can access
			// their values !!

			List<FeatureValueWrapper> wrapperObjects = new ArrayList<FeatureValueWrapper>();
			for (Feature f : features) {

				if (f.getRange().isPrimitive()) {
					wrapperObjects.add(new FeatureValueWrapper(f, anno.getFeatureValueAsString(f)));
				}

				else {

					if (!f.getName().contains(":sofa")) {
						wrapperObjects.add(new FeatureValueWrapper(f, anno.getFeatureValue(f)));
					}
				}
			}
			return wrapperObjects.toArray(new FeatureValueWrapper[0]);

		}

		else if (parentElement instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) parentElement;

			if (wrapper.feature.getRange().isPrimitive())
				return null;

			else {

				if (wrapper.value != null) {
					if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation")) {
						return new Object[] { (AnnotationFS) wrapper.value };
					}

					// TODO here need all the types to be added
					else if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation[]")) {

						List<AnnotationFS> annos = new ArrayList<AnnotationFS>();

						ArrayFSImpl arr = (ArrayFSImpl) wrapper.value;
						for (int i = 0; i < arr.size(); i++) {
							annos.add((AnnotationFS) arr.get(i));
						}
						return annos.toArray(new AnnotationFS[0]);
					}
				}
			}
		}

		return null;
	}

	@Override
	public Object getParent(Object element) {

		// TODO this seems wrong

		if (element instanceof Type) {

			return null;
		}

		else if (element instanceof AnnotationFS) {
			AnnotationFS a = (AnnotationFS) element;

			return a.getType();
		}

		return null;
	}

	@Override
	public boolean hasChildren(Object element) {

		if (element instanceof Type) {
			AnnotationIndex<AnnotationFS> annotationIndex = analyzer.getEditor().getCas()
					.getAnnotationIndex((Type) element);
			return annotationIndex.size() > 0;
		}

		if (element instanceof AnnotationFS) {
			return true;
		}

		else if (element instanceof FeatureValueWrapper) {

			FeatureValueWrapper wrapper = (FeatureValueWrapper) element;
			if (wrapper.feature.getRange().isPrimitive())
				return false;

			else {

				if (wrapper.value != null) {
					if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation")) {
						return true;
					}

					// TODO here need all the types to be added
					else if (wrapper.feature.getRange().toString().equals("uima.tcas.Annotation[]")) {
						return true;
					}
				}
			}
		}

		return false;
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

}
