package de.uniwue.mk.kall.athen.socialnetworks;

import java.awt.BasicStroke;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Stroke;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import javax.imageio.ImageIO;
import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import org.apache.commons.collections15.Transformer;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import edu.uci.ics.jung.algorithms.layout.CircleLayout;
import edu.uci.ics.jung.algorithms.layout.Layout;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.visualization.GraphZoomScrollPane;
import edu.uci.ics.jung.visualization.LayeredIcon;
import edu.uci.ics.jung.visualization.VisualizationViewer;
import edu.uci.ics.jung.visualization.control.CrossoverScalingControl;
import edu.uci.ics.jung.visualization.control.ModalGraphMouse;
import edu.uci.ics.jung.visualization.control.ScalingControl;
import edu.uci.ics.jung.visualization.decorators.DefaultVertexIconTransformer;
import edu.uci.ics.jung.visualization.decorators.EllipseVertexShapeTransformer;
import edu.uci.ics.jung.visualization.decorators.ToStringLabeller;
import edu.uci.ics.jung.visualization.decorators.VertexIconShapeTransformer;
import edu.uci.ics.jung.visualization.picking.PickedState;
import edu.uci.ics.jung.visualization.renderers.Checkmark;
import edu.uci.ics.jung.visualization.renderers.Renderer.VertexLabel.Position;

public class GraphVisualizer<V, E> implements Printable {

	private JSlider minSlider;
	private JSlider maxSlider;
	private SocialNetWorkJungGenerator generator;
	private Graph<NovelNode, NovelEdge> graph;
	private Container content;
	private Layout<NovelNode, NovelEdge> layout;
	private GraphZoomScrollPane gzsp;
	private JFrame frame;
	private JSpinner spinnerWindow;
	private JSpinner spinnerTopMostPerson;
	private VisualizationViewer<NovelNode, NovelEdge> vv;
	private int scale = 1;
	private ScalingControl scaler;
	private EditingModalGraphMouseSelf<NovelNode, NovelEdge> editingModalGraphMouse2;

	// TODO hardcoded types
	private Type chapterType;
	private Type paragraphType;

	private boolean docHasChapter = false;
	private boolean docHasPragaraphs = false;
	private JLabel sentWindow;
	private JCheckBox checkbox;
	private HashMap<NovelNode, Icon> resourceImageHash;
	private JTextField textToChap;

	public GraphVisualizer(Graph<NovelNode, NovelEdge> g, SocialNetWorkJungGenerator generator) {

		this.graph = g;
		this.generator = generator;

		resourceImageHash = new HashMap<NovelNode, Icon>();
		initTypes();
		vv = initGraphpanel(g);

		frame = new JFrame("Soziales Netzwerk");
		frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);

		Dimension space = new Dimension(20, 20);
		Box controls = Box.createVerticalBox();
		controls.add(Box.createRigidArea(space));

		JPanel zoomControls = new JPanel(new GridLayout(1, 3));

		zoomControls.setBorder(BorderFactory.createTitledBorder("Settings"));

		JPanel panelSentenceWindow = new JPanel(new GridLayout(1, 4));
		JPanel panelChapterBoundaries = new JPanel(new GridLayout(2, 3));
		JPanel panelSlider = new JPanel(new GridLayout(2, 2));

		zoomControls.add(panelSentenceWindow);
		zoomControls.add(panelChapterBoundaries);
		zoomControls.add(panelSlider);

		// buttons

		// panel window===

		JPanel subSentenceWindow1 = new JPanel(new GridLayout(3, 1));
		JPanel subSentenceWindow2 = new JPanel(new GridLayout(3, 1));

		if (docHasPragaraphs) {
			checkbox = new JCheckBox();

			checkbox.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					if (checkbox.isSelected()) {
						// set others invis
						spinnerWindow.setVisible(false);
						sentWindow.setVisible(false);
						update();

					}

					else {

						spinnerWindow.setVisible(true);
						sentWindow.setVisible(true);
						update();

					}

				}
			});
			JLabel labelCheckBox = new JLabel("Use Paragraphs");
			subSentenceWindow1.add(checkbox);
			subSentenceWindow2.add(labelCheckBox);
		}

		spinnerWindow = new JSpinner();
		spinnerWindow.setValue(1);

		spinnerTopMostPerson = new JSpinner();
		spinnerTopMostPerson.setValue(10);

		spinnerTopMostPerson.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {
				update();

			}
		});
		JLabel labelTopMostPersonCnt = new JLabel("Show most frequent:");

		sentWindow = new JLabel("Sentence Window");

		subSentenceWindow1.add(spinnerWindow);
		subSentenceWindow2.add(sentWindow);
		subSentenceWindow1.add(spinnerTopMostPerson);
		subSentenceWindow2.add(labelTopMostPersonCnt);

		panelSentenceWindow.add(subSentenceWindow1);
		panelSentenceWindow.add(subSentenceWindow2);
		panelSentenceWindow.add(new JPanel());
		panelSentenceWindow.add(new JPanel());

		// listener
		spinnerWindow.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {

				// update

				update();

			}

		});

		// === Edn panel window

		// panel chapter ==========
		if (docHasChapter) {
			JLabel labelFromChap = new JLabel("From Chapter:");
			JTextField textFromChap = new JTextField("   ");
			JButton nextChapter = new JButton(">>");

			nextChapter.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {

				}
			});

			panelChapterBoundaries.add(labelFromChap);
			panelChapterBoundaries.add(textFromChap);
			panelChapterBoundaries.add(nextChapter);

			JLabel labelToChap = new JLabel("To Chapter:");
			textToChap = new JTextField("   ");
			JButton prevChapter = new JButton("<<");

			nextChapter.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {

					int parseInt = Integer.parseInt(GraphVisualizer.this.textToChap.getText());

				}
			});

			panelChapterBoundaries.add(labelToChap);
			panelChapterBoundaries.add(textToChap);
			panelChapterBoundaries.add(prevChapter);
		}
		// END PANEL WITH CHAPTER

		// panel with slider

		JLabel labelMinSlider = new JLabel("From Sentence:");
		JLabel labelMaxSlider = new JLabel("To Sentence:");

		minSlider = new JSlider(0, generator.getMaxAmountOfSentences());
		maxSlider = new JSlider(0, generator.getMaxAmountOfSentences());

		panelSlider.add(labelMinSlider);
		panelSlider.add(minSlider);
		panelSlider.add(labelMaxSlider);
		panelSlider.add(maxSlider);

		minSlider.setValue(0);

		minSlider.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {

				update();
			}
		});

		maxSlider.setValue(generator.getMaxAmountOfSentences());

		maxSlider.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent e) {

				// TODO assure that they dont overlap
				update();

			}
		});

		// start pane EAST

		JPanel controlsEAST = new JPanel(new GridLayout(5, 1));

		scaler = new CrossoverScalingControl();
		JButton plus = new JButton("+");
		plus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scaler.scale(vv, 1.1f, vv.getCenter());
				scale++;
			}
		});
		JButton minus = new JButton("-");
		minus.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scaler.scale(vv, 1 / 1.1f, vv.getCenter());
				scale--;
			}
		});

		controlsEAST.add(minus);
		controlsEAST.add(plus);

		// build pane

		content = frame.getContentPane();

		controls.add(zoomControls);
		controls.add(Box.createRigidArea(space));

		gzsp = new GraphZoomScrollPane(vv);
		content.add(gzsp);
		content.add(controls, BorderLayout.SOUTH);
		// content.add(controlsEAST, BorderLayout.EAST);

		// menu

		JMenu menu = new JMenu("File");
		menu.add(new AbstractAction("Make Image") {
			public void actionPerformed(ActionEvent e) {
				JFileChooser chooser = new JFileChooser();
				int option = chooser.showSaveDialog(frame);
				if (option == JFileChooser.APPROVE_OPTION) {
					File file = chooser.getSelectedFile();
					writeJPEGImage(file);
				}
			}
		});
		menu.add(getPrintFunction(vv));
		JPopupMenu.setDefaultLightWeightPopupEnabled(false);
		JMenuBar menuBar = new JMenuBar();
		menuBar.add(menu);
		frame.setJMenuBar(menuBar);

		frame.pack();
		frame.setVisible(true);

	}

	private void initTypes() {
		CAS cas = generator.getEditor().getCas();

		this.chapterType = cas.getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Chapter");
		this.paragraphType = cas.getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Paragraph");

		if (cas.getAnnotationIndex(chapterType).size() > 0) {
			this.docHasChapter = true;
		}

		if (cas.getAnnotationIndex(paragraphType).size() > 0) {
			this.docHasPragaraphs = true;
		}

	}

	private AbstractAction getPrintFunction(VisualizationViewer<NovelNode, NovelEdge> vv) {
		return new AbstractAction("Print") {
			public void actionPerformed(ActionEvent e) {
				PrinterJob printJob = PrinterJob.getPrinterJob();
				printJob.setPrintable(new Printable() {

					@Override
					public int print(Graphics graphics, PageFormat pageFormat, int pageIndex) throws PrinterException {
						if (pageIndex > 0) {
							return (Printable.NO_SUCH_PAGE);
						} else {
							java.awt.Graphics2D g2d = (java.awt.Graphics2D) graphics;
							GraphVisualizer.this.vv.setDoubleBuffered(false);
							g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());

							GraphVisualizer.this.vv.paint(g2d);
							GraphVisualizer.this.vv.setDoubleBuffered(true);

							return (Printable.PAGE_EXISTS);
						}
					}
				});
				if (printJob.printDialog()) {
					try {
						printJob.print();
					} catch (Exception ex) {
						ex.printStackTrace();
					}
				}
			}
		};
	}

	private VisualizationViewer<NovelNode, NovelEdge> initGraphpanel(Graph<NovelNode, NovelEdge> g) {
		layout = new CircleLayout<NovelNode, NovelEdge>(g);
		// layout = new SpringLayout<NovelNode, NovelEdge>(g);
		layout.setSize(new Dimension(1000, 1000));
		// the space
		// The BasicVisualizationServer<V,E> is parameterized by the edge types
		vv = new VisualizationViewer<NovelNode, NovelEdge>(layout);
		vv.setPreferredSize(new Dimension(900, 900)); // Sets the viewing area
														// size
		vv.setBackground(Color.white);
		// Transformer<Integer, Paint> vertexPaint = new Transformer<Integer,
		// Paint>() {
		// public Paint transform(Integer i) {
		// return Color.GREEN;
		// }
		// };
		// // Set up a new stroke Transformer for the edges
		// float dash[] = { 10.0f };
		// final Stroke edgeStroke = new BasicStroke(1.0f, BasicStroke.CAP_BUTT,
		// BasicStroke.JOIN_MITER,
		// 10.0f, dash, 0.0f);
		// Transformer<String, Stroke> edgeStrokeTransformer = new
		// Transformer<String, Stroke>() {
		// public Stroke transform(String s) {
		// return edgeStroke;
		// }
		// };

		// vv.getRenderContext().setVertexFillPaintTransformer((Transformer<V,
		// Paint>) vertexPaint);
		// vv.getRenderContext().setEdgeStrokeTransformer((Transformer<E,
		// Stroke>)
		// edgeStrokeTransformer);

		vv.setVertexToolTipTransformer(new Transformer<NovelNode, String>() {

			@Override
			public String transform(NovelNode n) {

				HashSet<String> syns = generator.getAllSynonyms(n);

				String text = "";
				for (String s : syns) {

					text += s + " ; ";

				}
				return text;
			}
		});
		vv.getRenderContext().setEdgeStrokeTransformer(
				(Transformer<NovelEdge, Stroke>) new Transformer<NovelEdge, Stroke>() {

					@Override
					public Stroke transform(NovelEdge edge) {

						if (edge instanceof NovelEdge) {
							NovelEdge e = (NovelEdge) edge;

							return new BasicStroke((float) ((e.getWeight() / 30 * 200) / (200 + e.getWeight() / 30.0)));
						}
						return null;
					}
				});

		vv.getRenderContext().setVertexLabelTransformer(new ToStringLabeller<NovelNode>());

		// features on and off. For a real application, use
		// VertexIconAndShapeFunction instead.
		final VertexIconShapeTransformer<NovelNode> vertexImageShapeFunction = new VertexIconShapeTransformer<NovelNode>(
				new EllipseVertexShapeTransformer<NovelNode>());

		final DefaultVertexIconTransformer<NovelNode> vertexIconFunction = new DefaultVertexIconTransformer<NovelNode>();

		PickedState<NovelNode> ps = vv.getPickedVertexState();
		ps.addItemListener(new PickWithIconListener(vertexIconFunction));

		Map<NovelNode, Icon> iconMap = new HashMap<NovelNode, Icon>();

		for (NovelNode n : g.getVertices()) {

			try {
				iconMap.put(n, getIcon(n));
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}

		vertexImageShapeFunction.setIconMap(iconMap);
		vertexIconFunction.setIconMap(iconMap);

		vv.getRenderContext().setVertexShapeTransformer(vertexImageShapeFunction);
		vv.getRenderContext().setVertexIconTransformer(vertexIconFunction);

		vv.getRenderContext().setEdgeLabelTransformer(new ToStringLabeller<NovelEdge>());
		vv.getRenderer().getVertexLabelRenderer().setPosition(Position.AUTO);
		editingModalGraphMouse2 = new EditingModalGraphMouseSelf<NovelNode, NovelEdge>(vv.getRenderContext());
		vv.setGraphMouse(editingModalGraphMouse2);
		vv.addKeyListener(editingModalGraphMouse2.getModeKeyListener());

		editingModalGraphMouse2.setMode(ModalGraphMouse.Mode.PICKING);

		vv.addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent e) {

				if (e.getButton() == 3) {

					final PickedState<NovelNode> pickedState = vv.getPickedVertexState();

					if (pickedState.getPicked().size() == 1) {

						String eingabe = JOptionPane.showInputDialog(null, "Geben Sie einen gewnschten Namen ein",
								"Namensnderungsdialog", JOptionPane.PLAIN_MESSAGE);

						if (eingabe != null) {
							for (NovelNode n : pickedState.getPicked()) {
								n.setName(eingabe);

							}
							frame.getContentPane().validate();
							frame.getContentPane().repaint();
						}
					}
				}

			}

			@Override
			public void mousePressed(MouseEvent e) {

			}

			@Override
			public void mouseExited(MouseEvent e) {

			}

			@Override
			public void mouseEntered(MouseEvent e) {

			}

			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {

					final PickedState<NovelNode> pickedState = vv.getPickedVertexState();

					Set<NovelNode> picked = pickedState.getPicked();

					if (picked.size() == 1) {

						for (NovelNode node : picked) {

							Runnable r = new AttributeVisualisationNodeRunnable(generator, node);

							// execute in an async swt thread
							generator.getShell().getDisplay().asyncExec(r);

						}
					}
				}

			}
		});
		return vv;
	}

	private Icon getIcon(NovelNode n) throws IOException {

		// TODO differ between male and female
		String gender = n.getGender();
		File ressource = null;
		if (gender == null) {

			ressource = ApplicationUtil.getBundleRessource(ApplicationUtil.NODE_MAN);
		}

		else if (gender.equals("f")) {
			ressource = ApplicationUtil.getBundleRessource(ApplicationUtil.NODE_WOMAN);
		}

		else {

			ressource = ApplicationUtil.getBundleRessource(ApplicationUtil.NODE_MAN);
		}

		if (resourceImageHash.containsKey(n)) {

			return resourceImageHash.get(n);
		}

		BufferedImage bimg = resizeImage(ImageIO.read(ressource), 50, 50);
		Icon icon = new LayeredIcon(new ImageIcon(bimg).getImage());

		resourceImageHash.put(n, icon);
		return icon;
	}

	public BufferedImage resizeImage(BufferedImage image, int width, int height) {
		int type = 0;
		type = image.getType() == 0 ? BufferedImage.TYPE_INT_ARGB : image.getType();
		BufferedImage resizedImage = new BufferedImage(width, height, type);
		Graphics2D g = resizedImage.createGraphics();
		g.drawImage(image, 0, 0, width, height, null);
		g.dispose();
		return resizedImage;
	}

	private void update() {

		boolean useParagraphs = false;

		if (this.checkbox != null) {
			useParagraphs = this.checkbox.isSelected();
		}

		Graph<NovelNode, NovelEdge> updateGraph = generator.updateGraph(minSlider.getValue(), maxSlider.getValue(),
				(int) spinnerWindow.getValue(), (int) spinnerTopMostPerson.getValue(), useParagraphs);

		content.remove(gzsp);

		graph = updateGraph;

		vv = initGraphpanel(graph);

		gzsp = new GraphZoomScrollPane(vv);

		// repaint

		content.add(gzsp);

		frame.getContentPane().validate();
		frame.getContentPane().repaint();

	}

	@Override
	public int print(java.awt.Graphics graphics, java.awt.print.PageFormat pageFormat, int pageIndex)
			throws java.awt.print.PrinterException {
		if (pageIndex > 0) {
			return (Printable.NO_SUCH_PAGE);
		} else {
			java.awt.Graphics2D g2d = (java.awt.Graphics2D) graphics;
			vv.setDoubleBuffered(false);
			g2d.translate(pageFormat.getImageableX(), pageFormat.getImageableY());

			vv.paint(g2d);
			vv.setDoubleBuffered(true);

			return (Printable.PAGE_EXISTS);
		}
	}

	public void writeJPEGImage(File file) {
		int width = vv.getWidth();
		int height = vv.getHeight();

		BufferedImage bi = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics = bi.createGraphics();
		vv.paint(graphics);
		graphics.dispose();

		try {
			ImageIO.write(bi, "jpeg", file);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static class PickWithIconListener implements ItemListener {
		DefaultVertexIconTransformer<NovelNode> imager;
		Icon checked;

		public PickWithIconListener(DefaultVertexIconTransformer<NovelNode> imager) {
			this.imager = imager;
			checked = new Checkmark(Color.GREEN);
		}

		public void itemStateChanged(ItemEvent e) {
			Icon icon = imager.transform((NovelNode) e.getItem());
			if (icon != null && icon instanceof LayeredIcon) {
				if (e.getStateChange() == ItemEvent.SELECTED) {
					((LayeredIcon) icon).add(checked);
				} else {
					((LayeredIcon) icon).remove(checked);
				}
			}
		}
	}

}
