package de.uniwue.mk.kall.athen.socialnetworks;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.widgets.Shell;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import edu.uci.ics.jung.graph.Graph;
import edu.uci.ics.jung.graph.SparseMultigraph;

public class SocialNetWorkJungGenerator {

	private AnnotationEditorWidget editor;
	private Shell shell;

	private Type neType;
	private Type sentType;
	private Type parType;
	private Type chapType;
	private Feature neId;

	private HashMap<String, HashSet<String>> tempMapping;

	private HashMap<String, String> idGenderMap;

	private HashMap<String, Integer> idFreqMapping;

	private List<Pair<String, Integer>> topMostSorted;
	private Feature genderFeat;

	public SocialNetWorkJungGenerator(AnnotationEditorWidget editor, Shell currentShell) {

		this.editor = editor;
		this.shell = currentShell;

		idGenderMap = new HashMap<String, String>();

		initTypes();

		initMappings();

		topMostSorted = getIdsSortedByAppearance();

		Graph<NovelNode, NovelEdge> g = buildGraph(editor, 0, getMaxAmountOfSentences(), 1, 10, false);

		GraphVisualizer<String, Integer> gV = new GraphVisualizer(g, this);

	}

	private List<Pair<String, Integer>> getIdsSortedByAppearance() {

		List<Pair<String, Integer>> idList = new ArrayList<Pair<String, Integer>>();

		for (String s : idFreqMapping.keySet()) {

			Pair<String, Integer> p = new Pair<String, Integer>(s, idFreqMapping.get(s));

			idList.add(p);
		}

		Collections.sort(idList, new Comparator<Pair<String, Integer>>() {

			@Override
			public int compare(Pair<String, Integer> o1, Pair<String, Integer> o2) {
				return o2.getValue() - o1.getValue();
			}
		});

		return idList;
	}

	private Graph<NovelNode, NovelEdge> buildGraph(AnnotationEditorWidget editor, int beg, int end, int windowSize,
			int topMostfrequent, boolean useParagraphs) {

		// at first search for the most frequent NEs in range
		int begIndex = 0;
		int endIndex = 0;
		int index = 0;
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(sentType)) {

			if (index == beg) {
				begIndex = a.getBegin();
			}

			if (index + 1 == end) {
				endIndex = a.getEnd();
			}

			index++;
		}

		// key is stored as id1-id2 and int shows the frequency of interaction

		HashMap<String, Integer> adjacentMap = new HashMap<String, Integer>();

		List<String> topMostIds = getTopMostFrequentIds(topMostfrequent, begIndex, endIndex);

		if (!useParagraphs) {
			SentenceWindowMappingInit(editor, begIndex, endIndex, windowSize, adjacentMap, topMostIds);
		}

		else {

			paragraphMappingInit(editor, begIndex, endIndex, adjacentMap, topMostIds);
		}
		// end map creation

		// ===============create the graph=================

		Graph<NovelNode, NovelEdge> g = createGraphFromMap(adjacentMap);
		return g;
	}

	private void paragraphMappingInit(AnnotationEditorWidget editor, int begIndex, int endIndex,
			HashMap<String, Integer> adjacentMap, List<String> topMostIds) {
		// get beg and endindexes

		// paragraphs here
		for (AnnotationFS par : editor.getCas().getAnnotationIndex(parType)) {
			if (par.getBegin() >= begIndex && par.getEnd() <= endIndex) {

				// get all entities in that sentence
				List<AnnotationFS> coveredNe = getCoveredInPar(par, neType, topMostIds);

				// max 1 count per sentence
				HashSet<String> uniqueIds = new HashSet<String>();

				for (AnnotationFS ne : coveredNe) {
					uniqueIds.add(ne.getFeatureValueAsString(neId));
				}

				// all the ids that are storeed in the set are thought of as
				// interacting

				initInteractions(adjacentMap, uniqueIds);
			}
		}
	}

	private void SentenceWindowMappingInit(AnnotationEditorWidget editor, int beg, int end, int windowSize,
			HashMap<String, Integer> adjacentMap, List<String> topMostIds) {
		for (AnnotationFS nee : editor.getCas().getAnnotationIndex(neType)) {

			if (nee.getBegin() < beg || nee.getEnd() > end) {
				continue;
			}

			// get all entities in that sentence
			List<AnnotationFS> coveredNe = getCoveredInRange(getCoveringSentence(nee), neType, windowSize, topMostIds);

			// max 1 count per sentence
			HashSet<String> uniqueIds = new HashSet<String>();

			for (AnnotationFS ne : coveredNe) {
				uniqueIds.add(ne.getFeatureValueAsString(neId));
			}

			// all the ids that are storeed in the set are thought of as
			// interacting

			initInteractions(adjacentMap, uniqueIds);

		}
	}

	public AnnotationFS getCoveringSentence(AnnotationFS anno) {

		for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentType)) {

			// if (sent.getBegin() > anno.getEnd()) {
			// return null;
			// }

			if (sent.getBegin() <= anno.getBegin() && sent.getEnd() >= anno.getEnd()) {
				return sent;
			}
		}

		// get the sentence which coveres most of that annotation

		AnnotationFS covSent = null;
		int cov = 0;

		for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentType)) {

			int tempEnd = sent.getEnd() <= anno.getEnd() ? sent.getEnd() : anno.getEnd();
			int tempBeg = sent.getBegin() >= anno.getBegin() ? sent.getBegin() : anno.getBegin();

			if (tempEnd - tempBeg > cov) {
				cov = tempEnd - tempBeg;
				covSent = sent;
			}

		}

		return covSent;

	}

	private Graph<NovelNode, NovelEdge> createGraphFromMap(HashMap<String, Integer> adjacentMap) {
		Graph<NovelNode, NovelEdge> g = new SparseMultigraph<NovelNode, NovelEdge>();

		HashMap<String, NovelNode> nodeHash = new HashMap<String, NovelNode>();
		for (String s : adjacentMap.keySet()) {

			Integer count = adjacentMap.get(s);
			if (count > 0) {

				String id = s.split("-")[0];
				String name = determineNamesForID(id);

				if (!name.isEmpty() && Character.isLowerCase(name.charAt(0))) {
					continue;
				}

				NovelNode novelNode = null;
				if (!nodeHash.containsKey(name)) {
					novelNode = new NovelNode(name, s.split("-")[0], getGenderForId(s.split("-")[0]));
					g.addVertex(novelNode);
					nodeHash.put(name, novelNode);
				} else {

					novelNode = nodeHash.get(name);
				}

				String id2 = s.split("-")[1];
				name = determineNamesForID(id2);

				NovelNode novelNode2 = null;

				if (!nodeHash.containsKey(name)) {

					novelNode2 = new NovelNode(name, s.split("-")[1], getGenderForId(s.split("-")[1]));
					g.addVertex(novelNode2);
					nodeHash.put(name, novelNode2);
				}

				else {
					novelNode2 = nodeHash.get(name);
				}

				g.addEdge(new NovelEdge(s, adjacentMap.get(s)), novelNode, novelNode2);
			}

		}
		return g;
	}

	private String getGenderForId(String string) {

		if (idGenderMap.containsKey(string)) {
			return idGenderMap.get(string);
		}

		else {

			for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {

				String id = ne.getFeatureValueAsString(neId);

				if (id.equals(string)) {

					String gender = ne.getFeatureValueAsString(genderFeat);
					idGenderMap.put(string, gender);
					return gender;
				}
			}
		}

		return null;
	}

	private List<AnnotationFS> getCoveredInPar(AnnotationFS par, Type neType2, List<String> topMost) {

		return getCoveredInSentence(par, neType, topMost);
	}

	private List<String> getTopMostFrequentIds(int topMostfrequent, int begIndex, int endIndex) {

		List<String> frequent = new ArrayList<String>();
		// for (int i = 0; i < Math.min(topMostfrequent, idFreqMapping.size());
		// i++) {
		//
		// frequent.add(topMostSorted.get(i).getKey());
		// }
		// return frequent;

		HashMap<String, Integer> idCnt = new HashMap<String, Integer>();
		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {
			if (ne.getBegin() >= begIndex && ne.getEnd() <= endIndex) {

				String id = ne.getFeatureValueAsString(neId);

				if (idCnt.containsKey(id)) {

					Integer integer = idCnt.get(id);
					integer++;

					idCnt.put(id, integer);
				}

				else {
					idCnt.put(id, 1);
				}
			}
		}

		// map to pairs
		List<Pair<String, Integer>> idList = new ArrayList<Pair<String, Integer>>();
		for (String s : idCnt.keySet()) {

			Pair<String, Integer> p = new Pair<String, Integer>(s, idCnt.get(s));

			idList.add(p);
		}

		// sort
		Collections.sort(idList, new Comparator<Pair<String, Integer>>() {

			@Override
			public int compare(Pair<String, Integer> o1, Pair<String, Integer> o2) {
				return o2.getValue() - o1.getValue();
			}
		});

		// return topmost entries

		for (int i = 0; i < Math.min(topMostfrequent, idCnt.size()); i++) {
			frequent.add(idList.get(i).getKey());
		}
		return frequent;
	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void setEditor(AnnotationEditorWidget editor) {
		this.editor = editor;
	}

	public Shell getShell() {
		return shell;
	}

	public void setShell(Shell shell) {
		this.shell = shell;
	}

	private void initTypes() {

		// TODO
		this.neType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.NamedEntity");
		this.sentType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Sentence");
		this.neId = neType.getFeatureByBaseName("ID");
		this.genderFeat = neType.getFeatureByBaseName("Gender");
		this.chapType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Chapter");
		this.parType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Paragraph");

	}

	private void initInteractions(HashMap<String, Integer> adjacentMap, HashSet<String> uniqueIds) {
		for (String id : uniqueIds) {

			for (String id2 : uniqueIds) {
				if (id2.equals(id)) {
					continue;
				}

				// add to map

				int i1 = Integer.parseInt(id);
				int i2 = Integer.parseInt(id2);

				String key;
				if (i1 < i2) {
					key = id + "-" + id2;
				}

				else {
					key = id2 + "-" + id;
				}
				if (adjacentMap.containsKey(key)) {

					// increase count
					Integer integer = adjacentMap.get(key);

					integer = integer + 1;

					adjacentMap.put(key, integer);
				}

				else {
					adjacentMap.put(key, 1);
				}

			}
		}
	}

	private void initMappings() {

		// Maps ID -> set of Strings that have the same ID
		tempMapping = new HashMap<String, HashSet<String>>();
		idFreqMapping = new HashMap<String, Integer>();

		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(neType)) {

			String id = ne.getFeatureValueAsString(neId);

			// tempmapping
			if (tempMapping.containsKey(id)) {

				HashSet<String> hashSet = tempMapping.get(id);

				hashSet.add(ne.getCoveredText());
			}

			else {

				HashSet<String> name = new HashSet<String>();
				name.add(ne.getCoveredText());
				tempMapping.put(id, name);
			}

			if (idFreqMapping.containsKey(id)) {
				Integer integer = idFreqMapping.get(id);
				integer++;

				idFreqMapping.put(id, integer);
			}

			else {

				idFreqMapping.put(id, 1);
			}

		}

	}

	private String determineNamesForID(String idToSearch) {

		HashSet<String> hashSet = tempMapping.get(idToSearch);

		String longest = "";

		for (String s : hashSet) {

			if (s.length() > longest.length() && Character.isUpperCase(s.charAt(0))) {
				longest = s;
			}
		}

		// check if not null

		if (longest.isEmpty()) {

			for (String s : hashSet) {

				if (s.toLowerCase().equals("ich")) {
					longest = "ich";
					break;
				}
				if (s.toLowerCase().equals("wir")) {
					longest = "wir";
					break;
				}
			}
		}

		// if still empty just use the longest

		if (longest.isEmpty()) {
			for (String s : hashSet) {

				if (s.length() > longest.length()) {
					longest = s;
				}
			}

		}

		return longest;

	}

	public List<AnnotationFS> getCoveredInRange(AnnotationFS cover, Type t, int windowSize, List<String> topMostIds) {

		List<AnnotationFS> covered = new ArrayList<AnnotationFS>();
		List<AnnotationFS> validSentences = new ArrayList<AnnotationFS>();
		// get all sentences in the specified window

		// add actual sentence
		validSentences.add(cover);

		AnnotationFS prev = cover;
		AnnotationFS next = cover;
		for (int i = 0; i < windowSize; i++) {

			// get previous and next
			prev = getPrevious(prev);
			next = getNext(next);

			if (prev != null) {
				validSentences.add(prev);
			}

			if (next != null) {
				validSentences.add(next);
			}

		}

		//

		for (AnnotationFS sentence : validSentences) {
			covered.addAll(getCoveredInSentence(sentence, t, topMostIds));
		}
		return covered;

	}

	private AnnotationFS getCoveringParagraph(AnnotationFS cover) {

		for (AnnotationFS par : editor.getCas().getAnnotationIndex(this.parType))

			if (par.getBegin() <= cover.getBegin() && par.getEnd() >= cover.getEnd())
				return par;
		return null;
	}

	public AnnotationFS getNext(AnnotationFS anno) {

		if (anno == null) {
			return null;
		}

		FSIterator<AnnotationFS> iterator = editor.getCas().getAnnotationIndex(anno.getType()).iterator();

		iterator.moveTo(anno);

		iterator.moveToNext();

		if (iterator.isValid()) {
			// System.out.println(anno.getBegin() + "\t" +
			// iterator.get().getBegin());
			return iterator.get();
		}

		return null;

	}

	public AnnotationFS getPrevious(AnnotationFS anno) {

		if (anno == null) {
			return null;
		}

		FSIterator<AnnotationFS> iterator = editor.getCas().getAnnotationIndex(anno.getType()).iterator();

		iterator.moveTo(anno);

		iterator.moveToPrevious();

		if (iterator.isValid()) {
			// System.out.println(anno.getBegin() + "\t" +
			// iterator.get().getBegin());
			return iterator.get();
		}

		return null;

	}

	private List<AnnotationFS> getCoveredInSentence(AnnotationFS cover, Type t, List<String> topMostIds) {
		List<AnnotationFS> covered = new ArrayList<AnnotationFS>();

		int beg = (cover.getBegin() - 1) > 0 ? cover.getBegin() : cover.getBegin();

		int end = (cover.getEnd() + 1 > editor.getCas().getDocumentText().length()) ? cover.getEnd() : cover.getEnd();

		AnnotationFS dummyAnno = editor.getCas().createAnnotation(t, beg, end);

		FSIterator<AnnotationFS> subiterator = editor.getCas().getAnnotationIndex(t).subiterator(dummyAnno);

		while (subiterator.hasNext()) {

			AnnotationFS next = subiterator.next();

			if (topMostIds.contains(next.getFeatureValueAsString(neId))) {
				covered.add(next);
			}
		}
		return covered;
	}

	private List<AnnotationFS> getCovered(AnnotationFS cover, Type t) {
		List<AnnotationFS> covered = new ArrayList<AnnotationFS>();

		int beg = (cover.getBegin() - 1) > 0 ? cover.getBegin() : cover.getBegin();

		int end = (cover.getEnd() + 1 > editor.getCas().getDocumentText().length()) ? cover.getEnd() : cover.getEnd();

		AnnotationFS dummyAnno = editor.getCas().createAnnotation(t, beg, end);

		FSIterator<AnnotationFS> subiterator = editor.getCas().getAnnotationIndex(t).subiterator(dummyAnno);

		while (subiterator.hasNext()) {

			AnnotationFS next = subiterator.next();

			covered.add(next);
		}
		return covered;
	}

	public int getMaxAmountOfSentences() {
		return editor.getCas().getAnnotationIndex(sentType).size();
	}

	// from sentence index from to sentence index to
	public Graph<NovelNode, NovelEdge> updateGraph(int from, int to, int windowSize, int topMostfrequent,
			boolean useParagraphs) {

		return buildGraph(editor, from, to, windowSize, topMostfrequent, useParagraphs);
	}

	public HashSet<String> getAllSynonyms(NovelNode n) {

		HashSet<String> hashSet = tempMapping.get(n.getId());

		HashSet<String> apps = new HashSet<String>();

		for (String s : hashSet) {

			if (Character.isUpperCase(s.charAt(0)))
				apps.add(s);
		}
		return apps;
	}

}
