package de.uniwue.mk.kall.athen.templateview.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.DSSpeakerIDDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.NamedEntityDrawingStrategy;

public class TemplateView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<TemplateView> CLASS_THIS = TemplateView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "ENTER_ID_HERE";

	// WIDGETS
	private TableViewer viewer;

	// DATA CONTAINER - the model, since its usually very simple we have it here
	private List<AnnotationFS> annoList;
	public HashMap<String, Integer> freqMap;

	// the main type this view is about
	private Type mainType;

	// the Stringrepresentation of this maintype - used to get the type of the
	// preferencestore
	private String type;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	// if you want to display if a user has already annotated use this separate
	// annotype
	private Type typOverride;
	private String typeOverride;

	// all Features (UIMA) this view needs to know about
	private Feature numerusFeat;
	private Feature genderFeat;
	private Feature pseudoFeat;
	private Feature coreRangeFeat;

	// compounds - UI

	@Inject
	public TemplateView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		// layout this should stay as it is
		super.overlay = parent;
		super.overlay.setLayout(new FillLayout());

		// init the part, so that this view knows its position globally
		// (required to see if it has focus)
		super.part = partService.findPart(viewId);
	}

	// is called whenever the editor loads a new page
	@Override
	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		super.editor = editor;

		// only initialize if the part is visible
		MUIElement selectedElement = null;
		if (part != null) {
			MElementContainer<MUIElement> parent = part.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render but unsibscribe
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
			return;
		}

		// init all the types we need in this view, open a dialog to specify
		// them if not already done so
		boolean typesValid = initTypes();

		while (!typesValid) {

			// open preference dialogue and ask the user to enter valid types

			typesValid = initTypes();

		}

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// init the Model
		initModel(editor);

		// init the layout
		initLayout(overlay);

		// if we managed to get here subscribe to all editor events if not
		// already done so
		if (!isSubscribedToEditorEvents) {
			// use the standard method to get all events
			super.standardSubscribeToEditorEvents(CLASS_THIS);
		}

		// ====reload=====
		overlay.layout();
		overlay.getParent().layout();
	}

	private void initModel(AnnotationEditorWidget editor) {
		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(mainType)) {
			annoList.add(anno);
		}

		// ====FREQUENCIES=======
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getCoveredText())) {
				int freq = freqMap.get(anno.getCoveredText());
				freqMap.put(anno.getCoveredText(), freq + 1);
			}

			else {
				freqMap.put(anno.getCoveredText(), 1);
			}
		}
	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		type = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_TYPE, "null");
		typeOverride = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_OV_TYPE, "null");

		if (type.equals("null") || typeOverride.equals("null")) {
			return false;
		}

		this.mainType = editor.getCas().getTypeSystem().getType(type);

		if (this.mainType == null) {
			return false;
		}

		// set everything unnneded invisible

		this.editor.allStylesInvisible();

		// get Features of NE Type

		String idPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null");
		String numPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NUMERUS, "null");
		String genPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_GENDER, "null");
		String namePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NAME, "null");
		String pseudoPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_PSEUDO, "null");
		String corePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_CORE, "null");
		String isUncertain = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_UNCERTAIN, "null");
		String neTypePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NETYPE, "null");
		String coreRangePref = ApplicationUtil
				.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_CORERANGE, "null");

		if (idPref.equals("null") || numPref.equals("null") || coreRangePref.equals("null") || namePref.equals("null")
				|| pseudoPref.equals("null")) {
			return false;
		}

		this.pseudoFeat = this.mainType.getFeatureByBaseName(pseudoPref);
		this.numerusFeat = this.mainType.getFeatureByBaseName(numPref);
		this.genderFeat = this.mainType.getFeatureByBaseName(genPref);
		this.coreRangeFeat = this.mainType.getFeatureByBaseName(coreRangePref);

		// Override Type and Temporary Type
		this.typOverride = (editor.getCas().getTypeSystem().getType(typeOverride));

		// Visibility

		editor.changeVisibility(mainType, true);
		editor.changeVisibility(typOverride, true);

		// layer

		editor.changeLayer(mainType, 0);
		// editor.changeLayer(temporary, 0);
		editor.changeLayer(typOverride, 1);

		// yellow
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle style2 = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		// style - red
		StyleRange rangeTyp = new StyleRange();
		rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle style = new AnnoStyle(new NamedEntityDrawingStrategy(rangeTyp, null, null, pseudoFeat.getShortName(),
				numerusFeat.getShortName(), genderFeat.getShortName(), coreRangeFeat.getShortName()), rangeTyp);

		// DS if possible

		StyleRange rangeDS = new StyleRange();
		rangeDS.background = new Color(overlay.getDisplay(), 0, 0, 255);
		AnnoStyle styleDS = new AnnoStyle(new DSSpeakerIDDrawingStrategy(rangeDS, "ID", "Speaker"), rangeDS);

		// apply
		editor.changeStyle(mainType, style);
		// editor.changeStyle(temporary, style3);
		editor.changeStyle(typOverride, style2);

		// also change the modus
		editor.setActiveMode(mainType);

		return true;

	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

	}

	// use this method to update the view
	public void renewPage() throws CASException {
		updateModel();

		// redraw in a new Thread, executed by the Display
		Runnable r = new Runnable() {

			@Override
			public void run() {
				createTableViewer();

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void updateModel() {
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(type));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		// update frequencyMap
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getFeatureValueAsString(null))) {
				int freq = freqMap.get(anno.getFeatureValueAsString(null));
				freq += 1;
				freqMap.put(anno.getFeatureValueAsString(null), freq);
			}

			else {
				freqMap.put(anno.getFeatureValueAsString(null), 1);
			}
		}
	}

	private void createTableViewer() {

		viewer.setInput(annoList);

		viewer.refresh();
	}

	@PreDestroy
	public void preDestroy() {
		// TODO unused
	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr
	}

	// Perspective listener
	@Override
	public void coupleToPerspective() {

		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);

	}

	@Override
	public void decoupleFromPerspective() {
		System.out.println("Decouple CorefView");

		MasterPart.unsubscribeFromEvent(CLASS_THIS, 
				IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);
		super.standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);

	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications - yep this is important
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {

		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	// what to do when the editor has an added annotation?
	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an removed annotation?
	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation?
	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		AnnotationFS changed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation selection ( this is
	// an
	// arrayList<AnnotationFS> )
	@Override
	public void onAnnotationSelectionChanged(org.osgi.service.event.Event arg0) {
		// unused hjh

	}

	// what to do when the editor was double clicked
	@Override
	public void onEditorDoubleClicked(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was scrolled
	@Override
	public void onEditorWasScrolled(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor caret was changed
	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was clicked
	@Override
	public void onEditorMouseUp(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor is clicked
	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {
		// unused for this view

	}

	// what to do when the editor has a changed textselection
	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

	}

	// on editor mouse moved - use this if you want some fancy effects
	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void initModel() {
		// TODO Auto-generated method stub

	}

}