package de.uniwue.mk.kall.coreferenceview.ui;

import java.awt.Point;
import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

public class CoreMarkingDialog extends Dialog {

	private StyledText widget;
	private Button buttonMarkAll;

	private int beg = Integer.MAX_VALUE;
	private int end = -1;

	private String coreSpan;

	private List<Point> spans;

	private List<Point> markedSpans;

	private Shell shell;

	private AnnotationFS selection;

	public CoreMarkingDialog(Shell parentShell, AnnotationFS selection) {
		super(parentShell);

		this.selection = selection;
		this.shell = parentShell;
		this.spans = new ArrayList<Point>();
		this.markedSpans = new ArrayList<Point>();

		String[] split = this.selection.getCoveredText().split(" ");

		int b = this.selection.getBegin();
		for (String s : split) {
			Point p = new Point(b, b + s.length());

			b = b + s.length() + 1;

			spans.add(p);

		}
	}

	public String getCoreSpan() {

		// calculate the span from the marked points :)

		for (Point p : markedSpans) {

			// if not set already
			if (beg == Integer.MAX_VALUE) {
				beg = p.x;
			}

			if (end == -1) {
				end = p.y;
			}

			if (p.x < beg) {
				beg = p.x;
			}

			if (p.y > end) {
				end = p.y;
			}

		}
		return beg + ":" + end;
	}

	public void setCoreSpan(String coreSpan) {
		this.coreSpan = coreSpan;
	}

	protected Control createDialogArea(Composite parent) {

		parent.getShell().setText("Mark all required words as core");

		GridLayout layout = new GridLayout();
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		parent.setLayout(layout);

		widget = new StyledText(parent, SWT.BORDER_DASH);
		widget.setAlignment(SWT.CENTER);

		widget.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH));

		widget.setText(selection.getCoveredText());
		Font font = new Font(parent.getDisplay(), "Segoe UI", 10, SWT.NORMAL);
		widget.setFont(font);

		widget.addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				// get the covering word and mark it
				int caretOffset = widget.getCaretOffset();

				Point p = getSelectedWord(caretOffset + selection.getBegin());

				if (p != null) {

					// is this already marked ???

					if (markedSpans.contains(p)) {
						// mark white

						StyleRange range = new StyleRange(p.x - selection.getBegin(), p.y - p.x, new Color(shell
								.getDisplay(), new RGB(0, 0, 0)), new Color(shell.getDisplay(), new RGB(255, 255, 255)));

						widget.setStyleRange(range);
						markedSpans.remove(p);

					}

					else {
						StyleRange range = new StyleRange(p.x - selection.getBegin(), p.y - p.x, new Color(shell
								.getDisplay(), new RGB(0, 0, 0)), new Color(shell.getDisplay(), new RGB(0, 255, 255)));

						widget.setStyleRange(range);
						markedSpans.add(p);

						// highlight
					}

				}

			}

			private Point getSelectedWord(int caretOffset) {

				for (Point p : spans) {

					if (p.x <= caretOffset && p.y >= caretOffset) {
						return p;
					}

				}
				return null;
			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				// TODO Auto-generated method stub

			}
		});

		return parent;
	}

	@Override
	protected void okPressed() {

		setReturnCode(OK);
		close();
	}

}
