package de.uniwue.mk.kall.coreferenceview.ui;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.ruta.rule.AnnotationComparator;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.OvalDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.part.editor.IDFilter;
import de.uniwue.mk.kall.athen.part.editor.NameFilter;
import de.uniwue.mk.kall.athen.part.editor.UnderlyingAnnotationFilter;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.DSSpeakerIDDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.ImportantPPlOnlyDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.NamedEntityDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.RelationDanielDrawingStrategy;

public class CorefView extends AEditorSubordinateViewPart {

	private static final Class<CorefView> CLASS_THIS = CorefView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.3";

	// TODO a dirty boolean for a quick fix
	private boolean userIndependentJumpFeature = false;

	// WIDGETS

	private TableViewer viewer;
	private NameFilter filter;
	private IDFilter idFilter;
	private UnderlyingAnnotationFilter ovFilter;

	// DATA CONTAINER
	private List<AnnotationFS> annoList;
	public HashMap<String, Integer> freqMap;
	private Type typ;
	private String sTemporaryType = "de.uniwue.kalimachos.coref.type.temp3";

	private String type;

	// E4 DI OBJECTS
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	public Button getJumpButton() {
		return jumpButton;
	}

	public void setJumpButton(Button jumpButton) {
		this.jumpButton = jumpButton;
	}

	private Type typOverride;
	private Type typTemp;

	private String typeOverride;

	private Feature idFeat;

	private Feature numerusFeat;
	private Feature genderFeat;

	private static final String APPELATIV_ABSTRAKTA = "AppA";
	private static final String APPELATIV_TEIL_FIKT_WELT = "AppTdfW";
	private static final String CORE = "Core";
	private static final String PRONOMEN = "pron";

	private Feature pseudoFeat;

	private Feature coreRangeFeat;

	Feature nameFeat;

	private Feature coreFeat;
	private Feature featureIsUncertain;
	private Feature featureNEType;

	// compounds

	private ColViewerComparator colViewerComp;

	private Button buttonshowSocialNetwork;

	Button jumpButton;

	Button notCoreButton;

	private Button idButton;

	private Button buttonOnlyThatAreHandled;

	private boolean importantOnly;

	private Button buttonShowJustCore;

	private Type typeDS;

	private List<AnnotationFS> lastSelection;

	private Button buttonDeleteHandled;

	private Button buttonOldEntity;

	private Button buttonAddRelation;

	private Type relationType;

	private Feature agensFeature;

	private Feature agens2Feature;

	private Feature relationTypeFeature;

	private Feature relationAttributeFeature;

	private Type attSpanType;

	// Eventhandler

	@Inject
	public CorefView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Construct CorefView");

		super.overlay = parent;
		super.overlay.setLayout(new FillLayout());

		super.part = partService.findPart(viewId);

		super.setBroker(broker);
	}

	@Override
	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		super.editor = editor;

		// only initialize if the part is visible

		MUIElement selectedElement = null;
		if (part != null) {
			MElementContainer<MUIElement> parent = part.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but unsubscribe
			// the
			// listener if they are
			// remove all listeners
			System.out.println("Unsubscribe by part");
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
			return;
		}

		boolean typesValid = initTypes();
		initStandardTypes();

		while (!typesValid) {

			// open preference dialogue and ask the user to enter valid types
			CorefPreferencesDialogue dialogue = new CorefPreferencesDialogue(this.overlay.getShell());

			int open = dialogue.open();

			if (open == 0) {
				// success

				HashMap<String, String> preferences = dialogue.getPreferences();

				// save all the preferences to the store
				for (String l : preferences.keySet()) {

					ApplicationUtil.saveToPreferences(l, preferences.get(l));
				}

			}

			else {
				break;
			}

			typesValid = initTypes();

			// TODO dialog code handling
		}

		// if the part is active add the listener

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// init AnnoList

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(type));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		// ====FREQUENCIES=======
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getCoveredText())) {
				int freq = freqMap.get(anno.getCoveredText());
				freqMap.put(anno.getCoveredText(), freq + 1);
			}

			else {
				freqMap.put(anno.getCoveredText(), 1);
			}
		}

		initLayout(overlay);

		if (!isSubscribedToEditorEvents) {
			System.out.println("ubscribe if not done yet");
			super.standardSubscribeToEditorEvents(CLASS_THIS);
		}

		// ====reload=====

		overlay.layout();
		overlay.getParent().layout();
	}

	@Override
	public boolean initTypes() {

		type = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_TYPE, "null");
		typeOverride = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_OV_TYPE, "null");

		if (type.equals("null") || typeOverride.equals("null")) {
			return false;
		}

		this.typ = editor.getCas().getTypeSystem().getType(type);

		if (this.typ == null) {
			return false;
		}

		// set everything unnneded invisible

		this.editor.allStylesInvisible();

		// get Features of NE Type

		String idPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_ID, "null");
		String numPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NUMERUS, "null");
		String genPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_GENDER, "null");
		String namePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NAME, "null");
		String pseudoPref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_PSEUDO, "null");
		String corePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_CORE, "null");
		String isUncertain = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_UNCERTAIN, "null");
		String neTypePref = ApplicationUtil.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_NETYPE, "null");
		String coreRangePref = ApplicationUtil
				.getFromPreferences(ApplicationPreferences.COREFVIEW_NE_CORERANGE, "null");

		if (idPref.equals("null") || numPref.equals("null") || coreRangePref.equals("null") || namePref.equals("null")
				|| pseudoPref.equals("null")) {
			return false;
		}

		// TODO null handling
		this.idFeat = this.typ.getFeatureByBaseName(idPref);
		this.coreFeat = this.typ.getFeatureByBaseName("CoreNamedEntity");
		this.nameFeat = this.typ.getFeatureByBaseName(namePref);
		this.pseudoFeat = this.typ.getFeatureByBaseName(pseudoPref);
		this.numerusFeat = this.typ.getFeatureByBaseName(numPref);
		this.genderFeat = this.typ.getFeatureByBaseName(genPref);
		this.coreRangeFeat = this.typ.getFeatureByBaseName(coreRangePref);
		this.featureIsUncertain = this.typ.getFeatureByBaseName(isUncertain);
		this.featureNEType = this.typ.getFeatureByBaseName(neTypePref);

		// Override Type and Temporary Type
		this.typOverride = (editor.getCas().getTypeSystem().getType(typeOverride));
		this.typTemp = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp2");

		// relationtype
		this.relationType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Relation");
		this.attSpanType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.AttributeSpan");
		this.agensFeature = relationType.getFeatureByBaseName("Agens");
		this.agens2Feature = relationType.getFeatureByBaseName("Agens2");
		this.relationTypeFeature = relationType.getFeatureByBaseName("Type");
		this.relationAttributeFeature = relationType.getFeatureByBaseName("Attribut");

		// directspeech type TODO aufpassen !!

		this.typeDS = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.DirectSpeech");

		// Visibility

		editor.changeVisibility(typ, true);
		editor.changeVisibility(typOverride, true);
		editor.changeVisibility(typTemp, true);
		editor.changeVisibility(relationType, true);

		if (typeDS != null) {

			editor.changeVisibility(typeDS, true);
		}

		// layer

		editor.changeLayer(typ, 0);
		// editor.changeLayer(temporary, 0);
		editor.changeLayer(typOverride, 1);

		if (typeDS != null) {

			editor.changeLayer(typeDS, 0);
		}

		// relation attribute span
		StyleRange rangeTypAtt = new StyleRange();
		rangeTypAtt.background = new Color(Display.getCurrent(), 0, 200, 100);
		AnnoStyle styleAtt = new AnnoStyle(new BackgroundDrawingStrategy(rangeTypAtt), rangeTypAtt);

		editor.changeStyle(attSpanType, styleAtt);
		editor.changeLayer(attSpanType, 3);
		editor.changeVisibility(attSpanType, true);
		// yellow
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle style2 = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		// style - red
		StyleRange rangeTyp = new StyleRange();
		rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle style = new AnnoStyle(new NamedEntityDrawingStrategy(rangeTyp, idFeat.getShortName(),
				coreFeat.getShortName(), pseudoFeat.getShortName(), numerusFeat.getShortName(),
				genderFeat.getShortName(), coreRangeFeat.getShortName()), rangeTyp);

		// TEMP
		StyleRange rangeTemp = new StyleRange();
		rangeTemp.background = new Color(overlay.getDisplay(), 140, 120, 31);
		AnnoStyle styleTemp = new AnnoStyle(new OvalDrawingStrategy(rangeTemp), rangeTemp);
		// DS if possible

		StyleRange rangeDS = new StyleRange();
		rangeDS.background = new Color(overlay.getDisplay(), 0, 0, 255);
		AnnoStyle styleDS = new AnnoStyle(new DSSpeakerIDDrawingStrategy(rangeDS, "ID", "Speaker"), rangeDS);

		// RELATIONS
		StyleRange rangeTypRel = new StyleRange();
		rangeTypRel.background = new Color(Display.getCurrent(), 32, 47, 214);
		AnnoStyle styleRel = new AnnoStyle(new RelationDanielDrawingStrategy(rangeTypRel, agensFeature.getShortName(),
				agens2Feature.getShortName()), rangeTypRel);

		editor.changeStyle(relationType, styleRel);
		// apply
		editor.changeStyle(typ, style);
		editor.changeStyle(typTemp, styleTemp);
		editor.changeStyle(typOverride, style2);

		if (typeDS != null) {

			editor.changeStyle(typeDS, styleDS);
		}

		// also change the modus
		editor.setActiveMode(typ);

		return true;

	}

	@Override
	protected void initLayout(Composite parent) {
		// this.overlay = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 5;
		layout.numColumns = 2;
		layout.verticalSpacing = 5;
		layout.marginWidth = 5;
		layout.marginHeight = 5;
		overlay.setLayout(layout);

		// / ==== Buttons + Fields

		Composite comp2 = new Composite(overlay, SWT.BORDER);
		comp2.setVisible(true);
		GridLayout gdd = new GridLayout();
		GridData gridData2 = new GridData();
		gridData2.horizontalSpan = 2;
		gridData2.grabExcessHorizontalSpace = true;
		// gridData2.grabExcessVerticalSpace = true;
		gridData2.horizontalAlignment = GridData.FILL;
		comp2.setLayout(gdd);
		comp2.setLayoutData(gridData2);

		// Label actions = new Label(comp2, SWT.NULL);
		// actions.setText("Actions");
		// Composite comp = new Composite(overlay, SWT.BORDER);
		// comp.setVisible(true);

		GridLayout gridBelow = new GridLayout(3, false);
		comp2.setLayout(gridBelow);
		Button buttonNewEntity = new Button(comp2, SWT.NONE);
		// GridData gdButton = new GridData(GridData.GRAB_HORIZONTAL |
		// GridData.FILL_HORIZONTAL);
		// newEntity.setLayoutData(gdButton);
		buttonNewEntity.setText("New Entity");
		buttonNewEntity.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// remove vorschlag
				CAS cas = editor.getCas();
				Type t = cas.getTypeSystem().getType(type);
				// get the selection

				if (selected == null) {

					Point selection = editor.getActualSelectedText();
					if (selection == null || selection.x == selection.y) {
						return;
					}
					AnnotationFS anno = cas.createAnnotation(t, selection.x, selection.y);

					// anno.setFeatureValueFromString(idFeat,
					// getNextPossibleId());

					addAnnotation(anno);

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

				widgetSelected(e);

			}
		});
		// listener

		buttonOldEntity = new Button(comp2, SWT.NONE);
		buttonOldEntity.setText("Is Entity");

		buttonOldEntity.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// old means we create an annotation AND set the id to the
				// selected old
				if (selected != null) {

					// also update the name if necessary if the ID is already
					// present
					int cnt = viewer.getTable().getItemCount();
					String name = selected.getCoveredText();
					String id = selected.getFeatureValueAsString(idFeat);
					for (int i = 0; i < cnt; i++) {
						AnnotationFS anno = (AnnotationFS) viewer.getElementAt(i);
						String idComp = anno.getFeatureValueAsString(idFeat);
						if (idComp.equals(id)) {
							name = anno.getFeatureValueAsString(nameFeat);
							break;
						}
					}
					selected.setFeatureValueFromString(nameFeat, name);
					// document.update(selected);
					updateAnnotation(selected, nameFeat, name, true);

				}

			}

		});

		Button buttonNoEntity = new Button(comp2, SWT.NONE);
		buttonNoEntity.setText("Delete Entity");
		buttonNoEntity.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// we delete only the one
				if (selected != null) {

					deleteAnnotation(selected);
					// /deleteOverrideType(selected);
					// this.editor.getDocument().removeFeatureStructure(selected);

				}
				// this.editor.selectAndReveal(3, 7);

			}

		});
		buttonshowSocialNetwork = new Button(comp2, SWT.PUSH);

		buttonshowSocialNetwork.setText("Social network");

		buttonshowSocialNetwork.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

//				SocialNetWorkJungGenerator socialNetWork = new SocialNetWorkJungGenerator(editor,
//						CorefView.this.overlay.getShell());

			}

		});
		Button buttonDummy = new Button(comp2, SWT.NONE);
		buttonDummy.setVisible(false);

		buttonDeleteHandled = new Button(comp2, SWT.PUSH);

		buttonDeleteHandled.setText("Delete Handled");

		buttonDeleteHandled.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
				for (AnnotationFS handled : editor.getCas().getAnnotationIndex(typOverride)) {
					toDelete.add(handled);
				}

				for (AnnotationFS a : toDelete) {
					editor.getCas().removeFsFromIndexes(a);
				}

				if (toDelete.size() > 0) {
					editor.removeAnnotation(toDelete.get(0));
				}

			}

		});

		// ====Relation buttons====

		buttonAddRelation = new Button(comp2, SWT.PUSH);
		buttonAddRelation.setText("Add Relation");
		buttonAddRelation.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();
				if (selectionText.isEmpty())
					return;

				// create and add an empty relation
				Point selection = editor.getWidget().getSelection();
				AnnotationFS anno = editor.getCas().createAnnotation(relationType, selection.x, selection.y);
				editor.addAnnotation(anno);
			}

		});

		Button buttonDummy2 = new Button(comp2, SWT.PUSH);
		buttonDummy2.setVisible(false);

		Button buttonRemoveRelation = new Button(comp2, SWT.PUSH);
		buttonRemoveRelation.setText("Remove Relation");
		buttonRemoveRelation.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				List<AnnotationFS> covAnnos = editor.getCoveringAnnotationOfType(editor.getWidget().getCaretOffset(),
						relationType);

				for (AnnotationFS a : covAnnos) {
					AnnotationFS att = (AnnotationFS) a.getFeatureValue(relationAttributeFeature);
					if (att != null) {
						editor.removeAnnotation(att);
					}
					editor.removeAnnotation(a);
				}

			}

		});

		Composite composite = new Composite(overlay, SWT.NULL);
		gdd = new GridLayout();
		gdd.numColumns = 3;
		gdd.marginBottom = gdd.marginTop = gdd.marginLeft = gdd.marginRight = 0;
		composite.setLayout(gdd);
		// // ==== Table
		comp2.setSize(overlay.getSize());

		Composite compp = new Composite(overlay, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		idButton = new Button(compp, SWT.CHECK);
		idButton.setText("ID Filter active:");
		idButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		idButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				renewPage();
			}
		});

		buttonOnlyThatAreHandled = new Button(compp, SWT.CHECK);
		buttonOnlyThatAreHandled.setSelection(true);
		buttonOnlyThatAreHandled.setText("Show only handled");

		buttonOnlyThatAreHandled.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				renewPage();
			}
		});

		buttonShowJustCore = new Button(compp, SWT.CHECK);
		buttonShowJustCore.setText("Draw just Core");

		buttonShowJustCore.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				boolean selection = buttonShowJustCore.getSelection();

				if (selection) {
					// surpress all non CORE
					for (AnnotationFS anno : annoList) {
						String type = anno.getFeatureValueAsString(featureNEType);

						if (!type.toLowerCase().equals("core")) {
							editor.surpressAnnotation(anno, true);
							AnnotationFS handled = getCoveringHandled(anno);
							if (handled != null) {
								editor.surpressAnnotation(handled, true);
							}
						}
					}
				}

				else {

					// dont surpress anything
					for (AnnotationFS anno : annoList) {
						String type = anno.getFeatureValueAsString(featureNEType);

						if (!type.toLowerCase().equals("core")) {
							editor.surpressAnnotation(anno, false);
							AnnotationFS handled = getCoveringHandled(anno);
							if (handled != null) {
								editor.surpressAnnotation(handled, false);
							}
						}
					}
				}

				editor.invokeReload();

			}

			private AnnotationFS getCoveringHandled(AnnotationFS anno) {

				for (AnnotationFS handled : editor.getCas().getAnnotationIndex(typOverride)) {

					if (handled.getBegin() == anno.getBegin() && handled.getEnd() == anno.getEnd())
						return handled;
				}
				return null;
			}
		});

		// no jump option !

		jumpButton = new Button(compp, SWT.CHECK);
		jumpButton.setText("Jump to next?");
		jumpButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		jumpButton.setSelection(true);

		Composite compTable = new Composite(compp, SWT.NONE);
		compTable.setLayout(new GridLayout(2, false));
		GridData layoutData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_BOTH | GridData.GRAB_VERTICAL);
		layoutData.horizontalSpan = 2;
		compTable.setLayoutData(layoutData);

		// searchLabel
		Label searchLabel = new Label(compTable, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compTable, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		viewer = new TableViewer(compTable, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		// add Menu
		// MenuListener
		final Menu menu = new Menu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// Delete
		MenuItem del = new MenuItem(menu, SWT.None);

		del.setText("Delete Selected Annotation");

		del.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				deleteAnnotation(firstElement);

			}

		});

		// CoreFeature
		MenuItem core = new MenuItem(menu, SWT.None);

		core.setText("Change CoreFeatureValue (Strg + k)");

		core.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String string = firstElement.getFeatureValueAsString(coreFeat);

				if (string == null) {
					string = "true";
				}
				string = string.equals("true") ? "false" : "true";

				// create the dialog
				CoreMarkingDialog dialog = new CoreMarkingDialog(CorefView.this.overlay.getShell(), firstElement);

				int open = dialog.open();
				String coreValue = firstElement.getFeatureValueAsString(coreFeat);
				Boolean b = new Boolean(coreValue);
				b = !b;

				if (open == 0) {

					// ugly code ftw :)
					Boolean boolean1 = new Boolean(dialog.getCoreSpan().contains(
							new Integer(Integer.MAX_VALUE).toString()));
					boolean1 = !boolean1;
					updateAnnotation(firstElement, featureNEType, CORE, false);
					updateAnnotation(firstElement, coreFeat, boolean1.toString(), false);
					updateAnnotation(firstElement, coreRangeFeat, dialog.getCoreSpan(), true);
				}

				// end dialog

				// updateAnnotation(firstElement, coreFeat, string, true);

			}

		});

		// CoreFeature
		MenuItem numerus = new MenuItem(menu, SWT.None);

		numerus.setText("Change NumerusFeature (Strg + u)");

		numerus.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String string = firstElement.getFeatureValueAsString(numerusFeat);

				if (string == null) {
					string = "pl";
				}
				string = string.equals("pl") ? "sin" : "pl";

				updateAnnotation(firstElement, numerusFeat, string, true);

			}

		});

		MenuItem uncertain = new MenuItem(menu, SWT.None);

		uncertain.setText("Change Uncertain Feature (Strg + c)");

		uncertain.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String string = firstElement.getFeatureValueAsString(featureIsUncertain);

				if (string == null) {
					string = "";
				}
				string = string.equals("") ? "X" : "";

				updateAnnotation(firstElement, featureIsUncertain, string, true);

			}

		});

		MenuItem neType = new MenuItem(menu, SWT.None);

		neType.setText("Change NEType Feature (Strg + t)");

		neType.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String neType = firstElement.getFeatureValueAsString(featureNEType);

				// Core -> APPA ->APPTDFW -> PRON -> Core

				switch (neType) {
				case CORE:
					neType = APPELATIV_ABSTRAKTA;
					break;

				case APPELATIV_ABSTRAKTA:
					neType = APPELATIV_TEIL_FIKT_WELT;
					break;
				case APPELATIV_TEIL_FIKT_WELT:
					neType = PRONOMEN;
					break;
				default:
					neType = CORE;
					break;
				}

				updateAnnotation(firstElement, featureNEType, neType, true);

			}

		});

		// CoreFeature
		MenuItem pseudo = new MenuItem(menu, SWT.None);

		pseudo.setText("Change PseudoFeatureValue (Strg + p)");

		pseudo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String string = firstElement.getFeatureValueAsString(pseudoFeat);

				if (string == null) {
					string = "true";
				}
				string = string.equals("true") ? "false" : "true";

				updateAnnotation(firstElement, pseudoFeat, string, true);

			}

		});

		// GoTo

		MenuItem goTo = new MenuItem(menu, SWT.None);

		goTo.setText("GoTo Selected Annotation");

		goTo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				int beg = firstElement.getBegin();

				changeHighlighting(beg, firstElement.getEnd(), true);

			}

		});

		// highlight chain

		MenuItem hihgLightChain = new MenuItem(menu, SWT.None);

		hihgLightChain.setText("Highlight Chain");

		hihgLightChain.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				String idFeature = firstElement.getFeatureValueAsString(idFeat);

				// generate annotations of a temporary type that will only
				// highlight the annotations in that chain

				annotatChainAsTemporarys(idFeature);

			}

			private void annotatChainAsTemporarys(String idFeature) {

				Type temp = editor.getCas().getTypeSystem().getType(sTemporaryType);
				// delete all temporarys

				List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

				for (AnnotationFS a : editor.getCas().getAnnotationIndex(temp)) {
					toDelete.add(a);
				}

				editor.changeVisibility(temp, true);
				editor.changeLayer(temp, 10);

				for (AnnotationFS a : toDelete) {
					editor.getCas().removeFsFromIndexes(a);
				}

				for (AnnotationFS a : annoList) {
					if (idFeature.equals(a.getFeatureValueAsString(idFeat))) {
						AnnotationFS createAnnotation = editor.getCas()
								.createAnnotation(temp, a.getBegin(), a.getEnd());

						editor.addAnnotation(createAnnotation);
					}
				}

			}

		});

		MenuItem important = new MenuItem(menu, SWT.None);

		important.setText("Show Important Persons");

		importantOnly = false;
		important.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				// change the style of the namedentities TODO dirty
				Feature isImp = typ.getFeatureByBaseName("IsImportant");

				// style - red - deactivation
				if (importantOnly == true) {
					StyleRange rangeTyp = new StyleRange();
					rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
					AnnoStyle style = new AnnoStyle(new NamedEntityDrawingStrategy(rangeTyp, idFeat.getShortName(),
							coreFeat.getShortName(), pseudoFeat.getShortName(), numerusFeat.getShortName(), genderFeat
									.getShortName(), coreRangeFeat.getShortName()), rangeTyp);

					editor.changeStyle(typ, style);

					editor.changeVisibility(typOverride, true);
					importantOnly = false;
				}

				else {

					StyleRange rt = new StyleRange();

					rt.background = new Color(overlay.getDisplay(), 255, 0, 0);

					AnnoStyle style = new AnnoStyle(new ImportantPPlOnlyDrawingStrategy(isImp, idFeat), rt);

					editor.changeStyle(typ, style);
					editor.changeVisibility(typOverride, false);
					importantOnly = true;
				}

			}

		});

		// KEYLISTENER
		// enter listener to update the values
		viewer.getTable().addKeyListener(new KeyAdapter() {

			@Override
			public void keyPressed(KeyEvent e) {
				if (((IStructuredSelection) viewer.getSelection()).getFirstElement() == null) {
					return;
				}

				if (e.keyCode == SWT.CR) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
					AnnotationFS anno = (AnnotationFS) selection.getFirstElement();
					String id = anno.getFeatureValueAsString(idFeat);
					ArrayList<AnnotationFS> list = new ArrayList<AnnotationFS>();

					for (AnnotationFS a : annoList) {
						if (a.getFeatureValueAsString(idFeat).equals(id)) {
							list.add(a);
						}
					}

					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "", "Enter a new Name",
							anno.getFeatureValueAsString(nameFeat), null);
					if (dlg.open() == Window.OK) {

						// for (AnnotationFS a : list) {
						// a.setFeatureValueFromString(nameFeat,
						// dlg.getValue());
						// }
						// // document.update(list);
						// document.updateFeatureStructure(list);
						for (int i = 1; i < list.size(); i++) {
							AnnotationFS a = list.get(i);
							updateAnnotation(a, nameFeat, dlg.getValue(), false);
						}
						if (list.size() > 0) {
							AnnotationFS a = list.get(0);

							updateAnnotation(a, nameFeat, dlg.getValue(), true);
						}
					}

				}
				// coreFeat
				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'k')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					// create the dialog
					CoreMarkingDialog dialog = new CoreMarkingDialog(CorefView.this.overlay.getShell(), selected);

					int open = dialog.open();
					String coreValue = selected.getFeatureValueAsString(coreFeat);
					Boolean b = new Boolean(coreValue);
					b = !b;

					if (open == 0) {

						// ugly code ftw :)
						Boolean boolean1 = new Boolean(dialog.getCoreSpan().contains(
								new Integer(Integer.MAX_VALUE).toString()));
						boolean1 = !boolean1;
						updateAnnotation(selected, featureNEType, CORE, false);
						updateAnnotation(selected, coreFeat, boolean1.toString(), false);
						updateAnnotation(selected, coreRangeFeat, dialog.getCoreSpan(), true);
					}

					// end dialog

					viewer.setSelection(selection, true);

				}

				// numerusFeat
				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'u')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String numerusValue = selected.getFeatureValueAsString(numerusFeat);
					if (numerusValue == null) {
						numerusValue = "pl";
					}
					if (numerusValue.equals("pl")) {
						numerusValue = "si";
					}

					else {
						numerusValue = "pl";
					}
					updateAnnotation(selected, numerusFeat, numerusValue, true);

					viewer.setSelection(selection, true);

				}

				// uncertain
				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'c')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String uncertainValue = selected.getFeatureValueAsString(featureIsUncertain);
					if (uncertainValue == null) {
						uncertainValue = "";
					}
					if (uncertainValue.equals("")) {
						uncertainValue = "X";
					}

					else {
						uncertainValue = "";
					}
					updateAnnotation(selected, featureIsUncertain, uncertainValue, true);

					viewer.setSelection(selection, true);

				}

				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 't')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String neType = selected.getFeatureValueAsString(featureNEType);

					// Core -> APPA ->APPTDFW -> PRON -> Core

					switch (neType) {
					case CORE:
						neType = APPELATIV_ABSTRAKTA;
						break;

					case APPELATIV_ABSTRAKTA:
						neType = APPELATIV_TEIL_FIKT_WELT;
						break;
					case APPELATIV_TEIL_FIKT_WELT:
						neType = PRONOMEN;
						break;
					default:
						neType = CORE;
						break;
					}

					updateAnnotation(selected, featureNEType, neType, true);

					viewer.setSelection(selection, true);

				}

				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'g')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String genderValue = selected.getFeatureValueAsString(genderFeat);
					if (genderValue == null) {
						genderValue = "m";
					} else if (genderValue.equals("m")) {
						genderValue = "f";
					} else if (genderValue.equals("f")) {
						genderValue = "n";
					} else if (genderValue.equals("n")) {
						genderValue = "m";
					}

					// find all with the same id and also update them
					// for (AnnotationFS a : annoList) {
					//
					// String aID = a.getFeatureValueAsString(idFeat);
					//
					// if (aID.equals(selected.getFeatureValueAsString(idFeat)))
					// {
					// updateAnnotation(a, genderFeat, genderValue, false);
					// }
					// }
					updateAnnotation(selected, genderFeat, genderValue, true);

					viewer.setSelection(selection, true);

				}

				// pseudoFeat
				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'p')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String pseudoValue = selected.getFeatureValueAsString(pseudoFeat);
					Boolean b = new Boolean(pseudoValue);
					b = !b;
					updateAnnotation(selected, pseudoFeat, b.toString(), true);

					viewer.setSelection(selection, true);

				}
			}
		});

		// MOUSE DOUBLE CLICK LISTENER
		viewer.getTable().addListener(SWT.MouseDoubleClick, new Listener() {

			@Override
			public void handleEvent(Event e) {
				// get the selected item
				String id = null;
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS selected = (AnnotationFS) sel.getFirstElement();
				if (selected == null) {
					return;
				}

				AnnotationFS coveringNE = getCoveringNE(editor.getWidget().getCaretOffset());

				id = selected.getFeatureValueAsString(idFeat);
				String names = selected.getFeatureValueAsString(nameFeat);

				// get the selection in the cas editor

				Point pSelect = editor.getActualSelectedText();

				if (pSelect != null && pSelect.y - pSelect.x == 0 && coveringNE != null) {
					// here we update the Features of the annotation

					AnnotationFS anno = coveringNE;

					System.out.println("Set feature " + anno.getCoveredText() + " " + id);

					// use the last Feature change to reload the page
					anno.setFeatureValueFromString(nameFeat, names);
					updateAnnotation(anno, idFeat, id, true);
				}

				else {

					if (coveringNE != null) {
						// update the Annotation
						AnnotationFS ano = coveringNE;

						if (ano.getType().getName().equals(typ.getName())) {

							// ano.setFeatureValueFromString(idFeat, id);
							ano.setFeatureValueFromString(nameFeat, names);

							updateAnnotation(ano, idFeat, id, true);

						}

						else {

							// create An Annotation with the correct span and
							// ids
							AnnotationFS anno = editor.getCas().createAnnotation(typ, editor.getActualSelectedText().x,
									editor.getActualSelectedText().y);

							String name = anno.getFeatureValueAsString(nameFeat);
							if (name == "" || name == null) {
								anno.setFeatureValueFromString(nameFeat, anno.getCoveredText());
							}

							anno.setFeatureValueFromString(idFeat, id);
							anno.setFeatureValueFromString(nameFeat, names);

							addAnnotation(anno);

						}
					}

					else {

						if (editor.getActualSelectedText() != null) {
							// create An Annotation with the correct span and
							// ids
							AnnotationFS anno = editor.getCas().createAnnotation(typ, editor.getActualSelectedText().x,
									editor.getActualSelectedText().y);

							String name = anno.getFeatureValueAsString(nameFeat);
							if (name == "" || name == null) {
								anno.setFeatureValueFromString(nameFeat, anno.getCoveredText());
							}

							anno.setFeatureValueFromString(idFeat, id);
							anno.setFeatureValueFromString(nameFeat, names);

							addAnnotation(anno);
						}
					}
				}

			}

		});

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);
		// New to support the search

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		// filter
		filter = new NameFilter();
		viewer.addFilter(filter);

		// id filter

		ovFilter = new UnderlyingAnnotationFilter();
		viewer.addFilter(ovFilter);

		idFilter = new IDFilter();
		viewer.addFilter(idFilter);

		createTableViewer();
		// comparator

		this.colViewerComp = new ColViewerComparator(nameFeat, idFeat, featureNEType, this);
		viewer.setComparator(colViewerComp);

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

		// dragn drop
		int operations = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transferTypes = new Transfer[] { LocalSelectionTransfer.getTransfer() };
		viewer.addDragSupport(operations, transferTypes, new TableViewerDragListener(viewer));
		viewer.addDropSupport(operations, transferTypes, new TableViewerDropListener(viewer, this));

	}

	public void renewPage() {

		updateGuideLines();
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(type));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		// update frequencyMap
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getFeatureValueAsString(idFeat))) {
				int freq = freqMap.get(anno.getFeatureValueAsString(idFeat));
				freq += 1;
				freqMap.put(anno.getFeatureValueAsString(idFeat), freq);
			}

			else {
				freqMap.put(anno.getFeatureValueAsString(idFeat), 1);
			}
		}

		//

		Runnable r = new Runnable() {

			@Override
			public void run() {
				createTableViewer();

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	private void updateGuideLines() {

		// if (editor.getCas().getAnnotationIndex(editorErrorType).size() > 0)
		// return;
		//
		// for (AnnotationFS a : editor.getCas().getAnnotationIndex(typ)) {
		//
		// if (a.getCoveredText().startsWith("Gro")) {
		// AnnotationFS createAnnotation =
		// editor.getCas().createAnnotation(editorErrorType, a.getBegin(),
		// a.getEnd());
		//
		// editor.getCas().addFsToIndexes(createAnnotation);
		// }
		// }

	}

	public boolean isUserIndependentJumpFeature() {
		return userIndependentJumpFeature;
	}

	public void setUserIndependentJumpFeature(boolean userIndependentJumpFeature) {
		this.userIndependentJumpFeature = userIndependentJumpFeature;
	}

	private void createTableViewer() {

		Integer maxId = 0;
		for (AnnotationFS annot : this.annoList) {
			if (annot.getFeatureValueAsString(idFeat) != null && annot.getFeatureValueAsString(idFeat) != "") {
				Integer ids = Integer.parseInt(annot.getFeatureValueAsString(idFeat));
				if (ids > maxId) {
					maxId = ids;
				}
			}
		}
		for (AnnotationFS anno : this.annoList) {

			String name = anno.getFeatureValueAsString(nameFeat);
			if (name == "" || name == null) {
				anno.setFeatureValueFromString(nameFeat, anno.getCoveredText());
			}

			String numerus = anno.getFeatureValueAsString(numerusFeat);
			if (numerus == "" || numerus == null) {
				anno.setFeatureValueFromString(numerusFeat, "si");
			}

			String isUncertain = anno.getFeatureValueAsString(featureIsUncertain);

			if (isUncertain == null) {
				isUncertain = "";
			}

			String featNEType = anno.getFeatureValueAsString(featureNEType);

			if (featNEType == null && anno.getCoveredText().length() != 0) {

				if (Character.isUpperCase(anno.getCoveredText().charAt(0)))

					if (anno.getFeatureValueAsString(coreRangeFeat) == null
							|| anno.getFeatureValueAsString(coreRangeFeat).isEmpty()) {
						featNEType = APPELATIV_TEIL_FIKT_WELT;
					} else {
						featNEType = CORE;
					}
				else {
					featNEType = PRONOMEN;
				}

				anno.setFeatureValueFromString(featureNEType, featNEType);
			}

			String id = anno.getFeatureValueAsString(idFeat);
			if (id == "" || id == null) {
				maxId++;
				// unassigned ids to -1
				anno.setFeatureValueFromString(idFeat, maxId.toString());

			}
		}

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(typeOverride));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		ArrayList<AnnotationFS> overrides = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			overrides.add((AnnotationFS) iterator.next());
		}

		// set input of Tableviewer
		viewer.setInput(annoList);
		idFilter.setAnnotations(annoList, idFeat, idButton.getSelection(), overrides);

		ovFilter.setAnnotations(overrides, buttonOnlyThatAreHandled.getSelection());

		viewer.refresh();

	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Name", 100, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(nameFeat);

			}

		});

		TableViewerColumn colFreq = createTableViewerColumn("Freq.", 30, 1);
		colFreq.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String id = p.getFeatureValueAsString(idFeat);
				if (freqMap.containsKey(id)) {
					return freqMap.get(id).toString();

				}

				else {
					return "1";
				}
			}
		});

		TableViewerColumn colID = createTableViewerColumn("ID", 30, 2);
		colID.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(idFeat);
			}
		});

		TableViewerColumn synonyme = createTableViewerColumn("Synonyme", 90, 3);
		synonyme.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getCoveredText();
			}

		});

		TableViewerColumn colCore = createTableViewerColumn("Type", 60, 4);
		colCore.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String text = "";
				String featureValue = p.getFeatureValueAsString(featureNEType);

				return featureValue;
			}
		});

		TableViewerColumn colIsUncertain = createTableViewerColumn("Uncertain?", 30, 5);
		colIsUncertain.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String featureValue = p.getFeatureValueAsString(featureIsUncertain);
				if (featureValue == null)
					return "";

				else {
					return featureValue;
				}
			}
		});

		TableViewerColumn colNumerus = createTableViewerColumn("Numerus", 30, 6);
		colNumerus.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String featureValue = p.getFeatureValueAsString(numerusFeat);

				return featureValue;
			}
		});

		TableViewerColumn colGender = createTableViewerColumn("Gender", 30, 7);
		colGender.setLabelProvider(new ColumnLabelProvider() {

			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String featureValue = p.getFeatureValueAsString(genderFeat);

				if (featureValue == null)
					return "NOT_SET";

				return featureValue;
			}
		});

		TableViewerColumn colPseudo = createTableViewerColumn("Pseudo", 30, 8);
		colPseudo.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				String text = "";
				String featureValue = p.getFeatureValueAsString(pseudoFeat);
				if (featureValue != null && featureValue.equals("true"))
					text = "X";
				return text;
			}
		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				colViewerComp.setColumn(index);
				int dir = colViewerComp.getDirection();
				viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	// Methods to change the CAS
	private void deleteAnnotation(AnnotationFS firstElement) {

		// delete all overrides in that stage

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typOverride);
		ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : annotationIndex) {
			if (anno.getBegin() == firstElement.getBegin() && anno.getEnd() == firstElement.getEnd()) {
				toDelete.add(anno);
				// editor.removeAnnotation(anno);
			}
		}

		for (AnnotationFS a : toDelete) {
			editor.removeAnnotation(a);
		}

		editor.removeAnnotation(firstElement);

	}

	private void updateAnnotation(AnnotationFS firstElement, Feature feat, String string, boolean reload) {
		// if (feat.getName().equals(idFeat.getName())) {
		// String oldId = firstElement.getFeatureValueAsString(idFeat);
		// ICorefResolvingStrategy naiveResolver = new
		// NaiveResolvingStrategy(typ, typOverride, idFeat, oldId, string);
		// naiveResolver.resolve(editor.getCas(), firstElement);
		// }

		firstElement.setFeatureValueFromString(feat, string);

		// delete all overrides in that stage
		if (reload == true) {

			AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typOverride);
			ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
			for (AnnotationFS anno : annotationIndex) {
				if (anno.getBegin() == firstElement.getBegin() && anno.getEnd() == firstElement.getEnd()) {
					toDelete.add(anno);
				}
			}

			for (AnnotationFS a : toDelete) {
				// do not reload if we have to delete something since we add an
				// annotation anyway
				editor.getCas().removeFsFromIndexes(a);
			}
		}

		if (reload) {
			AnnotationFS fss = editor.getCas().createAnnotation(typOverride, firstElement.getBegin(),
					firstElement.getEnd());

			this.editor.addAnnotation(fss);
		}
	}

	// boolean moveCursor
	private void changeHighlighting(int beg, int end, boolean b) {
		editor.highlightRange(beg, end - beg, true);

	}

	private void addAnnotation(AnnotationFS anno) {
		// =====END INTERACTIVE RULES=========
		// add the override Annotation
		CAS cas = this.editor.getCas();

		this.editor.getCas().addFsToIndexes(anno);
		AnnotationFS fs = cas.createAnnotation(typOverride, anno.getBegin(), anno.getEnd());

		this.editor.addAnnotation(fs);

	}

	public List<AnnotationFS> getAnnoList() {
		return annoList;
	}

	public void setAnnoList(List<AnnotationFS> annoList) {
		this.annoList = annoList;
	}

	// boolean moveCursor

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void performDragnDropChange(AnnotationFS from, AnnotationFS to) {

		// change all ids from the from Anno to the To anno

		// finally renew the page

		String fromId = from.getFeatureValueAsString(idFeat);
		String toId = to.getFeatureValueAsString(idFeat);

		for (AnnotationFS a : annoList) {
			String id = a.getFeatureValueAsString(idFeat);

			if (id.equals(fromId)) {
				// update that annotation
				updateAnnotation(a, idFeat, toId, false);

			}
		}

		renewPage();

	}

	public TableViewer getViewer() {
		return viewer;
	}

	public void setViewer(TableViewer viewer) {
		this.viewer = viewer;
	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {

		// delegate to the superclass
		super.onFocus();

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	private AnnotationFS getNextToHighLight(AnnotationFS actual) {

		Collections.sort(this.annoList, new AnnotationComparator());

		for (int i = 0; i < annoList.size(); i++) {
			if (actual.getBegin() < annoList.get(i).getBegin()) {
				return annoList.get(i);
			}
		}
		return null;

	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components

		System.out.println("close corefview");

		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications

		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;
		this.editor = null;

	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

		renewPage();

		// do not jump for a temporary typehcange
		if (added.getType().equals(typTemp))
			return;

		if (getJumpButton().getSelection()) {

			// TODO dirty solution
			if (isUserIndependentJumpFeature() == false) {
				AnnotationFS toHIghlight = getNextToHighLight(added);

				if (toHIghlight != null) {
					getEditor().highlightRange(toHIghlight.getBegin(), toHIghlight.getEnd() - toHIghlight.getBegin(),
							true);

					getEditor().setSelection(toHIghlight);
				}
			}

			else {
				setUserIndependentJumpFeature(false);
			}
		}

	}

	@Override
	public void onAnnotationHovered(org.osgi.service.event.Event e) {

		AnnotationFS hovered = (AnnotationFS) e.getProperty(IEventBroker.DATA);

	}

	private String getNextPossibleId() {

		int max = -1;
		for (AnnotationFS anno : annoList) {

			Integer id = Integer.parseInt(anno.getFeatureValueAsString(idFeat));

			if (id > max)
				max = id;
		}
		return String.valueOf(max + 1);
	}

	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);
		renewPage();

		if (getJumpButton().getSelection()) {
			AnnotationFS toHIghlight = getNextToHighLight(removed);

			if (toHIghlight != null) {
				getEditor().highlightRange(toHIghlight.getBegin(), toHIghlight.getEnd() - toHIghlight.getBegin(), true);

				getEditor().setSelection(toHIghlight);
			}
		}

	}

	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		// unused

	}

	@Override
	public void onAnnotationSelectionChanged(org.osgi.service.event.Event arg0) {
		// unused hjh

	}

	// @Override
	// public void onTextSelectionChanged(org.osgi.service.event.Event arg0) {
	// Point point = (Point) arg0.getProperty(IEventBroker.DATA);
	//
	// // set the Textfield
	// getNeuEnt().setText(getEditor().getWidget().getText().substring(point.x,
	// point.y));
	//
	// // if we got the span of an annotation also create the selection
	// // in the
	// // tableviewer
	//
	// List<AnnotationFS> annoList = getAnnoList();
	//
	// // we dont want to highlight if someone marks a passage
	// if (point.y - point.x > 1) {
	// return;
	// }
	// for (AnnotationFS anno : annoList) {
	// if (anno.getBegin() <= point.x && anno.getEnd() >= point.x) {
	// StructuredSelection structuredSelection = new StructuredSelection(anno);
	// getViewer().setSelection(structuredSelection);
	// break;
	// }
	// }
	//
	// }

	@Override
	public void onEditorDoubleClicked(org.osgi.service.event.Event arg0) {
		// get the selected item
		String id = null;

		AnnotationFS anno = getCoveringNE(editor.getWidget().getCaretOffset());

		AnnotationFS prevSel = null;

		for (AnnotationFS a : this.lastSelection) {

			if (a != anno) {

				prevSel = a;
				break;

			}
		}

		// no click on an annotation
		if (anno == null)
			return;

		if (prevSel == null)
			return;

		// prevSel.setFeatureValueFromString(idFeat,
		// anno.getFeatureValueAsString(idFeat));
		// use the last Feature change to reload the page
		prevSel.setFeatureValueFromString(nameFeat, anno.getFeatureValueAsString(nameFeat));
		updateAnnotation(prevSel, idFeat, anno.getFeatureValueAsString(idFeat), true);

	}

	@Override
	public void onEditorWasScrolled(org.osgi.service.event.Event arg0) {
		// unused

	}

	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event e) {

		int caretPos = (int) e.getProperty(IEventBroker.DATA);

		if (this.lastSelection == null) {
			lastSelection = new ArrayList<AnnotationFS>();
		}

		AnnotationFS coveringNE = getCoveringNE(caretPos);
		if (coveringNE == null)
			return;

		if (!lastSelection.contains(coveringNE)) {

			if (lastSelection.size() < 2) {
				lastSelection.add(coveringNE);
			}

			else {

				lastSelection.remove(0);
				lastSelection.add(coveringNE);
			}
		}

		updateTempType(coveringNE);

	}

	private void updateTempType(AnnotationFS coveringNE) {

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typTemp)) {

			toDelete.add(temp);
		}

		for (AnnotationFS del : toDelete)
			editor.getCas().removeFsFromIndexes(del);

		// add the new one and reload

		AnnotationFS temp = editor.getCas().createAnnotation(typTemp, coveringNE.getBegin(), coveringNE.getEnd());

		editor.addAnnotation(temp);

	}

	private AnnotationFS getCoveringNE(int caretPos) {

		for (AnnotationFS ne : editor.getCas().getAnnotationIndex(typ)) {

			if (ne.getBegin() <= caretPos && ne.getEnd() >= caretPos) {

				return ne;
			}
		}

		return null;
	}

	@Override
	public void onEditorMouseUp(org.osgi.service.event.Event arg0) {
		// unused

	}

	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {
		// unused for this view

	}

	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

		// if we got the span of an annotation also create the selection
		// in the
		// tableviewer

		List<AnnotationFS> annoList = getAnnoList();

		// for (AnnotationFS anno : annoList) {
		// if (anno.getBegin() <= point.x && anno.getEnd() >= point.x) {
		// StructuredSelection structuredSelection = new
		// StructuredSelection(anno);
		// getViewer().setSelection(structuredSelection);
		// break;
		// }
		// }

	}

	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void initModel() {
		// TODO Auto-generated method stub

	}

}
