package de.uniwue.kalimachos.coref.editor.drawingstructure;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;

public class AnnotationSelectionProvider implements ISelectionProvider {
  
  private IStructuredSelection mCurrentSelection = new StructuredSelection();

  private Set<ISelectionChangedListener> mListeners = new HashSet<ISelectionChangedListener>();

  @Override
  public void addSelectionChangedListener(ISelectionChangedListener listener) {
    Assert.isNotNull(listener);

    mListeners.add(listener);
  }

  @Override
  public ISelection getSelection() {
    return mCurrentSelection;
  }

  @Override
  public void removeSelectionChangedListener(ISelectionChangedListener listener) {
    mListeners.remove(listener);
  }


  @Override
  public void setSelection(ISelection selection) {
    Assert.isNotNull(selection);

    IStructuredSelection structuredSelection = (IStructuredSelection) selection;

    mCurrentSelection = structuredSelection;

    for (ISelectionChangedListener listener : mListeners) {
      SelectionChangedEvent event = new SelectionChangedEvent(this, mCurrentSelection);

      listener.selectionChanged(event);
    }
  }

}
