package de.uniwue.kalimachos.coref.editor.drawingstructure;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.apache.uima.cas.Type;

public class StyleDataStructure implements IStyleVisitable {

  // this class contains all the TypeStyleDataStrcutures and knows if a given Type is visible or not

  private  HashMap<TypeStyleDataStructure, Boolean> styleMap;

  public HashMap<TypeStyleDataStructure, Boolean> getStyleMap() {
    return styleMap;
  }

  // Methods that are important

  // ==================Visibility==================
  public void changeVisibilityOfType(Type t) {

    TypeStyleDataStructure type = getDSFromType(t);

    if (type != null) {
      Boolean actual = styleMap.get(type);
      styleMap.put(type, !actual);
    }
  }
  
  public void changeVisibilityOfType(Type t,boolean isVisible) {

    TypeStyleDataStructure type = getDSFromType(t);

    if (type != null) {
      styleMap.put(type, isVisible);
    }
  }
  
  

  public void changeVisibilityOfType(TypeStyleDataStructure ds) {

    if (styleMap.containsKey(ds)) {
      Boolean actual = styleMap.get(ds);
      styleMap.put(ds, !actual);

    }

    else {
      throw new IllegalArgumentException("TypeStyleDataStructure not contained");
    }
  }

  // visibility of a collection

  public void changeVisibility(Collection<TypeStyleDataStructure> structs) {

    for (TypeStyleDataStructure tsds : structs) {
      changeVisibilityOfType(tsds);
    }

  }

  public void changeVisibilityofTypes(Collection<Type> types) {

    for (Type t : types) {
      changeVisibilityOfType(t);
    }
  }

  // ==================END VISIBILITY

  // ==================Layer===================
  

  // END LAYER=====================
  
  //add Types - also includes changes
  
  public void addTypeSytle(TypeStyleDataStructure tsds, Boolean visibility){
    styleMap.put(tsds, visibility);
  }

  // Access to a specific order - needed for drawing

  public List<TypeStyleDataStructure> getStylesFromLowestLayerToHighest() {

    ArrayList<TypeStyleDataStructure> structs = new ArrayList<TypeStyleDataStructure>(
            styleMap.keySet());

    Collections.sort(structs, new LayerComparator());
    return structs;

  }

  public void setStyleMap(HashMap<TypeStyleDataStructure, Boolean> styleMap) {
    this.styleMap = styleMap;
  }

  public StyleDataStructure(HashMap<TypeStyleDataStructure, Boolean> styleMap) {
    super();
    this.styleMap = styleMap;
    
  }

  @Override
  public void accept(IStyleVisitor visitor) {
    visitor.visit(this);

  }

  // helper

  public  TypeStyleDataStructure getDSFromType(Type type) {

    for (TypeStyleDataStructure ds : StyleDataStructure.this.styleMap.keySet()) {
    	//equality by name otherwise comparison from different Cases would not work!!
      if (ds.getAnnotationType().getName().equals(type.getName())) {
        return ds;
      }

    }
    return null;
  }
  
  public HashSet<Type> getVisibleTypes(){
    
    HashSet<Type> visible = new HashSet<Type>();
    
    for(TypeStyleDataStructure tsds : styleMap.keySet()){
      if(styleMap.get(tsds)==true){
        visible.add(tsds.getAnnotationType());
      }
    }
    
    return visible;
  }
  
  public int getLayerNumber(Type t){
    for(TypeStyleDataStructure tsds : styleMap.keySet()){
      
      if(tsds.getAnnotationType().equals(t)){
        return tsds.getLayer();
      }
    }
    return 0;
  }

}
