/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

/**
 * Draws brackets arround an annotation.
 */
public class BracketDrawingStrategy implements IAnnotationDrawingStrategy {
	private static final int BRACKET_WIDTH = 3;

	private StyleRange styleRange;

	public BracketDrawingStrategy(StyleRange styleRange) {

		this.styleRange = styleRange;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS annotation, StyledText textWidget, GC gc) {

		int length = annotation.getEnd() - annotation.getBegin();
		int offset = annotation.getBegin();

		if (length > 0) {

			if (gc != null) {
				Rectangle bounds = textWidget.getTextBounds(offset, offset + length - 1);

				gc.setForeground(styleRange.background);

				boolean isDrawOpenBracket = annotation.getBegin() == offset;
				if (isDrawOpenBracket) {
					gc.drawLine(bounds.x, bounds.y + bounds.height - 1, bounds.x + BRACKET_WIDTH, bounds.y
							+ bounds.height - 1);

					gc.drawLine(bounds.x, bounds.y, bounds.x, bounds.y + bounds.height - 1);

					gc.drawLine(bounds.x, bounds.y, bounds.x + BRACKET_WIDTH, bounds.y);
				}

				boolean isDrawCloseBracket = annotation.getEnd() == offset + length;
				if (isDrawCloseBracket) {
					gc.drawLine(bounds.x + bounds.width, bounds.y + bounds.height - 1, bounds.x + bounds.width
							- BRACKET_WIDTH, bounds.y + bounds.height - 1);

					gc.drawLine(bounds.x + bounds.width - 1, bounds.y, bounds.x + bounds.width - 1, bounds.y
							+ bounds.height - 1);

					gc.drawLine(bounds.x + bounds.width, bounds.y, bounds.x + bounds.width - BRACKET_WIDTH, bounds.y);
				}
			} else {
				textWidget.redrawRange(offset, length, true);
			}

		}
		gc.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}
}