package de.uniwue.kalimachos.coref.paintingStrategies;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class ConditionalBackGroundDrawingStrategy implements IExpensiveDrawingStrategy {

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy
	 * #drawAnnotation
	 * (de.uniwue.kalimachos.coref.editor.ui.TypeStyleDataStructure,
	 * org.apache.uima.cas.text.AnnotationFS, org.eclipse.swt.custom.StyledText,
	 * org.eclipse.swt.graphics.GC)
	 * 
	 * this strategy has the power to draw diffrent Annotations of one type with
	 * diffrent background colors
	 */

	// the color of the background if no condition is fullfilled
	Color defaultColor;

	// a mapping between a value and a color
	private HashMap<String, Color> conditionsMap;
	Feature f;
	StyleRange sr;

	public ConditionalBackGroundDrawingStrategy(StyleRange styleRange, Feature feat, HashMap<String, Color> conditions) {
		this.defaultColor = styleRange.background;
		this.conditionsMap = conditions;
		this.f = feat;
		this.sr = styleRange;
	}

	public Color getColor(AnnotationFS a) {

		if (conditionsMap.containsKey(a.getFeatureValueAsString(f))) {
			return conditionsMap.get(a.getFeatureValueAsString(f));
		}

		return defaultColor;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy
	 * #drawAnnotation
	 * (de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure
	 * , org.apache.uima.cas.text.AnnotationFS,
	 * org.eclipse.swt.custom.StyledText, org.eclipse.swt.graphics.GC) Draws the
	 * annotation based on the feature value F stored in the stylemap
	 */
	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		StyleRange styleRange = new StyleRange();
		styleRange.start = anno.getBegin();
		styleRange.length = anno.getEnd() - anno.getBegin();

		Color color = conditionsMap.get(anno.getFeatureValueAsString(f));

		if (color == null) {
			styleRange.background = defaultColor;
		} else {

			styleRange.background = conditionsMap.get(anno.getFeatureValueAsString(f));
		}
		widget.setStyleRange(styleRange);

		// Dispose ressource
		gc.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}

}
