package de.uniwue.kalimachos.coref.paintingStrategies;

import java.util.Collection;
import java.util.HashMap;

import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class DualEditorDrawingStrategy implements IAnnotationDrawingStrategy {

	private HashMap<AnnotationFS, AnnotationFS> goldCompMapping;
	private HashMap<AnnotationFS, AnnotationFS> combGoldMapping;

	// Colors

	private static final int FONTSIZE = 8;
	private Collection<FeaturePositionPair> pairs;

	// TODO replace this with an ID
	private String whichEditor;

	// only display diffrences
	private boolean onlyDiffrences;

	private Type actualType;

	public DualEditorDrawingStrategy(Collection<FeaturePositionPair> collection,
			AnnotationIndex<AnnotationFS> annotationIndexGold, AnnotationIndex<AnnotationFS> annotationIndex2,
			boolean onlyDiffrences) {

		this.pairs = collection;
		this.combGoldMapping = new HashMap<AnnotationFS, AnnotationFS>();
		this.goldCompMapping = new HashMap<AnnotationFS, AnnotationFS>();

		// TODO this is shitty
		for (AnnotationFS a : annotationIndexGold) {
			this.actualType = a.getType();
			break;
		}

		this.onlyDiffrences = onlyDiffrences;

		for (AnnotationFS a : annotationIndexGold) {

			for (AnnotationFS b : annotationIndex2) {

				if (a.getBegin() == b.getBegin() && a.getEnd() == b.getEnd()) {
					this.combGoldMapping.put(b, a);
					this.goldCompMapping.put(a, b);
					break;
				}
			}
		}

	}

	public Type getActualType() {
		return actualType;
	}

	public void setActualType(Type actualType) {
		this.actualType = actualType;
	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		if (widget.getData("NAME").equals("Gold")) {
			this.whichEditor = "Gold";
		}

		else {
			this.whichEditor = "Compare";
		}

		// System.out.println("Draw");
		Color greenColor = new Color(gc.getDevice(), new RGB(84, 232, 46));
		Color darkgreenColor = new Color(gc.getDevice(), new RGB(66, 125, 74));
		Color blue = new Color(gc.getDevice(), new RGB(0, 100, 255));
		Color red = new Color(gc.getDevice(), new RGB(232, 93, 46));
		Color yellow = new Color(gc.getDevice(), new RGB(242, 141, 39));

		int length = anno.getEnd() - anno.getBegin();

		// cant draw an empty annotation
		if (length == 0) {
			return;
		}

		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;
		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		// now determine if the annotation has an analog element
		// the anno of the input is always contained in the goldset !!

		AnnotationFS analogElement = getAnalogElement((String) widget.getData("NAME"), anno);

		// if there is an analog element everything might be printed green
		if (analogElement != null) {

			for (FeaturePositionPair pair : pairs) {

				if (featuresAreEqual(anno, analogElement, pair)) {

					if (!onlyDiffrences) {
						printFeature(bounds, lineHeight, pair, anno, gc, darkgreenColor, widget);
					}
				}

				else {
					// features arent equal

					printFeature(bounds, lineHeight, pair, anno, gc, yellow, widget);
				}

			}

		}

		// else its either all red or all blue

		else {

			if (this.whichEditor.equals("Gold")) {

				// print red

				for (FeaturePositionPair pair : pairs) {

					printFeature(bounds, lineHeight, pair, anno, gc, red, widget);

				}

			}

			else {

				// print blue
				for (FeaturePositionPair pair : pairs) {

					printFeature(bounds, lineHeight, pair, anno, gc, blue, widget);

				}
			}

		}

		gc.dispose();
		greenColor.dispose();
		red.dispose();
		blue.dispose();
		yellow.dispose();
		darkgreenColor.dispose();
	}

	private AnnotationFS getAnalogElement(String data, AnnotationFS anno) {

		if (data.equals("Gold")) {

			return this.goldCompMapping.get(anno);
		}
		return this.combGoldMapping.get(anno);
	}

	private boolean featuresAreEqual(AnnotationFS anno, AnnotationFS analogElement, FeaturePositionPair pair) {

		String annoF = anno.getFeatureValueAsString(pair.getFeature());
		String compF = analogElement.getFeatureValueAsString(pair.getFeature());
		if (annoF == null || compF == null) {
			return false;
		}
		return annoF.equals(compF);
	}

	private void printFeature(Rectangle bounds, int lineHeight, FeaturePositionPair pair, AnnotationFS anno, GC gc,
			Color color, StyledText widget) {

		String value = anno.getFeatureValueAsString(pair.getFeature());
		if (value == null) {
			return;
		}

		gc.setForeground(color);
		FeaturePosition position = pair.getPosition();
		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();

		if (position.equals(FeaturePosition.TEXT)) {

			// this is almost impossible here, if we call setStyleRange of the
			// widget it paints itself and we end in an
			// endless loop
		}

		else if (position.equals(FeaturePosition.LEFT)) {

			writeLeft(bounds, lineHeight, gc, widget, value);

			// draw the feature on the left side of the annotation
		} else if (position.equals(FeaturePosition.RIGHT)) {

			drawRight(bounds, lineHeight, gc, widget, value);
			// draw the feature on the right side of the annotation
		} else if (position.equals(FeaturePosition.CENTER)) {

			writeCenter(bounds, lineHeight, gc, widget, value);
			// draw the feature on the right side of the annotation

			// draw the feature on the center of the annotation
		} else if (position.equals(FeaturePosition.SQUIGGLES)) {

			drawSquiggles(gc, widget, offset, length);

			// draw squiggles below the annotation
		} else if (position.equals(FeaturePosition.UNDERLINE)) {
			drawUnderlined(bounds, gc);

			// underline the anno
		} else if (position.equals(FeaturePosition.BOX)) {

			drawBox(bounds, gc);

			// draw a box around the annotation
		}

	}

	private void writeLeft(Rectangle bounds, int lineHeight, GC gc, StyledText widget, String value) {
		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int y = bounds.y + lineHeight;

		// draw the one feature right
		// Only draw if String not empty
		gc.drawString(value, bounds.width - 1, y - lineHeight - FONTSIZE - 7);

		font.dispose();
		// draw the feature on the right side of the annotation
	}

	private void drawRight(Rectangle bounds, int lineHeight, GC gc, StyledText widget, String value) {
		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		// draw the one feature right
		// Only draw if String not empty
		gc.drawString(value, x + bounds.width - 1, y - lineHeight - FONTSIZE - 7);

		font.dispose();
	}

	private void writeCenter(Rectangle bounds, int lineHeight, GC gc, StyledText widget, String value) {
		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		// draw the one feature right
		// Only draw if String not empty
		gc.drawString(value, (x + bounds.width) / 2 - 1, y - lineHeight - FONTSIZE - 7);

		font.dispose();
	}

	private void drawBox(Rectangle bounds, GC gc) {
		int correctedHeight;

		if (bounds.height > 0) {
			correctedHeight = bounds.height - 1;
		} else {
			correctedHeight = bounds.height;
		}

		int correctedWidth;

		if (bounds.width > 0)
			correctedWidth = bounds.width - 1;
		else
			correctedWidth = bounds.width;

		gc.drawRectangle(bounds.x, bounds.y, correctedWidth, correctedHeight);
	}

	private void drawSquiggles(GC gc, StyledText widget, int offset, int length) {
		Point left = widget.getLocationAtOffset(offset);
		Point right = widget.getLocationAtOffset(offset + length);
		Rectangle rect = widget.getTextBounds(offset, offset + length - 1);
		left.x = rect.x;
		right.x = rect.x + rect.width;

		int[] polyline = computePolyline(left, right, widget.getBaseline(offset), widget.getLineHeight(offset));

		gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
							// optimized performance
		gc.setLineStyle(SWT.LINE_SOLID);
		gc.drawPolyline(polyline);
	}

	private void drawUnderlined(Rectangle bounds, GC gc) {
		int y = bounds.y + bounds.height - 1;

		gc.drawLine(bounds.x, y, bounds.x + bounds.width, y);
	}

	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH = 4; // must be even
		final int HEIGHT = 2; // can be any number

		int peaks = (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks = 1;

		int leftX = left.x;

		// compute (number of point) * 2
		int length = ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates = new int[length];

		// cache peeks' y-coordinates
		int top = left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom = top + HEIGHT;

		// populate array with peek coordinates
		for (int i = 0; i < peaks; i++) {
			int index = 4 * i;
			coordinates[index] = leftX + (WIDTH * i);
			coordinates[index + 1] = bottom;
			coordinates[index + 2] = coordinates[index] + WIDTH / 2;
			coordinates[index + 3] = top;
		}

		// the last down flank is missing
		coordinates[length - 2] = Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length - 1] = bottom;

		return coordinates;
	}

	public Color determineColor(AnnotationFS anno, StyledText widgetGold) {

		if (widgetGold.getData("NAME").equals("Gold")) {

			AnnotationFS annotationFS = this.goldCompMapping.get(anno);

			if (annotationFS == null) {

				// return red
				return new Color(widgetGold.getDisplay(), new RGB(232, 93, 46));
			}

			else {

				if (onlyDiffrences) {
					return null;
				}
				// return green
				return new Color(widgetGold.getDisplay(), new RGB(84, 232, 46));
			}

		}

		else {

			AnnotationFS comp = combGoldMapping.get(anno);

			if (comp == null) {

				// blue
				return new Color(widgetGold.getDisplay(), new RGB(0, 100, 255));

			}

			else {

				if (onlyDiffrences) {
					return null;
				}
				// green
				return new Color(widgetGold.getDisplay(), new RGB(84, 232, 46));

			}

		}
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		// TODO Auto-generated method stub
		return 0;
	}
}
