package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class EditorWarningDrawingStrategy implements IAnnotationDrawingStrategy, IBorderDrawingStrategy {

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// color for warning
		Color orange = new Color(Display.getCurrent(), new RGB(247, 142, 5));
		Color lightOrange = new Color(Display.getCurrent(), new RGB(250, 186, 102));

		// color for error
		Color red = new Color(Display.getCurrent(), new RGB(0, 0, 255));
		Color lightRed = new Color(Display.getCurrent(), new RGB(144, 195, 212));

		// draw a rectangle for an annotation

		Rectangle textBounds = widget.getTextBounds(0, 1);
		int lineTop = Math.abs(widget.getTextBounds(anno.getBegin(), anno.getBegin() + 1).y - textBounds.y);
		int lineBottom = Math.abs(widget.getTextBounds(anno.getEnd() - 1, anno.getEnd()).y - textBounds.y);

		Rectangle textBounds2 = widget.getTextBounds(widget.getText().length() - 2, widget.getText().length() - 1);
		int lineCnt = Math.abs(textBounds.y) + Math.abs(textBounds2.y);

		int topPixel = lineToPixel(widget.getBounds().height, lineTop, lineCnt);
		int bottomPixel = lineToPixel(widget.getBounds().height, lineBottom, lineCnt);

		gc.setForeground(orange);
		gc.setBackground(lightOrange);

		if (bottomPixel == topPixel) {
			// increase the rectangle artificially
			bottomPixel += 3;
			topPixel -= 3;
		}
		gc.drawRectangle(widget.getBounds().width - widget.getRightMargin() - 2, topPixel + 2, 6, bottomPixel
				- topPixel - 2);

		gc.fillRectangle(new Rectangle(widget.getBounds().width - (widget.getRightMargin() + 1), topPixel + 1 + 2,
				7 - 2, bottomPixel - topPixel - 1 - 2));

		gc.dispose();
		orange.dispose();
		lightOrange.dispose();
		red.dispose();
		lightRed.dispose();

	}

	public static int lineToPixel(int height, int line, int lineCnt) {

		double frac = (line + 0.0) / (lineCnt * 1.0);

		double pixel = height * frac;

		return (int) pixel;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}
}
