package de.uniwue.kalimachos.coref.paintingStrategies;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class FeatureComparisionDrawingStrategy implements IAnnotationDrawingStrategy {
	private static final int FONTSIZE = 8;
	private Feature goldFeature;

	private Feature compFeature;

	// important this drawing style does only work if the set of annotations
	// doesnt change !

	// a mapping between those annotationlists

	// mapps from gold to comp
	HashMap<AnnotationFS, AnnotationFS> annomapping;

	HashMap<String, AnnotationFS> featureFirstMap;

	public FeatureComparisionDrawingStrategy(Type goldType, Type comptype, Feature goldFeature, Feature compFeature,
			AnnotationIndex<AnnotationFS> goldAnnos, AnnotationIndex<AnnotationFS> compAnnos) {
		super();
		this.goldFeature = goldFeature;
		this.compFeature = compFeature;

		this.annomapping = new HashMap<AnnotationFS, AnnotationFS>();
		this.featureFirstMap = new HashMap<String, AnnotationFS>();

		initMapping(compAnnos, goldAnnos, goldFeature);
	}

	private void initMapping(AnnotationIndex<AnnotationFS> compAnnos2, AnnotationIndex<AnnotationFS> goldAnnos2,
			Feature goldFeature2) {

		for (AnnotationFS gold : goldAnnos2) {

			boolean inserted = false;
			for (AnnotationFS comp : compAnnos2) {

				if (gold.getBegin() == comp.getBegin() && gold.getEnd() == comp.getEnd()) {
					inserted = true;
					this.annomapping.put(gold, comp);
					break;
				}
			}

			if (!inserted) {
				this.annomapping.put(gold, null);
			}

			// also fill the featurefirstmap

			if (!featureFirstMap.containsKey(gold.getFeatureValueAsString(goldFeature2))) {
				featureFirstMap.put(gold.getFeatureValueAsString(goldFeature2), gold);
			}
		}

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// anno has to be gold Anno !

		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;
		widget.setLineSpacing(amountOfSpace + 10);

		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		Color goldColor = new Color(gc.getDevice(), new RGB(130, 138, 50));
		Color goldColorLight = new Color(gc.getDevice(), new RGB(206, 222, 36));
		Color greenColor = new Color(gc.getDevice(), new RGB(84, 232, 46));
		Color yellow = new Color(gc.getDevice(), new RGB(232, 229, 46));
		Color red = new Color(gc.getDevice(), new RGB(232, 93, 46));
		Color white = new Color(gc.getDevice(), new RGB(255, 255, 255));
		Color magenta = new Color(gc.getDevice(), new RGB(242, 22, 217));
		Color black = new Color(gc.getDevice(), new RGB(0, 0, 0));

		// draw the goldString
		String goldString = anno.getFeatureValueAsString(goldFeature);

		gc.setForeground(black);

		// determine where you can draw the strings

		boolean hasLeftN = hasImmediateLeftNeighbour(anno);
		boolean hasRightN = hasImmediateRightNeighbour(anno);

		int drawXG = x - 1;
		int drawYG = y - lineHeight;

		int drawXC = x + bounds.width - 1;

		if (!hasLeftN && !hasRightN) {

			// no direct neighbours => no changes
		}

		else if (hasLeftN && !hasRightN) {

			// if only right neighbour => shift everything to the left
			// drawXG = drawXG + gc.stringExtent(goldString).x;

			// drawXC = drawXC + gc.stringExtent(goldString).x;
		}

		else if (!hasLeftN && hasRightN) {

			// no left neighbour but right neighb

			// drawXG = drawXG - gc.stringExtent(goldString).x;

			// drawXC = drawXC - gc.stringExtent(goldString).x;

		}

		if (featureFirstMap.containsKey(goldString)) {

			// determine if first appearance

			if (featureFirstMap.get(goldString).getBegin() == anno.getBegin()) {
				gc.setBackground(goldColor);
				gc.drawString(anno.getFeatureValueAsString(goldFeature), drawXG, drawYG - FONTSIZE - 7);
			}

			else {
				gc.setBackground(goldColorLight);

				// draw in a lighter color
				gc.drawString(anno.getFeatureValueAsString(goldFeature), drawXG, drawYG - FONTSIZE - 7);
			}
		}

		gc.setBackground(white);

		// also draw an rectangle

		AnnotationFS comp = annomapping.get(anno);

		if (comp != null) {

			String compString = comp.getFeatureValueAsString(compFeature);

			if (goldString.equals(compString)) {

				// draw green
				// gc.setForeground(greenColor);
				gc.setBackground(greenColor);
				gc.drawString(comp.getFeatureValueAsString(compFeature), drawXC, y - lineHeight - FONTSIZE - 7);

			}

			else if (!goldString.equals(compString)) {

				// gc.setForeground(red);

				// determine whether the id is just a diffrent id that was
				// chosen
				// by the user

				Point result = determineDiffrence(goldString, compString);

				if (result.x == 1) {

					// draw red
					gc.setBackground(red);
				}

				else if (result.y == 0) {
					// draw green
					gc.setBackground(greenColor);
				} else if (result.x - result.y == 1) {
					// draw yellow ->kettenfehler
					gc.setBackground(yellow);
				}

				else {

					// red
					gc.setBackground(red);

				}

				gc.drawString(comp.getFeatureValueAsString(compFeature), drawXC, y - lineHeight - FONTSIZE - 7);
			}
		}

		// Dispose fonts
		// initialFont.dispose();
		goldColor.dispose();
		greenColor.dispose();
		yellow.dispose();
		red.dispose();
		font.dispose();
		gc.dispose();
		magenta.dispose();
		white.dispose();
		black.dispose();
		goldColorLight.dispose();

	}

	private boolean hasImmediateRightNeighbour(AnnotationFS anno) {

		int minDiffrence = 1000;

		for (AnnotationFS a : annomapping.keySet()) {
			int diff = a.getBegin() - anno.getEnd();

			if (diff != 0 && diff > 0) {

				if (diff < minDiffrence) {
					minDiffrence = diff;
				}

			}
		}

		return minDiffrence <= 3;
	}

	private boolean hasImmediateLeftNeighbour(AnnotationFS anno) {

		int minDiffrence = 1000;

		for (AnnotationFS a : annomapping.keySet()) {
			int diff = anno.getBegin() - a.getEnd();

			if (diff != 0 && diff < 0) {

				if (-1 * diff < minDiffrence) {
					minDiffrence = -1 * diff;
				}

			}
		}

		return minDiffrence <= 3;
	}

	private Point determineDiffrence(String goldString, String compString) {

		// x of the point shows the max appearances of that entity
		// y shows the amount of differences

		// get all gold anno with id = golstring

		int total = 0;
		int diff = 0;
		for (AnnotationFS gold : annomapping.keySet()) {

			String id = gold.getFeatureValueAsString(goldFeature);

			if (id.equals(goldString)) {

				// get the according compAnno

				AnnotationFS comp = annomapping.get(gold);

				if (comp != null) {
					total++;

					String idComp = comp.getFeatureValueAsString(compFeature);

					if (!idComp.equals(compString)) {
						diff++;
					}
				}
			}

		}
		return new Point(total, diff);
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
