package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class FeatureJoniDisplayDrawingStrategy
		implements IAnnotationDrawingStrategy, INeedBothAnnotationDrawingStrategy {

	String feature;
	private static final int FONTSIZE = 8;
	private static final int BRACKET_WIDTH = 5;

	StyleRange styleRange;
	private boolean useBrighterColor = false;
	private Color brightercolor;
	long timeOfLastReload;

	// draws the features above the Annotations
	public FeatureJoniDisplayDrawingStrategy(StyleRange range, String feature) {
		this.styleRange = range;
		this.feature = feature;

		brightercolor = new Color(Display.getCurrent(),
				new RGB(Math.min(styleRange.background.getRed() + 80, 255),
						Math.min(255, styleRange.background.getGreen() + 80),
						Math.min(255, styleRange.background.getBlue() + 80)));
	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		// get the bounds of the Annotation

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(10);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		// change the Color of the GC object
		StyleRange styleRangeForAnno = new StyleRange();
		Util_impl util = new Util_impl(anno.getCAS());
		if (util.getIndexOfTypeInDocument(anno) % 2 == 0) {
			styleRangeForAnno.background = this.styleRange.background;
			gc.setForeground(styleRangeForAnno.background);
		} else {
			styleRangeForAnno.background = brightercolor;
			gc.setForeground(brightercolor);
		}

		// determine the bounds of the annotation
		int topLeftX = Integer.MAX_VALUE;
		int topLeftY = Integer.MAX_VALUE;
		int width = 5;
		int height = 5;

		if (anno.getEnd() - anno.getBegin() == 0)
			return;
		Rectangle textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		topLeftX = textBounds.x;
		topLeftY = textBounds.y;
		width = textBounds.width;
		height = textBounds.height;

		for (int i = 0; i < 3; i++) {
			gc.drawRectangle(new Rectangle(topLeftX - i, topLeftY + i, width + i * 2, height));
		}
		drawFeatureString(anno, gc, lineHeight, topLeftX, topLeftY - widget.getLineHeight(), widget);

		gc.setFont(widget.getFont());

		gc.dispose();
		font.dispose();
		// brightercolor.dispose();
		useBrighterColor = !useBrighterColor;

	}

	private void drawFeatureString(AnnotationFS anno, GC gc, int lineHeight, int x, int y, StyledText widget) {
		Feature f = anno.getType().getFeatureByBaseName(feature);
		if (f != null) {
			String feat = anno.getFeatureValueAsString(f);
			// gc.drawRectangle(bounds);
			if (feat != null && feat.trim().length() > 0) {
				// Only draw if String not empty
				gc.drawString(anno.getFeatureValueAsString(f), x, y + FONTSIZE);
				y -= lineHeight + 1;
			}
		}
	}

	@Override
	public void manipulateWidget(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {
		// TODO Auto-generated method stub

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}

}
