package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class MultiFeatureDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	private static final int FONTSIZE = 8;

	// those are displayed at the top
	String[] features;

	StyleRange styleRange;

	// draws the features above the Annotations
	public MultiFeatureDrawingStrategy(StyleRange range, String feature, String... features) {
		this.styleRange = range;
		this.feature = feature;
		this.features = features;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		// draw the one feature right
		Feature f = anno.getType().getFeatureByBaseName(feature);
		if (f != null) {
			String feat = anno.getFeatureValueAsString(f);
			if (feat != null && feat.trim().length() > 0) {
				// Only draw if String not empty
				gc.drawString(anno.getFeatureValueAsString(f), x + bounds.width - 1, y - lineHeight - FONTSIZE - 7);
				gc.drawRectangle(bounds);
				y -= lineHeight + 1;
			}
		}

		// all remaining features left separated by |

		StringBuilder sb = new StringBuilder();
		for (String feats : this.features) {
			Feature fe = anno.getType().getFeatureByBaseName(feats);

			if (fe != null) {
				// build the string
				String value = anno.getFeatureValueAsString(fe);
				if (value != null && value.length() > 0) {
					if (value.equals("false")) {
						continue;
					}
					sb.append(value.charAt(0)).append("|");

				}
			}
		}

		// draw the String
		if (sb.length() > 0) {
			gc.drawString(sb.toString(), x, y - lineHeight / 2 - 2);
		}

		// TODO solution for an annotation which exceed 1 line

		gc.dispose();
		font.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
