package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class OvalDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;

	public OvalDrawingStrategy(StyleRange styleRange) {

		this.styleRange = styleRange;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int length = anno.getEnd() - anno.getBegin();
		int offset = anno.getBegin();
		if (length != 0) {
			if (gc != null) {
				Rectangle bounds = widget.getTextBounds(offset, offset + length - 1);

				gc.setForeground(styleRange.background);

				int correctedHeight;

				if (bounds.height > 0) {
					correctedHeight = bounds.height - 1;
				} else {
					correctedHeight = bounds.height;
				}

				int correctedWidth;

				if (bounds.width > 0)
					correctedWidth = bounds.width - 1;
				else
					correctedWidth = bounds.width;

				gc.drawRoundRectangle(bounds.x - 3, bounds.y - 3, correctedWidth + 8, correctedHeight + 8, 20, 20);
			} else {
				widget.redrawRange(offset, length, true);
			}
		}
		gc.dispose();
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
