/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

/**
 */
public class TextColDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;

	public TextColDrawingStrategy(StyleRange styleRange) {

		this.styleRange = styleRange;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText textWidget, GC gc) {

		int length = anno.getEnd() - anno.getBegin();
		int offset = anno.getBegin();

		if (length > 0) {
			if (gc != null) {

				int start = offset;
				int end = offset + length - 1;

				Rectangle bounds = textWidget.getTextBounds(start, end);

				gc.setForeground(styleRange.background);

				gc.drawText(textWidget.getText(start, end), bounds.x, bounds.y, true);

			} else {
				textWidget.redrawRange(offset, length, true);
			}
		}

		gc.dispose();
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}