package de.uniwue.kalimachos.coref.paintingStrategies;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;

public class UnderlineDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;

	public UnderlineDrawingStrategy(StyleRange styleRange) {

		this.styleRange = styleRange;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int length = anno.getEnd() - anno.getBegin();
		int offset = anno.getBegin();

		if (gc != null) {
			Rectangle bounds;

			if (length > 0) {
				bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
			} else {
				Point location = widget.getLocationAtOffset(offset);
				bounds = new Rectangle(location.x, location.y, 1, widget.getLineHeight());
			}

			int y = bounds.y + bounds.height - 1;

			gc.setForeground(styleRange.background);

			gc.drawLine(bounds.x, y, bounds.x + bounds.width, y);
		} else {
			widget.redrawRange(offset, length, true);
		}

		gc.dispose();
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}

}
