package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.util.EditorPaintingUtil;

public class AttributeDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;
	String priority;

	Image imgTime = null;
	Image imgAtt = null;
	Image imgUncertain = null;

	private static final int FONTSIZE = 8;

	StyleRange styleRange;
	private String tempFeature;
	private String featUncertain;

	// draws the features above the Annotations
	public AttributeDrawingStrategy(StyleRange range, String id, String priority, String tempFeature,
			String featUncertain) {
		this.styleRange = range;
		this.feature = id;
		this.priority = priority;
		this.tempFeature = tempFeature;
		this.featUncertain = featUncertain;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// init 3 Colors for 3 priorities

		Color orange = new Color(widget.getDisplay(), new RGB(250, 156, 32));
		Color green = new Color(widget.getDisplay(), new RGB(43, 250, 32));
		Color red = new Color(widget.getDisplay(), new RGB(250, 10, 32));

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		int linebreakIndex = 0;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the Strings that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					linebreakIndex = i - 1;
					break;
				}
			}
		}

		// System.out.println(bounds.height);

		// change the Color of the GC object

		// get priority
		Color foregroundColor = null;
		String priorityS = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(priority));

		// standard is medium
		if (priorityS == null) {
			priorityS = "medium";
		}
		if (priorityS.equals("medium")) {
			// orange
			foregroundColor = orange;
		}

		else if (priorityS.equals("high")) {
			// red
			foregroundColor = red;
		}

		else if (priorityS.equals("low")) {
			// green
			foregroundColor = green;
		}
		gc.setForeground(foregroundColor);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (!multipleLines) {

			gc.setForeground(styleRange.background);

			// also set more images if necessary

			setTemporaryImages(anno, gc, x, y);

			// uncertainty

			Feature uncertain = anno.getType().getFeatureByBaseName(this.featUncertain);

			String value = anno.getFeatureValueAsString(uncertain);

			if (value != null && value.startsWith("tr")) {

				// squiggles
				Point left = widget.getLocationAtOffset(offset);
				Point right = widget.getLocationAtOffset(offset + length);
				Rectangle rect = widget.getTextBounds(offset, offset + length - 1);
				left.x = rect.x;
				right.x = rect.x + rect.width;

				int[] polyline = computePolyline(left, right, widget.getBaseline(offset), widget.getLineHeight(offset));

				gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
									// optimized performance
				gc.setLineStyle(SWT.LINE_SOLID);
				gc.setForeground(styleRange.background);
				gc.drawPolyline(polyline);

			}
		}

		else {
			// multiple lines start here - we use 2 bounds

			Rectangle textBoundsFirstLine = widget.getTextBounds(anno.getBegin(), linebreakIndex);
			Rectangle textBoundsSecondLine = widget.getTextBounds(linebreakIndex + 1, anno.getEnd());

			Feature f = anno.getType().getFeatureByBaseName(feature);
			if (f != null) {
				String feat = anno.getFeatureValueAsString(f);
				if (feat != null && feat.trim().length() > 0) {
					// Only draw if String not empty
					gc.drawString(anno.getFeatureValueAsString(f), textBoundsSecondLine.x + textBoundsSecondLine.width
							- 1, textBoundsSecondLine.y - FONTSIZE - 7);
					boolean alreadyHandled = false;
					for (StyleRange r : widget.getStyleRanges(offset, length)) {
						// ugly solution
						if (r.background.getRGB().green == 245) {
							alreadyHandled = true;
							break;
						}
					}
					if (!alreadyHandled) {
						gc.drawRectangle(textBoundsFirstLine);
						gc.drawRectangle(textBoundsSecondLine);
					}
					y -= lineHeight + 1;
				}
			}

			gc.setForeground(styleRange.background);

		}
		// initialFont.dispose();
		font.dispose();
		gc.dispose();
		orange.dispose();
		green.dispose();
		red.dispose();

	}

	private void setTemporaryImages(AnnotationFS anno, GC gc, int x, int y) {
		Feature temp = anno.getType().getFeatureByBaseName(this.tempFeature);

		String tempValue = anno.getFeatureValueAsString(temp);
		if (tempValue != null) {
			if (tempValue.startsWith("temp")) {

				if (this.imgTime == null) {

					this.imgTime = EditorPaintingUtil.createImage(EditorPaintingUtil.WSD_CAT_ZEIT);
				}
				gc.drawImage(imgTime, x, y - FONTSIZE - 9);
			}

			else if (tempValue.startsWith("Fix")) {

				if (this.imgAtt == null) {
					this.imgAtt = EditorPaintingUtil.createImage(EditorPaintingUtil.WSD_CAT_ATTRIBUT);
				}
				gc.drawImage(imgAtt, x, y - FONTSIZE - 9);

			}
		}
	}

	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH = 4; // must be even
		final int HEIGHT = 2; // can be any number

		int peaks = (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks = 1;

		int leftX = left.x;

		// compute (number of point) * 2
		int length = ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates = new int[length];

		// cache peeks' y-coordinates
		int top = left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom = top + HEIGHT;

		// populate array with peek coordinates
		for (int i = 0; i < peaks; i++) {
			int index = 4 * i;
			coordinates[index] = leftX + (WIDTH * i);
			coordinates[index + 1] = bottom;
			coordinates[index + 2] = coordinates[index] + WIDTH / 2;
			coordinates[index + 3] = top;
		}

		// the last down flank is missing
		coordinates[length - 2] = Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length - 1] = bottom;

		return coordinates;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
