package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IBorderDrawingStrategy;

public class BorderSceneDrawingStrategy implements IAnnotationDrawingStrategy, IBorderDrawingStrategy {

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// Szene
		Color blue = new Color(Display.getCurrent(), new RGB(0, 0, 255));
		Color lightBlue = new Color(Display.getCurrent(), new RGB(144, 195, 212));

		// keine Szene
		Color orange = new Color(Display.getCurrent(), new RGB(245, 73, 5));
		Color lightOrange = new Color(Display.getCurrent(), new RGB(255, 178, 54));

		// Szenenuebergang
		Color green = new Color(Display.getCurrent(), new RGB(22, 148, 18));
		Color lightGreen = new Color(Display.getCurrent(), new RGB(59, 232, 53));

		Color grey = new Color(Display.getCurrent(), new RGB(234, 234, 234));
		// gc.setBackground(grey);
		// gc.fillRectangle(new Rectangle(widget.getBounds().x,
		// widget.getBounds().y, widget.getLeftMargin() - 10, widget
		// .getBounds().height));

		// draw a rectangle for an annotation

		Feature featCovered = anno.getType().getFeatureByBaseName("Covered");
		String covered = anno.getFeatureValueAsString(featCovered);

		Rectangle textBounds = widget.getTextBounds(0, 1);
		int lineTop = Math.abs(widget.getTextBounds(anno.getBegin(), anno.getBegin() + 1).y - textBounds.y);
		int lineBottom = Math.abs(widget.getTextBounds(anno.getEnd() - 1, anno.getEnd()).y - textBounds.y);

		Rectangle textBounds2 = widget.getTextBounds(widget.getText().length() - 2, widget.getText().length() - 1);
		int lineCnt = Math.abs(textBounds.y) + Math.abs(textBounds2.y);

		int topPixel = lineToPixel(widget.getBounds().height, lineTop, lineCnt);
		int bottomPixel = lineToPixel(widget.getBounds().height, lineBottom, lineCnt);

		gc = updateGCColor(green, lightGreen, orange, lightOrange, blue, lightBlue, anno, gc);
		if (covered == null || !covered.equals("true")) {

			gc.drawRectangle(3, topPixel, 10, bottomPixel - topPixel);

			gc.fillRectangle(new Rectangle(4, topPixel + 1, 9, bottomPixel - topPixel - 1));
		}

		else {

			gc.drawRectangle(5, topPixel + 2, 6, bottomPixel - topPixel - 2);

			gc.fillRectangle(new Rectangle(4 + 2, topPixel + 1 + 2, 7 - 2, bottomPixel - topPixel - 1 - 2));

		}
		gc.dispose();
		blue.dispose();
		grey.dispose();
		orange.dispose();
		lightOrange.dispose();
		lightBlue.dispose();
		green.dispose();
		lightGreen.dispose();

	}

	private GC updateGCColor(Color green, Color lightGreen, Color orange, Color lightOrange, Color blue,
			Color lightBlue, AnnotationFS anno, GC gc) {
		String type = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName("Type"));

		if (type.equals("Szene")) {

			gc.setForeground(blue);
			gc.setBackground(lightBlue);

		}

		else if (type.equals("Szenen�bergang")) {
			gc.setForeground(green);
			gc.setBackground(lightGreen);
		}

		else {
			gc.setForeground(orange);
			gc.setBackground(lightOrange);

		}
		return gc;
	}

	public static int lineToPixel(int height, int line, int lineCnt) {

		double frac = (line + 0.0) / (lineCnt * 1.0);

		double pixel = height * frac;

		return (int) pixel;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
