package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class ChunkDrawingStrategy implements IAnnotationDrawingStrategy {
	String feature;

	private static final int FONTSIZE = 8;

	StyleRange styleRange;

	// draws the features above the Annotations
	public ChunkDrawingStrategy(StyleRange range, String feature) {
		this.styleRange = range;
		this.feature = feature;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);

		boolean multipleLines = false;
		int linebreakIndex = 0;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the Strings that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					linebreakIndex = i - 1;
					break;
				}
			}
		}

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (!multipleLines) {
			Feature f = anno.getType().getFeatureByBaseName(feature);
			if (f != null) {
				String feat = anno.getFeatureValueAsString(f);
				// Set the color accordingly
				String value = anno.getFeatureValueAsString(f);
				setColor(value, gc);
				if (feat != null && feat.trim().length() > 0) {
					// Only draw if String not empty

					gc.drawString(value, (x + 2), y - lineHeight - FONTSIZE - 7);
					gc.drawRectangle(bounds);
					y -= lineHeight + 1;
				}
			}

			// TODO solution for an annotation which exceed 1 line
		}

		else {
			// multiple lines start here - we use 2 bounds

			Rectangle textBoundsFirstLine = widget.getTextBounds(anno.getBegin(), linebreakIndex);
			Rectangle textBoundsSecondLine = widget.getTextBounds(linebreakIndex + 1, anno.getEnd());

			Feature f = anno.getType().getFeatureByBaseName(feature);
			if (f != null) {
				String feat = anno.getFeatureValueAsString(f);
				if (feat != null && feat.trim().length() > 0) {
					String value = anno.getFeatureValueAsString(f);
					// Only draw if String not empty
					setColor(value, gc);
					gc.drawString(value, textBoundsSecondLine.x + 1, textBoundsSecondLine.y - FONTSIZE - 7);

					gc.drawRectangle(textBoundsFirstLine);
					gc.drawRectangle(textBoundsSecondLine);

					y -= lineHeight + 1;
				}
			}

			// gc.setForeground(styleRange.background);

			// draw it
			gc.drawRectangle(textBoundsFirstLine);
			gc.drawRectangle(textBoundsSecondLine);

		}

		gc.dispose();
		font.dispose();

	}

	private void setColor(String value, GC gc) {
		Color c = null;
		if (value.contains("VC")) {
			c = new Color(gc.getDevice(), 255, 0, 0);
		}

		else if (value.contains("NC")) {
			c = new Color(gc.getDevice(), 68, 173, 94);
		}

		else {
			c = new Color(gc.getDevice(), 0, 0, 255);
		}
		gc.setForeground(c);
		c.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
