package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.util.EditorPaintingUtil;

public class CorefCorrectionDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	private static final int FONTSIZE = 8;

	public static final String EXPLAIN_COREF_GENDER = "DebugGender Feature";
	public static final String EXPLAIN_COREF_NUMERUS = "DebugNumerus Feature";
	public static final String EXPLAIN_COREF_PERSON = "Debug PersonFeature";
	public static final String EXPLAIN_COREF_ISRESOLVEDCORRECTLY = "Enter IsResolvedCorrectly";
	public static final String EXPLAIN_COREF_ID_FEATURE = "Enter Debug-ID";

	// drawn in the middle

	private String sCorrectly = EditorPaintingUtil.getFromPreferences(EXPLAIN_COREF_ISRESOLVEDCORRECTLY, "null");

	// drawn right;
	private String sId = EditorPaintingUtil.getFromPreferences(EXPLAIN_COREF_ID_FEATURE, "null");;

	StyleRange styleRange;

	HashMap<String, Image> imageMapping;

	// draws the features above the Annotations
	public CorefCorrectionDrawingStrategy(StyleRange range) {
		this.styleRange = range;
		this.imageMapping = new HashMap<String, Image>();

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 15);

		// get the bounds of the Annotation
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		// System.out.println(bounds.height);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		// draw the one feature right
		Feature f = anno.getType().getFeatureByBaseName(sId);
		Feature correct = anno.getType().getFeatureByBaseName(sCorrectly);
		if (f != null) {
			String feat = anno.getFeatureValueAsString(f);
			Image correctImage = getCorrectFeature(anno, correct);
			if (feat != null && feat.trim().length() > 0) {
				// Only draw if String not empty
				gc.drawString(anno.getFeatureValueAsString(f), x - 1, y - lineHeight - FONTSIZE - 7);
				gc.drawImage(correctImage, x + bounds.width / 2 + bounds.width / 4, y - lineHeight - FONTSIZE - 7);
				// y -= lineHeight + 1;
			}
		}

		// get all other Features

		// initialFont.dispose();
		font.dispose();
		gc.dispose();

	}

	private Image getCorrectFeature(AnnotationFS anno, Feature correct) {

		String featureValueAsString = anno.getFeatureValueAsString(correct);

		if (featureValueAsString == null) {
			featureValueAsString = "true";
		}

		// true gets a green haken

		// red gets a cross

		if (imageMapping.containsKey(featureValueAsString)) {
			return imageMapping.get(featureValueAsString);
		}

		else {

			if (featureValueAsString.equals("true")) {
				imageMapping.put(featureValueAsString, EditorPaintingUtil.createImage(EditorPaintingUtil.CORRECT));
			}

			else {
				imageMapping.put(featureValueAsString, EditorPaintingUtil.createImage(EditorPaintingUtil.ERROR));
			}
		}

		return imageMapping.get(featureValueAsString);
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
