package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class DSSpeakerIDDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	private static final int FONTSIZE = 8;

	StyleRange styleRange;

	private String speaker;

	// draws the features above the Annotations
	public DSSpeakerIDDrawingStrategy(StyleRange range, String id, String speaker) {
		this.styleRange = range;
		this.feature = id;

		this.speaker = speaker;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		// widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					break;
				}
			}
		}

		// System.out.println(bounds.height);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (!multipleLines) {
			// draw the one feature right
			Feature speakerFeat = anno.getType().getFeatureByBaseName(speaker);

			if (speakerFeat != null) {

				AnnotationFS speaker = (AnnotationFS) anno.getFeatureValue(speakerFeat);
				if (speaker != null) {
					Feature f = speaker.getType().getFeatureByBaseName(feature);

					// Only draw if String not empty
					gc.drawString(speaker.getFeatureValueAsString(f), x + bounds.width - 1, y - lineHeight - 7);
					y -= lineHeight + 1;
				}

			}
		}

		else {
			// multiple lines start here - we use 2 bounds
			Rectangle textBoundsSecondLine = null;
			try {
				textBoundsSecondLine = widget.getTextBounds(anno.getEnd() - 1, anno.getEnd());
			} catch (IllegalArgumentException e) {
				return;
			}
			Feature speakerFeat = anno.getType().getFeatureByBaseName(speaker);

			if (speakerFeat != null) {

				AnnotationFS speaker = (AnnotationFS) anno.getFeatureValue(speakerFeat);
				if (speaker != null) {
					Feature f = speaker.getType().getFeatureByBaseName(feature);

					// Only draw if String not empty
					gc.drawString(speaker.getFeatureValueAsString(f), textBoundsSecondLine.x
							+ textBoundsSecondLine.width - 1, textBoundsSecondLine.y - 7);
					y -= lineHeight + 1;
				}

			}

		}
		// initialFont.dispose();
		font.dispose();
		gc.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
