package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class DirectSpeechDrawingStrategy implements IAnnotationDrawingStrategy {

	String feature;
	private static final int FONTSIZE = 8;

	private static final int BRACKET_WIDTH = 3;
	StyleRange styleRange;

	// draws the features above the Annotations
	public DirectSpeechDrawingStrategy(StyleRange range, String feature) {
		this.styleRange = range;
		this.feature = feature;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {
		int length = anno.getEnd() - anno.getBegin();
		int offset = anno.getBegin();
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		Feature f = anno.getType().getFeatureByBaseName(feature);
		if (f != null) {
			String feat = anno.getFeatureValueAsString(f);
			boolean isDrawOpenBracket = anno.getBegin() == offset;
			if (isDrawOpenBracket) {
				gc.drawLine(bounds.x, bounds.y + bounds.height - 1, bounds.x + BRACKET_WIDTH, bounds.y + bounds.height
						- 1);

				gc.drawLine(bounds.x, bounds.y, bounds.x, bounds.y + bounds.height - 1);

				gc.drawLine(bounds.x, bounds.y, bounds.x + BRACKET_WIDTH, bounds.y);
			}

			boolean isDrawCloseBracket = anno.getEnd() == offset + length;
			if (isDrawCloseBracket) {
				gc.drawLine(bounds.x + bounds.width, bounds.y + bounds.height - 1, bounds.x + bounds.width
						- BRACKET_WIDTH, bounds.y + bounds.height - 1);

				gc.drawLine(bounds.x + bounds.width - 1, bounds.y, bounds.x + bounds.width - 1, bounds.y
						+ bounds.height - 1);

				gc.drawLine(bounds.x + bounds.width, bounds.y, bounds.x + bounds.width - BRACKET_WIDTH, bounds.y);
			}
			// gc.drawRectangle(bounds);
			if (feat != null && feat.trim().length() > 0) {
				// Only draw if String not empty
				gc.drawString(anno.getFeatureValueAsString(f), x + 5, y - lineHeight - FONTSIZE - 7);
				y -= lineHeight + 1;
			}
		}

		// TODO solution for an annotation which exceed 1 line

		// StyleRange styleRange =this.styleRange;
		// styleRange.start = anno.getBegin();
		// styleRange.length = anno.getEnd() - anno.getBegin();
		// widget.setStyleRange(styleRange);
		gc.dispose();
		font.dispose();

	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
