package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class GrammarRealDrawingStrategy implements IAnnotationDrawingStrategy {

	private static final int LAYER_HEIGHT = 30;
	private static final int BRACKET_HEIGHT = 10;

	private Feature featPhraseType;

	private Feature featSemanticRole;

	private Feature featSyntacticRole;

	private Feature featLayer;
	private Feature featIsSelected;

	// draws the features above the Annotations
	public GrammarRealDrawingStrategy() {
	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		this.featPhraseType = anno.getType().getFeatureByBaseName("Phrasetype");
		this.featSemanticRole = anno.getType().getFeatureByBaseName("SemanticRole");
		this.featSyntacticRole = anno.getType().getFeatureByBaseName("SyntacticRole");
		anno.getType().getFeatureByBaseName("SyntacticHead");
		this.featLayer = anno.getType().getFeatureByBaseName("Layer");
		this.featIsSelected = anno.getType().getFeatureByBaseName("IsSelected");
		// init 3 Colors for 3 priorities

		Color orange = new Color(widget.getDisplay(), new RGB(250, 156, 32));
		Color green = new Color(widget.getDisplay(), new RGB(18, 138, 34));
		Color red = new Color(widget.getDisplay(), new RGB(250, 10, 32));
		Color blue = new Color(widget.getDisplay(), new RGB(0, 0, 255));

		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}

		// set the font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		// do the drawing
		String layerS = anno.getFeatureValueAsString(featLayer);

		// do not draw if the layer is null
		if (layerS == null)
			layerS = "1";

		int layer = Integer.parseInt(layerS);

		Rectangle textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd());

		// paint the bracket
		String drawingText = determinDrawingText(anno);

		setColorOfGc(gc, anno, blue, red, green);

		// up we go
		int y1 = textBounds.y - 1 - (layer - 1) * LAYER_HEIGHT;
		int y2 = y1 - BRACKET_HEIGHT;
		gc.drawLine(textBounds.x, y1, textBounds.x, y2);

		// to the right
		gc.drawLine(textBounds.x, y2, textBounds.x + textBounds.width - 5, y2);

		// back to the chopper
		gc.drawLine(textBounds.x + textBounds.width - 5, y2, textBounds.x + textBounds.width - 5, y1);

		// draw the text

		Point stringExtent = gc.stringExtent(drawingText);
		gc.drawString(drawingText, textBounds.x + textBounds.width / 2 - stringExtent.x / 2, y2 - 2 - stringExtent.y);
		// end drawing

		font.dispose();
		gc.dispose();
		orange.dispose();
		green.dispose();
		red.dispose();
		blue.dispose();

	}

	private void setColorOfGc(GC gc, AnnotationFS anno, Color blue, Color red, Color green) {

		String isSel = anno.getFeatureValueAsString(featIsSelected);

		if (isSel != null && isSel.equals("true")) {

			gc.setForeground(blue);
			return;

		}

		String phrasenType = anno.getFeatureValueAsString(featPhraseType);

		if (phrasenType != null) {

			if (phrasenType.equals("NP")) {
				gc.setForeground(green);
			}

			else if (phrasenType.equals("PP")) {
				gc.setForeground(red);
			}
		}

	}

	private String determinDrawingText(AnnotationFS anno) {

		String text = "";
		String phrasenType = anno.getFeatureValueAsString(featPhraseType);

		if (phrasenType != null) {
			text += phrasenType + "|";
		}

		String syntacticType = anno.getFeatureValueAsString(featSyntacticRole);

		if (syntacticType != null) {
			text += syntacticType + "|";
		}

		String semanticType = anno.getFeatureValueAsString(featSemanticRole);

		if (semanticType != null) {
			text += semanticType;
		}

		if (text.endsWith("|")) {
			text = text.substring(0, text.length() - 1);
		}

		if (text.equals(""))
			return "NOT_SET";
		return text;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
