package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class IEViewEntityDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;
	private static final int FONTSIZE = 8;

	public IEViewEntityDrawingStrategy(StyleRange range) {
		this.styleRange = range;
	}

	// draws a rectangle + the entity name if the annotation is not complex
	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		Util_impl util = new Util_impl(anno.getCAS());
		gc.setForeground(styleRange.background);
		// determine all charactersplitindexes where we need to start drawing in
		// a new line
		// get the bounds of the Annotation
		List<Integer> lineBreakIndexes = determineLineBreakIndexes(anno, widget);

		// change the font
		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		// this case means the annotation is not created by a complex indicator
		if (anno.getFeatureValueAsString(util.getIEEntityGoldConstructedByFeature()) == null) {

			// draw for every subsection
			if (lineBreakIndexes.size() != 0) {
				List<Rectangle> textbounds = new ArrayList<Rectangle>();
				textbounds.add(widget.getTextBounds(anno.getBegin(), lineBreakIndexes.get(0)));
				for (int j = 1; j < lineBreakIndexes.size(); j++) {
					textbounds.add(widget.getTextBounds(lineBreakIndexes.get(j - 1) + 1, lineBreakIndexes.get(j)));
				}
				// add the last box
				textbounds.add(
						widget.getTextBounds(lineBreakIndexes.get(lineBreakIndexes.size() - 1) + 1, anno.getEnd() - 1));

				// draw
				drawMultilineBox(textbounds, gc, anno, util);
			}

			else {
				// single line!
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
				gc.drawRectangle(textBounds);
				String label = anno.getFeatureValueAsString(util.getIEEntityGoldCategoryFeature());
				Point stringExtent = gc.stringExtent(label);
				label = shortenStringToAdjustBox(label,gc,textBounds.width);
				if (label != null) {
					gc.drawString(label, textBounds.x, textBounds.y - stringExtent.y);
				}
			}

		}

		// annotation was created by a complex case, we draw a bracket below the
		// annotation
		else {
			// draw for every subsection
			if (lineBreakIndexes.size() != 0) {
				List<Rectangle> textbounds = new ArrayList<Rectangle>();
				textbounds.add(widget.getTextBounds(anno.getBegin(), lineBreakIndexes.get(0)));
				for (int j = 1; j < lineBreakIndexes.size(); j++) {
					textbounds.add(widget.getTextBounds(lineBreakIndexes.get(j - 1) + 1, lineBreakIndexes.get(j)));
				}
				// add the last box
				textbounds.add(
						widget.getTextBounds(lineBreakIndexes.get(lineBreakIndexes.size() - 1) + 1, anno.getEnd() - 1));

				// draw multiline
				for (int i = 0; i < textbounds.size(); i++) {
					Rectangle actualRect = textbounds.get(i);
					if (i == 0) {
						// we need to leave the right Border
						String label = anno.getFeatureValueAsString(util.getIEEntityGoldCategoryFeature());
						Point stringExtent = gc.stringExtent(label);
						if (label != null) {
							gc.drawString(label, actualRect.x + actualRect.width / 2 - stringExtent.x / 2,
									actualRect.y + actualRect.height);
						}

						//left down
						gc.drawLine(actualRect.x, actualRect.y + actualRect.height + 3, actualRect.x,
								actualRect.y + actualRect.height + FONTSIZE + 5);


						// line parallel to the text
						gc.drawLine(actualRect.x, actualRect.y + actualRect.height + FONTSIZE + 5,
								actualRect.x + actualRect.width, actualRect.y + actualRect.height + FONTSIZE + 5);

					}

					else if (i > 0 && i < textbounds.size() - 1) {

						// line parallel to the text
						gc.drawLine(actualRect.x, actualRect.y + actualRect.height + FONTSIZE + 5,
								actualRect.x + actualRect.width, actualRect.y + actualRect.height + FONTSIZE + 5);

					} else if (i > 0 && i == textbounds.size() - 1) {

						// bracket right down
						gc.drawLine(actualRect.x + actualRect.width, actualRect.y + actualRect.height + 3,
								actualRect.x + actualRect.width, actualRect.y + actualRect.height + FONTSIZE + 5);

						// line parallel to the text
						gc.drawLine(actualRect.x, actualRect.y + actualRect.height + FONTSIZE + 5,
								actualRect.x + actualRect.width, actualRect.y + actualRect.height + FONTSIZE + 5);

					}
				}

			}

			else {
				drawComplexSingleLine(anno, widget, gc, util);
			}
		}

		gc.dispose();
		font.dispose();

	}

	private String shortenStringToAdjustBox(String label, GC gc, int width) {
		String newLabel = label;
		Point stringExtent = gc.stringExtent(newLabel);
		while(stringExtent.x>width){
			newLabel = newLabel.substring(0, newLabel.length()-1); 
			stringExtent = gc.stringExtent(newLabel);
		}
		return newLabel;
	}

	private void drawComplexSingleLine(AnnotationFS anno, StyledText widget, GC gc, Util_impl util) {
		// draw a bracket below the annotation
		Rectangle textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		// bracket left down
		gc.drawLine(textBounds.x, textBounds.y + textBounds.height + 3, textBounds.x,
				textBounds.y + textBounds.height + FONTSIZE + 5);

		// bracket right down
		gc.drawLine(textBounds.x + textBounds.width, textBounds.y + textBounds.height + 3,
				textBounds.x + textBounds.width, textBounds.y + textBounds.height + FONTSIZE + 5);

		// line parallel to the text
		gc.drawLine(textBounds.x, textBounds.y + textBounds.height + FONTSIZE + 5, textBounds.x + textBounds.width,
				textBounds.y + textBounds.height + FONTSIZE + 5);
		String label = anno.getFeatureValueAsString(util.getIEEntityGoldCategoryFeature());
		Point stringExtent = gc.stringExtent(label);
		if (label != null) {
			gc.drawString(label, textBounds.x + textBounds.width / 2 - stringExtent.x / 2,
					textBounds.y + textBounds.height);
		}
	}

	private void drawMultilineBox(List<Rectangle> textbounds, GC gc, AnnotationFS anno, Util_impl util) {

		for (int i = 0; i < textbounds.size(); i++) {
			Rectangle actualRect = textbounds.get(i);
			if (i == 0) {
				// we need to leave the right Border
				String label = anno.getFeatureValueAsString(util.getIEEntityGoldCategoryFeature());
				Point stringExtent = gc.stringExtent(label);
				if (label != null) {
					gc.drawString(label, actualRect.x, actualRect.y - stringExtent.y);
				}
				// top border
				gc.drawLine(actualRect.x, actualRect.y, actualRect.x + actualRect.width, actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y - 1, actualRect.x + actualRect.width, actualRect.y - 1);

				// bottom border
				gc.drawLine(actualRect.x, actualRect.y + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y + 1 + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + 1 + actualRect.height);

				// left border
				gc.drawLine(actualRect.x, actualRect.y, actualRect.x, actualRect.y + actualRect.height);
				gc.drawLine(actualRect.x - 1, actualRect.y, actualRect.x - 1, actualRect.y + actualRect.height);

			}

			else if (i > 0 && i < textbounds.size() - 1) {
				// we need to draw only bottom and top line
				// top border
				gc.drawLine(actualRect.x, actualRect.y, actualRect.x + actualRect.width, actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y - 1, actualRect.x + actualRect.width, actualRect.y - 1);

				// bottom border
				gc.drawLine(actualRect.x, actualRect.y + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y + 1 + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + 1 + actualRect.height);
			} else if (i > 0 && i == textbounds.size() - 1) {
				// the last rectangle of this anno, no left border

				// top border
				gc.drawLine(actualRect.x, actualRect.y, actualRect.x + actualRect.width, actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y - 1, actualRect.x + actualRect.width, actualRect.y - 1);

				// bottom border
				gc.drawLine(actualRect.x, actualRect.y + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y + 1 + actualRect.height, actualRect.x + actualRect.width,
						actualRect.y + 1 + actualRect.height);

				// right border
				gc.drawLine(actualRect.x + actualRect.width, actualRect.y, actualRect.x + actualRect.width,
						actualRect.y + actualRect.height);
				gc.drawLine(actualRect.x + 1 + actualRect.width, actualRect.y, actualRect.x + 1 + actualRect.width,
						actualRect.y + actualRect.height);

			}
		}

	}

	private List<Integer> determineLineBreakIndexes(AnnotationFS anno, StyledText widget) {
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		List<Integer> linebreakIndex = new ArrayList<Integer>();
		if (bounds.height > standardBounds.height) {
			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > (linebreakIndex.size() + 1)
						* (standardBounds.height + widget.getLineSpacing())) {
					linebreakIndex.add(i - 1);
				}
			}
		}
		return linebreakIndex;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return (int) (IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT*1.75);
	}

}
