package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class IEViewRelationDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;
	private static final int FONTSIZE = 8;

	public IEViewRelationDrawingStrategy(StyleRange range) {
		this.styleRange = range;
	}

	// draws a rectangle + the entity name if the annotation is not complex
	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		Util_impl util = new Util_impl(anno.getCAS());

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}

		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;


		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		int amountOfLines = 1;
		if (bounds.height > standardBounds.height) {

			if (bounds.height == 2 * standardBounds.height + widget.getLineSpacing())
				amountOfLines = 2;

			else {
				// or more but then we dont even care...
				amountOfLines = 3;
			}
		}

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(7);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		//
		// Color lightBlue = new Color(widget.getDisplay(), new RGB(50, 166,
		// 94));
		Color lightBlue = new Color(widget.getDisplay(), new RGB(32, 47, 214));
		Color red = new Color(widget.getDisplay(), new RGB(220, 0, 0));
		Feature f = util.getIERelationGoldEntityFromFeature();
		Feature f2 = util.getIERelationGoldEntityToFeature();

		if (amountOfLines == 1) {

			drawInSameLine(lightBlue, f, f2, anno, length, offset, gc, widget);

		}
		// MULTI LINES
		else if (amountOfLines == 2) {

			// we draw inbetween the space

			if (f != null && f2 != null) {

				AnnotationFS agens = (AnnotationFS) anno.getFeatureValue(f);
				AnnotationFS agens2 = (AnnotationFS) anno.getFeatureValue(f2);
				if (agens == null || agens2 == null)
					return;

				Point up;
				Point down;
				if (agens.getBegin() < agens2.getBegin()) {
					up = new Point(agens.getBegin(), agens.getEnd());
					down = new Point(agens2.getBegin(), agens2.getEnd());
				}

				else {
					down = new Point(agens.getBegin(), agens.getEnd());
					up = new Point(agens2.getBegin(), agens2.getEnd());
				}

				Rectangle textBoundsUp = widget.getTextBounds(up.x, up.y);
				Rectangle textBoundsDown = widget.getTextBounds(down.x, down.y);
				Rectangle textBounds2 = widget.getTextBounds(agens2.getBegin(), agens2.getEnd());
				Rectangle textBounds = widget.getTextBounds(agens.getBegin(), agens.getEnd());

				gc.setForeground(lightBlue);

				// we now draw from bottom to up
				gc.drawLine(textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y,
						textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y - 15);

				// to the x of up
				gc.drawLine(textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y - 15,
						textBoundsUp.x + textBoundsUp.width / 2, textBoundsDown.y - 15);

				// up into the other anno
				gc.drawLine(textBoundsUp.x + textBoundsUp.width / 2, textBoundsDown.y - 15,
						textBoundsUp.x + textBoundsUp.width / 2, textBoundsUp.y + textBoundsUp.height);

				// string
				String val = anno.getFeatureValueAsString(new Util_impl(anno.getCAS()).getIERelationGoldLabelFeature());				if (val != null) {
					Point textExtent = gc.textExtent(val);

					gc.drawString(val, textBounds.x - textExtent.x + 5, textBounds.y - 20);
				}

				// draw the arrow tip into agens2
				gc.setBackground(lightBlue);
				gc.fillPolygon(new int[] { textBounds2.x - 5 + textBounds2.width / 2, textBounds2.y - 5,
						textBounds2.x + 5 + textBounds2.width / 2, textBounds2.y - 5,
						textBounds2.x + textBounds2.width / 2, textBounds2.y });

			}

		} else {
			// multiple lines we draw from the anno at the bottom to the left
			// then up and to the right until we reached the wanted anno, then
			// add the little arrow

			if (f != null && f2 != null) {

				AnnotationFS agens = (AnnotationFS) anno.getFeatureValue(f);
				AnnotationFS agens2 = (AnnotationFS) anno.getFeatureValue(f2);
				if (agens == null || agens2 == null)
					return;

				Point up;
				Point down;
				if (agens.getBegin() < agens2.getBegin()) {
					up = new Point(agens.getBegin(), agens.getEnd());
					down = new Point(agens2.getBegin(), agens2.getEnd());
				}

				else {
					down = new Point(agens.getBegin(), agens.getEnd());
					up = new Point(agens2.getBegin(), agens2.getEnd());
				}

				Rectangle textBoundsUp = widget.getTextBounds(up.x, up.y);
				Rectangle textBoundsDown = widget.getTextBounds(down.x, down.y);
				Rectangle textBounds2 = widget.getTextBounds(agens2.getBegin(), agens2.getEnd());
				Rectangle textBounds = widget.getTextBounds(agens.getBegin(), agens.getEnd());

				// now we have the bounds of the annotations and know which one
				// is at the bottom and which one is at the top

				gc.setForeground(lightBlue);

				// we now draw from bottom to up
				gc.drawLine(textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y,
						textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y - 15);

				// to the left
				gc.drawLine(textBoundsDown.x + textBoundsDown.width / 2 - 1, textBoundsDown.y - 15, 4,
						textBoundsDown.y - 15);

				// draw to up + 15
				gc.drawLine(4, textBoundsDown.y - 15, 4, textBoundsUp.y - 15);

				// to the right until we reached the anno
				gc.drawLine(4, textBoundsUp.y - 15, textBoundsUp.x + textBoundsUp.width / 2, textBoundsUp.y - 15);

				// down into the anno
				gc.drawLine(textBoundsUp.x + textBoundsUp.width / 2, textBoundsUp.y - 15,
						textBoundsUp.x + textBoundsUp.width / 2, textBoundsUp.y);

				// string
				String val = anno.getFeatureValueAsString(new Util_impl(anno.getCAS()).getIERelationGoldLabelFeature());
				if (val != null) {
					Point textExtent = gc.textExtent(val);

					gc.drawString(val, textBounds.x - textExtent.x + 5, textBounds.y - 20);
				}

				// first draw the arrow tip into agens2
				gc.setBackground(lightBlue);
				gc.fillPolygon(new int[] { textBounds2.x - 5 + textBounds2.width / 2, textBounds2.y - 5,
						textBounds2.x + 5 + textBounds2.width / 2, textBounds2.y - 5,
						textBounds2.x + textBounds2.width / 2, textBounds2.y });
			}

		}

		lightBlue.dispose();
		red.dispose();
		font.dispose();
		gc.dispose();
	}
	
	private void drawInSameLine(Color lightBlue, Feature f, Feature f2, AnnotationFS anno, int length, int offset,
			GC gc, StyledText widget) {
		//drawBox(length, offset, gc, widget);

		// System.out.println(bounds.height);

		// check if agens and agens2 are in the same line

		if (f != null && f2 != null) {

			AnnotationFS agens = (AnnotationFS) anno.getFeatureValue(f);
			AnnotationFS agens2 = (AnnotationFS) anno.getFeatureValue(f2);
			if (agens == null || agens2 == null)
				return;

			Rectangle textBounds = widget.getTextBounds(agens.getBegin(), agens.getEnd());
			Rectangle textBounds2 = widget.getTextBounds(agens2.getBegin(), agens2.getEnd());

			gc.setForeground(lightBlue);

			// ==== DRAW ARC

			gc.drawArc(textBounds.x, textBounds.y - 17, textBounds2.x + textBounds2.width - (textBounds.x)-3, 28, 10,
					180);

			String val = anno.getFeatureValueAsString(new Util_impl(anno.getCAS()).getIERelationGoldLabelFeature());
			if (val != null) {
				Point textExtent = gc.textExtent(val);

				gc.drawString(val, Math.max(textBounds.x,textBounds2.x)-Math.abs(textBounds.x-textBounds2.x)/2, textBounds.y - 30);
			}
			gc.setBackground(lightBlue);

			gc.fillPolygon(new int[] { textBounds2.x - 5 + textBounds2.width-2, textBounds2.y - 5,
					textBounds2.x + 5 + textBounds2.width-2, textBounds2.y - 5,
					textBounds2.x + textBounds2.width-2, textBounds2.y });
		}

	}

	public void drawBox(int length, int offset, GC gc, StyledText widget) {

		// int length = anno.getEnd()-anno.getBegin();
		// int offset = anno.getBegin();
		if (length != 0) {
			if (gc != null) {
				Rectangle bounds = widget.getTextBounds(offset, offset + length - 1);

				gc.setForeground(styleRange.background);

				int correctedHeight;

				if (bounds.height > 0) {
					correctedHeight = bounds.height - 1;
				} else {
					correctedHeight = bounds.height;
				}

				int correctedWidth;

				if (bounds.width > 0)
					correctedWidth = bounds.width - 1;
				else
					correctedWidth = bounds.width;

				gc.drawRectangle(bounds.x, bounds.y, correctedWidth, correctedHeight);
			} else {
				widget.redrawRange(offset, length, true);
			}
		}
	}


	@Override
	public int getLineHeight(AnnotationFS anno) {
	 return (int) (IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT*1.75);
	}

}
