package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class ImportantPPlOnlyDrawingStrategy implements IAnnotationDrawingStrategy {

	private Feature isImp;
	private Feature id;
	private static final int COLORSTEP = 30;

	// initialize COlor and Mode
	private static Color DEFAULT_COLOR = null;
	private static final int FONTSIZE = 8;

	private HashMap<String, Color> colMap;

	public ImportantPPlOnlyDrawingStrategy(Feature isImp, Feature id) {

		this.isImp = isImp;
		this.id = id;

		colMap = new HashMap<String, Color>();
	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		if (DEFAULT_COLOR == null) {
			// initialize COlor and Mode
			DEFAULT_COLOR = new Color(gc.getDevice(), 255, 255, 0);
		}

		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (bounds.height > standardBounds.height) {
			// find the Strings that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					break;
				}
			}
		}

		String featureValueAsString = anno.getFeatureValueAsString(isImp);

		// just draw the important ones
		if (featureValueAsString == null)
			return;

		String idS = anno.getFeatureValueAsString(id);

		// get color for id

		Color c = getColorForId(idS, gc);

		// draw with fat border

		gc.setForeground(c);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		// draw the one feature right
		String feat = idS;
		if (feat != null && feat.trim().length() > 0) {
			// Only draw if String not empty
			gc.drawString(anno.getFeatureValueAsString(id), x + bounds.width - 1, y - lineHeight - FONTSIZE - 7);

			// draw it thick
			for (int i = 0; i < 3; i++) {
				Rectangle r = new Rectangle(bounds.x - i, bounds.y - i, bounds.width + i + 2, bounds.height + i + 2);
				gc.drawRectangle(r);
			}
			y -= lineHeight + 1;
		}
		gc.dispose();
	}

	private Color getColorForId(String idS, GC gc) {

		if (this.colMap.containsKey(idS)) {
			return colMap.get(idS);
		}

		else {

			Color actual = DEFAULT_COLOR;

			// add to the color

			StyleRange styleRange = new StyleRange();
			styleRange.background = actual;

			// update Color - - - per default the Colors are calculated and only
			// the background is
			// higlighted
			int r = (actual.getRed() + COLORSTEP) % 256;
			int g = (actual.getGreen() - COLORSTEP) % 256;
			int b = (actual.getBlue() - COLORSTEP / 2) % 256;
			if (g < 0) {
				g += 256;
			}
			if (b < 0) {
				b += 256;
			}
			actual = new Color(gc.getDevice(), r, g, b);
			DEFAULT_COLOR = actual;

			this.colMap.put(idS, actual);
			return actual;
		}
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
