package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class JoniSegmentDrawingStrategy implements IAnnotationDrawingStrategy {

	private StyleRange styleRange;
	private Color[] colors;

	public JoniSegmentDrawingStrategy(StyleRange range) {
		this.styleRange = range;
		colors = new Color[2];
		colors[0] = new Color(Display.getCurrent(), new RGB(100, 100, 220));
		colors[1] = new Color(Display.getCurrent(), new RGB(204, 0, 0));
	}

	// draws a rectangle + the entity name if the annotation is not complex
	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		Util_impl util = new Util_impl(anno.getCAS());
		if (util.getIndexOfTypeInDocument(anno) % 2 == 0) {
			gc.setForeground(colors[0]);
		} else {
			gc.setForeground(colors[1]);
		}
		// determine all charactersplitindexes where we need to start drawing in
		// a new line
		// get the bounds of the Annotation
		List<Integer> lineBreakIndexes = determineLineBreakIndexes(anno, widget);
		
		//change the font
		// adjust the Font



			// draw for every subsection
			if (lineBreakIndexes.size() != 0) {
				List<Rectangle> textbounds = new ArrayList<Rectangle>();
				textbounds.add(widget.getTextBounds(anno.getBegin(), lineBreakIndexes.get(0)));
				for (int j = 1; j < lineBreakIndexes.size(); j++) {
					textbounds.add(widget.getTextBounds(lineBreakIndexes.get(j - 1) + 1, lineBreakIndexes.get(j)));
				}
				// add the last box
				textbounds.add(
						widget.getTextBounds(lineBreakIndexes.get(lineBreakIndexes.size() - 1) + 1, anno.getEnd() - 1));

				// draw
				drawMultilineBox(textbounds,gc,anno,util);
			}
			
			else{
				//single line!
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), anno.getEnd()-1);
				gc.drawRectangle(textBounds);
			}

		
		gc.dispose();

	}

	private void drawMultilineBox(List<Rectangle> textbounds, GC gc, AnnotationFS anno, Util_impl util) {
		
		for(int i =0;i<textbounds.size();i++){
			Rectangle actualRect = textbounds.get(i);
			if(i==0){
				//we need to leave the right Border
				//top border
				gc.drawLine(actualRect.x, actualRect.y,actualRect.x+actualRect.width , actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y-1,actualRect.x+actualRect.width , actualRect.y-1);
			
				//bottom border
				gc.drawLine(actualRect.x, actualRect.y+actualRect.height,actualRect.x+actualRect.width , actualRect.y+actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y+1+actualRect.height,actualRect.x+actualRect.width , actualRect.y+1+actualRect.height);
				
				//left border
				gc.drawLine(actualRect.x, actualRect.y,actualRect.x , actualRect.y+actualRect.height);
				gc.drawLine(actualRect.x-1, actualRect.y,actualRect.x-1 , actualRect.y+actualRect.height);
				
			}
			
			else if(i>0 && i<textbounds.size()-1){
				//we need to draw only bottom and top line
				//top border
				gc.drawLine(actualRect.x, actualRect.y,actualRect.x+actualRect.width , actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y-1,actualRect.x+actualRect.width , actualRect.y-1);
			
				//bottom border
				gc.drawLine(actualRect.x, actualRect.y+actualRect.height,actualRect.x+actualRect.width , actualRect.y+actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y+1+actualRect.height,actualRect.x+actualRect.width , actualRect.y+1+actualRect.height);
			}
			else if(i>0 &&i==textbounds.size()-1){
				//the last rectangle of this anno, no left border
				
				//top border
				gc.drawLine(actualRect.x, actualRect.y,actualRect.x+actualRect.width , actualRect.y);
				gc.drawLine(actualRect.x, actualRect.y-1,actualRect.x+actualRect.width , actualRect.y-1);
			
				//bottom border
				gc.drawLine(actualRect.x, actualRect.y+actualRect.height,actualRect.x+actualRect.width , actualRect.y+actualRect.height);
				gc.drawLine(actualRect.x, actualRect.y+1+actualRect.height,actualRect.x+actualRect.width , actualRect.y+1+actualRect.height);
				
				//right border
				gc.drawLine(actualRect.x+actualRect.width, actualRect.y,actualRect.x+actualRect.width , actualRect.y+actualRect.height);
				gc.drawLine(actualRect.x+1+actualRect.width, actualRect.y,actualRect.x+1+actualRect.width , actualRect.y+actualRect.height);
				
			}
		}
		
	}

	private List<Integer> determineLineBreakIndexes(AnnotationFS anno, StyledText widget) {
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		List<Integer> linebreakIndex = new ArrayList<Integer>();
		if (bounds.height > standardBounds.height) {
			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > (linebreakIndex.size() + 1)
						* (standardBounds.height + widget.getLineSpacing())) {
					linebreakIndex.add(i - 1);
				}
			}
		}
		return linebreakIndex;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return 0;
	}



}
