package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class MedEvalDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	StyleRange styleRange;

	// draws the features above the Annotations
	public MedEvalDrawingStrategy(StyleRange range, String featEval) {
		this.styleRange = range;
		this.feature = featEval;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		// init 3 Colors for 3 priorities
		Color orange = new Color(widget.getDisplay(), new RGB(250, 156, 32));
		// Color green = new Color(widget.getDisplay(), new RGB(43, 250, 32));
		Color red = new Color(widget.getDisplay(), new RGB(250, 10, 32));

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();

		// widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the Strings that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					break;
				}
			}
		}

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int y = bounds.y + lineHeight;

		if (!multipleLines) {

			gc.setForeground(styleRange.background);

			// also set more images if necessary

			// uncertainty

			Feature evalType = anno.getType().getFeatureByBaseName(this.feature);

			String value = anno.getFeatureValueAsString(evalType);

			if (value != null && value.equals("FN")) {

				// squiggles
				Point left = widget.getLocationAtOffset(offset);
				Point right = widget.getLocationAtOffset(offset + length);
				Rectangle rect = widget.getTextBounds(offset, offset + length - 1);
				left.x = rect.x;
				right.x = rect.x + rect.width;

				int[] polyline = computePolyline(left, right, widget.getBaseline(offset), widget.getLineHeight(offset));

				gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
									// optimized performance
				gc.setLineStyle(SWT.LINE_SOLID);
				gc.setForeground(red);
				gc.drawPolyline(polyline);

			}

			if (value != null && value.equals("FP")) {

				// squiggles
				Point left = widget.getLocationAtOffset(offset);
				Point right = widget.getLocationAtOffset(offset + length);
				Rectangle rect = widget.getTextBounds(offset, offset + length - 1);
				left.x = rect.x;
				right.x = rect.x + rect.width;

				int[] polyline = computePolyline(left, right, widget.getBaseline(offset), widget.getLineHeight(offset));

				gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
									// optimized performance
				gc.setLineStyle(SWT.LINE_SOLID);
				gc.setForeground(orange);
				gc.drawPolyline(polyline);

			}

			else if (value != null && value.equals("TP")) {

				// underline
				gc.drawLine(bounds.x, y, bounds.x + bounds.width, y);

			}
		}

		// initialFont.dispose();
		font.dispose();
		gc.dispose();
		orange.dispose();
		// green.dispose();
		red.dispose();

	}

	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH = 4; // must be even
		final int HEIGHT = 2; // can be any number

		int peaks = (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks = 1;

		int leftX = left.x;

		// compute (number of point) * 2
		int length = ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates = new int[length];

		// cache peeks' y-coordinates
		int top = left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom = top + HEIGHT;

		// populate array with peek coordinates
		for (int i = 0; i < peaks; i++) {
			int index = 4 * i;
			coordinates[index] = leftX + (WIDTH * i);
			coordinates[index + 1] = bottom;
			coordinates[index + 2] = coordinates[index] + WIDTH / 2;
			coordinates[index + 3] = top;
		}

		// the last down flank is missing
		coordinates[length - 2] = Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length - 1] = bottom;

		return coordinates;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}
}
