package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.util.EditorPaintingUtil;

public class NamedEntityDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	private static final int FONTSIZE = 8;

	private String numberus;
	private String gender;

	private String core;

	private String pseudo;

	private String coreRange;

	StyleRange styleRange;

	// draws the features above the Annotations
	public NamedEntityDrawingStrategy(StyleRange range, String id, String core, String pseudo, String numerus,
			String gender, String coreRange) {
		this.styleRange = range;
		this.feature = id;
		this.core = core;
		this.numberus = numerus;
		this.pseudo = pseudo;
		this.coreRange = coreRange;
		this.gender = gender;

		if (feature == null) {
			feature = "";
		}
		if (core == null) {
			core = "";
		}
		if (numerus == null) {
			numerus = "";
		}
		if (pseudo == null) {
			pseudo = "";
		}
		if (coreRange == null) {
			coreRange = "";
		}

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		// widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		int linebreakIndex = 0;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					linebreakIndex = i - 1;
					break;
				}
			}
		}

		// System.out.println(bounds.height);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (!multipleLines) {
			// draw the one feature right
			Feature f = anno.getType().getFeatureByBaseName(feature);
			if (f != null) {
				String feat = anno.getFeatureValueAsString(f);
				if (feat != null && feat.trim().length() > 0) {
					// Only draw if String not empty
					gc.drawString(anno.getFeatureValueAsString(f), x + bounds.width - 1, y - lineHeight - FONTSIZE - 7);
					boolean alreadyHandled = false;
					for (StyleRange r : widget.getStyleRanges(offset, length)) {
						// ugly solution TODO
						if (r.background.getRGB().green == 247) {
							alreadyHandled = true;
							break;
						}
					}
					if (!alreadyHandled) {

						setGcForegroundBasedOnCertainty(anno, gc);
						gc.drawRectangle(bounds);
					}
					y -= lineHeight + 1;
				}
			}

			gc.setForeground(styleRange.background);
			// coreNameFeat if true => Kasten if false nothing
			Feature coreFeat = anno.getType().getFeatureByBaseName(core);

			if (coreFeat != null) {

				String coreValue = anno.getFeatureValueAsString(coreFeat);

				if (coreValue != null && coreValue.trim().length() > 0) {
					// draw it
					if (coreValue.equals("true")) {

						// get the String that tells us from and to bounds

						String range = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName(coreRange));
						if (range != null) {
							int beg = Integer.parseInt(range.split(":")[0]);
							int end = Integer.parseInt(range.split(":")[1]) - 1;

							if (beg != Integer.MAX_VALUE && end < widget.getText().length()) {
								Rectangle textBounds = widget.getTextBounds(beg, end);

								gc.drawRectangle(textBounds.x, textBounds.y, textBounds.width, textBounds.height + 2);
							}
						}
					}
				}
			}

			// pseudoperson => squiggles if there

			Feature pseudoFeat = anno.getType().getFeatureByBaseName(pseudo);

			if (pseudoFeat != null) {

				String pseudoValue = anno.getFeatureValueAsString(pseudoFeat);

				if (pseudoValue != null && pseudoValue.trim().length() > 0) {
					// draw it
					if (pseudoValue.equals("true")) {
						// draw squiggles

						Point left = widget.getLocationAtOffset(offset);
						Point right = widget.getLocationAtOffset(offset + length);
						Rectangle rect = widget.getTextBounds(offset, offset + length - 1);
						left.x = rect.x;
						right.x = rect.x + rect.width;

						int[] polyline = computePolyline(left, right, widget.getBaseline(offset),
								widget.getLineHeight(offset));

						gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
											// optimized performance
						gc.setLineStyle(SWT.LINE_SOLID);
						gc.setForeground(styleRange.background);
						gc.drawPolyline(polyline);

					}
				}
			}

			Feature numberFeat = anno.getType().getFeatureByBaseName(numberus);
			Feature genderFeat = anno.getType().getFeatureByBaseName(gender);

			if (numberFeat != null) {

				String numberValue = anno.getFeatureValueAsString(numberFeat);

				if (numberValue != null && numberValue.trim().length() > 0) {
					// draw it
					if (numberValue.equals("pl")) {
						// draw underlined

						Image image = EditorPaintingUtil.createImage(EditorPaintingUtil.PLURAL_IMAGE);
						gc.drawImage(image, x, y - FONTSIZE - 4);
						// gc.drawLine(x + 2, y + bounds.height, x +
						// bounds.width - 2, y + bounds.height);
						image.dispose();
					}

					else {

						// draw gender
						String gender = anno.getFeatureValueAsString(genderFeat);
						Image image = null;
						if (gender != null) {
							if (gender.startsWith("f")) {
								image = EditorPaintingUtil.createImage(EditorPaintingUtil.GENDER_FEMALE);
							}

							else if (gender.startsWith("m")) {
								image = EditorPaintingUtil.createImage(EditorPaintingUtil.GENDER_MALE);
							}

							else {
								image = EditorPaintingUtil.createImage(EditorPaintingUtil.GENDER_UNKNOWN);
							}
							gc.drawImage(image, x, y - FONTSIZE - 8);
							image.dispose();
						}
					}
				}
			}
		}

		else {
			// multiple lines start here - we use 2 bounds

			Rectangle textBoundsFirstLine = widget.getTextBounds(anno.getBegin(), linebreakIndex);
			Rectangle textBoundsSecondLine = widget.getTextBounds(linebreakIndex + 1, anno.getEnd());

			Feature f = anno.getType().getFeatureByBaseName(feature);
			if (f != null) {
				String feat = anno.getFeatureValueAsString(f);
				if (feat != null && feat.trim().length() > 0) {
					// Only draw if String not empty
					gc.drawString(anno.getFeatureValueAsString(f), textBoundsSecondLine.x + textBoundsSecondLine.width
							- 1, textBoundsSecondLine.y - FONTSIZE - 7);
					boolean alreadyHandled = false;
					for (StyleRange r : widget.getStyleRanges(offset, length)) {
						// ugly solution
						if (r.background.getRGB().green == 247) {
							alreadyHandled = true;
							break;
						}
					}
					if (!alreadyHandled) {
						gc.drawRectangle(textBoundsFirstLine);
						gc.drawRectangle(textBoundsSecondLine);
					}
					y -= lineHeight + 1;
				}
			}

			gc.setForeground(styleRange.background);
			// coreNameFeat if true => Kasten if false nothing
			Feature coreFeat = anno.getType().getFeatureByBaseName(core);

			if (coreFeat != null) {

				String coreValue = anno.getFeatureValueAsString(coreFeat);

				if (coreValue != null && coreValue.trim().length() > 0) {
					// draw it
					if (coreValue.equals("true")) {
						gc.drawRectangle(textBoundsFirstLine);
						gc.drawRectangle(textBoundsSecondLine);
					}
				}
			}

			// pseudoperson => squiggles if there

			Feature pseudoFeat = anno.getType().getFeatureByBaseName(pseudo);

			if (pseudoFeat != null) {

				String pseudoValue = anno.getFeatureValueAsString(pseudoFeat);

				if (pseudoValue != null && pseudoValue.trim().length() > 0) {
					// draw it
					if (pseudoValue.equals("true")) {
						// draw squiggles

						Point left = widget.getLocationAtOffset(offset);
						Point right = widget.getLocationAtOffset(offset + length);
						left.x = textBoundsFirstLine.x;
						right.x = textBoundsFirstLine.x + textBoundsFirstLine.width;

						int[] polyline = computePolyline(left, right, widget.getBaseline(offset),
								widget.getLineHeight(offset));

						gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
											// optimized performance
						gc.setLineStyle(SWT.LINE_SOLID);
						gc.setForeground(styleRange.background);
						gc.drawPolyline(polyline);

						// second line

						Point left2 = widget.getLocationAtOffset(linebreakIndex + 1);
						Point right2 = widget.getLocationAtOffset(offset + length);
						left.x = textBoundsSecondLine.x;
						right.x = textBoundsSecondLine.x + textBoundsSecondLine.width;

						int[] polyline2 = computePolyline(left2, right2, widget.getBaseline(linebreakIndex + 1),
								widget.getLineHeight(offset));

						gc.setLineWidth(0); // NOTE: 0 means width is 1 but with
											// optimized performance
						gc.setLineStyle(SWT.LINE_SOLID);
						gc.setForeground(styleRange.background);
						gc.drawPolyline(polyline2);

					}
				}
			}

			Feature numberFeat = anno.getType().getFeatureByBaseName(numberus);

			if (numberFeat != null) {

				String numberValue = anno.getFeatureValueAsString(numberFeat);

				if (numberValue != null && numberValue.trim().length() > 0) {
					// draw it
					if (numberValue.equals("pl")) {
						// draw underlined

						// ImageDescriptor imageDescriptor =
						// CorefUiPlugin.getImageDescriptor("/icons/plural.jpg");
						// Image image = imageDescriptor.createImage();
						// gc.drawImage(image, textBoundsFirstLine.x + 3,
						// textBoundsFirstLine.y - FONTSIZE - 4);
						// gc.drawLine(x + 2, y + bounds.height, x +
						// bounds.width - 2, y + bounds.height);
					}
				}
			}

		}
		// initialFont.dispose();
		font.dispose();
		gc.dispose();

	}

	private void setGcForegroundBasedOnCertainty(AnnotationFS anno, GC gc) {

		Feature certainty = anno.getType().getFeatureByBaseName("Uncertain");

		String certaintyS = anno.getFeatureValueAsString(certainty);

		if (certaintyS == null)
			return;

		if (certaintyS.equals("CERTAIN_CORRECT")) {
			gc.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_GREEN));
		}

		else if (certaintyS.equals("CERTAIN_WRONG")) {
			gc.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_MAGENTA));
		}

		else if (certaintyS.equals("UNCERTAIN")) {
			gc.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_YELLOW));
		}

		else if (certaintyS.equals("UNKNOWN")) {
			gc.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLACK));
		}

	}

	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH = 4; // must be even
		final int HEIGHT = 2; // can be any number

		int peaks = (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks = 1;

		int leftX = left.x;

		// compute (number of point) * 2
		int length = ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates = new int[length];

		// cache peeks' y-coordinates
		int top = left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom = top + HEIGHT;

		// populate array with peek coordinates
		for (int i = 0; i < peaks; i++) {
			int index = 4 * i;
			coordinates[index] = leftX + (WIDTH * i);
			coordinates[index + 1] = bottom;
			coordinates[index + 2] = coordinates[index] + WIDTH / 2;
			coordinates[index + 3] = top;
		}

		// the last down flank is missing
		coordinates[length - 2] = Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length - 1] = bottom;

		return coordinates;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
