package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Display;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class POSDrawingStrategy implements IAnnotationDrawingStrategy {

	// this feature is displayed to the right
	String feature;

	private static final int FONTSIZE = 8;

	StyleRange styleRange;

	// draws the features above the Annotations
	public POSDrawingStrategy(StyleRange range, String posFeat) {
		this.styleRange = range;
		this.feature = posFeat;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		Color blue = new Color(Display.getCurrent(), new RGB(0, 0, 220));
		Color red = new Color(Display.getCurrent(), new RGB(230, 0, 0));
		Color yellow = new Color(Display.getCurrent(), new RGB(196, 185, 27));
		Color brown = new Color(Display.getCurrent(), new RGB(92, 87, 23));
		Color orange = new Color(Display.getCurrent(), new RGB(250, 172, 5));
		Color black = new Color(Display.getCurrent(), new RGB(0, 0, 0));

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}
		// getLineHeight
		int lineHeight = widget.getLineHeight();
		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		int linebreakIndex = 0;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					linebreakIndex = i - 1;
					break;
				}
			}
		}

		// System.out.println(bounds.height);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(8);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		int x = bounds.x + 1;
		int y = bounds.y + lineHeight;

		if (!multipleLines) {
			// draw the one feature right
			Feature posTag = anno.getType().getFeatureByBaseName(feature);
			if (posTag != null) {
				String feat = anno.getFeatureValueAsString(posTag);
				if (feat != null && feat.trim().length() > 0) {
					// Only draw if String not empty
					gc.setForeground(getAccordingColor(feat, blue, red, yellow, brown, orange, black));

					gc.drawString(anno.getFeatureValueAsString(posTag), x + 1, y - lineHeight - FONTSIZE - 7);
					boolean alreadyHandled = false;
					for (StyleRange r : widget.getStyleRanges(offset, length)) {
						// ugly solution
						if (r.background.getRGB().green == 247) {
							alreadyHandled = true;
							break;
						}
					}
					if (!alreadyHandled) {
						gc.drawRectangle(bounds);
					}
					y -= lineHeight + 1;
				}
			}

			gc.setForeground(styleRange.background);
			// coreNameFeat if true => Kasten if false nothing

		}

		// initialFont.dispose();
		font.dispose();
		blue.dispose();
		orange.dispose();
		yellow.dispose();
		red.dispose();
		brown.dispose();
		black.dispose();
		gc.dispose();

	}

	private Color getAccordingColor(String feat, Color blue, Color red, Color yellow, Color brown, Color orange,
			Color black) {

		if (feat.startsWith("VV"))
			return blue;

		else if (feat.startsWith("N"))
			return orange;

		else if (feat.startsWith("A"))
			return yellow;

		else if (feat.startsWith("P"))
			return brown;

		else if (feat.startsWith("AP"))
			return red;
		return black;
	}

	private int[] computePolyline(Point left, Point right, int baseline, int lineHeight) {

		final int WIDTH = 4; // must be even
		final int HEIGHT = 2; // can be any number

		int peaks = (right.x - left.x) / WIDTH;
		if (peaks == 0 && right.x - left.x > 2)
			peaks = 1;

		int leftX = left.x;

		// compute (number of point) * 2
		int length = ((2 * peaks) + 1) * 2;
		if (length < 0)
			return new int[0];

		int[] coordinates = new int[length];

		// cache peeks' y-coordinates
		int top = left.y + Math.min(baseline + 1, lineHeight - HEIGHT - 1);
		int bottom = top + HEIGHT;

		// populate array with peek coordinates
		for (int i = 0; i < peaks; i++) {
			int index = 4 * i;
			coordinates[index] = leftX + (WIDTH * i);
			coordinates[index + 1] = bottom;
			coordinates[index + 2] = coordinates[index] + WIDTH / 2;
			coordinates[index + 3] = top;
		}

		// the last down flank is missing
		coordinates[length - 2] = Math.min(Math.max(0, right.x - 1), left.x + (WIDTH * peaks));
		coordinates[length - 1] = bottom;

		return coordinates;
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
