package de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;

public class RelationDrawingStrategy implements IAnnotationDrawingStrategy {
	private static final int FONTSIZE = 8;

	private StyleRange styleRange;
	private String agens;
	private String agens2;

	public RelationDrawingStrategy(StyleRange range, String agens, String agens2) {
		this.styleRange = range;

		this.agens = agens;
		this.agens2 = agens2;

	}

	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {

		int offset = anno.getBegin();
		int length = anno.getEnd() - anno.getBegin();
		// cant draw an empty annotation
		if (length == 0) {
			return;
		}

		// calculate the amount of space which is needed to display the features
		int amountOfSpace = FONTSIZE + 1;

		widget.setLineSpacing(amountOfSpace + 10);

		// get the bounds of the Annotation
		Rectangle standardBounds = widget.getTextBounds(0, 0);
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);

		boolean multipleLines = false;
		if (bounds.height > standardBounds.height) {
			multipleLines = true;

			// find the String that are in 1 line
			for (int i = anno.getBegin(); i < anno.getEnd(); i++) {
				Rectangle textBounds = widget.getTextBounds(anno.getBegin(), i);
				if (textBounds.height > standardBounds.height) {
					break;
				}
			}
		}

		// drawbox
		if (!multipleLines)
			drawBox(length, offset, gc, widget);

		// System.out.println(bounds.height);

		// change the Color of the GC object
		gc.setForeground(styleRange.background);

		// adjust the Font
		Font initialFont = widget.getFont();
		FontData[] fontData = initialFont.getFontData();
		for (int i = 0; i < fontData.length; i++) {
			fontData[i].setHeight(7);
		}

		Font font = new Font(gc.getDevice(), fontData);
		gc.setFont(font);

		//
		// Color lightBlue = new Color(widget.getDisplay(), new RGB(50, 166,
		// 94));
		Color lightBlue = new Color(widget.getDisplay(), new RGB(32, 47, 214));
		Color red = new Color(widget.getDisplay(), new RGB(220, 0, 0));
		Feature f = anno.getType().getFeatureByBaseName(agens);
		Feature f2 = anno.getType().getFeatureByBaseName(agens2);

		// check if agens and agens2 are in the same line

		if (f != null && f2 != null) {

			AnnotationFS agens = (AnnotationFS) anno.getFeatureValue(f);
			AnnotationFS agens2 = (AnnotationFS) anno.getFeatureValue(f2);
			if (agens == null || agens2 == null)
				return;
			int lineAtOffsetAgens = widget.getLineAtOffset(agens.getEnd());
			int lineAtOffsetAgens2 = widget.getLineAtOffset(agens2.getBegin());

			if (lineAtOffsetAgens != lineAtOffsetAgens2)
				return;

			Rectangle textBounds = widget.getTextBounds(agens.getBegin(), agens.getEnd());
			Rectangle textBounds2 = widget.getTextBounds(agens2.getBegin(), agens2.getEnd());

			gc.setForeground(lightBlue);

			// ==== DRAW ARC

			gc.drawArc(textBounds.x, textBounds.y - 17, textBounds2.x + textBounds2.width / 2 - (textBounds.x), 30, 10,
					180);

			String val = anno.getFeatureValueAsString(anno.getType().getFeatureByBaseName("Type"));
			if (val != null) {
				Point textExtent = gc.textExtent(val);

				gc.drawString(val, textBounds.x - textExtent.x + 5, textBounds.y - 20);
			}
			gc.setBackground(lightBlue);

			gc.fillPolygon(new int[] { textBounds2.x - 5 + textBounds2.width / 2, textBounds2.y - 5,
					textBounds2.x + 5 + textBounds2.width / 2, textBounds2.y - 5,
					textBounds2.x + textBounds2.width / 2, textBounds2.y });
		}

		// if (f != null) {
		//
		// AnnotationFS agensF = (AnnotationFS) anno.getFeatureValue(f);
		//
		// if (agensF != null)
		// // draw oval - blue
		// drawOval(agensF.getBegin(), agensF.getEnd(), widget, gc, lightBlue);
		//
		// }
		// f = anno.getType().getFeatureByBaseName(agens2);
		// if (f != null) {
		// AnnotationFS agens2F = (AnnotationFS) anno.getFeatureValue(f);
		//
		// if (agens2F != null) {
		// // draw oval - red
		// drawOval(agens2F.getBegin(), agens2F.getEnd(), widget, gc, red);
		// }
		// }

		font.dispose();
		gc.dispose();
		lightBlue.dispose();
		red.dispose();

	}

	public void drawBox(int length, int offset, GC gc, StyledText widget) {

		// int length = anno.getEnd()-anno.getBegin();
		// int offset = anno.getBegin();
		if (length != 0) {
			if (gc != null) {
				Rectangle bounds = widget.getTextBounds(offset, offset + length - 1);

				gc.setForeground(styleRange.background);

				int correctedHeight;

				if (bounds.height > 0) {
					correctedHeight = bounds.height - 1;
				} else {
					correctedHeight = bounds.height;
				}

				int correctedWidth;

				if (bounds.width > 0)
					correctedWidth = bounds.width - 1;
				else
					correctedWidth = bounds.width;

				gc.drawRectangle(bounds.x, bounds.y, correctedWidth, correctedHeight);
			} else {
				widget.redrawRange(offset, length, true);
			}
		}
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}
