package de.uniwue.mk.kall.editorPainting.util;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;
import org.osgi.service.prefs.Preferences;

public class EditorPaintingUtil {

	public static final String PROJECT_IMAGE = "icons/project.png";

	public static final String FOLDER_IMAGE = "icons/folder.png";

	public static final String XML_IMAGE = "icons/xml.png";

	public static final String XMI_IMAGE = "icons/xmi.png";

	public static final String PLURAL_IMAGE = "icons/plural.jpg";

	public static final String APPLICATION_ID = "de.uniwue.mkrug.kallimachos.annotation.editor";

	// WSD categorie images

	public static final String WSD_CAT_TOPS = "icons/catTops.png";
	public static final String WSD_CAT_ARTEFAKT = "icons/catArtefact.png";
	public static final String WSD_CAT_ATTRIBUT = "icons/catAttribute.jpg";
	public static final String WSD_CAT_BESITZ = "icons/catProperty.jpg";
	public static final String WSD_CAT_RELATION = "icons/catRelation.png";
	public static final String WSD_CAT_GESCHEHEN = "icons/catEvent.jpg";
	public static final String WSD_CAT_FORM = "icons/catForm.png";
	public static final String WSD_CAT_GEFUEHL = "icons/catFeeling.png";
	public static final String WSD_CAT_GRUPPE = "icons/catGroup.png";
	public static final String WSD_CAT_KOERPER = "icons/catBody.png";
	public static final String WSD_CAT_KOGNITION = "icons/catCognition.jpg";
	public static final String WSD_CAT_KOMMUNIKATION = "icons/catCommunication.png";
	public static final String WSD_CAT_MENGE = "icons/catCrowd.png";
	public static final String WSD_CAT_MENSCH = "icons/catHuman.jpg";
	public static final String WSD_CAT_MOTIV = "icons/catReason.jpg";
	public static final String WSD_CAT_NAHRUNG = "icons/catnahrung.png";
	public static final String WSD_CAT_NATGGST = "icons/catNature.png";
	public static final String WSD_CAT_NATPH = "icons/catphaenomen.png";
	public static final String WSD_CAT_ORT = "icons/catlocation.png";
	public static final String WSD_CAT_PFLANZE = "icons/catplant.png";
	public static final String WSD_CAT_SUBSTANZ = "icons/catSubstance.png";
	public static final String WSD_CAT_TIER = "icons/catAnimal.png";
	public static final String WSD_CAT_ZEIT = "icons/cattime.png";
	public static final String WSD_CAT_UNBEKANNT = "icons/catUnknown.png";

	// graph view

	public static final String NODE_WOMAN = "icons/testWoman.png";
	public static final String NODE_MAN = "icons/Man-Icon.png";

	// Grammar view
	public static final String GRAMMAR_SUBJECT = "icons/subject.png";
	public static final String GRAMMAR_PREDICATE = "icons/predicate.png";
	public static final String GRAMMAR_OBJECT = "icons/object.png";

	// perspecives

	public static final String PERSPECTIVE_CHANGE = "icons/perspectiveChange.png";
	public static final String PERSPECTIVE_ANNOTATION = "icons/perspectiveAnnotation.png";
	public static final String PERSPECTIVE_CORRECTION = "icons/perspectiveCorrection.jpg";
	public static final String PERSPECTIVE_INTERANNOTATOR_AGREEMENT = "icons/agreementPerspective.jpg";

	// corefExplain
	public static final String GENDER_MALE = "icons/male.png";
	public static final String GENDER_FEMALE = "icons/female.png";
	public static final String GENDER_UNKNOWN = "icons/unknownGender.png";
	public static final String CONVERSATION = "icons/conversation.jpg";

	public static final String CORRECT = "icons/correct.png";
	public static final String ERROR = "icons/error.png";

	public static HashMap<String, String> imageMapping = new HashMap<String, String>();

	static {
		imageMapping.put("Tops", WSD_CAT_TOPS);
		imageMapping.put("Artefakt", WSD_CAT_ARTEFAKT);
		imageMapping.put("Attribut", WSD_CAT_ATTRIBUT);
		imageMapping.put("Besitz", WSD_CAT_BESITZ);
		imageMapping.put("Relation", WSD_CAT_RELATION);
		imageMapping.put("Geschehen", WSD_CAT_GESCHEHEN);
		imageMapping.put("Form", WSD_CAT_FORM);
		imageMapping.put("Gefuehl", WSD_CAT_GEFUEHL);
		imageMapping.put("Gruppe", WSD_CAT_GRUPPE);
		imageMapping.put("Koerper", WSD_CAT_KOERPER);
		imageMapping.put("Kognition", WSD_CAT_KOGNITION);
		imageMapping.put("Kommunikation", WSD_CAT_KOMMUNIKATION);
		imageMapping.put("Menge", WSD_CAT_MENGE);
		imageMapping.put("Mensch", WSD_CAT_MENSCH);
		imageMapping.put("Motiv", WSD_CAT_MOTIV);
		imageMapping.put("Nahrung", WSD_CAT_NAHRUNG);
		imageMapping.put("natGegenstand", WSD_CAT_NATGGST);
		imageMapping.put("natPhaenomen", WSD_CAT_NATPH);
		imageMapping.put("Ort", WSD_CAT_ORT);
		imageMapping.put("Pflanze", WSD_CAT_PFLANZE);
		imageMapping.put("Substanz", WSD_CAT_SUBSTANZ);
		imageMapping.put("Tier", WSD_CAT_TIER);
		imageMapping.put("Zeit", WSD_CAT_ZEIT);
		imageMapping.put("Unbekannt", WSD_CAT_UNBEKANNT);

		imageMapping.put("Subject", GRAMMAR_SUBJECT);
		imageMapping.put("Object", GRAMMAR_OBJECT);
		imageMapping.put("Predicate", GRAMMAR_PREDICATE);

	}

	public static void saveToPreferences(String preference, String value) {

		Preferences prefs = InstanceScope.INSTANCE.getNode(APPLICATION_ID);

		prefs.put(preference, value);
		try {
			prefs.flush();
		} catch (org.osgi.service.prefs.BackingStoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public static String getFromPreferences(String preference, String defaultValue) {

		Preferences prefs = InstanceScope.INSTANCE.getNode(APPLICATION_ID);

		String wsPref = prefs.get(preference, defaultValue);

		return wsPref;

	}

	public static Image createImage(String image) {
		Bundle bundle = FrameworkUtil.getBundle(EditorPaintingUtil.class);
		URL url = FileLocator.find(bundle, new Path(image), null);
		ImageDescriptor imageDcr = ImageDescriptor.createFromURL(url);
		return imageDcr.createImage();
	}

	public static File getBundleRessource(String path) {
		Bundle bundle = Platform.getBundle(APPLICATION_ID);
		URL fileURL = bundle.getEntry(path);
		File file = null;
		try {
			file = new File(FileLocator.resolve(fileURL).toURI());
		} catch (URISyntaxException e1) {
			e1.printStackTrace();
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		return file;
	}

	// this does work
	public static InputStream getBundleRessourceAsStream(String path) {
		Bundle bundle = FrameworkUtil.getBundle(EditorPaintingUtil.class);
		URL url = FileLocator.find(bundle, new Path(path), null);

		InputStream inputStream = null;
		try {
			inputStream = url.openConnection().getInputStream();
		} catch (IOException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}

		return inputStream;
	}

	public static File urlToFile(URL url) {
		File file = null;
		try {
			file = new File(url.toURI());
		} catch (URISyntaxException e) {
			file = new File(url.getPath());
		}
		return file;
	}
}
