package de.uniwue.mk.kall.grammarView;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.GrammarRealDrawingStrategy;

public class GrammarRealView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<GrammarRealView> CLASS_THIS = GrammarRealView.class;

	@Inject
	static EPartService partService;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.7";

	// DATA CONTAINER - the model, since its usually very simple we have it here
	private List<AnnotationFS> annoList;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	private Type grammaticType;

	private Feature featPhraseType;

	private Feature featSemanticRole;

	private Feature featSyntacticRole;

	private Feature featIsSelected;
	private Feature featSyntacticHead;

	private Button buttonAddGrammarUnit;
	private Button buttonRemoveGrammarUnit;

	private Text textPhraseType;
	private Text textSyntacticRole;
	private Text textSemanticRole;
	private Text textReferingAnno;

	private Button buttonSetPhraseType;
	private Button buttonSetSemanticRoleType;
	private Button buttonSetSyntacticRoleType;

	private int defaultTopMargin;

	private Feature featLayer;

	private AnnotationFS smallestSelection;

	// compounds - UI

	@Inject
	public GrammarRealView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;

	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("GRAMMAR REAL VIEW POSTCONSTRUCT");

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);
	}

	@Override
	public void initModel() {
		annoList = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(this.grammaticType)) {
			annoList.add(anno);
		}

	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		this.grammaticType = editor.getCas().getTypeSystem().getType("de.uniwue.kallimachos.coref.type.GrammaticUnit");
		this.featPhraseType = this.grammaticType.getFeatureByBaseName("Phrasetype");
		this.featSemanticRole = this.grammaticType.getFeatureByBaseName("SemanticRole");
		this.featSyntacticRole = this.grammaticType.getFeatureByBaseName("SyntacticRole");
		this.featSyntacticHead = this.grammaticType.getFeatureByBaseName("SyntacticHead");
		this.featLayer = this.grammaticType.getFeatureByBaseName("Layer");
		this.featIsSelected = this.grammaticType.getFeatureByBaseName("IsSelected");
		// Visibility

		// layer
		editor.allStylesInvisible();

		editor.changeVisibility(grammaticType, true);
		editor.changeLayer(grammaticType, 0);

		// yellow
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle style2 = new AnnoStyle(new GrammarRealDrawingStrategy(), rangeOv);

		// apply
		editor.changeStyle(grammaticType, style2);

		// also change the modus
		editor.setActiveMode(grammaticType);

		return true;

	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

		this.defaultTopMargin = this.editor.getWidget().getTopMargin();
		this.editor.getWidget().setTopMargin(450);
		parent.setLayout(new GridLayout(3, false));

		GridData gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);

		parent.setLayoutData(gd);

		buttonAddGrammarUnit = new Button(parent, SWT.PUSH);

		buttonAddGrammarUnit.setText("Add Unit");
		buttonAddGrammarUnit.setEnabled(true);

		buttonAddGrammarUnit.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				Point selection = editor.getActualSelectedText();

				if (selection == null || selection.x == selection.y)
					return;

				AnnotationFS anno = editor.getCas().createAnnotation(grammaticType, selection.x, selection.y);

				// add quietly
				editor.addAnnotation(anno);
				updateModel();

				smallestSelection = anno;

			}

		});

		Button dummyButton = new Button(parent, SWT.PUSH);
		dummyButton.setVisible(false);

		buttonRemoveGrammarUnit = new Button(parent, SWT.PUSH);

		buttonRemoveGrammarUnit.setText("Remove Unit");
		buttonRemoveGrammarUnit.setEnabled(false);

		buttonRemoveGrammarUnit.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (smallestSelection != null) {

					editor.removeAnnotation(smallestSelection);
				}

			}

		});

		GridData gd2 = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		Label labelPhraseType = new Label(parent, SWT.NONE);
		labelPhraseType.setText("Phrasetype");

		textPhraseType = new Text(parent, SWT.BORDER);
		textPhraseType.setLayoutData(gd2);

		buttonSetPhraseType = new Button(parent, SWT.PUSH);
		buttonSetPhraseType.setText("Set");

		buttonSetPhraseType.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (smallestSelection == null)
					return;

				String phrase = textPhraseType.getText();

				// if (phrase.isEmpty())
				// return;

				smallestSelection.setFeatureValueFromString(featPhraseType, phrase);

				editor.changeAnnotation(smallestSelection);
			}

		});

		Label labelSyntacticRole = new Label(parent, SWT.NONE);
		labelSyntacticRole.setText("Syntactic Role");

		textSyntacticRole = new Text(parent, SWT.BORDER);
		textSyntacticRole.setLayoutData(gd2);

		buttonSetSyntacticRoleType = new Button(parent, SWT.PUSH);
		buttonSetSyntacticRoleType.setText("Set");

		buttonSetSyntacticRoleType.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (smallestSelection == null)
					return;

				String phrase = textSyntacticRole.getText();

				// if (phrase.isEmpty())
				// return;

				smallestSelection.setFeatureValueFromString(featSyntacticRole, phrase);

				editor.changeAnnotation(smallestSelection);
			}

		});

		Label labelSemanticRole = new Label(parent, SWT.NONE);
		labelSemanticRole.setText("Semantic Role");

		textSemanticRole = new Text(parent, SWT.BORDER);
		textSemanticRole.setLayoutData(gd2);

		buttonSetSemanticRoleType = new Button(parent, SWT.PUSH);
		buttonSetSemanticRoleType.setText("Set");

		buttonSetSemanticRoleType.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (smallestSelection == null)
					return;

				String phrase = textSemanticRole.getText();

				// if (phrase.isEmpty())
				// return;

				smallestSelection.setFeatureValueFromString(featSemanticRole, phrase);

				editor.changeAnnotation(smallestSelection);
			}

		});

		// disable wrapping
		editor.getWidget().setWordWrap(false);
	}

	// use this method to update the view
	public void renewPage() {

		// // redraw in a new Thread, executed by the Display
		// Runnable r = new Runnable() {
		//
		// @Override
		// public void run() {
		// createTableViewer();
		//
		// }
		// };
		//
		// // reload asynchronous
		// overlay.getDisplay().asyncExec(r);

	}

	private void updateModel() {
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(this.grammaticType);

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		updateLayerInformation();

		renewPage();

	}

	private void updateLayerInformation() {

		List<AnnotationFS> annos = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : this.annoList) {

			int layerIndex = getLayerIndexForAnno(anno);

			anno.setFeatureValueFromString(featLayer, String.valueOf(layerIndex));
			annos.add(anno);
		}

		// check for conflicts - those are possible with the current code

		while (checkForLayerConflicts()) {

			resolveLayerConflicts();
		}

		// bulk change - only reloads once
		// if (annos.size() > 0) {
		// editor.changeAnnotations(annos);
		// }

		// do not reload

	}

	private void resolveLayerConflicts() {

		for (AnnotationFS a : this.annoList) {

			for (AnnotationFS b : this.annoList) {

				if (a.getBegin() == b.getBegin())
					continue;

				// check if a smaller anno has the same layer
				if (a.getBegin() <= b.getBegin() && a.getEnd() >= b.getEnd()) {

					String layerA = a.getFeatureValueAsString(featLayer);
					String layerB = b.getFeatureValueAsString(featLayer);

					if (layerA.equals(layerB)) {

						// increase the layer of a by 1
						a.setFeatureValueFromString(featLayer, String.valueOf(Integer.parseInt(layerA) + 1));
						return;
					}

				}
			}
		}

	}

	private boolean checkForLayerConflicts() {

		for (AnnotationFS a : this.annoList) {

			for (AnnotationFS b : this.annoList) {

				if (a.getBegin() == b.getBegin())
					continue;

				// check if a smaller anno has the same layer
				if (a.getBegin() <= b.getBegin() && a.getEnd() >= b.getEnd()) {

					String layerA = a.getFeatureValueAsString(featLayer);
					String layerB = b.getFeatureValueAsString(featLayer);

					if (layerA.equals(layerB))
						return true;
				}
			}
		}
		return false;
	}

	private int getLayerIndexForAnno(AnnotationFS anno) {

		int layer = 0;

		AnnotationFS temp = anno;

		while (temp != null) {
			layer++;

			// get the child
			AnnotationFS cover = getCoveredAnno(temp);

			temp = cover;
		}
		return layer;
	}

	private AnnotationFS getCoveredAnno(AnnotationFS temp) {

		AnnotationFS biggestCovered = null;
		for (AnnotationFS a : this.annoList) {

			if (a.getBegin() == temp.getBegin() && a.getEnd() == temp.getEnd())
				continue;

			if (temp.getBegin() < a.getBegin() && temp.getEnd() >= a.getEnd() || temp.getBegin() <= a.getBegin()
					&& temp.getEnd() > a.getEnd()) {

				if (biggestCovered == null) {
					biggestCovered = a;
				}

				else {

					if (a.getEnd() - a.getBegin() > biggestCovered.getEnd() - biggestCovered.getBegin()) {
						biggestCovered = a;
					}
				}

			}
		}
		return biggestCovered;
	}

	@PreDestroy
	public void preDestroy() {
	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

		if (this.editor == null) {

			// request the editor if possible
			broker.send(IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR, this);
			super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr

	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications - yep this is important
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {

		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners

			// special for this view -> wrap the widget again
			this.editor.getWidget().setWordWrap(true);
			this.editor.getWidget().setTopMargin(defaultTopMargin);
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	// what to do when the editor has an added annotation?
	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		// AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);
		updateModel();
		renewPage();

	}

	// what to do when the editor has an removed annotation?
	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		// AnnotationFS removed = (AnnotationFS)
		// arg0.getProperty(IEventBroker.DATA);

		updateModel();

		renewPage();

	}

	// what to do when the editor has an changed annotation?
	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		// AnnotationFS changed = (AnnotationFS)
		// arg0.getProperty(IEventBroker.DATA);

		updateModel();

		renewPage();

	}

	// what to do when the editor caret was changed
	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {
		// unused

		// dont do anythign when the editor is in dragging mode
		if (editor.getMouseDrag())
			return;
		// deselect all others
		for (AnnotationFS a : this.annoList) {
			a.setFeatureValueFromString(featIsSelected, "");
		}

		smallestSelection = null;
		for (AnnotationFS sel : this.annoList) {

			if (!(sel.getBegin() <= this.editor.getWidget().getCaretOffset() && sel.getEnd() >= this.editor.getWidget()
					.getCaretOffset()))
				continue;

			if (smallestSelection == null) {
				smallestSelection = sel;
			}

			else {
				if (sel.getEnd() - sel.getBegin() < smallestSelection.getEnd() - smallestSelection.getBegin()) {
					smallestSelection = sel;
				}
			}

		}

		if (smallestSelection != null) {
			smallestSelection.setFeatureValueFromString(featIsSelected, "true");

			editor.changeAnnotation(smallestSelection);
		}

		if (smallestSelection == null) {

			buttonRemoveGrammarUnit.setEnabled(false);
		} else {
			buttonRemoveGrammarUnit.setEnabled(true);

			// also set the features in the textfields

			String phrase = smallestSelection.getFeatureValueAsString(featPhraseType);
			String syntactic = smallestSelection.getFeatureValueAsString(featSyntacticRole);
			String semantic = smallestSelection.getFeatureValueAsString(featSemanticRole);

			if (phrase != null) {
				textPhraseType.setText(phrase);
			} else {
				textPhraseType.setText("");
			}

			if (syntactic != null) {
				textSyntacticRole.setText(syntactic);
			}

			else {
				textSyntacticRole.setText("");
			}

			if (semantic != null) {
				textSemanticRole.setText(semantic);
			}

			else {
				textSemanticRole.setText("");
			}

		}
		editor.invokeReload();

	}

	// what to do when the editor was clicked
	@Override
	public void onEditorMouseUp(org.osgi.service.event.Event e) {

		Point p = (Point) e.getProperty(IEventBroker.DATA);

		int base = editor.getWidget().getTextBounds(0, 1).y;

		int layerHeight = 30;

		// 460 -430 means layer 1 ;430 -400 is layer 2 and so on ....

		int layerNr = (base - p.y) / layerHeight + 1;

		if (layerNr <= 1)
			return;

		// search for an annotation that spans the caret and has layer = layerNR

		int offset = editor.getWidget().getCaretOffset();
		AnnotationFS sel = null;
		for (AnnotationFS a : this.annoList) {

			if (a.getBegin() <= offset && a.getEnd() >= offset) {
				String layer = a.getFeatureValueAsString(featLayer);

				if (layer != null && layer.equals(String.valueOf(layerNr))) {

					sel = a;
					break;
				}
			}
		}

		if (sel != null) {
			// deselect all others
			for (AnnotationFS a : this.annoList) {
				a.setFeatureValueFromString(featIsSelected, "");
			}

			// update the selected anno !!!
			sel.setFeatureValueFromString(featIsSelected, "true");
			editor.changeAnnotation(sel);
			smallestSelection = sel;

			// also set the features in the textfields
			String phrase = sel.getFeatureValueAsString(featPhraseType);
			String syntactic = sel.getFeatureValueAsString(featSyntacticRole);
			String semantic = sel.getFeatureValueAsString(featSemanticRole);

			if (phrase != null) {
				textPhraseType.setText(phrase);
			} else {
				textPhraseType.setText("");
			}

			if (syntactic != null) {
				textSyntacticRole.setText(syntactic);
			}

			else {
				textSyntacticRole.setText("");
			}

			if (semantic != null) {
				textSemanticRole.setText(semantic);
			}

			else {
				textSemanticRole.setText("");
			}
		}

	}

}