package de.uniwue.mk.kall.ie.ieview.dialog;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLOntology;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;

public class BinaryRelationManyEntitiesSelectionDialog extends Dialog {

	private static final String BUTTON_DATA_ANNOTATION_FS = "ANNOTATION_FS";
	private boolean exchangeDomain;
	// UI
	private Button buttonDomain;
	private Button buttonRange;
	private Button buttonSwap;

	private Label labelDomain;
	private Label labelRange;
	private AnnotationFS domainAnno;
	private AnnotationFS rangeAnno;
	private AnnotationFS[] entities;

	public BinaryRelationManyEntitiesSelectionDialog(Shell parentShell, AnnotationFS... entities) {
		super(parentShell);
		parentShell.setText("Choose Domain and Range");

		if (entities.length >= 2) {
			this.domainAnno = entities[0];
			this.rangeAnno = entities[1];
		}
		this.entities = entities;
	}

	protected Control createDialogArea(Composite parent) {

		parent.setLayout(new GridLayout(1, true));

		Composite subComp1 = new Composite(parent, SWT.NONE);
		subComp1.setLayout(new GridLayout(this.entities.length, false));
		for (AnnotationFS entity : entities) {
			final Button buttonEnt = new Button(subComp1, SWT.PUSH);
			buttonEnt.setText(entity.getCoveredText());
			buttonEnt.setData(BUTTON_DATA_ANNOTATION_FS, entity);
			buttonEnt.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					updateSelection((AnnotationFS) buttonEnt.getData(BUTTON_DATA_ANNOTATION_FS));
				}

			});
		}

		Composite subComp = new Composite(parent, SWT.NONE);
		subComp.setLayout(new GridLayout(3, true));
		labelDomain = new Label(subComp, SWT.NONE);
		labelDomain.setText("Domain(from):");

		Label labelDummy = new Label(subComp, SWT.NONE);
		labelDummy.setVisible(false);

		labelRange = new Label(subComp, SWT.NONE);
		labelRange.setText("Range(to):");

		ISharedImages images = JavaUI.getSharedImages();
		Image image = images.getImage(ISharedImages.IMG_OBJS_INTERFACE);
		buttonDomain = new Button(subComp, SWT.FLAT);
		buttonDomain.setText(entities[0].getCoveredText());
		buttonDomain.setImage(image);

		buttonSwap = new Button(subComp, SWT.PUSH);
		buttonSwap.setText("swap");
		buttonSwap.setImage(ApplicationUtil.createImage(ApplicationUtil.IMG_SYNCHRONIZE));
		buttonSwap.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				AnnotationFS temp = domainAnno;
				domainAnno = rangeAnno;
				rangeAnno = temp;
				buttonDomain.setText(domainAnno.getCoveredText());
				buttonRange.setText(rangeAnno.getCoveredText());
			}

		});

		buttonRange = new Button(subComp, SWT.PUSH);
		buttonRange.setText(entities[1].getCoveredText());
		buttonRange.setImage(image);

		return parent;
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE | SWT.RESIZE);
		setBlockOnOpen(false);
	}

	public AnnotationFS getDomain() {
		return this.domainAnno;
	}

	public AnnotationFS getRange() {
		return this.rangeAnno;
	}

	private void updateSelection(AnnotationFS data) {
		if (exchangeDomain) {
			this.domainAnno = data;
			buttonDomain.setText(data.getCoveredText());
			exchangeDomain = !exchangeDomain;
		} else {
			this.rangeAnno = data;
			buttonRange.setText(data.getCoveredText());
			exchangeDomain = !exchangeDomain;
		}

	}

}
