package de.uniwue.mk.kall.ie.ieview.dialog;

import java.util.List;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.OwnerDrawLabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class ComplexClassComposite extends Composite {

	private OWLClass classWithAnnos;
	private IEOntology ontology;

	// UI Elements
	private Button buttonAdd;
	private TableViewer viewer;

	public ComplexClassComposite(Composite parent, int style) {
		super(parent, style);

		this.setLayout(new GridLayout(2, false));
		this.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));

		initLayout();
	}

	public void setInput(IEOntology ontology, OWLClass classWithAnnos) {
		this.classWithAnnos = classWithAnnos;
		this.ontology = ontology;
		// TODO set input
		updateTableInput();

	}

	private void initLayout() {

		initTable();

	}

	private void updateTableInput() {
		viewer.setInput(OWLUtil.getConstructionProperties(classWithAnnos, ontology));

	}

	private void initTable() {
		viewer = new TableViewer(this,
				SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER | SWT.FILL);

		GridData layoutDataTable = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		layoutDataTable.horizontalSpan = 6;
		layoutDataTable.heightHint = 200;
		viewer.getControl().setLayoutData(layoutDataTable);

		createColumns(viewer);

		// make lines and header visible
		final Table table2 = viewer.getTable();
		table2.setHeaderVisible(true);
		table2.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
	}

	private void createColumns(TableViewer viewer) {
		TableViewerColumn colAnnoProperty = createTableViewerColumn("Property", 100, 0);
		colAnnoProperty.setLabelProvider(new ColumnLabelProvider() {

			@Override
			public Image getImage(Object element) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_CLASSPATH_VAR_ENTRY);
				return image;
			}

			@Override
			public String getText(Object element) {
				OWLObjectProperty prop = (OWLObjectProperty) element;
				return element == null ? "" : prop.getIRI().getFragment();
			}

		});

		TableViewerColumn colAnnoConstruction = createTableViewerColumn("Construction", 300, 0);
		colAnnoConstruction.setLabelProvider(new OwnerDrawLabelProvider() {
			@Override
			protected void measure(Event event, Object element) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_CLASS);
				Rectangle rectangle = image.getBounds();
				event.setBounds(new Rectangle(event.x, event.y, rectangle.width + 200, rectangle.height));
			}

			@Override
			protected void paint(Event event, Object element) {
				Rectangle bounds = event.getBounds();
				OWLObjectProperty prop = (OWLObjectProperty) element;
				List<OWLClass> classesInRange = OWLUtil.getClassesInRange(prop, ontology.getOntology());

				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_CLASS);
				int offsetX = bounds.x + 5;
				int offSetY = bounds.y;
				for (OWLClass c : classesInRange) {
					event.gc.drawImage(image, offsetX, offSetY);
					Point point = event.gc.stringExtent(c.getIRI().getFragment());
					event.gc.drawText(c.getIRI().getFragment(), offsetX + image.getBounds().width + 3, offSetY);
					offsetX += point.x + 9 + image.getBounds().width;

				}
			}

		});
		
//		TableColumnLayout tableLayout = new TableColumnLayout();
//		this.setLayout(tableLayout);
//		   colAnnoConstruction.getColumn().pack();
//		    colAnnoProperty.getColumn().pack();
//		    // Use the packed widths as the minimum widths
//		    int stylesheetWidth = colAnnoConstruction.getColumn().getWidth();
//		    int conceptWidth = colAnnoProperty.getColumn().getWidth();
//		    // Set stylesheet column to fill 100% and concept column to fit 0%, but with their packed widths as minimums
//		    tableLayout.setColumnData(colAnnoConstruction.getColumn(), new ColumnWeightData(100, stylesheetWidth));
//		    tableLayout.setColumnData(colAnnoProperty.getColumn(), new ColumnWeightData(0, conceptWidth));

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// table.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
