package de.uniwue.mk.kall.ie.ieview.dialog;

import java.util.Iterator;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.OwnerDrawLabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;
import org.semanticweb.owlapi.model.OWLAnnotation;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLLiteral;
import org.semanticweb.owlapi.model.OWLOntology;

import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class OWLAnnotationEditComposite extends Composite {

	private OWLClass classWithAnnos;
	private OWLOntology ontology;

	// UI Elements
	private Label labelNewAnnotation;
	private ComboViewer comboViewer;
	private Label labelValue;
	private Text textValue;
	private Button buttonAdd;
	private TableViewer viewer;

	public OWLAnnotationEditComposite(Composite parent, int style) {
		super(parent, style);

		this.setLayout(new GridLayout(5, false));
		GridData layoutData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		this.setLayoutData(layoutData);

		initLayout();
	}

	public void setInput(OWLOntology ontology, OWLClass classWithAnnos) {
		this.classWithAnnos = classWithAnnos;
		this.ontology = ontology;
		// TODO set input
		updateTableInput();

	}

	private void updateTableInput() {
		viewer.setInput(OWLUtil.getAnnotationsOnEntity(classWithAnnos, ontology));

	}

	private void initLayout() {

		initButtons();

		initTable();

	}

	private void initTable() {
		viewer = new TableViewer(this,
				SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER | SWT.FILL);

		GridData layoutDataTable = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		layoutDataTable.horizontalSpan = 6;
		layoutDataTable.heightHint = 200;
		viewer.getControl().setLayoutData(layoutDataTable);

		createColumns(viewer);

		// make lines and header visible
		final Table table2 = viewer.getTable();
		table2.setHeaderVisible(true);
		table2.setLinesVisible(true);

		viewer.getTable().addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.DEL) {

					IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

					Iterator iterator = sel.iterator();
					
					while(iterator.hasNext()){
						Object next = iterator.next();
						
						if(next instanceof OWLAnnotation ){
							OWLAnnotation anno = (OWLAnnotation) next;
							
							OWLUtil.deleteAnnotationFromOntology(anno,classWithAnnos, ontology);
						}
					}
					// TODO delete the annotation
					updateTableInput();
				}

			}
		});

		// content
		viewer.setContentProvider(new ArrayContentProvider());
	}

	private void initButtons() {

		labelNewAnnotation = new Label(this, SWT.NONE);
		labelNewAnnotation.setText("New @nnotation:");

		comboViewer = new ComboViewer(this, SWT.NONE);
		comboViewer.setContentProvider(ArrayContentProvider.getInstance());
		comboViewer.setInput(ESupportedOWLAnnotations.values());
		comboViewer.setSelection(new StructuredSelection(ESupportedOWLAnnotations.Synonym));
		// TODO set current selection

		labelValue = new Label(this, SWT.NONE);
		labelValue.setText("Value:");

		textValue = new Text(this, SWT.BORDER);
		GridData layoutData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		textValue.setLayoutData(layoutData);

		buttonAdd = new Button(this, SWT.PUSH);
		buttonAdd.setText("Add");
		buttonAdd.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) comboViewer.getSelection();

				Object firstElement = sel.getFirstElement();
				if (firstElement instanceof ESupportedOWLAnnotations) {
					ESupportedOWLAnnotations eType = (ESupportedOWLAnnotations) firstElement;
					OWLAnnotation annotation = OWLUtil.createAnnotation(eType.toString(), textValue.getText(), ontology);

					if (annotation != null) {
						// we add the annotation to our class
						OWLUtil.addAnnotationToEntity(annotation, classWithAnnos, ontology);
						updateTableInput();
					}

				}
			}

		});
	}

	private void createColumns(TableViewer viewer) {
		TableViewerColumn colAnnoName = createTableViewerColumn("Type", 100, 0);
		colAnnoName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				OWLAnnotation p = (OWLAnnotation) element;
				return p.getProperty().getIRI().getFragment();

			}
			
			@Override
			public Image getImage(Object element) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_ANNOTATION);
				return image;

			}
			

		});

		TableViewerColumn colAnnoValue = createTableViewerColumn("Value", 300, 0);
		colAnnoValue.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				OWLAnnotation p = (OWLAnnotation) element;
				if (p.getValue() instanceof OWLLiteral) {
					return ((OWLLiteral) p.getValue()).getLiteral();
				}
				return "<UNKNOWN>";

			}
			



		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// table.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

}
