package de.uniwue.mk.kall.ie.ieview.dialog;

import java.beans.PropertyChangeSupport;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyPropertyWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class PropertyAssignClassComposite extends Composite {

	private OWLOntologyPropertyWrapper property;
	private IEOntology ontology;
	private String behaviourString;

	private TableViewer tableViewer;
	private Button buttonRemove;
	private Button buttonAdd;

	public PropertyAssignClassComposite(Composite parent, int style, String behaviourString) {
		super(parent, style);
		this.behaviourString = behaviourString;
		this.setLayout(new GridLayout(2, false));
		GridData gd2 = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_BOTH | GridData.GRAB_VERTICAL);
		this.setLayoutData(gd2);
		Composite leftComposite = new Composite(this, SWT.NONE);
		leftComposite.setLayout(new GridLayout(1, false));
		leftComposite.setLayoutData(gd2);

		Label labelBehaviour = new Label(leftComposite, SWT.NONE);
		labelBehaviour.setText(behaviourString.split("_")[0]);
		tableViewer = new TableViewer(leftComposite, SWT.BORDER);
		tableViewer.getControl().setLayoutData(gd2);

		tableViewer.setContentProvider(ArrayContentProvider.getInstance());
		tableViewer.setLabelProvider(new LabelProvider() {

			public String getText(Object element) {
				return ((OWLClass) element).getIRI().getFragment();
			}

			@Override
			public Image getImage(Object element) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_INNER_CLASS_PUBLIC);
				return image;
			}
		});

		Composite rightComposite = new Composite(this, SWT.NONE);
		rightComposite.setLayout(new FillLayout(SWT.VERTICAL));
		buttonRemove = new Button(rightComposite, SWT.PUSH);
		buttonRemove.setText("Remove");
		buttonRemove.setEnabled(false);
		buttonRemove.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				removeSelectedClasses(behaviourString);
			}



		});

		buttonAdd = new Button(rightComposite, SWT.PUSH);
		buttonAdd.setText("Add");
		buttonAdd.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// open a dialog that carries a multiselectiontree with classes
				SelectClassesDialog dialog = new SelectClassesDialog(Display.getCurrent().getActiveShell(), ontology);
				int open = dialog.open();

				if (open == Dialog.OK) {
					List<OWLClass> selectedClasses = dialog.getSelectedClasses();
					for (OWLClass cl : selectedClasses) {
						if (behaviourString.equals(DomainAndRangeComposite.DOMAIN_BEHAVIOUR)) {
							OWLUtil.addClassToDomain(property.getWrappedProperty(), ontology.getOntology(), cl);
							refreshGranparent();
						} else if (behaviourString.equals(DomainAndRangeComposite.RANGE_BEHAVIOUR)) {
							OWLUtil.addClassToRange(property.getWrappedProperty(), ontology.getOntology(), cl);
							refreshGranparent();
						}

					}
					// refresh
					setInput(property, ontology);
				}

			}



		});

		tableViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				ISelection selection = tableViewer.getSelection();
				if (selection.isEmpty()) {
					buttonRemove.setEnabled(false);
				} else {
					buttonRemove.setEnabled(true);
				}
			}
		});
		
		tableViewer.getTable().addKeyListener(new KeyAdapter() {
			
			@Override
			public void keyReleased(KeyEvent e) {

				if(e.character==SWT.DEL){
					removeSelectedClasses(behaviourString);
				}
			}
			
	
		});
	}

	public void setInput(OWLOntologyPropertyWrapper property, IEOntology ontology) {
		this.property = property;
		this.ontology = ontology;

		// TODO get all classes in range or domain
		if (this.behaviourString.equals(DomainAndRangeComposite.DOMAIN_BEHAVIOUR)) {
			List<OWLClass> classesInDomain = OWLUtil.getClassesInDomain(property.getWrappedProperty(), ontology.getOntology());
			tableViewer.setInput(classesInDomain);

		}

		else if (this.behaviourString.equals(DomainAndRangeComposite.RANGE_BEHAVIOUR)) {
			List<OWLClass> classesInDomain = OWLUtil.getClassesInRange(property.getWrappedProperty(), ontology.getOntology());
			tableViewer.setInput(classesInDomain);
		}
		updateLayout();

	}

	private void updateLayout() {
		tableViewer.refresh();

	}
	
	private void removeSelectedClasses(String behaviourString) {
		IStructuredSelection sel = (IStructuredSelection) tableViewer.getSelection();

		Iterator<?> iterator = sel.iterator();
		while (iterator.hasNext()) {
			Object next = iterator.next();
			if (next instanceof OWLClass) {
				OWLClass owlClass = (OWLClass) next;

				if (behaviourString.equals(DomainAndRangeComposite.RANGE_BEHAVIOUR)) {
					OWLUtil.removeClassFromRange(property.getWrappedProperty(), owlClass, ontology.getOntology());
				}
				else if(behaviourString.equals(DomainAndRangeComposite.DOMAIN_BEHAVIOUR)){
					OWLUtil.removeClassFromDomain(property.getWrappedProperty(),owlClass,ontology.getOntology());
				}
			}
		}
		// refresh
		refreshGranparent();
		//setInput(property, ontology);
	}
	
	//this method is a workaround so that i can update the characteristics when changes occur here
	private void refreshGranparent() {
		
		Composite parent2 = getParent().getParent();
		if(parent2 instanceof PropertyEditingComposite){
			PropertyEditingComposite grandfather = (PropertyEditingComposite) parent2;
			grandfather.setInput(property, ontology);
		}
		
	}

}
