package de.uniwue.mk.kall.ie.ieview.dialog;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.semanticweb.owlapi.model.OWLObjectProperty;
import org.semanticweb.owlapi.model.OWLOntology;

import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class PropertyCharacteristicsComposite extends Composite {

	private Button buttonIsFunctional;
	private Button buttonIsInverseFunctional;
	private Button buttonIsTransitive;
	private Button buttonIsSymmetric;
	private Button buttonIsAsymmetric;
	private Button buttonIsReflexive;
	private Button buttonisIrreflexive;
	private Button buttonIsConstructionProperty;

	// the model
	private OWLObjectProperty property;
	private OWLOntology ontology;

	public PropertyCharacteristicsComposite(Composite parent, int style) {
		super(parent, style);

		this.setLayout(new GridLayout(4, false));
		GridData layoutData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		this.setLayoutData(layoutData);
		initLayout();
	}

	private void initLayout() {
		buttonIsFunctional = new Button(this, SWT.CHECK);
		buttonIsInverseFunctional = new Button(this, SWT.CHECK);
		buttonIsTransitive = new Button(this, SWT.CHECK);
		buttonIsSymmetric = new Button(this, SWT.CHECK);
		buttonIsAsymmetric = new Button(this, SWT.CHECK);
		buttonIsReflexive = new Button(this, SWT.CHECK);
		buttonisIrreflexive = new Button(this, SWT.CHECK);
		buttonIsConstructionProperty = new Button(this, SWT.CHECK);

		buttonIsFunctional.setText("Functional");
		buttonIsInverseFunctional.setText("Inverse Functional");
		buttonIsTransitive.setText("Transitive");
		buttonIsSymmetric.setText("Symmetric");
		buttonIsAsymmetric.setText("Asymmetric");
		buttonIsReflexive.setText("Reflexive");
		buttonisIrreflexive.setText("Irreflexive");
		// this is saved as an annotation
		buttonIsConstructionProperty.setText("Construction");

		// listener
		buttonIsFunctional.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIsFunctional(property, ontology, buttonIsFunctional.getSelection());

			}

		});
		buttonIsInverseFunctional.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIsInverseFunctionalProperty(property, ontology, buttonIsInverseFunctional.getSelection());

			}

		});
		buttonIsTransitive.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIsTransitiveProperty(property, ontology, buttonIsTransitive.getSelection());

			}

		});
		buttonIsSymmetric.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIsSymmetricProperty(property, ontology, buttonIsSymmetric.getSelection());

			}

		});
		buttonIsAsymmetric.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIsAsymmetricProperty(property, ontology, buttonIsAsymmetric.getSelection());

			}

		});
		buttonIsReflexive.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setReflexiveProperty(property, ontology, buttonIsReflexive.getSelection());

			}

		});
		buttonisIrreflexive.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setIrreflexiveProperty(property, ontology, buttonisIrreflexive.getSelection());

			}

		});
		// this is saved as an annotation
		buttonIsConstructionProperty.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				OWLUtil.setConstructionProperty(property, ontology, buttonIsConstructionProperty.getSelection());

			}

		});
	}

	public void setInput(OWLObjectProperty property, OWLOntology ontology) {
		this.property = property;
		this.ontology = ontology;
		updateLayout();
	}

	public void updateLayout() {

		buttonIsConstructionProperty.setEnabled(true);
		boolean isConstructionPropety = OWLUtil.isConstructionProperty(property, ontology);
		buttonIsConstructionProperty.setSelection(isConstructionPropety);

		// assure that this field cannot be removed when we have a multirange
		// property
		boolean isDefinitiveConstructionProperty = OWLUtil.isDefiniteConstructionProperty(property, ontology);
		if (isDefinitiveConstructionProperty) {
			buttonIsConstructionProperty.setSelection(true);
			buttonIsConstructionProperty.setEnabled(false);
		}

		if (OWLUtil.getClassesInDomain(property, ontology).size() != 1
				|| OWLUtil.getClassesInRange(property, ontology).size() == 0) {
			buttonIsConstructionProperty.setEnabled(false);
		}
		buttonIsFunctional.setSelection(property.isFunctional(ontology));
		buttonIsInverseFunctional.setSelection(property.isInverseFunctional(ontology));
		buttonIsTransitive.setSelection(property.isTransitive(ontology));
		buttonIsSymmetric.setSelection(property.isSymmetric(ontology));
		buttonIsAsymmetric.setSelection(property.isAsymmetric(ontology));
		buttonIsReflexive.setSelection(property.isReflexive(ontology));
		buttonisIrreflexive.setSelection(property.isIrreflexive(ontology));

		layout();

	}

}
