
package de.uniwue.mk.kall.ie.ieview.ui;

import java.io.File;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;
import javax.xml.bind.JAXB;

import org.apache.uima.cas.CASException;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.osgi.service.event.Event;
import org.semanticweb.owlapi.model.OWLOntology;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.IEViewEntityDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.IEViewRelationDrawingStrategy;
import de.uniwue.mk.kall.ie.ieview.widget.IEOntologyTreeComposite;
import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class IEView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<IEView> CLASS_THIS = IEView.class;

	@Inject
	static EPartService partService;
	
	public static final String ONTOLOGY_BASE_IRI = "http://www.semanticweb.org/uniwue/athen/ontologies/";

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.ieview";

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	// UI
	private Button buttonCreateTerminology;
	private Button buttonLoadTerminology;
	private Button buttonSaveTerminology;
	private Button buttonApplyTerminology;
	private Button buttonUseAutosave;

	// the ontology editor
	private IEOntologyTreeComposite tree;
	// helper
	private String openedTerminologyPath;
	private boolean useAutosave;
	private Util_impl util;
	private Thread autoSaveThread;

	// model
	private IEOntology ontology;

	@Inject
	public IEView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {
		// layout this should stay as it is
		super.overlay = parent;

		// init the part, so that this view knows its position globally
		// (required to see if it has focus)
		super.partService = partService;
		super.setBroker(broker);
		super.part = partService.findPart(viewId);
	}

	@PreDestroy
	public void preDestroy() {

	}

	@Focus
	public void onFocus() {
		super.onFocus();
		
		if(ontology!=null && !tree.isDisposed()){
			tree.setOntology(ontology);

			// also save it once !! TODO
			renewPage();
		}
	}

	@Persist
	public void save() {

	}

	@Override
	public void initModel() {
		// TODO Auto-generated method stub

	}

	@Override
	public boolean initTypes() {
		// set the util, update for every document change
		this.util = new Util_impl(super.editor.getCas());
		// set the relevant types to visible
		editor.changeVisibility(util.getIEEntityGoldType(), true);
		editor.changeVisibility(util.getIERelationGoldType(), true);
		
		//TODO do we need any layer?
		
		//create the style
		StyleRange rangeTempAtt = new StyleRange();
		rangeTempAtt.background = new Color(overlay.getDisplay(), 156, 151, 17);
		AnnoStyle styleTempAtt = new AnnoStyle(new IEViewEntityDrawingStrategy(rangeTempAtt), rangeTempAtt);
		
		editor.changeStyle(util.getIEEntityGoldType(), styleTempAtt);
		
		
		//style for IERelation
		StyleRange rangeIERel = new StyleRange();
		rangeIERel.background = new Color(overlay.getDisplay(), 156, 151, 17);
		AnnoStyle styleIERel = new AnnoStyle(new IEViewRelationDrawingStrategy(rangeIERel), rangeIERel);
		
		editor.changeStyle(util.getIERelationGoldType(), styleIERel);
		

		return true;
	}

	@Override
	protected void initLayout(Composite parent) {
		parent.setLayout(new GridLayout(3, false));

		GridData gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);
		parent.setLayoutData(new GridData(GridData.GRAB_VERTICAL|GridData.FILL_VERTICAL));
		parent.setLayoutData(gd);

		// button composite
		initButtonComposite(parent);

		tree = new IEOntologyTreeComposite(parent, SWT.NONE,editor);
		GridData gdTree = new GridData(GridData.GRAB_HORIZONTAL|GridData.GRAB_VERTICAL|GridData.FILL_BOTH);
		gdTree.horizontalSpan = 3;
	
		tree.setLayoutData(gdTree);
		
		if(this.ontology!=null){
			tree.setOntology(ontology);
			renewPage();
		}


	}

	private void initButtonComposite(Composite parent) {
		buttonCreateTerminology = new Button(parent, SWT.PUSH);

		buttonCreateTerminology.setText("Create Terminology");

		buttonCreateTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "", "Enter a terminology name",
						"", null);
				if (dlg.open() == Window.OK) {
					String ontIri = ONTOLOGY_BASE_IRI +dlg.getValue();
					
					OWLOntology ont =OWLUtil.createOntology(ontIri);
					
					if(ont!=null){
						ontology = new IEOntology(ont);
						updateOntology();
					}
					
				}

			}

		});

		buttonLoadTerminology = new Button(parent, SWT.PUSH);

		buttonLoadTerminology.setText("Load Terminology");

		buttonLoadTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				FileDialog dlg = new FileDialog(new Shell(), SWT.OPEN);
				dlg.setText("Open");
				String path = dlg.open();
				if (path == null)
					return;

				openedTerminologyPath = path;

				ontology = IEOntology.loadOntologyFromFile(openedTerminologyPath);
				updateOntology();
				// TODO add the loaded terminology to the widget
				
				

				// also save it once !! TODO
				renewPage();

			}



		});

		buttonSaveTerminology = new Button(parent, SWT.PUSH);

		buttonSaveTerminology.setText("Save Terminology");

		buttonSaveTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (ontology == null)
					return;

				FileDialog dlg = new FileDialog(new Shell(), SWT.SAVE);
				dlg.setText("Save Terminology");
				dlg.setFileName(OWLUtil.getOntologyIRI(ontology.getOntology()).getFragment());
				String[] filterExt = { "*.owl", "*.txt", "*.png" };
				dlg.setFilterExtensions(filterExt);
				openedTerminologyPath = dlg.open();

				if (openedTerminologyPath == null || dlg.getFilterIndex() == -1)
					return;

				if (dlg.getFilterIndex() == 1) {
					// Save as text

					if (openedTerminologyPath.endsWith(".txt")) {
						openedTerminologyPath += ".txt";
					}
					// TODO yet unsupported

				}

				else if (dlg.getFilterIndex() == 2) {

					// save as PNG
					if (!openedTerminologyPath.endsWith(".png")) {
						openedTerminologyPath += ".png";
					}

					// TODO unsupported yet
				}

				else if (dlg.getFilterIndex() == 0) {

					// save as xml
					if (!openedTerminologyPath.endsWith(".owl")) {
						openedTerminologyPath += ".owl";
					}
					IEOntology.saveOntologyToFile(openedTerminologyPath, ontology);

				}

			}

		});

		buttonApplyTerminology = new Button(parent, SWT.PUSH);

		buttonApplyTerminology.setText("Apply Terminology");

		buttonApplyTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// TODO

			}

		});

		buttonUseAutosave = new Button(parent, SWT.CHECK);
		buttonUseAutosave.setText("Use Autosave?:");
		buttonUseAutosave.setSelection(true);
		buttonUseAutosave.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IEView.this.useAutosave = buttonUseAutosave.getSelection();

				if (useAutosave) {
					IEView.this.autoSaveThread.start();
				}

				else {
					if (IEView.this.autoSaveThread != null) {
						IEView.this.autoSaveThread.stop();
					}
				}
			}

		});

		IEView.this.autoSaveThread = new Thread(new Runnable() {

			@Override
			public void run() {

				while (true) {
					try {
						Thread.sleep(5 * 1000 * 60);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

					if (openedTerminologyPath == null)
						continue;
					String autosavePath = openedTerminologyPath.replaceAll("\\.xml", "").replaceAll("\\.txt", "");

					//JAXB.marshal(terminology, new File(autosavePath + "_autosave.xml"));

				}
			}
		});

		IEView.this.autoSaveThread.start();
	}

	private void renewPage() {
		super.overlay.layout();

	}
	@Override
	public void onEditorTextSelectionChanged(Event arg0) {

		this.tree.setEditorTextSelection(editor.getWidget().getSelection());
	}
	
	private void updateOntology() {
		//we need to change the tree obviously
		tree.setOntology(ontology);
		
	}


}