package de.uniwue.mk.kall.ie.ieview.widget;

import org.eclipse.jdt.ui.ISharedImages;
import org.eclipse.jdt.ui.JavaUI;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Image;

import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyClassWrapper;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyPropertyWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class IEOntologyLabelProvider extends StyledCellLabelProvider {

	private IEOntology ontology;

	public IEOntologyLabelProvider(IEOntology ontology) {
		this.ontology = ontology;
	}

	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof OWLOntologyClassWrapper) {

			OWLOntologyClassWrapper owlClassWrapper = (OWLOntologyClassWrapper) element;

			// ROOT Node
			if (owlClassWrapper.getWrappedClass() == null) {
				text.append("Thing");
			} else {
				text.append(owlClassWrapper.getWrappedClass().getIRI().getFragment());
			}

			// set image
			// TODO the whole error handling
			if (OWLUtil.getConstructionProperties(owlClassWrapper.getWrappedClass(), ontology).size() > 0) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_INNER_CLASS_PROTECTED);
				cell.setImage(image);
			} else {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_INNER_CLASS_PUBLIC);
				cell.setImage(image);
			}
		}

		else if (element instanceof OWLOntologyPropertyWrapper) {

			OWLOntologyPropertyWrapper owlPropetyWrapper = (OWLOntologyPropertyWrapper) element;

			// ROOT Node
			if (owlPropetyWrapper.getWrappedProperty() == null) {
				text.append("ROOT");
			} else {
				text.append(owlPropetyWrapper.getWrappedProperty().getIRI().getFragment());
			}

			// set image - construction property
			if (OWLUtil.isConstructionProperty(owlPropetyWrapper.getWrappedProperty(), ontology.getOntology())
					|| OWLUtil.isDefiniteConstructionProperty(owlPropetyWrapper.getWrappedProperty(),
							ontology.getOntology())) {
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_IMPCONT);
				cell.setImage(image);
			}
			else{
				//regular property
				ISharedImages images = JavaUI.getSharedImages();
				Image image = images.getImage(ISharedImages.IMG_OBJS_CLASSPATH_VAR_ENTRY);
				cell.setImage(image);
			}
		}

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	public void setOntology(IEOntology ontology) {
		this.ontology = ontology;
		
	}

}
