package de.uniwue.mk.kall.ie.ieview.widget;

import java.util.LinkedList;
import java.util.List;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyClassWrapper;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyPropertyWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class OWLEntitiyTreeFilter extends ViewerFilter {

	private IEOntology ontology;

	public OWLEntitiyTreeFilter(IEOntology ontology) {
		this.ontology = ontology;
	}

	private String searchString;
	private List<AnnotationFS> selectedEntities;

	public void setSearchText(String s) {
		// ensure that the value can be used for matching
		this.searchString = "" + s + ".*";
	}

	@Override
	public boolean select(Viewer viewer, Object parentElement, Object element) {

		boolean searchStringFiltering = getSearchStringFiltering(element);

		// now get the selection filtering TODO do this when you are able to...
		boolean selectionFilter = getSelectionFiltering(element);

		if (searchStringFiltering == false || selectionFilter == false)
			return false;
		return true;
	}

	private boolean getSelectionFiltering(Object element) {

		if (this.selectedEntities == null || this.selectedEntities.size() == 0)
			return true;

		// only leave those that are not applicable
		List<String> iriSetToCheck = new LinkedList<>();
		for (AnnotationFS anno : selectedEntities) {
			iriSetToCheck
					.add(anno.getFeatureValueAsString(new Util_impl(anno.getCAS()).getIEEntityGoldCategoryFeature()));
		}

		if (element instanceof OWLOntologyPropertyWrapper) {
			OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) element;
			// we need all IRIs for this
			// do not surpress the root
			if (wrapper.getWrappedProperty() == null
					|| wrapper.getWrappedProperty().equals(OWLUtil.getOWLTopObjectProperty(ontology.getOntology())))
				return true;

			boolean propertyRemains = decideForProperty(wrapper.getWrappedProperty(), iriSetToCheck);

			if (propertyRemains)
				return propertyRemains;

			// a node remains if one of his successors matches the query
			List<OWLObjectProperty> allSubsumedProperties = OWLUtil
					.getAllSubsumedProperties(wrapper.getWrappedProperty(), ontology.getOntology());

			for (OWLObjectProperty c : allSubsumedProperties) {
				if (decideForProperty(c, iriSetToCheck))
					return true;
			}
		}

		return false;
	}

	private boolean decideForProperty(OWLObjectProperty property, List<String> iriSetToCheck) {
		List<OWLClass> assignedClasses = OWLUtil.getClassesInDomain(property, ontology.getOntology());
		
		//is this required?
//		if (OWLUtil.isConstructionProperty(property, ontology.getOntology())
//				|| OWLUtil.isDefiniteConstructionProperty(property, ontology.getOntology())) {
//			assignedClasses.clear();
//		}
		assignedClasses.addAll(OWLUtil.getClassesInRange(property, ontology.getOntology()));

		// a simple check is that we need to have the same amount of arguments
		if (iriSetToCheck.size() < (assignedClasses.size()))
			return false;
		// unfinished property!
		if (assignedClasses.size() == 0)
			return false;

		// now we can check
		int matchedClasses = 0;
		for (String iriToCheck : iriSetToCheck) {

			// we need to assure that we have at least one parentclass or
			// matching class in assignedClass
			boolean matchFound = false;
			for (OWLClass clazz : assignedClasses) {
				boolean subClassOf = OWLUtil.isSubClassOf(OWLUtil.getClassByIRI(iriToCheck, ontology.getOntology()),
						clazz, ontology.getOntology());

				if (subClassOf || clazz.getIRI().getFragment().equals(iriToCheck)) {
					matchFound = true;
					break;
				}
			}
			if (matchFound) {
				matchedClasses++;
			}
		}

		if (matchedClasses < assignedClasses.size()) {
			return false;
		}

		return true;
	}

	private boolean getSearchStringFiltering(Object element) {
		if (searchString == null || searchString.length() == 0) {
			return true;
		}
		if (element instanceof OWLOntologyClassWrapper) {
			OWLOntologyClassWrapper wrapper = (OWLOntologyClassWrapper) element;
			// do not surpress the root
			if (wrapper.getWrappedClass() == null
					|| wrapper.getWrappedClass().equals(OWLUtil.getOWLThing(ontology.getOntology())))
				return true;

			// if the classname matches the query
			if (wrapper.getWrappedClass().getIRI().getFragment().matches(searchString))
				return true;

			// a node remains if one of his successors matches the query
			List<OWLClass> allSubsumedClasses = OWLUtil.getAllSubsumedClasses(wrapper.getWrappedClass(),
					ontology.getOntology());

			for (OWLClass c : allSubsumedClasses) {
				if (c.getIRI().getFragment().matches(searchString))
					return true;
			}

		}

		if (element instanceof OWLOntologyPropertyWrapper) {
			OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) element;

			// do not surpress the root
			if (wrapper.getWrappedProperty() == null
					|| wrapper.getWrappedProperty().equals(OWLUtil.getOWLTopObjectProperty(ontology.getOntology())))
				return true;

			// if the classname matches the query
			if (wrapper.getWrappedProperty().getIRI().getFragment().matches(searchString))
				return true;

			// a node remains if one of his successors matches the query
			List<OWLObjectProperty> allSubsumedProperties = OWLUtil
					.getAllSubsumedProperties(wrapper.getWrappedProperty(), ontology.getOntology());

			for (OWLObjectProperty c : allSubsumedProperties) {
				if (c.getIRI().getFragment().matches(searchString))
					return true;
			}
		}

		return false;
	}

	public void setEditorSelection(List<AnnotationFS> selectedEntities) {
		this.selectedEntities = selectedEntities;

	}

	public void setOntology(IEOntology ontology) {
		this.ontology = ontology;

	}
}
