package de.uniwue.mk.kall.ie.ieview.widget;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Text;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.ie.ieview.dialog.OWLClassWrapperEditDialog;
import de.uniwue.mk.kall.ie.ieview.widget.dragndrop.TreeHierarchyViewerDragListener;
import de.uniwue.mk.kall.ie.ieview.widget.dragndrop.TreeHierarchyViewerDropListener;
import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyClassWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class TreeHierarchyComposite extends Composite {

	private IEOntology ontology;

	// UI
	private Label labelName;
	private Button buttonAddSibling;
	private Button buttonAddChild;
	private Button buttonDeleteNode;
	private Button buttonEditNode;

	private TreeViewer viewer;

	// UI helper
	private OWLEntitiyTreeFilter filter;
	private TreeViewerAlphabeticalSorter treeSorter;
	private AnnotationEditorWidget editor;

	private IEOntologyLabelProvider labelProvider;

	private TreeHierarchyViewerDropListener dropListener;

	public TreeHierarchyComposite(Composite parent, int style, AnnotationEditorWidget editor) {
		super(parent, style);
		this.setLayout(new GridLayout(5, false));
		this.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH));
		initTreeHeader();
		this.editor = editor;

	}

	private void initTreeHeader() {

		labelName = new Label(this, SWT.NONE);
		labelName.setText("Class Hierarchie");
		buttonAddChild = new Button(this, SWT.NONE);
		buttonAddChild.setText("Add Child");

		buttonAddChild.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				if (sel.getFirstElement() instanceof OWLOntologyClassWrapper) {
					OWLOntologyClassWrapper wrapper = (OWLOntologyClassWrapper) sel.getFirstElement();

					// TODO do not create classes with the same IRI twice!
					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "", "Enter a class name",
							"", null);
					if (dlg.open() == Window.OK) {

						OWLClass newClass = OWLUtil.createNewClass(ontology, dlg.getValue());

						// we need to add this class to our ontology in the
						// given hierarchy
						OWLUtil.addEntityToOntology(newClass, ontology);
						OWLUtil.assignAsSubclass(wrapper.getWrappedClass(), newClass, ontology);

						// add to expaneded elements TODO
						if (wrapper.getWrappedClass() != null) {
							viewer.setExpandedState(wrapper.getWrappedClass(), true);
						}

						// define the according subclass axiom

						// refresh the parent
						refreshParent();
					}
				}
			}

		});

		buttonAddSibling = new Button(this, SWT.NONE);
		buttonAddSibling.setText("Add Sibling");

		buttonAddSibling.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof OWLOntologyClassWrapper) {
					OWLOntologyClassWrapper wrapper = (OWLOntologyClassWrapper) sel.getFirstElement();
					// get the parent
					OWLOntologyClassWrapper parent = wrapper.getParent();
					if (parent == null)
						return;

					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "", "Enter a class name",
							"", null);
					if (dlg.open() == Window.OK) {

						OWLClass newClass = OWLUtil.createNewClass(ontology, dlg.getValue());

						// we need to add this class to our ontology in the
						// given hierarchy
						OWLUtil.addEntityToOntology(newClass, ontology);
						OWLUtil.assignAsSubclass(parent.getWrappedClass(), newClass, ontology);
						viewer.setExpandedState(parent.getWrappedClass(), true);

						// define the according subclass axiom

						// refresh the parent
						refreshParent();
					}
				}
			}

		});

		buttonDeleteNode = new Button(this, SWT.NONE);
		buttonDeleteNode.setText("Delete");

		buttonDeleteNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof OWLOntologyClassWrapper) {

					OWLOntologyClassWrapper wrapper = (OWLOntologyClassWrapper) sel.getFirstElement();
					deleteClass(wrapper);
				}

			}

		});

		buttonEditNode = new Button(this, SWT.NONE);
		buttonEditNode.setText("Edit");

		buttonEditNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof OWLOntologyClassWrapper) {

					OWLOntologyClassWrapper wrapper = (OWLOntologyClassWrapper) sel.getFirstElement();
					editClass(wrapper);
				}
			}

		});

		// create search field
		Label labelSearch = new Label(this, SWT.NONE);
		labelSearch.setText("Search: ");

		Text textSearch = new Text(this, SWT.BORDER);
		GridData gdText = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		gdText.horizontalSpan = 4;
		textSearch.setLayoutData(gdText);

		textSearch.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {

				filter.setSearchText(textSearch.getText());
				viewer.refresh();
			}

		});

	}

	public void setOntology(IEOntology ontology) {
		this.ontology = ontology;
		if (labelProvider != null) {
			labelProvider.setOntology(ontology);
		}
		if (filter != null) {
			filter.setOntology(ontology);
		}
		if (dropListener != null) {
			dropListener.setOntology(ontology);
		}
		updateLayout();

	}

	private void updateLayout() {

		// dispose the tree if it was already presentt
		if (viewer == null) {
			createTreeViewerAndAssignListener();
			// set the input
			if (ontology != null) {
				viewer.setInput(ontology);
			}
		} else {
			// just adapt the input, set the input a new and keep the expanded
			// state
			// viewer.
			Object[] expandedElements = viewer.getExpandedElements();
			viewer.setInput(ontology);
			viewer.setExpandedElements(expandedElements);
			viewer.refresh();
		}

		layout();
		getParent().layout();
	}

	private void createTreeViewerAndAssignListener() {
		// we create the tree
		createViewer();

		addKeyListener();

		// create menumanager
		addMenu();

		enableDragAndDrop();

		addSelectionListener();

		addMouseListener();

	}

	private void addMouseListener() {
		viewer.getTree().addMouseListener(new MouseAdapter() {

			@Override
			public void mouseUp(MouseEvent e) {

				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));
				if (cell == null)
					return;
				Object element = cell.getElement();

				// if (element instanceof Concept) {
				// lastSelectedConcept = (Concept) element;
				// }
			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				// if we do not get a wrapped class object then we do nothing
				if (!(sel.getFirstElement() instanceof OWLOntologyClassWrapper)) {
					return;
				}
				OWLOntologyClassWrapper wrappedClass = (OWLOntologyClassWrapper) sel.getFirstElement();
				if (wrappedClass.getClass() == null)
					return;
				// get the selection of the editor
				if (editor != null) {

					Point actualSelectedText = editor.getActualSelectedText();
					Util_impl util = new Util_impl(editor.getCas());

					if (actualSelectedText.x == actualSelectedText.y) {
						// check if there is a surrounding entity
						List<AnnotationFS> currentSelectionOfType = editor
								.getCurrentSelectionOfType(util.getIEEntityGoldType());
						if (currentSelectionOfType.size() != 0) {
							// find the smallest annotation TODO
							AnnotationFS annotationFS = currentSelectionOfType.get(0);
							// we change the category feature to the name of the
							// class
							annotationFS.setFeatureValueFromString(util.getIEEntityGoldCategoryFeature(),
									wrappedClass.getWrappedClass().getIRI().getFragment());
							// change the annotation
							editor.changeAnnotation(annotationFS);
						} else {

							// we have no real selection => we edit the selected
							// class
							editClass(wrappedClass);
						}

					} else {
						// now we have a real selection and we create a new
						// annotation
						AnnotationFS entity = editor.getCas().createAnnotation(util.getIEEntityGoldType(),
								actualSelectedText.x, actualSelectedText.y);
						entity.setFeatureValueFromString(util.getIEEntityGoldCategoryFeature(),
								wrappedClass.getWrappedClass().getIRI().getFragment());

						// either we need to create a complex annotation (which
						// does only differ in a given string feature)
						// or we can leave the string blank
						List<AnnotationFS> selectedEntities = getEditorSelectedEntities();
						if (selectedEntities != null) {
							List<OWLObjectProperty> constructionProperties = OWLUtil
									.getConstructionProperties(wrappedClass.getWrappedClass(), ontology);

							// now we need to check if one of the construction
							// properties is applicable
							for (OWLObjectProperty objProperty : constructionProperties) {
								if (constructionPropertyIsApplicable(objProperty, selectedEntities)) {
									// we set the string
									entity.setFeatureValueFromString(util.getIEEntityGoldConstructedByFeature(),
											objProperty.getIRI().getFragment());
									break;
								}
							}
						}

						editor.addAnnotation(entity);

					}

				} else {
					// we just open the dialog to edit the selected node
					editClass(wrappedClass);
				}

			}

		});
	}

	// TODO only exact IRI matches??? currently i think this is the only
	// reasonable way
	private boolean constructionPropertyIsApplicable(OWLObjectProperty objProperty,
			List<AnnotationFS> selectedEntities) {
		List<String> iriSetToMatch = new LinkedList<>();
		for (AnnotationFS anno : selectedEntities) {
			iriSetToMatch
					.add(anno.getFeatureValueAsString(new Util_impl(anno.getCAS()).getIEEntityGoldCategoryFeature()));
		}

		List<OWLClass> classesInRange = OWLUtil.getClassesInRange(objProperty, ontology.getOntology());
		List<String> iriSet = new LinkedList<String>();

		for (OWLClass c : classesInRange) {
			iriSet.add(c.getIRI().getFragment());
		}

		// if the sizes are not even equal then we cannot have a match
		if (iriSet.size() != iriSetToMatch.size())
			return false;

		// now we need to assure that every iri from itiToMatch is in iriSet
		for (String iriToMatch : iriSetToMatch) {
			boolean subClassFound = false;
			for (OWLClass potentialParent : classesInRange) {
				if (OWLUtil.isSubClassOf(OWLUtil.getClassByIRI(iriToMatch, ontology.getOntology()), potentialParent,
						ontology.getOntology())) {
					subClassFound = true;
					break;
				}
				if (potentialParent.getIRI().getFragment().equals(iriToMatch)) {
					subClassFound = true;
					break;
				}
			}
			if (!subClassFound)
				return false;
		}

		return true;
	}

	private List<AnnotationFS> getEditorSelectedEntities() {

		// get the current selection of the editor
		Util_impl util = new Util_impl(editor.getCas());
		Point selection = editor.getWidget().getSelection();
		List<AnnotationFS> annoSelection = new ArrayList<AnnotationFS>();

		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(util.getIEEntityGoldType())) {

			if (anno.getBegin() >= selection.x && anno.getEnd() <= selection.y) {
				annoSelection.add(anno);
			}
		}
		return annoSelection;
	}

	private void enableDragAndDrop() {
		// dragn drop
		int operations = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transferTypes = new Transfer[] { LocalSelectionTransfer.getTransfer() };
		viewer.addDragSupport(operations, transferTypes, new TreeHierarchyViewerDragListener(viewer));
		dropListener = new TreeHierarchyViewerDropListener(viewer, ontology);
		viewer.addDropSupport(operations, transferTypes, dropListener);
	}

	private void addSelectionListener() {
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				updateHeaderActivation();

			}

		});
	}

	private void addMenu() {
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(viewer.getControl());

		// attach that menu manager to the treeviewer
		viewer.getControl().setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {
				if (viewer.getSelection().isEmpty()) {
					return;
				}

			}
		});

		menuMgr.setRemoveAllWhenShown(true);
	}

	private void addKeyListener() {
		viewer.getTree().addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {

					if (buttonEditNode.isEnabled()) {
						buttonEditNode.notifyListeners(SWT.Selection, new Event());
					}
				}

				else if (e.keyCode == SWT.DEL) {

					if (buttonDeleteNode.isEnabled()) {
						buttonDeleteNode.notifyListeners(SWT.Selection, new Event());
					}
				}

			}

		});
	}

	private void createViewer() {
		viewer = new TreeViewer(this, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		GridData gdTree = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);
		gdTree.horizontalSpan = 5;
		gdTree.heightHint = getParent().getBounds().height / 2;
		viewer.getTree().setLayoutData(gdTree);

		// viewer.addFilter(new AnnotationBrowserNoAnnoFilter());

		viewer.setContentProvider(new IEOntologyContentProvider(IProviderConstants.WIDGET_CLASS));

		labelProvider = new IEOntologyLabelProvider(ontology);
		viewer.setLabelProvider(labelProvider);

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

		// add the filter for the search field
		filter = new OWLEntitiyTreeFilter(ontology);
		viewer.addFilter(filter);

		// sorter
		treeSorter = new TreeViewerAlphabeticalSorter(IProviderConstants.WIDGET_CLASS);
		viewer.setComparator(treeSorter);
	}

	private void updateHeaderActivation() {
		// TODO manage the button activation

	}

	public void refreshParent() {
		Composite treeParent = getParent();
		if (treeParent instanceof IEOntologyTreeComposite) {
			IEOntologyTreeComposite compPar = (IEOntologyTreeComposite) treeParent;
			compPar.rehreshOntology();
		}
	}

	private void deleteClass(OWLOntologyClassWrapper wrapper) {
		List<OWLClass> allSubsumedClasses = OWLUtil.getAllSubsumedClasses(wrapper.getWrappedClass(),
				ontology.getOntology());
		// delete the selected node
		OWLUtil.deleteEntitiyFromOntology(wrapper.getWrappedClass(), ontology);

		// delete all children recursively
		for (OWLClass ce : allSubsumedClasses) {
			OWLUtil.deleteEntitiyFromOntology(ce, ontology);
		}
		refreshParent();
		// TODO there now might be properties that are invalid
		// because the node does no longer exists!
	}

	private void editClass(OWLOntologyClassWrapper wrappedClass) {

		OWLClassWrapperEditDialog dialog = new OWLClassWrapperEditDialog(Display.getCurrent().getActiveShell(),
				ontology, wrappedClass);
		int open = dialog.open();

		// success
		if (open == 0) {
			// rename if the entity was renamed this is done here because on
			// keylistener this had threading issues
			if (dialog.getNewIRI() != null) {
				OWLUtil.renameEntity(wrappedClass.getWrappedClass(), ontology.getOntology(), dialog.getNewIRI());
			}
			refreshParent();
			// also rename the entity
		}

	}

	public TreeViewer getViewer() {
		return viewer;
	}

	public void setViewer(TreeViewer viewer) {
		this.viewer = viewer;
	}

}
