package de.uniwue.mk.kall.ie.ieview.widget;

import java.util.ArrayList;
import java.util.List;

import javax.swing.text.WrappedPlainView;

import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.semanticweb.owlapi.model.OWLClass;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.ie.ieview.dialog.BinaryRelationManyEntitiesSelectionDialog;
import de.uniwue.mk.kall.ie.ieview.dialog.BinaryRelationTwoEntitiesSelectionDialog;
import de.uniwue.mk.kall.ie.ieview.dialog.OWLClassWrapperEditDialog;
import de.uniwue.mk.kall.ie.ieview.dialog.OWLPropertyWrapperEditDialog;
import de.uniwue.mk.kall.ie.ieview.widget.dragndrop.TreeRelationViewerDragListener;
import de.uniwue.mk.kall.ie.ieview.widget.dragndrop.TreeRelationViewerDropListener;
import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyClassWrapper;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyPropertyWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class TreeRelationComposite extends Composite {

	private IEOntology ontology;

	// UI
	private Label labelName;
	private Button buttonAddSibling;
	private Button buttonAddChild;
	private Button buttonDeleteNode;
	private Button buttonEditNode;

	private TreeViewer viewer;

	// UI helper
	private OWLEntitiyTreeFilter filter;

	private TreeViewerAlphabeticalSorter treeSorter;

	private AnnotationEditorWidget editor;

	private IEOntologyLabelProvider labelProvider;

	private TreeRelationViewerDropListener dropListener;

	public TreeRelationComposite(Composite parent, int style, AnnotationEditorWidget editor) {
		super(parent, style);
		this.setLayout(new GridLayout(5, false));
		this.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH));
		initTreeHeader();
		this.editor = editor;
	}

	private void initTreeHeader() {

		labelName = new Label(this, SWT.NONE);
		labelName.setText("Properties Hierarchy");
		buttonAddChild = new Button(this, SWT.NONE);
		buttonAddChild.setText("Add Child");

		buttonAddChild.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				if (sel.getFirstElement() instanceof OWLOntologyPropertyWrapper) {
					OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) sel.getFirstElement();

					// TODO do not create classes with the same IRI twice!
					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "",
							"Enter a property name", "", null);
					if (dlg.open() == Window.OK) {

						OWLObjectProperty newProperty = OWLUtil.createNewProperty(ontology.getOntology(),
								dlg.getValue());

						// we need to add this class to our ontology in the
						// given hierarchy
						OWLUtil.addEntityToOntology(newProperty, ontology);
						OWLUtil.assignAsSubObjectProperty(wrapper.getWrappedProperty(), newProperty, ontology);

						// add to expaneded elements TODO
						if (wrapper.getWrappedProperty() != null) {
							viewer.setExpandedState(wrapper.getWrappedProperty(), true);
						}

						// define the according subclass axiom

						// refresh the parent
						refreshParent();
					}
				}
			}

		});

		buttonAddSibling = new Button(this, SWT.NONE);
		buttonAddSibling.setText("Add Sibling");

		buttonAddSibling.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				if (sel.getFirstElement() instanceof OWLOntologyPropertyWrapper) {
					OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) sel.getFirstElement();

					// get the parent
					OWLOntologyPropertyWrapper parent = wrapper.getParent();
					if (parent == null)
						return;
					// TODO do not create classes with the same IRI twice!
					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "",
							"Enter a property name", "", null);
					if (dlg.open() == Window.OK) {

						OWLObjectProperty newProperty = OWLUtil.createNewProperty(ontology.getOntology(),
								dlg.getValue());

						// we need to add this class to our ontology in the
						// given hierarchy
						OWLUtil.addEntityToOntology(newProperty, ontology);
						OWLUtil.assignAsSubObjectProperty(parent.getWrappedProperty(), newProperty, ontology);

						// add to expaneded elements TODO
						if (wrapper.getWrappedProperty() != null) {
							viewer.setExpandedState(parent.getWrappedProperty(), true);
						}

						// define the according subclass axiom

						// refresh the parent
						refreshParent();
					}
				}
			}

		});

		buttonDeleteNode = new Button(this, SWT.NONE);
		buttonDeleteNode.setText("Delete");

		buttonDeleteNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof OWLOntologyPropertyWrapper) {

					OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) sel.getFirstElement();
					List<OWLObjectProperty> allSubsumedProperties = OWLUtil
							.getAllSubsumedProperties(wrapper.getWrappedProperty(), ontology.getOntology());
					// delete the selected node
					OWLUtil.deleteEntitiyFromOntology(wrapper.getWrappedProperty(), ontology);

					// delete all children recursively
					for (OWLObjectProperty ce : allSubsumedProperties) {
						OWLUtil.deleteEntitiyFromOntology(ce, ontology);
					}
					refreshParent();
				}

			}
		});

		buttonEditNode = new Button(this, SWT.NONE);
		buttonEditNode.setText("Edit");

		buttonEditNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof OWLOntologyPropertyWrapper) {

					OWLOntologyPropertyWrapper wrapper = (OWLOntologyPropertyWrapper) sel.getFirstElement();
					editProperty(wrapper);
				}
			}

		});

		// create search field
		Label labelSearch = new Label(this, SWT.NONE);
		labelSearch.setText("Search: ");

		Text textSearch = new Text(this, SWT.BORDER);
		GridData gdText = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		gdText.horizontalSpan = 4;
		textSearch.setLayoutData(gdText);

		textSearch.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {

				filter.setSearchText(textSearch.getText());
				viewer.refresh();
			}

		});

	}

	public void setEditorSelection(List<AnnotationFS> selectedEntities) {
		filter.setEditorSelection(selectedEntities);
		viewer.refresh();
	}

	public void setOntology(IEOntology ontology) {
		this.ontology = ontology;
		if (labelProvider != null) {
			labelProvider.setOntology(ontology);
		}
		if (filter != null) {
			filter.setOntology(ontology);
		}
		if (dropListener != null) {
			dropListener.setOntology(ontology);
		}
		updateLayout();

	}

	private void updateLayout() {

		// dispose the tree if it was already presentt
		if (viewer == null) {
			createTreeViewerAndAssignListener();
			// set the input
			if (ontology != null) {
				viewer.setInput(ontology);
			}
		} else {
			// just adapt the input, set the input a new and keep the expanded
			// state
			// viewer.
			Object[] expandedElements = viewer.getExpandedElements();
			viewer.setInput(ontology);
			viewer.setExpandedElements(expandedElements);
			viewer.refresh();
		}

		layout();
		getParent().layout();
	}

	private void createTreeViewerAndAssignListener() {
		// we create the tree
		createViewer();

		// TODO comparator
		// viewer.setComparator(treeSorter);

		addKeyListener();

		// create menumanager
		addMenu();

		enableDragAndDrop();

		addSelectionListener();

		addMouseListener();

	}

	private void addMouseListener() {
		viewer.getTree().addMouseListener(new MouseAdapter() {

			@Override
			public void mouseUp(MouseEvent e) {

				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));
				if (cell == null)
					return;
				Object element = cell.getElement();

				// if (element instanceof Concept) {
				// lastSelectedConcept = (Concept) element;
				// }
			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				// if we do not get a wrapped class object then we do nothing
				if (!(sel.getFirstElement() instanceof OWLOntologyPropertyWrapper)) {
					return;
				}
				OWLOntologyPropertyWrapper wrappedProperty = (OWLOntologyPropertyWrapper) sel.getFirstElement();
				if (wrappedProperty.getWrappedProperty() == null)
					return;

				// also return if the double click was performed on the root
				if (wrappedProperty.getWrappedProperty().isOWLTopDataProperty())
					return;
				// get the selection of the editor
				if (editor != null) {
					Point actualSelectedText = editor.getActualSelectedText();
					Util_impl util = new Util_impl(editor.getCas());

					// change the currently selected TODO is this really
					// required??
					if (actualSelectedText.x == actualSelectedText.y) {
						// check if there is a surrounding entity
						List<AnnotationFS> currentSelectionOfType = editor
								.getCurrentSelectionOfType(util.getIERelationGoldType());
						if (currentSelectionOfType.size() != 0) {
							// find the smallest annotation TODO
							AnnotationFS annotationFS = currentSelectionOfType.get(0);
							// we change the category feature to the name of the
							// class

							// TODO we update the filter attribute of the tree
							// annotationFS.setFeatureValueFromString(util.getIEEntityGoldCategoryFeature(),
							// wrappedProperty.getWrappedProperty().getIRI().getFragment());
							// // change the annotation
							// editor.changeAnnotation(annotationFS);
						} else {

							// we have no real selection => we edit the selected
							// class
							editProperty(wrappedProperty);
						}

					} else {
						// now we have a real selection and we create a new
						// annotation
						List<AnnotationFS> selectedEntities = getEditorSelectedEntities(
								editor.getWidget().getSelection());
						if (selectedEntities.size() == 0) {
							return;
						}

						AnnotationFS entity = editor.getCas().createAnnotation(util.getIERelationGoldType(),
								actualSelectedText.x, actualSelectedText.y);
						entity.setFeatureValueFromString(util.getIERelationGoldLabelFeature(),
								wrappedProperty.getWrappedProperty().getIRI().getFragment());

						// now perform slot filling, if this click was performed
						// we are guaranteed that the slots can be filled (but
						// not unique!)!! (by the tree filter)
						performSlotFilling(entity, selectedEntities, wrappedProperty.getWrappedProperty());

						editor.addAnnotation(entity);

					}

				} else {
					// we just open the dialog to edit the selected node
					editProperty(wrappedProperty);
				}

			}

			private void performSlotFilling(AnnotationFS entity, List<AnnotationFS> selectedEntities,
					OWLObjectProperty wrappedProperty) {
				// if we have a unique constellation we use it otherwise we ask
				// the user to specify what he wants

				// this handles 0-n
				if (OWLUtil.isFrameProperty(wrappedProperty, ontology.getOntology())) {
					// TODO ...
				}
				// this is 1-1 with annotation or 1-n
				else if (OWLUtil.isDefiniteConstructionProperty(wrappedProperty, ontology.getOntology())
						|| OWLUtil.isConstructionProperty(wrappedProperty, ontology.getOntology())) {
					// those should not be assigned as relations!!
					return;
				}
				// 0-0 is what remains and it does not make any sense at all!
				else if (OWLUtil.getClassesInDomain(wrappedProperty, ontology.getOntology()).size() == 0
						&& OWLUtil.getClassesInRange(wrappedProperty, ontology.getOntology()).size() == 0) {
					return;
				}

				// 1-1 is left and those are the real binary relations
				else if (OWLUtil.getClassesInDomain(wrappedProperty, ontology.getOntology()).size() == 1
						&& OWLUtil.getClassesInRange(wrappedProperty, ontology.getOntology()).size() == 1) {

					// check if selected entities also has the size of 2
					if (selectedEntities.size() < 2) {
						return;
					} else if (selectedEntities.size() == 2) {
						fillSlots(selectedEntities, wrappedProperty, entity);
					} else {
						// we have a more complex task to fulfill
						BinaryRelationManyEntitiesSelectionDialog dialog = new BinaryRelationManyEntitiesSelectionDialog(
								Display.getCurrent().getActiveShell(), selectedEntities.toArray(new AnnotationFS[0]));

						if (dialog.open() == Dialog.OK) {

							// set the features
							Util_impl util = new Util_impl(dialog.getDomain().getCAS());
							AnnotationFS domainAnno = dialog.getDomain();
							AnnotationFS rangeAnno = dialog.getRange();

							entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), domainAnno);
							entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), rangeAnno);
						}
					}

				}

			}

			private void fillSlots(List<AnnotationFS> selectedEntities, OWLObjectProperty wrappedProperty,
					AnnotationFS entity) {
				Util_impl util = new Util_impl(entity.getCAS());
				boolean firstEntityFitsDomain = checkIfAppropiate(
						OWLUtil.getClassesInDomain(wrappedProperty, ontology.getOntology()).get(0),
						selectedEntities.get(0));
				boolean firstEntityFitsRange = checkIfAppropiate(
						OWLUtil.getClassesInRange(wrappedProperty, ontology.getOntology()).get(0),
						selectedEntities.get(0));

				// second entity
				boolean secondEntityFitsDomain = checkIfAppropiate(
						OWLUtil.getClassesInDomain(wrappedProperty, ontology.getOntology()).get(0),
						selectedEntities.get(1));
				boolean secondEntityFitsRange = checkIfAppropiate(
						OWLUtil.getClassesInRange(wrappedProperty, ontology.getOntology()).get(0),
						selectedEntities.get(1));

				// now we have all information we need!
				boolean firstEntityFitsBoth = firstEntityFitsDomain & firstEntityFitsRange;
				boolean secondEntityFitsBoth = secondEntityFitsDomain & secondEntityFitsRange;

				if (firstEntityFitsBoth && !secondEntityFitsBoth) {
					if (secondEntityFitsDomain) {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(1));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(0));
					} else {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(0));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(1));
					}
				} else if (!firstEntityFitsBoth && secondEntityFitsBoth) {
					if (firstEntityFitsDomain) {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(0));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(1));
					} else {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(1));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(0));
					}

				} else if (!firstEntityFitsBoth && !secondEntityFitsBoth) {
					// here we assume our treefilter actually works and that
					// there is a unique solution!!
					if (firstEntityFitsDomain) {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(0));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(1));
					} else {
						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), selectedEntities.get(1));
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), selectedEntities.get(0));
					}

				}
				// both can be used anywhere -> we need a dialog!
				else {
					BinaryRelationTwoEntitiesSelectionDialog dialog = new BinaryRelationTwoEntitiesSelectionDialog(
							Display.getCurrent().getActiveShell(), wrappedProperty, ontology.getOntology(),
							selectedEntities.get(0), selectedEntities.get(1));

					if (dialog.open() == Dialog.OK) {
						AnnotationFS domainAnno = dialog.getDomain();
						AnnotationFS rangeAnno = dialog.getRange();

						entity.setFeatureValue(util.getIERelationGoldEntityFromFeature(), domainAnno);
						entity.setFeatureValue(util.getIERelationGoldEntityToFeature(), rangeAnno);
					}
				}
			}

			private boolean checkIfAppropiate(OWLClass owlClass, AnnotationFS entity) {
				// appropiate means that the category of entity is a subclass of
				// owlclass
				Util_impl util = new Util_impl(entity.getCAS());
				String category = entity.getFeatureValueAsString(util.getIEEntityGoldCategoryFeature());
				if (category == null) {
					return false;
				}
				if (owlClass.getIRI().getFragment().equals(category))
					return true;

				return OWLUtil.isSubClassOf(OWLUtil.getClassByIRI(category, ontology.getOntology()), owlClass,
						ontology.getOntology());
			}
		});
	}

	private void enableDragAndDrop() {
		// dragn drop
		int operations = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transferTypes = new Transfer[] { LocalSelectionTransfer.getTransfer() };
		viewer.addDragSupport(operations, transferTypes, new TreeRelationViewerDragListener(viewer));
		dropListener = new TreeRelationViewerDropListener(viewer, ontology);
		viewer.addDropSupport(operations, transferTypes, dropListener);
	}

	private void addSelectionListener() {
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				updateHeaderActivation();

			}

		});
	}

	private void addMenu() {
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(viewer.getControl());

		// attach that menu manager to the treeviewer
		viewer.getControl().setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {
				if (viewer.getSelection().isEmpty()) {
					return;
				}

			}
		});

		menuMgr.setRemoveAllWhenShown(true);
	}

	private void addKeyListener() {
		viewer.getTree().addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {

					if (buttonEditNode.isEnabled()) {
						buttonEditNode.notifyListeners(SWT.Selection, new Event());
					}
				}

				else if (e.keyCode == SWT.DEL) {

					if (buttonDeleteNode.isEnabled()) {
						buttonDeleteNode.notifyListeners(SWT.Selection, new Event());
					}
				}

			}

		});
	}

	private void createViewer() {
		viewer = new TreeViewer(this, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);
		GridData gdTree = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);
		gdTree.horizontalSpan = 5;
		gdTree.heightHint = getParent().getBounds().height / 2;
		viewer.getTree().setLayoutData(gdTree);

		// viewer.addFilter(new AnnotationBrowserNoAnnoFilter());

		viewer.setContentProvider(new IEOntologyContentProvider(IProviderConstants.WIDGET_PROPERTIES));

		labelProvider = new IEOntologyLabelProvider(ontology);
		viewer.setLabelProvider(labelProvider);

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

		// add the filter for the search field
		filter = new OWLEntitiyTreeFilter(ontology);
		viewer.addFilter(filter);

		// sorter
		// sorter
		treeSorter = new TreeViewerAlphabeticalSorter(IProviderConstants.WIDGET_PROPERTIES);
		viewer.setComparator(treeSorter);
	}

	private void updateHeaderActivation() {
		// TODO Auto-generated method stub

	}

	public void refreshParent() {
		Composite treeParent = getParent();
		if (treeParent instanceof IEOntologyTreeComposite) {
			IEOntologyTreeComposite compPar = (IEOntologyTreeComposite) treeParent;
			compPar.rehreshOntology();
		}
	}

	private void editProperty(OWLOntologyPropertyWrapper wrapper) {
		OWLPropertyWrapperEditDialog dialog = new OWLPropertyWrapperEditDialog(Display.getCurrent().getActiveShell(),
				ontology, wrapper);
		int open = dialog.open();
		// success
		if (open == 0) {
			// rename if the entity was renamed this is done here because on
			// keylistener this had threading issues
			if (dialog.getNewIRI() != null) {
				OWLUtil.renameEntity(wrapper.getWrappedProperty(), ontology.getOntology(), dialog.getNewIRI());
			}
			refreshParent();
			// also rename the entity
		}
	}

	public TreeViewer getViewer() {
		return this.viewer;
	}

	public void setEditorTextSelection(Point selection) {
		// get the current selection of the editor
		List<AnnotationFS> annoSelection = getEditorSelectedEntities(selection);

		// set the editor selection
		setEditorSelection(annoSelection);

	}

	private List<AnnotationFS> getEditorSelectedEntities(Point selection) {
		Util_impl util = new Util_impl(editor.getCas());
		List<AnnotationFS> annoSelection = new ArrayList<AnnotationFS>();

		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(util.getIEEntityGoldType())) {

			if (anno.getBegin() >= selection.x && anno.getEnd() <= selection.y) {
				annoSelection.add(anno);
			}
		}
		return annoSelection;
	}

}
