package de.uniwue.mk.kall.ie.ieview.widget.dragndrop;

import java.util.List;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Composite;
import org.semanticweb.owlapi.model.OWLClass;

import de.uniwue.mk.kall.ie.ieview.widget.TreeHierarchyComposite;
import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyClassWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class TreeHierarchyViewerDropListener extends ViewerDropAdapter {

	private TreeViewer viewer;
	private OWLOntologyClassWrapper fromNode;
	private OWLOntologyClassWrapper toNode;
	private IEOntology ontology;

	public TreeHierarchyViewerDropListener(TreeViewer viewer, IEOntology ontology) {
		super(viewer);
		this.viewer = viewer;
		this.ontology = ontology;
	}

	@Override
	public void drop(DropTargetEvent event) {
		// event has "To" information
		// int location = this.determineLocation(event);
		OWLOntologyClassWrapper anno = (OWLOntologyClassWrapper) determineTarget(event);
		toNode = anno;
		if (LocalSelectionTransfer.getTransfer().isSupportedType(event.currentDataType)) {
			ISelection sel = LocalSelectionTransfer.getTransfer().getSelection();
			IStructuredSelection struct = (IStructuredSelection) sel;
			fromNode = (OWLOntologyClassWrapper) struct.getFirstElement();
		}

		super.drop(event);

	}

	private void refreshGrandparent() {
		Composite parent = viewer.getTree().getParent();

		if (parent instanceof TreeHierarchyComposite) {
			TreeHierarchyComposite comp = (TreeHierarchyComposite) parent;
			comp.refreshParent();
		}
	}

	@Override
	public boolean performDrop(Object data) {
		if (fromNode == null || toNode == null || toNode.equals(fromNode))
			return false;

		//from parent to child is impossible
		List<OWLClass> allSubsumedClasses = OWLUtil.getAllSubsumedClasses(fromNode.getWrappedClass(),
				ontology.getOntology());
		if (allSubsumedClasses.contains(toNode.getWrappedClass()))
			return false;
		
		//thing can never be moved!!
		if(fromNode.getWrappedClass().equals(OWLUtil.getOWLThing(ontology.getOntology())))return false;

		// we need to delete from from its old parent
		OWLUtil.removeChildClass(fromNode.getParent().getWrappedClass(), fromNode.getWrappedClass(),
				ontology.getOntology());

		// and change its parent
		OWLUtil.assignAsSubclass(toNode.getWrappedClass(), fromNode.getWrappedClass(), ontology);

		refreshGrandparent();
		return true;
	}

	@Override
	public boolean validateDrop(Object target, int operation, TransferData transferType) {
		return true;
	}

	public void setOntology(IEOntology ontology) {
		this.ontology=ontology;
		
	}

}
