package de.uniwue.mk.kall.ie.ieview.widget.dragndrop;

import java.util.List;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Composite;
import org.semanticweb.owlapi.model.OWLObjectProperty;

import de.uniwue.mk.kall.ie.ieview.widget.TreeRelationComposite;
import de.uniwue.mk.kall.ie.terminology.struct.IEOntology;
import de.uniwue.mk.kall.ie.terminology.struct.OWLOntologyPropertyWrapper;
import de.uniwue.mk.kall.ie.terminology.util.OWLUtil;

public class TreeRelationViewerDropListener extends ViewerDropAdapter {

	private TreeViewer viewer;
	private OWLOntologyPropertyWrapper fromNode;
	private OWLOntologyPropertyWrapper toNode;
	private IEOntology ontology;

	public TreeRelationViewerDropListener(TreeViewer viewer, IEOntology ontology) {
		super(viewer);
		this.viewer = viewer;
		this.ontology = ontology;
	}

	@Override
	public void drop(DropTargetEvent event) {
		// event has "To" information
		// int location = this.determineLocation(event);
		OWLOntologyPropertyWrapper anno = (OWLOntologyPropertyWrapper) determineTarget(event);
		toNode = anno;
		if (LocalSelectionTransfer.getTransfer().isSupportedType(event.currentDataType)) {
			ISelection sel = LocalSelectionTransfer.getTransfer().getSelection();
			IStructuredSelection struct = (IStructuredSelection) sel;
			fromNode = (OWLOntologyPropertyWrapper) struct.getFirstElement();
		}

		super.drop(event);

	}

	private void refreshGrandparent() {
		Composite parent = viewer.getTree().getParent();

		if (parent instanceof TreeRelationComposite) {
			TreeRelationComposite comp = (TreeRelationComposite) parent;
			comp.refreshParent();
		}
	}

	@Override
	public boolean performDrop(Object data) {
		if (fromNode == null || toNode == null || toNode.equals(fromNode))
			return false;

		// from parent to child is impossible
		List<OWLObjectProperty> allSubsumedProperties = OWLUtil.getAllSubsumedProperties(fromNode.getWrappedProperty(),
				ontology.getOntology());
		if (allSubsumedProperties.contains(toNode.getWrappedProperty()))
			return false;

		// topProperty can never be moved!!
		if (fromNode.getWrappedProperty().equals(OWLUtil.getOWLTopObjectProperty(ontology.getOntology())))
			return false;

		// we need to delete from from its old parent
		OWLUtil.removeChildProperty(fromNode.getParent().getWrappedProperty(), fromNode.getWrappedProperty(),
				ontology.getOntology());

		// and change its parent
		OWLUtil.assignAsSubObjectProperty(toNode.getWrappedProperty(), fromNode.getWrappedProperty(), ontology);

		refreshGrandparent();
		return true;
	}

	@Override
	public boolean validateDrop(Object target, int operation, TransferData transferType) {
		return true;
	}

	public void setOntology(IEOntology ontology) {
		this.ontology = ontology;
		
	}

}
