package de.uniwue.mk.kall.lucene.app;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Paths;

import org.apache.commons.io.FileUtils;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.StringField;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

public class LuceneIndexQuery {

  public static void main(String[] args) throws Exception {

    Analyzer analyzer = new StandardAnalyzer();
    Directory index = FSDirectory
            .open(Paths
                    .get("\\\\hastur\\scratch\\kallimachos\\kalimachos Doks\\Romane_Alle_Preprocessed\\index.lucene"));

    IndexWriterConfig config = new IndexWriterConfig(analyzer);

    // buildIndex(index, config);

    // String querystr = "Er war";
    // Query q = new QueryParser("Text", analyzer).parse(querystr);

    String query = "die ???? geht zu".toLowerCase();

    Query q = new PhraseQuery("Text", query.split(" "));

    int hitsPerPage = 100;
    IndexReader reader = DirectoryReader.open(index);
    IndexSearcher searcher = new IndexSearcher(reader);
    TopDocs docs = searcher.search(q, hitsPerPage);
    ScoreDoc[] hits = docs.scoreDocs;

    FileWriter fw = new FileWriter(new File(
            "C:\\owncloud\\koreferenz_Verbesserungen\\Indexdaten\\war.txt"));
    System.out.println("Found " + hits.length + " hits.");
    for (int i = 0; i < hits.length; ++i) {
      int docId = hits[i].doc;
      Document d = searcher.doc(docId);
      System.out.println((i + 1) + ". " + "\t" + d.get("Text"));
      fw.append((i + 1) + ". " + "\t" + d.get("Text"));
    }
    fw.flush();
    fw.close();
  }

  private static void buildIndex(Directory index, IndexWriterConfig config) throws IOException {
    IndexWriter w = new IndexWriter(index, config);

    for (File f : new File(
            "\\\\hastur\\scratch\\kallimachos\\kalimachos Doks\\Romane_Alle_Preprocessed\\RomanePLUS")
            .listFiles()) {

      System.out.println(f);
      String[] docs = FileUtils.readFileToString(f, "UTF-8").split("\n");

      System.out.println(docs.length);
      for (String doc : docs) {

        Document document = new Document();

        document.add(new TextField("Text", doc, Field.Store.YES));

        w.addDocument(document);
      }
    }
    w.close();
  }

  private static void addDoc(IndexWriter w, String title, String isbn) throws IOException {
    Document doc = new Document();
    doc.add(new TextField("title", title, Field.Store.YES));
    doc.add(new StringField("isbn", isbn, Field.Store.YES));
    w.addDocument(doc);
  }

}
