package de.uniwue.mk.kall.lucene.kallimachos.index;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.nio.file.Paths;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.TextField;
import org.apache.lucene.index.DirectoryReader;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.impl.XmiCasDeserializer;
import org.apache.uima.cas.text.AnnotationFS;
import org.xml.sax.SAXException;

import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class LuceneUIMAIndexBuilder {

  public static void main(String[] args) throws Exception {

    Analyzer analyzer = new CustomUIMAAnalyzer();
    Directory index = FSDirectory
            .open(Paths
                    .get("\\\\hastur\\scratch\\kallimachos\\kalimachos Doks\\Romane_Alle_Preprocessed\\index2.lucene"));

    IndexWriterConfig config = new IndexWriterConfig(analyzer);

    // buildIndex(index, config);

    String query = "Lemma=sagen ihr POS=NE";

    PhraseQuery q = new PhraseQuery(0, "Text", query.split(" "));




    int hitsPerPage = 10000000;
    IndexReader reader = DirectoryReader.open(index);
    IndexSearcher searcher = new IndexSearcher(reader);
    TopDocs docs = searcher.search(q, hitsPerPage);
    ScoreDoc[] hits = docs.scoreDocs;

    System.out.println("Found " + hits.length + " hits.");
    for (int i = 0; i < hits.length; ++i) {
      int docId = hits[i].doc;
      Document d = searcher.doc(docId);
      System.out.println((i + 1) + ". " + "\t" + d.get("Text"));
    }
  }

  private static void buildIndex(Directory index, IndexWriterConfig config) throws IOException,
          SAXException {
    IndexWriter w = new IndexWriter(index, config);

    CAS cas = Util_impl.createCas();
    for (File f : new File("\\\\hastur\\scratch\\kallimachos\\abenteuerromaneXMI\\out").listFiles()) {

      System.out.println(f);
      XmiCasDeserializer.deserialize(new FileInputStream(f), cas);

      CustomUIMAAnalyzer analyzer = (CustomUIMAAnalyzer) w.getAnalyzer();
      analyzer.setCas(cas);
      Util_impl util = new Util_impl(cas);
      for (AnnotationFS sent : cas.getAnnotationIndex(util.getSentenceType())) {

        // create new document
        Document document = new Document();
        analyzer.setSentence(sent);

        // add the text to the document, the tokenizer adds the according indexes
        document.add(new TextField("Text", sent.getCoveredText(), Field.Store.YES));

        // add the new document
        w.addDocument(document);
      }
    }
    w.close();
  }

}
