package de.uniwue.mk.kall.lucene.kallimachos.index;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.analysis.Tokenizer;
import org.apache.lucene.analysis.tokenattributes.CharTermAttribute;
import org.apache.lucene.analysis.tokenattributes.PositionIncrementAttribute;
import org.apache.uima.cas.CAS;
import org.apache.uima.cas.text.AnnotationFS;

import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class UIMALuceneTokenizer extends Tokenizer {

  private Util_impl util;

  private int index;

  private List<AnnotationFS> tokens;

  private PositionIncrementAttribute posIncrement;

  private CharTermAttribute termAttribute;

  private List<UIMAAnnotationIndexStruct> structs;

  private CAS cas;

  private AnnotationFS sentence;

  private int structIndex;

  public UIMALuceneTokenizer() {

    posIncrement = addAttribute(PositionIncrementAttribute.class);
    termAttribute = addAttribute(CharTermAttribute.class);

  }

  public void setCas(CAS cas) {
    this.cas = cas;
    this.util = new Util_impl(cas);

    structs = new ArrayList<UIMAAnnotationIndexStruct>();

    structs.add(new UIMAAnnotationIndexStruct(util.getPOSType(), util.getPOSTagFeature(), "POS"));
    structs.add(new UIMAAnnotationIndexStruct(util.getPOSType(), null, ""));
    structs.add(new UIMAAnnotationIndexStruct(util.getPOSType(), util.getLemmaFeature(), "Lemma"));
  }

  public AnnotationFS getSentence() {
    return sentence;
  }

  public void setSentence(AnnotationFS sentence) {
    this.sentence = sentence;
    this.index = 1;
    this.structIndex = 0;
    this.tokens = util.getCovered(sentence, util.getPOSType());
  }

  @Override
  public boolean incrementToken() throws IOException {
    if (index >= tokens.size())
      return false;

    clearAttributes();

    UIMAAnnotationIndexStruct uimaAnnotationIndexStruct = structs.get(structIndex);
    AnnotationFS annotationFS = util.getCovered(sentence,
            uimaAnnotationIndexStruct.getTypeToIndex()).get(index - 1);

    String featToIndex = null;

    if (uimaAnnotationIndexStruct.getFeatureOfTypeToIndex() != null) {
      featToIndex = annotationFS.getFeatureValueAsString(uimaAnnotationIndexStruct
              .getFeatureOfTypeToIndex());

      featToIndex = uimaAnnotationIndexStruct.getSuffix() + "=" + featToIndex;
    }

    else {
      // use the covered text feature
      featToIndex = annotationFS.getCoveredText();
    }

    // add the text
    char[] asCharArray = featToIndex.toCharArray();
    termAttribute.copyBuffer(asCharArray, 0, asCharArray.length);

    // update for next iteration

    if (structIndex == 0) {
      posIncrement.setPositionIncrement(1);
    } else {
      posIncrement.setPositionIncrement(0);
    }

    structIndex++;
    if (structIndex == structs.size()) {
      structIndex = 0;
      index++;
    }
    return true;

  }

}
