package de.uniwue.mk.kall.parserDevelopmentView;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class ParserDevelopmentView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<ParserDevelopmentView> CLASS_THIS = ParserDevelopmentView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.20";

	// the main type this view is about
	private Type typeParsedSuccesful;
	private Type typeParsedUnsuccesful;
	private Type typeSentence;
	private Type typeToken;

	private Button buttonApplyParser;

	private Button buttonEditGrammar;

	private String grammar;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	// compounds - UI

	@Inject
	public ParserDevelopmentView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		// layout this should stay as it is
		super.overlay = parent;
		super.overlay.setLayout(new FillLayout());

		// init the part, so that this view knows its position globally
		// (required to see if it has focus)
		super.part = partService.findPart(viewId);
	}

	// is called whenever the editor loads a new page
	@Override
	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		super.editor = editor;

		// only initialize if the part is visible
		MUIElement selectedElement = null;
		if (part != null) {
			MElementContainer<MUIElement> parent = part.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render but unsibscribe
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
			return;
		}

		// init all the types we need in this view, open a dialog to specify
		// them if not already done so
		boolean typesValid = initTypes();

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// init the layout
		initLayout(overlay);

		// if we managed to get here subscribe to all editor events if not
		// already done so
		if (!isSubscribedToEditorEvents) {
			// use the standard method to get all events
			super.standardSubscribeToEditorEvents(CLASS_THIS);
		}

		// ====reload=====
		overlay.layout();
		overlay.getParent().layout();
	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {
		typeParsedSuccesful = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp4");
		typeParsedUnsuccesful = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp5");
		typeSentence = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Sentence");
		typeToken = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.POS");

		deleteEvalTypes();

		// Visibility

		editor.changeVisibility(typeParsedSuccesful, true);
		editor.changeVisibility(typeParsedUnsuccesful, true);

		// green
		StyleRange rangeSuccess = new StyleRange();
		rangeSuccess.background = new Color(overlay.getDisplay(), 0, 255, 0);
		AnnoStyle styleSuccess = new AnnoStyle(new BackgroundDrawingStrategy(rangeSuccess), rangeSuccess);

		// style - red
		StyleRange rangeFail = new StyleRange();
		rangeFail.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle styleFail = new AnnoStyle(new BackgroundDrawingStrategy(rangeFail), rangeFail);

		// apply
		editor.changeStyle(typeParsedSuccesful, styleSuccess);
		editor.changeStyle(typeParsedUnsuccesful, styleFail);

		// also change the modus
		editor.setActiveMode(typeParsedSuccesful);

		return true;

	}

	private void deleteEvalTypes() {

		List<AnnotationFS> toDel = new ArrayList<AnnotationFS>();

		for (AnnotationFS tok : editor.getCas().getAnnotationIndex(typeParsedSuccesful)) {
			toDel.add(tok);
		}

		for (AnnotationFS tok : editor.getCas().getAnnotationIndex(typeParsedUnsuccesful)) {
			toDel.add(tok);
		}

		for (AnnotationFS a : toDel)
			editor.getCas().removeFsFromIndexes(a);

	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

		parent.setLayout(new GridLayout(2, true));

		buttonEditGrammar = new Button(parent, SWT.PUSH);

		buttonEditGrammar.setText("Edit Grammar");
		buttonEditGrammar.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				ParserGrammarDevelopmentDialog dialog = new ParserGrammarDevelopmentDialog(grammar);

				grammar = dialog.getGrammar();

			}

		});

		buttonApplyParser = new Button(parent, SWT.PUSH);

		buttonApplyParser.setText("Apply Parser");
		buttonApplyParser.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				applyParser();

			}

		});
	}

	// use this method to update the view
	public void renewPage() throws CASException {

		// redraw in a new Thread, executed by the Display
		Runnable r = new Runnable() {

			@Override
			public void run() {

			}
		};

		// reload asynchronous
		overlay.getDisplay().asyncExec(r);

	}

	@PreDestroy
	public void preDestroy() {
		// TODO unused
	}

	private void applyParser() {
		deleteEvalTypes();

		List<AnnotationFS> evalTypes = new ArrayList<AnnotationFS>();
		for (AnnotationFS sent : editor.getCas().getAnnotationIndex(typeSentence)) {

			// apply parser

			double random = Math.random();

			if (random > 0.6) {
				AnnotationFS fs = editor.getCas().createAnnotation(typeParsedUnsuccesful, sent.getBegin(),
						sent.getEnd());

				evalTypes.add(fs);
			}

			else {
				AnnotationFS fs = editor.getCas().createAnnotation(typeParsedSuccesful, sent.getBegin(), sent.getEnd());

				evalTypes.add(fs);
			}
		}

		// Bulk add without reload
		for (int i = 0; i < evalTypes.size() - 1; i++) {

			editor.getCas().addFsToIndexes(evalTypes.get(i));
		}

		// add and reload
		editor.addAnnotation(evalTypes.get(evalTypes.size() - 1));

	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr
	}

	// Perspective listener
	@Override
	public void coupleToPerspective() {

		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);

		super.standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);

	}

	@Override
	public void decoupleFromPerspective() {

		MasterPart.unsubscribeFromEvent(CLASS_THIS, 
				IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);
		super.standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);

	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		// unsubscribe from editor notifications - yep this is important
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {

		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			super.standardUnsubscribeFromEditorEvents(CLASS_THIS);
		}

	}

	// what to do when the editor has an added annotation?
	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an removed annotation?
	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation?
	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		AnnotationFS changed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);

	}

	// what to do when the editor has an changed annotation selection ( this is
	// an
	// arrayList<AnnotationFS> )
	@Override
	public void onAnnotationSelectionChanged(org.osgi.service.event.Event arg0) {
		// unused hjh

	}

	// what to do when the editor was double clicked
	@Override
	public void onEditorDoubleClicked(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was scrolled
	@Override
	public void onEditorWasScrolled(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor caret was changed
	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor was clicked
	@Override
	public void onEditorMouseUp(org.osgi.service.event.Event arg0) {
		// unused

	}

	// what to do when the editor is clicked
	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {
		// unused for this view

	}

	// what to do when the editor has a changed textselection
	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

	}

	// on editor mouse moved - use this if you want some fancy effects
	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {
		// TODO Auto-generated method stub

	}

	@Override
	public void initModel() {
		// TODO Auto-generated method stub

	}

}