package de.uniwue.mk.kall.relationAnalyzer.ui;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.fieldassist.AutoCompleteField;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.osgi.service.event.Event;

import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class AttributeAnalyzer extends AEditorSubordinateViewPart {

	// CONSTANTS
	private static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.27";

	private static final String ACCORDING_NE = "ACCORDING_NE";

	// Eventhandler

	// model
	private AttributeStruct struct;

	// UI
	private Composite compAttributes;

	@Inject
	IEventBroker broker;

	private Composite compAttPerson;

	private Composite compAttPersonSpecific;

	private Text textType;

	private Spinner spinnerAttFrom;

	private Spinner spinnerAttTo;

	private AutoCompleteField autoCompletetextType;

	// model
	// FIELDS======

	private static Class<?> CLASS_THIS = AttributeAnalyzer.class;

	@Inject
	static EPartService partService;

	@Inject
	EModelService modelService;

	private Font font;

	private Type relationType;

	private Feature agensFeature;

	private Feature agens2Feature;

	private Feature relationTypeFeature;

	private Feature relationAttributeFeature;

	@Inject
	public AttributeAnalyzer() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

		font = new Font(Display.getCurrent(), new FontData("Segoe UI", 15, SWT.NONE));

	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	public void initLayout(Composite parent) {

		initDummyLayout();

	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void setEditor(AnnotationEditorWidget editor) {
		this.editor = editor;
	}

	public void renewPage() {

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	@Override
	public void onEditorCaretChanged(Event arg0) {

		int caretOffset = (int) arg0.getProperty(IEventBroker.DATA);
		AttributeStruct offsetStruct = null;
		for (AnnotationFS rel : editor.getCas().getAnnotationIndex(relationType)) {

			if (editor.isSurpressed(rel))
				continue;

			if (rel.getBegin() <= caretOffset && rel.getEnd() >= caretOffset) {

				Util_impl util = new Util_impl(rel.getCAS());
				offsetStruct = new AttributeStruct(rel, util.getCovered(rel, util.getNamedEntityType()),
						util.getCovered(rel, util.getPOSType()), util.getCas());

				break;
			}

		}

		clearLayout();
		if (offsetStruct != null) {
			// override the modelobject
			this.struct = offsetStruct;
			overlay.setBackgroundImage(null);
			resetLayout();

		}

		else {

			clearLayout();
			initDummyLayout();

			overlay.layout();

		}

	}

	@Override
	public void initModel() {
		// no model in this view

	}

	@Override
	public boolean initTypes() {
		System.out.println("init types");

		// relationtype
		this.relationType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Relation");
		this.agensFeature = relationType.getFeatureByBaseName("Agens");
		this.agens2Feature = relationType.getFeatureByBaseName("Agens2");
		this.relationTypeFeature = relationType.getFeatureByBaseName("Type");
		this.relationAttributeFeature = relationType.getFeatureByBaseName("Attribut");
		return true;
	}

	private void initDummyLayout() {
		Button bDummy = new Button(overlay, SWT.PUSH);
		bDummy.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));
		bDummy.setText("Select a Relation...");
		bDummy.setEnabled(false);
		Font boldFont = new Font(Display.getCurrent(), new FontData("Segoe UI", 30, SWT.BOLD));

		bDummy.setFont(boldFont);
		Color white = new Color(Display.getCurrent(), new RGB(255, 255, 255));
		bDummy.setBackground(white);

		white.dispose();
	}

	private void resetLayout() {

		// adapt the layout to the new struct TODO

		// clear old layout
		clearLayout();

		// init the right composite
		initAttributeComposite();

		// refresh
		overlay.layout();

	}

	private void initAttributeComposite() {

		compAttributes = new Composite(overlay, SWT.BORDER);
		compAttributes.setLayout(new GridLayout(1, true));
		compAttributes.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL
				| GridData.GRAB_VERTICAL));

		if (this.struct == null)
			return;
		// split layout into 2 parts - first is a comp of buttons to select the
		// person in the relation
		compAttPerson = new Composite(compAttributes, SWT.NONE);
		compAttPerson.setLayout(new GridLayout(struct.getCoveredNEs().size(), false));
		compAttPerson.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		compAttPersonSpecific = new Composite(compAttributes, SWT.NONE);

		compAttPersonSpecific.setLayout(new GridLayout(7, false));
		compAttPersonSpecific.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		for (AnnotationFS ne : struct.getCoveredNEs()) {

			final Button bNE = new Button(compAttPerson, SWT.PUSH);

			bNE.setText(ne.getCoveredText());
			bNE.setData(ACCORDING_NE, ne);
			bNE.setFont(font);

			bNE.addSelectionListener(new SelectionListener() {

				@Override
				public void widgetSelected(SelectionEvent e) {

					// clear all lines from the old selection

					for (Control c : compAttPersonSpecific.getChildren())
						c.dispose();

					GridData gdDummy = new GridData();
					gdDummy.horizontalSpan = 7;
					Button dummy1 = new Button(compAttPersonSpecific, SWT.PUSH);
					dummy1.setVisible(false);
					dummy1.setLayoutData(gdDummy);

					Button dummy2 = new Button(compAttPersonSpecific, SWT.PUSH);
					dummy2.setVisible(false);
					dummy2.setLayoutData(gdDummy);

					// show all attributes that are already assigned to this
					// person, also add an additional line to be able to
					// specifiy a new attribute

					// TODO add the words instead of anno span

					// add an additional line
					boolean attributeFound = false;

					Util_impl util = new Util_impl(struct.getCas());

					addNewAttributeLine((AnnotationFS) bNE.getData(ACCORDING_NE));

				}

				@Override
				public void widgetDefaultSelected(SelectionEvent e) {

				}
			});

			bNE.addFocusListener(new FocusListener() {

				@Override
				public void focusLost(FocusEvent e) {

				}

				@Override
				public void focusGained(FocusEvent e) {

					for (Control c : compAttPerson.getChildren()) {
						c.setBackground(null);
					}
					Color color = new Color(Display.getCurrent(), new RGB(255, 0, 0));
					bNE.setBackground(color);
					color.dispose();

				}
			});

		}

		// preselect if the incoming relation has attributes already

		AnnotationFS selectedNE = struct.getSelectedNE();

		// set focus of the selected annotation
		if (selectedNE != null && struct.isAttribute()) {

			for (Control c : compAttPerson.getChildren()) {

				Object data = c.getData(ACCORDING_NE);

				if (data != null && data instanceof AnnotationFS) {

					AnnotationFS ne = (AnnotationFS) data;

					if (ne.getBegin() == selectedNE.getBegin()) {
						c.setFocus();
						c.notifyListeners(SWT.Selection, new org.eclipse.swt.widgets.Event());
					}
				}
			}
		}

		// if theres just 1 possibility we use it
		else if (selectedNE == null && compAttPerson.getChildren().length == 1) {
			compAttPerson.getChildren()[0].setFocus();
			compAttPerson.getChildren()[0].notifyListeners(SWT.Selection, new org.eclipse.swt.widgets.Event());
		}

		compAttPersonSpecific.pack();
		compAttPersonSpecific.layout();
		compAttributes.layout();
	}

	private void clearLayout() {

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

	}

	private void addNewAttributeLine(final AnnotationFS annotationFS) {

		// check if there is an already set annotation

		String from = struct.getFromWordIndex();
		String to = struct.getToWordIndex();
		String selectedType = struct.getType();

		// Labels From....To

		// update the attribute annotation
		updateAttributeAnnotation(from, to);

		Label labelAttributeFrom = new Label(compAttPersonSpecific, SWT.NONE);
		labelAttributeFrom.setText("From:");
		labelAttributeFrom.setFont(font);

		spinnerAttFrom = new Spinner(compAttPersonSpecific, SWT.BORDER);
		spinnerAttFrom.setFont(font);
		spinnerAttFrom.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		spinnerAttFrom.setSelection(Integer.parseInt(from));

		spinnerAttFrom.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				String fromS = spinnerAttFrom.getText();
				int toInt = Integer.parseInt(spinnerAttTo.getText());

				if (fromS.isEmpty())
					return;
				try {
					int fromInt = Integer.parseInt(fromS);
					if (fromInt > 0 && fromInt <= toInt) {

						updateAttributeAnnotation(fromS, spinnerAttTo.getText());
					}
				}

				catch (NumberFormatException exc) {
					exc.printStackTrace();
				}

			}

		});

		Label labelAttributeTo = new Label(compAttPersonSpecific, SWT.NONE);
		labelAttributeTo.setText("To:");
		labelAttributeTo.setFont(font);

		spinnerAttTo = new Spinner(compAttPersonSpecific, SWT.BORDER);
		spinnerAttTo.setFont(font);
		spinnerAttTo.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		spinnerAttTo.setSelection(Integer.parseInt(to));

		spinnerAttTo.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String to = spinnerAttTo.getText();
				int fromInt = Integer.parseInt(spinnerAttFrom.getText());

				if (to.isEmpty())
					return;
				try {
					int toInt = Integer.parseInt(to);
					if (toInt >= fromInt && toInt <= struct.getCoveredTokens().size()) {

						updateAttributeAnnotation(spinnerAttFrom.getText(), to);
					}
				}

				catch (NumberFormatException exc) {
					exc.printStackTrace();
				}

			}

		});

		// Type

		Label labelType = new Label(compAttPersonSpecific, SWT.None);
		labelType.setText("Type:");
		labelType.setAlignment(SWT.CENTER);
		labelType.setFont(font);

		textType = new Text(compAttPersonSpecific, SWT.BORDER);
		textType.setFont(font);
		textType.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		autoCompletetextType = new AutoCompleteField(textType, new TextContentAdapter(), new String[0]);

		if (!selectedType.isEmpty()) {
			textType.setText(selectedType);
		}

		Button buttonAdd = new Button(compAttPersonSpecific, SWT.PUSH);

		buttonAdd.setText("ADD");
		buttonAdd.setFont(font);

		buttonAdd.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				Util_impl util = new Util_impl(struct.getCas());
				// set the features to the Anno
				AnnotationFS rel = struct.getRelationAnno();

				rel.setFeatureValue(util.getRelationAgensFeature(), annotationFS);
				rel.setFeatureValue(util.getRelationAgens2Feature(), annotationFS);

				rel.setFeatureValueFromString(util.getRelationTypeFeature(), textType.getText());

				// notifiy editor to redraw

				editor.changeAnnotation(rel);

			}

		});

		compAttPersonSpecific.pack();
		compAttPersonSpecific.layout();
		compAttributes.layout();

	}

	// assumes from and to are valid !!
	private void updateAttributeAnnotation(String from, String to) {
		// delete the old one

		AnnotationFS att = (AnnotationFS) struct.getRelationAnno().getFeatureValue(
				struct.getUtil().getRelationAttributeFeature());

		if (att != null) {
			struct.getCas().removeFsFromIndexes(att);
		}
		// add a new one

		int fromIndex = struct.getCoveredTokens().get(Integer.parseInt(from) - 1).getBegin();
		int toIndex = struct.getCoveredTokens().get(Integer.parseInt(to) - 1).getEnd();

		AnnotationFS attSpan = struct.getCas().createAnnotation(struct.getUtil().getAttSpanType(), fromIndex, toIndex);

		struct.getCas().addFsToIndexes(attSpan);
		struct.getRelationAnno().setFeatureValue(struct.getUtil().getRelationAttributeFeature(), attSpan);
		// refresh the editor
		editor.changeAnnotation(attSpan);
	}

}
