package de.uniwue.mk.kall.relationAnalyzer.ui;

import java.util.List;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.text.AnnotationFS;

import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class AttributeStruct {

	private AnnotationFS relationAnno;
	private List<AnnotationFS> coveredNEs;
	private List<AnnotationFS> coveredTokens;

	private CAS cas;
	private Util_impl util;

	public AttributeStruct(AnnotationFS relationAnno, List<AnnotationFS> coveredNEs, List<AnnotationFS> coveredTokens,
			CAS cas) {
		super();
		this.relationAnno = relationAnno;
		this.coveredNEs = coveredNEs;
		this.coveredTokens = coveredTokens;
		this.cas = cas;
		this.util = new Util_impl(cas);
	}

	public Util_impl getUtil() {
		return util;
	}

	public void setUtil(Util_impl util) {
		this.util = util;
	}

	public AnnotationFS getRelationAnno() {
		return relationAnno;
	}

	public void setRelationAnno(AnnotationFS relationAnno) {
		this.relationAnno = relationAnno;
	}

	public List<AnnotationFS> getCoveredNEs() {
		return coveredNEs;
	}

	public void setCoveredNEs(List<AnnotationFS> coveredNEs) {
		this.coveredNEs = coveredNEs;
	}

	public List<AnnotationFS> getCoveredTokens() {
		return coveredTokens;
	}

	public void setCoveredTokens(List<AnnotationFS> coveredTokens) {
		this.coveredTokens = coveredTokens;
	}

	public CAS getCas() {
		return cas;
	}

	public void setCas(CAS cas) {
		this.cas = cas;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((cas == null) ? 0 : cas.hashCode());
		result = prime * result + ((coveredNEs == null) ? 0 : coveredNEs.hashCode());
		result = prime * result + ((coveredTokens == null) ? 0 : coveredTokens.hashCode());
		result = prime * result + ((relationAnno == null) ? 0 : relationAnno.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		AttributeStruct other = (AttributeStruct) obj;
		if (cas == null) {
			if (other.cas != null)
				return false;
		} else if (!cas.equals(other.cas))
			return false;
		if (coveredNEs == null) {
			if (other.coveredNEs != null)
				return false;
		} else if (!coveredNEs.equals(other.coveredNEs))
			return false;
		if (coveredTokens == null) {
			if (other.coveredTokens != null)
				return false;
		} else if (!coveredTokens.equals(other.coveredTokens))
			return false;
		if (relationAnno == null) {
			if (other.relationAnno != null)
				return false;
		} else if (!relationAnno.equals(other.relationAnno))
			return false;
		return true;
	}

	public String getToWordIndex() {
		FeatureStructure featureValue = relationAnno.getFeatureValue(util.getRelationAttributeFeature());

		if (featureValue != null) {

			int beg = ((AnnotationFS) featureValue).getEnd();

			int index = 0;
			for (AnnotationFS pos : coveredTokens) {
				if (beg == pos.getEnd()) {
					return String.valueOf(index + 1);
				}
				index++;
			}

		} else {
			return String.valueOf(coveredTokens.size());
		}

		return null;
	}

	public String getFromWordIndex() {
		FeatureStructure featureValue = relationAnno.getFeatureValue(util.getRelationAttributeFeature());

		if (featureValue != null) {

			int beg = ((AnnotationFS) featureValue).getBegin();

			int index = 0;
			for (AnnotationFS pos : coveredTokens) {
				if (beg == pos.getBegin()) {
					return String.valueOf(index + 1);
				}
				index++;
			}

		} else {
			return "1";
		}

		return null;
	}

	public String getType() {

		String type = relationAnno.getFeatureValueAsString(util.getRelationTypeFeature());

		if (type == null)
			return "";

		return type;
	}

	public AnnotationFS getSelectedNE() {

		FeatureStructure featureValue = relationAnno.getFeatureValue(util.getRelationAgensFeature());

		if (featureValue == null)
			return null;
		return (AnnotationFS) featureValue;
	}

	public boolean isAttribute() {

		AnnotationFS agens = (AnnotationFS) this.relationAnno.getFeatureValue(util.getRelationAgensFeature());
		AnnotationFS agens2 = (AnnotationFS) this.relationAnno.getFeatureValue(util.getRelationAgens2Feature());

		if (agens == null || agens2 == null)
			return false;

		if (agens.getBegin() == agens2.getBegin())
			return true;
		return false;
	}

}
