package de.uniwue.mk.kall.relationAnalyzer.ui;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.fieldassist.AutoCompleteField;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.osgi.service.event.Event;

import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class RelationAnalyzer extends AEditorSubordinateViewPart {

	// CONSTANTS
	private static final String ACCORDING_NE = "ACCORDING_NE";

	private static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.26";

	private boolean firstButtonChangedLast = false;

	// UI
	private Composite compRelations;
	private Button buttonFirstNE;
	private Button buttonSecondNe;
	private Composite compAttPerson;
	private Composite compAttPersonRelation;
	private AutoCompleteField autoCompletetextType;
	private Text textType;

	// model
	private AttributeStruct struct;

	// FIELDS======

	private static Class<?> CLASS_THIS = RelationAnalyzer.class;

	@Inject
	private static IEventBroker broker;
	@Inject
	static EPartService partService;

	@Inject
	EModelService modelService;

	private Font font;

	private Type relationType;

	private Feature agensFeature;

	private Feature agens2Feature;

	private Feature relationTypeFeature;

	private Feature relationAttributeFeature;

	@Inject
	public RelationAnalyzer() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

		font = new Font(Display.getCurrent(), new FontData("Segoe UI", 15, SWT.NONE));

	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	public void initLayout(Composite parent) {

		initDummyLayout();

	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void setEditor(AnnotationEditorWidget editor) {
		this.editor = editor;
	}

	public void renewPage() {

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	@Override
	public void onEditorCaretChanged(Event arg0) {

		int caretOffset = (int) arg0.getProperty(IEventBroker.DATA);
		AttributeStruct offsetStruct = null;
		for (AnnotationFS rel : editor.getCas().getAnnotationIndex(relationType)) {

			if (editor.isSurpressed(rel))
				continue;

			if (rel.getBegin() <= caretOffset && rel.getEnd() >= caretOffset) {

				Util_impl util = new Util_impl(rel.getCAS());
				offsetStruct = new AttributeStruct(rel, util.getCovered(rel, util.getNamedEntityType()),
						util.getCovered(rel, util.getPOSType()), util.getCas());

				break;
			}

		}

		clearLayout();
		if (offsetStruct != null) {
			// override the modelobject
			this.struct = offsetStruct;
			overlay.setBackgroundImage(null);
			resetLayout();

		}

		else {

			clearLayout();
			initDummyLayout();

			overlay.layout();

		}

	}

	@Override
	public void initModel() {
		// no model in this view

	}

	@Override
	public boolean initTypes() {
		System.out.println("init types");

		// relationtype
		this.relationType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Relation");
		this.agensFeature = relationType.getFeatureByBaseName("Agens");
		this.agens2Feature = relationType.getFeatureByBaseName("Agens2");
		this.relationTypeFeature = relationType.getFeatureByBaseName("Type");
		this.relationAttributeFeature = relationType.getFeatureByBaseName("Attribut");
		return true;
	}

	private void initDummyLayout() {
		Button bDummy = new Button(super.overlay, SWT.PUSH);
		bDummy.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));
		bDummy.setText("Select a Relation...");
		bDummy.setEnabled(false);
		Font boldFont = new Font(Display.getCurrent(), new FontData("Segoe UI", 30, SWT.BOLD));

		bDummy.setFont(boldFont);
		Color white = new Color(Display.getCurrent(), new RGB(255, 255, 255));
		bDummy.setBackground(white);

		white.dispose();
	}

	private void resetLayout() {

		// adapt the layout to the new struct TODO

		// clear old layout
		clearLayout();

		// init the right composite
		initRelationComposite();

		// refresh
		overlay.layout();

	}

	private void initRelationComposite() {

		compRelations = new Composite(overlay, SWT.BORDER);
		compRelations.setLayout(new GridLayout(1, true));
		compRelations
				.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL));

		if (this.struct == null)
			return;
		// split layout into 2 parts - first is a comp of buttons to select the
		// person in the relation
		compAttPerson = new Composite(compRelations, SWT.NONE);
		compAttPerson.setLayout(new GridLayout(struct.getCoveredNEs().size() + 1, false));
		compAttPerson.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		compAttPersonRelation = new Composite(compRelations, SWT.NONE);

		compAttPersonRelation.setLayout(new GridLayout(7, false));
		compAttPersonRelation.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		// DUMMIES====

		GridData gdDummy = new GridData();
		gdDummy.horizontalSpan = 7;
		Button dummy1 = new Button(compAttPersonRelation, SWT.PUSH);
		dummy1.setVisible(false);
		dummy1.setLayoutData(gdDummy);

		Button dummy2 = new Button(compAttPersonRelation, SWT.PUSH);
		dummy2.setVisible(false);
		dummy2.setLayoutData(gdDummy);

		// ==LAYOUT
		for (AnnotationFS ne : struct.getCoveredNEs()) {

			final Button bNE = new Button(compAttPerson, SWT.PUSH);

			bNE.setFont(font);
			bNE.setText(ne.getCoveredText());
			bNE.setData(ACCORDING_NE, ne);

			bNE.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {

					// clear all lines from the old selection

					updateRelation((AnnotationFS) bNE.getData(ACCORDING_NE));

				}
			});

		}

		// also add a dummybutton to simulate the match to the speaker
		final Button bDummySpeaker = new Button(compAttPerson, SWT.PUSH);

		bDummySpeaker.setFont(font);
		bDummySpeaker.setText("Dummy-Speaker");

		// add a dummy ne with span 0,1
		AnnotationFS dummyNe = struct.getCas().createAnnotation(struct.getUtil().getNamedEntityType(), 0, 1);

		// set type to dummyspeaker
		dummyNe.setFeatureValueFromString(struct.getUtil().getNeFeatureType(), "DummySpeaker");
		bDummySpeaker.setData(ACCORDING_NE, dummyNe);

		bDummySpeaker.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// clear all lines from the old selection

				updateRelation((AnnotationFS) bDummySpeaker.getData(ACCORDING_NE));

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

			}
		});

		Label labelFrom = new Label(compAttPersonRelation, SWT.NONE);
		labelFrom.setAlignment(SWT.CENTER);
		labelFrom.setText("From:");
		labelFrom.setFont(font);

		Util_impl util = new Util_impl(struct.getCas());
		buttonFirstNE = new Button(compAttPersonRelation, SWT.PUSH);
		buttonFirstNE.setFont(font);

		AnnotationFS neAgens = (AnnotationFS) struct.getRelationAnno().getFeatureValue(util.getRelationAgensFeature());
		AnnotationFS neAgens2 = (AnnotationFS) struct.getRelationAnno()
				.getFeatureValue(util.getRelationAgens2Feature());
		String type = struct.getRelationAnno().getFeatureValueAsString(util.getRelationTypeFeature());

		if (neAgens == null) {
			buttonFirstNE.setText(" FROM ");
		} else if (neAgens.getBegin() == 0 && neAgens.getEnd() == 1) {

			buttonFirstNE.setText(neAgens.getFeatureValueAsString(struct.getUtil().getNeFeatureType()));
		} else {
			buttonFirstNE.setText(neAgens.getCoveredText());
			buttonFirstNE.setData(ACCORDING_NE, neAgens);
		}
		buttonFirstNE.setEnabled(false);

		Label labelTo = new Label(compAttPersonRelation, SWT.NONE);
		labelTo.setAlignment(SWT.CENTER);
		labelTo.setText("To:");
		labelTo.setFont(font);

		buttonSecondNe = new Button(compAttPersonRelation, SWT.PUSH);

		if (neAgens2 == null) {
			buttonSecondNe.setText(" TO ");
		} else if (neAgens2.getBegin() == 0 && neAgens2.getEnd() == 1) {
			buttonSecondNe.setText(neAgens2.getFeatureValueAsString(struct.getUtil().getNeFeatureType()));
		} else {
			buttonSecondNe.setText(neAgens2.getCoveredText());
			buttonSecondNe.setData(ACCORDING_NE, neAgens2);
		}
		buttonSecondNe.setEnabled(false);
		buttonSecondNe.setFont(font);

		Label labelType = new Label(compAttPersonRelation, SWT.NONE);
		labelType.setAlignment(SWT.CENTER);
		labelType.setText("Type:");
		labelType.setFont(font);

		textType = new Text(compAttPersonRelation, SWT.BORDER);
		textType.setFont(font);
		textType.setLayoutData(new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL));

		autoCompletetextType = new AutoCompleteField(textType, new TextContentAdapter(), new String[0]);

		if (type != null) {
			textType.setText(type);
		}

		Button buttonAdd = new Button(compAttPersonRelation, SWT.PUSH);
		buttonAdd.setFont(font);

		buttonAdd.setText("ADD");

		buttonAdd.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				Util_impl util = new Util_impl(struct.getCas());
				// set the features to the Anno
				AnnotationFS rel = struct.getRelationAnno();

				rel.setFeatureValue(util.getRelationAgensFeature(), (AnnotationFS) buttonFirstNE.getData(ACCORDING_NE));
				rel.setFeatureValue(util.getRelationAgens2Feature(),
						(AnnotationFS) buttonSecondNe.getData(ACCORDING_NE));

				rel.setFeatureValueFromString(util.getRelationTypeFeature(), textType.getText());

				// no attribute for a binary relation
				AnnotationFS attribute = (AnnotationFS) rel.getFeatureValue(util.getRelationAttributeFeature());
				if (attribute != null) {
					struct.getCas().removeFsFromIndexes(attribute);
				}

				editor.changeAnnotation(rel);
			}

		});

		compAttPersonRelation.pack();
		compAttPersonRelation.layout();
		compRelations.layout();

	}

	private void clearLayout() {

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

	}

	private void updateRelation(AnnotationFS ne) {

		if (firstButtonChangedLast) {
			if (ne.getBegin() == 0 && ne.getEnd() == 1) {
				buttonSecondNe.setText(ne.getFeatureValueAsString(struct.getUtil().getNeFeatureType()));
			} else {

				buttonSecondNe.setText(ne.getCoveredText());
			}
			firstButtonChangedLast = false;
			buttonSecondNe.setData(ACCORDING_NE, ne);
		}

		else {
			firstButtonChangedLast = true;

			if (ne.getBegin() == 0 && ne.getEnd() == 1) {
				buttonFirstNE.setText(ne.getFeatureValueAsString(struct.getUtil().getNeFeatureType()));
			} else {

				buttonFirstNE.setText(ne.getCoveredText());
			}

			buttonFirstNE.setData(ACCORDING_NE, ne);
		}

		compAttPersonRelation.pack();
		compAttPersonRelation.layout();
		compRelations.layout();

	}
}
