package de.uniwue.mk.kall.ruleAnalyzer.view.ui;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.osgi.service.event.Event;
import org.osgi.service.event.EventHandler;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.FeatureComparisionDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;

public class RuleAnalyzer implements IPerspectiveDecoupledPart {

	// CONSTANTS

	private String tpString = "de.uniwue.kalimachos.coref.type.TP";
	private String fnString = "de.uniwue.kalimachos.coref.type.FN";
	private String fpString = "de.uniwue.kalimachos.coref.type.FP";

	// Hashmap that maps all evaluation types to the rule annotations

	private HashMap<AnnotationFS, AnnotationFS> evalMapping;

	String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.1";

	// FIELDS======df

	private Composite composite;
	private LocalResourceManager resManager;
	@Inject
	private IEventBroker broker;
	@Inject
	EPartService partService;

	@Inject
	EModelService modelService;

	EventHandler editorAnnotationAddedHandler;
	EventHandler editorAnnotationRemovedHandler;
	EventHandler editorCaretChangedHandler;

	private boolean subscribed;

	private AnnotationEditorWidget editor;

	// Cas relevant stuff

	// the type that will be compared to the gold annotations
	// needs to have a stringlist feature with ruleexplanations
	private Type compareType;

	// this one exists, this is assured by the hierarchy in which the rule
	// annotions are
	private Feature explainListFeat;

	private String explainListString = "RuleExplainArray";

	// the gold type
	private Type toType;

	private Feature goldFeature;
	private Feature compFeature;

	private Type tpType;
	private Type fnType;
	private Type fpType;

	// Widgets

	private Text textCompare;
	private Label labelCompare;
	private Button buttonCompare;

	private Text textTo;
	private Label labelTo;
	private Button buttonTo;

	// elements to display features
	private Button buttonCompareFeatures;

	private ComboViewer compareCombo;
	private ComboViewer goldCombo;

	// shows all tps/fps etc...
	private TreeViewer viewer;

	private TreeViewer selectionTree;
	private Object part;
	private EventHandler onViewStackChangedhandler;
	private EventHandler onEditorInputHandler;
	private EventHandler onEditorClosedhandler;

	@Inject
	public RuleAnalyzer() {
		// TODO Your code here
	}

	public String getExplainListString() {
		return explainListString;
	}

	public void setExplainListString(String explainListString) {
		this.explainListString = explainListString;
	}

	public Feature getExplainListFeat() {
		return explainListFeat;
	}

	public HashMap<AnnotationFS, AnnotationFS> getEvalMapping() {
		return evalMapping;
	}

	public void setEvalMapping(HashMap<AnnotationFS, AnnotationFS> evalMapping) {
		this.evalMapping = evalMapping;
	}

	public void setExplainListFeat(Feature explainListFeat) {
		this.explainListFeat = explainListFeat;
	}

	public Type getToType() {
		return toType;
	}

	public void setToType(Type toType) {
		this.toType = toType;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Construct RuleAnalyzer");

		this.composite = parent;
		resManager = new LocalResourceManager(JFaceResources.getResources(), composite);

		// editor events

		this.composite.setLayout(new FillLayout());

		// ==========SUBSCRIBE TO EVENTS=============
		this.part = partService.findPart(viewId);

		// ==========SUBSCRIBE TO EVENTS=============

		onEditorClosedhandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event arg0) {
				onEditorClosed();

			}
		};

		onEditorInputHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event e) {

				RuleAnalyzer.this.onEditorInputChanged((AnnotationEditorWidget) e.getProperty(IEventBroker.DATA));

			}
		};

		onViewStackChangedhandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event e) {

				// TODO

				MPart findPart = partService.findPart(viewId);

				if (findPart == null) {
					return;
				}
				MElementContainer<MUIElement> parent = findPart.getParent();

				MUIElement selectedElement = parent.getSelectedElement();

				System.out.println(selectedElement.getElementId());
				if (!selectedElement.getElementId().equals(viewId)) {
					// this part is not visible => dont render it but
					// unsubscribe the
					// listener if they are
					// remove all listeners
					unsubscribeFromEvents();
					// dispose all widgets

					for (Control c : composite.getChildren()) {
						c.dispose();
					}

					toType = null;
					compareType = null;

				}

			}
		};

	}

	private void subscribeToEvents() {

		subscribed = true;
		editorAnnotationAddedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event e) {

				AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

				System.out.println(added.getCoveredText());
				try {
					renewPage();
				} catch (CASException ev) {
					ev.printStackTrace();
				}

			}
		};

		editorAnnotationRemovedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event arg0) {
				AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);
				try {
					renewPage();
				} catch (CASException e) {
					e.printStackTrace();
				}

			}
		};

		editorCaretChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(Event arg0) {

				int caretPosition = (int) arg0.getProperty(IEventBroker.DATA);

				if (tpType == null || fpType == null || fnType == null) {
					return;
				}

				// get the 3 Annotation indexes

				AnnotationIndex<AnnotationFS> tpIndex = editor.getCas().getAnnotationIndex(tpType);
				AnnotationIndex<AnnotationFS> fpIndex = editor.getCas().getAnnotationIndex(fpType);
				AnnotationIndex<AnnotationFS> fnIndex = editor.getCas().getAnnotationIndex(fnType);

				List<AnnotationFS> covered = new ArrayList<AnnotationFS>();

				for (AnnotationFS tp : tpIndex) {
					if (tp.getBegin() <= caretPosition && tp.getEnd() >= caretPosition) {
						covered.add(tp);
					}
				}

				for (AnnotationFS fp : fpIndex) {
					if (fp.getBegin() <= caretPosition && fp.getEnd() >= caretPosition) {
						covered.add(fp);
					}
				}

				for (AnnotationFS fn : fnIndex) {
					if (fn.getBegin() <= caretPosition && fn.getEnd() >= caretPosition) {
						covered.add(fn);
					}
				}

				//
				selectionTree.setInput(covered);
				try {
					renewPage();
				} catch (CASException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}

			}
		};

		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_ANNOTATION_ADDED,
				editorAnnotationAddedHandler);

		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_ANNOTATION_REMOVED,
				editorAnnotationRemovedHandler);

		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);

	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");
		if (!subscribed && this.editor != null) {
			onEditorInputChanged(this.editor);
		}

	}

	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		this.editor = editor;

		// only initialize if the part is visible

		String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.1";
		MPart findPart = partService.findPart(viewId);

		MUIElement selectedElement = null;
		if (findPart != null) {
			MElementContainer<MUIElement> parent = findPart.getParent();
			selectedElement = parent.getSelectedElement();

			if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
				// this part is not visible => dont render it but unsubscribe
				// the
				// listener if they are
				// remove all listeners
				unsubscribeFromEvents();
				return;
			}
		}

		// if the part is active add the listener

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : composite.getChildren()) {
			c.dispose();
		}

		// init AnnoList

		initLayout();

		if (!subscribed) {
			subscribeToEvents();
		}

		// annotate types if they are already set

		if (toType != null && compareType != null) {
			annotateEvaluationTypes();

			// also set the text fields
			this.textCompare.setText(compareType.getShortName());
			this.textTo.setText(toType.getShortName());
		}

		// ====reload=====

		composite.layout();
		composite.getParent().layout();
	}

	private void initLayout() {

		// composite gets three coloumns

		GridLayout layout = new GridLayout();

		layout.numColumns = 3;

		this.composite.setLayout(layout);
		this.composite.setLayoutData(new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL
				| GridData.GRAB_VERTICAL));

		// add a Label/textfield for the types that are about to be compared

		this.labelCompare = new Label(composite, SWT.NULL);
		this.labelCompare.setText("Compare: ");
		this.textCompare = new Text(composite, SWT.BORDER);
		this.buttonCompare = new Button(composite, SWT.PUSH);
		this.buttonCompare.setText("Choose Type");

		// add selection listener

		this.buttonCompare.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// open a dialog that allows to choose a type
				RuleAnalyzerTypeChoosingDialog dialog = new RuleAnalyzerTypeChoosingDialog(composite.getShell(),
						editor, false);
				int open = dialog.open();

				if (open == 0) {
					compareType = dialog.getSelectedType();
					textCompare.setText(compareType.getShortName());

					// also get the feature
					explainListFeat = compareType.getFeatureByBaseName(explainListString);
				}

				if (compareType != null && toType != null) {
					annotateEvaluationTypes();
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		this.labelTo = new Label(composite, SWT.NULL);
		this.labelTo.setText("To (Gold): ");
		this.textTo = new Text(composite, SWT.BORDER);
		this.buttonTo = new Button(composite, SWT.PUSH);
		this.buttonTo.setText("Choose Type");

		// add selection listener

		this.buttonTo.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// TODO Auto-generated method stub
				// open a dialog that allows to choose a type
				RuleAnalyzerTypeChoosingDialog dialog = new RuleAnalyzerTypeChoosingDialog(composite.getShell(),
						editor, false);
				int open = dialog.open();

				if (open == 0) {
					toType = dialog.getSelectedType();
					textTo.setText(toType.getShortName());
				}

				if (compareType != null && toType != null) {
					annotateEvaluationTypes();
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// feature button

		buttonCompareFeatures = new Button(composite, SWT.CHECK);
		GridData dd = new GridData();
		dd.horizontalSpan = 3;
		buttonCompareFeatures.setLayoutData(dd);
		buttonCompareFeatures.setText("Features: ");

		buttonCompareFeatures.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				//

				if (buttonCompareFeatures.getSelection()) {

					// add the Comboboxes

					// show all the primitive features of the selected types in
					// the combobox in the Combobox

					if (compareType != null) {

						List<Feature> features = compareType.getFeatures();

						goldCombo.setInput(features.toArray(new Feature[0]));
					}

					if (toType != null) {

						List<Feature> features = toType.getFeatures();

						compareCombo.setInput(features.toArray(new Feature[0]));

					}
					composite.layout();

				}

				else {

					// dispose the Comboboxes
					// RuleAnalyzer.this.compareCombo.getCombo().dispose();
					// RuleAnalyzer.this.goldCombo.getCombo().dispose();

				}

				composite.layout();

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		RuleAnalyzer.this.compareCombo = new ComboViewer(RuleAnalyzer.this.composite, SWT.NULL);
		RuleAnalyzer.this.goldCombo = new ComboViewer(RuleAnalyzer.this.composite, SWT.NULL);

		goldCombo.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection sel = (IStructuredSelection) goldCombo.getSelection();

				Feature f = (Feature) sel.getFirstElement();

				goldFeature = f;

				updateDrawingSytle();

			}
		});

		compareCombo.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {
				IStructuredSelection sel = (IStructuredSelection) compareCombo.getSelection();

				Feature f = (Feature) sel.getFirstElement();

				compFeature = f;

				updateDrawingSytle();

			}

		});

		// set the contenprovider
		RuleAnalyzer.this.compareCombo.setContentProvider(ArrayContentProvider.getInstance());

		compareCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof Feature) {
					Feature category = (Feature) element;
					return category.getShortName();
				}
				return "not found";
			}
		});

		RuleAnalyzer.this.goldCombo.setContentProvider(ArrayContentProvider.getInstance());
		goldCombo.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof Feature) {
					Feature category = (Feature) element;
					return category.getShortName();
				}
				return "not found";
			}
		});

		// ====Start tree creration
		GridData gridData = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		GridData data = new GridData();

		// /selection tree

		Composite treeCompSelection = new Composite(composite, SWT.BORDER);

		treeCompSelection.setLayout(new GridLayout());

		treeCompSelection.setLayoutData(data);

		gridData.horizontalSpan = 3;
		gridData.verticalSpan = 3;
		treeCompSelection.setLayoutData(gridData);

		selectionTree = new TreeViewer(treeCompSelection, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		selectionTree.getControl().setLayoutData(gridData);

		selectionTree.setContentProvider(new RuleAnalyzerSelectionTreeContentProvider(this));
		selectionTree
				.setLabelProvider(new RuleAnalyzerStyledLabelProvider(this.composite.getDisplay(), editor.getCas()));

		// full tp tree

		Composite treeComp = new Composite(composite, SWT.BORDER);

		treeComp.setLayout(new GridLayout());

		gridData = new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
		gridData.horizontalSpan = 3;
		treeComp.setLayoutData(gridData);

		viewer = new TreeViewer(treeComp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		viewer.getControl().setLayoutData(gridData);

		viewer.setContentProvider(new RuleAnalyzerContentProvider(this));
		viewer.setLabelProvider(new RuleAnalyzerStyledLabelProvider(this.composite.getDisplay(), editor.getCas()));

		createTreeViewer();

	}

	public AnnotationEditorWidget getEditor() {
		return editor;
	}

	public void setEditor(AnnotationEditorWidget editor) {
		this.editor = editor;
	}

	public Type getTpType() {
		return tpType;
	}

	public void setTpType(Type tpType) {
		this.tpType = tpType;
	}

	public Type getFnType() {
		return fnType;
	}

	public void setFnType(Type fnType) {
		this.fnType = fnType;
	}

	public Type getFpType() {
		return fpType;
	}

	public void setFpType(Type fpType) {
		this.fpType = fpType;
	}

	private void createTreeViewer() {

		viewer.setInput(evalMapping);
		viewer.refresh();

		selectionTree.refresh();
		selectionTree.expandAll();

		this.composite.layout();

	}

	private void annotateEvaluationTypes() {

		if (editor == null)
			return;

		this.tpType = editor.getCas().getTypeSystem().getType(tpString);
		this.fpType = editor.getCas().getTypeSystem().getType(fpString);
		this.fnType = editor.getCas().getTypeSystem().getType(fnString);

		deleteEvaluationTypes();
		evalMapping = new HashMap<AnnotationFS, AnnotationFS>();

		// add the new ones

		// check if the typesystem has POS types defined and if they are
		// available

		boolean tokenBased = false;
		String posTypeS = "de.uniwue.kalimachos.coref.type.POS";
		Type type = editor.getCas().getTypeSystem().getType(posTypeS);

		if (type != null) {

			int size = editor.getCas().getAnnotationIndex(type).size();

			if (size > 0) {
				// use POS Tags for a token based evaluation
				tokenBased = true;
			}
		}

		// TODO
		tokenBased = false;
		AnnotationIndex<AnnotationFS> goldIndex = editor.getCas().getAnnotationIndex(toType);

		AnnotationIndex<AnnotationFS> compareIndex = editor.getCas().getAnnotationIndex(compareType);

		ArrayList<AnnotationFS> goldList = new ArrayList<AnnotationFS>();

		for (AnnotationFS a : goldIndex) {

			if (tokenBased) {

				for (AnnotationFS token : editor.getCas().getAnnotationIndex(type)) {
					if (token.getBegin() >= a.getBegin() && token.getEnd() <= a.getEnd()) {
						goldList.add(token);
					}
				}
			}

			else {
				goldList.add(a);
			}
		}

		// a set to detect all fns
		HashSet<AnnotationFS> annoSet = new HashSet<AnnotationFS>();

		for (AnnotationFS comp : compareIndex) {

			boolean found = false;

			for (AnnotationFS gold : goldList) {

				if (gold.getBegin() == comp.getBegin() && gold.getEnd() == comp.getEnd()) {
					found = true;
					annoSet.add(gold);
					break;
				}

			}

			if (found == true) {
				AnnotationFS tp = editor.getCas().createAnnotation(tpType, comp.getBegin(), comp.getEnd());
				editor.getCas().addFsToIndexes(tp);

				evalMapping.put(tp, comp);
			}

			else {

				AnnotationFS fp = editor.getCas().createAnnotation(fpType, comp.getBegin(), comp.getEnd());
				editor.getCas().addFsToIndexes(fp);
				evalMapping.put(fp, comp);

			}
		}

		// last but not least the Fns

		for (AnnotationFS gold : goldList) {

			if (!annoSet.contains(gold)) {
				AnnotationFS fn = editor.getCas().createAnnotation(fnType, gold.getBegin(), gold.getEnd());

				editor.getCas().addFsToIndexes(fn);
				evalMapping.put(fn, gold);
			}
		}

		// change visibility

		// Visibility

		editor.changeVisibility(tpType, true);
		editor.changeVisibility(fnType, true);
		editor.changeVisibility(fpType, true);

		// layer

		editor.changeLayer(tpType, 2);
		editor.changeLayer(fpType, 3);
		editor.changeLayer(fnType, 3);

		// tp - green
		StyleRange rangeOv = new StyleRange();

		rangeOv.background = resManager.createColor(new RGB(0, 255, 0));
		AnnoStyle styleTP = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		// fp blue
		StyleRange rangeFP = new StyleRange();

		rangeFP.background = resManager.createColor(new RGB(35, 35, 255));
		AnnoStyle styleFP = new AnnoStyle(new BackgroundDrawingStrategy(rangeFP), rangeFP);

		// fn red
		StyleRange rangeFN = new StyleRange();

		rangeFN.background = resManager.createColor(new RGB(255, 0, 0));
		AnnoStyle styleFN = new AnnoStyle(new BackgroundDrawingStrategy(rangeFN), rangeFN);

		editor.changeStyle(tpType, styleTP);
		editor.changeStyle(fpType, styleFP);
		editor.changeStyle(fnType, styleFN);

		try {
			renewPage();
		} catch (CASException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	private void deleteEvaluationTypes() {

		if (this.tpType == null || this.fnType == null || this.fpType == null || this.editor == null) {
			return;
		}

		ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS tp : editor.getCas().getAnnotationIndex(tpType)) {
			toDelete.add(tp);
		}

		for (AnnotationFS fp : editor.getCas().getAnnotationIndex(fpType)) {
			toDelete.add(fp);
		}

		for (AnnotationFS fn : editor.getCas().getAnnotationIndex(fnType)) {
			toDelete.add(fn);
		}

		for (AnnotationFS anno : toDelete) {
			editor.getCas().removeFsFromIndexes(anno);
		}

		System.out.println("Old types deleted");
	}

	private void updateDrawingSytle() {

		if (goldFeature == null || compFeature == null) {
			return;
		}

		System.out.println("Update Comparision drawingstyle");

		// Visibility - we use this one as a fake comparison type
		editor.changeVisibility(this.toType, true);

		// layer

		editor.changeLayer(this.toType, 0);

		// yellow
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(composite.getDisplay(), 255, 255, 255);
		AnnoStyle style = new AnnoStyle(new FeatureComparisionDrawingStrategy(toType, compareType, goldFeature,
				compFeature, editor.getCas().getAnnotationIndex(toType), editor.getCas()
						.getAnnotationIndex(compareType)), rangeOv);

		// apply
		editor.changeStyle(toType, style);

		editor.setActiveMode(toType);

	}

	public void onEditorClosed() {

		// remove all components

		// delete all TP/FP/FN

		for (Control c : this.composite.getChildren()) {
			c.dispose();
		}

		// unsubscribe from editor notifications

		unsubscribeFromEvents();
		subscribed = false;
		this.editor = null;
	}

	public void renewPage() throws CASException {

		if (!subscribed) {
			return;
		}

		//

		// Runnable r = new Runnable() {
		//
		// @Override
		// public void run() {
		// createTreeViewer();
		// }
		// };
		//
		// // reload asynchronous
		// composite.getDisplay().asyncExec(r);

		createTreeViewer();

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	private void unsubscribeFromEvents() {

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_ANNOTATION_ADDED,
				editorAnnotationAddedHandler);

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_ANNOTATION_REMOVED,
				editorAnnotationRemovedHandler);

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_CARET_CHANGED,
				editorCaretChangedHandler);

		subscribed = false;

	}

	@Override
	public void coupleToPerspective() {

		subscribeToEvents();
		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedhandler);

		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedhandler);

		MasterPart.subscribeForEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputHandler);

	}

	@Override
	public void decoupleFromPerspective() {

		unsubscribeFromEvents();

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedhandler);

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_CLOSED_EVENT,
				onEditorClosedhandler);

		MasterPart.unsubscribeFromEvent(RuleAnalyzer.class, IAnnotationEditorEvents.EDITOR_INPUT_CHANGED,
				onEditorInputHandler);
	}

}