package de.uniwue.mk.kall.ruleAnalyzer.view.ui;

import java.util.HashMap;

import org.apache.uima.cas.FeatureStructure;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.AnnotationImpl;
import org.apache.uima.cas.impl.StringArrayFSImpl;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

public class RuleAnalyzerContentProvider implements ITreeContentProvider {

	private RuleAnalyzer analyzer;

	public RuleAnalyzerContentProvider(RuleAnalyzer analyzer) {

		this.analyzer = analyzer;
	}

	@Override
	public void dispose() {
	}

	@Override
	public Object[] getElements(Object inputElement) {

		if (inputElement instanceof HashMap<?, ?>) {
			// should return 3 elements tp/fp/fn

			Type[] elements = new Type[3];

			elements[0] = analyzer.getTpType();
			elements[1] = analyzer.getFpType();
			elements[2] = analyzer.getFnType();

			return elements;

		}

		return new Object[0];
	}

	@Override
	public Object[] getChildren(Object parentElement) {

		// if (parentElement instanceof IAnnotationProject) {
		//
		// IAnnotationProject proj = (IAnnotationProject) parentElement;
		//
		// File[] fArr = new File[3];
		//
		// fArr[0] = proj.getDescFolder();
		// fArr[1] = proj.getInputFolder();
		// fArr[2] = proj.getOutputFolder();
		//
		// return fArr;
		//
		// }

		if (parentElement instanceof Type) {
			AnnotationIndex<AnnotationFS> annotationIndex = analyzer.getEditor().getCas()
					.getAnnotationIndex((Type) parentElement);

			AnnotationFS[] indexArr = new AnnotationFS[annotationIndex.size()];

			int index = 0;
			for (AnnotationFS a : annotationIndex) {
				indexArr[index] = a;
				index++;
			}
			return indexArr;
		}

		else if (parentElement instanceof AnnotationImpl) {
			AnnotationFS anno = (AnnotationFS) parentElement;

			// that one has to exist !!
			if (analyzer.getEvalMapping().containsKey(anno)) {
				AnnotationFS type = analyzer.getEvalMapping().get(anno);
				if (type.getType().equals(analyzer.getToType())) {
					return new Object[0];
				}
				FeatureStructure stringArray = type.getFeatureValue(analyzer.getExplainListFeat());

				if (stringArray instanceof StringArrayFSImpl) {
					StringArrayFSImpl array = (StringArrayFSImpl) stringArray;

					return array.toStringArray();

				}
			}

			else {
				return new Object[0];
			}

		}

		return new Object[0];
	}

	@Override
	public Object getParent(Object element) {

		// TODO this seems wrong

		if (element instanceof Type) {

			return null;
		}

		else if (element instanceof AnnotationFS) {
			AnnotationFS a = (AnnotationFS) element;

			return a.getType();
		}

		return new Object[0];
	}

	@Override
	public boolean hasChildren(Object element) {

		if (element instanceof Type) {
			AnnotationIndex<AnnotationFS> annotationIndex = analyzer.getEditor().getCas()
					.getAnnotationIndex((Type) element);
			return annotationIndex.size() > 0;
		}

		if (element instanceof AnnotationFS) {
			return true;
		}

		return false;
	}

	@Override
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub

	}

}
