package de.uniwue.mk.kall.ruleAnalyzer.view.ui;

import java.net.URL;
import java.util.HashMap;

import javax.annotation.PreDestroy;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;
import org.osgi.framework.Bundle;
import org.osgi.framework.FrameworkUtil;

public class RuleAnalyzerStyledLabelProvider extends StyledCellLabelProvider {

	HashMap<Type, Image> typesHash = new HashMap<Type, Image>();
	private TreeViewer viewer;
	private GC gc;

	private CAS cas;

	private Display dis;

	public RuleAnalyzerStyledLabelProvider(Display dis, CAS cas) {

		this.dis = dis;
		this.cas = cas;
	}

	@Override
	public void update(ViewerCell cell) {

		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof Type) {
			Type t = (Type) element;
			text.append((t.getShortName()) + " (" + cas.getAnnotationIndex(t).size() + " )");
			setImage(cell, element);

		}

		else if (element instanceof AnnotationFS) {
			AnnotationFS a = (AnnotationFS) element;

			text.append(a.getCoveredText() + " ");
			setImage(cell, element);

		}

		else if (element instanceof String) {
			text.append((String) element);
		}

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	private void setImage(ViewerCell cell, Object element) {

		Type toInput = null;

		if (element instanceof AnnotationFS) {
			toInput = ((AnnotationFS) element).getType();
		}

		else if (element instanceof Type) {

			toInput = (Type) element;
		}

		Image im = null;

		if (typesHash.containsKey(toInput)) {

			im = typesHash.get(toInput);
		}

		else {

			im = new Image(dis, 16, 16);

			typesHash.put(toInput, im);

		}

		gc = new GC(im);

		Color black = new Color(dis, new RGB(222, 222, 222));
		gc.setBackground(black);

		String substring = toInput.getName();

		String[] split = substring.split("\\.");

		String name = split[split.length - 1];

		if (name.length() > 2) {
			name = name.substring(0, 2);
		}

		name = name.toUpperCase();

		gc.fillRectangle(0, 0, 16, 16);
		// gc.drawRectangle(0, 0, 16, 16);
		gc.drawString(name, 0, 0);
		cell.setImage(im);

		gc.dispose();
		black.dispose();

	}

	@Focus
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	@PreDestroy
	public void dispose() {
	}

	private Image createImage(String image) {
		Bundle bundle = FrameworkUtil.getBundle(RuleAnalyzerStyledLabelProvider.class);
		URL url = FileLocator.find(bundle, new Path(image), null);
		ImageDescriptor imageDcr = ImageDescriptor.createFromURL(url);
		return imageDcr.createImage();
	}
}
