package de.uniwue.mk.kall.wsdView;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.FSIterator;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.apache.uima.cas.text.AnnotationIndex;
import org.apache.uima.ruta.rule.AnnotationComparator;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.part.editor.NameFilter;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.WSDDrawingStrategy;

public class WordSenseDisambiguationView extends AEditorSubordinateViewPart {

	@Inject
	static EPartService partService;

	@Inject
	EModelService modelService;

	private static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.5";

	private Composite suggestionComp;

	// TODO a dirty boolean for a quick fix
	private boolean userIndependentJumpFeature = false;

	public boolean isUserIndependentJumpFeature() {
		return userIndependentJumpFeature;
	}

	public void setUserIndependentJumpFeature(boolean userIndependentJumpFeature) {
		this.userIndependentJumpFeature = userIndependentJumpFeature;
	}

	// WIDGETS
	// germaNET
	String[] germaNetCategories = new String[] { "Tops", "Artefakt", "Attribut", "Besitz", "Relation", "Geschehen",
			"Form", "Gefuehl", "Gruppe", "Koerper", "Kognition", "Kommunikation", "Menge", "Mensch", "Motiv",
			"Nahrung", "natGegenstand", "natPhaenomen", "Ort", "Pflanze", "Substanz", "Tier", "Zeit", "Unbekannt" };

	// germaNet
	private HashMap<String, List<String>> germaNetCandidates;
	private HashMap<String, Image> imageMapping;

	private TableViewer viewer;
	private NameFilter filter;
	private WSDCategoryFilter idFilter;
	private UnderlyingWSDAnnotationFilter ovFilter;

	// DATA CONTAINER
	private List<AnnotationFS> annoList;
	public HashMap<String, Integer> freqMap;
	private Type typ;

	private String type;

	// E4 DI OBJECTS
	@Inject
	static IEventBroker broker;

	public Button getJumpButton() {
		return jumpButton;
	}

	public void setJumpButton(Button jumpButton) {
		this.jumpButton = jumpButton;
	}

	private Type typOverride;

	private Type posType;

	private String typeOverride;

	private Feature wsdCategory;

	private Feature wsdCandidateFeature;

	// compounds

	private ComboViewer wsdCategoryviewer;
	private Text neuEnt;

	private Text oldEnt;

	private Button box;

	Button jumpButton;

	Button coreButton;

	Button notCoreButton;

	private Button idButton;

	private static final Class<WordSenseDisambiguationView> CLASS_THIS = WordSenseDisambiguationView.class;

	@Inject
	public WordSenseDisambiguationView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

		// read germanet

		try {
			readGermanet();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	private void readGermanet() throws IOException {
		germaNetCandidates = new HashMap<String, List<String>>();

		InputStream germaNetRessource = ApplicationUtil.getBundleRessourceAsStream("/ressources/nounCategories.txt");

		BufferedReader reader = new BufferedReader(new InputStreamReader(germaNetRessource));

		String line = null;

		while ((line = reader.readLine()) != null) {

			String[] arr = line.split("\t");

			if (germaNetCandidates.containsKey(arr[0])) {

				List<String> list = germaNetCandidates.get(arr[0]);

				list.add(arr[1]);

			}

			else {

				List<String> categories = new ArrayList<String>();

				categories.add(arr[1]);

				germaNetCandidates.put(arr[0], categories);
			}

		}

		reader.close();

	}

	@Override
	public boolean initTypes() {

		type = ApplicationUtil.getFromPreferences(ApplicationPreferences.WSDView_WSD_Type, "null");
		typeOverride = ApplicationUtil.getFromPreferences(ApplicationPreferences.WSDView_WSD_OVERRIDE_Type, "null");
		String posStringType = ApplicationUtil.getFromPreferences(ApplicationPreferences.WSDView_WSD_POS_Type, "null");

		if (type.equals("null") || typeOverride.equals("null") || posStringType.equals("null")) {
			return false;
		}

		this.typ = editor.getCas().getTypeSystem().getType(type);
		this.posType = editor.getCas().getTypeSystem().getType(posStringType);

		if (this.typ == null || this.posType == null) {
			return false;
		}

		// set everything unnneded invisible

		this.editor.allStylesInvisible();

		// get Features of NE Type

		String wsdCategoryText = ApplicationUtil.getFromPreferences(ApplicationPreferences.WSDView_WSD_FEATURE, "null");
		String wsdCandidatesString = ApplicationUtil.getFromPreferences(
				ApplicationPreferences.WSDView_WSD_CANDIDATE_FEATURE, "null");
		if (wsdCategoryText.equals("null") || wsdCandidatesString.equals("null")) {
			return false;
		}

		// TODO null handling
		this.wsdCategory = this.typ.getFeatureByBaseName(wsdCategoryText);
		this.wsdCandidateFeature = this.typ.getFeatureByBaseName(wsdCandidatesString);

		// Override Type and Temporary Type
		this.typOverride = (editor.getCas().getTypeSystem().getType(typeOverride));

		// Visibility

		editor.changeVisibility(typ, true);
		editor.changeVisibility(typOverride, true);

		// layer

		editor.changeLayer(typ, 0);
		// editor.changeLayer(temporary, 0);
		editor.changeLayer(typOverride, 1);

		// yellow
		StyleRange rangeOv = new StyleRange();

		// TODO color needs to be disposed
		rangeOv.background = new Color(overlay.getDisplay(), 224, 136, 20);
		AnnoStyle style2 = new AnnoStyle(new BackgroundDrawingStrategy(rangeOv), rangeOv);

		// style - red
		StyleRange rangeTyp = new StyleRange();
		rangeTyp.background = new Color(overlay.getDisplay(), 255, 0, 0);
		AnnoStyle style = new AnnoStyle(
				new WSDDrawingStrategy(rangeTyp, wsdCategoryText, editor.getCas(), typOverride), rangeTyp);

		// apply
		editor.changeStyle(typ, style);
		// editor.changeStyle(temporary, style3);
		editor.changeStyle(typOverride, style2);

		// also change the modus
		editor.setActiveMode(typ);

		return true;

	}

	@Override
	protected void initLayout(Composite parent) {
		// this.overlay = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout();
		layout.horizontalSpacing = 5;
		layout.numColumns = 2;
		layout.verticalSpacing = 5;
		layout.marginWidth = 5;
		layout.marginHeight = 5;
		overlay.setLayout(layout);

		// / ==== Buttons + Fields

		Composite comp2 = new Composite(overlay, SWT.BORDER);
		comp2.setVisible(true);
		GridLayout gdd = new GridLayout();
		GridData gridData2 = new GridData();
		gridData2.horizontalSpan = 2;
		gridData2.grabExcessHorizontalSpace = true;
		// gridData2.grabExcessVerticalSpace = true;
		gridData2.horizontalAlignment = GridData.FILL;
		comp2.setLayout(gdd);
		comp2.setLayoutData(gridData2);

		// Label actions = new Label(comp2, SWT.NULL);
		// actions.setText("Actions");
		// Composite comp = new Composite(overlay, SWT.BORDER);
		// comp.setVisible(true);

		GridLayout gridBelow = new GridLayout(2, false);
		comp2.setLayout(gridBelow);
		Button newEntity = new Button(comp2, SWT.NONE);
		newEntity.setText("New Entity:");
		newEntity.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// remove vorschlag
				CAS cas = editor.getCas();
				Type t = cas.getTypeSystem().getType(type);
				// get the selection

				if (selected == null) {

					Point selection = editor.getActualSelectedText();
					AnnotationFS anno = cas.createAnnotation(t, selection.x, selection.y);

					IStructuredSelection sel = (IStructuredSelection) wsdCategoryviewer.getSelection();
					anno.setFeatureValueFromString(wsdCategory, (String) sel.getFirstElement());

					// TODO use radio buttons for the selection

					addAnnotation(anno);

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

				widgetSelected(e);

			}
		});

		this.neuEnt = new Text(comp2, SWT.BORDER);
		neuEnt.setText("TODO");
		neuEnt.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		Button oldEntity = new Button(comp2, SWT.NONE);
		oldEntity.setText("Is Entity");

		oldEntity.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// old means we create an annotation AND set the id to the
				// selected old
				if (selected != null) {

					// also update the name if necessary if the ID is already
					// present

					// document.update(selected);
					String cat = selected.getFeatureValueAsString(wsdCategory);
					updateAnnotation(selected, wsdCategory, cat, true);
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		this.oldEnt = new Text(comp2, SWT.BORDER);
		// TODO needs a category listener
		oldEnt.setText("Category");
		oldEnt.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		Button noEntity = new Button(comp2, SWT.NONE);
		noEntity.setText("No Entity");
		noEntity.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS selected = null;
				if (editor.getSelectionOfVisibleAnnotations().size() > 0) {
					for (AnnotationFS anno : editor.getSelectionOfVisibleAnnotations()) {
						if (anno.getType().equals(typ)) {
							selected = anno;

						}
					}
				}

				// get boolean of box
				if (box.getSelection()) {

					// we delete all

				}

				else {
					// we delete only the one
					if (selected != null) {

						deleteAnnotation(selected);
						// /deleteOverrideType(selected);
						// this.editor.getDocument().removeFeatureStructure(selected);

					}
				}
				// this.editor.selectAndReveal(3, 7);

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {

				widgetSelected(e);

			}
		});
		box = new Button(comp2, SWT.CHECK);
		box.setText("Delete All");

		Composite composite = new Composite(overlay, SWT.NULL);
		gdd = new GridLayout();
		gdd.numColumns = 3;
		gdd.marginBottom = gdd.marginTop = gdd.marginLeft = gdd.marginRight = 0;
		composite.setLayout(gdd);

		// combobox with germanet categories
		wsdCategoryviewer = new ComboViewer(composite, SWT.BORDER);

		wsdCategoryviewer.setContentProvider(ArrayContentProvider.getInstance());
		wsdCategoryviewer.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof String) {
					String category = (String) element;
					return category;
				}
				return super.getText(element);
			}
		});

		// add selectionlistener

		wsdCategoryviewer.getCombo().addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// set the wsdFeature of the selected Annotation if possible

				if (editor != null && editor.getSelectionOfVisibleAnnotations().size() == 1) {

					AnnotationFS selection = editor.getSelectionOfVisibleAnnotations().get(0);

					IStructuredSelection comboSel = (IStructuredSelection) wsdCategoryviewer.getSelection();

					updateAnnotation(selection, wsdCategory, (String) comboSel.getFirstElement(), true);

				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		// static GermaNET Categories
		GridData l = new GridData();
		l.horizontalSpan = 2;
		wsdCategoryviewer.getCombo().setLayoutData(l);
		wsdCategoryviewer.setInput(germaNetCategories);

		// // ==== Table
		comp2.setSize(overlay.getSize());

		Composite compp = new Composite(overlay, SWT.NULL);

		compp.setVisible(true);

		GridLayout lay = new GridLayout(2, false);
		compp.setLayout(lay);

		// will contains lots of images
		suggestionComp = new Composite(compp, SWT.FILL);

		GridLayout lay2 = new GridLayout();
		lay2.numColumns = 10;
		suggestionComp.setLayout(lay2);

		GridData suggCompData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL);
		suggCompData.horizontalSpan = 4;
		suggCompData.verticalSpan = 1;
		suggCompData.grabExcessHorizontalSpace = true;
		suggestionComp.setLayoutData(suggCompData);

		Color white = new Color(suggestionComp.getDisplay(), new RGB(255, 255, 255));
		suggestionComp.setBackground(white);
		white.dispose();

		Label idFilterLabel = new Label(compp, SWT.NONE);
		idFilterLabel.setText("Category Filter active: ");
		idButton = new Button(compp, SWT.CHECK);
		idButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		idButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				renewPage();
			}
		});

		// no jump option !
		Label noJumpLabel = new Label(compp, SWT.NONE);
		noJumpLabel.setText("Jump to next?: ");
		jumpButton = new Button(compp, SWT.CHECK);
		jumpButton.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));
		jumpButton.setSelection(true);

		// searchLabel
		Label searchLabel = new Label(compp, SWT.NONE);
		searchLabel.setText("Search: ");
		final Text searchText = new Text(compp, SWT.BORDER | SWT.SEARCH);
		searchText.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.HORIZONTAL_ALIGN_FILL));

		viewer = new TableViewer(compp, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		// add Menu
		// MenuListener
		final Menu menu = new Menu(viewer.getControl());
		viewer.getControl().setMenu(menu);

		// Delete
		MenuItem del = new MenuItem(menu, SWT.None);

		del.setText("Delete Selected Annotation");

		del.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				deleteAnnotation(firstElement);

			}

		});

		// Category
		MenuItem pseudo = new MenuItem(menu, SWT.None);

		pseudo.setText("Change Category (Strg + p)");

		pseudo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();
				String string = firstElement.getFeatureValueAsString(wsdCategory);

				if (string == null) {
					string = "true";
				}
				string = string.equals("true") ? "false" : "true";

				// TODO open a dialogue ...
				// updateAnnotation(firstElement, wsdCategory, string, true);

			}

		});

		// GoTo

		MenuItem goTo = new MenuItem(menu, SWT.None);

		goTo.setText("GoTo Selected Annotation");

		goTo.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent event) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS firstElement = (AnnotationFS) sel.getFirstElement();

				int beg = firstElement.getBegin();

				changeHighlighting(beg, firstElement.getEnd(), true);

			}

		});

		// KEYLISTENER
		// enter listener to update the values
		viewer.getTable().addKeyListener(new KeyAdapter() {

			@Override
			public void keyPressed(KeyEvent e) {
				if (((IStructuredSelection) viewer.getSelection()).getFirstElement() == null) {
					return;
				}

				if (e.keyCode == SWT.CR) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
					AnnotationFS anno = (AnnotationFS) selection.getFirstElement();
					String id = anno.getFeatureValueAsString(wsdCategory);
					ArrayList<AnnotationFS> list = new ArrayList<AnnotationFS>();

					for (AnnotationFS a : annoList) {
						if (a.getFeatureValueAsString(wsdCategory).equals(id)) {
							list.add(a);
						}
					}

				}

				// numerusFeat
				else if (((e.stateMask & SWT.CTRL) == SWT.CTRL) && (e.keyCode == 'u')) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();

					AnnotationFS selected = (AnnotationFS) selection.getFirstElement();

					String numerusValue = selected.getFeatureValueAsString(wsdCategory);
					if (numerusValue == null) {
						numerusValue = "pl";
					}
					if (numerusValue.equals("pl")) {
						numerusValue = "si";
					}

					else {
						numerusValue = "pl";
					}

					// TODO dialogue
					// updateAnnotation(selected, wsdCategory, numerusValue,
					// true);

					viewer.setSelection(selection, true);

				}

			}
		});

		// MOUSE DOUBLE CLICK LISTENER
		viewer.getTable().addListener(SWT.MouseDoubleClick, new Listener() {

			@Override
			public void handleEvent(Event event) {
				// get the selected item
				String id = null;
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				AnnotationFS selected = (AnnotationFS) sel.getFirstElement();
				if (selected == null) {
					return;
				}

				id = selected.getFeatureValueAsString(wsdCategory);

				// get the selection in the cas editor

				Point pSelect = editor.getActualSelectedText();

				if (pSelect.y - pSelect.x == 0) {
					// here we update the Features of the annotation

					AnnotationFS anno = editor.getSelectionOfVisibleAnnotations().get(0);

					// use the last Feature change to reload the page
					updateAnnotation(anno, wsdCategory, id, true);
				}

				else {

					if (editor.getSelectionOfVisibleAnnotations().size() == 1) {
						// update the Annotation
						AnnotationFS ano = editor.getSelectionOfVisibleAnnotations().get(0);

						if (ano.getType().getName().equals(typ.getName())) {

							updateAnnotation(ano, wsdCategory, id, true);

						}

						else {

							// create An Annotation with the correct span and
							// ids
							AnnotationFS anno = editor.getCas().createAnnotation(typ, editor.getActualSelectedText().x,
									editor.getActualSelectedText().y);

							anno.setFeatureValueFromString(wsdCategory, id);

							addAnnotation(anno);

						}
					}

					else {

						// create An Annotation with the correct span and ids
						AnnotationFS anno = editor.getCas().createAnnotation(typ, editor.getActualSelectedText().x,
								editor.getActualSelectedText().y);

						anno.setFeatureValueFromString(wsdCategory, id);

						addAnnotation(anno);
					}
				}

			}

		});

		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// content
		viewer.setContentProvider(new ArrayContentProvider());
		GridData gridData = new GridData(GridData.FILL_VERTICAL);
		gridData.horizontalSpan = 2;
		gridData.grabExcessHorizontalSpace = true;
		gridData.grabExcessVerticalSpace = true;
		gridData.horizontalAlignment = GridData.FILL;
		viewer.getControl().setLayoutData(gridData);
		compp.setLayoutData(gridData);
		// New to support the search

		searchText.addKeyListener(new KeyAdapter() {
			public void keyReleased(KeyEvent ke) {
				filter.setSearchText(searchText.getText());
				viewer.refresh();
			}

		});

		// filter
		filter = new NameFilter();
		viewer.addFilter(filter);

		// id filter

		ovFilter = new UnderlyingWSDAnnotationFilter();
		viewer.addFilter(ovFilter);

		idFilter = new WSDCategoryFilter();
		viewer.addFilter(idFilter);

		createTableViewer();
		// comparator

		// this.colViewerComp = new ColViewerComparator(nameFeat, idFeat, this);
		// viewer.setComparator(colViewerComp);

		// ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);

		// dragn drop
		// int operations = DND.DROP_COPY | DND.DROP_MOVE;
		// Transfer[] transferTypes = new Transfer[] {
		// LocalSelectionTransfer.getTransfer() };
		// viewer.addDragSupport(operations, transferTypes, new
		// TableViewerDragListener(viewer));
		// viewer.addDropSupport(operations, transferTypes, new
		// TableViewerDropListener(viewer, this));

	}

	private void createTableViewer() {

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(typeOverride));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		ArrayList<AnnotationFS> overrides = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			overrides.add((AnnotationFS) iterator.next());
		}

		// set input of Tableviewer
		viewer.setInput(annoList);
		idFilter.setAnnotations(annoList, wsdCategory, idButton.getSelection(), overrides);

		ovFilter.setAnnotations(overrides);

		viewer.refresh();
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("WordText", 100, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getCoveredText();

			}

		});

		TableViewerColumn colID = createTableViewerColumn("Category", 50, 1);
		colID.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				AnnotationFS p = (AnnotationFS) element;
				return p.getFeatureValueAsString(wsdCategory);
			}
		});

		TableViewerColumn synonyme = createTableViewerColumn("Synonyme", 40, 2);
		synonyme.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return null;
			}

			@Override
			public Image getImage(Object element) {
				AnnotationFS anno = (AnnotationFS) element;

				String cat = anno.getFeatureValueAsString(wsdCategory);

				if (cat == null || cat.length() == 0) {
					cat = "empty";
				}

				if (imageMapping.containsKey(cat)) {
					return imageMapping.get(cat);
				}

				else {

					if (cat.equals("empty")) {

						Image im = new Image(editor.getParent().getDisplay(), 16, 16);

						GC gc = new GC(im);

						gc.setBackground(new Color(editor.getParent().getDisplay(), new RGB(255, 0, 0)));

						gc.fillRectangle(0, 0, 16, 16);

						gc.dispose();

						imageMapping.put(cat, im);
						return im;
					}

					else {
						Image createImage = ApplicationUtil.createImage(ApplicationUtil.imageMapping.get(cat));
						imageMapping.put(cat, createImage);

						return createImage;
					}
				}

			}

		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	private void createCandidates() {

		AnnotationIndex<AnnotationFS> germIndex = editor.getCas().getAnnotationIndex(typ);

		if (germIndex.size() == 0) {
			// create candidates

			for (AnnotationFS token : editor.getCas().getAnnotationIndex(posType)) {

				if (germaNetCandidates.containsKey(token.getCoveredText())) {
					List<String> list = germaNetCandidates.get(token.getCoveredText());

					StringBuilder sb = new StringBuilder();

					AnnotationFS createAnnotation = editor.getCas().createAnnotation(typ, token.getBegin(),
							token.getEnd());
					editor.getCas().addFsToIndexes(createAnnotation);
					if (list.size() == 1) {

						// set category
						createAnnotation.setFeatureValueFromString(wsdCategory, list.get(0));

					}

					else {
						createAnnotation.setFeatureValueFromString(wsdCategory, list.get(0));

						for (String s : list) {
							sb.append(s).append("#");
						}

						createAnnotation.setFeatureValueFromString(wsdCandidateFeature, sb.toString());
					}
				}

				else {

					// all NN or NE have to be marked as well

					// TODO dirty
					Feature posTag = this.posType.getFeatureByBaseName("POSTag");

					String tag = token.getFeatureValueAsString(posTag);

					if (tag.equals("NN") || tag.equals("NE")) {

						AnnotationFS noun = editor.getCas().createAnnotation(typ, token.getBegin(), token.getEnd());
						editor.getCas().addFsToIndexes(noun);

					}

				}
			}
		}

	}

	public void renewPage() {
		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(type));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		//

		createTableViewer();

	}

	// Methods to change the CAS
	private void deleteAnnotation(AnnotationFS firstElement) {

		// delete all overrides in that stage

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typOverride);
		ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
		for (AnnotationFS anno : annotationIndex) {
			if (anno.getBegin() == firstElement.getBegin() && anno.getEnd() == firstElement.getEnd()) {
				toDelete.add(anno);
				// editor.removeAnnotation(anno);
			}
		}

		for (AnnotationFS a : toDelete) {
			editor.removeAnnotation(a);
		}

		editor.removeAnnotation(firstElement);

	}

	private void updateAnnotation(AnnotationFS firstElement, Feature feat, String string, boolean reload) {

		firstElement.setFeatureValueFromString(feat, string);

		// delete all overrides in that stage
		if (reload == true) {

			AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(typOverride);
			ArrayList<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();
			for (AnnotationFS anno : annotationIndex) {
				if (anno.getBegin() == firstElement.getBegin() && anno.getEnd() == firstElement.getEnd()) {
					toDelete.add(anno);
				}
			}

			for (AnnotationFS a : toDelete) {
				editor.removeAnnotation(a);
			}
		}

		if (reload) {
			AnnotationFS fss = editor.getCas().createAnnotation(typOverride, firstElement.getBegin(),
					firstElement.getEnd());

			this.editor.addAnnotation(fss);
		}
	}

	// boolean moveCursor
	private void changeHighlighting(int beg, int end, boolean b) {
		editor.highlightRange(beg, end - beg, true);

	}

	private void addAnnotation(AnnotationFS anno) {

		// add Annotation - means we add an Annotation of type NER also an
		// Override Type
		// to display that this annotation was already handled

		this.editor.addAnnotation(anno);

		// add Annotations for the same Text

		// =====END INTERACTIVE RULES=========

		// add the override Annotation
		CAS cas = this.editor.getCas();

		AnnotationFS fs = cas.createAnnotation(typOverride, anno.getBegin(), anno.getEnd());

		this.editor.addAnnotation(fs);

	}

	@Override
	public void onAnnotationAdded(org.osgi.service.event.Event e) {
		AnnotationFS added = (AnnotationFS) e.getProperty(IEventBroker.DATA);

		renewPage();

		if (getJumpButton().getSelection()) {

			// TODO dirty solution
			if (isUserIndependentJumpFeature() == false) {
				AnnotationFS toHIghlight = getNextToHighLight(added);

				if (toHIghlight != null) {
					editor.highlightRange(toHIghlight.getBegin(), toHIghlight.getEnd() - toHIghlight.getBegin(), true);

					editor.setSelection(toHIghlight);
				}
			}

			else {
				setUserIndependentJumpFeature(false);
			}
		}

	}

	@Override
	public void onAnnotationRemoved(org.osgi.service.event.Event arg0) {
		AnnotationFS removed = (AnnotationFS) arg0.getProperty(IEventBroker.DATA);
		renewPage();

		if (getJumpButton().getSelection()) {
			AnnotationFS toHIghlight = getNextToHighLight(removed);

			if (toHIghlight != null) {
				editor.highlightRange(toHIghlight.getBegin(), toHIghlight.getEnd() - toHIghlight.getBegin(), true);

				editor.setSelection(toHIghlight);
			}
		}

	}

	@Override
	public void onAnnotationChanged(org.osgi.service.event.Event arg0) {
		// TODO Auto-generated method stub

	}

	private AnnotationFS getNextToHighLight(AnnotationFS actual) {

		Collections.sort(this.annoList, new AnnotationComparator());

		for (int i = 0; i < annoList.size(); i++) {
			if (actual.getBegin() < annoList.get(i).getBegin()) {
				return annoList.get(i);
			}
		}
		return null;

	}

	@Override
	public void onAnnotationSelectionChanged(org.osgi.service.event.Event arg0) {
		Object property = arg0.getProperty(IEventBroker.DATA);

		if (property instanceof List) {

			if (editor != null && editor.getSelectionOfVisibleAnnotations().size() == 1) {

				AnnotationFS annotationFS = editor.getSelectionOfVisibleAnnotations().get(0);

				// delete all children from suggComp

				if (suggestionComp != null) {
					for (Control c : suggestionComp.getChildren()) {
						c.dispose();
					}
				}

				if (germaNetCandidates.containsKey(annotationFS.getCoveredText())) {
					List<String> list = germaNetCandidates.get(annotationFS.getCoveredText());

					// get images from categories
					HashSet<String> unique = new HashSet<String>();
					for (String s : list) {
						if (!unique.contains(s)) {
							unique.add(s);
						} else {
							continue;
						}

						Image image = imageMapping.get(s);

						Label l = new Label(suggestionComp, SWT.BORDER);
						l.setImage(image);

					}

					suggestionComp.layout();
					suggestionComp.getParent().layout();

				}
				String cat = annotationFS.getFeatureValueAsString(wsdCategory);

				if (cat != null) {

					// update the isEntity Field
					WordSenseDisambiguationView.this.oldEnt.setText(cat);
					WordSenseDisambiguationView.this.wsdCategoryviewer.setSelection(new StructuredSelection(cat));
					// WSDView.this.germanetSuggestions
				}
			}

			else {
				WordSenseDisambiguationView.this.oldEnt.setText("");
			}

		}

	}

	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

		// set the Textfield
		neuEnt.setText(editor.getWidget().getText().substring(point.x, point.y));

		// if we got the span of an annotation also create the selection
		// in the
		// tableviewer

		List<AnnotationFS> annoList = this.annoList;

		for (AnnotationFS anno : annoList) {
			if (anno.getBegin() <= point.x && anno.getEnd() >= point.x) {
				StructuredSelection structuredSelection = new StructuredSelection(anno);
				viewer.setSelection(structuredSelection);
				break;
			}
		}

	}

	@PreDestroy
	public void preDestroy() {
		// TODO Your code here
	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	@Persist
	public void save() {
		// TODO Your code here
	}

	@Override
	public void initModel() {

		createCandidates();

		AnnotationIndex<AnnotationFS> annotationIndex = editor.getCas().getAnnotationIndex(
				editor.getCas().getTypeSystem().getType(type));

		FSIterator<AnnotationFS> iterator = annotationIndex.iterator();
		annoList = new ArrayList<AnnotationFS>();
		while (iterator.hasNext()) {
			annoList.add((AnnotationFS) iterator.next());
		}

		// ====FREQUENCIES=======
		freqMap = new HashMap<String, Integer>();
		for (AnnotationFS anno : annoList) {
			if (freqMap.containsKey(anno.getCoveredText())) {
				int freq = freqMap.get(anno.getCoveredText());
				freqMap.put(anno.getCoveredText(), freq + 1);
			}

			else {
				freqMap.put(anno.getCoveredText(), 1);
			}
		}

	}

}
