package de.uniwue.mk.medIE.extraction.algorithm;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.apache.uima.cas.CAS;

import de.uniwue.mk.medIE.terminology.MedicalTerminology;

public abstract class ATerminologyMatchingAlgorithm {
	public static final String EVENT_PROGRESS_CHANGED = "progressChanged";
	public static final String EVENT_TOTALWORK_CHANGED = "totalWorkChanged";
	
	private int progress = 0;
	private int totalWork = 0;
	private PropertyChangeSupport pcs = new PropertyChangeSupport(this);
	
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		pcs.addPropertyChangeListener(listener);
	}
	
	public void removePropertyChangeListener(PropertyChangeListener listener) {
		pcs.removePropertyChangeListener(listener);
	}
	
	protected void setTotalWork(int work) {
		int oldVal = totalWork;
		totalWork = work;
		pcs.firePropertyChange(EVENT_TOTALWORK_CHANGED, oldVal, totalWork);
	}
	
	public int getTotalWork() {
		return totalWork;
	}
	
	protected void increaseProgress(int additionalProgress) {
		int oldValue = progress;
		progress += additionalProgress;
		pcs.firePropertyChange(EVENT_PROGRESS_CHANGED, oldValue, progress);
	}
	
	public int getProgress() {
		return progress;
	}
	
	protected void resetProgress() {
		progress = 0;
	}
	
	/**
	 * Resets the current progress and sets totalWork. <br/>
	 * Should be called at the beginning of {@link #apply(MedicalTerminology, CAS)}. 
	 * @param totalWork
	 */
	protected void initProgress(int totalWork) {
		resetProgress();
		setTotalWork(totalWork);
	}
	

	/**
	 * Applies the given MedicalTerminology to the given CAS. <br/>
	 * Should call {@link #initProgress(int)} at the beginning and report progress 
	 * via {@link #increaseProgress(int)} during the execution of the algorithm. 
	 * @param terminology
	 * @param cas
	 */
	public abstract void apply(MedicalTerminology terminology, CAS cas);

}
