package de.uniwue.mk.medIE.extraction.algorithm;

import java.util.List;
import java.util.regex.Pattern;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.text.AnnotationFS;

import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mkrug.kall.typesystemutil.Util_impl;

public class SimpleTerminologyMatchingAlgorithm extends ATerminologyMatchingAlgorithm {

	@Override
	public void apply(MedicalTerminology terminology, CAS cas) {

		Util_impl util = new Util_impl(cas);

		// match all concepts to the text
		for (AnnotationFS token : cas.getAnnotationIndex(util.getPOSType())) {

			// match the terminology
			matchTerminologyToToken(token, terminology, util, cas);
		}

		// second step is to assign the values to attributes

		for (AnnotationFS value : cas.getAnnotationIndex(util.getMedicalValueType())) {

			// we use the information provided by the dependency parser
			AnnotationFS accordingAttribute = null;

			AnnotationFS father = util.getFather(value);

			while (father != null) {

				if (getCoveringAttribute(father, util, cas) != null) {
					accordingAttribute = getCoveringAttribute(father, util, cas);
					break;
				}

				father = util.getFather(father);
			}

			if (accordingAttribute != null) {
				// set feature values TODO

			}
		}

	}

	private AnnotationFS getCoveringAttribute(AnnotationFS father, Util_impl util, CAS cas) {

		for (AnnotationFS att : cas.getAnnotationIndex(util.getMedicalAttributeType())) {

			if (att.getBegin() <= father.getBegin() && att.getEnd() >= father.getEnd()) {
				return att;
			}
		}
		return null;
	}

	private void matchTerminologyToToken(AnnotationFS token, MedicalTerminology terminology, Util_impl util, CAS cas) {

		List<Concept> concepts = terminology.getRoot().getSuccessors();

		for (Concept concept : concepts) {

			matchConceptToToken(token, concept, util, cas);
		}

	}

	private void matchConceptToToken(AnnotationFS token, Concept concept, Util_impl util, CAS cas) {

		List<String> regexes = concept.getSynonymsAsRegex();

		for (String regex : regexes) {

			if (Pattern.matches(regex, token.getCoveredText())) {

				// create annotation
				if (concept.isAttribute()) {

					AnnotationFS att = cas.createAnnotation(util.getMedicalAttributeType(), token.getBegin(),
							token.getEnd());

					att.setFeatureValueFromString(util.getMedicalAttributeConceptId(), String.valueOf(concept.getId()));

					cas.addFsToIndexes(att);
				}

				else if (concept.isValue()) {

					AnnotationFS val = cas.createAnnotation(util.getMedicalValueType(), token.getBegin(),
							token.getEnd());

					val.setFeatureValueFromString(util.getMedicalValueConceptId(), String.valueOf(concept.getId()));
					cas.addFsToIndexes(val);
				}
			}
		}
	}
}
