package de.uniwue.mk.medIE.extraction.terminology.conversion;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.ENodeType;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mk.medIE.terminology.Variation;

public class TerminologyConverter {

	private static final String CONCEPT_XML_ELEMENT = "CONCEPT_XML_ELEMENT";
	private static final String REGEX_NUMBER = "\\d+([,\\.]\\d+)?";

	/*
	 * XML conversion with plain xml methodology
	 */
	public static MedicalTerminology convertXTLToMedTerm(File xmlToConvert) throws ParserConfigurationException,
			SAXException, IOException {

		Concept root = new Concept("ROOT", null, null, 0);
		root.setRoot(true);
		MedicalTerminology convertedTerminology = new MedicalTerminology(root, xmlToConvert.getName());

		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

		docBuilderFactory.setValidating(false);
		docBuilderFactory.setNamespaceAware(true);
		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		Document document = docBuilder.parse(xmlToConvert);

		NodeList conceptElements = document.getElementsByTagName("Concept");

		List<Concept> rawConcepts = createRawConceptsByElements(conceptElements, convertedTerminology);

		return convertedTerminology;

	}

	// some sort of ugly excel format conversion, probably useless except for
	// one use case
	@SuppressWarnings("resource")
	public static MedicalTerminology convertCSVToMedTerm(File csvFile) throws IOException {

		MedicalTerminology terminology = new MedicalTerminology(null, "Hernien");

		Concept root = new Concept("ROOT", null, null, 0);
		terminology.setRoot(root);
		root.setRoot(true);

		List<String> lines = Files.readAllLines(csvFile.toPath(), StandardCharsets.ISO_8859_1);

		HashMap<String, List<String>> nodeNames = new HashMap<String, List<String>>();
		int rowIndex = 0;
		for (String line : lines) {

			if (rowIndex == 0) {
				rowIndex++;
				continue;
			}
			rowIndex++;

			String[] split = line.split(";");

			for (int i = 0; i < split.length - 1; i++) {

				String nodeName = split[i];
				if (nodeName.trim().isEmpty())
					continue;
				ArrayList<String> value = new ArrayList<String>();
				value.add(split[split.length - 1]);

				if (nodeNames.containsKey(nodeName)) {
					nodeNames.get(nodeName).addAll(value);
				} else {

					nodeNames.put(nodeName, value);
				}
			}

			// create the concepts

		}
		int index = 1;
		for (String attribute : nodeNames.keySet()) {
			Concept c = new Concept(attribute, root, null, terminology.getNextFreeId());
			root.addChild(c);

			c.setId(index);
			index++;
			c.setNodeType(ENodeType.CHOICE);
			c.setPureExistence(true);
			for (String synonym : nodeNames.get(attribute)) {

				List<Variation> synonyms = parseToSynonyms(synonym);
				for (Variation v : synonyms)
					c.addSynonym(v);

			}
		}
		return terminology;
	}

	private static List<Variation> parseToSynonyms(String synonym) {

		List<Variation> vars = new ArrayList<Variation>();
		// easiest case
		if (!synonym.contains("*")) {
			Variation v = new Variation(synonym);
			v.setRegex(true);
			vars.add(v);
			return vars;
		}
		String[] words = synonym.split(" ");

		// convert all the words to regex
		List<String> revisedWords = new ArrayList<String>();
		for (String s : words) {

			if (s.startsWith("*")) {
				StringBuilder sb = new StringBuilder(s);
				sb.setCharAt(1, Character.toLowerCase(s.charAt(1)));

				s = sb.toString();

				s = "[^ ]" + s;
			}

			if (s.endsWith("*")) {
				s = s.substring(0, s.length() - 1) + "[a-zäöü]*";
			}
			revisedWords.add(s);
		}

		// all permutations -YAY! -finally another search

		List<String> states = new ArrayList<String>();

		states.addAll(revisedWords);

		for (int i = 0; i < words.length; i++) {

			// for each given state - try to add a new state
			List<String> toAdd = new ArrayList<String>();
			for (String s : states) {

				for (String revW : revisedWords) {
					if (!s.contains(revW)) {
						String newS = s + ".*?" + revW;
						toAdd.add(newS);
					}
				}
			}

			states.addAll(toAdd);
		}

		Set<String> revisedStates = new HashSet<String>();
		for (String s : states) {

			boolean isComplete = true;

			for (String w : revisedWords) {
				if (!s.contains(w)) {
					isComplete = false;
					break;
				}
			}

			if (isComplete) {
				revisedStates.add(s);
			}
		}

		// finally build the states

		for (String s : revisedStates) {
			Variation v = new Variation(s);
			v.setRegex(true);
			vars.add(v);
		}
		return vars;
	}

	private static List<Concept> createRawConceptsByElements(NodeList conceptElements,
			MedicalTerminology convertedTerminology) {

		List<Concept> rawConcepts = new ArrayList<Concept>();
		for (int i = 0; i < conceptElements.getLength(); i++) {

			Node concept = conceptElements.item(i);
			NamedNodeMap attributes = concept.getAttributes();

			// TODO was ist davon das richtige???
			String conceptName = attributes.getNamedItem("displayName").getNodeValue();
			String id = attributes.getNamedItem("id").getNodeValue();
			id += conceptName;

			Concept c = new Concept(conceptName, null, null, i + 1);
			c.setProperty("CONCEPT_FORMER_ID", id);
			c.setProperty(CONCEPT_XML_ELEMENT, concept);

			// also add some attributes of that concept

			// the type
			String nodeValue = attributes.getNamedItem("type").getNodeValue();

			if (nodeValue.equals("structure")) {
				c.setStructure(true);
			}

			if (nodeValue.equals("object_attribute")) {
				c.setObjectAttribute(true);
			}

			ENodeType type = mapToNodeType(nodeValue);

			if (type != null) {
				c.setNodeType(type);
			}

			// Variants
			List<Variation> variants = getVariantsToConcept(concept);

			for (Variation v : variants) {
				c.addVariation(v);
			}

			rawConcepts.add(c);

		}

		// now try to handle the parent/children relation!!
		for (Concept c : rawConcepts) {

			Node conceptNode = (Node) c.getProperty(CONCEPT_XML_ELEMENT);

			// get the parent of the conceptNode
			Node parentNode = conceptNode.getParentNode().getParentNode();

			String parentId = parentNode.getAttributes().getNamedItem("id").getNodeValue();

			if (parentId.equals("0")) {
				convertedTerminology.getRoot().addChild(c);
				c.setParent(convertedTerminology.getRoot());
				c.setProperty(CONCEPT_XML_ELEMENT, null);
				c.changeAttribute("Is Attribute");

				continue;
			}

			parentId += parentNode.getAttributes().getNamedItem("displayName").getNodeValue();

			// now find the concept that has this id attached
			Concept parentConcept = null;
			for (Concept conc : rawConcepts) {

				String idConc = (String) conc.getProperty("CONCEPT_FORMER_ID");
				if (idConc.equals(parentId)) {
					parentConcept = conc;
					break;
				}
			}

			if (parentConcept != null) {

				if (c.getName().startsWith("Wert (") && !c.getName().contains("]")) {
					System.out.println(parentConcept.getName());
					String uni = c.getName().split("\\(")[1];
					uni = uni.substring(0, uni.length() - 1);

					parentConcept.setNodeType(ENodeType.UNIT);

					Variation v = new Variation(uni);
					v.setUnit(true);
					parentConcept.addVariation(v);
					// no children - force it!!
					parentConcept.setChildren(new ArrayList<Concept>());
				}

				else if (c.getName().endsWith("]")) {

					String uni = c.getName().split("\\[")[1];
					uni = uni.substring(0, uni.length() - 1);

					parentConcept.setNodeType(ENodeType.UNIT);

					Variation v = new Variation(uni);
					v.setUnit(true);
					parentConcept.addVariation(v);
					// no children - force it!!
					parentConcept.setChildren(new ArrayList<Concept>());

				} else {

					parentConcept.addChild(c);
					c.setParent(parentConcept);
					c.setProperty(CONCEPT_XML_ELEMENT, null);
				}
			}

			else {

			}
		}

		return rawConcepts;
	}

	private static List<Variation> getVariantsToConcept(Node concept) {

		List<Variation> vars = new ArrayList<Variation>();
		NodeList childNodes = concept.getChildNodes();

		for (int i = 0; i < childNodes.getLength(); i++) {

			Node item = childNodes.item(i);

			if (item.getNodeName().equals("Variants")) {
				NodeList childNodesVar = item.getChildNodes();

				for (int k = 0; k < childNodesVar.getLength(); k++) {

					Node item2 = childNodesVar.item(k);

					NamedNodeMap attributes = item2.getAttributes();

					if (item2.getNodeName().equals("Variant")) {
						String varname = attributes.getNamedItem("name").getNodeValue();
						String varType = attributes.getNamedItem("type").getNodeValue();

						varname = varname.replaceAll("XX", REGEX_NUMBER);
						String varNameTemp = varname;
						if (varname.endsWith("YY")) {
							varname = varname.replaceAll("YY", "");
						}
						Variation v = new Variation(varname);

						// modify varname to get rid of the super ugly YY and XX
						// who even thinks of adding this function like that
						// should get punished for eternity !!

						if (varNameTemp.endsWith("YY")) {
							v.setRespectEndings(true);

						}

						if (varType.equals("regExp")) {
							v.setRegex(true);
						}

						vars.add(v);
					}
				}
			}
		}
		return vars;
	}

	private static ENodeType mapToNodeType(String nodeValue) {

		if (nodeValue.startsWith("numeric")) {
			return ENodeType.NUMERIC;
		}
		return ENodeType.CHOICE;
	}
}
