package de.uniwue.mk.medIE.terminology;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement(namespace = "de.uniwue.mk.medIE.terminology")
public class MedicalTerminology {

	private Concept root;
	private String name;

	private Lexikon lexikon;

	public MedicalTerminology(Concept root, String name) {
		super();
		this.root = root;
		this.name = name;

		LexikonEntry lexikonEntry = new LexikonEntry("Negation", new Variation("nicht"), new Variation("nie"));
		LexikonEntry lexikonEntry2 = new LexikonEntry("Position", new Variation("plus"), new Variation("sowieso"));
		this.lexikon = new Lexikon(lexikonEntry, lexikonEntry2);
	}

	public Lexikon getLexikon() {
		return lexikon;
	}

	public void setLexikon(Lexikon lexikon) {
		this.lexikon = lexikon;
	}

	public MedicalTerminology() {

	}

	public Concept getRoot() {
		return root;
	}

	public void setRoot(Concept root) {
		this.root = root;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public void deleteNode(Concept concept) {

		if (concept == null)
			return;

		// get All children
		List<Concept> children = root.getSuccessors();
		children.add(root);

		for (Concept c : children) {

			if (c != null && c.hasChild(concept)) {
				c.removeChild(concept);
			}
		}

	}

	public int getNextFreeId() {

		List<Concept> successors = root.getSuccessors();
		successors.add(root);

		int max = -1;

		for (Concept c : successors) {
			if (c.getId() > max)
				max = c.getId();
		}
		return max + 1;

	}

	public List<Concept> getAttributeNodes() {

		List<Concept> successors = root.getSuccessors();

		List<Concept> attributes = new ArrayList<Concept>();

		for (Concept c : successors) {
			if (c.isAttribute()) {
				attributes.add(c);
			}
		}

		return attributes;

	}

	public List<Concept> getValuesToConcept(Concept attribute) {
		List<Concept> successors = attribute.getSuccessors();

		List<Concept> values = new ArrayList<Concept>();

		for (Concept c : successors) {
			if (c.isValue()) {
				values.add(c);
			}
		}
		return values;
	}

}
