package de.uniwue.mk.medIE.terminology;

import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

import de.uniwue.mk.medIE.stemming.germanStemmer;

public class Variation {

	private String synonym;

	private boolean isRegex;
	private boolean respectEndings;
	private boolean useStemming;
	private boolean isUnit;
	private boolean isLexikon;
	private boolean isCaseSensitive;

	// name of the lexikonentry
	private String entry;

	public Variation() {
		super();
	}

	public Variation(String synonym) {
		super();
		this.synonym = synonym;
	}

	public String getSynonym() {
		return synonym;
	}

	public void setSynonym(String synonym) {
		this.synonym = synonym;
	}

	public boolean isRegex() {
		return isRegex;
	}

	public void setRegex(boolean isRegex) {
		this.isRegex = isRegex;
	}

	public boolean isRespectEndings() {
		return respectEndings;
	}

	public void setRespectEndings(boolean respectEndings) {
		this.respectEndings = respectEndings;
	}

	public boolean isUseStemming() {
		return useStemming;
	}

	public void setUseStemming(boolean useStemming) {
		this.useStemming = useStemming;
	}

	public boolean isUnit() {
		return isUnit;
	}

	public void setUnit(boolean isUnit) {
		this.isUnit = isUnit;
	}

	public Collection<? extends String> getVariationsAsList() {
		Set<String> regexes = new HashSet<String>();

		// all synonyms
		regexes.add("\\b" + Pattern.quote(synonym) + "\\b");

		if (isRegex) {
			regexes.add("\\b" + synonym + "\\b");
		}

		if (respectEndings) {
			regexes.add("\\b" + synonym + "e" + "\\b");
			regexes.add("\\b" + synonym + "er" + "\\b");
			regexes.add("\\b" + synonym + "es" + "\\b");
			regexes.add("\\b" + synonym + "en" + "\\b");
			regexes.add("\\b" + synonym + "em" + "\\b");
			regexes.add("\\b" + synonym + "s" + "\\b");
		}

		if (useStemming) {

			germanStemmer stemmer = new germanStemmer();

			stemmer.setCurrent(synonym);

			stemmer.stem();
			regexes.add("\\b" + stemmer.getCurrent() + "\\b");
		}

		if (!isCaseSensitive) {
			Character firstChar = synonym.charAt(0);

			if (Character.isLowerCase(firstChar)) {
				Character uc = Character.toUpperCase(firstChar);

				String modified = uc.toString() + synonym.substring(1);

				regexes.add(modified);
			}
			firstChar = synonym.charAt(0);

			if (Character.isUpperCase(firstChar)) {
				Character lc = Character.toLowerCase(firstChar);

				String modified = lc.toString() + synonym.substring(1);

				regexes.add(modified);
			}
		}

		return regexes;
	}

	// returns all attributes that are set
	public String getCategoriesAsString() {
		String category = "";

		if (isRegex)
			category += "REGEX|";

		if (respectEndings)
			category += "ENDINGS|";

		if (useStemming)
			category += "STEMMING|";

		if (isUnit)
			category += "UNIT|";

		if (isLexikon) {
			category += "LEXIKON|";
		}

		if (isCaseSensitive) {
			category += "CASE_SENSITIVE";
		}

		if (category.endsWith("|")) {
			category = category.substring(0, category.length() - 1);
		}

		return category;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((synonym == null) ? 0 : synonym.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Variation other = (Variation) obj;
		if (synonym == null) {
			if (other.synonym != null)
				return false;
		} else if (!synonym.equals(other.synonym))
			return false;
		return true;
	}

	public boolean isLexikon() {
		return isLexikon;
	}

	public void setLexikon(boolean isLexikon) {
		this.isLexikon = isLexikon;
	}

	public String getEntry() {
		return entry;
	}

	public void setEntry(String entry) {
		this.entry = entry;
	}

	public boolean isCaseSensitive() {
		return isCaseSensitive;
	}

	public void setCaseSensitive(boolean isCaseSensitive) {
		this.isCaseSensitive = isCaseSensitive;
	}

}
