package de.uniwue.mk.medIE.terminologyView;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;

public class AttributeAcceptanceDialog extends Dialog {

	private Shell shell;
	private IEventBroker broker;
	private MedicalTerminology terminology;

	private Button buttonAccept;
	private Button buttonRemove;
	private Button buttonEnlargeLeft;
	private Button buttonEnlargeRight;
	private Button buttonShrinkRight;
	private Button buttonShrinkLeft;
	private Button buttonHighLightNext;
	private Button buttonHighlightPrevious;
	private AnnotationEditorWidget editor;
	private Type conceptType;
	private Feature featConceptType;

	public AttributeAcceptanceDialog(Shell parentShell, MedicalTerminology terminology, IEventBroker broker,
			AnnotationEditorWidget editor, Type conceptType, Feature featConceptType) {
		super(parentShell);

		this.shell = parentShell;

		this.broker = broker;

		this.conceptType = conceptType;

		this.featConceptType = featConceptType;

		this.editor = editor;

		this.terminology = terminology;

	}

	protected Control createDialogArea(Composite parent) {

		parent.setLayout(new GridLayout(2, true));

		buttonAccept = new Button(parent, SWT.PUSH);
		buttonAccept.setText("Accept");
		buttonAccept.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonAccept.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS actualSelection = getActualeditorSelection();

				if (actualSelection == null)
					return;

				Concept root = terminology.getRoot();

				// add a new Concept as child
				Concept child = new Concept(editor.getWidget().getSelectionText(), root, null, terminology
						.getNextFreeId());

				root.addChild(child);

				broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, root);

				updateAcceptedAnnos(actualSelection.getCoveredText());

				AnnotationFS next = getNextAnnoToHighLight(actualSelection);

				if (next != null) {
					editor.highlightRange(next.getBegin(), next.getEnd() - next.getBegin(), true);
				}

			}

		});

		buttonRemove = new Button(parent, SWT.PUSH);
		buttonRemove.setText("Remove");
		buttonRemove.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonRemove.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();

				AnnotationFS actualSelection = getActualeditorSelection();

				if (actualSelection == null)
					return;

				deleteAnnosWithText(selectionText);

				AnnotationFS next = getNextAnnoToHighLight(actualSelection);

				if (next != null) {
					editor.highlightRange(next.getBegin(), next.getEnd() - next.getBegin(), true);
				}

			}

		});

		buttonEnlargeLeft = new Button(parent, SWT.PUSH);
		buttonEnlargeLeft.setText("Expand left");
		GridData gdBtEnlargeLeft = new GridData(GridData.FILL, SWT.CENTER, false, false);
		gdBtEnlargeLeft.verticalIndent = 15;
		buttonEnlargeLeft.setLayoutData(gdBtEnlargeLeft);
		buttonEnlargeLeft.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();

				deleteAnnosWithText(selectionText);

				Point p = editor.getWidget().getSelection();
				System.out.println(p);

				int lineAtOffset = editor.getWidget().getLineAtOffset(p.x - 1);
				int lineAtOffsetNow = editor.getWidget().getLineAtOffset(p.x);

				if (lineAtOffset == lineAtOffsetNow) {
					editor.highlightRange((p.x - 1) >= 0 ? p.x - 1 : 0, (p.x - 1) >= 0 ? (p.y - p.x + 1) : (p.y - p.x),
							true);
					addAnnosWithText(editor.getWidget().getSelectionText());
				}

			}

		});

		buttonEnlargeRight = new Button(parent, SWT.PUSH);
		buttonEnlargeRight.setText("Expand right");
		GridData gdBtEnlargeRight = new GridData(GridData.FILL, SWT.CENTER, false, false);
		gdBtEnlargeRight.verticalIndent = 15;
		buttonEnlargeRight.setLayoutData(gdBtEnlargeRight);
		buttonEnlargeRight.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();

				deleteAnnosWithText(selectionText);

				Point p = editor.getWidget().getSelection();
				int textlen = editor.getWidget().getText().length();
				editor.highlightRange(p.x, (p.y + 1) < textlen ? p.y + 1 - p.x : p.y - p.x, true);

				addAnnosWithText(editor.getWidget().getSelectionText());

			}

		});

		buttonShrinkLeft = new Button(parent, SWT.PUSH);
		buttonShrinkLeft.setText("Shrink Left");
		buttonShrinkLeft.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonShrinkLeft.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();

				deleteAnnosWithText(selectionText);

				Point p = editor.getWidget().getSelection();
				int newLeft = (p.x + 1) < p.y ? p.x + 1 : p.x;
				editor.highlightRange(newLeft, p.y - newLeft, true);

				addAnnosWithText(editor.getWidget().getSelectionText());

			}
		});

		buttonShrinkRight = new Button(parent, SWT.PUSH);
		buttonShrinkRight.setText("Shrink Right");
		buttonShrinkRight.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonShrinkRight.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String selectionText = editor.getWidget().getSelectionText();

				deleteAnnosWithText(selectionText);

				Point p = editor.getWidget().getSelection();
				int newRight = (p.y - 1) > p.x ? p.y - 1 : p.y;
				editor.highlightRange(p.x, newRight - p.x, true);

				addAnnosWithText(editor.getWidget().getSelectionText());

			}
		});

		buttonHighlightPrevious = new Button(parent, SWT.PUSH);
		buttonHighlightPrevious.setText("ToPrevious");
		GridData gdBtPrevious = new GridData(GridData.FILL, SWT.CENTER, false, false);
		gdBtPrevious.verticalIndent = 15;
		buttonHighlightPrevious.setLayoutData(gdBtPrevious);
		buttonHighlightPrevious.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				AnnotationFS previous = getPreviousAnnoToHighLight(editor.getWidget().getCaretOffset());

				if (previous != null) {
					editor.highlightRange(previous.getBegin(), previous.getEnd() - previous.getBegin(), true);
				}
			}

		});

		buttonHighLightNext = new Button(parent, SWT.PUSH);
		buttonHighLightNext.setText("ToNext");
		GridData gdBtNext = new GridData(GridData.FILL, SWT.CENTER, false, false);
		gdBtNext.verticalIndent = 15;
		buttonHighLightNext.setLayoutData(gdBtPrevious);
		buttonHighLightNext.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				AnnotationFS next = getNextAnnoToHighLight(editor.getWidget().getCaretOffset());

				if (next != null) {
					editor.highlightRange(next.getBegin(), next.getEnd() - next.getBegin(), true);
				}

			}

		});

		return parent;

	}

	private void addAnnosWithText(String selectionText) {

		Matcher m = Pattern.compile(Pattern.quote(selectionText)).matcher(editor.getCas().getDocumentText());

		List<AnnotationFS> toAdd = new ArrayList<AnnotationFS>();
		while (m.find()) {

			AnnotationFS fs = editor.getCas().createAnnotation(conceptType, m.start(), m.end());
			fs.setFeatureValueFromString(featConceptType, "ATTRIBUTE_GOLD");

			toAdd.add(fs);

		}

		for (AnnotationFS add : toAdd) {
			editor.getCas().addFsToIndexes(add);
		}

		if (!toAdd.isEmpty()) {
			editor.changeAnnotation(toAdd.get(0));
		}

	}

	private AnnotationFS getNextAnnoToHighLight(AnnotationFS selectedAnno) {

		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

			if (a.getBegin() > selectedAnno.getBegin() && a.getType().equals(conceptType)) {

				String featType = a.getFeatureValueAsString(featConceptType);

				if (featType.contains("_GOLD"))
					continue;
				return a;
			}
		}
		return null;
	}

	private AnnotationFS getPreviousAnnoToHighLight(int caretOffset) {

		AnnotationFS old = null;
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

			String featType = a.getFeatureValueAsString(featConceptType);
			if (a.getBegin() > caretOffset && a.getType().equals(conceptType)) {

				return old;
			}

			if (featType.contains("_GOLD"))
				continue;
			old = a;
		}
		return null;
	}

	private AnnotationFS getNextAnnoToHighLight(int caretOffset) {

		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

			if (a.getBegin() > caretOffset && a.getType().equals(conceptType)) {

				String featType = a.getFeatureValueAsString(featConceptType);

				if (featType.contains("_GOLD"))
					continue;
				return a;
			}
		}
		return null;

	}

	private AnnotationFS getActualeditorSelection() {

		for (AnnotationFS sel : editor.getSelectionOfVisibleAnnotations()) {

			if (sel.getType().toString().equals(conceptType.toString()))
				return sel;
		}
		return null;
	}

	private void updateAcceptedAnnos(String aceptedText) {

		List<AnnotationFS> updated = new ArrayList<AnnotationFS>();

		for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {

			if (concept.getCoveredText().equals(aceptedText)) {

				concept.setFeatureValueFromString(featConceptType, "ATTRIBUTE_GOLD");
				updated.add(concept);
			}
		}

		editor.changeAnnotations(updated);

	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

	private void deleteAnnosWithText(String selectionText) {

		List<AnnotationFS> toRemove = new ArrayList<AnnotationFS>();

		for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {

			if (concept.getCoveredText().equals(selectionText)) {
				toRemove.add(concept);
			}
		}

		for (AnnotationFS toDel : toRemove) {
			editor.getCas().removeFsFromIndexes(toDel);
		}

		// update if there have been changes
		if (!toRemove.isEmpty()) {
			editor.removeAnnotation(toRemove.get(0));
		}

	}
}
