package de.uniwue.mk.medIE.terminologyView;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.fieldassist.AutoCompleteField;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.ELexikonEntryType;
import de.uniwue.mk.medIE.terminology.LexikonEntry;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mk.medIE.terminology.Variation;
import de.uniwue.mk.medIE.terminologyView.widgets.NodeAttributeComposite;
import de.uniwue.mk.medIE.terminologyView.widgets.VariationEditingComposite;

public class ConceptEditingDialog extends Dialog {

	private StyledText widget;
	private Button buttonAddSynonym;
	private Button buttonSetContext;
	private StyledText textAddSynonym;

	private Shell shell;

	private Concept concept;

	private Text textKontext;
	private ComboViewer comboConceptReference;

	private MedicalTerminology terminology;
	private IEventBroker broker;
	private VariationEditingComposite comp;
	private AutoCompleteField autoCompletetextType;

	public ConceptEditingDialog(Shell parentShell, Concept conceptToEdit, MedicalTerminology terminology,
			IEventBroker broker) {
		super(parentShell);

		this.shell = parentShell;

		this.concept = conceptToEdit;
		this.broker = broker;

		this.terminology = terminology;

	}

	protected Control createDialogArea(Composite parent) {

		parent.getShell().setText("Edit the concept " + concept.getName());

		GridLayout layout = new GridLayout(6, false);
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		parent.setLayout(layout);

		Font font = new Font(parent.getDisplay(), "Segoe UI", 10, SWT.NORMAL);

		Label labelName = new Label(parent, SWT.NONE);
		labelName.setText(String.format("Name (ID = %s): ", concept.getId()));
		labelName.setFont(font);
		widget = new StyledText(parent, SWT.BORDER);
		widget.setBackground(Display.getCurrent().getSystemColor(SWT.COLOR_TITLE_BACKGROUND_GRADIENT));
		widget.setAlignment(SWT.CENTER);

		widget.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				concept.setName(widget.getText());

			}

		});

		NodeAttributeComposite compAttributes = new NodeAttributeComposite(parent, SWT.NONE, broker, 22);
		compAttributes.setConcept(concept);

		GridData layoutData2 = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		layoutData2.horizontalSpan = 4;
		Label labelKontext = new Label(parent, SWT.NONE);
		labelKontext.setFont(font);
		labelKontext.setText("Context:");

		textKontext = new Text(parent, SWT.BORDER);
		textKontext.setFont(font);
		textKontext.setLayoutData(layoutData2);

		if (concept.getContext() != null) {
			textKontext.setText(concept.getContext().getName());
		}

		List<String> lexikaNames = new ArrayList<String>();

		for (LexikonEntry entry : terminology.getLexikon().getEntries()) {
			lexikaNames.add(entry.getName());
		}
		autoCompletetextType = new AutoCompleteField(textKontext, new TextContentAdapter(),
				lexikaNames.toArray(new String[0]));

		buttonSetContext = new Button(parent, SWT.PUSH);

		buttonSetContext.setText("Set Context");
		buttonSetContext.setFont(font);
		buttonSetContext.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String text = textKontext.getText();

				if (text.isEmpty())
					return;

				// try to find this lexikonentry

				LexikonEntry entryByLexikonName = terminology.getLexikon().getEntryByLexikonName(text);

				if (entryByLexikonName != null) {
					concept.setContext(entryByLexikonName);
				}

				else {
					// System.out.println(entryByLexikonName);

					LexikonEntry entry = new LexikonEntry(text, ELexikonEntryType.CONTEXT);
					// add one entry into that lexikon entry
					entry.addEntry(new Variation(text));

					terminology.getLexikon().addEntry(entry);

					List<String> lexikaNames = new ArrayList<String>();

					for (LexikonEntry en : terminology.getLexikon().getEntries()) {
						lexikaNames.add(en.getName());
					}
					concept.setContext(entry);

					autoCompletetextType.setProposals(lexikaNames.toArray(new String[0]));
				}

			}

		});

		// TODO add into a composite with a border

		Label labelVariation = new Label(parent, SWT.NONE);
		labelVariation.setText("Variation:");
		GridData gdComp = new GridData();
		gdComp.horizontalSpan = 6;
		compAttributes.setLayoutData(gdComp);
		GridData layoutData = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);
		layoutData.horizontalSpan = 6;
		widget.setLayoutData(layoutData);

		widget.setFont(font);

		widget.setText(concept.getName());

		textAddSynonym = new StyledText(parent, SWT.BORDER);
		textAddSynonym.setFont(font);

		textAddSynonym.setLayoutData(layoutData2);

		buttonAddSynonym = new Button(parent, SWT.PUSH);
		buttonAddSynonym.setText("Add Variation");
		buttonAddSynonym.setFont(font);

		buttonAddSynonym.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String syn = textAddSynonym.getText();

				if (!syn.isEmpty()) {

					Variation v = new Variation(syn);
					v.setRegex(comp.getButtonCheckIsRegex().getSelection());
					v.setRespectEndings(comp.getButtonRespectStemming().getSelection());
					v.setUnit(comp.getButtonCheckIsUnit().getSelection());
					v.setUseStemming(comp.getButtonCheckUseStemming().getSelection());
					v.setCaseSensitive(comp.getButtonCheckIsCaseSensitive().getSelection());
					v.setRespectContext(comp.getButtonCheckRespectContext().getSelection());

					System.out.println(comp.getButtonCheckRespectContext().getSelection());

					concept.addSynonym(v);
				}

				// empty the field
				textAddSynonym.setText("");
				refresh();

			}

		});

		Label labelAddReference = new Label(parent, SWT.NONE);
		labelAddReference.setText("Reference To Lexikon:");
		comboConceptReference = new ComboViewer(parent, SWT.READ_ONLY);
		comboConceptReference.getCombo().setLayoutData(layoutData2);

		comboConceptReference.setContentProvider(ArrayContentProvider.getInstance());
		comboConceptReference.setInput(terminology.getLexikon().getEntries());

		comboConceptReference.setLabelProvider(new LabelProvider() {
			@Override
			public String getText(Object element) {
				if (element instanceof LexikonEntry) {
					LexikonEntry concept = (LexikonEntry) element;
					return concept.getName();
				}
				return super.getText(element);
			}
		});

		comboConceptReference.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				IStructuredSelection sel = (IStructuredSelection) comboConceptReference.getSelection();

				LexikonEntry entry = (LexikonEntry) sel.getFirstElement();

				concept.addReference(entry);

				refresh();

			}
		});

		GridData layoutData3 = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		layoutData3.horizontalSpan = 6;
		comp = new VariationEditingComposite(parent, SWT.NONE, font, concept);
		comp.setLayoutData(layoutData3);

		Button buttonDummy1 = new Button(parent, SWT.NONE);
		buttonDummy1.setVisible(false);

		return parent;
	}

	private void refresh() {

		comp.getViewer().setInput(concept.getAllVariations().toArray(new Variation[0]));
		comp.getViewer().refresh();

		TableColumn[] columns = comp.getViewer().getTable().getColumns();

		for (TableColumn cl : columns) {
			cl.pack();
		}

		// comp.getViewer().getTable().pack();
		shell.layout();
		shell.redraw();
		shell.update();
	}

	@Override
	protected void okPressed() {

		buttonAddSynonym.notifyListeners(SWT.Selection, new Event());
		setReturnCode(OK);
		close();
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE | SWT.RESIZE);
		setBlockOnOpen(false);
	}

	// @Override
	// protected Point getInitialSize() {
	// return new Point(600, 470);
	// }

}
