package de.uniwue.mk.medIE.terminologyView;

import java.util.List;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;

import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.Variation;

public class ConceptNameFilter extends ViewerFilter {

	private static final String ID_PREFIX = "ID=";
	private String searchString;

	public void setSearchText(String s) {
		// ensure that the value can be used for matching
		this.searchString = "" + s + ".*";
	}

	@Override
	public boolean select(Viewer viewer, Object parentElement, Object element) {
		if (searchString == null || searchString.length() == 0) {
			return true;
		}

		Concept p = (Concept) element;

		List<Concept> successors = p.getSuccessors();
		if (searchString.startsWith(ID_PREFIX)) {
			searchString = searchString.replaceAll("\\.\\*", "");

			String[] idSplit = searchString.split("=");

			if (idSplit.length != 2)
				return false;
			String[] ids = idSplit[1].split(",");

			for (String id : ids) {
				if (String.valueOf(p.getId()).equals(id)) {
					return true;
				}

				for (Concept c : successors) {

					if (String.valueOf(c.getId()).equals(id)) {
						return true;
					}
				}
			}

		}

		else {
			searchString = searchString.toLowerCase();

			for (Concept c : successors) {

				if (c.getName().toLowerCase().matches(searchString))
					return true;

				// also try all variations
				for (Variation v : c.getAllVariations()) {
					if (v.getSynonym().toLowerCase().matches(searchString))
						return true;
				}
			}

			if (p.getName().toLowerCase().matches(searchString)) {
				return true;
			}

			// also try all variations
			for (Variation v : p.getAllVariations()) {
				if (v.getSynonym().toLowerCase().matches(searchString))
					return true;
			}
		}
		return false;
	}
}
