package de.uniwue.mk.medIE.terminologyView;

import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ComboViewer;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.medIE.terminology.ELexikonEntryType;
import de.uniwue.mk.medIE.terminology.IGotVariations;
import de.uniwue.mk.medIE.terminology.Lexikon;
import de.uniwue.mk.medIE.terminology.LexikonEntry;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mk.medIE.terminology.Variation;
import de.uniwue.mk.medIE.terminologyView.widgets.VariationEditingComposite;

public class LexikonEditingDialog extends Dialog {

	private Shell shell;
	private Lexikon lexikon;

	private ComboViewer comboViewer;
	private Font font;
	private Composite compInvis;
	private VariationEditingComposite compo;
	private Text textAddVariation;
	private Button buttonAddLexikon;
	private Button buttonRemoveLexikon;
	private Button buttonLoadLexika;
	private Text textLexikonName;

	public LexikonEditingDialog(Shell parentShell, Lexikon lex) {
		super(parentShell);

		this.shell = parentShell;
		this.lexikon = lex;

	}

	protected Control createDialogArea(final Composite parent) {

		shell.setSize(new Point(600, 400));
		parent.getShell().setText("Edit the lexikon of the terminology ");

		parent.setLayout(new GridLayout(3, false));

		Label labelChooseLexikonEntry = new Label(parent, SWT.NONE);
		labelChooseLexikonEntry.setText("Choose Lexikonentry:");

		comboViewer = new ComboViewer(parent, SWT.READ_ONLY);

		comboViewer.setContentProvider(ArrayContentProvider.getInstance());
		comboViewer.setInput(lexikon.getEntries().toArray(new LexikonEntry[0]));

		comboViewer.setLabelProvider(new LabelProvider() {

			@Override
			public String getText(Object element) {
				if (element instanceof LexikonEntry) {
					LexikonEntry entry = (LexikonEntry) element;
					return entry.getName();
				}
				return super.getText(element);
			}

		});
		font = new Font(parent.getDisplay(), "Segoe UI", 10, SWT.NORMAL);

		buttonRemoveLexikon = new Button(parent, SWT.PUSH);

		buttonRemoveLexikon.setText("Remove Lexikon");

		buttonRemoveLexikon.setEnabled(false);

		buttonRemoveLexikon.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection selection = (IStructuredSelection) comboViewer.getSelection();

				IGotVariations firstElement = (IGotVariations) selection.getFirstElement();

				if (firstElement != null) {

					lexikon.removeEntry((LexikonEntry) firstElement);
					comboViewer.setInput(lexikon.getEntries().toArray(new LexikonEntry[0]));
					comboViewer.refresh();

					compInvis.setVisible(false);
				}

			}

		});

		Label labelVariation = new Label(parent, SWT.NONE);
		labelVariation.setText("Variation:");

		textAddVariation = new Text(parent, SWT.BORDER);
		textAddVariation.setFont(font);

		GridData gdText = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		textAddVariation.setLayoutData(gdText);

		textAddVariation.addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {

				if (e.character != SWT.CR)
					return;

				String syn = textAddVariation.getText();

				IStructuredSelection selection = (IStructuredSelection) comboViewer.getSelection();

				IGotVariations firstElement = (IGotVariations) selection.getFirstElement();

				if (!syn.isEmpty() && firstElement != null) {

					Variation v = new Variation(syn);
					v.setRegex(compo.getButtonCheckIsRegex().getSelection());
					v.setRespectEndings(compo.getButtonRespectStemming().getSelection());
					v.setUnit(compo.getButtonCheckIsUnit().getSelection());
					v.setUseStemming(compo.getButtonCheckUseStemming().getSelection());
					firstElement.addVariation(v);
					refresh(firstElement);
				}

			}

		});

		textAddVariation.setFocus();

		Button buttonAddVariation = new Button(parent, SWT.PUSH);
		buttonAddVariation.setText("Add Variation");
		buttonAddVariation.setEnabled(false);
		buttonAddVariation.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonAddVariation.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				String syn = textAddVariation.getText();

				IStructuredSelection selection = (IStructuredSelection) comboViewer.getSelection();

				IGotVariations firstElement = (IGotVariations) selection.getFirstElement();

				if (!syn.isEmpty() && firstElement != null) {

					Variation v = new Variation(syn);
					v.setRegex(compo.getButtonCheckIsRegex().getSelection());
					v.setRespectEndings(compo.getButtonRespectStemming().getSelection());
					v.setUnit(compo.getButtonCheckIsUnit().getSelection());
					v.setUseStemming(compo.getButtonCheckUseStemming().getSelection());
					firstElement.addVariation(v);
					refresh(firstElement);
				}

			}

		});

		comboViewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				IStructuredSelection selection = (IStructuredSelection) comboViewer.getSelection();

				IGotVariations firstElement = (IGotVariations) selection.getFirstElement();

				for (Control c : compInvis.getChildren()) {
					c.dispose();
				}

				if (firstElement != null) {
					buttonRemoveLexikon.setEnabled(true);
					buttonAddVariation.setEnabled(true);
				}

				compInvis.setVisible(true);
				compo = new VariationEditingComposite(compInvis, SWT.NONE, font, firstElement);

				GridData gd = new GridData();
				gd.horizontalSpan = 3;
				compo.setLayoutData(gd);

				compInvis.layout();

				parent.layout();

			}
		});

		Label labelNexLexikon = new Label(parent, SWT.NONE);
		labelNexLexikon.setText("New Lexikon:");

		textLexikonName = new Text(parent, SWT.BORDER);
		textLexikonName.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL));
		buttonAddLexikon = new Button(parent, SWT.PUSH);
		buttonAddLexikon.setText("Add Lexikon");
		buttonAddLexikon.setLayoutData(new GridData(GridData.FILL, SWT.CENTER, false, false));
		buttonAddLexikon.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (textLexikonName.getText().isEmpty())
					return;
				LexikonEntry entry = new LexikonEntry(textLexikonName.getText(), ELexikonEntryType.DICTIONARY);
				lexikon.addEntry(entry);

				comboViewer.setInput(lexikon.getEntries().toArray(new LexikonEntry[0]));
				comboViewer.refresh();

			}

		});

		buttonLoadLexika = new Button(parent, SWT.PUSH);
		buttonLoadLexika.setText("Load Lexika");
		buttonLoadLexika.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// open a dialog to select an existing terminology
				FileDialog fd = new FileDialog(Display.getCurrent().getActiveShell(), SWT.OPEN);
				fd.setText("Select existing terminology");
				// fd.setFilterPath("C:/");
				String[] filterExt = { "*.xml", "*.*" };
				fd.setFilterExtensions(filterExt);
				String selected = fd.open();

				if (selected == null)
					return;

				MedicalTerminology term = MedicalTerminology.readTerminologyFromFile(selected);

				List<LexikonEntry> entries = term.getLexikon().getEntries();

				System.out.println(entries.size());
			}

		});

		compInvis = new Composite(parent, SWT.NONE);
		compInvis.setVisible(false);

		compInvis.setLayout(new GridLayout());

		GridData gd = new GridData();
		gd.horizontalSpan = 3;
		compInvis.setLayoutData(gd);

		GridData gdCombo = new GridData();
		gdCombo.horizontalSpan = 1;
		comboViewer.getCombo().setLayoutData(gdCombo);

		return parent;
	}

	private void refresh(IGotVariations firstElement) {

		compo.getViewer().setInput(firstElement.getAllVariations());
		compo.getViewer().refresh();

		TableColumn[] columns = compo.getViewer().getTable().getColumns();

		for (TableColumn cl : columns) {
			cl.pack();
		}

		shell.layout();
		shell.redraw();
		shell.pack();
		shell.update();
	}

	@Override
	protected void setShellStyle(int newShellStyle) {
		super.setShellStyle(SWT.CLOSE | SWT.MODELESS | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(false);
	}

	@Override
	protected Point getInitialSize() {
		return new Point(600, 400);
	}

	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		Button button = createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, false);
	}
}
