package de.uniwue.mk.medIE.terminologyView;

import java.util.HashMap;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;

import de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure;
import de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.IExpensiveDrawingStrategy;

public class TerminologyDrawingStrategy implements IExpensiveDrawingStrategy {

	// the color of the background if no condition is fullfilled
	Color defaultColor;

	// a mapping between a value and a color
	private HashMap<String, Color> conditionsMap;
	Feature f;
	StyleRange sr;

	private TerminologyView view;

	private Color color;

	public TerminologyDrawingStrategy(StyleRange styleRange, Feature feat, HashMap<String, Color> conditions,
			TerminologyView view) {
		this.defaultColor = styleRange.background;
		this.conditionsMap = conditions;
		this.f = feat;
		this.sr = styleRange;
		this.view = view;
	}

	public Color getColor(AnnotationFS a) {

		if (conditionsMap.containsKey(a.getFeatureValueAsString(f))) {
			return conditionsMap.get(a.getFeatureValueAsString(f));
		}

		return defaultColor;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * de.uniwue.kalimachos.coref.paintingStrategies.IAnnotationDrawingStrategy
	 * #drawAnnotation
	 * (de.uniwue.kalimachos.coref.editor.drawingstructure.TypeStyleDataStructure
	 * , org.apache.uima.cas.text.AnnotationFS,
	 * org.eclipse.swt.custom.StyledText, org.eclipse.swt.graphics.GC) Draws the
	 * annotation based on the feature value F stored in the stylemap
	 */
	@Override
	public void drawAnnotation(TypeStyleDataStructure struct, AnnotationFS anno, StyledText widget, GC gc) {
		color = conditionsMap.get(anno.getFeatureValueAsString(f));
		StyleRange styleRange = new StyleRange();
		styleRange.start = anno.getBegin();
		styleRange.length = anno.getEnd() - anno.getBegin();

		if (color == null) {
			styleRange.background = defaultColor;
		} else {
			styleRange.background = conditionsMap.get(anno.getFeatureValueAsString(f));
		}

		widget.setStyleRange(styleRange);

		/*
		 * Create labels and tooltips to allow showing the associated concept in
		 * a TerminologyTreeView.
		 */
		Rectangle bounds = widget.getTextBounds(anno.getBegin(), anno.getEnd() - 1);
		// determine whether the annotation spans several lines
		int standardHeight = widget.getTextBounds(0, 0).height;
		int linebreakOffset = 0;
		if (bounds.height > standardHeight) {
			while (widget.getTextBounds(anno.getBegin(), anno.getBegin() + linebreakOffset).height <= standardHeight) {
				linebreakOffset++;
			}
			bounds = widget.getTextBounds(anno.getBegin(), anno.getBegin() + linebreakOffset - 1);
		}

		Label lb = createLabel(widget, color, bounds);
		/*
		 * used in the AnnotationEditor to identify controls that should be
		 * disposed on a reload
		 */
		lb.setData("origin", "ttv");
		createTooltip(anno, widget, lb);

		/*
		 * If annotation spans several lines create several Labels
		 */
		if (linebreakOffset > 0) {
			Rectangle bounds2 = widget.getTextBounds(anno.getBegin() + linebreakOffset, anno.getEnd() - 1);
			Label lb2 = createLabel(widget, color, bounds2);
			lb2.setData("origin", "ttv");
			createTooltip(anno, widget, lb2);
		}

		// Dispose resource
		gc.dispose();

	}

	private Label createLabel(StyledText widget, Color color, Rectangle bounds) {
		Label bt = new Label(widget, SWT.CENTER);
		bounds.y = bounds.y - 10;
		bounds.height = 10;
		bt.setBounds(bounds);
		bt.setBackground(widget.getBackground());
		bt.addMouseTrackListener(new MouseTrackListener() {

			@Override
			public void mouseHover(MouseEvent e) {
			}

			@Override
			public void mouseExit(MouseEvent e) {
				bt.setBackground(widget.getBackground());
			}

			@Override
			public void mouseEnter(MouseEvent e) {
				bt.setBackground(color);
			}
		});
		return bt;
	}

	private void createTooltip(AnnotationFS anno, StyledText widget, Label bt) {
		ToolTip toolTip = new ToolTip(bt, ToolTip.NO_RECREATE, false) {

			@Override
			protected Composite createToolTipContentArea(Event event, Composite parent) {
				Button t = new Button(parent, SWT.PUSH);
				t.setText("Open TerminologyTreeView");
				t.addSelectionListener(new SelectionListener() {

					@Override
					public void widgetSelected(SelectionEvent e) {
						Feature idFeat = anno.getType().getFeatureByBaseName("ConceptID");
						String id = anno.getFeatureValueAsString(idFeat);
						if (id == null) {
							return;
						}
						Dialog dialog = new TerminologyTreeViewDialog(widget.getShell(), view, Integer.valueOf(id));
						dialog.open();
					}

					@Override
					public void widgetDefaultSelected(SelectionEvent e) {
					}

				});

				return parent;
			}

		};
		toolTip.setHideOnMouseDown(false);
	}

	@Override
	public int getLineHeight(AnnotationFS anno) {
		return IAnnotationDrawingStrategy.WIDGET_DEFAULT_LINEHEIGHT;
	}

}