package de.uniwue.mk.medIE.terminologyView;

import java.util.HashMap;
import java.util.List;

import javax.annotation.PreDestroy;

import org.eclipse.jface.viewers.StyledCellLabelProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Tree;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminologyView.menu.actions.TooltipDisplayOptionsAction;

public class TerminologyLabelContentProvider extends StyledCellLabelProvider {

	private GC gc;

	HashMap<String, Image> imageHash = new HashMap<String, Image>();

	private Tree tree;

	private static final String IMG_ROOT = "IMAGE_ROOT";

	private static final String IMG_ATTRIBUTE = "IMAGE_ATTRIBUTE";

	private static final String IMG_VALUE = "IMAGE_VALUE";

	public TerminologyLabelContentProvider(Tree tree) {

		this.gc = new GC(Display.getCurrent());
		this.tree = tree;
	}

	@Override
	public void update(ViewerCell cell) {
		Object element = cell.getElement();
		StyledString text = new StyledString();

		if (element instanceof Concept) {

			Concept concept = (Concept) element;

			text.append(concept.getName());

			String property = (String) concept.getProperty("CONCEPT_HIGHLIGHT");
			if (property != null && property.equals("TRUE")) {
				cell.setForeground(Display.getCurrent().getSystemColor(SWT.COLOR_BLUE));
			}

			// set image
			setImage(cell, concept);
		}

		cell.setText(text.toString());
		cell.setStyleRanges(text.getStyleRanges());
		super.update(cell);

	}

	@Override
	public String getToolTipText(Object element) {

		if (!(element instanceof Concept))
			return null;

		String tooltipStrategy = (String) tree.getData(TooltipDisplayOptionsAction.TREE_TOOLTIP_OPTION);
		Concept c = (Concept) element;

		if (tooltipStrategy.equals(TooltipDisplayOptionsAction.TREE_TOOLTIP_FULL)) {
			List<String> synonymsAsRegex = c.getSynonymsAsRegex();

			String tooltip = "";

			for (String s : synonymsAsRegex) {
				tooltip += s + "\n";
			}
			return tooltip;
		}

		else if (tooltipStrategy.equals(TooltipDisplayOptionsAction.TREE_TOOLTIP_FLAT)) {
			return c.getName();

		}

		return c.getName();
	}

	@Override
	public Point getToolTipShift(Object object) {
		return new Point(5, 5);
	}

	@Override
	public int getToolTipDisplayDelayTime(Object object) {
		return 10;
	}

	private void setImage(ViewerCell cell, Concept concept) {

		Image im = null;

		if (concept.isRoot() || concept.isStructure()) {

			im = getImage(IMG_ROOT);
		}

		else if (concept.isAttribute()) {

			im = getImage(IMG_ATTRIBUTE);
		}

		else if (concept.isValue()) {
			im = getImage(IMG_VALUE);
		}

		else {
			im = getImage("DEFAULT");
		}

		cell.setImage(im);

	}

	private Image getImage(String imgRoot) {

		if (imageHash.containsKey(imgRoot)) {
			return imageHash.get(imgRoot);
		}

		else {

			Image im = null;
			if (imgRoot.equals(IMG_ROOT)) {

				im = ApplicationUtil.createImage(ApplicationUtil.TERMINOLOGY_ROOT);
				imageHash.put(imgRoot, im);

			} else if (imgRoot.equals(IMG_ATTRIBUTE)) {
				im = ApplicationUtil.createImage(ApplicationUtil.WSD_CAT_ATTRIBUT);
				imageHash.put(imgRoot, im);
			}

			else if (imgRoot.equals(IMG_VALUE)) {
				im = ApplicationUtil.createImage(ApplicationUtil.TERMINOLOGY_VALUE);
				imageHash.put(imgRoot, im);
			}

			else {
				im = ApplicationUtil.createImage(ApplicationUtil.TERMINOLOGY_NODE);
				imageHash.put(imgRoot, im);
			}

			return im;
		}
	}

	@PreDestroy
	public void dispose() {

		for (Image i : imageHash.values()) {

			i.dispose();
		}

		gc.dispose();
	}
}
