package de.uniwue.mk.medIE.terminologyView;

import java.awt.MouseInfo;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;
import javax.xml.bind.JAXB;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.impl.XmiCasDeserializer;
import org.apache.uima.cas.impl.XmiCasSerializer;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.MElementContainer;
import org.eclipse.e4.ui.model.application.ui.MUIElement;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.AbstractTreeViewer;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Cursor;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.ImageLoader;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.ProgressBar;
import org.eclipse.swt.widgets.Shell;
import org.osgi.service.event.EventHandler;
import org.xml.sax.SAXException;

import de.uniwue.kalimachos.coref.editor.drawingstructure.AnnoStyle;
import de.uniwue.kalimachos.coref.paintingStrategies.BackgroundDrawingStrategy;
import de.uniwue.kalimachos.coref.paintingStrategies.BorderDrawingStrategy;
import de.uniwue.mk.kall.athen.appDelegation.ui.MasterPart;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationPreferences;
import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;
import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.part.editor.DeserializeXMIHandler;
import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.MedEvalDrawingStrategy;
import de.uniwue.mk.kall.editorPainting.paintingstrategies.specialized.TermViewConceptGoldDrawingStrategy;
import de.uniwue.mk.medIE.extraction.algorithm.ATerminologyMatchingAlgorithm;
import de.uniwue.mk.medIE.extraction.algorithm.SimpleInSegmentMatchingAlgorithm;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mk.medIE.terminologyView.widgets.TerminologyTreeView;

public class TerminologyView extends AEditorSubordinateViewPart {

	// copy and modify this class if you want to create a new view
	private static final Class<TerminologyView> CLASS_THIS = TerminologyView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.19";

	// comment
	private static final int FONTSIZE = 8;
	
	// WIDGETS
	private Button buttonCreateTerminology;
	private Button buttonLoadTerminology;
	private Button buttonSaveTerminology;
	private Button buttonApplyTerminology;

	private TerminologyTreeView terminologyTreeView;

	private AttributeAcceptanceDialog dialog;

	private Image imageAccept = ApplicationUtil.createImage(ApplicationUtil.CORRECT);
	private Image imageDelete = ApplicationUtil.createImage(ApplicationUtil.ERROR);

	private String openedTerminologyPath;

	private Label labelProgress;

	// the algorithm
	ATerminologyMatchingAlgorithm algorithm = new SimpleInSegmentMatchingAlgorithm();

	// MODEL
	private List<AnnotationFS> annos;

	private MedicalTerminology terminology;

	// the main type this view is about
	private Type conceptType;
	private Type conceptGoldType;
	private Type medEvalType;
	private Type sentType;

	// E4 DI OBJECTS used to communicate with the editor
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	private Feature featMedEvalType;

	private Feature featTypeContent;
	private Feature featConceptId;

	private EventHandler conceptChangedHandler;

	private Button buttonPreprocessTerminology;

	// those types are required for this view to work
	private Type posType;
	private Type typeHighlightAction;
	private Type typeTempAttributes;

	private Feature featPOSTag;

	private Label labelProgressCountColor;

	private Feature featTypeContentGold;

	private Feature featConceptIdGold;

	protected Thread autoSaveThread;

	private Button buttonApplyToFolder;

	protected boolean useAutosave;

	private Button buttonUseAutosave;

	private Button buttonShowEvaluationTypes;

	// compounds - UI

	@Inject
	public TerminologyView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		// layout this should stay as it is
		super.overlay = parent;
		super.overlay.setLayout(new FillLayout());

		// init the part, so that this view knows its position globally
		// (required to see if it has focus)
		super.part = partService.findPart(viewId);

		initHandler();

	}

	private void initHandler() {

		conceptChangedHandler = new EventHandler() {

			@Override
			public void handleEvent(org.osgi.service.event.Event arg0) {

				try {
					renewPage();
				} catch (CASException e) {
					e.printStackTrace();
				}

				terminologyTreeView.getViewer().expandToLevel(2);

			}
		};

	}

	// is called whenever the editor loads a new page
	@Override
	public void onEditorInputChanged(AnnotationEditorWidget editor) {

		super.editor = editor;

		// only initialize if the part is visible
		MUIElement selectedElement = null;
		if (part != null) {
			MElementContainer<MUIElement> parent = part.getParent();
			selectedElement = parent.getSelectedElement();
		}
		if (selectedElement == null || !selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render but unsibscribe
			standardUnsubscribeFromEditorEvents(CLASS_THIS);
			return;
		}

		// init all the types we need in this view, open a dialog to specify
		// them if not already done so
		initTypes();

		// ====REMOVE ALL CONTROLS THAT WERE ADDED BEFORE====

		for (Control c : overlay.getChildren()) {
			c.dispose();
		}

		// change the linespacing
		int amountOfSpace = FONTSIZE + 1;

		editor.getWidget().setLineSpacing(amountOfSpace + 10);

		// init the Model
		initModel(editor);

		// init the layout
		initLayout(overlay);

		// if we managed to get here subscribe to all editor events if not
		// already done so
		if (!isSubscribedToEditorEvents) {
			// use the standard method to get all events
			standardSubscribeToEditorEvents(CLASS_THIS);
		}

		// ====reload=====
		overlay.layout();
		overlay.getParent().layout();
	}

	private void initModel(AnnotationEditorWidget editor) {

		this.annos = new ArrayList<AnnotationFS>();

		// all attributes
		for (AnnotationFS attribute : editor.getCas().getAnnotationIndex(conceptType)) {

			this.annos.add(attribute);
		}

	}

	// use this method to init all typings and stylings used in this view
	// load them from preferences
	@Override
	public boolean initTypes() {

		conceptType = editor.getCas().getTypeSystem().getType("de.uniwue.medIE.Concept");
		conceptGoldType = editor.getCas().getTypeSystem().getType("de.uniwue.medIE.Conceptgold");
		this.featTypeContent = conceptType.getFeatureByBaseName("NodeType");
		this.featTypeContentGold = conceptGoldType.getFeatureByBaseName("NodeType");
		this.medEvalType = editor.getCas().getTypeSystem().getType("de.uniwue.medIE.Eval");
		this.sentType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.Sentence");
		this.posType = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.POS");
		this.featPOSTag = posType.getFeatureByBaseName("POSTag");
		this.typeHighlightAction = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp2");
		this.typeTempAttributes = editor.getCas().getTypeSystem().getType("de.uniwue.kalimachos.coref.type.temp3");

		this.featMedEvalType = medEvalType.getFeatureByBaseName("Type");
		this.featConceptId = conceptType.getFeatureByBaseName("ConceptID");
		this.featConceptIdGold = conceptGoldType.getFeatureByBaseName("ConceptID");

		// set everything unnneded invisible

		this.editor.allStylesInvisible();

		// Visibility

		editor.changeVisibility(conceptType, true);
		editor.changeVisibility(conceptGoldType, true);
		// editor.changeVisibility(medEvalType, true);
		editor.changeVisibility(typeHighlightAction, true);
		editor.changeVisibility(typeTempAttributes, true);

		// layer
		editor.changeLayer(medEvalType, 4);
		editor.changeLayer(typeTempAttributes, 2);
		editor.changeLayer(conceptGoldType, 3);

		deleteTemporaryTypes();

		// gold for the temporary Attributes
		StyleRange rangeTempAtt = new StyleRange();
		rangeTempAtt.background = new Color(overlay.getDisplay(), 156, 151, 17);
		AnnoStyle styleTempAtt = new AnnoStyle(new BackgroundDrawingStrategy(rangeTempAtt), rangeTempAtt);

		// border drawing for the highlighting type
		StyleRange styleHighlight = new StyleRange();
		styleHighlight.background = new Color(overlay.getDisplay(), 156, 151, 17);
		AnnoStyle rangeHighlight = new AnnoStyle(new BorderDrawingStrategy(), styleHighlight);

		// yellow
		HashMap<String, Color> colorMap = new HashMap<String, Color>();
		colorMap.put("ATTRIBUTE", new Color(overlay.getDisplay(), 233, 247, 74));
		colorMap.put("VALUE", new Color(overlay.getDisplay(), 250, 141, 17));
		colorMap.put("ATTRIBUTE_GOLD", Display.getCurrent().getSystemColor(SWT.COLOR_GREEN));
		colorMap.put("ATTRIBUTE_VORHANDEN", Display.getCurrent().getSystemColor(SWT.COLOR_CYAN));
		StyleRange rangeOv = new StyleRange();
		rangeOv.background = new Color(overlay.getDisplay(), 233, 247, 74);
		AnnoStyle styleAtt = new AnnoStyle(new TerminologyDrawingStrategy(rangeOv, featTypeContent, colorMap, this),
				rangeOv);

		// medeval
		StyleRange rangeMedEval = new StyleRange();
		rangeMedEval.background = new Color(overlay.getDisplay(), 245, 114, 44);
		AnnoStyle styleMedEval = new AnnoStyle(
				new MedEvalDrawingStrategy(rangeMedEval, featMedEvalType.getShortName()), rangeMedEval);

		// conceptgoldtype
		StyleRange rangeConceptGold = new StyleRange();
		rangeConceptGold.background = new Color(overlay.getDisplay(), 158, 139, 63);
		AnnoStyle styleConceptGold = new AnnoStyle(new TermViewConceptGoldDrawingStrategy(rangeConceptGold,
				featConceptIdGold), rangeConceptGold);

		// apply
		editor.changeStyle(conceptType, styleAtt);
		editor.changeStyle(conceptGoldType, styleConceptGold);
		editor.changeStyle(medEvalType, styleMedEval);
		editor.changeStyle(typeTempAttributes, styleTempAtt);
		editor.changeStyle(typeHighlightAction, rangeHighlight);

		// also change the modus
		editor.setActiveMode(conceptType);

		return true;

	}

	private void deleteTemporaryTypes() {
		// delete quietly

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeHighlightAction)) {
			toDelete.add(temp);
		}

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTempAttributes)) {
			toDelete.add(temp);
		}

		for (AnnotationFS del : toDelete)
			editor.getCas().removeFsFromIndexes(del);

	}

	protected AnnotationFS getSelectedAnno() {

		updateModel();

		int caretOffset = editor.getWidget().getCaretOffset();
		for (AnnotationFS a : this.annos) {
			if (a.getBegin() <= caretOffset && a.getEnd() >= caretOffset)
				return a;
		}
		return null;
	}

	// init your Layout in here
	@Override
	protected void initLayout(Composite parent) {

		parent.setLayout(new GridLayout(3, false));

		GridData gd = new GridData(GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL | GridData.FILL_BOTH);
		parent.setLayoutData(gd);

		// // =====FX TEST probably needs the javafx as bundle

		// /* Create an FXCanvas */
		// final javafx.embed.swt.FXCanvas fxCanvas = new
		// javafx.embed.swt.FXCanvas(parent, SWT.NONE) {
		// public Point computeSize(int wHint, int hHint, boolean changed) {
		// getScene().getWindow().sizeToScene();
		// int width = (int) getScene().getWidth();
		// int height = (int) getScene().getHeight();
		// return new Point(width, height);
		// }
		// };

		// // // // END FX TEST

		buttonCreateTerminology = new Button(parent, SWT.PUSH);

		buttonCreateTerminology.setText("Create Terminology");

		buttonCreateTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "",
						"Enter a terminology name", "", null);
				if (dlg.open() == Window.OK) {
					// User clicked OK; update the label with the input
					String termName = dlg.getValue();

					Concept root = new Concept("ROOT", null, null, 0);
					root.setRoot(true);

					terminology = new MedicalTerminology(root, termName);
					terminologyTreeView.setTerminology(terminology);

					buttonApplyToFolder.setEnabled(true);
					try {
						renewPage();
					} catch (CASException e1) {
						e1.printStackTrace();
					}
				}

			}

		});

		buttonLoadTerminology = new Button(parent, SWT.PUSH);

		buttonLoadTerminology.setText("Load Terminology");

		buttonLoadTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				FileDialog dlg = new FileDialog(new Shell(), SWT.OPEN);
				dlg.setText("Open");
				String path = dlg.open();
				if (path == null)
					return;

				openedTerminologyPath = path;

				terminology = MedicalTerminology.readTerminologyFromFile(openedTerminologyPath);
				terminologyTreeView.setTerminology(terminology);
				buttonApplyToFolder.setEnabled(true);

				// also save it once !! TODO
				try {
					renewPage();
				} catch (CASException e1) {
					e1.printStackTrace();
				}

			}

		});

		buttonSaveTerminology = new Button(parent, SWT.PUSH);

		buttonSaveTerminology.setText("Save Terminology");

		buttonSaveTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (terminology == null)
					return;

				FileDialog dlg = new FileDialog(new Shell(), SWT.SAVE);
				dlg.setText("Save Terminology");
				dlg.setFileName(terminology.getName());
				String[] filterExt = { "*.xml", "*.txt", "*.png" };
				dlg.setFilterExtensions(filterExt);
				openedTerminologyPath = dlg.open();

				if (openedTerminologyPath == null || dlg.getFilterIndex() == -1)
					return;

				if (dlg.getFilterIndex() == 1) {
					// Save as text

					if (openedTerminologyPath.endsWith(".txt")) {
						openedTerminologyPath += ".txt";
					}
					FileWriter fw;
					try {
						fw = new FileWriter(new File(openedTerminologyPath));
						fw.append(terminology.terminologyToText());

						fw.flush();
						fw.close();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}

				}

				else if (dlg.getFilterIndex() == 2) {

					// save as PNG
					if (!openedTerminologyPath.endsWith(".png")) {
						openedTerminologyPath += ".png";
					}
					TreeViewer viewer = terminologyTreeView.getViewer();
					viewer.getTree().getParent().setRedraw(false);
					final Point originalSize = viewer.getTree().getSize();
					Object[] expandedElements = viewer.getExpandedElements();
					viewer.expandAll();
					final Point size = viewer.getTree().computeSize(SWT.DEFAULT, SWT.DEFAULT, true);
					final Image image = new Image(Display.getCurrent(), size.x, size.y);
					final GC gc = new GC(image);

					viewer.getTree().setSize(size);
					viewer.getTree().print(gc);
					viewer.getTree().setSize(originalSize);

					final ImageLoader loader = new ImageLoader();
					loader.data = new ImageData[] { image.getImageData() };
					loader.save(openedTerminologyPath, SWT.IMAGE_PNG);
					gc.dispose();
					viewer.setExpandedElements(expandedElements);
					viewer.getTree().getParent().setRedraw(true);
				}

				else if (dlg.getFilterIndex() == 0) {

					// save as xml
					if (openedTerminologyPath.endsWith(".xml")) {
						openedTerminologyPath += ".xml";
					}
					JAXB.marshal(terminology, new File(openedTerminologyPath));

				}

			}

		});

		buttonApplyTerminology = new Button(parent, SWT.PUSH);

		buttonApplyTerminology.setText("Apply Terminology");

		buttonApplyTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				applyTerminology("Apply to document", editor.getCas());

			}

		});

		buttonPreprocessTerminology = new Button(parent, SWT.PUSH);
		buttonPreprocessTerminology.setText("Quick Terminology");
		buttonPreprocessTerminology.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (terminology == null) {

					// fire new terminology event
					buttonCreateTerminology.notifyListeners(SWT.Selection, new Event());

					// if the terminology is still null we dont do anything
					if (terminology == null)
						return;
					// find all substantives
					deleteAttributesAndValues();

					// find new Attributes
					findAttributesBySubstantives();

					for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {
						editor.highlightRange(concept.getBegin(), concept.getEnd() - concept.getBegin(), true);
						break;
					}

					dialog = new AttributeAcceptanceDialog(Display.getCurrent().getActiveShell(), terminology, broker,
							editor, conceptType, featTypeContent);

					int open = dialog.open();

					if (open == 0) {

						// delete them again

					}

					terminologyTreeView.setAttributeAcceptanceDialog(dialog);

					deleteAttributesAndValues();
					buttonPreprocessTerminology.setEnabled(false);
				}

			}

		});

		if (this.terminology != null) {
			buttonPreprocessTerminology.setEnabled(false);
		}

		buttonApplyToFolder = new Button(parent, SWT.PUSH);
		buttonApplyToFolder.setText("Apply to folder");
		buttonApplyToFolder.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				DirectoryDialog dlg = new DirectoryDialog(Display.getCurrent().getActiveShell());

				// Set the initial filter path according
				// to anything they've selected or typed in
				dlg.setFilterPath(ApplicationUtil.getFromPreferences(ApplicationPreferences.WORKSPACE, "nope"));

				// Change the title bar text
				dlg.setText("XMI Terminology Application");

				// Customizable message displayed in the dialog
				dlg.setMessage("Select a directory to apply the terminology to(XMI Documents only)");

				// Calling open() will open and run the dialog.
				// It will return the selected directory, or
				// null if user cancels
				String dir = dlg.open();

				if (dir == null)
					return;
				File directory = new File(dir);

				if (directory.exists() && directory.isDirectory()) {

					Map<File, CAS> workloadMap = new HashMap<File, CAS>();
					for (File f : directory.listFiles()) {

						if (!f.getName().endsWith(".xmi")) {
							continue;
						}

						CAS cas = DeserializeXMIHandler.createCas();

						try {
							FileInputStream fis = new FileInputStream(f);
							XmiCasDeserializer.deserialize(fis, cas);
							fis.close();
						} catch (SAXException | IOException e1) {
							// TODO Auto-generated catch block
							e1.printStackTrace();
						}

						workloadMap.put(f, cas);

					}

					// now apply to all documents
					applyTerminologyToBatch(workloadMap);
				}
			}

		});

		if (this.terminology == null) {
			buttonApplyToFolder.setEnabled(false);
		}

		buttonUseAutosave = new Button(parent, SWT.CHECK);
		buttonUseAutosave.setText("Use Autosave?:");
		buttonUseAutosave.setSelection(true);
		buttonUseAutosave.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				TerminologyView.this.useAutosave = buttonUseAutosave.getSelection();

				if (useAutosave) {
					TerminologyView.this.autoSaveThread.start();
				}

				else {
					if (TerminologyView.this.autoSaveThread != null) {
						TerminologyView.this.autoSaveThread.stop();
					}
				}
			}

		});

		TerminologyView.this.autoSaveThread = new Thread(new Runnable() {

			@Override
			public void run() {

				while (true) {
					try {
						Thread.sleep(5 * 1000 * 60);
					} catch (InterruptedException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

					if (openedTerminologyPath == null)
						continue;
					String autosavePath = openedTerminologyPath.replaceAll("\\.xml", "").replaceAll("\\.txt", "");

					JAXB.marshal(terminology, new File(autosavePath + "_autosave.xml"));

				}
			}
		});

		TerminologyView.this.autoSaveThread.start();

		buttonShowEvaluationTypes = new Button(parent, SWT.CHECK);
		buttonShowEvaluationTypes.setText("Show Evaluation?:");
		buttonShowEvaluationTypes.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				editor.changeVisibility(medEvalType, buttonShowEvaluationTypes.getSelection());

			}

		});

		Label labelDummy = new Label(parent, SWT.NONE);
		labelDummy.setVisible(false);

		Label labelFort = new Label(parent, SWT.NONE);
		labelFort.setText("Progress:");

		labelProgress = new Label(parent, SWT.BORDER);

		Label labelProgressCount = new Label(parent, SWT.NONE);
		labelProgressCount.setText("Progress (Counts):");

		labelProgressCountColor = new Label(parent, SWT.BORDER);

		GridData layoutData2 = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		layoutData2.horizontalSpan = 2;
		labelProgress.setLayoutData(layoutData2);
		labelProgressCountColor.setLayoutData(layoutData2);

		terminologyTreeView = createTerminologyTreeView(parent);

		parent.layout();
		// we cant set this any earlier !!
		labelProgress.setImage(createImageByOverlap(null, labelProgress));
		labelProgressCountColor.setImage(createImageByOverlap("RESPECT_COUNTS", labelProgressCountColor));

	}

	private Image createImageByOverlap(String option, Label label) {

		Rectangle bounds = label.getBounds();

		Image im = new Image(Display.getCurrent(), bounds.width, bounds.height);

		GC gc = new GC(im);

		String overlap = determineOverlap(option);
		Color color = determineColorForOverlap(overlap);
		gc.setBackground(color);
		gc.fillRectangle(0, 0, bounds.width, bounds.height);

		gc.drawString((overlap + "%"), bounds.width / 2 - 5, 0);

		gc.dispose();
		color.dispose();
		return im;
	}

	private Color determineColorForOverlap(String overlap) {

		int val = Integer.parseInt(overlap);

		if (val <= 30) {
			// red
			return new Color(Display.getCurrent(), new RGB(227, 70, 96));
		}

		else if (val > 30 && val <= 50) {
			// orange
			return new Color(Display.getCurrent(), new RGB(250, 148, 65));
		}

		else if (val > 50 && val <= 75) {
			// yellow
			return new Color(Display.getCurrent(), new RGB(242, 240, 82));

		}

		else if (val > 75 && val <= 90) {
			// light green
			return new Color(Display.getCurrent(), new RGB(16, 235, 82));

		}

		else if (val > 90) {
			// green
			return new Color(Display.getCurrent(), new RGB(65, 181, 100));

		}

		return null;
	}

	private String determineOverlap(String option) {

		if (option == null) {
			double sentTotal = editor.getCas().getAnnotationIndex(sentType).size();
			double overlap = 0.0;

			for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentType)) {

				for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {

					if (concept.getBegin() >= sent.getBegin() && concept.getEnd() <= sent.getEnd()) {
						overlap++;
						break;
					}
				}
			}
			int overlapInt = (int) ((overlap * 100) / sentTotal);
			return String.valueOf(overlapInt);
		}

		else {
			double ovelapMax = 0.0;
			double overlap = 0.0;

			for (AnnotationFS sent : editor.getCas().getAnnotationIndex(sentType)) {

				Matcher matcher = Pattern.compile("\\(\\(.*?\\)\\)").matcher(sent.getCoveredText());
				int cnt = 0;
				while (matcher.find()) {
					String group = matcher.group();

					String number = group.substring(2, group.length() - 2);
					cnt = Integer.parseInt(number);
					break;
				}
				ovelapMax += cnt;
				for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {

					if (concept.getBegin() >= sent.getBegin() && concept.getEnd() <= sent.getEnd()) {

						// iterate back to front in the sentence and parse the
						// counts
						overlap += cnt;
						break;
					}
				}
			}

			int overlapInt = (int) ((overlap * 100) / ovelapMax);
			return String.valueOf(overlapInt);

		}
	}

	// use this method to update the view
	public void renewPage() throws CASException {
		updateModel();

		reloadTree();
	}

	private void updateModel() {

		initModel(editor);
	}

	private void deleteAttributesAndValues() {

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS att : editor.getCas().getAnnotationIndex(conceptType)) {
			toDelete.add(att);
		}

		// also delete evaluation types

		for (AnnotationFS eval : editor.getCas().getAnnotationIndex(medEvalType)) {
			toDelete.add(eval);
		}

		// delete without a reload
		for (AnnotationFS a : toDelete) {
			editor.getCas().removeFsFromIndexes(a);
		}

		if (!toDelete.isEmpty()) {
			editor.removeAnnotation(toDelete.get(0));
		}

	}

	private void reloadTree() {
		terminologyTreeView.reloadTree();
	}

	@PreDestroy
	public void preDestroy() {
		// TODO unused
	}

	@Focus
	public void onFocus() {

		// notify all other parts that we are visible now :)
		broker.send(IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED, "");

		// if we missed that we are active somehow we fix this here otherwise do
		// nothing
		if (!isSubscribedToEditorEvents && this.editor != null) {
			onEditorInputChanged(this.editor);

			// we also load the terminology if we still have one
			try {
				renewPage();
			} catch (CASException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		}

		if (this.editor == null) {

			// request the editor if possible
			broker.send(IAnnotationEditorEvents.REQUEST_ACTIVE_EDITOR, this);
			standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		}

	}

	@Persist
	public void save() {
		// editor takes care of that usually, we only modify the annotations in
		// the editr

	}

	// Perspective listener
	@Override
	public void coupleToPerspective() {
		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);
		standardSubscribeToEditorDocumentChangedEvents(CLASS_THIS);
		super.coupleToPerspective();
	}

	@Override
	public void decoupleFromPerspective() {
		MasterPart.unsubscribeFromEvent(CLASS_THIS, IAnnotationEditorEvents.VIEW_STACK_FOCUS_CHANGED,
				onViewStackChangedHandler);
		standardUnsubscribeFromEditorDocumentChangedEvents(CLASS_THIS);
		super.decoupleFromPerspective();
	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorClosed(org.osgi.service.event.Event arg0) {
		// remove all components
		if (this.overlay != null && !this.overlay.isDisposed()) {
			for (Control c : this.overlay.getChildren()) {
				c.dispose();
			}
		}

		this.imageAccept.dispose();
		this.imageDelete.dispose();

		if (this.openedTerminologyPath != null) {

			// save the terminology
			JAXB.marshal(terminology, new File(this.openedTerminologyPath));
		}

		// unsubscribe from editor notifications - yep this is important
		standardUnsubscribeFromEditorEvents(CLASS_THIS);

		isSubscribedToEditorEvents = false;

		// free the expensive object
		this.editor = null;

	}

	@Override
	public void onEditorKeyPressed(org.osgi.service.event.Event arg0) {

		Object property = arg0.getProperty(IEventBroker.DATA);
		if (terminology == null)
			return;

		if (property instanceof Character) {
			Character c = (Character) property;

			if (c == '\r') {
				if (!editor.getWidget().getSelectionText().isEmpty()) {

					Concept selectedConcept = terminologyTreeView.getSelectedConcept();

					if (selectedConcept == null) {
						selectedConcept = terminology.getRoot();
					}

					Concept createChild = terminologyTreeView.createChild(selectedConcept, editor.getWidget()
							.getSelectionText());
					selectedConcept.addChild(createChild);

					reloadTree();
				}
			}

			else if (c == 'g') {

				List<AnnotationFS> conceptsSel = new ArrayList<AnnotationFS>();
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

					if (a.getBegin() <= editor.getWidget().getCaretOffset()
							&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
						conceptsSel.add(a);
					}
				}

				List<AnnotationFS> conceptsSelGold = new ArrayList<AnnotationFS>();
				for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptGoldType)) {

					if (a.getBegin() <= editor.getWidget().getCaretOffset()
							&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
						conceptsSelGold.add(a);
					}
				}

				if (conceptsSelGold.size() > 0) {

					for (AnnotationFS a : conceptsSelGold) {

						editor.removeAnnotation(a);
					}
				}

				else if (conceptsSelGold.size() == 0 && conceptsSel.size() == 1) {
					AnnotationFS annotationFS = conceptsSel.get(0);

					cloneAndAddAsGoldAnnotation(annotationFS);

				}

			}
		}
	}

	@Override
	public void onViewStackChanged(org.osgi.service.event.Event arg0) {

		// is someone else in focus now => we loose our purpose since we arent
		// visible
		MPart findPart = partService.findPart(viewId);

		if (findPart == null) {
			return;
		}

		MElementContainer<MUIElement> parent = findPart.getParent();
		MUIElement selectedElement = parent.getSelectedElement();

		if (!selectedElement.getElementId().equals(viewId)) {
			// this part is not visible => dont render it but
			// unsubscribe the
			// listener if they are
			// remove all listeners
			standardUnsubscribeFromEditorEvents(CLASS_THIS);

			if (this.openedTerminologyPath != null) {

				// save the terminology
				JAXB.marshal(terminology, new File(this.openedTerminologyPath));
			}
		}

	}

	@Override
	public void standardUnsubscribeFromEditorEvents(Class<?> classToUnsubscribe) {
		super.standardUnsubscribeFromEditorEvents(CLASS_THIS);

		MasterPart.unsubscribeFromEvent(CLASS_THIS, IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED,
				conceptChangedHandler);
	}

	@Override
	public void standardSubscribeToEditorEvents(Class<?> classToSubscribe) {
		super.standardSubscribeToEditorEvents(CLASS_THIS);
		MasterPart.subscribeForEvent(CLASS_THIS, IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED,
				conceptChangedHandler);
	}

	@Override
	public void onEditorRightClickMenuRequested(org.osgi.service.event.Event arg0) {

		MenuManager manager = (MenuManager) arg0.getProperty(IEventBroker.DATA);

		List<AnnotationFS> conceptsSel = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

			if (a.getBegin() <= editor.getWidget().getCaretOffset()
					&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
				conceptsSel.add(a);
			}
		}

		List<AnnotationFS> conceptsSelGold = new ArrayList<AnnotationFS>();
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptGoldType)) {

			if (a.getBegin() <= editor.getWidget().getCaretOffset()
					&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
				conceptsSelGold.add(a);
			}
		}

		if (conceptsSel.size() == 1) {

			// accept as gold annotation
			Action acceptAsGoldAction = new Action() {

				@Override
				public void run() {

					AnnotationFS annotationFS = conceptsSel.get(0);

					cloneAndAddAsGoldAnnotation(annotationFS);
				}

			};
			acceptAsGoldAction.setText("Accept as gold!");
			manager.add(acceptAsGoldAction);
		} else {

			// TODO unclear which annotation is to be sleected
		}

		if (conceptsSelGold.size() > 0) {

			Action deleteGoldAnnosAction = new Action() {
				@Override
				public void run() {

					for (AnnotationFS anno : conceptsSelGold) {
						editor.removeAnnotation(anno);
					}
				}
			};

			deleteGoldAnnosAction.setText("Delete gold annotations..");
			manager.add(deleteGoldAnnosAction);
		}

		else if (conceptsSelGold.size() == 0 && !editor.getWidget().getSelectionText().isEmpty()) {

			Action addGoldWithoutIDAction = new Action() {
				@Override
				public void run() {
					AnnotationFS fs = editor.getCas().createAnnotation(conceptGoldType,
							editor.getWidget().getSelection().x, editor.getWidget().getSelection().y);

					fs.setFeatureValueFromString(featConceptIdGold, "???");
					editor.addAnnotation(fs);

				}
			};
			addGoldWithoutIDAction.setText("Add Gold without terminology node");
			manager.add(addGoldWithoutIDAction);

		}

	}

	private void applyTerminologyToBatch(final Map<File, CAS> workloadMap) {

		buttonApplyToFolder.setEnabled(false);
		final Shell shell = new Shell(Display.getCurrent(), SWT.ON_TOP | SWT.TITLE);
		shell.setText("Apply terminology...");
		shell.setLayout(new FillLayout(SWT.VERTICAL));
		final Label lbProgress = new Label(shell, SWT.HORIZONTAL);
		lbProgress.setText("Applying Terminology...");
		ProgressBar bar = new ProgressBar(shell, SWT.HORIZONTAL | SWT.SMOOTH);
		bar.setMaximum(workloadMap.size());
		shell.pack();

		java.awt.Point location = MouseInfo.getPointerInfo().getLocation();
		shell.setLocation(location.x, location.y);
		shell.open();

		Runnable runnable = new Runnable() {

			@Override
			public void run() {
				// apply the algorithm
				for (File f : workloadMap.keySet()) {

					// we need to delete the old types in those cases!
					deleteOldAnnotations(workloadMap.get(f));

					algorithm.apply(terminology, workloadMap.get(f));
					addEvaluationTypes(workloadMap.get(f));
					Runnable r = new Runnable() {

						@Override
						public void run() {
							lbProgress.setText("Applying Terminology to " + workloadMap.size() + " documents!");

							bar.setSelection(bar.getSelection() + 1);
							shell.pack();
						}
					};
					Display.getDefault().asyncExec(r);

					FileOutputStream fos;
					try {
						fos = new FileOutputStream(f);
						XmiCasSerializer.serialize(workloadMap.get(f), fos);
						fos.close();
					} catch (SAXException | IOException e) {
						e.printStackTrace();
					}
				}
				Display.getDefault().asyncExec(() -> shell.close());
				// reload
				Display.getDefault().asyncExec(() -> updateModelAndReloadEditor());
				Display.getDefault().asyncExec(() -> buttonApplyToFolder.setEnabled(true));
				Display.getDefault().asyncExec(
						() -> new DocumentResultDialog(Display.getDefault().getActiveShell(), workloadMap, broker)
								.open());

			}

			private void deleteOldAnnotations(CAS cas) {

				List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

				for (AnnotationFS att : cas.getAnnotationIndex(conceptType)) {
					toDelete.add(att);
				}

				// also delete evaluation types

				for (AnnotationFS eval : cas.getAnnotationIndex(medEvalType)) {
					toDelete.add(eval);
				}

				for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeHighlightAction)) {
					toDelete.add(temp);
				}

				for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTempAttributes)) {
					toDelete.add(temp);
				}

				// delete without a reload
				for (AnnotationFS a : toDelete) {
					cas.removeFsFromIndexes(a);
				}

			}

		};

		// delete all annotations from previous run
		new Thread(runnable).start();

	}

	// what to do when the editor caret was changed
	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {

		AnnotationFS conceptSel = null;
		for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptType)) {

			if (a.getBegin() <= editor.getWidget().getCaretOffset()
					&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
				conceptSel = a;
				break;
			}
		}

		if (conceptSel == null)
			return;

		String conceptId = conceptSel.getFeatureValueAsString(featConceptId);

		MedicalTerminology input = (MedicalTerminology) terminologyTreeView.getViewer().getInput();

		if (input == null)
			return;
		List<Concept> successors = input.getRoot().getSuccessors();

		for (Concept c : successors) {

			if (String.valueOf(c.getId()).equals(conceptId)) {

				terminologyTreeView.getViewer().expandToLevel(c, AbstractTreeViewer.ALL_LEVELS);
				terminologyTreeView.getViewer()
						.expandToLevel(new StructuredSelection(c), AbstractTreeViewer.ALL_LEVELS);
				terminologyTreeView.getViewer().setSelection(new StructuredSelection(c), true);
			}
		}

	}

	private void cloneAndAddAsGoldAnnotation(AnnotationFS annotationFS) {

		AnnotationFS fs = editor.getCas().createAnnotation(conceptGoldType, annotationFS.getBegin(),
				annotationFS.getEnd());

		fs.setFeatureValueFromString(featConceptIdGold, annotationFS.getFeatureValueAsString(featConceptId));
		fs.setFeatureValueFromString(featTypeContentGold, annotationFS.getFeatureValueAsString(featTypeContent));

		editor.addAnnotation(fs);

	}

	// what to do when the editor is clicked
	@Override
	public void onEditorMouseDown(org.osgi.service.event.Event arg0) {

		// if (isHoverAcceptable) {
		//
		// // we accept this annotation
		// AnnotationFS selectedAnno = this.hoverSelection;
		//
		// if (selectedAnno != null) {

		//
		// if (selectedAnno.getType().equals(conceptType)) {
		//
		// selectedAnno.setFeatureValueFromString(featureAttIsGold, "true");
		// }
		//
		// editor.changeAnnotation(selectedAnno);
		// updateEvaluationType(new Point(selectedAnno.getBegin(),
		// selectedAnno.getEnd()), "TP");
		//
		// }
		// }
		//
		// else if (isHoverDeletable) {
		//
		// // we delete
		// AnnotationFS selectedAnno = this.hoverSelection;
		//
		// if (selectedAnno != null) {
		//
		// editor.removeAnnotation(selectedAnno);
		// }
		//
		// updateModel();
		// updateEvaluationType(new Point(selectedAnno.getBegin(),
		// selectedAnno.getEnd()), "FP");
		// }

		Point point = (Point) arg0.getProperty(IEventBroker.DATA);

		if (point.x < editor.getWidget().getLeftMargin()) {

			AnnotationFS smallest = null;
			for (AnnotationFS scene : editor.getCas().getAnnotationIndex(typeHighlightAction)) {

				boolean doesCover = coveresRectangle(point, scene);
				if (doesCover) {

					if (smallest == null)
						smallest = scene;

					if (scene.getBegin() >= smallest.getBegin() && scene.getEnd() <= smallest.getEnd()) {
						smallest = scene;
					}

				}
			}
			// jump to the beginning
			if (smallest != null) {
				editor.highlightRange(smallest.getBegin(), smallest.getEnd() - smallest.getBegin(), true);

			}
		}

	}

	// what to do when the editor has a changed textselection
	@Override
	public void onEditorTextSelectionChanged(org.osgi.service.event.Event arg0) {
		// Point point = (Point) arg0.getProperty(IEventBroker.DATA);

	}

	// on editor mouse moved - use this if you want some fancy effects
	@Override
	public void onEditorMouseMoved(org.osgi.service.event.Event e) {

		// Point point = (Point) e.getProperty(IEventBroker.DATA);
		//
		// if (point.x >= 0)
		// return;
		//
		// for (AnnotationFS a : this.annos) {
		//
		// Rectangle textBounds = editor.getWidget().getTextBounds(a.getBegin(),
		// a.getEnd());
		// Rectangle textBoundsReal =
		// editor.getWidget().getTextBounds(a.getBegin(), a.getEnd());
		//
		// textBounds.height = textBounds.height +
		// imageAccept.getBounds().height;
		// textBounds.y = textBounds.y - imageAccept.getBounds().height;
		//
		// if (textBounds.contains(point)) {
		//
		// this.hoverSelection = a;
		// GC gc = new GC(editor.getWidget());
		//
		// this.hasHoverDisplay = true;
		// gc.drawImage(imageAccept, textBounds.x, textBounds.y - 2);
		// gc.drawImage(imageDelete, textBounds.x + textBounds.width -
		// imageDelete.getBounds().width - 3,
		// textBounds.y - 3);
		//
		// gc.dispose();
		//
		// // mouse is in the left half - accept !!
		// if ((point.x <= textBoundsReal.x + imageAccept.getBounds().width) &&
		// point.y < textBoundsReal.y) {
		// editor.getWidget().setCursor(new Cursor(Display.getCurrent(),
		// SWT.CURSOR_HAND));
		//
		// this.isHoverAcceptable = true;
		// this.isHoverDeletable = false;
		//
		// }
		//
		// // right we delete
		// else if ((point.x >= textBoundsReal.x + textBounds.width -
		// imageDelete.getBounds().width)
		// && point.y < textBoundsReal.y) {
		// editor.getWidget().setCursor(new Cursor(Display.getCurrent(),
		// SWT.CURSOR_HAND));
		//
		// this.isHoverDeletable = true;
		// this.isHoverAcceptable = false;
		// }
		//
		// else {
		// editor.getWidget().setCursor(new Cursor(Display.getCurrent(),
		// SWT.CURSOR_IBEAM));
		// this.isHoverDeletable = false;
		// this.isHoverAcceptable = false;
		//
		// }
		//
		// // we only need 1 match
		// return;
		// }
		// }
		// if (this.hasHoverDisplay) {
		// editor.invokeReload(editor.getWidget().getCaretOffset(), 2000);
		// editor.getWidget().setCursor(new Cursor(Display.getCurrent(),
		// SWT.CURSOR_IBEAM));
		// this.isHoverDeletable = false;
		// this.isHoverAcceptable = false;
		//
		// this.hasHoverDisplay = false;
		// }

		// used for border drawing
		Point point = (Point) e.getProperty(IEventBroker.DATA);

		if (point.x < editor.getWidget().getLeftMargin()) {

			// check if we are hovered over a rectangle

			boolean found = false;
			for (AnnotationFS scene : editor.getCas().getAnnotationIndex(typeHighlightAction)) {

				boolean doesCover = coveresRectangle(point, scene);
				if (doesCover) {
					editor.getWidget().setCursor(new Cursor(Display.getCurrent(), SWT.CURSOR_HAND));
					found = true;
					break;
				}
			}

			if (found) {

				return;
			}
		}
		editor.getWidget().setCursor(new Cursor(Display.getCurrent(), SWT.CURSOR_IBEAM));

	}

	private void findAttributesBySubstantives() {

		List<AnnotationFS> concepts = new ArrayList<AnnotationFS>();
		for (AnnotationFS tok : editor.getCas().getAnnotationIndex(posType)) {

			String posTag = tok.getFeatureValueAsString(featPOSTag);
			if (posTag.startsWith("N")) {
				AnnotationFS concept = editor.getCas().createAnnotation(conceptType, tok.getBegin(), tok.getEnd());
				concept.setFeatureValueFromString(featTypeContent, "ATTRIBUTE");
				concepts.add(concept);

			}
		}

		for (AnnotationFS concept : concepts) {
			editor.getCas().addFsToIndexes(concept);
		}

		editor.changeAnnotation(concepts.get(0));
	}

	private boolean coveresRectangle(Point point, AnnotationFS anno) {
		Rectangle textBounds = editor.getWidget().getTextBounds(0, 1);
		int lineTop = Math.abs(editor.getWidget().getTextBounds(anno.getBegin(), anno.getBegin() + 1).y - textBounds.y);
		int lineBottom = Math.abs(editor.getWidget().getTextBounds(anno.getEnd() - 1, anno.getEnd()).y - textBounds.y);

		Rectangle textBounds2 = editor.getWidget().getTextBounds(editor.getWidget().getText().length() - 2,
				editor.getWidget().getText().length() - 1);
		int lineCnt = Math.abs(textBounds.y) + Math.abs(textBounds2.y);

		int pixelTop = BorderDrawingStrategy.lineToPixel(editor.getWidget().getBounds().height, lineTop, lineCnt);
		int pixelBottom = BorderDrawingStrategy.lineToPixel(editor.getWidget().getBounds().height, lineBottom, lineCnt);

		if (pixelTop == pixelBottom) {
			// increase the rectangle artificially
			pixelBottom += 3;
			pixelTop -= 3;
		}

		if (point.y >= pixelTop && point.y <= pixelBottom) {
			return true;
		}

		return false;
	}

	@Override
	public void initModel() {
		// TODO Auto-generated method stub

	}

	public TerminologyTreeView createTerminologyTreeView(Composite parent) {
		return new TerminologyTreeView(parent, broker, terminology, typeHighlightAction, typeTempAttributes,
				conceptType, medEvalType, featTypeContent, featMedEvalType, algorithm, editor, featConceptIdGold,
				conceptGoldType);
	}

	private void applyTerminology(String shellText, CAS casToApply) {
		if (terminology == null || casToApply == null) {
			return;
		}

		buttonApplyTerminology.setEnabled(false);
		Shell shell = new Shell(Display.getCurrent(), SWT.ON_TOP | SWT.TITLE);
		shell.setText(shellText);
		shell.setLayout(new FillLayout(SWT.VERTICAL));
		Label lbProgress = new Label(shell, SWT.HORIZONTAL);
		lbProgress.setText("Applying Terminology...");
		ProgressBar bar = new ProgressBar(shell, SWT.HORIZONTAL | SWT.SMOOTH);
		bar.setMaximum(algorithm.getTotalWork());
		shell.pack();

		java.awt.Point location = MouseInfo.getPointerInfo().getLocation();
		shell.setLocation(location.x, location.y);
		shell.open();

		// Listener that propagates progress from algorithm to progressbar
		PropertyChangeListener listener = new PropertyChangeListener() {

			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				if (!(evt.getNewValue() instanceof Integer)) {
					return;
				}
				Integer newValue = (Integer) evt.getNewValue();
				if (evt.getPropertyName().equals(ATerminologyMatchingAlgorithm.EVENT_TOTALWORK_CHANGED)) {
					shell.getDisplay().asyncExec(() -> bar.setMaximum(newValue));
				} else if (evt.getPropertyName().equals(ATerminologyMatchingAlgorithm.EVENT_PROGRESS_CHANGED)) {
					shell.getDisplay().asyncExec(() -> bar.setSelection(newValue));
				}
			}
		};

		// actual work done in background so UI is not blocked
		Runnable runnable = new Runnable() {

			@Override
			public void run() {
				// apply the algorithm
				algorithm.addPropertyChangeListener(listener);
				algorithm.apply(terminology, casToApply);
				algorithm.removePropertyChangeListener(listener);
				addEvaluationTypes(casToApply);

				Display.getDefault().asyncExec(() -> shell.close());
				// reload
				Display.getDefault().asyncExec(() -> updateModelAndReloadEditor());
				Display.getDefault().asyncExec(() -> buttonApplyTerminology.setEnabled(true));
			}

		};

		// delete all annotations from previous run
		deleteAttributesAndValues();
		deleteTemporaryTypes();
		new Thread(runnable).start();
	}

	private void updateModelAndReloadEditor() {
		updateModel();
		labelProgress.setImage(createImageByOverlap(null, labelProgress));
		labelProgressCountColor.setImage(createImageByOverlap("RESPECT_COUNTS", labelProgressCountColor));
		editor.invokeReload();
	}

	private void addEvaluationTypes(CAS casToApply) {

		List<AnnotationFS> goldTp = new ArrayList<AnnotationFS>();
		List<AnnotationFS> systemTp = new ArrayList<AnnotationFS>();

		for (AnnotationFS concept : casToApply.getAnnotationIndex(conceptType)) {

			for (AnnotationFS conceptGold : casToApply.getAnnotationIndex(conceptGoldType)) {

				if (concept.getBegin() == conceptGold.getBegin() && concept.getEnd() == conceptGold.getEnd()) {

					String idSystem = concept.getFeatureValueAsString(featConceptId);
					String idGold = concept.getFeatureValueAsString(featConceptIdGold);

					if (idSystem.equals(idGold)) {
						goldTp.add(conceptGold);
						systemTp.add(concept);
						break;
					}

				}
			}
		}

		// create the annotations
		for (AnnotationFS concept : casToApply.getAnnotationIndex(conceptType)) {

			if (systemTp.contains(concept)) {
				// true positive
				AnnotationFS fs = casToApply.createAnnotation(medEvalType, concept.getBegin(), concept.getEnd());
				fs.setFeatureValueFromString(featMedEvalType, "TP");
				casToApply.addFsToIndexes(fs);
			}

			else {

				// false positive
				AnnotationFS fs = casToApply.createAnnotation(medEvalType, concept.getBegin(), concept.getEnd());
				fs.setFeatureValueFromString(featMedEvalType, "FP");
				casToApply.addFsToIndexes(fs);
			}
		}

		for (AnnotationFS conceptGold : casToApply.getAnnotationIndex(conceptGoldType)) {

			if (!goldTp.contains(conceptGold)) {
				// FN
				AnnotationFS fs = casToApply
						.createAnnotation(medEvalType, conceptGold.getBegin(), conceptGold.getEnd());
				fs.setFeatureValueFromString(featMedEvalType, "FN");
				casToApply.addFsToIndexes(fs);
			}
		}

	}
}
