package de.uniwue.mk.medIE.terminologyView;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.TransferData;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.medIE.terminology.Concept;

public class TreeViewerDropListener extends ViewerDropAdapter {

	private TreeViewer viewer;

	private Concept fromConcept;

	private Concept toConcept;

	private IEventBroker broker;

	public TreeViewerDropListener(Viewer viewer, IEventBroker broker) {
		super(viewer);

		this.viewer = (TreeViewer) viewer;
		this.broker = broker;
	}

	@Override
	public void drop(DropTargetEvent event) {
		// even has "To" information
		// int location = this.determineLocation(event);
		Concept anno = (Concept) determineTarget(event);
		toConcept = anno;
		if (LocalSelectionTransfer.getTransfer().isSupportedType(event.currentDataType)) {
			ISelection sel = LocalSelectionTransfer.getTransfer().getSelection();
			IStructuredSelection struct = (IStructuredSelection) sel;
			fromConcept = (Concept) struct.getFirstElement();
		}

		super.drop(event);
	}

	// This method performs the actual drop
	// We simply add the String we receive to the model and trigger a refresh of
	// the
	// viewer by calling its setInput method.
	@Override
	public boolean performDrop(Object data) {

		if (fromConcept == null || toConcept == null || toConcept.getId() == fromConcept.getId())
			return false;

		// we need to delete from from its old parent
		fromConcept.getParent().removeChild(fromConcept);

		// and change its parent
		fromConcept.setParent(toConcept);
		toConcept.addChild(fromConcept);

		// viewer.refresh();

		broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, toConcept);
		return true;
	}

	@Override
	public boolean validateDrop(Object target, int operation, TransferData transferType) {
		return true;

	}
}
