package de.uniwue.mk.medIE.terminologyView.menu.actions;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.action.Action;

import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.medIE.extraction.algorithm.ATerminologyMatchingAlgorithm;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;

public class ApplyTerminologyForNode extends Action {

	IEventBroker broker;
	private Concept concept;
	private AnnotationEditorWidget editor;
	private Type typeTempAttributes;
	private Type typeHighlight;
	private MedicalTerminology terminology;
	private ATerminologyMatchingAlgorithm algorithm;
	private Type medEvalType;
	private Type conceptType;
	private Feature featTypeContent;

	public ApplyTerminologyForNode(IEventBroker broker, AnnotationEditorWidget editor, Concept selectedConcept,
			Type typeHighlightAction, Type typeTempAttributes, ATerminologyMatchingAlgorithm algorithm,
			Type conceptType, Type medEvalType, Feature featTypeContent) {
		super("Apply Terminology For Node");
		this.broker = broker;
		this.editor = editor;
		this.concept = selectedConcept;
		this.typeTempAttributes = typeTempAttributes;
		this.typeHighlight = typeHighlightAction;
		this.algorithm = algorithm;
		this.medEvalType = medEvalType;
		this.conceptType = conceptType;
		this.featTypeContent = featTypeContent;

	}

	@Override
	public void run() {

		deleteTemporaryTypes();
		deleteAttributesAndValues();

		// apply

		// create a dummy terminology for that purpose

		Concept root = new Concept("ROOT", null, null, 0);
		MedicalTerminology term = new MedicalTerminology(root, "DUMMY");

		root.addChild(concept);

		algorithm.apply(term, editor.getCas());

		List<AnnotationFS> toAdd = new ArrayList<AnnotationFS>();
		for (AnnotationFS concept : editor.getCas().getAnnotationIndex(conceptType)) {

			if (concept.getFeatureValueAsString(featTypeContent).contains("ATTRIBUTE")) {
				AnnotationFS fs = editor.getCas().createAnnotation(typeHighlight, concept.getBegin(), concept.getEnd());

				toAdd.add(fs);
			}

		}

		for (AnnotationFS add : toAdd) {
			editor.getCas().addFsToIndexes(add);
		}

		if (!toAdd.isEmpty()) {
			editor.addAnnotation(toAdd.get(0));
		}

	}

	private void deleteTemporaryTypes() {
		// delete quietly

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeHighlight)) {
			toDelete.add(temp);
		}

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTempAttributes)) {
			toDelete.add(temp);
		}

		for (AnnotationFS del : toDelete)
			editor.getCas().removeFsFromIndexes(del);

		if (!toDelete.isEmpty()) {
			editor.removeAnnotation(toDelete.get(0));
		}

	}

	private void deleteAttributesAndValues() {

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS att : editor.getCas().getAnnotationIndex(conceptType)) {
			toDelete.add(att);
		}

		// also delete evaluation types

		for (AnnotationFS eval : editor.getCas().getAnnotationIndex(medEvalType)) {
			toDelete.add(eval);
		}

		// delete without a reload
		for (AnnotationFS a : toDelete) {
			editor.getCas().removeFsFromIndexes(a);
			;
		}

		if (!toDelete.isEmpty()) {
			editor.removeAnnotation(toDelete.get(0));
		}

	}

}
