package de.uniwue.mk.medIE.terminologyView.menu.actions;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.action.Action;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;

public class HighlightSegmentsAction extends Action {

	IEventBroker broker;
	private Concept concept;
	private AnnotationEditorWidget editor;
	private Type typeTempAttributes;
	private Type typeHighlight;
	private MedicalTerminology terminology;

	public HighlightSegmentsAction(IEventBroker broker, AnnotationEditorWidget editor, Concept selectedConcept,
			Type typeHighlightAction, Type typeTempAttributes, MedicalTerminology terminology) {
		super("Highlight Segments");
		this.broker = broker;
		this.editor = editor;
		this.concept = selectedConcept;
		this.typeTempAttributes = typeTempAttributes;
		this.typeHighlight = typeHighlightAction;
		this.terminology = terminology;

	}

	@Override
	public void run() {

		// set the property for the treepainting
		for (Concept c : terminology.getRoot().getSuccessors()) {
			c.setProperty("CONCEPT_HIGHLIGHT", "FALSE");
		}
		deleteTemporaryTypes();
		concept.setProperty("CONCEPT_HIGHLIGHT", "TRUE");
		// we add annotations that display all the segments and highlight them

		List<String> synonymsAsRegex = concept.getSynonymsAsRegex();

		List<AnnotationFS> toAdd = new ArrayList<AnnotationFS>();
		for (String regex : synonymsAsRegex) {

			Matcher m = Pattern.compile(regex).matcher(editor.getCas().getDocumentText());

			while (m.find()) {

				AnnotationFS fsTemp = editor.getCas().createAnnotation(typeTempAttributes, m.start(), m.end());
				AnnotationFS fsTemp2 = editor.getCas().createAnnotation(typeHighlight, m.start(), m.end());
				toAdd.add(fsTemp);
				toAdd.add(fsTemp2);

			}
		}

		for (AnnotationFS add : toAdd) {
			editor.getCas().addFsToIndexes(add);
		}

		if (toAdd.size() > 0) {
			editor.changeAnnotation(toAdd.get(0));
		}

		broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, concept);
	}

	private void deleteTemporaryTypes() {
		// delete quietly

		List<AnnotationFS> toDelete = new ArrayList<AnnotationFS>();

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeHighlight)) {
			toDelete.add(temp);
		}

		for (AnnotationFS temp : editor.getCas().getAnnotationIndex(typeTempAttributes)) {
			toDelete.add(temp);
		}

		for (AnnotationFS del : toDelete)
			editor.getCas().removeFsFromIndexes(del);

	}

}
