package de.uniwue.mk.medIE.terminologyView.widgets;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.ENodeType;

public class NodeAttributeComposite extends Composite {

	private int fontSize = 8;

	private static final String ATTRIBUTE = "ATTRIBUTE";

	private static final String CONCEPT = "CONCEPT";

	private static final String ENODETYPE = "ENODETYPE";

	private Composite compConceptAttributes;

	private Composite compNodeTypes;

	private IEventBroker broker;

	private Concept concept;

	private Map<ENodeType, Button> typeButtons;

	private Map<String, Button> attributeButtons;

	public NodeAttributeComposite(Composite parent, int style, IEventBroker broker, int fontsize) {
		super(parent, style);

		this.broker = broker;
		createCompNodeType();
		createCompAttributesType();
		this.fontSize = fontsize;

		setLayout(parent.getLayout());
	}

	private void createCompNodeType() {
		compNodeTypes = new Composite(this, SWT.NONE);

		compNodeTypes.setLayout(new GridLayout(5, false));
		GridData gd = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		gd.horizontalSpan = 6;

		compNodeTypes.setLayoutData(gd);

		// add some checkboxes to the composite
		// Label labelNodeType = new Label(compNodeTypes, SWT.NONE);
		// labelNodeType.setText("Type:");

		typeButtons = new HashMap<>();
		for (ENodeType type : ENodeType.values()) {

			final Button buttonType = new Button(compNodeTypes, SWT.RADIO);
			buttonType.setText(type.toString());
			buttonType.setData(ENODETYPE, type);
			buttonType.setData(CONCEPT, concept);

			FontData[] fD = buttonType.getFont().getFontData();
			fD[0].setHeight(fontSize);
			buttonType.setFont(new Font(Display.getCurrent(), fD[0]));

			buttonType.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					// cannot do anything if current concept is null
					if (concept == null) {
						return;
					}
					// change
					concept.setNodeType((ENodeType) buttonType.getData(ENODETYPE));

					compNodeTypes.layout();

					broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, concept);
				}

			});
			typeButtons.put(type, buttonType);
		}

		compNodeTypes.layout();
	}

	private void createCompAttributesType() {
		compConceptAttributes = new Composite(this, SWT.NONE);

		compConceptAttributes.setLayout(new GridLayout(4, false));
		GridData gd = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		gd.horizontalSpan = 6;

		compConceptAttributes.setLayoutData(gd);

		// add some checkboxes to the composite

		attributeButtons = new HashMap<>();
		for (String type : Concept.getBooleanAttributesAsStringArray()) {

			int buttonAppearance = SWT.CHECK;
			final Button buttonType = new Button(compConceptAttributes, buttonAppearance);
			buttonType.setText(type.toString());
			buttonType.setData(ATTRIBUTE, type);

			FontData[] fD = buttonType.getFont().getFontData();
			fD[0].setHeight(fontSize);
			buttonType.setFont(new Font(Display.getCurrent(), fD[0]));

			buttonType.addSelectionListener(new SelectionAdapter() {

				@Override
				public void widgetSelected(SelectionEvent e) {
					// cannot do anything if current concept is null
					if (concept == null) {
						return;
					}
					// change
					String data = (String) buttonType.getData(ATTRIBUTE);

					// set the attribute
					concept.changeAttribute(data);

					layout();
					broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, concept);
				}

			});
			attributeButtons.put(type, buttonType);
		}

		layout();
	}

	/**
	 * Sets the concept that can be modified via this Composite. 
	 * @param newConcept
	 */
	public void setConcept(Concept newConcept) {
		concept = newConcept;
		setEnabled(newConcept != null);
		updateSelections();
	}

	/**
	 * Updates the buttons so they display the state of the current concept. 
	 */
	private void updateSelections() {
		if (concept == null) {
			return;
		}
		typeButtons.entrySet().stream().forEach(entry -> entry.getValue().setSelection(false));
		typeButtons.entrySet().stream().filter(entry -> entry.getKey().equals(concept.getNodeType()))
		.forEach(entry -> entry.getValue().setSelection(true));
		attributeButtons.entrySet().forEach(entry 
				-> entry.getValue().setSelection(concept.getAttribute(entry.getKey())));
	}
}
