package de.uniwue.mk.medIE.terminologyView.widgets;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.CAS;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.widget.editor.CASEditorInput;

public class TerminologyApplicationResultComposite extends Composite {

	private TableViewer viewer;
	private TerminologyBulkDialogViewerComparator colViewerComp;
	private List<IBulkTableListener> listenerList;
	private Type medEvalType;
	private Feature featMedEvalType;

	public TerminologyApplicationResultComposite(Composite parent, int style) {
		super(parent, style);

		listenerList = new ArrayList<IBulkTableListener>();
		this.setLayout(new GridLayout(1, false));

		this.addDisposeListener(new DisposeListener() {

			@Override
			public void widgetDisposed(DisposeEvent e) {
				// TODO Auto-generated method stub

			}
		});

		initLayout(parent);
	}

	public void addBulkTableListener(IBulkTableListener listener) {
		if (!this.listenerList.contains(listener)) {
			this.listenerList.add(listener);
		}
	}

	public void removeBulkTableListener(IBulkTableListener listener) {
		this.listenerList.remove(listener);
	}

	public void setInput(List<CASEditorInput> input) {

		if (input.size() != 0) {
			this.medEvalType = input.get(0).getCas().getTypeSystem().getType("de.uniwue.medIE.Eval");
			this.featMedEvalType = medEvalType.getFeatureByBaseName("Type");
		}
		this.viewer.setInput(input);
		this.viewer.refresh();

	}

	private void initLayout(Composite parent) {

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);

		viewer.setContentProvider(ArrayContentProvider.getInstance());
		viewer.getTable().addMouseListener(new MouseAdapter() {

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				for (IBulkTableListener listener : listenerList) {
					listener.tableDoubleClicked((CASEditorInput) ((IStructuredSelection) viewer.getSelection())
							.getFirstElement());
				}

			}
		});

		colViewerComp = new TerminologyBulkDialogViewerComparator();
		// make lines and header visible
		final Table table = viewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);
		viewer.setComparator(colViewerComp);

		GridData layoutDataTable = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		layoutDataTable.horizontalSpan = 1;
		layoutDataTable.heightHint = 350;
		layoutDataTable.widthHint = 400;
		viewer.getControl().setLayoutData(layoutDataTable);

	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Document", 150, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CASEditorInput input = (CASEditorInput) element;

				return input.getEditorInputFile().getName();

			}

		});

		TableViewerColumn colTP = createTableViewerColumn("TP", 50, 0);
		colTP.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CASEditorInput input = (CASEditorInput) element;
				int count = countEvaluationTypes(input.getCas(), "TP");
				return String.valueOf(count);

			}

		});

		TableViewerColumn colFP = createTableViewerColumn("FP", 50, 0);
		colFP.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CASEditorInput input = (CASEditorInput) element;
				int count = countEvaluationTypes(input.getCas(), "FP");
				return String.valueOf(count);
			}

		});

		TableViewerColumn colFN = createTableViewerColumn("FN", 50, 0);
		colFN.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				CASEditorInput input = (CASEditorInput) element;
				int count = countEvaluationTypes(input.getCas(), "FN");
				return String.valueOf(count);
			}

		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				colViewerComp.setColumn(index);
				int dir = colViewerComp.getDirection();
				viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	private int countEvaluationTypes(CAS cas, String featType) {

		int cnt = 0;
		for (AnnotationFS a : cas.getAnnotationIndex(medEvalType)) {

			String feat = a.getFeatureValueAsString(featMedEvalType);
			if (feat.equals(featType)) {
				cnt++;
			}
		}
		return cnt;
	}

}
