package de.uniwue.mk.medIE.terminologyView.widgets;

import java.util.ArrayList;
import java.util.List;

import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.events.IAnnotationEditorEvents;
import de.uniwue.mk.kall.athen.widget.editor.AnnotationEditorWidget;
import de.uniwue.mk.medIE.extraction.algorithm.ATerminologyMatchingAlgorithm;
import de.uniwue.mk.medIE.terminology.Concept;
import de.uniwue.mk.medIE.terminology.ENodeType;
import de.uniwue.mk.medIE.terminology.Lexikon;
import de.uniwue.mk.medIE.terminology.MedicalTerminology;
import de.uniwue.mk.medIE.terminology.Variation;
import de.uniwue.mk.medIE.terminologyView.AttributeAcceptanceDialog;
import de.uniwue.mk.medIE.terminologyView.ConceptEditingDialog;
import de.uniwue.mk.medIE.terminologyView.ConceptIdFilter;
import de.uniwue.mk.medIE.terminologyView.ConceptNameFilter;
import de.uniwue.mk.medIE.terminologyView.LexikonEditingDialog;
import de.uniwue.mk.medIE.terminologyView.TerminologyContentProvider;
import de.uniwue.mk.medIE.terminologyView.TerminologyLabelContentProvider;
import de.uniwue.mk.medIE.terminologyView.TreeViewerAlphabeticalSorter;
import de.uniwue.mk.medIE.terminologyView.TreeViewerDragListener;
import de.uniwue.mk.medIE.terminologyView.TreeViewerDropListener;
import de.uniwue.mk.medIE.terminologyView.menu.actions.AddNegationAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.ApplyTerminologyForNode;
import de.uniwue.mk.medIE.terminologyView.menu.actions.ConvertConceptToDictionaryAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.DeselectConceptAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.DuplicateNodeAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.HighlightSegmentsAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.MergeConceptsAction;
import de.uniwue.mk.medIE.terminologyView.menu.actions.TooltipDisplayOptionsAction;

public class TerminologyTreeView extends Composite {
	private Button buttonAddSibling;
	private Button buttonAddChild;
	private Button buttonDeleteNode;
	private Button buttonEditNode;
	private Button buttonShowLexika;

	private TreeViewer viewer;
	private TreeViewerAlphabeticalSorter treeSorter;
	private ConceptNameFilter filter;
	private NodeAttributeComposite compNodeAttributes;
	private AttributeAcceptanceDialog dialog;

	private AnnotationEditorWidget editor;

	private IEventBroker broker;

	private Concept lastSelectedConcept;

	private MedicalTerminology terminology;

	private Type typeHighlightAction;
	private Type typeTempAttributes;
	private Type typeConcept;
	private Type typeMedEval;
	private Feature featTypeContent;
	private Feature featMedEvalType;

	private ATerminologyMatchingAlgorithm algorithm;
	private Feature featConceptGoldid;
	private Type conceptGoldType;

	public TerminologyTreeView(Composite parent, IEventBroker broker, MedicalTerminology terminology,
			Type typeHighlightAction, Type typeTempAttributes, Type conceptType, Type medEvalType,
			Feature featTypeContent, Feature featMedEvalType, ATerminologyMatchingAlgorithm algorithm,
			AnnotationEditorWidget editor, Feature featConceptGoldId, Type conceptGoldType) {
		super(parent, SWT.BORDER);

		this.broker = broker;
		this.terminology = terminology;
		this.typeHighlightAction = typeHighlightAction;
		this.typeTempAttributes = typeTempAttributes;
		this.typeConcept = conceptType;
		this.typeMedEval = medEvalType;
		this.featTypeContent = featTypeContent;
		this.featMedEvalType = featMedEvalType;
		this.algorithm = algorithm;
		this.editor = editor;
		this.featConceptGoldid = featConceptGoldId;
		this.conceptGoldType = conceptGoldType;

		createControls();
	}

	private void createControls() {
		setLayout(new GridLayout(5, false));
		GridData layoutData = new GridData(GridData.FILL_BOTH | GridData.GRAB_HORIZONTAL | GridData.GRAB_VERTICAL);
		layoutData.horizontalSpan = 6;
		setLayoutData(layoutData);

		// create the header Buttons
		createTopButtons();

		// create search field
		Label labelSearch = new Label(this, SWT.NONE);
		labelSearch.setText("Search: ");

		Text textSearch = new Text(this, SWT.BORDER);
		GridData gdText = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		gdText.horizontalSpan = 4;
		textSearch.setLayoutData(gdText);

		textSearch.addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {

				filter.setSearchText(textSearch.getText());
				viewer.refresh();
			}

			@Override
			public void keyPressed(KeyEvent e) {

			}
		});

		// create the tree
		createTreeViewer();
		viewer.getControl().setLayoutData(layoutData);

		compNodeAttributes = new NodeAttributeComposite(this, SWT.NONE, broker, 8);
		GridData gdComp = new GridData();
		gdComp.horizontalSpan = 6;
		compNodeAttributes.setLayoutData(gdComp);
	}

	private void createTreeViewer() {
		viewer = new TreeViewer(this, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL);

		// viewer.addFilter(new AnnotationBrowserNoAnnoFilter());

		viewer.setContentProvider(new TerminologyContentProvider());

		viewer.setLabelProvider(new TerminologyLabelContentProvider(viewer.getTree()));
		viewer.getTree().setData(TooltipDisplayOptionsAction.TREE_TOOLTIP_OPTION,
				TooltipDisplayOptionsAction.TREE_TOOLTIP_FULL);
		treeSorter = new TreeViewerAlphabeticalSorter();

		viewer.setComparator(treeSorter);

		viewer.getTree().addKeyListener(new KeyAdapter() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.CR) {

					if (buttonEditNode.isEnabled()) {
						buttonEditNode.notifyListeners(SWT.Selection, new Event());
					}
				}

				else if (e.keyCode == SWT.DEL) {

					if (buttonDeleteNode.isEnabled()) {
						buttonDeleteNode.notifyListeners(SWT.Selection, new Event());
					}
				}

				// TODO add key to assign gold feature
			}

		});

		// create menumanager
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(viewer.getControl());

		// attach that menu manager to the treeviewer
		viewer.getControl().setMenu(menu);

		menuMgr.addMenuListener(new IMenuListener() {

			@Override
			public void menuAboutToShow(IMenuManager manager) {
				if (viewer.getSelection().isEmpty()) {
					return;
				}

				Action tooltipDisplayOptionsAction = new TooltipDisplayOptionsAction(broker, viewer.getTree());
				manager.add(tooltipDisplayOptionsAction);

				if (viewer.getSelection() instanceof IStructuredSelection) {
					IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
					Object object = selection.getFirstElement();
					List selectedElements = selection.toList();

					if (object == null)
						return;
					// correctionFiles can be deleted
					if (object instanceof Concept) {
						Concept concept = (Concept) object;

						String property = (String) concept.getProperty("CONCEPT_HIGHLIGHT");

						if (property != null && property.equals("TRUE")) {
							Action deselectConceptAction = new DeselectConceptAction(broker, editor, concept,
									typeHighlightAction, typeTempAttributes);
							manager.add(deselectConceptAction);
						}

						if (selectedElements.size() > 1) {

							if (lastSelectedConcept != null) {
								Action mergeConceptsAction = new MergeConceptsAction(broker, editor, selectedElements,
										lastSelectedConcept, terminology);
								manager.add(mergeConceptsAction);
							}
						}
						if (concept.isAttribute() && !concept.isStructure()) {
							Action highlightAction = new HighlightSegmentsAction(broker, editor, concept,
									typeHighlightAction, typeTempAttributes, terminology);
							manager.add(highlightAction);

							Action applyTermForNodeAction = new ApplyTerminologyForNode(broker, editor, concept,
									typeHighlightAction, typeTempAttributes, algorithm, typeConcept, typeMedEval,
									featTypeContent);
							manager.add(applyTermForNodeAction);

							Action addNegationAction = new AddNegationAction(broker, editor, concept, terminology);
							manager.add(addNegationAction);

							// add variation
							if (!editor.getWidget().getSelectionText().isEmpty()) {
								Action addVariationAction = new Action() {

									@Override
									public void run() {

										Variation v = new Variation(editor.getWidget().getSelectionText());
										v.setCaseSensitive(false);
										v.setRespectEndings(true);

										concept.addVariation(v);
										broker.send(IAnnotationEditorEvents.TERMINOLOGY_NODE_ATTRIBUTE_CHANGED, concept);
									}
								};
								addVariationAction.setText("Add Variation");
								manager.add(addVariationAction);
							}
						}
						if (!concept.isRoot()) {
							Action duplicateNodeAction = new DuplicateNodeAction(broker, concept, terminology);
							manager.add(duplicateNodeAction);

							Action editNodeAction = new Action() {
								@Override
								public void run() {

									buttonEditNode.notifyListeners(SWT.Selection, new Event());
								}
							};
							editNodeAction.setText("Edit");

							manager.add(editNodeAction);
						}
						if (!concept.isStructure()) {

							// convert to dictionary
							Action converToDictionaryAction = new ConvertConceptToDictionaryAction(broker, editor,
									concept, terminology);
							manager.add(converToDictionaryAction);

						}

						// get the actual selection of the editor
						List<AnnotationFS> conceptsSelGold = new ArrayList<AnnotationFS>();
						for (AnnotationFS a : editor.getCas().getAnnotationIndex(conceptGoldType)) {

							if (a.getBegin() <= editor.getWidget().getCaretOffset()
									&& a.getEnd() >= editor.getWidget().getCaretOffset()) {
								conceptsSelGold.add(a);
							}
						}

						if (conceptsSelGold.size() == 1) {

							Action actionAssertId = new Action() {

								@Override
								public void run() {

									assignNewGoldID(concept, conceptsSelGold);
								}

							};

							actionAssertId.setText("Assert ID to selected concept");
							manager.add(actionAssertId);
						}

					}
				}
			}
		});

		menuMgr.setRemoveAllWhenShown(true);

		ColumnViewerToolTipSupport.enableFor(viewer, ToolTip.NO_RECREATE);
		// dragn drop
		int operations = DND.DROP_COPY | DND.DROP_MOVE;
		Transfer[] transferTypes = new Transfer[] { LocalSelectionTransfer.getTransfer() };
		viewer.addDragSupport(operations, transferTypes, new TreeViewerDragListener(viewer));
		viewer.addDropSupport(operations, transferTypes, new TreeViewerDropListener(viewer, broker));

		viewer.addSelectionChangedListener(new ISelectionChangedListener() {

			@Override
			public void selectionChanged(SelectionChangedEvent event) {

				updateButtonActivation();

			}

		});

		filter = new ConceptNameFilter();
		viewer.addFilter(filter);

		viewer.getTree().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {

				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));
				if (cell == null)
					return;
				Object element = cell.getElement();

				if (element instanceof Concept) {
					lastSelectedConcept = (Concept) element;
				}
			}

			@Override
			public void mouseDown(MouseEvent e) {

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof Concept) {

					String editorText = editor.getWidget().getSelectionText();

					if (editorText == null)
						return;

					Concept concept = (Concept) sel.getFirstElement();

					if (editorText.isEmpty() && buttonEditNode.isEnabled()) {
						buttonEditNode.notifyListeners(SWT.Selection, new Event());
						return;
					}

					// add a new Concept as child

					if (concept.isValue()) {

						// create a synonym instead of a whole new node
						Variation v = new Variation(editor.getWidget().getSelectionText());
						v.setCaseSensitive(false);
						v.setRespectEndings(true);
						concept.addVariation(v);
					}

					else {

						if (concept.isAttribute() && !concept.isStructure() && dialog != null
								&& dialog.getShell() != null && !concept.isRoot()) {

							// add as a variation of that attribute
							Variation v = new Variation(editor.getWidget().getSelectionText());
							v.setCaseSensitive(false);
							v.setRespectEndings(true);
							concept.addVariation(v);

						} else {

							Concept child = createChild(concept, editorText);

							concept.addChild(child);

							reloadTree();

							updateEvaluationType(editor.getWidget().getSelection(), "FN");
						}
					}
				}

			}
		});
		if (terminology != null) {
			viewer.setInput(terminology);
		}
	}

	private void assignNewGoldID(Concept concept, List<AnnotationFS> conceptsSelGold) {
		AnnotationFS annotationFS = conceptsSelGold.get(0);

		annotationFS.setFeatureValueFromString(featConceptGoldid, String.valueOf(concept.getId()));
		editor.changeAnnotation(annotationFS);
	}

	private void createTopButtons() {
		buttonAddChild = new Button(this, SWT.NONE);
		buttonAddChild.setText("Add Child");

		buttonAddChild.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof Concept) {

					Concept concept = (Concept) sel.getFirstElement();

					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "",
							"Enter a concept name", "", null);
					if (dlg.open() == Window.OK) {
						Concept child = createChild(concept, dlg.getValue());

						concept.addChild(child);
						viewer.setExpandedState(concept, true);
					}

					reloadTree();
				}
			}

		});

		buttonAddSibling = new Button(this, SWT.NONE);
		buttonAddSibling.setText("Add Sibling");

		buttonAddSibling.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof Concept) {

					Concept concept = (Concept) sel.getFirstElement();

					InputDialog dlg = new InputDialog(Display.getCurrent().getActiveShell(), "",
							"Enter a concept name", "", null);
					if (dlg.open() == Window.OK) {

						Concept parentConcept = concept.getParent();
						Concept child = createChild(parentConcept, dlg.getValue());
						child.setPureExistence(true);

						parentConcept.addChild(child);
						viewer.setExpandedState(parentConcept, true);
					}

					reloadTree();
				}
			}

		});

		buttonDeleteNode = new Button(this, SWT.NONE);
		buttonDeleteNode.setText("Delete");

		buttonDeleteNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof Concept) {

					Concept concept = (Concept) sel.getFirstElement();

					boolean delete = MessageDialog.open(MessageDialog.QUESTION, Display.getCurrent().getActiveShell(),
							"Deleting Concept", "Are you sure to delete the concept: " + concept.getName() + "?",
							SWT.NONE);
					if (delete) {
						terminology.deleteNode(concept);

						reloadTree();
					}
				}
			}
		});

		buttonEditNode = new Button(this, SWT.NONE);
		buttonEditNode.setText("Edit");

		buttonEditNode.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();
				if (sel.getFirstElement() instanceof Concept) {

					Concept concept = (Concept) sel.getFirstElement();

					ConceptEditingDialog dialog = new ConceptEditingDialog(Display.getCurrent().getActiveShell(),
							concept, terminology, broker);
					dialog.open();

					// success

					reloadTree();
				}
			}

		});

		buttonShowLexika = new Button(this, SWT.PUSH);
		buttonShowLexika.setText("Edit Lexicon");

		buttonShowLexika.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				Lexikon lexikon = terminology.getLexikon();

				LexikonEditingDialog dialog = new LexikonEditingDialog(new Shell(), lexikon);

				dialog.open();

			}

		});

		if (this.terminology == null) {
			buttonShowLexika.setEnabled(false);
		}
	}

	private void updateButtonActivation() {
		IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

		if (sel.getFirstElement() instanceof Concept) {

			Concept concept = (Concept) sel.getFirstElement();
			if (concept.isRoot()) {
				compNodeAttributes.setConcept(null);
				buttonAddChild.setEnabled(true);
				buttonAddSibling.setEnabled(false);
				buttonDeleteNode.setEnabled(false);
				buttonEditNode.setEnabled(false);
			}

			else {
				compNodeAttributes.setConcept(concept);
				buttonAddChild.setEnabled(true);
				buttonAddSibling.setEnabled(true);
				buttonDeleteNode.setEnabled(true);
				buttonEditNode.setEnabled(true);
			}

		}

		else {

			buttonAddChild.setEnabled(false);
			buttonAddSibling.setEnabled(false);
			buttonDeleteNode.setEnabled(false);
			buttonEditNode.setEnabled(false);
		}

		// reload that stuff
		this.viewer.getTree().getParent().layout();

	}

	public Concept createChild(Concept parent, String name) {
		Concept child = new Concept(name, parent, null, terminology.getNextFreeId());

		// TODO pure existence always set to true?
		child.setPureExistence(true);
		child.setNodeType(ENodeType.CHOICE);
		viewer.setExpandedState(parent, true);
		return child;
	}

	public void reloadTree() {
		Runnable r = new Runnable() {

			@Override
			public void run() {

				if (viewer == null || viewer.getTree().isDisposed())
					return;
				Object[] expandedElements = viewer.getExpandedElements();
				viewer.setInput(terminology);

				viewer.setExpandedElements(expandedElements);

				viewer.refresh();
			}
		};

		// reload asynchronous
		getDisplay().asyncExec(r);
	}

	private void updateEvaluationType(Point point, String evaltype) {

		// delete all eval types that covered this annos
		List<AnnotationFS> covering = new ArrayList<AnnotationFS>();

		for (AnnotationFS eval : editor.getCas().getAnnotationIndex(typeMedEval)) {

			if (eval.getBegin() == point.x && eval.getEnd() == point.y) {
				covering.add(eval);
			}
		}

		// delete quietly
		for (AnnotationFS toDel : covering)
			editor.getCas().removeFsFromIndexes(toDel);

		// add another annotation and set the feature

		AnnotationFS medEval = editor.getCas().createAnnotation(typeMedEval, point.x, point.y);

		medEval.setFeatureValueFromString(featMedEvalType, evaltype);

		// force a reload in the editor
		editor.addAnnotation(medEval);

	}

	public void setAttributeAcceptanceDialog(AttributeAcceptanceDialog dialog) {
		this.dialog = dialog;
	}

	public void setTerminology(MedicalTerminology terminology) {
		this.terminology = terminology;
		buttonShowLexika.setEnabled(terminology != null);
	}

	public TreeViewer getViewer() {
		return viewer;
	}

	public void filterConceptById(int conceptId) {
		if (terminology == null) {
			return;
		}
		// TODO remove old filters?
		viewer.addFilter(new ConceptIdFilter(conceptId));
		;
	}

	public Concept getSelectedConcept() {

		IStructuredSelection sel = (IStructuredSelection) this.viewer.getSelection();

		return (Concept) sel.getFirstElement();

	}

	public TreeViewer getTree() {
		return this.viewer;
	}
}
