package de.uniwue.mk.medIE.terminologyView.widgets;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.jface.viewers.ViewerCell;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.medIE.terminology.IGotVariations;
import de.uniwue.mk.medIE.terminology.Variation;

public class VariationEditingComposite extends Composite {

	private static final String WHICH_BOOLEAN = "WHICH_BOOLEAN";
	private static final String SUFFIXES = "Suffixes?";
	private static final String USE_STEMMING = "Use Stemming";
	private static final String UNIT = "Unit";
	private static final String REGEX = "Regex";
	private static final String RESPECT_CONTEXT = "Respect Context";
	private static final String CASE_SENSITIVE = "Case Sensitive";
	private Button buttonCheckIsRegex;
	private Button buttonCheckUseStemming;
	private Button buttonRespectStemming;
	private Button buttonCheckIsUnit;
	private Button buttonCheckIsCaseSensitive;
	private Button buttonCheckRespectContext;
	private TableViewer viewer;
	private IGotVariations concept;

	public VariationEditingComposite(Composite parent, int style, Font font, IGotVariations concept) {
		super(parent, style);

		setLayout(new GridLayout(6, false));

		this.concept = concept;
		buttonCheckIsRegex = new Button(this, SWT.CHECK);
		buttonCheckIsRegex.setSelection(false);
		buttonCheckIsRegex.setText("Is RegEx");
		buttonCheckIsRegex.setFont(font);

		buttonCheckUseStemming = new Button(this, SWT.CHECK);
		buttonCheckUseStemming.setText(USE_STEMMING);
		buttonCheckUseStemming.setFont(font);

		buttonRespectStemming = new Button(this, SWT.CHECK);
		buttonRespectStemming.setText("Respect Suffix");
		buttonRespectStemming.setFont(font);

		buttonCheckIsUnit = new Button(this, SWT.CHECK);
		buttonCheckIsUnit.setText("Is Unit");
		buttonCheckIsUnit.setFont(font);

		buttonCheckIsCaseSensitive = new Button(this, SWT.CHECK);
		buttonCheckIsCaseSensitive.setText(CASE_SENSITIVE);
		buttonCheckIsCaseSensitive.setFont(font);

		buttonCheckRespectContext = new Button(this, SWT.CHECK);
		buttonCheckRespectContext.setText(RESPECT_CONTEXT);
		buttonCheckRespectContext.setFont(font);

		// contains the tableviewer

		viewer = new TableViewer(this, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		GridData layoutDataTable = new GridData(GridData.FILL_HORIZONTAL | GridData.GRAB_HORIZONTAL);
		layoutDataTable.horizontalSpan = 6;
		layoutDataTable.heightHint = 200;
		viewer.getControl().setLayoutData(layoutDataTable);

		createColumns(viewer);

		// make lines and header visible
		final Table table2 = viewer.getTable();
		table2.setHeaderVisible(true);
		table2.setLinesVisible(true);

		viewer.getTable().addKeyListener(new KeyListener() {

			@Override
			public void keyReleased(KeyEvent e) {
				if (e.keyCode == SWT.DEL) {

					IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

					Variation v = (Variation) sel.getFirstElement();

					if (v != null) {

						concept.removeVariation(v);
					}

					refresh();
				}

			}

			@Override
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub

			}
		});

		viewer.getTable().addMouseListener(new MouseAdapter() {

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				ViewerCell cell = viewer.getCell(new Point(e.x, e.y));

				if (cell == null)
					return;

				TableColumn column = viewer.getTable().getColumn(cell.getColumnIndex());

				Object data2 = column.getData(WHICH_BOOLEAN);

				if (data2 == null)
					return;

				String feature = (String) data2;

				Variation selectedVar = (Variation) ((IStructuredSelection) viewer.getSelection()).getFirstElement();

				if (selectedVar == null)
					return;

				if (feature.equals(CASE_SENSITIVE)) {

					selectedVar.setCaseSensitive(!selectedVar.isCaseSensitive());
				}

				else if (feature.equals(REGEX)) {
					selectedVar.setRegex(!selectedVar.isRegex());
				}

				else if (feature.equals(SUFFIXES)) {
					selectedVar.setRespectEndings(!selectedVar.isRespectEndings());
				}

				else if (feature.equals(UNIT)) {
					selectedVar.setUnit(!selectedVar.isUnit());
				}

				else if (feature.equals(USE_STEMMING)) {
					selectedVar.setUseStemming(!selectedVar.isUseStemming());
				}

				else if (feature.equals(RESPECT_CONTEXT)) {
					selectedVar.setRespectContext(!selectedVar.isRespectContext());
				}

				viewer.refresh();

			}
		});

		// content
		viewer.setContentProvider(new ArrayContentProvider());

		viewer.setInput(concept.getAllVariations().toArray(new Variation[0]));
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Variation", 100, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return p.getSynonym();

			}

		});

		colName.setEditingSupport(new VariationNameEditingSupport(viewer));

		TableViewerColumn colCaseSensitive = createTableViewerColumn(CASE_SENSITIVE, 80, 1);
		colCaseSensitive.getColumn().setData(WHICH_BOOLEAN, CASE_SENSITIVE);
		colCaseSensitive.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isCaseSensitive());

			}

		});

		TableViewerColumn colRegex = createTableViewerColumn(REGEX, 50, 2);
		colRegex.getColumn().setData(WHICH_BOOLEAN, REGEX);
		colRegex.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isRegex());

			}

		});

		TableViewerColumn colIsUnit = createTableViewerColumn(UNIT, 50, 3);
		colIsUnit.getColumn().setData(WHICH_BOOLEAN, UNIT);
		colIsUnit.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isUnit());

			}

		});

		TableViewerColumn colStemming = createTableViewerColumn(USE_STEMMING, 80, 4);
		colStemming.getColumn().setData(WHICH_BOOLEAN, USE_STEMMING);
		colStemming.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isUseStemming());

			}

		});

		TableViewerColumn colEndings = createTableViewerColumn(SUFFIXES, 50, 5);
		colEndings.getColumn().setData(WHICH_BOOLEAN, SUFFIXES);
		colEndings.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isRespectEndings());

			}

		});

		TableViewerColumn colContext = createTableViewerColumn(RESPECT_CONTEXT, 50, 5);
		colContext.getColumn().setData(WHICH_BOOLEAN, RESPECT_CONTEXT);
		colContext.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				Variation p = (Variation) element;
				return String.valueOf(p.isRespectContext());

			}

		});

	}

	private void refresh() {

		viewer.setInput(concept.getAllVariations().toArray(new Variation[0]));
		viewer.refresh();

		TableColumn[] columns = viewer.getTable().getColumns();

		for (TableColumn cl : columns) {
			cl.pack();
		}

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// table.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	public Button getButtonCheckIsRegex() {
		return buttonCheckIsRegex;
	}

	public Button getButtonCheckUseStemming() {
		return buttonCheckUseStemming;
	}

	public Button getButtonRespectStemming() {
		return buttonRespectStemming;
	}

	public Button getButtonCheckIsUnit() {
		return buttonCheckIsUnit;
	}

	public TableViewer getViewer() {
		return viewer;
	}

	public Button getButtonCheckIsCaseSensitive() {
		return buttonCheckIsCaseSensitive;
	}

	public IGotVariations getConcept() {
		return concept;
	}

	public Button getButtonCheckRespectContext() {
		return buttonCheckRespectContext;
	}

	public void setButtonCheckRespectContext(Button buttonCheckRespectContext) {
		this.buttonCheckRespectContext = buttonCheckRespectContext;
	}

}
