package de.uniwue.mkrug.kallimachos.annotation.editor.ui;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.inject.Inject;

import org.apache.uima.cas.CASException;
import org.apache.uima.cas.Feature;
import org.apache.uima.cas.Type;
import org.apache.uima.cas.text.AnnotationFS;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.di.Persist;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.bindings.keys.KeyStroke;
import org.eclipse.jface.fieldassist.ContentProposalAdapter;
import org.eclipse.jface.fieldassist.SimpleContentProposalProvider;
import org.eclipse.jface.fieldassist.TextContentAdapter;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.part.editor.subordinate.AEditorSubordinateViewPart;

public class GeneralAnnotationView extends AEditorSubordinateViewPart {

	private static final Class<GeneralAnnotationView> CLASS_THIS = GeneralAnnotationView.class;

	@Inject
	static EPartService partService;

	@Inject
	MApplication app;

	static String viewId = "de.uniwue.mkrug.kallimachos.annotation.editor.part.16";

	// DATA CONTAINER
	private List<AnnotationFS> annoList;
	private Type typSelected;

	// E4 DI OBJECTS
	@Inject
	static IEventBroker broker;

	@Inject
	EModelService modelService;

	// WIDGETS
	private TableViewer viewer;

	private Label labelAnnotyp;
	private Text textSelectType;
	private Button buttonBrowse;
	private Button buttonSelect;

	private Composite compAnnoSelected;
	private Composite compNoAnnoSelected;

	private Button buttonChangeVisibilities;

	private Text textActualSelectedType;

	@Inject
	public GeneralAnnotationView() {
		super();
		super.viewId = viewId;
		super.class_this = CLASS_THIS;
	}

	@PostConstruct
	public void postConstruct(Composite parent) {

		System.out.println("Construct GeneralannotationView");

		super.overlay = parent;
		super.part = partService.findPart(viewId);
		super.partService = partService;

		// this is important since in the struct the broker is not injected
		// yet!!
		super.setBroker(broker);

		super.overlay.setLayout(new GridLayout(1, true));

	}

	@Override
	protected void initLayout(Composite parent) {

		// composite for annotyp selection
		initFirstComposite(parent);

		// second composite this is dynamic and gets changed by the caret
		// listener
		compAnnoSelected = new Composite(parent, SWT.BORDER);
		compAnnoSelected.setVisible(false);

		parent.pack();
		parent.layout();

	}

	private void initFirstComposite(Composite parent) {
		Composite compSelectAnnotype = new Composite(parent, SWT.BORDER);

		compSelectAnnotype.setLayout(new GridLayout(4, true));

		labelAnnotyp = new Label(compSelectAnnotype, SWT.NONE);
		labelAnnotyp.setText("Choose Type:");

		textSelectType = new Text(compSelectAnnotype, SWT.BORDER);

		textSelectType.addKeyListener(new KeyAdapter() {

			private String KEY_PRESS = "Ctrl+Space";

			public void keyReleased(KeyEvent ke) {
				// Method for autocompletion
				setAutoCompletion(textSelectType, textSelectType.getText());
			}

			private void setAutoCompletion(Text text, String value) {
				try {
					ContentProposalAdapter adapter = null;
					String[] defaultProposals = getAllProposals(value);
					SimpleContentProposalProvider scp = new SimpleContentProposalProvider(defaultProposals);
					scp.setProposals(defaultProposals);
					KeyStroke ks = KeyStroke.getInstance(KEY_PRESS);
					adapter = new ContentProposalAdapter(text, new TextContentAdapter(), scp, ks, null);
					adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}

			private String[] getAllProposals(String text) {
				String[] proposals = new String[5];
				if (text == null || text.length() == 0) {
					Iterator<Type> typeIterator = editor.getCas().getTypeSystem().getTypeIterator();

					List<String> candidates = new ArrayList<String>();

					while (typeIterator.hasNext()) {

						String shortName = typeIterator.next().getShortName();

						candidates.add(shortName);
					}

					proposals = candidates.toArray(new String[0]);

				}

				else {

					Iterator<Type> typeIterator = editor.getCas().getTypeSystem().getTypeIterator();

					List<String> candidates = new ArrayList<String>();

					while (typeIterator.hasNext()) {

						String shortName = typeIterator.next().getShortName();

						if (shortName.startsWith(text)) {
							candidates.add(shortName);
						}
					}

					proposals = candidates.toArray(new String[0]);
				}
				return proposals;
			}

		});

		buttonSelect = new Button(compSelectAnnotype, SWT.PUSH);
		buttonSelect.setText(" Select ");

		buttonSelect.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (typSelected != null) {
					editor.changeVisibility(typSelected, false);
				}
				String text = textSelectType.getText();

				if (text == null || text.length() == 0)
					return;

				Iterator<Type> typeIterator = editor.getCas().getTypeSystem().getTypeIterator();

				while (typeIterator.hasNext()) {

					Type t = typeIterator.next();

					if (t.getShortName().equals(text)) {
						typSelected = t;
						textActualSelectedType.setText(typSelected.getName());
						// turn on the new type
						editor.changeVisibility(typSelected, true);

						return;
					}
				}

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});
		buttonBrowse = new Button(compSelectAnnotype, SWT.PUSH);
		buttonBrowse.setText(" Browse ");

		textActualSelectedType = new Text(compSelectAnnotype, SWT.BORDER);
		GridData gridData = new GridData(GridData.GRAB_HORIZONTAL | GridData.FILL_HORIZONTAL);
		gridData.horizontalSpan = 4;
		textActualSelectedType.setLayoutData(gridData);
		textActualSelectedType.setEditable(false);
		// / ==== Buttons + Fields
	}

	public void renewPage() throws CASException {

		// reload asynchronous
		// overlay.getDisplay().asyncExec(r);

	}

	@PreDestroy
	public void preDestroy() {
		// nothing to destroy
	}

	@Focus
	public void onFocus() {

		super.onFocus();

	}

	@Persist
	public void save() {
		// nothing to save
	}

	// ====== Abstract Methods from Superclass

	@Override
	public void onEditorCaretChanged(org.osgi.service.event.Event arg0) {

		if (typSelected == null)
			return;

		AnnotationFS selectedAnno = null;
		for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typSelected)) {
			if (anno.getBegin() <= editor.getWidget().getCaretOffset()
					&& anno.getEnd() >= editor.getWidget().getCaretOffset()) {
				selectedAnno = anno;
				break;

			}

		}

		initSecondComposite(selectedAnno);

		super.overlay.pack();
		super.overlay.layout();

	}

	private void initSecondComposite(final AnnotationFS selectedAnno) {

		if (typSelected == null)
			return;

		compAnnoSelected.setVisible(true);
		compAnnoSelected.setLayout(new GridLayout(3, true));

		// remove everything from that composite

		for (Control c : compAnnoSelected.getChildren()) {
			c.dispose();
		}

		// we add an add and a remove button in any case

		Button buttonAddAnnotation = new Button(compAnnoSelected, SWT.PUSH);

		buttonAddAnnotation.setText("Add Annotation");
		GridData gd = new GridData();
		gd.horizontalSpan = 2;
		// buttonAddAnnotation.setLayoutData(gd);

		buttonAddAnnotation.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				// take the span if not a length of 0 and create an annotation
				Point selection = editor.getWidget().getSelection();

				if (selection.y - selection.x == 0) {
					return;
				}

				AnnotationFS fs = editor.getCas().createAnnotation(typSelected, selection.x, selection.y);

				addAnnotation(fs);

			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		Button buttonRemoveAnnotation = new Button(compAnnoSelected, SWT.PUSH);

		buttonRemoveAnnotation.setText("Remove Annotation");
		Button imADummy = new Button(compAnnoSelected, SWT.NONE);
		imADummy.setVisible(false);

		buttonRemoveAnnotation.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {

				if (selectedAnno != null) {
					removeAnnotation(selectedAnno);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
				widgetSelected(e);

			}
		});

		if (selectedAnno != null) {
			// add the primitve features for that anno

			List<Feature> features = selectedAnno.getType().getFeatures();

			List<Feature> primitves = new ArrayList<Feature>();

			for (Feature f : features) {
				if (f.getRange().isPrimitive()) {

					if (f.getShortName().matches("begin|end")) {
						continue;
					}

					Label l = new Label(compAnnoSelected, SWT.NONE);
					l.setText(f.getShortName() + " as " + f.getRange().getShortName());

					final Text textFeature = new Text(compAnnoSelected, SWT.BORDER);
					textFeature.addKeyListener(new org.eclipse.swt.events.KeyListener() {

						@Override
						public void keyReleased(KeyEvent e) {

							// enter pressed
							if (e.keyCode == 13) {

								String text = textFeature.getText();

								Feature f = (Feature) textFeature.getData("FEATURE");
								AnnotationFS anno = (AnnotationFS) textFeature.getData("ANNOTATION");

								setFeatureFromString(text, f, anno);
							}

						}

						private void setFeatureFromString(String text, Feature f, AnnotationFS anno) {

							// TODO check if the input is even valid !!
							if (f.getRange().getShortName().equals("String")) {

								anno.setFeatureValueFromString(f, text);
							}

							else if (f.getRange().getShortName().equals("Boolean")) {
								anno.setBooleanValue(f, Boolean.parseBoolean(text));

							}

							// TODO support a little more here but no time und
							// so...

						}

						@Override
						public void keyPressed(KeyEvent e) {

						}
					});

					textFeature.setText(getFeatureValueAsString(selectedAnno, f));
					textFeature.setData("FEATURE", f);
					textFeature.setData("ANNOTATION", selectedAnno);

					if (f.getRange().isPrimitive()) {
						textFeature.addKeyListener(new KeyAdapter() {

							private String KEY_PRESS = "Ctrl+Space";

							public void keyReleased(KeyEvent ke) {
								// Method for autocompletion
								setAutoCompletion(textFeature, textFeature.getText());

							}

							private void setAutoCompletion(Text text, String value) {
								try {
									ContentProposalAdapter adapter = null;
									String[] defaultProposals = getAllProposals(value);
									SimpleContentProposalProvider scp = new SimpleContentProposalProvider(
											defaultProposals);
									scp.setProposals(defaultProposals);
									KeyStroke ks = KeyStroke.getInstance(KEY_PRESS);
									adapter = new ContentProposalAdapter(text, new TextContentAdapter(), scp, ks, null);
									adapter.setProposalAcceptanceStyle(ContentProposalAdapter.PROPOSAL_REPLACE);
								} catch (Exception e) {
									e.printStackTrace();
								}
							}

							private String[] getAllProposals(String text) {
								String[] proposals = new String[5];
								if (text == null || text.length() == 0) {

									// all featurevalues that have been set in
									// this
									// document are candidates
									Set<String> candidates = new HashSet<String>();

									for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typSelected)) {
										String featureValueAsString = getFeatureValueAsString(anno,
												(Feature) textFeature.getData("FEATURE"));

										if (featureValueAsString != null) {

											candidates.add(featureValueAsString);
										}
									}

									proposals = candidates.toArray(new String[0]);
								}

								else {

									// all featurevalues that have been set in
									// this
									// document are candidates
									Set<String> candidates = new HashSet<String>();

									for (AnnotationFS anno : editor.getCas().getAnnotationIndex(typSelected)) {
										String featureValueAsString = getFeatureValueAsString(anno,
												(Feature) textFeature.getData("FEATURE"));

										if (featureValueAsString != null && featureValueAsString.startsWith(text)) {

											candidates.add(featureValueAsString);
										}
									}
									proposals = candidates.toArray(new String[0]);

								}
								return proposals;
							}

						});
					}
					final Button setFeature = new Button(compAnnoSelected, SWT.PUSH);
					setFeature.setData("FEATURE", f);
					setFeature.setData("ANNOTATION", selectedAnno);

					setFeature.setText(" Set ");

					setFeature.addSelectionListener(new SelectionListener() {

						@Override
						public void widgetSelected(SelectionEvent e) {
							String text = textFeature.getText();

							Feature f = (Feature) setFeature.getData("FEATURE");
							AnnotationFS anno = (AnnotationFS) setFeature.getData("ANNOTATION");

							setFeatureFromString(text, f, anno);
						}

						private void setFeatureFromString(String text, Feature f, AnnotationFS anno) {

							// TODO check if the input is even valid !!
							if (f.getRange().getShortName().equals("String")) {

								anno.setFeatureValueFromString(f, text);
							}

							else if (f.getRange().getShortName().equals("Boolean")) {
								anno.setBooleanValue(f, Boolean.parseBoolean(text));

							}

							// TODO support a little more here but no time und
							// so...

							editor.changeAnnotation(anno);
						}

						@Override
						public void widgetDefaultSelected(SelectionEvent e) {

						}
					});

				}
			}

		}

		compAnnoSelected.layout();

	}

	private String getFeatureValueAsString(AnnotationFS selectedAnno, Feature f) {

		if (f.getRange().getShortName().equals("String")) {
			String featureValueAsString = selectedAnno.getFeatureValueAsString(f);

			if (featureValueAsString == null)
				return "null";
			return selectedAnno.getFeatureValueAsString(f);
		}

		// TODO mehr dampf ....mehr featuretypen untersttzen

		return "unsupported cause lazy";
	}

	private void addAnnotation(AnnotationFS fs) {

		editor.addAnnotation(fs);

	}

	private void removeAnnotation(AnnotationFS selectedAnno) {
		editor.removeAnnotation(selectedAnno);

	}

	@Override
	public void initModel() {
		// no model to hold

	}

	@Override
	public boolean initTypes() {
		// no types needed
		return true;
	}

}