package de.uniwue.mkrug.kallimachos.annotations.editor.dialogues;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import de.uniwue.mk.kall.athen.projectExplorer.conversion.ADocument;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.DocumentFormatUtil;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.EFormat;
import de.uniwue.mk.kall.athen.projectExplorer.conversion.UnsupportedDocument;

public class DocumentConversionDialog extends Dialog {
	private Text txtFolderIn;
	private Text txtFolderOut;
	private Combo cbFormatIn;
	private Combo cbFormatOut;


	public DocumentConversionDialog(Shell parentShell) {
		super(parentShell);
	}


	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);

		GridLayout layout = new GridLayout(3, false);
		layout.horizontalSpacing = 15;
		layout.verticalSpacing = 10;
		layout.marginTop = 10;
		layout.marginBottom = 10;
		layout.marginLeft = 10;
		layout.marginRight = 10;
		composite.setLayout(layout);

		/*
		 * controls for folders
		 */
		Label lbFolderIn = new Label(composite, SWT.NONE);
		lbFolderIn.setText("Input Folder:");

		txtFolderIn = new Text(composite, SWT.BORDER);
		GridData datatfi = new GridData();
		datatfi.widthHint = 200;
		datatfi.grabExcessHorizontalSpace = true;
		txtFolderIn.setLayoutData(datatfi);

		Button btFolderIn = new Button(composite, SWT.PUSH);
		btFolderIn.setText("Browse...");

		Label lbFolderOut = new Label(composite, SWT.NONE);
		lbFolderOut.setText("Output Folder:");

		txtFolderOut = new Text(composite, SWT.BORDER);
		GridData datatfo = new GridData();
		datatfo.widthHint = 200;
		datatfo.grabExcessHorizontalSpace = true;
		txtFolderOut.setLayoutData(datatfo);

		Button btFolderOut = new Button(composite, SWT.PUSH);
		btFolderOut.setText("Browse...");

		/*
		 * controls for formats
		 */
		Label lbFormatIn = new Label(composite, SWT.NONE);
		lbFormatIn.setText("Input Format:");

		cbFormatIn = new Combo(composite, SWT.READ_ONLY);
		Set<String> collect = Arrays.asList(EFormat.values()).stream()
				.filter(format -> !format.equals(EFormat.UNSUPPORTED_FORMAT))
				.map(format -> format.toString()).collect(Collectors.toSet());
		cbFormatIn.setItems(collect.toArray(new String[0]));
		GridData datacfi = new GridData();
		datacfi.widthHint = 180;
		datacfi.grabExcessHorizontalSpace = true;
		cbFormatIn.setLayoutData(datacfi);

		// dummy element
		new Label(composite, SWT.NONE);

		Label lbFormatOut = new Label(composite, SWT.NONE);
		lbFormatOut.setText("Output Format:");

		cbFormatOut = new Combo(composite, SWT.READ_ONLY);
		GridData datacfo = new GridData();
		datacfo.widthHint = 180;
		datacfo.grabExcessHorizontalSpace = true;
		cbFormatOut.setLayoutData(datacfo);

		// dummy element
		new Label(composite, SWT.NONE);

		/*
		 * Selection listeners
		 */
		btFolderIn.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dialog = new DirectoryDialog(parent.getShell(), SWT.OPEN);
				dialog.setFilterPath(txtFolderIn.getText());
				String reVal = dialog.open();
				if (reVal != null) {
					txtFolderIn.setText(reVal);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		btFolderOut.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				DirectoryDialog dialog = new DirectoryDialog(parent.getShell(), SWT.OPEN);
				dialog.setFilterPath(txtFolderOut.getText());
				String reVal = dialog.open();
				if (reVal != null) {
					txtFolderOut.setText(reVal);
				}
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		cbFormatIn.addSelectionListener(new SelectionListener() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				int selection = cbFormatIn.getSelectionIndex();
				if (selection < 0) {
					cbFormatOut.setItems(new String[0]);
					return;
				}
				EFormat source = EFormat.valueOf(cbFormatIn.getItem(selection));
				Collection<EFormat> targets = DocumentFormatUtil.getPossibleTargetFormats(source);
				Set<String> set = targets.stream()
						.filter(format -> !format.equals(EFormat.UNSUPPORTED_FORMAT))
						.map(format -> format.toString()).collect(Collectors.toSet());
				cbFormatOut.setItems(set.toArray(new String[0]));
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent e) {
			}

		});

		return parent;
	}

	@Override
	protected void okPressed() {
		if (!inputIsValid()) {
			return;
		}
		// get folders and formats from the controls
		File sourceFolder = new File(txtFolderIn.getText());
		File targetFolder = new File(txtFolderOut.getText());
		EFormat sourceFormat = EFormat.valueOf(
				cbFormatIn.getItem(cbFormatIn.getSelectionIndex()));
		EFormat targetFormat = EFormat.valueOf(
				cbFormatOut.getItem(cbFormatOut.getSelectionIndex()));

		convertFiles(sourceFolder, targetFolder, sourceFormat, targetFormat);
		super.okPressed();
	}


	private void convertFiles(File sourceFolder, File targetFolder, EFormat sourceFormat, 
			EFormat targetFormat) {
		int successfullConversions = 0;
		for (File sourceFile : sourceFolder.listFiles()) {
			// skip directories and files that have the wrong format
			if (sourceFile.isDirectory() || 
					!sourceFormat.equals(DocumentFormatUtil.determineFormat(sourceFile))) {
				continue;
			}
			ADocument result = DocumentFormatUtil.convertDocument(sourceFile, 
					sourceFormat, targetFormat);
			if (result instanceof UnsupportedDocument) {
				MessageDialog.openWarning(getShell(), "Conversion problem", 
						sourceFile.getName() + " could not be converted.");
				continue;
			}
			File targetFile = new File(targetFolder, sourceFile.getName());
			result.serialize(targetFile);
			successfullConversions++;
		}
		MessageDialog.openInformation(getShell(), "Conversion complete", 
				successfullConversions + " files have been successfully converted.");
	}

	private boolean inputIsValid() {
		File fileIn = new File(txtFolderIn.getText());
		if (!fileIn.exists() || !fileIn.isDirectory()) {
			return false;
		}
		File fileOut = new File(txtFolderOut.getText());
		if (!fileOut.exists() || !fileOut.isDirectory()) {
			return false;
		}
		if (cbFormatIn.getSelectionIndex() < 0 || cbFormatOut.getSelectionIndex() < 0) {
			return false;
		}
		return true;
	}
}
