package de.uniwue.mkrug.kallimachos.annotations.editor.dialogues;

import java.util.HashMap;
import java.util.List;

import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;

public class ViewVisibilityDialog extends Dialog {

	private Composite parent;

	private EPartService partService;
	private EModelService modelService;

	private MApplication app;

	private TableViewer viewer;

	private HashMap<String, Image> imageMapping;

	private List<MPart> visibleParts;

	public ViewVisibilityDialog(Shell parentShell, EPartService service, EModelService modelService,
			List<MPart> visibleParts) {
		super(parentShell);

		this.partService = service;

		this.visibleParts = visibleParts;

		this.modelService = modelService;
	}

	protected Control createDialogArea(Composite parent) {

		this.parent = parent;

		imageMapping = new HashMap<String, Image>();

		parent.getShell().setText("Select Visibility of According Parts");

		GridLayout layout = new GridLayout(1, true);
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		parent.setLayout(layout);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);
		viewer.setContentProvider(new ArrayContentProvider());

		viewer.setInput(visibleParts.toArray(new MPart[0]));

		viewer.getTable().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {
				MPart part = (MPart) ((IStructuredSelection) viewer.getSelection()).getFirstElement();

				// project explorer cannot
				if (part.getLabel().matches("Project Explorer"))
					return;

				if (part.isVisible()) {
					part.setVisible(false);
				}

				else {

					part.setVisible(true);
					part.setOnTop(true);
				}

				viewer.refresh();
			}

		});

		return parent;
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colCheck = createTableViewerColumn("Part", 30, 0);
		colCheck.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {

				return null;
			}

			@Override
			public Image getImage(Object element) {

				MPart part = (MPart) element;

				if (part.isVisible()) {
					return ApplicationUtil.createImage(ApplicationUtil.CORRECT);
				}

				else {

					return ApplicationUtil.createImage(ApplicationUtil.ERROR);
				}

			}

		});

		TableViewerColumn colName = createTableViewerColumn("Icon", 30, 1);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return null;

			}

			@Override
			public Image getImage(Object element) {
				MPart part = (MPart) element;

				String iconURI = part.getIconURI();

				if (iconURI == null) {
					return ApplicationUtil.createImage(ApplicationUtil.CORRECT);
				}

				String[] split = iconURI.split("/");
				String path = split[split.length - 2] + "/" + split[split.length - 1];

				Image persIcon = null;
				if (imageMapping.containsKey(path)) {

					persIcon = imageMapping.get(path);
				}

				else {
					persIcon = ApplicationUtil.createImage(path);

					imageMapping.put(path, persIcon);

				}

				return persIcon;

			}

		});

		TableViewerColumn colFreq = createTableViewerColumn("Part", 100, 2);
		colFreq.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				MPart perspective = (MPart) element;

				return perspective.getLabel();
			}
		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	@Override
	protected void okPressed() {

		// STORE to PREFERENCES

		for (MPart part : this.visibleParts) {

			ApplicationUtil.saveToPreferences(part.getElementId(), String.valueOf(part.isVisible()));
		}

		setReturnCode(OK);
		close();
	}

}
