package de.uniwue.mkrug.mainMenu.handler;

import java.util.HashMap;
import java.util.List;

import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.advanced.MPerspective;
import org.eclipse.e4.ui.model.application.ui.advanced.MPerspectiveStack;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TableColumn;

import de.uniwue.mk.kall.athen.appDelegation.util.ApplicationUtil;

public class SwitchPerspectiveDialog extends Dialog {

	private Composite parent;

	private EPartService partService;
	private EModelService modelService;

	private MPerspective selectedPerspective;

	private MApplication app;

	private TableViewer viewer;

	private MPerspective activePerspective = null;

	private HashMap<String, Image> imageMapping;

	protected SwitchPerspectiveDialog(Shell parentShell, EPartService service, EModelService modelService,
			MApplication app) {
		super(parentShell);

		this.partService = service;
		this.app = app;

		this.modelService = modelService;
	}

	public MPerspective getActivePerspective() {
		return activePerspective;
	}

	public void setActivePerspective(MPerspective activePerspective) {
		this.activePerspective = activePerspective;
	}

	protected Control createDialogArea(Composite parent) {

		this.parent = parent;

		imageMapping = new HashMap<String, Image>();

		parent.getShell().setText("Choose desired Perspective");

		GridLayout layout = new GridLayout(2, true);
		layout.marginBottom = 10;
		layout.marginTop = 10;
		layout.marginLeft = 15;
		layout.marginRight = 15;
		parent.setLayout(layout);

		List<MPerspectiveStack> findElements = modelService.findElements(app, null, MPerspectiveStack.class, null);

		viewer = new TableViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.BORDER
				| SWT.FILL);

		createColumns(viewer);
		viewer.setContentProvider(new ArrayContentProvider());

		List<MPerspective> children = findElements.get(0).getChildren();
		viewer.setInput(children.toArray(new MPerspective[0]));

		viewer.getControl().addMouseListener(new MouseListener() {

			@Override
			public void mouseUp(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDown(MouseEvent e) {
				// TODO Auto-generated method stub

			}

			@Override
			public void mouseDoubleClick(MouseEvent e) {

				IStructuredSelection sel = (IStructuredSelection) viewer.getSelection();

				MPerspective firstElement = (MPerspective) sel.getFirstElement();

				activePerspective = firstElement;

			}
		});

		return parent;
	}

	private void createColumns(TableViewer viewer) {

		TableViewerColumn colName = createTableViewerColumn("Icon", 30, 0);
		colName.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				return null;

			}

			@Override
			public Image getImage(Object element) {
				MPerspective perspective = (MPerspective) element;

				String iconURI = perspective.getIconURI();

				String[] split = iconURI.split("/");
				String path = split[split.length - 2] + "/" + split[split.length - 1];

				Image persIcon = null;
				if (imageMapping.containsKey(path)) {

					persIcon = imageMapping.get(path);
				}

				else {
					persIcon = ApplicationUtil.createImage(path);

					imageMapping.put(path, persIcon);

				}

				return persIcon;

			}

		});

		TableViewerColumn colFreq = createTableViewerColumn("Perspective", 100, 1);
		colFreq.setLabelProvider(new ColumnLabelProvider() {
			@Override
			public String getText(Object element) {
				MPerspective perspective = (MPerspective) element;

				return perspective.getLabel();
			}
		});

	}

	private TableViewerColumn createTableViewerColumn(String viewer2, int bound, final int colNumber) {
		final TableViewerColumn viewerColumn = new TableViewerColumn(viewer, SWT.NONE);
		final TableColumn column = viewerColumn.getColumn();
		column.setText(viewer2);
		column.setWidth(bound);
		column.setResizable(true);
		column.setMoveable(true);
		column.addSelectionListener(getSelectionAdapter(column, colNumber));
		return viewerColumn;
	}

	private SelectionAdapter getSelectionAdapter(final TableColumn column, final int index) {
		SelectionAdapter selectionAdapter = new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// colViewerComp.setColumn(index);
				// int dir = colViewerComp.getDirection();
				// viewer.getTable().setSortDirection(dir);
				viewer.getTable().setSortColumn(column);
				viewer.refresh();
			}
		};
		return selectionAdapter;
	}

	@Override
	protected void okPressed() {

		if (activePerspective == null) {

			activePerspective = (MPerspective) ((IStructuredSelection) viewer.getSelection()).getFirstElement();
		}

		// eventually send an event ????
		setReturnCode(OK);
		close();
	}

}
