package de.uniwue.mkrug.mainMenu.handler;

import java.util.Set;
import java.util.stream.Collectors;

import javax.inject.Inject;

import org.eclipse.e4.core.di.annotations.CanExecute;
import org.eclipse.e4.core.di.annotations.Execute;
import org.eclipse.e4.core.services.events.IEventBroker;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.model.application.ui.basic.MPart;
import org.eclipse.e4.ui.workbench.modeling.EModelService;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.swt.widgets.Shell;

import de.uniwue.mk.kall.athen.appDelegation.hierarchie.IPerspectiveDecoupledPart;

public class SwitchPerspectiveHandler {

	@Inject
	EPartService partService;

	@Inject
	EModelService modelService;

	@Inject
	MApplication app;

	@Inject
	IEventBroker broker;

	@Execute
	public void execute(Shell shell) {

		// create a new Shell that shows all available Perspectives

		SwitchPerspectiveDialog dialog = new SwitchPerspectiveDialog(shell, partService, modelService, app);

		int open = dialog.open();

		if (open == 0 && dialog.getActivePerspective() != null) {

			Set<MPart> partsVisibleBefore = partService.getParts().stream()
					.filter(part -> partService.isPartVisible(part)).collect(Collectors.toSet());

			// switch the perspective
			partService.switchPerspective(dialog.getActivePerspective());

			Set<MPart> partsVisibleAfter = partService.getParts().stream()
					.filter(part -> partService.isPartVisible(part)).collect(Collectors.toSet());

			// decouple parts that were visible before but are not any more
			partsVisibleBefore
					.stream()
					.filter(part -> !partsVisibleAfter.contains(part))
					.filter(part -> part != null && part.getObject() != null
							&& part.getObject() instanceof IPerspectiveDecoupledPart)
					.map(part -> (IPerspectiveDecoupledPart) part.getObject()).forEach(part -> {
						part.decoupleFromPerspective();
						System.out.println("decoupled: " + part.getClass());
					});

			// couple parts that were not visible before but are now
			partsVisibleAfter
					.stream()
					.filter(part -> !partsVisibleBefore.contains(part))
					.filter(part -> part != null && part.getObject() != null
							&& part.getObject() instanceof IPerspectiveDecoupledPart)
					.map(part -> (IPerspectiveDecoupledPart) part.getObject()).forEach(part -> {
						part.coupleToPerspective();
						System.out.println("coupled: " + part.getClass());
					});

			// debug output for the number of parts
			System.out.println(modelService.findElements(app, null, MPart.class, null).size());

			System.out.println("subscribed classes: ");
			// MasterPart.dosth().forEach(entry -> {
			// System.out.println("\t" + entry.getSubScribingClass() + " => " +
			// entry.getEvent());
			// });
		}

	}

	@CanExecute
	public boolean canExecute() {
		return true;
	}

}